function getLocalStorageData(item) {
    if (typeof (Storage) !== "undefined") {
        return localStorage.getItem(item);
    } else {
        // Sorry! No Web Storage support..
    }
}

function setLocalStorageData(item, value) {
    if (typeof (Storage) !== "undefined") {
        localStorage.setItem(item, value);
    } else {
        // Sorry! No Web Storage support..
    }
}

function WindStrengthPayload(id, status) {
   status = !status;
    var payload="";
    var suffixURL="";
    switch (id) {
        case "3d":
            payload = status ? '{"Wind": { "direction": "Center" }}' : '{"Wind": { "direction": "Off" }}';
            suffixURL = '/wind';
            break;
        case "wide":
            payload = status ? '{"Wind": { "direction": "Wide" }}' : '{"Wind": { "direction": "Off" }}';
            suffixURL = '/wind';
            break;
        case "spi":
            payload = status ? '{"Mode": { "options": ["Spi_On"] }}' : '{"Mode": { "options": ["Spi_Off"] }}';
            suffixURL = '/mode';
            break;
        case "speed":
            payload = status ? '{"Device": {"Mode": {"options": ["Comode_Speed"]},  "Wind": {"direction": "Off"}}}' : '{"Device": {"Mode": {"options": ["Comode_Off"]},  "Wind": {}}}';
            suffixURL = '';
            break;
        default:
            break;
    }
    
    return [payload, suffixURL];
}

/*
* make payload to turn on/off modes for HAP.
* */
 function humidityModePayload(id, currentStatus){
    var payload = "";
    var suffixURL = "";
    var statusText = "";

    if(currentStatus === true){
        statusText = "Off";
    }else if(currentStatus === false){
        statusText = "On";
    }

    switch(id){
        case "humidi":
            payload = '{"Device":{"Mode" : {"options" : ["Humidi_' + statusText + '"]} } }';
            break;
        case "sterilizeClean":
            payload = '{"Device":{"Mode" : {"options" : ["SterilizeClean_' + statusText + '"]} } }';
            break;
        case "sterilize":
            payload = '{"Device":{"Mode" : {"options" : ["Sterilize_' + statusText + '"]} } }';
            break;
        default:
            console.log('unexpected humidity id:' + id);
            break;
    }
     return [payload, suffixURL];
}

function settingPayload(id, status) {
    var payload, suffixURL;
    status = !status;
    
    switch (id) {
        case "lighting":
            payload = status ? '{"Device": {"Light": { "dimmingLevel": 1 }}}' : '{"Device": {"Light": { "dimmingLevel": 0 }}}';
            suffixURL = '';
            break;
        default:
            break;
    }
    
    return [payload, suffixURL];
}

function powerOnOffPayload(status) {
    var payload;
    if (status) {
        payload = '{"Device":{"Mode" : {"options" : ["Comode_Off"]},"Operation": {"power": "Off"}}}';
    } else {
        payload = '{"Device":{"Operation": {"power": "On"}}}';
    }
    return payload;
}

function _getAirPurityColorKR(value) {
    value = parseInt(value,10);
    switch (value) {
        case 1:
            return 'goodColor';
        case 2:
            return 'normalColor';
        case 3:
            return 'badColor';
        case 4:
        case 5:
            return 'veryBadColor';
        default:
            return 'offColor';
    }
}
function _getAirPurityColorCN5(value) {
    value = parseInt(value,10);
    switch (value) {
        case 1:
            return 'goodColor';
        case 2:
            return 'normalColor';
        case 3:
            return 'littleBadColor';
        case 4:
            return 'badColor5Steps';
        case 5:
            return 'veryBadColor';
        default:
            return 'offColor';
    }
}
function _getAirPurityColorCN6(value) {
    value = parseInt(value,10);
    switch (value) {
        case 1:
            return 'excellentColorCN';
        case 2:
            return 'goodColorCN';
        case 3:
            return 'lightColorCN';
        case 4:
            return 'moderateColorCN';
        case 5:
            return 'heavyColorCN';
        case 6:
            return 'seriousColorCN';
        default:
            return 'offColor';
    }
}
function getAirPurityColor(value, selectedCountry, steps) {
    value = parseInt(value,10);
    if(selectedCountry === 'CN' && steps == 5) {
        return _getAirPurityColorCN5(value);
    } else if(selectedCountry === 'CN' && steps == 6) {
        return _getAirPurityColorCN6(value);
    } else {
       return _getAirPurityColorKR(value); 
    }    
 };

function _getPurityTextKR(value) {
    switch (value) {
        case 1:
            return 'ACMOB_VTL_fresslevel_good';
        case 2:
            return 'WEBMOB_VTL_level_normal';
        case 3:
            return 'ACMOB_VTL_fresslevel_bad';
        case 4:
        case 5:
            return 'ACMOB_VTL_fresslevel_very_bad';
        default:
            return '';
    }
}
function _getPurityTextCN5(value) {
    switch (value) {
        case 1:
            return 'ACMOB_VTL_fresslevel_good';
        case 2:
            return 'WEBMOB_VTL_level_normal';
        case 3:
            return 'ACMOB_VTL_fresslevel_little_bad';
        case 4:
            return 'ACMOB_VTL_fresslevel_bad';
        case 5:
            return 'ACMOB_VTL_fresslevel_very_bad';
        default:
            return '';
    }
}
function _getPurityTextCN6(value) {
    switch (value) {
        case 1:
            return 'WEBMOB_device_ap_freshlevel_china_excellent';
        case 2:
            return 'WEBMOB_device_ap_freshlevel_china_good';
        case 3:
            return 'WEBMOB_device_ap_freshlevel_china_light';
        case 4:
            return 'WEBMOB_device_ap_freshlevel_china_moderate';
        case 5:
            return 'WEBMOB_device_ap_freshlevel_china_heavy';
        case 6:
            return 'WEBMOB_device_ap_freshlevel_china_serious';
        default:
            return '';
    }
}
function getPurityText(value, selectedCountry, steps) {
    value = parseInt(value,10);
    if(selectedCountry === 'CN' && steps == 5) {
        return _getPurityTextCN5(value);
    } else if(selectedCountry === 'CN' && steps == 6) {
        return _getPurityTextCN6(value);
    } else {
       return _getPurityTextKR(value); 
    }
};

function getPurityTextCNAQI(value) {
    value = parseInt(value,10);
    var cvalue;
    if(value >= 0 && value <= 50 ) {
        cvalue = 1;
    } else if(value > 50 && value <= 100 ) {
        cvalue = 2;
    } else if(value > 100 && value <= 150 ) {
        cvalue = 3;
    } else if(value > 150 && value <= 200 ) {
        cvalue = 4;
    } else if(value > 200 && value <= 300 ) {
        cvalue = 5;
    } else if(value > 300 ) {
        cvalue = 6;
    } else {
        cvalue = -1;
    }
    return _getPurityTextCN6(cvalue);

};

function getPurityTextCNPM25(value) {
    value = parseInt(value,10);
    var cvalue;
    if(value >= 0 && value <= 35 ) {
        cvalue = 1;
    } else if(value > 35 && value <= 75 ) {
        cvalue = 2;
    } else if(value > 75 && value <= 115 ) {
        cvalue = 3;
    } else if(value > 115 && value <= 150 ) {
        cvalue = 4;
    } else if(value > 150 && value <= 250 ) {
        cvalue = 5;
    } else if(value > 250 ) {
        cvalue = 6;
    } else {
        cvalue = -1;
    }
    return _getPurityTextCN6(cvalue);
};

function getAirPurityColorCNAQI(value) {
    value = parseInt(value,10);
    if(value >= 0 && value <= 50 ) {
        return 'excellentColorCN';
    } else if(value > 50 && value <= 100 ) {
        return 'goodColorCN';
    } else if(value > 100 && value <= 150 ) {
        return 'lightColorCN';
    } else if(value > 150 && value <= 200 ) {
        return 'moderateColorCN';
    } else if(value > 200 && value <= 300 ) {
        return 'heavyColorCN';
    } else if(value > 300 ) {
        return 'seriousColorCN';
    } else {
        return 'offColor';
    }
};
    
function getAirPurityCNAQI(value) {
    value = parseInt(value, 10);
    if(value >= 0 && value <= 50 ) {
        return 1;
    } else if(value > 50 && value <= 100 ) {
        return 2;
    } else if(value > 100 && value <= 150 ) {
        return 3;
    } else if(value > 150 && value <= 200 ) {
        return 4;
    } else if(value > 200 && value <= 300 ) {
        return 5;
    } else if(value > 300 ) {
        return 6;
    } else {
        return -1;
    }
}

function _getAirPurityImageKR(value) {
    switch (value) {
        case 1:
            return 'purity-level-good';
        case 2:
            return 'purity-level-normal';
        case 3:
            return 'purity-level-bad';
        case 4:
        case 5:
            return 'purity-level-verybad';
        default:
            return 'purity-level-off';
    }
}
function _getAirPurityImageCN5(value) {
    switch (value) {
        case 1:
            return 'purity-level-good';
        case 2:
            return 'purity-level-normal';
        case 3:
            return 'purity-level-little-bad';
        case 4:
            return 'purity-level-bad_5Steps';
        case 5:
            return 'purity-level-verybad';
        default:
            return 'purity-level-off';
    }
}
function _getAirPurityImageCN6(value) {
    switch (value) {
        case 1:
            return 'purity-level-excellent_cn';
        case 2:
            return 'purity-level-good_cn';
        case 3:
            return 'purity-level-light_cn';
        case 4:
            return 'purity-level-moderate_cn';
        case 5:
            return 'purity-level-heavy_cn';
        case 6:
            return 'purity-level-verySerious_cn';
        default:
            return 'purity-level-off';
    }
}

function getAirPurityImage(value, selectedCountry, steps) {
    value = parseInt(value,10);
    if(selectedCountry === 'CN' && steps == 5) {
        return _getAirPurityImageCN5(value);
    } else if(selectedCountry === 'CN' && steps == 6) {
        return _getAirPurityImageCN6(value);
    } else {
       return _getAirPurityImageKR(value); 
    }
}

function getPM10GradeKR(value) {
    var temp;
    value = parseInt(value, 10);
    if(value >= 0 && value <= 30) {
        temp = 1;
    } else if(value > 30 && value <= 80) {
        temp = 2;
    } else if(value > 80 && value <= 150) {
        temp = 3;
    } else if(value > 150 && value <= 999) {
        temp = 4;
    } else {
        temp = -1;
    }
    return temp;
}

function getPM25GradeKR(value) {
    var temp;
    value = parseInt(value, 10);
    if(value >= 0 && value <= 15) {
        temp = 1;
    } else if(value > 15 && value <= 50) {
        temp = 2;
    } else if(value > 50 && value <= 100) {
        temp = 3;
    } else if(value > 100 && value <= 999) {
        temp = 4;
    } else {
        temp = -1;
    }
    return temp;
}

function getKHAIGradeKR(value) {
    var temp;
    value = parseInt(value, 10);
    if(value >= 0 && value <= 50) {
        temp = 1;
    } else if(value > 50 && value <= 100) {
        temp = 2;
    } else if(value > 100 && value <= 250) {
        temp = 3;
    } else if(value > 250 && value <= 999) {
        temp = 4;
    } else {
        temp = -1;
    }
    return temp;
}

function getThreeVentImageClass(ventlevel, APCONST) {
    switch (ventlevel) {
    case APCONST.VENTLEVEL_0:
        return "vent_3_none";
    case APCONST.VENTLEVEL_1:
        return "vent_3_upper";
    case APCONST.VENTLEVEL_2:
        return "vent_3_middle";
    case APCONST.VENTLEVEL_3:
        return "vent_3_upper_middle";
    case APCONST.VENTLEVEL_4:
        return "vent_3_lower";
    case APCONST.VENTLEVEL_5:
        return "vent_3_upper_lower";
    case APCONST.VENTLEVEL_6:
        return "vent_3_middle_lower";
    case APCONST.VENTLEVEL_7:
        return "vent_3_upper_middle_lower";
    default:
        return "";
    }
}

function escapeRegExp(string) {
    return string.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, "\\$1");
}

function _switchOpt(m) {
    var id = "#" + m;
    $(id).toggleClass('off-state').toggleClass('on-state');
    if ($(id).find(".toggle-state span").html() === "ON") {
        $(id).find(".cir").show().addClass("cir-scale");
        setTimeout(function () {
            $(id).find(".toggle-state span").html("OFF");
            $(id).find(".toggle-state span").removeClass('on-txt');
            $(id).find(".toggle-state span").addClass('off-txt');
            setTimeout(function () {

                $(id).find(".toggle-state").removeClass("color-ch-on");
                $(id).find(".toggle-state").addClass("color-ch-off");
            }, 200);
            $(id).find(".cir").hide();
        }, 500);
    }
    else {
        $(id).find(".cir").show().addClass("cir-scale");
        setTimeout(function () {
            $(id).find(".toggle-state span").html("ON");
            $(id).find(".toggle-state span").removeClass('off-txt');
            $(id).find(".toggle-state span").addClass('on-txt');
            setTimeout(function () {

                $(id).find(".toggle-state").removeClass("color-ch-off");
                $(id).find(".toggle-state").addClass("color-ch-on");
            }, 200);
            $(id).find(".cir").hide();
        }, 500);
    }
}