/*******************************************************************************
 * @author Gaurav Sharma (g.sharma@samsung.com)
 * @file This file implements Dishwasher controller function  
 * Copyright 2016 by Samsung Electronics, Inc.,
 * @license This software is the confidential and proprietary information of Samsung
 * Electronics, Inc. ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Samsung.
 ******************************************************************************/
app.controller("DWController", ["$scope", "SHPService", "EnergyService", "CourseParseService", "toaster", "$timeout", "$interval", "$state", DWController]);

var DWCONST = {
    LEN_COURSE: 2,
    LEN_OPTION: 4,
    STATUS_204: 204,
    STATUS_201: 201,
    STATUS_400: 400,
    STATUS_99000: 99000,
    COURSE_INDEX_NAME: 0,
    COURSE_INDEX_ICON: 1,
    START_INDEX_OPTION1: 2,
    OPTIONTYPE_SANITIZE : 0xB,
    OPTIONTYPE_UPPER_LOWER : 0x8,
    OPTIONTYPE_HITEMP_WASH : 0xC,
    OPTIONTYPE_ZONE_BOOSTER : 0x9,
    OPTIONTYPE_SPEED_BOOSTER : 0xA,

    DEVICETYPE_US : '0812',
    DEVICETYPE_EUROPE : '0813',
    DEVICETYPE_KOREA : '0811',

    GET: "GET",
    PUT: "PUT",
    POST: "POST",
    GET_DEVICES: "getDevices",
    DIAGNOSIS: "diagnosis",
    DW_DEVICE_TYPE: "Dishwasher",
    SCHEDULEDATA: "SCHEDULEDATA"
};
var WASHER_COMMANDS = {
    "GET_DEVICES": "getDevices",
    "PARSE_CONFIGURATION": "PARSE_CONFIGURATION",
    "PAUSE_COMMAND":"PAUSE_COMMAND",
    "START_COMMAND":"START_COMMAND",
    "CANCEL_COMMAND":"CANCEL_COMMAND",
    "MODULE_INFO":"MODULE_INFO",
    "DIAGNOSIS_COMMAND":"DIAGNOSIS_COMMAND",
    "SETTINGS_COMMAND":"SETTINGS_COMMAND",
    DB_RESET: 'DB_RESET'
};
var MATERIAL_DESIGN = {
    "LIST_ANIMATION_TIME": 150, // Animation time for ripple effect in list
    "POWER_ANIMTION_TIMEOUT": 130,
    "BACK_ANIMTION_TIMEOUT": 130,
    "TOGGLE_TIME": 330,
    "BTN_ANIMATION_TIME": 480
};
var ALERT_STATE = {
    CANCEL_CYCLE: 1,
    CYCLEFINISHED: 2,
    POWEROFF: 3,
    CONNECTION_FAILURE: 4,
    SMART_CONTROL_DISABLED:5,
    VERSION_INFO:6,
    COUNTRY_SELECTION:7,
    SELF_CLEAN_ALARM:8,
    ADD_DISH_ALARM:9,
    CANCEL_AND_DRAIN:10,
    SERVICE_CALL:11,
    DOOR_OPEN:12,
    SERVICE_CENTER_SELECTION:13, 
    DUPLICATE_NAME: 14,
    DELETE_FAVORITE_ITEM:15,
    DELETE_FAVORITE_ITEMS:16
};
var WASHER_SCREEN = {
    HOME_PAGE: 1,
    SETTINGS_PAGE: 2,
    ABOUT_PAGE: 3,
    SELF_CARE_PAGE: 4,
    ERROR_PAGE:5,
    NO_ERROR_PAGE:6,
    SPLASH_PAGE:7,    
    ENERGY_MONITOR:8,
    FAVORITE_PAGE:9,
    ADD_FAVORITE_PAGE:10,
    SELF_CLEAN_HISTORY_PAGE:11, 
    CYCLE_PAGE:12
};

function DWController($scope, SHPService, EnergyService, CourseParseService, toaster, $timeout, $interval, $state){
    var promise, reqParamCountrySelection, previousPage = -1, langArr = Object.keys(LANGUAGE_FILE.language), MAX_FAVORITE_LIST = 20, contentLoadedListener=null;
    
    if (langArr.indexOf(LANG.toUpperCase())!== -1){
        $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase()];
    } else if(langArr.indexOf(LANG.toUpperCase().substring(0,2))!== -1){
        $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase().substring(0,2)];
    } else {
        $scope.translation = LANGUAGE_FILE["language"]["DEFAULT"];
        LANG = "EN";
    }
    
    if (window.navigator.platform === 'iPhone' || window.navigator.platform === 'iPad') {
        $scope.applePhone = true;
    } else {
        $scope.applePhone = false;
    }
    
    var errorCodeMap = {
        'OC':{code:'OC', title:$scope.translation.WEBMOB_device_dishwasher_error_title_oc, description:$scope.translation.WEBMOB_device_dishwasher_error_desc_oc},
        'LC':{code:'LC', title:$scope.translation.WEBMOB_device_dishwasher_error_title_lc, description:$scope.translation.WEBMOB_device_dishwasher_error_desc_lc},
        '4C':{code:'4C', title:$scope.translation.WEBMOB_device_dishwasher_error_title_4c, description:$scope.translation.WEBMOB_device_dishwasher_error_desc_4c},
        '5C':{code:'5C', title:$scope.translation.WEBMOB_device_dishwasher_error_title_5c, description:$scope.translation.WEBMOB_device_dishwasher_error_desc_5c},
        'HC':{code:'HC', title:$scope.translation.WEBMOB_device_dishwasher_error_title_hc, description:$scope.translation.WEBMOB_device_dishwasher_error_desc_hc},
        '3C':{code:'3C', title:$scope.translation.WEBMOB_device_dishwasher_error_title_3c, description:$scope.translation.WEBMOB_device_dishwasher_error_desc_3c},
        'PC':{code:'PC', title:$scope.translation.WEBMOB_device_dishwasher_error_title_pc, description:$scope.translation.WEBMOB_device_dishwasher_error_desc_pc}
    };
    
    $scope.isLanguageEnglish = (LANG==='EN')?true:false;
    $scope.gradient = true;  
    $scope.isAppV2 = false;
    $scope.homePage = false;    
    $scope.currentPage = WASHER_SCREEN.SPLASH_PAGE;
    $scope.isStatic = false;
    $scope.clockwise = true;
    $scope.progressHead = false;
    $scope.progressStart = false;
    $scope.isUpperRackSelected = false;
    $scope.isLowerRackSelected = true;
    $scope.isZoneBooster = true;
    $scope.isSpeedBooster = true;
    $scope.isHighTempWash = true;
    $scope.isSanitize = true;
    $scope.mainLoadingScreen = false;
    $scope.rackSelectionPopup = false;
    $scope.remoteControlEnabled = false;
    $scope.kidsLock = false;
    $scope.max = 100;
    $scope.current  = 0;
    $scope.iterations = 50;
    $scope.circleWidth = 0;
    $scope.selectedModeIndex = 1;
    $scope.circleHeight = 0;
    $scope.incrementProgress = 0.1;
    $scope.radius = $scope.circleHeight / 2;
    $scope.stroke = Math.ceil($scope.circleHeight / 20);
    $scope.disableContents = false;
    $scope.progressStateMap = {
        "None":'',
        "Prewash":$scope.translation.WEBMOB_device_dishwasher_progressing_washing,
        "Wash":$scope.translation.WEBMOB_device_dishwasher_progressing_washing,
        "Rinse":$scope.translation.WEBMOB_device_dishwasher_progressing_rinsing,
        "Drying":$scope.translation.WEBMOB_device_dishwasher_progressing_drying,
        "Predrain":$scope.translation.WEBMOB_device_dishwasher_progressing_draining,
        "Delaywash":$scope.translation.WEBMOB_device_washer_progressing_delay_start,
        "Finish":"Finish"
    };
  
    if($scope.applePhone) {
        $scope.diameter = $scope.radius * 2 + 2;
    } else {
        $scope.diameter = $scope.radius * 2;
    }

    $scope.switchON = (LANG.toUpperCase().indexOf('EN') !== -1)?'ON':'I';
    $scope.switchOff = (LANG.toUpperCase().indexOf('EN') !== -1)?'OFF': 'O';
    $scope.rounded = true;
    $scope.bgColor = '#FAFAFA';
    $scope.headerTitle = $scope.translation.WEBMOB_common_device_dishwasher;
    $scope.selectedLanguage = LANG;
    $scope.remainingTime = "";
    $scope.remHrs = "";
    $scope.remMin = "";
    $scope.delayStartStatus = $scope.translation.WEBMOB_device_dishwasher_delaystart_none;
    $scope.delHrs = "00";
    $scope.delHrsTemp = "00";
    $scope.delMin = "00";
    $scope.delayStartValue = "00"; // This is used for setting delat time from UI
    $scope.estimationTime = "";
    $scope.currentColor = 'rgb(0,224,232)';
    $scope.guideMessageRack = "";
    $scope.currentAnimation = 'easeOutCubic';
    $scope.smartControlOnOffText = $scope.translation.WEBMOB_common_off_capital;
    $scope.checkResponse = [];
    $scope.showHelpPage = false;
    $scope.dialogPopup = false;
    $scope.alertState = -1;
    $scope.isRadioSelection = false;
    $scope.showServiceCenterText = false;
    $scope.popupTitle = '';
    $scope.popupMessage = '';
    $scope.popupOkBtn = '';
    $scope.popupCancelBtn = '';
    $scope.optionPopup = false;
    $scope.smartControlPopup = false;
    $scope.delayStartPopup = false;
    $scope.rackSelectionPopup = false;
    $scope.modePopup=false;
    $scope.radioBtnToSelect = [];
    $scope.radioBtnSelectedIndex = 0;
    $scope.smartCare = false;
    $scope.smartCareMessage = false;
    $scope.diagnosisState = '';
    $scope.showSelfCheck = false;
    $scope.optionsMenu = [{
        id:0,
        title:$scope.translation.WEBMOB_common_settings
    },{
        id: 1,
        title: $scope.translation.WEBMOB_common_about_device
    },{
        id: 2,
        title: $scope.translation.WEBMOB_common_energy_monitor_reset
    },{
        id: 3,
        title: $scope.translation.WEBMOB_device_dishwasher_comm_delete
    }];
    
    $scope.stateDraining = false; 
    $scope.stateFinish = false;
    $scope.showEnergyMonitorOption = false;
    $scope.energyMonitorPage = false;
    $scope.isDBDownloaded = false;
    $scope.startDiagnosis = false; 
    //Alarm popup variables
    $scope.isAlarmClosed = false;
    
    $scope.displayAidAlarm = false;
    $scope.inAppAidAlarms = [];
    $scope.AddDishAlarm = false;
    $scope.selfCleanAlarm = false;
    $scope.alarmInfoText = '';
    $scope.alarmIcon = '';
    $scope.alarmExpanded = false;
    
    $scope.errorTitle = "";
    $scope.errorCode = "";
    $scope.errorDescription = "";
    
    $scope.cancelDrainBtn = $scope.translation.WEBMOB_device_dishwasher_btn_cancel_drain;
    $scope.zoneBoosterLabel = $scope.translation.WEBMOB_device_dishwasher_option_zonebooster;
    $scope.autoReleaseLabel = $scope.translation.WEBMOB_device_dishwasher_settings_autorelease;
    
    $scope.washingTimes = null;
    $scope.drumCleanProposal = null;
    $scope.drumCleanLog = 'Empty';
    $scope.drumCleanAlert = false;
    
    $scope.addFavoritePage = false;
    $scope.favoritePage = false;
    $scope.favoriteObject = {
        'id':'',
        'name':'',
        'CurrentCourseHexString':'',
        'modeIndex':0,
        'isUpperRackSelected':false,
        'isLowerRackSelected':true,
        'isZoneBooster':false,
        'isSpeedBooster':false,
        'isHighTempWash':false,
        'isSanitize':false,
        'isChecked':false
    };
    
    $scope.favoriteWashName = '';
    $scope.favoriteList = [];
    $scope.isMultiSelectMode = false;
    $scope.allSelected = false;
    $scope.isAnyItemChecked = false;
    $scope.selectedFavoriteItemIndex = null;
    $scope.backFromCyclePage = false;
    
    
    $scope.init = init;
    $scope.selectIndex = selectIndex;
    $scope.callGetDevice = callGetDevice;
    $scope.onBackPressed = onBackPressed;
    $scope.toggleSanitize = toggleSanitize;
    $scope.onRackSelected = onRackSelected;
    $scope.closeModePopUp = closeModePopUp;
    $scope.getModeSelected = getModeSelected;
    $scope.parseStaticData = parseStaticData;
    $scope.parseDeviceData = parseDeviceData;
    $scope.parseModuleInformation = parseModuleInformation;
    $scope.versionTouchEnd = versionTouchEnd;
    $scope.versionTouchStart = versionTouchStart;
    $scope.getProgressColor = getProgressColor;
    $scope.updateDeviceData = updateDeviceData;
    $scope.handleStatusCode = handleStatusCode;
    $scope.toggleZoneBooster = toggleZoneBooster;
    $scope.parseNotification = parseNotification;
    $scope.parseConfiguration = parseConfiguration;
    $scope.requestFailHandler = requestFailHandler;
    $scope.toggleSpeedBooster = toggleSpeedBooster;
    $scope.toggleHighTempWash = toggleHighTempWash;
    $scope.parseRequestResponse = parseRequestResponse;
    $scope.parseRequestAccepted = parseRequestAccepted;
    $scope.openPopup = openPopup;
    $scope.showConnectionFailurePopup = showConnectionFailurePopup;
    $scope.closeConnectionFailurePopup = closeConnectionFailurePopup;
    $scope.showUsageHelpPage = showUsageHelpPage;
    $scope.closeUsageHelpPage = closeUsageHelpPage;
    $scope.startProgress = startProgress;
    $scope.cancelCycleConfirm = cancelCycleConfirm;
    $scope.closeApp = closeApp;
    $scope.popupBtnClicked = popupBtnClicked;
    $scope.selectIndexFromOptionMenu = selectIndexFromOptionMenu;
    $scope.closeAllPopups = closeAllPopups;
    $scope.aboutAction = aboutAction;
    $scope.settingsAction = settingsAction;
    $scope.onRadioBtnSelected = onRadioBtnSelected;
    $scope.startDiagnosisNow = startDiagnosisNow;
    $scope.openEnergyMonitor = openEnergyMonitor;
    $scope.toggleAlarmInfoArea = toggleAlarmInfoArea;
    $scope.showAlarmInfo = showAlarmInfo;
    $scope.hideAlarmInfo = hideAlarmInfo;
    $scope.alarmBtnClicked = alarmBtnClicked;
    $scope.updateDelayStartTime = updateDelayStartTime;
    $scope.setDelayStartValue = setDelayStartValue;
    $scope.increaseDelayContinuously = increaseDelayContinuously;
    $scope.decreaseDelayContinuously = decreaseDelayContinuously;
    $scope.stopLongPressEvent = stopLongPressEvent;
    $scope.fetchDBfromDevice = fetchDBfromDevice;
    $scope.sendResetCommand = sendResetCommand;
    $scope.parseEnergyUsageData = parseEnergyUsageData;
    $scope.isEnergyReset = isEnergyReset;
    $scope.isEnergyResetDisabled = isEnergyResetDisabled;
    $scope.showPreviousDaysData = showPreviousDaysData;	
    $scope.showNextDaysData = showNextDaysData;
    $scope.showMonthlyData = showMonthlyData;
    $scope.showWeeklyData = showWeeklyData;
    $scope.showDailyData = showDailyData;
    $scope.showOptionItemsVisible = showOptionItemsVisible;
    $scope.openSelfCleanHistory = openSelfCleanHistory;
    $scope.openMyfavorite = openMyfavorite;
    $scope.openAddFavorite = openAddFavorite;
    $scope.createArray = createArray;
    $scope.saveFavorite = saveFavorite;
    $scope.isChecked = isChecked;
    $scope.toggleAllCheckList = toggleAllCheckList;
    $scope.toggleCheckList = toggleCheckList;
    $scope.numOfCheckedItems = numOfCheckedItems;
    $scope.cancelMultiSelect = cancelMultiSelect;
    $scope.showSelectableFavoriteList = showSelectableFavoriteList;
    $scope.deleteItemFromFavorite = deleteItemFromFavorite;
    $scope.confirmDeleteItemsFromFavorite = confirmDeleteItemsFromFavorite;
    //Commented function to check whether to enable save button in add/edit favorite page
    //$scope.hasFavoriteChanged = hasFavoriteChanged;
    $scope.editMyFavorite = editMyFavorite;
    $scope.startFavorite = startFavorite;    
    $scope.openCycleSelectionPage = openCycleSelectionPage;
    
    if (contentLoadedListener !== null){ //remove listener if already assigned
        contentLoadedListener();
        contentLoadedListener = null;
    }
    //Returns a deregistration function for this listener.
    contentLoadedListener = $scope.$on('$viewContentLoaded',  function(event){ //listen to page load event and do page specifc assignments
        debugMessage("$viewContentLoaded with page " + $scope.currentPage);
        switch($scope.currentPage){
            case WASHER_SCREEN.HOME_PAGE:
                $scope.homePage = true;
                $scope.headerTitle = $scope.translation.WEBMOB_common_device_dishwasher;
                // if (!$scope.mainLoadingScreen){//For the first launch make configuration call
                // configurationCall();
                // }
                $scope.mainLoadingScreen = true;
                //checking if to show usage help page - just in case of first launch
                var localStorageFUGKey = ($scope.isStatic)?'Demo_Dishwasher_firstTimeLaunch':'Dishwasher_firstTimeLaunch';
                var hasLaunchedBefore = getLocalStorageData(localStorageFUGKey);
                if (!hasLaunchedBefore) {
                    showUsageHelpPage();
                    setLocalStorageData(localStorageFUGKey, true);
                }
                break;
            case WASHER_SCREEN.SETTINGS_PAGE://
                $scope.headerTitle = $scope.translation.WEBMOB_common_settings;
                break;
            case WASHER_SCREEN.ABOUT_PAGE:   
                $scope.headerTitle = $scope.translation.WEBMOB_common_about_device;
                break;
            case WASHER_SCREEN.NO_ERROR_PAGE://FALLTHROUGH
            case WASHER_SCREEN.SELF_CARE_PAGE:
                configurationCall();
                $scope.mainLoadingScreen = true;
                $scope.smartCare = true;
                $scope.smartCareMessage = false;
                $scope.headerTitle = $scope.translation.WEBMOB_common_self_diagnosis;
                break;
            case WASHER_SCREEN.ENERGY_MONITOR:
                EnergyService.goToEnergyMonitor($scope);
                $scope.headerTitle = $scope.translation.WEBMOB_common_energy_monitor_title;
                break;
            case WASHER_SCREEN.ERROR_PAGE:
                if(previousPage === WASHER_SCREEN.SPLASH_PAGE){ //on app launch make configuration call
                    configurationCall();
                }
                $scope.mainLoadingScreen = true;//in case on launch error page is loaded instead of home page
                $scope.headerTitle = $scope.translation.WEBMOB_common_device_dishwasher;
                break;
            case WASHER_SCREEN.FAVORITE_PAGE:
                $scope.favoritePage = true;
                $scope.isMultiSelectMode = false;
                $scope.isAnyItemChecked = false;
                $scope.allSelected = false;   
                $scope.headerTitle = $scope.translation.WEBMOB_device_dishwasher_settings_my_favorite;
                $scope.favoriteList = [];
                $scope.selectedFavoriteItemIndex = null;
                var favList = getLocalStorageData($scope.deviceUuid + '_Favorite') || [];
                if (favList.length !==0){
                    $scope.favoriteList = JSON.parse(favList);                        
                }
                break;
            case WASHER_SCREEN.SELF_CLEAN_HISTORY_PAGE:
                $scope.headerTitle = $scope.translation.WEBMOB_device_dishwasher_settings_self_clean_history;
                applyHistoryFilterStyle();
                break;
            case WASHER_SCREEN.ADD_FAVORITE_PAGE:  
                $scope.addFavoritePage = true;
                if ($scope.selectedFavoriteItemIndex === null && !$scope.backFromCyclePage){//Edit case + back from cycle page donot initialize favorite object
                    $scope.favoriteWashName = getMyFavoriteName();
                    $scope.favoriteObject = { //reinitialize favorite with default values
                        'id':'',
                        'name':$scope.favoriteWashName,
                        'CurrentCourseHexString':'',
                        'modeIndex':0,
                        'isUpperRackSelected':false,
                        'isLowerRackSelected':true,
                        'isZoneBooster':false,
                        'isSpeedBooster':false,
                        'isHighTempWash':false,
                        'isSanitize':false, 
                        'isChecked':false
                    };
                    setFavoriteCourseOptions();
                }else {
                     $scope.favoriteWashName = (!$scope.backFromCyclePage)?$scope.favoriteObject.name:$scope.favoriteWashName;
                }
                $scope.backFromCyclePage = false; //reset flag
                break;
            case WASHER_SCREEN.CYCLE_PAGE:
                $scope.headerTitle = $scope.translation.WEBMOB_device_dishwasher_comm_cycle;
                break; 
            default:
                 //Do nothing
                break;
        }        
    });
    
    function init() {
        debugMessage("Init called >>>");
        $scope.stateFinish = false;
        $scope.Device = {
            Alarms: [{
                alarmType: '',
                code: '',
                id: '',
                triggeredTime: ''
            }],
            AlarmIndicator:{            // UI preferred Data
                isRinseAidAlarmShow: false,
                isSaltIconShow: false,
                isAddDishAlarmShow: false, 
                isSelfCleanAlarmShow: false,
                isDoorOpenAlarmShow:false
            },
            ConfigurationLink: {
                href: ''
            },
            InformationLink: {
                href: ''
            },
            Mode: {
                options: [],
                supportedOptions: ''
            },
            Operation: {
                delayStartTime: '',
                progress: '',
                progressPercentage: '',
                remainingTime: '',
                state: 'Ready'
            },
            Washer:{
                supportedUpperLower: [],    // Washer::supportedWaterTemperature
                supportedZoneBooster: [],   // Washer::supportedRinseCycles
                supportedSpeedBooster: [],  // Washer::supportedSpinLevel
                supportedHiTempWash: [],    // Washer::supportedSoilLevel
                supportedSanitize: [],      // Washer::supportedDryLevel
                upperLower: '',             // Washer::waterTemperature
                zoneBooster: '',            // Washer::rinseCycles
                speedBooster: '',           // Washer::spinLevel
                hiTempWash: '',             // Washer::soilLevel
                sanitize: ''                // Washer::dryLevel
            },
            CurrentCourseHexString: '',     // UI preferred Data
            CourseData: [],                 // UI preferred Data
            CurrentCycleOptions:{           // UI preferred Data, set it from Washer::rinseCycles, spinLevel ...
                isSpeedBoosterOn: false,
                isHiTempWashOn: false,
                isSanitizeOn: false,
                isZoneBoosterOn: false,
                isUpperRackSelected: false,
                isLowerRackSelected: true
            },
            SettingOptions:{                // UI preferred Data
                isAutoReleaseOn: false,
                isSoundOn: false
            },
            connected: '',
            description: '',
            id: '', name: '',
            resources: [],
            type: '',
            uuid: ''
        };
       
       
        if (!$scope.isStatic) {
            configurationCall();
        }
    }
    
    function callGetDevice() {
        SHPService.getDevice($scope);
    }
    
    function parseStaticData() {
        $scope.isStatic = true;
        $scope.isAppV2 = true;
        /* RUN State response */
        //'{"type":"requestResponse","status":200,"resourceURL":"/000001_R8S7TCH3ULO6OCLLKDA1F1AA44000/devices/0","data":{"Device":{"description":"TP6X_DW80M9970US(C097271BF9E5)","id":"0","Alarms":[{"id":"0","alarmType":"Device","code":"DrumClean_Need","triggeredTime":"2016-08-22T11:50:45"},{"id":"1","alarmType":"Device","code":"SaltA_Not_Supported","triggeredTime":"2016-08-22T11:50:45"},{"id":"2","alarmType":"Device","code":"RinseA_Normal","triggeredTime":"2016-08-22T11:50:45"},{"id":"3","alarmType":"Device","code":"DoorA_Closed","triggeredTime":"2016-08-22T11:50:45"},{"id":"4","alarmType":"Device","code":"DishA_Enable","triggeredTime":"2016-08-22T11:50:45"}],"ConfigurationLink":{"href":"/devices/0/configuration"},"Diagnosis":{"diagnosisStart":"Ready"},"EnergyConsumption":{"saveLocation":"/files/usage.db"},"InformationLink":{"href":"/devices/0/information"},"Mode":{"options":["Course_86","DeviceType_0812","AutoDoorRelease_Off","Sound_Off","UsagesDB_ok"],"supportedOptions":["582830C9002A002B002C00283830C9002A002B002C00284830C9002A002B002C00285830C9000A002B000C00086830C9002A000B002C00287830C9000A000B000C0008883089000A000B000C000"]},"Operation":{"delayStartTime":"00:00:00","kidsLock":"Ready","power":"On","progress":"Prewash","progressPercentage":2,"remainingTime":"01:21:00","state":"Run","supportedProgress":["None","Predrain","Wash","Rinse","Drying","Finish"]},"Washer":{"dryLevel":"Off","rinseCycles":"ON_OFF","soilLevel":"On","spinLevel":"Off","supportedDryLevel":["Off","On"],"supportedRinseCycles":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"supportedSoilLevel":["Off","On"],"supportedSpinLevel":["Off","On"],"supportedWaterTemperature":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"waterTemperature":"ON_ON"},"connected":true,"name":"Dishwasher","resources":["Alarms","Configuration","Diagnosis","EnergyConsumption","Information","Mode","Operation","Washer"],"type":"Dishwasher","uuid":"C097271B-F9E5-0000-0000-000000000000"}}}';

        // old shp resource below
        //var deviceResponse = '{"type":"requestResponse","status":200,"resourceURL":"/000001_R8S7TCH3ULO6OCLLKDA1F1AA44000/devices/0","data":{"Device":{"description":"TP6X_DW80M9970US(C097271BF9E5)","id":"0","Alarms":[{"id":"0","alarmType":"Device","code":"DrumClean_Need","triggeredTime":"2016-08-29T03:31:04"},{"id":"1","alarmType":"Device","code":"SaltA_Not_Supported","triggeredTime":"2016-08-29T03:31:04"},{"id":"2","alarmType":"Device","code":"RinseA_Normal","triggeredTime":"2016-08-29T03:31:04"},{"id":"3","alarmType":"Device","code":"DoorA_Closed","triggeredTime":"2016-08-29T03:31:04"},{"id":"4","alarmType":"Device","code":"DishA_Disable","triggeredTime":"2016-08-29T03:31:04"}],"ConfigurationLink":{"href":"/devices/0/configuration"},"Diagnosis":{"diagnosisStart":"Ready"},"EnergyConsumption":{"saveLocation":"/files/usage.db"},"InformationLink":{"href":"/devices/0/information"},"Mode":{"options":["Course_83","DeviceType_0812","AutoDoorRelease_On","Sound_On","UsagesDB_ok"],"supportedOptions":["582830C9002A002B002C00283830C9002A002B002C00284830C9002A002B002C00285830C9000A002B000C00086830C9002A000B002C00287830C9000A000B000C0008883089000A000B000C000"]},"Operation":{"delayStartTime":"00:00:00","kidsLock":"Ready","power":"On","progress":"None","progressPercentage":1,"remainingTime":"02:30:00","state":"Ready","supportedProgress":["None","Predrain","Prewash","Wash","Rinse","Drying","Finish"]},"Washer":{"dryLevel":"Off","rinseCycles":"OFF_OFF","soilLevel":"Off","spinLevel":"Off","supportedDryLevel":["Off","On"],"supportedRinseCycles":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"supportedSoilLevel":["Off","On"],"supportedSpinLevel":["Off","On"],"supportedWaterTemperature":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"waterTemperature":"ON_ON"},"connected":true,"name":"Dishwasher","resources":["Alarms","Configuration","Diagnosis","EnergyConsumption","Information","Mode","Operation","Washer"],"type":"Dishwasher","uuid":"C097271B-F9E5-0000-0000-000000000000"}}}';

        // new shp resource below
        //var deviceResponse = '{"type":"requestResponse","status":200,"resourceURL":"/000001_R8S7TCH3ULO6OCLLKDA1F1AA44000/devices/0","data":{"Device":{"description":"TP6X_DW80M9960US(C097271BF9E5)","id":"0","Alarms":[{"id":"0","alarmType":"Device","code":"DrumClean_Needless","triggeredTime":"2016-10-13T11:11:15"},{"id":"1","alarmType":"Device","code":"SaltA_Not_Supported","triggeredTime":"2016-10-13T11:11:15"},{"id":"2","alarmType":"Device","code":"RinseA_Empty","triggeredTime":"2016-10-13T11:11:15"},{"id":"3","alarmType":"Device","code":"DoorA_Closed","triggeredTime":"2016-10-13T11:11:15"},{"id":"4","alarmType":"Device","code":"DishA_Disable","triggeredTime":"2016-10-13T11:11:15"}],"ConfigurationLink":{"href":"/devices/0/configuration"},"Diagnosis":{"diagnosisStart":"Ready"},"EnergyConsumption":{"saveLocation":"/files/usage.db"},"InformationLink":{"href":"/devices/0/information"},"Mode":{"options":["Course_83","DeviceType_0813","AutoDoorRelease_Off","Sound_On","UsagesDB_ok"],"supportedOptions":["582830C9002A002B002C00283830C9002A002B002C00284830C9002A002B002C00285830C9000A002B000C00086830C9002A000B002C00287830C9000A000B000C0008883089000A000B000C000"]},"Operation":{"delayStartTime":"00:00:00","kidsLock":"Ready","power":"On","progress":"None","progressPercentage":1,"remainingTime":"00:01:00","state":"Ready","supportedProgress":["None","Predrain","Finish"]},"Washer":{"highTemperatureDry":"Off","sanitize":"Off","selectedZone":"ON_ON","speedBooster":"Off","supportedHighTemperatureDry":["Off","On"],"supportedSanitize":["Off","On"],"supportedSelectedZone":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"supportedSpeedBooster":["Off","On"],"supportedZoneBooster":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"zoneBooster":"OFF_OFF"},"connected":true,"name":"Dishwasher","resources":["Alarms","Configuration","Diagnosis","EnergyConsumption","Information","Mode","Operation","Washer"],"type":"Dishwasher","uuid":"C097271B-F9E5-0000-0000-000000000000"}}}';

        //DW Europe
        //var deviceResponse = '{"type":"requestResponse","status":200,"resourceURL":"/000001_K18LFTT4M8BPKN50946GAERBFC000/devices/0","data":{"Device":{"description":"TP6X_DW80M9960US(C097271BF9CB)","id":"0","Alarms":[{"id":"0","alarmType":"Device","code":"DrumClean_Needless","triggeredTime":"2016-12-05T08:45:48"},{"id":"1","alarmType":"Device","code":"SaltA_Empty","triggeredTime":"2016-12-05T08:45:48"},{"id":"2","alarmType":"Device","code":"RinseA_Empty","triggeredTime":"2016-12-05T08:45:48"},{"id":"3","alarmType":"Device","code":"DoorA_Closed","triggeredTime":"2016-12-05T08:45:48"},{"id":"4","alarmType":"Device","code":"DishA_Disable","triggeredTime":"2016-12-05T08:45:48"}],"ConfigurationLink":{"href":"/devices/0/configuration"},"Diagnosis":{"diagnosisStart":"Ready"},"EnergyConsumption":{"saveLocation":"/files/usage.db"},"InformationLink":{"href":"/devices/0/information"},"Mode":{"options":["Course_8A","DeviceType_0813","AutoDoorRelease_Off","Sound_On","UsagesDB_ok","WashingTimes_21","DrumCleanProposal_50","DrumCleanLog_Empty"],"supportedOptions":["582830C9002A002B002C0028A830C9002A002B002C002A7830C9002A002B002C00285830C9000A002B000C000A8830C9002A000B000C0008B830C9000A000B000C0008883089000A000B000C000"]},"Operation":{"delayStartTime":"14:00:00","kidsLock":"Ready","power":"On","progress":"None","remainingTime":"03:48:00","state":"Ready","supportedProgress":["None","Delaywash","Predrain","Prewash","Wash","Rinse","Drying","Finish"],"progressPercentage":1},"Washer":{"highTemperatureDry":"Off","sanitize":"Off","selectedZone":"ON_ON","speedBooster":"Off","supportedHighTemperatureDry":["Off","On"],"supportedSanitize":["Off","On"],"supportedSelectedZone":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"supportedSpeedBooster":["Off","On"],"supportedZoneBooster":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"zoneBooster":"OFF_OFF"},"connected":true,"name":"Dishwasher","resources":["Alarms","Configuration","Diagnosis","EnergyConsumption","Information","Mode","Operation","Washer"],"type":"Dishwasher","uuid":"C097271B-F9CB-0000-0000-000000000000"}}}';
        //var deviceResponse = '{"type":"requestResponse","status":200,"resourceURL":"/000001_K18LFTT4M8BPKN50946GAERBFC000/devices/0","data":{"Device":{"description":"TP6X_DW80M9000(C097271BF9CB)","id":"0","Alarms":[{"id":"0","alarmType":"Device","code":"DrumClean_Needless","triggeredTime":"2017-01-12T07:46:55"},{"id":"1","alarmType":"Device","code":"RinseA_Empty","triggeredTime":"2017-01-12T07:46:55"},{"id":"2","alarmType":"Device","code":"DoorA_Closed","triggeredTime":"2017-01-12T07:46:55"},{"id":"3","alarmType":"Device","code":"DishA_Disable","triggeredTime":"2017-01-12T07:46:55"}],"ConfigurationLink":{"href":"/devices/0/configuration"},"Diagnosis":{"diagnosisStart":"Ready"},"EnergyConsumption":{"saveLocation":"/files/usage.db"},"InformationLink":{"href":"/devices/0/information"},"Mode":{"options":["EnergyKW_396","DrumCleanLog_2016-12-29T07:13:57_003|2016-12-29T07:16:17_655|2016-12-29T07:17:12_003|2016-12-29T07:17:19_003|2016-12-29T07:17:25_004|2016-12-29T07:18:56_003|2017-01-03T03:14:04_040|2017-01-03T09:48:46_004","Course_8B","DeviceType_0813","AutoDoorRelease_Off","Sound_On","UsagesDB_ok","WashingTimes_27","DrumCleanProposal_20"],"supportedOptions":["582830C9002A002B002C0028A830C9002A002B002C002A7830C9002A002B002C00285830C9000A002B000C000A8830C9000A000B000C0008B830C9000A000B000C0008883089000A000B000C000"]},"Operation":{"delayStartTime":"00:00:00","kidsLock":"Ready","power":"On","progress":"Finish","progressPercentage":100,"remainingTime":"00:01:00","state":"Run","supportedProgress":["None","Predrain","Drying","Finish"]},"Washer":{"highTemperatureDry":"Off","sanitize":"Off","selectedZone":"ON_ON","speedBooster":"Off","supportedHighTemperatureDry":["Off","On"],"supportedSanitize":["Off","On"],"supportedSelectedZone":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"supportedSpeedBooster":["Off","On"],"supportedZoneBooster":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"zoneBooster":"OFF_OFF"},"connected":true,"name":"Dishwasher","resources":["Alarms","Configuration","Diagnosis","EnergyConsumption","Information","Mode","Operation","Washer"],"type":"Dishwasher","uuid":"C097271B-F9CB-0000-0000-000000000000"}}}';

        // note : do not push deviceResponse data. It should be us dishwasher ready response.
        //DW US
        var deviceResponse = '{"type":"requestResponse","status":200,"resourceURL":"/000001_R8S7TCH3ULO6OCLLKDA1F1AA44000/devices/0","data":{"Device":{"description":"TP6X_DW80M9960US(C097271BF9E5)","id":"0","Alarms":[{"id":"0","alarmType":"Device","code":"DrumClean_Needless","triggeredTime":"2017-01-16T07:50:02"},{"id":"1","alarmType":"Device","code":"SaltA_Not_Supported","triggeredTime":"2017-01-16T07:50:02"},{"id":"2","alarmType":"Device","code":"RinseA_Empty","triggeredTime":"2017-01-16T07:50:02"},{"id":"3","alarmType":"Device","code":"DoorA_Closed","triggeredTime":"2017-01-16T07:50:02"},{"id":"4","alarmType":"Device","code":"DishA_Disable","triggeredTime":"2017-01-16T07:50:02"}],"ConfigurationLink":{"href":"/devices/0/configuration"},"Diagnosis":{"diagnosisStart":"Ready"},"EnergyConsumption":{"saveLocation":"/files/usage.db"},"InformationLink":{"href":"/devices/0/information"},"Mode":{"options":["Course_83","DeviceType_0812","AutoDoorRelease_On","Sound_On","UsagesDB_ok"],"supportedOptions":["582830C9002A002B002C00283830C9002A002B002C00284830C9002A002B002C00285830C9000A002B000C00086830C9002A000B002C00287830C9000A000B000C0008883089000A000B000C000"]},"Operation":{"delayStartTime":"00:00:00","kidsLock":"Ready","power":"On","progress":"None","progressPercentage":1,"remainingTime":"02:30:00","state":"Ready","supportedProgress":["None","Predrain","Prewash","Wash","Rinse","Drying","Finish"]},"Washer":{"highTemperatureDry":"Off","sanitize":"Off","selectedZone":"ON_ON","speedBooster":"Off","supportedHighTemperatureDry":["Off","On"],"supportedSanitize":["Off","On"],"supportedSelectedZone":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"supportedSpeedBooster":["Off","On"],"supportedZoneBooster":["OFF_OFF","ON_OFF","OFF_ON","ON_ON"],"zoneBooster":"OFF_OFF"},"connected":true,"name":"Dishwasher","resources":["Alarms","Configuration","Diagnosis","EnergyConsumption","Information","Mode","Operation","Washer"],"type":"Dishwasher","uuid":"C097271B-F9E5-0000-0000-000000000000"}}}';

        $timeout(function(){
            parseRequestResponse(JSON.parse(deviceResponse));
            $scope.showEnergyMonitorOption = true;
            $scope.isDBDownloaded = true;
            $scope.remoteControlEnabled = true;
            $scope.smartControlOnOffText = "ON";
            $scope.Device.AlarmIndicator.isAddDishAlarmShow = true;
            $scope.Device.AlarmIndicator.isSelfCleanAlarmShow = true;
        }, 1000);        
    }
    
    function parseRequestResponse(response){
        debugMessage("Parse request response");
        if (angular.isDefined(response["status"]) && response["status"] === DWCONST.STATUS_99000) {
            showConnectionFailurePopup();
            return;
        }

        if (angular.isDefined(response["data"])) {
            var device;

            if (angular.isDefined(response["data"]["Device"])) {
                device = response["data"]["Device"];
                init();
                parseDeviceData(device);
                isCallFromReset = 1;
            }
            else if (angular.isDefined(response["data"]["Configuration"])) {
                parseConfiguration(response["data"]["Configuration"]);
            }

            if (angular.isDefined(response["data"]["Information"])) {
                parseModuleInformation(response["data"]["Information"]);
            }
        }

        if (angular.isDefined(response["resourceURL"])) {
            if (response["resourceURL"].indexOf("files") !== -1) {
                $scope.isDBDownloaded = true;
                isDBDownloadedJS = true;
                var d = new Date();
                var m = d.getMonth() > 9 ? d.getMonth() : "0" + d.getMonth();
                var dd = d.getDate() > 9 ? d.getDate() : "0" + d.getDate();
                var today = d.getFullYear() + "" + m + "" + dd;
                if (isCallFromReset === 2) {
                    dismissLoadingBar(WASHER_COMMANDS.DB_RESET);
                    isCallFromReset = 1;
                    energyResetDone = true;
                    if ($scope.currentPage === WASHER_SCREEN.ENERGY_MONITOR){//fetch the data again
                        EnergyService.goToEnergyMonitor($scope);
                    }else{//move to energy monitor page
                        openEnergyMonitor();
                    }                    
                }
            }

            if (angular.isDefined(response["data"])) {
                if (angular.isDefined(response["data"]["message"]) && (response["resourceURL"].indexOf("/operation") !== -1) && response["status"] === DWCONST.STATUS_204) {
                    parseDeviceData({Operation: ''});
                }
                
            }
        }

        if (response.status >= 200 && response.status < 300) { //<300 is succes only. Refer http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html for status code definitions.
            handleStatusCode(response.status, response);
        }
        
        dismissLoadingBar(WASHER_COMMANDS.GET_DEVICES);
        
        if ($scope.currentPage === WASHER_SCREEN.SPLASH_PAGE){
            $scope.currentPage = WASHER_SCREEN.HOME_PAGE;                
            changeScreen();
        }
    }

    function parseRequestAccepted(response){
        if (angular.isDefined(response["data"]) && angular.isDefined(response["data"]["energyData"])) {
            $scope.parseEnergyUsageData(response["data"]);
        }
        else if (angular.isDefined(response["isSuccess"])) {    // todo : need to check whether is being used.
            $scope.parseDeviceData(response);
        }
    }

    function parseNotification(response){
        // if (angular.isDefined(response["deviceOrientation"])) {
        //     $scope.deviceOrientationChangeEvent(response["deviceOrientation"]);
        //     return;
        // }

        if (angular.isDefined(response["data"])) {
            if (response["data"] === "backPressed") {
                //if loding bar visible, don't handle back key
                if (!!!$scope.loadingBar) {
                    onBackPressed();
                }
                return;
            }
            // if (angular.isDefined(response["data"].isApplicationActive) && !$scope.isStatic) {
            //     $scope.appAlive = true
            //     callGetDevice();
            //     return;
            // }
            if (angular.isDefined(response["data"][0]) && angular.isDefined(response["data"][0]["Device"]) && angular.isDefined(response["data"][0]["Device"]["scsConnectionState"])) {
//                $scope.checkCycleCompleteScreen(response["data"][0]["Device"]["scsConnectionState"]);
//                return;
                if (!response["data"][0]["Device"]["scsConnectionState"]) {
                    showConnectionFailurePopup();
                } else if (response["data"][0]["Device"]["scsConnectionState"]) {
                    $scope.closeConnectionFailurePopup();
                    callGetDevice();
                }
                return;
            }

            // if ($scope.stateFinish){ //DONOT ,parse any notification from the device after cycle complete
            //     return;
            // }
            
            if(response["data"] instanceof Array) {
                response["data"].forEach(function (value) {
                    parseDeviceData(value);
                });
            } else {
                parseDeviceData(response["data"]);
            }
        }
    }

    function parseDeviceData(device) {
        debugMessage("parse device data >>>>>>>>>");        
        if (!!!$scope.isStatic && angular.isDefined(device["uuid"]) && device["uuid"] !== $scope.deviceUuid) {
            return;
        }

        if(!!!$scope.Device) {
            return;
        }

        if (angular.isDefined(device["id"])) {
            $scope.deviceID = parseInt(device["id"], 10);
        }

        // if (angular.isDefined(device["ConfigurationLink"])) {
        //     $scope.Device.ConfigurationLink.href = device["ConfigurationLink"]["href"];
        //      configurationCall();
        // }

        if ($scope.stateFinish){ //DONOT ,parse any notification from the device after cycle complete except for configuration
            if (angular.isDefined(device["Configuration"])) {
                parseConfiguration(device["Configuration"]);
            }
            return;
        }

        if (!!device.Alarms && device.Alarms.length) {
            angular.copy(device.Alarms, $scope.Device.Alarms);
            parseAlarms(device);
        }
        
        if (angular.isDefined(device["Operation"])) {
            parseOperation(device["Operation"]);
        }

        if(angular.isDefined(device["Washer"])){
            parseCycleOptions(device["Washer"]);
        }

        if (angular.isDefined(device["EnergyConsumption"]) && angular.isDefined(device["EnergyConsumption"].saveLocation)) {
            $scope.energyMonitorLocation = device["EnergyConsumption"].saveLocation;
            $scope.showEnergyMonitorOption = true;
            $scope.optionCodeValue = "OptionCode_16384";   // hardcoded for native processing
        }

        if (angular.isDefined(device["Mode"])) {
            parseMode(device["Mode"]);
        }
        
        if (angular.isDefined(device["Configuration"])) {
            parseConfiguration(device["Configuration"]);
        }

        if (angular.isDefined(device["Diagnosis"])) {
            showDiagnosisPage(device["Diagnosis"]);
        }
        
        if (angular.isDefined(device["resources"])) {
            $scope.Device.resources = device["resources"];
            //check if Diagnosis is there, self check option will be displayed iff diagnosis is present in resources
            $scope.Device.resources.forEach(function (value) {            
                if (value.indexOf("Diagnosis") !== -1) {
                    $scope.showSelfCheck = true;
                }
            });
        }
        
        if (angular.isDefined(device["type"])) {
            $scope.deviceType = device["type"];            
        }
        
        if (angular.isDefined(device["mobileTimeChanged"])){//if time changed on device and notified then update estimated time 
            updateEstimatedTime();
        }
    }

    function parseOperation(operation){
        if(angular.isDefined(operation.delayStartTime)){
            $scope.Device.Operation.delayStartTime = operation.delayStartTime;
            $scope.delHrs = $scope.Device.Operation.delayStartTime.substring(0, 2);
            $scope.delMin = $scope.Device.Operation.delayStartTime.substring(3, 5);
            debugMessage("delayStartTime updated>>>>>>>>>>>" + $scope.Device.Operation.delayStartTime);
            debugMessage("delayStartTime updated>>>>>>>>>>>>>>hrs and mins are "+$scope.delHrs+" "+$scope.delMin);
            $scope.delayStartValue = $scope.delHrs;
            $scope.delHrsTemp = $scope.delHrs;

            var tmpString = parseInt($scope.delHrs, 10) + $scope.translation.WEBMOB_device_washer_comm_hr + " " + parseInt($scope.delMin, 10) + $scope.translation.WEBMOB_device_washer_comm_min;
            if($scope.delHrs === '00' && $scope.delMin == '00' && $scope.Device.Operation.progress !== 'Delaywash'){
                tmpString = 'NONE';
            }
            else if ($scope.delHrs === '00'){
                tmpString = parseInt($scope.delMin, 10)+ $scope.translation.WEBMOB_device_washer_comm_min;            
            }else if ($scope.delMin === '00'){
                tmpString = parseInt($scope.delHrs, 10) + $scope.translation.WEBMOB_device_washer_comm_hr;
            }
            $scope.delayStartStatus = ($scope.Device.Operation.progress === 'Delaywash')?tmpString + " " + $scope.translation.WEBMOB_device_dishwasher_remaining:tmpString;
            ($scope.Device.Operation.state === 'Run') && updateEstimatedTime(); //update finish time to be displayed in case of Run state
        }
        if(angular.isDefined(operation.progress)){
            $scope.Device.Operation.progress = operation.progress;
            if ($scope.Device.Operation.progress === 'Finish'){
                $scope.stateFinish = true;
            }else if ($scope.Device.Operation.progress === 'Delaywash'){//reset if delay wash time is set 
                var tmpString = parseInt($scope.delHrs, 10) + $scope.translation.WEBMOB_device_washer_comm_hr + " " + parseInt($scope.delMin, 10) + $scope.translation.WEBMOB_device_washer_comm_min;
                if ($scope.delHrs === '00'){
                    tmpString = parseInt($scope.delMin, 10)+ $scope.translation.WEBMOB_device_washer_comm_min;            
                }else if ($scope.delMin === '00'){
                    tmpString = parseInt($scope.delHrs, 10) + $scope.translation.WEBMOB_device_washer_comm_hr;
                }
                $scope.delayStartStatus = tmpString +  " " + $scope.translation.WEBMOB_device_dishwasher_remaining;       
            }else {
                $scope.delayStartStatus = ($scope.delHrs === '00' && $scope.delMin == "00")?$scope.translation.WEBMOB_device_dishwasher_delaystart_none:$scope.delayStartStatus;
            }
        }
        if(angular.isDefined(operation.progressPercentage)){
            $scope.Device.Operation.progressPercentage = operation.progressPercentage;
            //update progressbar
            $scope.progressHead = true;//Roundprogress variable
            debugMessage($scope.current + ":::Progress percentage is:::" + $scope.max);
            if ($scope.current < $scope.max) {
                $scope.current = $scope.Device.Operation.progressPercentage;                
            }
        }
        if(angular.isDefined(operation.remainingTime)){
            $scope.Device.Operation.remainingTime = operation.remainingTime;
            $scope.remainingTime = $scope.Device.Operation.remainingTime;   // FIXME : remove remainingTime variable
            $scope.remHrs = $scope.remainingTime.substring(0, 2);
            $scope.remMin = $scope.remainingTime.substring(3, 5);
            debugMessage("Remaining time updated>>>>>>>>>>>" + $scope.remainingTime);
            debugMessage("Remaining timeupdated>>>>>>>>>>>>>>hrs and mins are "+$scope.remHrs+" "+$scope.remMin);

            $scope.remainingTime = $scope.remHrs + " " + $scope.translation.WEBMOB_device_washer_comm_hr + " " + $scope.remMin + " " + $scope.translation.WEBMOB_device_washer_comm_min;
            ($scope.Device.Operation.state === 'Run') && updateEstimatedTime(); //update finish time to be displayed in case of Run state
            debugMessage("operation.remainingTime>>>>>>> "+ $scope.remainingTime)
        }
        if(angular.isDefined(operation.state)){
            $scope.Device.Operation.state = operation.state;
            
            if ($scope.alertState === ALERT_STATE.CANCEL_AND_DRAIN || $scope.alertState === ALERT_STATE.CANCEL_CYCLE){//close cancel popup in case opened
                closeAllPopups();
            }
            
            switch($scope.Device.Operation.state){ //dismiss command loading bar after getting response
                case 'Ready':
                    $scope.progressStart = false;//Roundprogress variable
                    $scope.progressHead = false;//Roundprogress variable
                    $scope.current = 0;//Roundprogress variable
                    ($scope.checkResponse.indexOf(WASHER_COMMANDS.CANCEL_COMMAND) !== -1) && dismissLoadingBar(WASHER_COMMANDS.CANCEL_COMMAND);
                    $scope.disableContents = false;
                    /*
                        note : need to check progress since sometimes course_89 comes a head of {"Operation":{"progress":"Predrain","remainingTime":"00:01:00","state":"Ready"}
                               then, stateDraining would be maintained as false value.
                    */
                    if ($scope.Device.Operation.progress !== 'Predrain'){
                        $scope.stateDraining = false;
                    }
                    /*CHECK IF NEED TO SHOW SELF CLEAN ALARM*/
                    ($scope.Device.AlarmIndicator.isSelfCleanAlarmShow) && showAlarmInfo(ALERT_STATE.SELF_CLEAN_ALARM);
                    break;
                case 'Run':
                    ($scope.checkResponse.indexOf(WASHER_COMMANDS.START_COMMAND) !== -1) && dismissLoadingBar(WASHER_COMMANDS.START_COMMAND);                    
                    /*CHECK IF NEED TO SHOW ADD DISH ALARM*/
                    ($scope.Device.AlarmIndicator.isAddDishAlarmShow) && showAlarmInfo(ALERT_STATE.ADD_DISH_ALARM);
                    /*CHECK MODE SELECTION/RACK SELECTION / DELAY START POPUP, IF ANY OF THESE ARE OPEN THEN CLOSE WHEN "STATE" BECOME "RUN"*/
                    if ($scope.modePopup || $scope.rackSelectionPopup || $scope.delayStartPopup) {
                        closeAllPopups();
                    }
                    
                    /*On RUN state SET selected course index as recieved from the DEVICE earlier/discard user selection*/
                    var indexes = $.map($scope.Device.CourseData, function(obj, index){
                        if (obj.courseNameHex === $scope.Device.CurrentCourseHexString){
                            return index;
                        }
                    });
                    
                    $scope.selectedModeIndex = (indexes instanceof Array && indexes.length === 1)?indexes[0]:$scope.selectedModeIndex;
                    /*SET course options based on the cycle*/
                    setCourseOptions();
                    resetDelayStartTime(); //reset delay start time if state changed to run [if delay is not set on the device then only delay start will reset]
                    
                case 'Pause'://FALLTHROUGH
                    $scope.progressStart = true;//Roundprogress variable
                    $scope.progressHead = true;//Roundprogress variable
                    $scope.disableContents = true;
                    updateEstimatedTime(); //update finish time to be displayed
                    ($scope.checkResponse.indexOf(WASHER_COMMANDS.PAUSE_COMMAND) !== -1) && dismissLoadingBar(WASHER_COMMANDS.PAUSE_COMMAND);
                    break;
                default:
                    //Nothing here 
                    break;                    
            }
            
        }
        if (angular.isDefined(operation.kidsLock)) {
            if (operation.kidsLock.indexOf("Ready") === -1) {
                $scope.kidsLock = false;
            } else {
                $scope.kidsLock = true;
            }
        }
        debugMessage("parseOperation>>>>>>>>> " + JSON.stringify($scope.Device.Operation));
    }

    function parseMode(mode){
        debugMessage("parseMode>>>>>>>>> " + JSON.stringify(mode));
        if (angular.isDefined(mode.supportedOptions)) {
            $scope.Device.Mode.supportedOptions = mode.supportedOptions;
            CourseParseService.getCourseParsed(mode.supportedOptions[0], $scope);
        }
        if (angular.isDefined(mode.options)) {
            $scope.Device.Mode.options = mode.options;
            angular.forEach( $scope.Device.Mode.options, function(value, key){
                if(value.indexOf("AutoDoorRelease") !== -1){
                    $scope.Device.SettingOptions.isAutoReleaseOn = value.indexOf("AutoDoorRelease_On") !== -1 ? true : false;
                    if ($scope.checkResponse.indexOf(WASHER_COMMANDS.SETTINGS_COMMAND) !== -1){
                        dismissLoadingBar(WASHER_COMMANDS.SETTINGS_COMMAND);
                    }
                }
                if(value.indexOf("Sound") !== -1){
                    $scope.Device.SettingOptions.isSoundOn = value.indexOf("Sound_On") !== -1 ? true : false;
                    if ($scope.checkResponse.indexOf(WASHER_COMMANDS.SETTINGS_COMMAND) !== -1){
                        dismissLoadingBar(WASHER_COMMANDS.SETTINGS_COMMAND);
                    }
                }
                if(value.indexOf("Course") !== -1){
                    $scope.Device.CurrentCourseHexString = value.split('_')[1];
                    var indexes = $.map($scope.Device.CourseData, function(obj, index){
                        if (obj.courseNameHex === $scope.Device.CurrentCourseHexString){
                            return index;
                        }
                    });
                    
                    $scope.selectedModeIndex = (indexes instanceof Array && indexes.length === 1)?indexes[0]:$scope.selectedModeIndex;
                    resetDelayStartTime(); //reset delay start time if course changed
                    setCourseOptions();
                    
                    if ( $scope.Device.CurrentCourseHexString ==='89'){//Draining mode
                        if ($scope.checkResponse.indexOf(WASHER_COMMANDS.SETTINGS_COMMAND) !== -1){
                            dismissLoadingBar(WASHER_COMMANDS.SETTINGS_COMMAND);
                        }
                        $scope.stateDraining = true;
                    }else{
                        $scope.stateDraining = false;
                    }                    
                }
                
                if (value.indexOf("UsagesDB_ok") !== -1) {
                    // get DB request
                    if(isCallFromReset === 1 && $scope.currentPage === WASHER_SCREEN.ENERGY_MONITOR){
                        isCallFromReset = 2;
                    }
                    isDBDownloadedJS = false;
                    fetchDBfromDevice();                
                }
                
                if (value.indexOf("DeviceType_") !== -1) { //parse DEVICE MODEL 
                    var tempValueSplit = value.split('_');
                    var DeviceModel = (tempValueSplit instanceof Array && tempValueSplit.length > 1)?tempValueSplit[1]:"";
                    switch(DeviceModel){
                        case DWCONST.DEVICETYPE_EUROPE:
                            $scope.zoneBoosterLabel = $scope.translation.WEBMOB_device_dishwasher_option_zonebooster_eu;
                            $scope.autoReleaseLabel = $scope.translation.WEBMOB_device_dishwasher_auto_door_open;
                            break;                        
                    }
                }
                
                if (value.indexOf("WashingTimes_") !== -1) {
                    var time = value.split("_")[1];
                    $scope.washingTimes = time;                    
                }
                
                if (value.indexOf("DrumCleanProposal_") !== -1) {
                    var time = value.split("_")[1];
                    $scope.drumCleanProposal = time;  
                }
                
                if (value.indexOf("DrumCleanLog_") !== -1) {
                    var log = value.split("g_")[1];
                     if(log !== "Empty"){
                        log=log.split("|");
                        log=log.slice().reverse();
                        for (var i = 0, size = log.length; i < size; i++) {                           
                            var splitDate = log[i].split("_")[0];
                            if (splitDate.toUpperCase().indexOf('Z') === -1){//Add Z to UTC date string if missing
                                splitDate +='Z';
                            }
                            log[i] = new Date(splitDate); 
                        }
                    }
                    
                    $scope.drumCleanLog = log;                    
                }
            });
        }        
    }

    function parseCycleOptions(options){
        if(angular.isDefined(options.supportedSanitize)){   //     //options.supportedDryLevel
            $scope.Device.Washer.supportedSanitize = options.supportedSanitize;
        }
        if(angular.isDefined(options.supportedZoneBooster)){    // supportedZoneBooster //options.supportedRinseCycles
            $scope.Device.Washer.supportedZoneBooster = options.supportedZoneBooster;
        }
        if(angular.isDefined(options.supportedHighTemperatureDry)){      // supportedHighTemperatureDry  //options.supportedSoilLevel
            $scope.Device.Washer.supportedHiTempWash = options.supportedHighTemperatureDry;
        }
        if(angular.isDefined(options.supportedSpeedBooster)){      // supportedSpeedBooster    //options.supportedSpinLevel
            $scope.Device.Washer.supportedSpeedBooster = options.supportedSpeedBooster;
        }
        if(angular.isDefined(options.supportedSelectedZone)){   // supportedSelectedZone    //options.supportedWaterTemperature
            $scope.Device.Washer.supportedUpperLower = options.supportedSelectedZone;
        }
        if(angular.isDefined(options.selectedZone)){//UpperLowerSet : ON_ON      //options.waterTemperature
            $scope.Device.Washer.upperLower = options.selectedZone;
            angular.forEach( options.selectedZone.split("_"), function (value, key) {
                if(key === 0){   // upper
                    if(value === "ON"){
                        $scope.Device.CurrentCycleOptions.isUpperRackSelected = true;
                    }
                    else if(value === "OFF"){
                        $scope.Device.CurrentCycleOptions.isUpperRackSelected = false;
                    }
                }else if(key === 1){ // lower
                    if(value === "ON"){
                        $scope.Device.CurrentCycleOptions.isLowerRackSelected = true;
                    }
                    else if(value === "OFF"){
                        $scope.Device.CurrentCycleOptions.isLowerRackSelected = false;
                    }
                }
            });
        }
        if(angular.isDefined(options.zoneBooster)){//ZoneBooster : OFF_ON (OFF) //options.rinseCycles
            $scope.Device.Washer.zoneBooster = options.zoneBooster;
            var zoneBoosterValue = options.zoneBooster.split("_")[0];
            if(zoneBoosterValue === "ON"){
                $scope.Device.CurrentCycleOptions.isZoneBoosterOn = true;
            }
            else if (zoneBoosterValue === "OFF") {
                $scope.Device.CurrentCycleOptions.isZoneBoosterOn = false;
            }
        }
        if(angular.isDefined(options.sanitize)){//Sanitize  //options.dryLevel
            $scope.Device.Washer.sanitize = options.sanitize;
            if(options.sanitize === "On"){
                $scope.Device.CurrentCycleOptions.isSanitizeOn = true;
            }
            else if(options.sanitize === "Off"){
                $scope.Device.CurrentCycleOptions.isSanitizeOn = false;
            }
        }
        if(angular.isDefined(options.highTemperatureDry)){//HiTempWash  //options.soilLevel
            $scope.Device.Washer.hiTempWash = options.highTemperatureDry;
            if(options.highTemperatureDry === "On"){
                $scope.Device.CurrentCycleOptions.isHiTempWashOn = true;
            }
            else if(options.highTemperatureDry === "Off") {
                $scope.Device.CurrentCycleOptions.isHiTempWashOn = false;
            }
        }
        if(angular.isDefined(options.speedBooster)){//SpeedBooster     //options.spinLevel
            $scope.Device.Washer.speedBooster = options.speedBooster;
            if(options.speedBooster === "On"){
                $scope.Device.CurrentCycleOptions.isSpeedBoosterOn = true;
            }
            else if(options.speedBooster === "Off"){
                $scope.Device.CurrentCycleOptions.isSpeedBoosterOn = false;
            }
        }
        
        setCourseOptions(); //GUI update based on the selected mode and its supported options and their respective values
    }

    function parseAlarms(device){
        var alarms = device.Alarms;
        angular.forEach(alarms, function(value, key){
            if(angular.isDefined(value.code)){
                if(value.code.indexOf("DishA_") !== -1){// DishA_Enable/Disable
                    var addDishStatus = value.code.split("_")[1];
                    if( addDishStatus === "Enable"){
                        $scope.Device.AlarmIndicator.isAddDishAlarmShow = true;
                        showAlarmInfo(ALERT_STATE.ADD_DISH_ALARM);
                    } else if( addDishStatus === "Disable"){
                        $scope.Device.AlarmIndicator.isAddDishAlarmShow = false;
                        hideAlarmInfo(ALERT_STATE.ADD_DISH_ALARM);
                    }
                }
                if(value.code.indexOf("SaltA_") !== -1) {// SaltA_Empty/Normal
                    var saltStatus = value.code.split("_")[1];
                    if(saltStatus === "Empty"){
                        $scope.Device.AlarmIndicator.isSaltIconShow = true;
                        $scope.displayAidAlarm = true;
                        $scope.inAppAidAlarms.push({'icon':'DW/assets/img/dw_noti_ic_salt.svg#Layer_1', 'description':$scope.translation.WEBMOB_device_dishwasher_salt_need});
                    }
                    else if( saltStatus === "Normal"){
                        $scope.Device.AlarmIndicator.isSaltIconShow = false;
                    }
                }
                if(value.code.indexOf("RinseA_") !== -1){// RinseA_Empty/Normal
                    var rinseStatus = value.code.split("_")[1];
                    if(rinseStatus === "Empty"){
                        $scope.Device.AlarmIndicator.isRinseAidAlarmShow = true;
                        $scope.displayAidAlarm = true;
                        $scope.inAppAidAlarms.push({'icon':'DW/assets/img/dw_noti_ic_rinse.svg#Layer_1', 'description':$scope.translation.WEBMOB_device_dishwasher_rinseaid_need});                                                
                    } else if(rinseStatus === "Normal"){
                        $scope.Device.AlarmIndicator.isRinseAidAlarmShow = false;
                    }
                }
                // DrumClean_Need/Needless
                if(value.code.indexOf("DrumClean_") !== -1){
                    var drumCleanStatus = value.code.split("_")[1];
                    if(drumCleanStatus === "Need"){
                        $scope.Device.AlarmIndicator.isSelfCleanAlarmShow = true; 
                        showAlarmInfo(ALERT_STATE.SELF_CLEAN_ALARM);
                    } else if(drumCleanStatus === "Needless"){
                        $scope.Device.AlarmIndicator.isSelfCleanAlarmShow = false;
                        hideAlarmInfo(ALERT_STATE.SELF_CLEAN_ALARM);
                    }
                }
                // ErrorChecking_On/Off
                // Vane_Blocked/Unblocked
                // SNSF_Reached/Unreached

                // DoorA_Opened/Closed
                if(value.code.indexOf("DoorA_") !== -1){
                    var drumCleanStatus = value.code.split("_")[1];
                    if(drumCleanStatus === "Opened"){
                        $scope.Device.AlarmIndicator.isDoorOpenAlarmShow = true; 
                    } else if(drumCleanStatus === "Closed"){
                        $scope.Device.AlarmIndicator.isDoorOpenAlarmShow = false;
                        if ($scope.dialogPopup && ($scope.alertState === ALERT_STATE.DOOR_OPEN)){ 
                            closeAllPopups();
                            $scope.alertState = -1;
                        }
                    }
                }    

                // ErrorCode_OC/LC/4C/5C/HC/3C/PC
                if(value.code.indexOf("ErrorCode_") !== -1){
                    $scope.errorCode = value.code.split("_")[1];
                    debugMessage("Error notification,  $scope.errorCode = "+ value.code +" current page is >>"+ $scope.currentPage+" Alarm event is  >>> " +alarms.event);
                    var isValidErrorCode = true;
                    if(angular.isDefined(errorCodeMap[$scope.errorCode]) === false ){
                        isValidErrorCode = false;
                    }
                    if(isValidErrorCode){
                        // show error screen
                        if ($scope.currentPage !== WASHER_SCREEN.ERROR_PAGE){
                            previousPage = $scope.currentPage;
                        }
                        $scope.currentPage = WASHER_SCREEN.ERROR_PAGE;
                        changeScreen();
                    }
                }else if (value.code.indexOf("ErrorCode") !== -1 && angular.isDefined(device.event) && device.event === "Deleted"){//dismiss error
                    debugMessage("Error dismissed, previous page is >>"+ previousPage+" >>> " +alarms.event);
                    if (previousPage !== -1){
                        //var makeConfigCall = false;
                        if (previousPage === WASHER_SCREEN.SPLASH_PAGE){//in case error screen came during the app launch, run configuration command 
                            previousPage = WASHER_SCREEN.HOME_PAGE;
                            //makeConfigCall = true;
                        }
                        $scope.currentPage = previousPage;
                        previousPage = -1;
                        changeScreen();
                        //makeConfigCall && configurationCall();
                    }
                }
            }
        });
    }
        
    function parseConfiguration(configura) {
        debugMessage("Configuration.............");
        if (angular.isDefined(configura["remoteControlEnabled"])) {
            dismissLoadingBar(WASHER_COMMANDS.PARSE_CONFIGURATION);            
           
            $scope.remoteControlEnabled = configura["remoteControlEnabled"];
            $scope.smartControlOnOffText = $scope.remoteControlEnabled ? $scope.translation.WEBMOB_common_on_capital : $scope.translation.WEBMOB_common_off_capital;
            if ($scope.remoteControlEnabled && $scope.dialogPopup && ($scope.alertState === ALERT_STATE.SMART_CONTROL_DISABLED)){ 
                closeAllPopups();
                $scope.alertState = -1;
            }
           
            if (!$scope.remoteControlEnabled){//Show popup if smart control is off
                showDialog(ALERT_STATE.SMART_CONTROL_DISABLED, $scope.translation.WEBMOB_common_smart_control_off, $scope.translation.WEBMOB_common_smart_control_off_desc2, '', $scope.translation.WEBMOB_common_ok);
            }
            debugMessage("$scope.remoteControlEnabled" + $scope.remoteControlEnabled);
        }
        if (angular.isDefined(configura["region"])) {
            $scope.region = configura["region"];
        }
    }
    
    function parseModuleInformation (moduleInfo) {
        if (angular.isDefined(moduleInfo.Versions) && moduleInfo.Versions.length > 0) {
            var moduleInfoText = "Software Ver : " + moduleInfo.Versions[0].number + " <br> Firmware Ver : " + moduleInfo.Versions[1].number;
            dismissLoadingBar(WASHER_COMMANDS.MODULE_INFO);
            showDialog(ALERT_STATE.VERSION_INFO, $scope.translation.WEBMOB_common_version_info, moduleInfoText, '', $scope.translation.WEBMOB_common_ok);
        }
    }
    
    function cancelCycleConfirm(isCancelDrain){
        debugMessage('Cancel cycle confirm isCancelDrain >>> ' + isCancelDrain);
        var msgText='', alertState;
        if (isCancelDrain){ //Drain and cancel
            msgText = ($scope.Device.Operation.progress === 'Prewash' || $scope.Device.Operation.progress === 'Delaywash')?$scope.translation.WEBMOB_device_dishwasher_cancel_drain_desc_prewash:$scope.translation.WEBMOB_device_dishwasher_cancel_drain_desc_mainwash;
            alertState = ALERT_STATE.CANCEL_AND_DRAIN;
        }else {//cancel
            msgText = ($scope.Device.Operation.progress === 'Prewash' || $scope.Device.Operation.progress === 'Delaywash')?$scope.translation.WEBMOB_device_dishwasher_cancel_desc:$scope.translation.WEBMOB_device_dishwasher_cancel_desc_mainwash;
            alertState = ALERT_STATE.CANCEL_CYCLE;
        }
        
        showDialog(alertState, '', msgText, $scope.translation.WEBMOB_device_dishwasher_comm_no_capital, $scope.translation.WEBMOB_common_cancel_capital);        
    }
    
    function cancelWasherOperation(isCancelDrain){
        if (isCancelDrain){//send drain command
            (!$scope.isStatic) && settingsAction('drain');            
        }else{//cancel current cycle
           (!$scope.isStatic) && cancelProgress();
        }
        
        if ($scope.isStatic){ //For try it now
            debugMessage("Cancel Progress: For static mode");
            $scope.Device.Operation.state = "Ready";
            $scope.progressStart = false;//Roundprogress variable
            $scope.progressHead = false;//Roundprogress variable
            $scope.current = 0;  
            $scope.disableContents = false; 
            /*CHECK IF NEED TO SHOW SELF CLEAN ALARM*/
            ($scope.Device.AlarmIndicator.isSelfCleanAlarmShow) && showAlarmInfo(ALERT_STATE.SELF_CLEAN_ALARM);
        } 
    }
        
    function cancelProgress(){
        if (!$scope.isStatic) {
            $scope.checkResponse.push(WASHER_COMMANDS.CANCEL_COMMAND);
            SHPService.sendSHPCommand(DWCONST.PUT, cancelCommand(), "/" + $scope.peerId + "/devices/0/operation");
            debugMessage("sending cancel command to washer");
        }
    }
    
    function startProgress(){
        if ($scope.Device.AlarmIndicator.isDoorOpenAlarmShow){
            showDialog(ALERT_STATE.DOOR_OPEN, '', $scope.translation.WEBMOB_device_dishwasher_door_open_popup_desc, '', $scope.translation.WEBMOB_common_ok);
            return;
        }
        
        if (!$scope.isStatic) {
            if (($scope.Device.Operation.state === "Run")) {
                $scope.checkResponse.push(WASHER_COMMANDS.PAUSE_COMMAND);
                SHPService.sendSHPCommand(DWCONST.PUT, pauseCommand(), "/" + $scope.peerId + "/devices/0/operation");
            } else if (($scope.Device.Operation.state === "Ready") || ($scope.Device.Operation.state === "Pause")) {
                $scope.checkResponse.push(WASHER_COMMANDS.START_COMMAND);
                SHPService.sendSHPCommand(DWCONST.PUT, startCommand(), "/" + $scope.peerId + "/devices/0");
            } else {
                ;//Nothing
            }
            debugMessage("Start Progress: sending the command to washer");
        }else{
            debugMessage("Start Progress: Demo mode");
            if (($scope.Device.Operation.state === "Ready") || ($scope.Device.Operation.state === "Pause")) {
                $scope.Device.Operation.state = "Run";
                $scope.Device.Operation.progress = "Wash";
                $scope.progressStart = true;//Roundprogress variable
                $scope.progressHead = true;//Roundprogress variable
                $scope.current = 40;  
                updateEstimatedTime(); //update finish time to be displayed
                $scope.disableContents = true;
                
                /*CHECK IF NEED TO SHOW SELF CLEAN ALARM*/
                ($scope.Device.AlarmIndicator.isAddDishAlarmShow) && showAlarmInfo(ALERT_STATE.ADD_DISH_ALARM);
            } else {
               $scope.Device.Operation.state = "Pause";
            }
        }      
    }
    
    function startFavorite(itemIndex){
        if ($scope.Device.AlarmIndicator.isDoorOpenAlarmShow){
            showDialog(ALERT_STATE.DOOR_OPEN, '', $scope.translation.WEBMOB_device_dishwasher_door_open_popup_desc, '', $scope.translation.WEBMOB_common_ok);
            return;
        }
        
        if (!$scope.isStatic) {
            if (($scope.Device.Operation.state === "Ready")) {
                $scope.checkResponse.push(WASHER_COMMANDS.START_COMMAND);
                SHPService.sendSHPCommand(DWCONST.PUT, startFavoriteCommand(itemIndex), "/" + $scope.peerId + "/devices/0");
            } else {
                ;//Nothing
            }
            debugMessage("Start Progress: sending the command to washer from my favorite");
        }else{
            debugMessage("Start Progress: Demo mode");
            $scope.Device.Operation.state = "Run";
            $scope.Device.Operation.progress = "Wash";
            $scope.progressStart = true;//Roundprogress variable
            $scope.progressHead = true;//Roundprogress variable
            $scope.current = 40;  
            updateEstimatedTime(); //update finish time to be displayed
            $scope.disableContents = true;

            /*CHECK IF NEED TO SHOW SELF CLEAN ALARM*/
            ($scope.Device.AlarmIndicator.isAddDishAlarmShow) && showAlarmInfo(ALERT_STATE.ADD_DISH_ALARM);            
        }        
        onBackPressed(); //go back to home page
    }
    
    
    
    function setCourseOptions(){
        //set otions enable/disable and their respective value based on selected course
        if (angular.isDefined($scope.Device.CourseData[$scope.selectedModeIndex])){
            if ($scope.Device.CourseData[$scope.selectedModeIndex].courseNameHex === $scope.Device.CurrentCourseHexString ){//if selected course is same as currently selected on SET
                $scope.isZoneBooster = $scope.Device.CurrentCycleOptions.isZoneBoosterOn;
                $scope.isSpeedBooster = $scope.Device.CurrentCycleOptions.isSpeedBoosterOn;
                $scope.isHighTempWash = $scope.Device.CurrentCycleOptions.isHiTempWashOn;
                $scope.isSanitize = $scope.Device.CurrentCycleOptions.isSanitizeOn;
                $scope.isUpperRackSelected = $scope.Device.CurrentCycleOptions.isUpperRackSelected;
                $scope.isLowerRackSelected = $scope.Device.CurrentCycleOptions.isLowerRackSelected;            
            }else{
                $scope.isZoneBooster = ($scope.Device.CourseData[$scope.selectedModeIndex].defaultZoneBooster === 'OFF_OFF')?false:true;
                $scope.isSpeedBooster = ($scope.Device.CourseData[$scope.selectedModeIndex].defaultSpeedBooster === 'Off')?false:true;
                $scope.isHighTempWash = ($scope.Device.CourseData[$scope.selectedModeIndex].defaultHiTempWash === 'Off')?false:true;
                $scope.isSanitize = ($scope.Device.CourseData[$scope.selectedModeIndex].defaultSanitize === 'Off')?false:true;
                var rackSelection = $scope.Device.CourseData[$scope.selectedModeIndex].defaultRackSelection.split('_');
                $scope.isUpperRackSelected = (rackSelection[0] === 'ON')?true:false;
                $scope.isLowerRackSelected = (rackSelection[1] === 'ON')?true:false;
            }
            
            if ($scope.isUpperRackSelected && $scope.isLowerRackSelected){
                $scope.guideMessageRack = $scope.translation.WEBMOB_device_dishwasher_all_rack_selected;
            }else if( $scope.isLowerRackSelected){
                $scope.guideMessageRack = $scope.translation.WEBMOB_device_dishwasher_lower_rack_selected;
            }
        }
    }
   
   function setFavoriteCourseOptions(){
        //set otions enable/disable and their respective value based on selected course
        if (angular.isDefined($scope.Device.CourseData[$scope.favoriteObject.modeIndex])){
            if ($scope.Device.CourseData[$scope.favoriteObject.modeIndex].courseNameHex !== $scope.favoriteObject.CurrentCourseHexString ){//if selected course is not same as saved in favorite [EDIT CASE]                
                $scope.favoriteObject.isZoneBooster = ($scope.Device.CourseData[$scope.favoriteObject.modeIndex].defaultZoneBooster === 'OFF_OFF')?false:true;
                $scope.favoriteObject.isSpeedBooster = ($scope.Device.CourseData[$scope.favoriteObject.modeIndex].defaultSpeedBooster === 'Off')?false:true;
                $scope.favoriteObject.isHighTempWash = ($scope.Device.CourseData[$scope.favoriteObject.modeIndex].defaultHiTempWash === 'Off')?false:true;
                $scope.favoriteObject.isSanitize = ($scope.Device.CourseData[$scope.favoriteObject.modeIndex].defaultSanitize === 'Off')?false:true;
                var rackSelection = $scope.Device.CourseData[$scope.favoriteObject.modeIndex].defaultRackSelection.split('_');
                $scope.favoriteObject.isUpperRackSelected = (rackSelection[0] === 'ON')?true:false;
                $scope.favoriteObject.isLowerRackSelected = (rackSelection[1] === 'ON')?true:false;
            }
            
            //ADD ANOTHER CASE HERE - check if same mode index is selected as present in favorite object
        }
    }
   
    function closeAllPopups() {
        debugMessage("close popups called")
        if ($scope.rackSelectionPopup) {
            $scope.rackSelectionPopup = false;
        }
        if ($scope.rackSelectionFavorite) {
            $scope.rackSelectionFavorite = false;
        }
        
        if ($scope.modePopup) {
            $scope.modePopup = false;
        }
        
        if ($scope.optionPopup){
            $scope.optionPopup = false;
        }
        
        if ($scope.delayStartPopup){
            $scope.delayStartPopup = false;
        }
        
        if ($scope.smartControlPopup){
            $scope.smartControlPopup = false;
        }
        
        if ($scope.dialogPopup){
            $scope.dialogPopup = false;
            $scope.isRadioSelection = false;
            $scope.showServiceCenterText = false;           
        }
        
        toggleBottomPopUpClass(false);
        changePopUpClass(false);
    }

    function getProgressColor() {
        return $scope.gradient ? 'url(#gradient)' : $scope.currentColor;
    }
    
    function updateDeviceData(data) {
        SHPService.updateDeviceData($scope, data);
    }
    
     function updateEstimatedTime() {
         debugMessage("Update estimation time called");
        var currentTime = new Date(); // current date
        var crhours = currentTime.getHours();
        var crminutes = currentTime.getMinutes();
        if ($scope.remainingTime !== undefined && $scope.remainingTime !=='') {
            debugMessage("updateEstimatedTime ", crhours, crminutes, $scope.remHrs, $scope.remMin, $scope.delHrs, $scope.delMin);
            var estHrs = parseInt($scope.remHrs,10) + crhours;
            var estMin = parseInt($scope.remMin,10) + crminutes;
            
            if ($scope.Device.Operation.progress === 'Delaywash' && $scope.delHrs !== '' && $scope.delMin !== ''){//consider delay time also for estimation
                debugMessage("Adding delay time to estimated time " + $scope.delHrs + " hrs " + $scope.delMin);
                estHrs += parseInt($scope.delHrs,10);
                estMin += parseInt($scope.delMin,10);
            }
            
            debugMessage("Time is:: " + crminutes + ":::" + $scope.remainingTime.substring(4, 5));
            var newdate = new Date();
            newdate.setHours(estHrs, estMin, 0);
            var hours = newdate.getHours();
            var minutes = newdate.getMinutes();
            $scope.ampm = hours >= 12 ? ' PM' : ' AM';
            hours = hours % 12;
            hours = hours ? (hours < 10 ? '0' + hours : hours) : 12;
            minutes = minutes < 10 ? '0' + minutes : minutes;
            $scope.estimationTime = " " + hours + ":" + minutes;
        }
    }
    
    function dismissLoadingBar(value) { 
        if ($scope.checkResponse.indexOf(value) !== -1) {
            $scope.checkResponse.splice($scope.checkResponse.indexOf(value), 1);
            if ($scope.checkResponse.length === 0) {
                $scope.loadingBar = false;
            }
        }
    }

    function showDialog (state, title, message,cancelBtn, okBtn) {
        debugMessage("showDialog", title);
        closeAllPopups();
        $scope.alertState = state;        
        $scope.popupTitle = title;
        $scope.popupMessage = message;
        $scope.popupOkBtn = okBtn;
        $scope.popupCancelBtn = cancelBtn;
        $scope.dialogPopup = true;
        toggleBottomPopUpClass(true);
    }
    
    function popupBtnClicked (state) {
        debugMessage("popupBtnClick >>> State >>> " +state);
        if (state === 1){//OK Click
            switch($scope.alertState){
                case ALERT_STATE.CONNECTION_FAILURE:
                    closeConnectionFailurePopup();
                    if($scope.stateFinish === false)
                        closeApp();
                    break;
                case ALERT_STATE.CANCEL_CYCLE:
                    cancelWasherOperation(false);    
                    break;
                 case ALERT_STATE.CANCEL_AND_DRAIN:
                    cancelWasherOperation(true);    
                    break;    
                case ALERT_STATE.VERSION_INFO://FALLTHROUGH    
                case ALERT_STATE.SMART_CONTROL_DISABLED:
                    case ALERT_STATE.DOOR_OPEN://FALLTHROUGH
                    //Nothing to do here
                    break;  
                case ALERT_STATE.COUNTRY_SELECTION:
                    if($scope.radioBtnSelectedIndex === 0){
                        $timeout(function () {
                            nativeInterface.runOnNative("customerService", aboutCommand(reqParamCountrySelection));
                        }, MATERIAL_DESIGN.LIST_ANIMATION_TIME);
                    } else {
                        $timeout(function () {
                            nativeInterface.runOnNative("customerService", aboutCommand(reqParamCountrySelection, true));
                        }, MATERIAL_DESIGN.LIST_ANIMATION_TIME);
                    }                
                    break;
                case ALERT_STATE.SERVICE_CENTER_SELECTION:
                        serviceCallCommand($scope.radioBtnToSelect[$scope.radioBtnSelectedIndex]);
                    break;
                 case ALERT_STATE.SERVICE_CALL:
                     serviceCallCommand($scope.radioBtnToSelect[0]);
                    break;
                case ALERT_STATE.DUPLICATE_NAME:
                     //DO NOTHING
                    break;
                case ALERT_STATE.DELETE_FAVORITE_ITEM:
                    $scope.favoriteList.splice($scope.selectedFavoriteItemIndex, 1);
                    setLocalStorageData($scope.deviceUuid + '_Favorite', JSON.stringify($scope.favoriteList));
                    toaster.success({title: "", body: $scope.translation.WEBMOB_device_dishwasher_alarm_my_favorite_toast_delete, bodyOutputType: 'trustedHtml'});
                    //openMyfavorite();
                     $scope.selectedFavoriteItemIndex = null;
                    break;
                case ALERT_STATE.DELETE_FAVORITE_ITEMS:
                    deleteItemsFromFavorite();                    
                    break;
                default:
                    break;
            }
        }else if (state===2){//Cancel Click
            switch($scope.alertState){
                case ALERT_STATE.CANCEL_AND_DRAIN://FALLTHROUGH
                case ALERT_STATE.CANCEL_CYCLE:
                    debugMessage("User canceled confirm cancel cycle popup");
                    break;
                case ALERT_STATE.COUNTRY_SELECTION:
                    debugMessage("User canceled country selection popup");
                    break;
                case ALERT_STATE.SERVICE_CALL:
                    debugMessage("User canceled service call popup");
                    break; 
                case ALERT_STATE.DELETE_FAVORITE_ITEM:
                    $scope.selectedFavoriteItemIndex = null;
                    debugMessage("User canceled delete favorite popup");
                    break; 
                case ALERT_STATE.DELETE_FAVORITE_ITEMS:
                    debugMessage("User canceled delete favorite popup [multi select delete]");
                    cancelMultiSelect();
                    break;    
                default:
                    break;
            }
        }else{
            //nothing
        }
        
        $scope.dialogPopup = false;
        $scope.alertState = -1;
        $scope.isRadioSelection = false;
        $scope.showServiceCenterText = false;
        toggleBottomPopUpClass(false);
    }

    function showConnectionFailurePopup () {
        debugMessage("showConnectionFailurePopup");
        //forcefully removing loading bar in case of connection failure
        $scope.loadingBar = false;
        $scope.checkResponse = [];
        document.activeElement.blur();
        showDialog(ALERT_STATE.CONNECTION_FAILURE, $scope.translation.WEBMOB_device_dishwasher_alarm_connection_end_title, $scope.translation.WEBMOB_common_check_power_and_network, '', $scope.translation.WEBMOB_common_ok);
    }

    function closeConnectionFailurePopup() {
        $scope.dialogPopup = false;
        toggleBottomPopUpClass(false);
    }

    function handleStatusCode(status, response) {  
        debugMessage("handleStatusCode status and respose is::" + status + JSON.stringify(response));
        // do nothing
    }

    function requestFailHandler(response) {
        debugMessage("Fail Handler respose is::" + response);
        $scope.loadingBar = false;
        $scope.checkResponse = [];
        if (angular.isDefined(response["status"]) && response["status"] === DWCONST.STATUS_400){
            showConnectionFailurePopup();
        }
    }
    
    function onBackPressed() {
        if ($scope.modePopup || $scope.rackSelectionPopup || $scope.optionPopup || $scope.delayStartPopup || $scope.smartControlPopup || $scope.dialogPopup || $scope.rackSelectionFavorite) {
            if ($scope.dialogPopup && $scope.alertState === ALERT_STATE.CONNECTION_FAILURE && $scope.stateFinish === false){
               closeConnectionFailurePopup();
               closeApp();
            }
            closeAllPopups();
            return;
        }
        
        if ($scope.diagnosisState === 'Run'){
            return;
        }
        
        if ($scope.showHelpPage === true ){
            closeUsageHelpPage();
            return;
        }
        
        if ($scope.isMultiSelectMode){
            cancelMultiSelect();
            return;
        }
        
        switch($scope.currentPage){
            case WASHER_SCREEN.SPLASH_PAGE:
            case WASHER_SCREEN.HOME_PAGE:
                closeApp();
                break;
            case WASHER_SCREEN.SETTINGS_PAGE://FALLTHROUGH                  
            case WASHER_SCREEN.ABOUT_PAGE:                
                $scope.currentPage = WASHER_SCREEN.HOME_PAGE;
                changeScreen();
                break;
            case WASHER_SCREEN.NO_ERROR_PAGE://FALLTHROUGH
            case WASHER_SCREEN.SELF_CARE_PAGE:
                $scope.currentPage = WASHER_SCREEN.ABOUT_PAGE;
                changeScreen();
                break;
            case WASHER_SCREEN.ENERGY_MONITOR:
                $scope.currentPage = WASHER_SCREEN.SETTINGS_PAGE;
                changeScreen();           
                break;
            case WASHER_SCREEN.ERROR_PAGE:
                closeApp();
                break;
            case WASHER_SCREEN.FAVORITE_PAGE: 
                
                $scope.currentPage = WASHER_SCREEN.HOME_PAGE;
                changeScreen();
                break;
            case WASHER_SCREEN.SELF_CLEAN_HISTORY_PAGE:
                $scope.currentPage = WASHER_SCREEN.SETTINGS_PAGE;
                changeScreen(); 
                break;
            case WASHER_SCREEN.ADD_FAVORITE_PAGE:
                closeAllPopups(); //close any open popup                 
                $scope.currentPage = WASHER_SCREEN.FAVORITE_PAGE;
                changeScreen(); 
                break;
            case WASHER_SCREEN.CYCLE_PAGE:
                $scope.backFromCyclePage = true;
                $scope.currentPage = WASHER_SCREEN.ADD_FAVORITE_PAGE;
                changeScreen();
                break; 
            default:
                //Do nothing
                break;
        }        
    }
    
    function changeScreen(){
        debugMessage("change screen to " + $scope.currentPage);
        //reset values
        $scope.homePage = false;
        $scope.addFavoritePage = false;
        $scope.favoritePage = false;
        $scope.smartCare = false;
        $scope.smartCareMessage = false;
        $scope.energyMonitorPage = false;
        closeAllPopups();
        switch($scope.currentPage){
            case WASHER_SCREEN.HOME_PAGE:
                $state.transitionTo('HomePage', {}, {reload:true});
                break;
            case WASHER_SCREEN.SETTINGS_PAGE://
                $state.transitionTo('SettingsPage');
                break;
            case WASHER_SCREEN.ABOUT_PAGE:   
                $state.transitionTo('AboutPage');
                break;
            case WASHER_SCREEN.NO_ERROR_PAGE://FALLTHROUGH
            case WASHER_SCREEN.SELF_CARE_PAGE:
                $state.transitionTo('SelfCarePage');
                break;
            case WASHER_SCREEN.ENERGY_MONITOR:
                $scope.energyMonitorPage = true;
                $state.transitionTo('EnergyMonitorPage');
                break;
            case WASHER_SCREEN.ERROR_PAGE:
                $state.transitionTo('ErrorPage', {}, {reload:true});
                var errorDetails = (angular.isDefined(errorCodeMap[$scope.errorCode]))?errorCodeMap[$scope.errorCode]:'';
                $scope.errorTitle = errorDetails.title || "Unknown Error";
                $scope.errorCode = errorDetails.code || "";
                $scope.errorDescription = errorDetails.description || "";
                break;
            case WASHER_SCREEN.FAVORITE_PAGE: 
                $scope.favoritePage = true;
                $state.transitionTo('FavoritePage');
                break;
            case WASHER_SCREEN.ADD_FAVORITE_PAGE:    
                $scope.addFavoritePage = true;    
                $state.transitionTo('AddFavoritePage');
                break;
            case WASHER_SCREEN.CYCLE_PAGE:
                $scope.favoriteWashName = $('#inputMFName').val(); //set name with the input field value
                $state.transitionTo('CyclePage');
                break;  
            case WASHER_SCREEN.SELF_CLEAN_HISTORY_PAGE:
                $state.transitionTo('SelfCleanHistoryPage');
                break;    
            default:
                //Do nothing
                break;
        }        
    }

    function closeApp(){        
        nativeInterface.runOnNative("closeWebApp", "");
    }

    function openPopup(popupName){
        debugMessage("openPopup >>>>>> name >>>>> " + popupName);
        switch(popupName){
            case "mode"://mode selection
                openModeSelectionPopUp();
                break;
            case "rack"://rack selection
                openRackSelectionPopUp();
                break;
            case "delay"://delay start selection
                $scope.delayStartPopup = true;
                $scope.delHrsTemp = $scope.delayStartValue;
                toggleBottomPopUpClass(true);
                changePopUpClass(true);
                break;
            case "smartControl"://smart control selection
                $scope.smartControlPopup = true;
                toggleBottomPopUpClass(true);
                changePopUpClass(true);
                break;
                break;
            case "options"://contextual menu selection
                showOptionPopup();
                break;
            case "serviceCenter"://Display service center numbers
                if ($scope.serviceCenterNumber !== undefined) {
                    $scope.radioBtnToSelect = [];
                    $scope.radioBtnSelectedIndex = 0;
                    if ($scope.serviceCenterNumber.indexOf(',') !== -1) {
                        $scope.isRadioSelection = true;
                        $scope.showServiceCenterText = true;
                        $scope.radioBtnToSelect = $scope.serviceCenterNumber.split(",");
                        var msg = $scope.translation.WEBMOB_device_dishwasher_error_service_center;
                        showDialog(ALERT_STATE.SERVICE_CENTER_SELECTION, $scope.translation.WEBMOB_common_service_center, msg, $scope.translation.WEBMOB_common_cancel_capital, $scope.translation.WEBMOB_common_ok);
                    } else {
                        $scope.radioBtnToSelect[0] = $scope.serviceCenterNumber;
                        var msg = $scope.translation.WEBMOB_device_dishwasher_error_service_center +"<br>" + $scope.serviceCenterNumber;
                        showDialog(ALERT_STATE.SERVICE_CALL, $scope.translation.WEBMOB_common_service_center, msg, $scope.translation.WEBMOB_common_cancel_capital, $scope.translation.WEBMOB_common_ok);
                    }
                }
                break;    
            default:
                //Do nothing
                break;
        }
    }
    
    function openRackSelectionPopUp() {
        switch($scope.currentPage){
            case WASHER_SCREEN.HOME_PAGE:
                $scope.rackSelectionPopup = true;
                break;
            case WASHER_SCREEN.ADD_FAVORITE_PAGE:
                $scope.rackSelectionFavorite = true;
                break;    
        }
        toggleBottomPopUpClass(true);
        changePopUpClass(true);        
    }
    
    // Toggle class for Mode drop down
    function toggleBottomPopUpClass(popupName) {
        if (popupName) {
            $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivIn";
            $scope.BottomPopupDivCls = "BottomPopupBGIn";
        } else {
            $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivOut";
            $scope.BottomPopupDivCls = "BottomPopupBGOut";
        }
    }
    
    function changePopUpClass(closemultiDialogPopUp) {
        if (closemultiDialogPopUp) {
            $scope.ModeSelectionPopupDivCls = "ModeSelectionPopupDiv";
            var scroll_middle_area = document.getElementById("scrollable_area_id");
            scroll_middle_area.style.overflowY = "hidden";
        } else {
            $scope.ModeSelectionPopupDivCls = "ModeSelectionPopOutDiv";
            var scroll_middle_area = document.getElementById("scrollable_area_id");
            scroll_middle_area.style.overflowY = "overlay";
        }
    }
    
    function onRackSelected(indexVal) {
        if ($scope.currentPage === WASHER_SCREEN.HOME_PAGE){
            switch(indexVal) {
                case 0:
                    $scope.isUpperRackSelected = true;
                    $scope.isLowerRackSelected = true;
                    $scope.guideMessageRack = $scope.translation.WEBMOB_device_dishwasher_all_rack_selected;
                    break;
                case 1:
                    $scope.isUpperRackSelected = false;
                    $scope.isLowerRackSelected = true;
                    $scope.guideMessageRack = $scope.translation.WEBMOB_device_dishwasher_lower_rack_selected;
                    break;
                default:
                    break;
            }
        } else {
            switch(indexVal) {
                case 0:
                    $scope.favoriteObject.isUpperRackSelected = true;
                    $scope.favoriteObject.isLowerRackSelected = true;
                    break;
                case 1:
                    $scope.favoriteObject.isUpperRackSelected = false;
                    $scope.favoriteObject.isLowerRackSelected = true;
                    break;
                default:
                    break;
            }
        }
        
        closeAllPopups(); // Close rack popup after selection
    }
    
    function toggleZoneBooster() {
        switch($scope.currentPage){
            case WASHER_SCREEN.HOME_PAGE:
                $scope.isZoneBooster = !$scope.isZoneBooster;
                break;
            case WASHER_SCREEN.ADD_FAVORITE_PAGE:
                $scope.favoriteObject.isZoneBooster = !$scope.favoriteObject.isZoneBooster;
                break;    
        }
        
    }
    
    function toggleSpeedBooster() {
        switch($scope.currentPage){
            case WASHER_SCREEN.HOME_PAGE:
                $scope.isSpeedBooster = !$scope.isSpeedBooster;
                break;
            case WASHER_SCREEN.ADD_FAVORITE_PAGE:
                $scope.favoriteObject.isSpeedBooster = !$scope.favoriteObject.isSpeedBooster;
                break;    
        }
        
    }
    
    function toggleHighTempWash() {
        switch($scope.currentPage){
            case WASHER_SCREEN.HOME_PAGE:
                $scope.isHighTempWash = !$scope.isHighTempWash;
                break;
            case WASHER_SCREEN.ADD_FAVORITE_PAGE:
                $scope.favoriteObject.isHighTempWash = !$scope.favoriteObject.isHighTempWash;
                break;    
        }
        
    }
    
    function toggleSanitize() {
        switch($scope.currentPage){
            case WASHER_SCREEN.HOME_PAGE:
                 $scope.isSanitize = !$scope.isSanitize;
                break;
            case WASHER_SCREEN.ADD_FAVORITE_PAGE:
                $scope.favoriteObject.isSanitize = !$scope.favoriteObject.isSanitize;
                break;    
        }
       
    }
    
    function openModeSelectionPopUp() {
        $scope.modePopup = true;
        changePopUpClass($scope.modePopup);
    }
    
    function selectIndex(selectedItem, indexval) {
        switch($scope.currentPage){
            case WASHER_SCREEN.HOME_PAGE:
                // if we change the course reset( NONE ) delay start time
                ($scope.selectedModeIndex !== indexval) && resetDelayStartTime();
                $scope.selectedModeIndex = indexval;      
                setCourseOptions();
                $scope.modePopup = false;
                changePopUpClass($scope.modePopup);
                break;
            case WASHER_SCREEN.CYCLE_PAGE:
                if ($scope.favoriteObject.modeIndex !== indexval){
                    $scope.favoriteObject.modeIndex = indexval;
                    setFavoriteCourseOptions();
                }
                
                onBackPressed();//back to add favorite page
                break;
        }
    }
    
    function closeModePopUp(){
        $scope.modePopup = false;
        changePopUpClass();        
    }
    
    function getModeSelected(selectedItem, index) {
        //console.log("getModeSelected >>> selectedItem.courseNameHex  " + selectedItem.courseNameHex + " >>>> $scope.Device.CurrentCourseHexString" + $scope.Device.CurrentCourseHexString);
        var result = false;
        switch($scope.currentPage){
            case WASHER_SCREEN.HOME_PAGE:
                result = (selectedItem.courseNameHex === $scope.Device.CurrentCourseHexString);
                break;
            case WASHER_SCREEN.CYCLE_PAGE:
                result = ($scope.favoriteObject.modeIndex === index);
                break;    
        }
        
        return result;
    }
    
    function configurationCall() {
        if (!$scope.isStatic) {
            debugMessage("configurationCall");
            //if (!!!configTimer) {
                //configTimer = $interval(function () {
                   // debugMessage("configurationCall Interval call");
                    // fixme : ios native app don't give configurationLink. need to check.
                    nativeInterface.getResourceUrl("/" + $scope.peerId + "/devices/0/configuration");
                    // nativeInterface.getResourceUrl("/" + $scope.peerId + $scope.Device.ConfigurationLink.href);
                //}, Sec20Call);
           // }
        }
    }
    
    function startCommand() {
        // : spinLevel refers to Speed Booster of DW
        // : soilLevel refers to Hi-Temp Wash
        // : dryLevel refers to Sanitize
        // : rinseCycle refers to Zone Booster
        // : waterTemperature refers to Rack selection
        // xxx : consider using supported values rather than static string.
        var courseHex = $scope.Device.CourseData[$scope.selectedModeIndex].courseNameHex,
        rackSelection = ($scope.isUpperRackSelected && $scope.isLowerRackSelected)?'ON_ON':'OFF_ON',//either all or lower
        sanitize = ($scope.isSanitize)?'On':'Off',
        hightTempWash = ($scope.isHighTempWash)?'On':'Off',
        speedBoost = ($scope.isSpeedBooster)?'On':'Off',
        zoneBoost = ($scope.isZoneBooster)?'ON_OFF':'OFF_OFF';
        
        var command = '{"Device":{"Mode":{"options":["Course_' + courseHex + '"]},"Operation":{"state":"Run"},"Washer":{"selectedZone":"' + rackSelection + '","zoneBooster":"' + zoneBoost + '","speedBooster":"' + speedBoost + '","highTemperatureDry":"' + hightTempWash + '", "sanitize":"' + sanitize + '"}}}';
        
        if ($scope.delayStartValue !== '00' && ($scope.Device.Operation.state === "Ready")){//if delay start is set then send delayStartTime along other options 
            command = '{"Device":{"Mode":{"options":["Course_' + courseHex + '"]},"Operation":{"state":"Run","delayStartTime":"'+$scope.delayStartValue+':00:00"},"Washer":{"selectedZone":"' + rackSelection + '","zoneBooster":"' + zoneBoost + '","speedBooster":"' + speedBoost + '","highTemperatureDry":"' + hightTempWash + '", "sanitize":"' + sanitize + '"}}}';
        }
        
        debugMessage(command);
        return command;
    }
    
    function startFavoriteCommand(itemIndex) {
        var courseHex = $scope.Device.CourseData[$scope.favoriteList[itemIndex].modeIndex].courseNameHex,
        rackSelection = ($scope.favoriteList[itemIndex].isUpperRackSelected && $scope.favoriteList[itemIndex].isLowerRackSelected)?'ON_ON':'OFF_ON',//either all or lower
        sanitize = ($scope.favoriteList[itemIndex].isSanitize)?'On':'Off',
        hightTempWash = ($scope.favoriteList[itemIndex].isHighTempWash)?'On':'Off',
        speedBoost = ($scope.favoriteList[itemIndex].isSpeedBooster)?'On':'Off',
        zoneBoost = ($scope.favoriteList[itemIndex].isZoneBooster)?'ON_OFF':'OFF_OFF';
        
        var command = '{"Device":{"Mode":{"options":["Course_' + courseHex + '"]},"Operation":{"state":"Run"},"Washer":{"selectedZone":"' + rackSelection + '","zoneBooster":"' + zoneBoost + '","speedBooster":"' + speedBoost + '","highTemperatureDry":"' + hightTempWash + '", "sanitize":"' + sanitize + '"}}}';
        debugMessage(command);
        return command;
    }
    
    function cancelCommand() {
        debugMessage('CANCEL>>>>>>>>>>>>>>>>{"Operation":{"state":"Pause"}}');
        return '{"Operation":{"state":"Ready"}}';
    }
    
    function pauseCommand() {
        debugMessage('PAUSE>>>>>>>>>>>>>>>>{"Operation":{"state":"Pause"}}');
        return '{"Operation":{"state":"Pause"}}';
    }
    
    function aboutCommand(cmdstring, changedCode) {
        if(changedCode) {
            return '{"service" : "' + cmdstring + '","selectedCountry" : "' + $scope.selectedCountry + '_FR"}';
        } else {
            return '{"service" : "' + cmdstring + '","selectedCountry" : "' + $scope.selectedCountry + '"}';
        }
    }
    
    function serviceCallCommand(centerNumber){
        var payload = '{"selectedCountry" : "' + $scope.selectedCountry + '","selectedDial" :"'+centerNumber+'"}';
        nativeInterface.runOnNative("callDial", payload);
    }
    
    function diagnosisCommand(){
        return '{"Diagnosis": {"diagnosisStart": "Run"}}';
    }
    
    function getResetCommand() {
        return '{"Device":{"Mode":{"options":["UsagesDB_reset"]}}}';
    }
    
    function showUsageHelpPage(){
        $scope.showHelpPage = true;
        //scroll to top while showing help page
        document.getElementById("scrollable_area_id").scrollTop = 0;
    }
    
    function closeUsageHelpPage(){
        $scope.showHelpPage = false;
    }
    
    function showOptionPopup(){
        debugMessage("showOptionPopup");
        $scope.optionPopup= true;
        changePopUpClass(true);  
    }
       
    function selectIndexFromOptionMenu(index){
        debugMessage("selectIndexFromOptionMenu >>> index is >> " +index);
        switch(index){
            case 0://settings page
                $scope.currentPage = WASHER_SCREEN.SETTINGS_PAGE;
                changeScreen();
                break;
            case 1: //About page
                $scope.currentPage = WASHER_SCREEN.ABOUT_PAGE;
                changeScreen();
                break;
              case 2: //Reset Energy data
                sendResetCommand();
                closeAllPopups();
                break;  
            case 3:
                showSelectableFavoriteList();
                closeAllPopups();
                break;
        default:
            //Do nothing
            break;
        }
    }
    
    function aboutAction (action) {
        var selectedCountry;
        selectedCountry = $scope.selectedCountry; 
        
        if(action === 'tutorial' || action === 'manual' || action === 'helpdesk') {
            $scope.radioBtnSelectedIndex = 0;            
            if($scope.selectedCountry === "BE") {
                $scope.radioBtnToSelect = [];
                $scope.radioBtnToSelect.push('Belgium');
                $scope.radioBtnToSelect.push('Belgium(French)');
                reqParamCountrySelection = action;
            } else if($scope.selectedCountry === "CA") {
                $scope.radioBtnToSelect = [];
                $scope.radioBtnToSelect.push('Canada');
                $scope.radioBtnToSelect.push('Canada(French)');
                reqParamCountrySelection = action;
            } else if($scope.selectedCountry === "CH") {
                $scope.radioBtnToSelect = [];
                $scope.radioBtnToSelect.push('Switzerland');
                $scope.radioBtnToSelect.push('Switzerland(French)');
                reqParamCountrySelection = action;                
            } else {
                nativeInterface.runOnNative("customerService", aboutCommand(action));
                return;
            }
            $scope.isRadioSelection = true;
            showDialog(ALERT_STATE.COUNTRY_SELECTION, $scope.translation.WEBMOB_common_tutorial_select_language, '', $scope.translation.WEBMOB_common_cancel, $scope.translation.WEBMOB_common_ok);
        } else if (action === 'smartCare'){
            //goto smart care page
                $scope.currentPage = WASHER_SCREEN.SELF_CARE_PAGE;
                changeScreen();                
        }else {
            if (!$scope.isStatic) {
                $timeout(function () {
                    nativeInterface.runOnNative("customerService", '{"service" : "' + action + '","selectedCountry" : "' + selectedCountry + '"}');
                }, MATERIAL_DESIGN.LIST_ANIMATION_TIME);
            }
        }
    }
    
    function settingsAction(action){
        debugMessage("settingsAction >>>>>>>> REQUEST PARAM IS " + action)
        var sendCommand = false, value = '';        
        switch(action){
            case 'sound':
                sendCommand = true;
                value = ($scope.Device.SettingOptions.isSoundOn)? 'Sound_Off' : 'Sound_On';
                break;
            case 'autoRelease':
                sendCommand = true;
                value = ($scope.Device.SettingOptions.isAutoReleaseOn)? 'AutoDoorRelease_Off' : 'AutoDoorRelease_On';
                break;
            case 'drain':
                sendCommand = true;
                value = 'Course_89'; //Send to start drain on dishwasher
                break;
            default: //Do nothing
                break;
        }
        
        if (sendCommand && !$scope.isStatic){
            var command = '{"Mode":{"options":["' + value + '"]}}';
            debugMessage("send Setting command " + command);
            $scope.checkResponse.push(WASHER_COMMANDS.SETTINGS_COMMAND);
            SHPService.sendSHPCommand(DWCONST.PUT, command, "/" + $scope.peerId + "/devices/0/mode");
        }else {//just static
            debugMessage("settings command not send either 'sendCommand' command param is " +sendCommand + " or in static mode");
            
            if (action === 'sound') { //For demo/try it now mode
                $scope.Device.SettingOptions.isSoundOn = !$scope.Device.SettingOptions.isSoundOn;
            }else if (action === 'autoRelease'){
                $scope.Device.SettingOptions.isAutoReleaseOn = !$scope.Device.SettingOptions.isAutoReleaseOn;
            }
        }
    }
    
    function onRadioBtnSelected (selectedIndex) {
        debugMessage("selected index is "+selectedIndex);
        $scope.radioBtnSelectedIndex = selectedIndex;
    }
    
    function startDiagnosisNow(){
        if (!$scope.isStatic) {
            $scope.startDiagnosis = true;
            $scope.checkResponse.push(WASHER_COMMANDS.DIAGNOSIS_COMMAND);
            SHPService.sendSHPCommand(DWCONST.PUT, diagnosisCommand(), "/" + $scope.peerId + "/devices/0/diagnosis", 300000);
        } else {
            $scope.currentPage = WASHER_SCREEN.NO_ERROR_PAGE;
            $scope.smartCare = false;
            $scope.smartCareMessage = true;
        }
    }
    
    function openEnergyMonitor() {
        if((!$scope.isStatic) && (!$scope.isDBDownloaded)){
            return;
        }
        
        $scope.currentPage = WASHER_SCREEN.ENERGY_MONITOR;        
        changeScreen();
    }
        
    function showDiagnosisPage(diagnosisVal){
        if(angular.isDefined(diagnosisVal.diagnosisStart)) {
            $scope.diagnosisState = diagnosisVal.diagnosisStart;
            if(diagnosisVal.diagnosisStart === "Ready"){
                dismissLoadingBar(WASHER_COMMANDS.DIAGNOSIS_COMMAND);
                if($scope.startDiagnosis) {
                    $scope.startDiagnosis = false;
                    if ($scope.currentPage === WASHER_SCREEN.ERROR_PAGE) {
                        debugMessage("return showDiagnosisPage " + $scope.currentPage + " && " + $scope.diagnosisState);
                        return;
                    }
                    $scope.currentPage = WASHER_SCREEN.NO_ERROR_PAGE;
                    $scope.smartCare = false;
                    $scope.smartCareMessage = true;
                }
            } else if(diagnosisVal.diagnosisStart === "Run"){
                $scope.startDiagnosis = true;
                dismissLoadingBar(WASHER_COMMANDS.DIAGNOSIS_COMMAND);                    
                if ($scope.currentPage === WASHER_SCREEN.ERROR_PAGE) {
                    debugMessage("return showDiagnosisPage " + $scope.currentPage + " && " + $scope.diagnosisState);
                    return;
                }
                // If we get diagnosis run notification that means user has sent request from another mobile, then show loading bar
                if ($scope.checkResponse.indexOf(WASHER_COMMANDS.DIAGNOSIS_COMMAND) === -1) {
                    $scope.checkResponse.push(WASHER_COMMANDS.DIAGNOSIS_COMMAND);
                }
               
                if ($scope.currentPage !== WASHER_SCREEN.SELF_CARE_PAGE){
                    $scope.currentPage = WASHER_SCREEN.SELF_CARE_PAGE;
                    changeScreen();
                    $scope.smartCare = true;
                }
            } else {
                //Do Nothing
            }
        } 
    }
    
    function toggleAlarmInfoArea(){
        $scope.alarmExpanded = !$scope.alarmExpanded;
       }
    
    function showAlarmInfo(alarmType){
        debugMessage('showAlarmInfo >>>>>>>>> alarm type ' + alarmType);
        switch(alarmType){
            case ALERT_STATE.ADD_DISH_ALARM:
                if (!$scope.AddDishAlarm && $scope.Device.Operation.state === 'Run'){
                    $scope.AddDishAlarm = true;
                    $scope.selfCleanAlarm = false;
                    $scope.alarmInfoText = $scope.translation.WEBMOB_device_dishwasher_add_dish_desc;
                    $scope.alarmIcon = 'DW/assets/img/dw_main_ic_add_dish.svg#Layer_1';
                    $scope.alarmExpanded = false;
                }
                break;
            case ALERT_STATE.SELF_CLEAN_ALARM:
                if (!$scope.isAlarmClosed && !$scope.selfCleanAlarm && $scope.Device.Operation.state === 'Ready'){
                    $scope.AddDishAlarm = false;
                    $scope.selfCleanAlarm = true;
                    $scope.alarmInfoText = $scope.translation.WEBMOB_device_dishwasher_selfclean_desc1 + $scope.translation.WEBMOB_device_dishwasher_selfclean_desc2;
                    $scope.alarmIcon = 'DW/assets/img/dw_main_ic_self_clean.svg#Layer_1';
                    $scope.alarmExpanded = false;
                }    
                break;
            default://Do nothing
                break;
        }
        
        var svg = document.getElementById('alarm-icon');        
        SHPService.loadSVG($scope.alarmIcon ,svg);
    }
    
    function hideAlarmInfo(alarmType){
        if (alarmType === ALERT_STATE.ADD_DISH_ALARM && $scope.AddDishAlarm){
            $scope.AddDishAlarm = false;
            $scope.alarmExpanded = false;
            $scope.alarmInfoText = '';
            $scope.alarmIcon = '';
        }
        
        if (alarmType === ALERT_STATE.SELF_CLEAN_ALARM && $scope.selfCleanAlarm){
            $scope.selfCleanAlarm = false;
            $scope.alarmExpanded = false;
            $scope.alarmInfoText = '';
            $scope.alarmIcon = '';
        }      
    }
    
    function alarmBtnClicked (btnType){
        switch(btnType){
            case 1://Apply button on self clean alarm info
                //change to self care cycle and hide self clean alarm popup
                var indexes = $.map($scope.Device.CourseData, function(obj, index){
                    if (obj.courseNameHex === '88'){// 88 - Self Clean
                        return index;
                    }
                });
                
                $scope.selectedModeIndex = (indexes instanceof Array && indexes.length === 1)?indexes[0]:$scope.selectedModeIndex;
                resetDelayStartTime(); //reset delay start time if course changed
                setCourseOptions();
                hideAlarmInfo(ALERT_STATE.SELF_CLEAN_ALARM);
                toaster.success({title: "", body: $scope.translation.WEBMOB_device_dishwasher_toast_self_clean, bodyOutputType: 'trustedHtml'});
                break;
            case 2://Close button on self clean alarm info
                $scope.isAlarmClosed = true;
                hideAlarmInfo(ALERT_STATE.SELF_CLEAN_ALARM);
                break;
            case 3://OK button on rinse aid alarm - GAURAV change here 
                aidNotificationClose();
                break;
        }
    }
    
        /**    	 
         * @function
         * @private
    	 * @name aidNotificationClose        
    	 * @description Manages display of in in app aid notificaytions.
    	 */
        function aidNotificationClose() {
            if($scope.inAppAidAlarms.length > 0) {
                $scope.inAppAidAlarms.splice(0, 1);               
            } 
            if($scope.inAppAidAlarms.length === 0) {
                $scope.displayAidAlarm = false;
            }else { //reload SVG when changing path
                var svg = document.getElementById('rinse-aid-icon');        
                SHPService.loadSVG($scope.inAppAidAlarms[0].icon ,svg);
            }
        }
    
    function versionTouchStart () {
        if (!promise) {
            promise = $timeout(function () {
                if (!$scope.isStatic) {
                    $scope.checkResponse.push(WASHER_COMMANDS.MODULE_INFO);
                    SHPService.sendSHPCommand(DWCONST.GET, "/" + $scope.peerId + "/devices/0/information");
                }
            }, 5000);
        }
    }

    function versionTouchEnd () {
        if (promise) {
            $timeout.cancel(promise);
            promise = null;
        }
    }
    
    function setDelayStartValue (){
        $scope.delayStartValue = $scope.delHrsTemp;
        if ($scope.delHrsTemp !== '00'){
            $scope.delayStartStatus = parseInt($scope.delayStartValue, 10) + $scope.translation.WEBMOB_device_washer_comm_hr;
        }else {
            $scope.delayStartStatus = $scope.translation.WEBMOB_device_dishwasher_delaystart_none;
        }
        
        closeAllPopups();
    }
    
    function updateDelayStartTime (isUpTapped) {
        var value = 0;
        if (isUpTapped) { // increase
            value = parseInt($scope.delHrsTemp, 10);
            if(value < 24){
                value++;
            }
            else{
                value = 0;
            }
        } else { // decrease
            value = parseInt($scope.delHrsTemp, 10);
            if(value > 0){
                value--;
            }
            else{
                value = 24;
            }
        }
        //console.log("value:" + value);
        $scope.delHrsTemp = (value < 10)?"0"+value:value;        
    }
    
    function resetDelayStartTime(){
        //RESET DELAY START IF SET
        //console.log(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> RESET DELAY START TIME");
        if ($scope.delHrs === '00' && $scope.delMin === '00' ){ //If delay start is not sent by device then reset
            $scope.delHrsTemp = "00";
            $scope.delayStartValue = "00";
            $scope.delayStartStatus = $scope.translation.WEBMOB_device_dishwasher_delaystart_none;
        }
    }
    
    function increaseDelayContinuously () {
        //$interval should be called only once. Hence, check for promise.
        if (!!!promise) {
            //It has been observed that touchend event is not fired every time. 
            //Hence, limit the $interval to a number of executions. 
            //Here this number is 24 = intervals required to reach maximum from minimum value.
            promise = $interval(function () {
                updateDelayStartTime(true);
            }, 200, 24);    //24 -> number of maximum calls in case touchend event could not be fired.
        }
    }

    function decreaseDelayContinuously() {
       //$interval should be called only once. Hence, check for promise.
        if (!!!promise) {
            //It has been observed that touchend event is not fired every time. 
            //Hence, limit the $interval to a number of executions. 
            //Here this number is 24 = intervals required to reach minimum from maximum value.
            promise = $interval(function () {
                updateDelayStartTime(false);
            }, 200, 24);    //24 -> number of maximum calls in case touchend event could not be fired.
        }
    }
    
    function stopLongPressEvent () {
        if (!!promise) {
            //check if promise exist. Then call cancel.
            $interval.cancel(promise);
            promise = undefined;
        }
    }
    
    function showOptionItemsVisible(itemIndex) {
        var isVisible = (($scope.homePage && itemIndex < 2) || ($scope.energyMonitorPage && itemIndex === 2 && isEnergyReset()) || ($scope.favoritePage && $scope.favoriteList.length > 0 && itemIndex === 3 ))?true:false;
        //console.log(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", itemIndex, isVisible);
        return isVisible;
    }
    
    function fetchDBfromDevice () {
        debugMessage("fetchDBfromDevice >>> >>> nativeInterface.getResourceUrl" + "/" + $scope.peerId + $scope.energyMonitorLocation);
        if (!$scope.isStatic) {
            nativeInterface.getResourceUrl("/" + $scope.peerId + $scope.energyMonitorLocation);
        }
    }
    
    function showDailyData () {
        EnergyService.showDailyData($scope);
    }
    
    function showWeeklyData () {
        EnergyService.showWeeklyData($scope);
    }
    
    function showMonthlyData () {
        EnergyService.showMonthlyData($scope);
    }
    
    function showNextDaysData () {
        EnergyService.showNextDaysData($scope);
    }
    
    function showPreviousDaysData () {
        EnergyService.showPreviousDaysData($scope);
    }
        
    function isEnergyReset() {
        return true;
    }
    
    function isEnergyResetDisabled(){
        return !$scope.remoteControlEnabled;
    }

    function parseEnergyUsageData(energyResponse) {
        EnergyService.parseEnergyUsageData($scope, energyResponse);
    }
    
    function sendResetCommand () {
        if (!$scope.remoteControlEnabled){
            debugMessage("Do not send Reset command if smart control is off");
            return;
        }
        debugMessage("Reset command sending");
        if (!$scope.isStatic) {
            $scope.checkResponse.push(WASHER_COMMANDS.DB_RESET);
            SHPService.sendSHPCommand(DWCONST.PUT, getResetCommand(), "/" + $scope.peerId + "/devices/0");
        }
    }
    
    /*SELF CLEAN HISTORY AND FAVORITE SECTION*/
    function createArray(n) {
        n = Math.floor(n);
        if (n > 0) {
            return new Array(n);
        }
        return new Array(0);
    }
        
    function applyHistoryFilterStyle(){
        //console.log($scope.washingTimes, $scope.drumCleanProposal, ">>>>>>>>>>>>>>>>>>>>>>>>");
        var el = document.getElementById("filterlayerstyle");
        var elt = document.getElementById("drumCleanWashingTextTop");        
        if (el && elt){
            if(Number($scope.washingTimes) < Number($scope.drumCleanProposal)){
                el.style.width = $scope.washingTimes*12.825/$scope.drumCleanProposal + 'rem';
                el.style.backgroundColor = '#89cfec';
                elt.style.color = 'rgb(0,171,241)';
                $scope.drumCleanAlert = false;
            }else {
                $timeout(function(){
                    el.style.backgroundColor = 'rgb(249,132,122)';
                    el.style.width = 16 + 'rem';
                    elt.style.color = 'rgb(255,115,102)';
                },0)
                
                $scope.drumCleanAlert = true;
            }
        }
    }
    
    $scope.$watchGroup(['washingTimes' , 'drumCleanProposal','screenAreaHeight','drumCleanHistoryPage'],applyHistoryFilterStyle);
    
    function openSelfCleanHistory() {
        $scope.currentPage = WASHER_SCREEN.SELF_CLEAN_HISTORY_PAGE;        
        changeScreen();
    }
    
    function openMyfavorite() {
        $scope.currentPage = WASHER_SCREEN.FAVORITE_PAGE;        
        changeScreen();
    }
    
    function openAddFavorite() {
        if ($scope.favoriteList.length >= MAX_FAVORITE_LIST) {
            toaster.success({title: "", body: $scope.translation.WEBMOB_device_dishwasher_alarm_my_favorite_toast_max_list, bodyOutputType: 'trustedHtml'});
            return;
        }
        $scope.currentPage = WASHER_SCREEN.ADD_FAVORITE_PAGE;        
        changeScreen();
    }
    
    function openCycleSelectionPage(){
        $scope.currentPage = WASHER_SCREEN.CYCLE_PAGE;        
        changeScreen();
    }
    
    function saveFavorite(){
        if ($scope.rackSelectionFavorite) {
            closeAllPopups();
            return;
        }
        $scope.favoriteObject.id = $scope.favoriteObject.id || (new Date()).getTime();
        var name = $("#inputMFName").val();
        if (name !== undefined && name.trim() === '') {
            $scope.favoriteWashName = ($scope.selectedFavoriteItemIndex === null)?getMyFavoriteName():$scope.favoriteObject.name;
        } else {
            if (isDuplicateName(name, $scope.favoriteObject.id)) {
                return;
            }
            $scope.favoriteWashName = name;
        }
        
        $scope.favoriteObject.CurrentCourseHexString = $scope.Device.CourseData[ $scope.favoriteObject.modeIndex].courseNameHex;
        $scope.favoriteObject.name = $scope.favoriteWashName;
        if ($scope.selectedFavoriteItemIndex === null){ //create new 
            $scope.favoriteList.unshift($scope.favoriteObject);
            //console.log($scope.favoriteObject, $scope.favoriteList);
            setLocalStorageData($scope.deviceUuid + '_Favorite', JSON.stringify($scope.favoriteList));
            toaster.success({title: "", body: $scope.translation.WEBMOB_device_dishwasher_alarm_my_favorite_toast_save_new, bodyOutputType: 'trustedHtml'});
        }else{//update existing 
            //TODO ADD A CHECK TO SEE IF NOTHING IS CHANGED THEN 
            angular.copy($scope.favoriteObject, $scope.favoriteList[$scope.selectedFavoriteItemIndex]);
            //console.log($scope.favoriteObject, $scope.favoriteList);
            setLocalStorageData($scope.deviceUuid + '_Favorite', JSON.stringify($scope.favoriteList));
            toaster.success({title: "", body: $scope.translation.WEBMOB_device_dishwasher_alarm_my_favorite_toast_save_change, bodyOutputType: 'trustedHtml'});
        }
        openMyfavorite();
    }
    
    function isDuplicateName(title, id) {
        debugMessage('validation for  name ' + title);
        var result = $scope.favoriteList.filter(function (obj) {
            return obj.name === title && obj.id !== id;
        });
        
        if (result instanceof Array && result.length > 0){
            alertState = ALERT_STATE.DUPLICATE_NAME;
            $timeout(function(){showDialog(alertState, '',$scope.translation.WEBMOB_device_dishwasher_alarm_my_favorite_same_name_msg, '', $scope.translation.WEBMOB_common_ok);}, 500);
            return true;
        }
        
        return false;        
    }
    
    function getMyFavoriteName() {
        var defaultName = $scope.translation.WEBMOB_device_dishwasher_settings_my_favorite, j = 1;
        var list = getLocalStorageData($scope.deviceUuid + '_Favorite') || [];
        if (list.length === 0) {
            return defaultName + ' 1';
        }
        
        var newLocalStorage = JSON.parse(list);

        while (true) {
            var uniqueNameFound = true;
            defaultName = $scope.translation.WEBMOB_device_dishwasher_settings_my_favorite + ' ' + j;
            for (var i = 0; i < newLocalStorage.length; i++) {
                if (newLocalStorage[i]["name"] !== undefined && newLocalStorage[i]["name"].trim() === (defaultName).trim()) {
                    uniqueNameFound = false;
                    break;
                }
            }
            if (uniqueNameFound) {
                break;
            }
            j++;
        }

        return defaultName;
    }
    
    
    /*REVISIT AND CHECK BELOW FUNCTIONS FOR MULTI SELECT - TAKEN FROM WASHER*/
    function numOfCheckedItems() {
        var numOfCheckedItem = 0;
        for (var i = 0; i < $scope.favoriteList.length; i++) {
            if ($scope.favoriteList[i].isChecked) {
                numOfCheckedItem++;
            }
        }
        return numOfCheckedItem;
    }
    
    function toggleCheckList(item, index) {
        //$scope.indexSelected = index;
        debugMessage("toggleCheckList called");
        for (var i = 0; i < $scope.favoriteList.length; i++) {
            if ($scope.favoriteList[i].name === item.name) {
                $scope.favoriteList[i].isChecked = !$scope.favoriteList[i].isChecked;
            }
        }
        isAllCheckBoxSelected();
        isAnyCheckedItem();
    }

    function isAllCheckBoxSelected() {
        for (var i = 0; i < $scope.favoriteList.length; i++) {
            if ($scope.favoriteList[i].isChecked === false) {
                $scope.allSelected = false;
                return;
            }
        }
        $scope.allSelected = true;
    }

    function toggleAllCheckList() {
        if ($scope.isAnyItemChecked) {
            $scope.isAnyItemChecked = false;
        }
        
        $scope.allSelected = !$scope.allSelected;
        for (var i = 0; i < $scope.favoriteList.length; i++) {
            $scope.favoriteList[i].isChecked = $scope.allSelected;
        }        
    }

    function isChecked(currentListInfo, index) {
        for (var i = 0; i < $scope.favoriteList.length; i++) {
            if ($scope.favoriteList[i].name === currentListInfo.name) {
                return $scope.favoriteList[i].isChecked;
            }
        }
        return false;
    }

    function isAnyCheckedItem() {
        for (var i = 0; i < $scope.favoriteList.length; i++) {
            if ($scope.favoriteList[i].isChecked) {
                $scope.isAnyItemChecked = true;
                return;
            }
        }
        $scope.isAnyItemChecked = false;
    }
    
    function cancelMultiSelect() {
        $scope.isMultiSelectMode = false;
        $scope.isAnyItemChecked = false;
        $scope.allSelected = false;
        for (var i = 0; i < $scope.favoriteList.length; i++) { //reset isChecked flag from myFavorite
            if ($scope.favoriteList[i].isChecked) {
                $scope.favoriteList[i].isChecked = false;
            }
        }
        openMyfavorite();
    }

    function showSelectableFavoriteList() {
        $scope.isMultiSelectMode = true;
        $scope.allSelected = false;
        $scope.isAnyItemChecked = false;
    }
    
    // note : Commented function to check whether to enable save button in add/edit favorite page - check later use
//    function hasFavoriteChanged() {
//        var isEnable = false;
//        if ($scope.selectedFavoriteItemIndex === null){
//            isEnable = true;
//        }else{
//            var key;
//            for (key in $scope.favoriteList[$scope.selectedFavoriteItemIndex]){//check
//                if ($scope.favoriteList[$scope.selectedFavoriteItemIndex].hasOwnProperty(key) && $scope.favoriteObject.hasOwnProperty(key)){
//                    if ( key !== 'isChecked'){//ignore isChecked field
//                        if (key === 'name' && ($scope.favoriteList[$scope.selectedFavoriteItemIndex][key] !== $('#inputMFName').val())){ //for name change use input field model instead of object value
//                            isEnable = true;
//                            break;
//                        }
//                        
//                        if ($scope.favoriteList[$scope.selectedFavoriteItemIndex][key] !== $scope.favoriteObject[key]){
//                            isEnable = true;
//                            break;
//                        }                    
//                    }
//                }
//            }
//        }        
//        return isEnable;
//    }
    
    function confirmDeleteItemsFromFavorite() {
        alertState = ALERT_STATE.DELETE_FAVORITE_ITEMS;
        showDialog(alertState, '',$scope.translation.WEBMOB_device_dishwasher_alarm_my_favorite_delete_msg, $scope.translation.WEBMOB_common_cancel_capital, $scope.translation.WEBMOB_common_ok);        
    }
    
    function deleteItemsFromFavorite(){
        if ($scope.allSelected) { //all selected
            $scope.favoriteList = [];
        } else if (!$scope.allSelected && $scope.isAnyItemChecked) {
            for (var i = 0; i < $scope.favoriteList.length; i++) {
                if ($scope.favoriteList[i].isChecked) {
                    var index = i;
                    if (index > -1) {
                        $scope.favoriteList.splice(index, 1);
                        --i;
                    }
                }
            }
        }
        
        setLocalStorageData($scope.deviceUuid + '_Favorite', JSON.stringify($scope.favoriteList));
        cancelMultiSelect();
        toaster.success({title: "", body: $scope.translation.WEBMOB_device_dishwasher_alarm_my_favorite_toast_delete, bodyOutputType: 'trustedHtml'});
    }
        
    function deleteItemFromFavorite(index) {
        alertState = ALERT_STATE.DELETE_FAVORITE_ITEM;
        $scope.selectedFavoriteItemIndex = index;
        showDialog(alertState, '',$scope.translation.WEBMOB_device_dishwasher_alarm_my_favorite_delete_msg, $scope.translation.WEBMOB_common_cancel_capital, $scope.translation.WEBMOB_common_ok);        
    }
    
    function editMyFavorite(item, index){
        $scope.selectedFavoriteItemIndex = index;
        var copyObject = angular.copy($scope.favoriteList[$scope.selectedFavoriteItemIndex], $scope.favoriteObject);
        //console.log($scope.favoriteList[$scope.selectedFavoriteItemIndex], $scope.favoriteObject);
        
        $scope.currentPage = WASHER_SCREEN.ADD_FAVORITE_PAGE;
        changeScreen();
    }
}

app.directive('roundProgressLoaded', function ($interval) {
    return {      
        restrict: 'A',
        link: function (scope, element) {
            if (scope.circleWidth === 0){
                document.getElementById("round_progress_container").style.visibility = 'visible';
                scope.circleWidth = Number(document.getElementById("round_progress_container").offsetWidth);
                scope.circleHeight = Number(document.getElementById("round_progress_container").offsetHeight);

                scope.max = 100;
                scope.stroke = Math.ceil(scope.circleHeight / 20);
                scope.radius = scope.circleHeight / 2;
                if(scope.applePhone) {
                    scope.diameter = scope.radius * 2 + 2;
                } else {
                    scope.diameter = scope.radius * 2;
                }
                //TODO - JUST FOR TESTING
    //            scope.progressStart = false;
    //            scope.progressHead = false;
    //            scope.current = 0;
    //            $interval(function(){
    //               scope.progressHead = true;
    //               scope.current++;
    //               if (scope.current > 99)
    //                   scope.current = 0;
    //            }, 1000);
            }
        }
    };
});

app.service('CourseParseService', function () {
    this.getCourseParsed = function (supportedOptions, $scope/*, parseVal*/) {
        //supportedOptions : 5 6680029001A001B001C0006780029001A001B001C0006880029001A001B001C0006980029000A001B000C0006E80029001A000B001C0006B80009000A000B000C000
        // 5 8280029002A000B0010000 8380029002A000B0010000 8480029002A000B0010000 8580029000A000B0000000 8680029000A000B0010000 8780039000A101B0000000 8880039001A000B1000000

        // 5 - hardcode
        // 82 830C 9002 A002 B002 C002
        // 83 830C 9002 A002 B002 C002
        // 84 830C 9002 A002 B002 C002
        // 85 830C 9000 A002 B000 C000
        // 86 830C 9002 A000 B002 C002
        // 87 830C 9000 A000 B000 C000
        // 88 8308 9000 A000 B000 C000

        // note : use static data til device sends proper data.
        //supportedOptions = '582 830C 9002 A002 B002 C00283 830C 9002 A002 B002 C00284 830C 9002 A002 B002 C00285 830C 9000 A002 B000 C00086 830C 9002 A000 B002 C00287 830C 9000 A000 B000 C00088 8308 9000 A000 B000 C000';
        //supportedOptions = supportedOptions.replace(/ /g, '');
        
        $scope.CourseEnumData = {
            '82': ['WEBMOB_device_dishwasher_course_auto','DW/assets/img/dw_drop_list_ic_auto.svg#Layer_1'],        // 82 - Auto
            '83': ['WEBMOB_device_dishwasher_course_normal','DW/assets/img/dw_drop_list_ic_normal.svg#Layer_1'],      // 83 - Normal
            '84': ['WEBMOB_device_dishwasher_course_heavy','DW/assets/img/dw_drop_list_ic_heavy.svg#Layer_1'],       // 84 - Heavy
            '85': ['WEBMOB_device_dishwasher_course_delicate','DW/assets/img/dw_drop_list_ic_delicate.svg#Layer_1'],    // 85 - Delicate
            '86': ['WEBMOB_device_dishwasher_course_express60','DW/assets/img/dw_drop_list_ic_express_60.svg#Layer_1'],  // 86 - Express 60
            '87': ['WEBMOB_device_dishwasher_course_rinseonly','DW/assets/img/dw_drop_list_ic_rinse_only.svg#Layer_1'],    // 87 - Rinse Only
            '88': ['WEBMOB_device_dishwasher_course_selfclean','DW/assets/img/dw_drop_list_ic_self_clean.svg#Layer_1'],   // 88 - Self Clean
            '8A': ['WEBMOB_device_dishwasher_course_8A', 'DW/assets/img/dw_drop_list_ic_eco.svg#Layer_1'],			// 8A - Eco
            '8B': ['WEBMOB_device_dishwasher_course_8B', 'DW/assets/img/dw_drop_list_ic_zone_quick.svg#Layer_1'],	// 8B - Quick
            'A7': ['WEBMOB_device_dishwasher_course_A7', 'DW/assets/img/dw_drop_list_ic_heavy.svg#Layer_1'],		// A7 - Intensive
            'A8': ['WEBMOB_device_dishwasher_course_A8', 'DW/assets/img/dw_drop_list_ic_express.svg#Layer_1'],		// A8 - Daily55
            'B7': ['WEBMOB_device_dishwasher_course_B7', 'DW/assets/img/dw_drop_list_ic_chef.svg#Layer_1']			// B7 - Chef

        };

        var optionNumberStr = supportedOptions.substring(0, 1);
        var optionNumber = parseInt(optionNumberStr,10);
        var lengthOfCourseString = DWCONST.LEN_COURSE + (optionNumber * DWCONST.LEN_OPTION);
        var lengthOfSupportedOptions = supportedOptions.length;
        if ((lengthOfSupportedOptions - 1) % lengthOfCourseString !== 0) {
            debugMessage("Not a correct course data string!");
            return null;
        }

        $scope.Device.CourseData = [];

        for (var startIndexOfOption = 1; startIndexOfOption < lengthOfSupportedOptions; startIndexOfOption += lengthOfCourseString) {
            var CourseData = {
                // course info
                courseIconPathAndName   : '',
                courseName              : '',
                courseNameHex           : '',

                // options info
                optionUpperLowerList    :[],
                optionZoneBoosterList   :[],

                isRackSelectionEnable   : false,
                isZoneBoosterEnable     : false,
                isSpeedBoosterEnable    : false,
                isHiTempWashEnable      : false,
                isSanitizeEnable        : false,

                defaultRackSelection    : '',
                defaultZoneBooster      : '',
                defaultSpeedBooster     : '',
                defaultHiTempWash       : '',
                defaultSanitize         : '',

                optionTypes: [],    // todo : remove? not necessary?
                optionNumber: 0     // todo : remove? not necessary?
            };
            /*
             * 66(Course) 8002(Option1) 9001(Option2) A001(Option3) B001(Option4) C000(Option5)
             */
            var courseAndOptionsString = supportedOptions.substring(startIndexOfOption, startIndexOfOption + lengthOfCourseString);
            CourseData.courseNameHex = courseAndOptionsString.substring(0, DWCONST.LEN_COURSE);
            CourseData.courseName = $scope.CourseEnumData[CourseData.courseNameHex][DWCONST.COURSE_INDEX_NAME];
            CourseData.courseIconPathAndName = $scope.CourseEnumData[CourseData.courseNameHex][DWCONST.COURSE_INDEX_ICON];

            /*
             * taking care of options 8002(Option1) 9001(Option2) A001(Option3) B001(Option4) C000(Option5)
             */
            for (var j = DWCONST.START_INDEX_OPTION1; j < courseAndOptionsString.length; j += DWCONST.LEN_OPTION) {

                // Hex to int converter - 8002(Option1)
                var optionString = courseAndOptionsString.substring(j, j + DWCONST.LEN_OPTION);
                var option = parseInt(optionString, 16);

                // 8 : [0] Option Type - 4bit(15-12 bit)
                var optionType = (option >> 12) & 0x0000000F;
                // 0 : [1] Defaults - 4bit(11-8 bit) => Index values
                var defaultValue = (option >> 8) & 0x0000000F;
                // 02 : [2] Available options - 8bit(7-0 bit) => Bit per available options
                var availableOptions = option & 0x000000FF;

                if (optionType === DWCONST.OPTIONTYPE_UPPER_LOWER) {
                    // if(parseVal === 0) {
                    var supportedRackSelection = $scope.Device["Washer"].supportedUpperLower; // OFF_OFF(Not Selected), ON_OFF, OFF_ON, ON_ON
                    // } else {
                        //list = $scope.listItemsTemp;
                    // }
                    CourseData.defaultRackSelection = supportedRackSelection[defaultValue];   // todo : reset data if the value is 'OFF_OFF"?
                    var len = supportedRackSelection.length;
                    for (var k = 0; k < len; k++) { // 02 - 0000 0010
                        if (((availableOptions >> k) & 0x00000001) === 0x00000001 && (CourseData.optionUpperLowerList.indexOf(supportedRackSelection[k]) === -1)) {    // todo : check indexOf of array
                            CourseData.optionUpperLowerList.push(supportedRackSelection[k]);
                            if(k === 1 || k === 2 || k === 3 ) {
                                CourseData.isRackSelectionEnable = true;
                            }
                        }
                    }
                    if (CourseData.optionTypes.indexOf("8") === -1) {   // todo : remove? not necessary?
                        CourseData.optionTypes.push("8");
                    }
                }
                else if (optionType === DWCONST.OPTIONTYPE_ZONE_BOOSTER) {
                    // if(parseVal === 0) {
                    var supportedZoneBooster = $scope.Device["Washer"].supportedZoneBooster;
                    // } else {
                    //     //list_rin = $scope.listItemsRinse;
                    // }
                    CourseData.defaultZoneBooster = supportedZoneBooster[defaultValue];
                    for (var ri = 0; ri < supportedZoneBooster.length; ri++) {
                        if (((availableOptions >> ri) & 0x00000001) === 0x00000001 && (CourseData.optionZoneBoosterList.indexOf(supportedZoneBooster[ri]) === -1)) {
                            CourseData.optionZoneBoosterList.push(supportedZoneBooster[ri]);
                            if(ri === 1 || ri === 2 || ri === 3 ){
                                CourseData.isZoneBoosterEnable = true;
                            }
                        }
                    }
                    CourseData.optionTypes.push("9");                   // todo : remove? not necessary?
                }
                else if (optionType === DWCONST.OPTIONTYPE_SPEED_BOOSTER) {
                    // if(parseVal === 0) {
                    var supportedSpeedBooster = $scope.Device["Washer"].supportedSpeedBooster;
                    // } else {
                    //     //list_sp = $scope.listItemsSpin;
                    // }
                    if(supportedSpeedBooster !== undefined) {
                        CourseData.defaultSpeedBooster = supportedSpeedBooster[defaultValue];
                    }
                    CourseData.isSpeedBoosterEnable = (availableOptions & 0x00000002) ? true : false;

                    CourseData.optionTypes.push("A");                   // todo : remove? not necessary?
                }
                else if (optionType === DWCONST.OPTIONTYPE_SANITIZE) {
                    var supportedSanitize = $scope.Device["Washer"].supportedSanitize;
                    if(supportedSanitize !== undefined){
                        CourseData.defaultSanitize = supportedSanitize[defaultValue];
                    }
                    CourseData.isSanitizeEnable = (availableOptions & 0x00000002) ? true : false;

                    CourseData.optionTypes.push("B");                   // todo : remove? not necessary?
                }
                else if (optionType === DWCONST.OPTIONTYPE_HITEMP_WASH) {
//                  if(parseVal === 0) {
                    var supportedHiTempWash = $scope.Device["Washer"].supportedHiTempWash;
                    // } else {
                    //     //list_soil = $scope.listItemsSoilLevel;
                    // }
                    if(supportedHiTempWash !== undefined){
                        CourseData.defaultHiTempWash = supportedHiTempWash[defaultValue];
                    }
                    CourseData.isHiTempWashEnable = (availableOptions & 0x00000002) ? true : false;

                    CourseData.optionTypes.push("C");                   // todo : remove? not necessary?
                }
                else{
                    // Nothing
                }

            }
            $scope.Device['CourseData'].push(CourseData);
        }
    };
});
