/*******************************************************************************
 * @author Amit Kumar (kr1.amit@samsung.com)
 * @file This file implements Refrigerator controller function  
 * Copyright 2016 by Samsung Electronics, Inc.,
 * @license This software is the confidential and proprietary information of Samsung
 * Electronics, Inc. ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Samsung.
 ******************************************************************************/
//(function () {
    
    //'use strict';

    var app = angular.module("SmartThings", ['ngSanitize', 'toaster', 'ngTouch']);
    app.controller("RefController", ["$scope", "SHPService", "ScheduleService", "EnergyService", "$timeout", "toaster", "$interval", RefController]);

    /**		 
    * @private
    * @function
    * @name RefController
    * 
    * @description This method is a constructor function for REF module and is used to define 
    * properties and methods to module's scope (angular scope).
    */ 
    function RefController($scope, SHPService, ScheduleService, EnergyService, $timeout, toaster, $interval) {
 	var strShowModuleInfoPopup = 'showModuleInfoPopup', strDialogPopup_Show = 'dialogPopup.show', strBlock = 'block', strNone = 'none',
            strRippleContainer = ".ripple-container";
        
        var reqParamCountrySelection, EnergyConsumptionInterval, promise;;

        var langArr = Object.keys(LANGUAGE_FILE.language);
        if (langArr.indexOf(LANG.toUpperCase())!== -1){
            $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase()];
        } else if(langArr.indexOf(LANG.toUpperCase().substring(0,2))!== -1){
            $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase().substring(0,2)];
        } else {
            $scope.translation = LANGUAGE_FILE["language"]["DEFAULT"];
        }

        LANGUAGE_FILE["language"] = {};
        
        $scope.schedulePeriod = 1;
        $scope.countrySelectedIndex = 0;
        
        $scope.isAppV2 = false;
        $scope.isRef = true;
        $scope.isStatic = false;
        $scope.homePage = false;
        $scope.aboutPage = false;
        $scope.bOptionPopUp = false;
        $scope.showNotificationPopUp = false;
        $scope.energyManagerPage = false;
        $scope.countrySelectionPopupVisible = false;
        //$scope.isEnergyData = false;
        
        $scope.arrImages = ["ref"];
        $scope.arrSubHeader = ["Refrigerator"];
        $scope.arrHeader = ["How to change the water filter"];
        
        $scope.SERVERURL = '';
        $scope.BottomPopupDivCls = '';
        $scope.featureLoweIconVal = 'freeze';
        $scope.BottomPopupDimBackgroundDiv = '';
        $scope.getYAxisValue = EnergyService.getYAxisValue;
        
        $scope.checkResponse = [];
        $scope.countryToSelect = [];
        $scope.countryToSelect.push({'Country': 'Belgium'});
        $scope.countryToSelect.push({'Country': 'Belgium(F)'});
        
        var REFCONST = {
            GET: "GET",
            PUT: "PUT",
            POST: "POST",
            DELETE: "DELETE",
            RUNONNATIVE: "RUNONNATIVE",
            LCD16K: "16K_REF_LCD_T9000",
            LCD15K: "15K_REF_LCD_T9000",
            LED15K: "15K_REF_LED_T9000",
            SCREENS_HOME: 1,
            SCREENS_ABOUT: 2,
            SCREENS_SETTINGS: 3,
            SCREENS_ENERGY_MONITOR: 4,
            SCREENS_SMART_CARE: 5,
            SCREENS_ENERGY_MANAGER: 6,
            SCREENS_DEMAND_RESPONSE: 7,
            SCREENS_DEFROST: 8,
            SCREENS_ADDSCHEDULE: 9,
            SCREENS_ADD_DATE_SCHEDULE : 10,
            SCREENS_ADD_TIME_SCHEDULE : 11,
            SCREENS_TUTORIAL : 12,
            SCREENS_DETAILEDTUTORIAL : 13,
            SCREENS_ERRORPAGE : 14,
            GET_DEVICE:"getDevices",
            POWER_COOL: "powerFridge",
            POWER_FREEZE: "powerFreeze",
            ICE_MAKER: "iceMaker",
            DIAGNOSIS: "diagnosis",
            PARSE_MODULEINFO: "PARSE_MODULEINFO",
            DRLC: "DRLC",
            DEFROST_ADD_SCHEDULE : "DefrostAddSchedule",
            DELAY_DEFROST : 'delayDefrost',
            DELAY_DEFROST_DELETDSCHEDULE : 'deleteSchedule',  
            DELAY_DEFROST_GETSCHEDULE : 'getSchedule', 
            "LIST_ANIMATION_TIME": 400, // Animation time for ripple effect in list
            "POWER_ANIMTION_TIMEOUT": 130,
            "BACK_ANIMTION_TIMEOUT": 130,
            "TOGGLE_TIME": 330,
            "BTN_ANIMATION_TIME": 480,
            "STATUS_507" : 507        
        };
        
        var serviceCenterNum = [
            {"KR": "1588-3366"}, 
            {"CA": "1-800-726-7864"}, 
            {"MX": "01-800-726-7864"}, 
            {"US": "1-800-726-7864"}, 
            {"AR": "0800-333-3733"}, 
            {"UY": "40543733"}, 
            {"PY": "98005420001"}, 
            {"BR": "0800-124-421"}, 
            {"CL": "800-726-7864"}, 
            {"BO": "800-10-7260"}, 
            {"CO": "01-8000-112-112"}, 
            {"CR": "0-800-507-7267"}, 
            {"DM": "1-800-751-2676"}, 
            {"EC": "1-800-10-7267"}, 
            {"SV": "800-6225"}, 
            {"GT": "1-800-299-0013"}, 
            {"HN": "800-27919267"}, 
            {"JM": "1-800-234-7267"}, 
            {"NI": "00-1800-5077267"}, 
            {"PA": "800-7267"}, 
            {"PE": "0-800-777-08"}, 
            {"PR": "1-800-682-3180"}, 
            {"TT": "1-800-726-7864"}, 
            {"VE": "0-800-100-5303"}, 
            {"AT": "0810-7267864"}, 
            {"BE": "02-201-24-18"}, 
            {"BA": "051-133-1999"}, 
            {"BG": "07001-33-11"}, 
            {"HR": "062-726-786"}, 
            {"CZ": "800-726786"}, 
            {"DK": "70-70-19-70"}, 
            {"FI": "030-6227-515"}, 
            {"FR": "01-48-63-00-00"}, 
            {"DE": "0180-5-7267864*"}, 
            {"CY": "8009-4000"}, 
            {"GR": "80111-726-7864"}, 
            {"HU": "06-80-726-7864"}, 
            {"IT": "800-726-7864"}, 
            {"LU": "261-03-710"}, 
            {"ME": "020-405-888"}, 
            {"NL": "0900-7267864"}, 
            {"NO": "815-56480"}, 
            {"PL": "0-801-172-678*"}, 
            {"PT": "8808-20-7267"}, 
            {"RO": "08008-726-7864"}, 
            {"RS": "381-11-321-6899"}, 
            {"SK": "0800-726-786"}, 
            {"ES": "902-172-678"}, 
            {"SE": "0771-726-7864"}, 
            {"CH": "0848-7267864"}, 
            {"GB": "0330-7267864"}, 
            {"IE": "0818 717100"}, 
            {"LI": "8-800-77777"}, 
            {"LV": "8000-7267"}, 
            {"EE": "800-7267"}, 
            {"RU": "8-800-555-55-55"}, 
            {"GE": "0-800-555-555"}, 
            {"AM": "0-800-05-555"}, 
            {"AZ": "088-55-55-555"}, 
            {"KZ": "8-10-800-500-55-500"}, 
            {"UZ": "8-10-800-500-55-500"}, 
            {"KG": "00-800-500-55-500"}, 
            {"TJ": "8-10-800-500-55-500"}, 
            {"MN": "7-800-555-55-55"}, 
            {"UA": "0-800-502-000"}, 
            {"BY": "810-800-500-55-500"}, 
            {"MD": "0-800-614-40"}, 
            {"AU": "1300-362-603"}, 
            {"NZ": "0800-726-786"}, 
            {"CN": "400-810-5858"}, 
            {"HK": "852-3698-4698"}, 
            {"IN": "1800-266-8282"}, 
            {"ID": "021-5699-7777"}, 
            {"JP": "0120-327-527"}, 
            {"MY": "1800-88-9999"}, 
            {"PH": "1-800-10-726-7864"}, 
            {"SG": "1800-726-7864"}, 
            {"TH": "02-689-3232"}, 
            {"TW": "0800-329-999"}, 
            {"VN": "1-800-588-889"}, 
            {"AE": "800-726-7864"}, 
            {"OM": "800-726-7864"}, 
            {"KW": "183-2255"}, 
            {"BH": "8000-4726"}, 
            {"QA": "800-2255"}, 
            {"EG": "08000-726786"}, 
            {"JO": "800-22273"}, 
            {"SY": "18252273"}, 
            {"IR": "021-8255"}, 
            {"MA": "080 100 2255"}, 
            {"SA": "9200-21230"}, 
            {"TR": "444-77-11"}, 
            {"NG": "0800-726-7864"}, 
            {"GH": "0800-10077"}, 
            {"CI": "8000-0077"}, 
            {"SN": "800-00-0077"}, 
            {"CM": "7095-0077"}, 
            {"KE": "0800-545-545"}, 
            {"UG": "0800-300-300"}, 
            {"TZ": "0685-88-99-00"}, 
            {"ZA": "0860-726-7864"}, 
            {"BW": "0800-726-000"}, 
            {"NA": "8197267864"}, 
            {"AO": "91-726-7864"}, 
            {"ZM": "211350370"}, 
            {"MK": "023-207-777"}
        ];
                
        $scope.energyManager = {
            thisMonthEnergy : 0,
            lastMonthEnergy : 0    	
        };
        
        $scope.defrostDelay = {
            userScheduleDisplay_2 : false,
            userScheduleDisplay_3 : false,
            scheduleId : '',
            addScheduleId : '',
            deleteScheduleId : '',    	
            defrostScheduleDate : [],	
            defrostScheduleTime : [],
            timeData :{start:'', startAmPm:'', end:'', endAmPm:''}   	                      
        };

        $scope.selectedSchedule = {
            id:'', 
            hours: "", 
            minutes: "", 
            amPm: "AM", 
            month : "",
            day : "",
            keypadOn: false 
        };
        
        $scope.smartCare = {
            started: true,
            diagnosisStarted: false,
            detected:false,
            viewErros:false,
            totalErrors:0,
            errors:[]
        };

        $scope.dialogPopup = {
            show: false,
            dialogPopupTitle: '',
            dialogPopupText: '',
            button1Title: '',
            button1Handler: function() {toggleBottomPopUpClass(false); $scope.dialogPopup.show = false;},
            button2Enabled: false,
            button2Title: '',
            button2Handler: function() {}
        };
        
        $scope.monthNames = [
            "Jan", 
            "Feb",
            "Mar",
            "Apr",
            "May",
            "Jun",
            "Jul",
            "Aug",
            "Sep",
            "Oct",
            "Nov",
            "Dec"
        ];
    
        $scope.demandResponseStatusNames = [
                $scope.translation.WEBMOB_device_ref_energymanager_demand_respons_state_txt_normal,
                $scope.translation.WEBMOB_device_ref_energymanager_demand_respons_state_txt_normal,
                $scope.translation.WEBMOB_device_ref_energymanager_demand_respons_state_txt_normal,
                $scope.translation.WEBMOB_device_ref_energymanager_demand_respons_state_txt_dal,
                $scope.translation.WEBMOB_device_ref_energymanager_demand_respons_state_txt_talr
        ];
        
        $scope.optionsMenu = [{
            id: 0,
            title: $scope.translation.WEBMOB_common_settings
        }, {
            id: 1,
            title: $scope.translation.WEBMOB_common_about_device
        }];
        
        if ($scope.isStatic === true && !$scope.deviceUuid) {
            $scope.parseStaticData();       
        }

	// Toggle class for body
	function toggleAnimClass() {
            if ($scope.containSubCl === "containSub") {
                $scope.containSubCl = "";
            } else {
                $scope.containSubCl = "containSub";
            }
        }
        
        $scope.init = init;
        $scope.setDate = setDate;
        $scope.closeApp = closeApp;
        $scope.saveWind = saveWind;
        $scope.playVideo = playVideo;
        $scope.scrollEnd = scrollEnd;
        $scope.goToDefrost = goToDefrost;
        $scope.iceMakerOpr = iceMakerOpr;
        $scope.isPopupOpen = isPopupOpen;
	$scope.scrollStart = scrollStart;
        $scope.inputKeyDown = inputKeyDown;
        $scope.moduleInfoOk = moduleInfoOk;
        $scope.increaseDays = increaseDays;
        $scope.decreaseDays = decreaseDays;
        $scope.saveSchedule = saveSchedule;
        $scope.goToSmartCare = goToSmartCare;
        $scope.onAboutAction = onAboutAction;
        $scope.showDailyData = showDailyData;
        $scope.increaseHours = increaseHours;
        $scope.isEnergyReset = isEnergyReset;
        $scope.decreaseHours = decreaseHours;
        $scope.onBackPressed = onBackPressed;
        $scope.showWeeklyData = showWeeklyData;
        $scope.changeMeridian = changeMeridian;
        $scope.setEnergyValue = setEnergyValue;
        $scope.powerFridgeOpr = powerFridgeOpr;
        $scope.powerFreezeOpr = powerFreezeOpr;
        $scope.terminatePopup = terminatePopup;
        $scope.increaseMonths = increaseMonths;
        $scope.decreaseMonths = decreaseMonths;
        $scope.doAlarmService = doAlarmService;
        $scope.startSmartCare = startSmartCare;
        $scope.showMonthlyData = showMonthlyData;
        $scope.goToAddSchedule = goToAddSchedule;
        $scope.versionTouchEnd = versionTouchEnd;
        $scope.parseDeviceData = parseDeviceData;
        $scope.parseStaticData = parseStaticData;
        $scope.increaseMinutes = increaseMinutes;
        $scope.decreaseMinutes = decreaseMinutes;
        $scope.showOptionPopUp = showOptionPopUp;
        $scope.closeOptionPopUp = closeOptionPopUp;
        $scope.showNextDaysData = showNextDaysData;
        $scope.updateDeviceData = updateDeviceData;
        $scope.fetchDBfromDevice = fetchDBfromDevice;
        $scope.demandResponseOpr = demandResponseOpr;
        $scope.onCountrySelected = onCountrySelected;
        $scope.versionTouchStart = versionTouchStart;
        $scope.goToEnergyMonitor = goToEnergyMonitor;
        $scope.goToEnergyManager = goToEnergyManager;
        $scope.powerButtonPressed = powerButtonPressed;
        $scope.requestFailHandler = requestFailHandler;
        $scope.defrostResponseOpr = defrostResponseOpr;
        $scope.stopLongPressEvent = stopLongPressEvent;
        $scope.goToDemandResponse = goToDemandResponse;
        $scope.viewSmartCareErrors = viewSmartCareErrors;
        $scope.goToAddDateSchedule = goToAddDateSchedule;
        $scope.goToAddTimeSchedule = goToAddTimeSchedule;
        $scope.showPreviousDaysData = showPreviousDaysData;
        $scope.parseEnergyUsageData = parseEnergyUsageData;
        $scope.errorVisibilityToggle = errorVisibilityToggle;
        $scope.parsePowerConsumption = parsePowerConsumption;
        $scope.increaseHrContinuously = increaseHrContinuously;
        $scope.decreaseHrContinuously = decreaseHrContinuously;
        $scope.parseModuleInformation = parseModuleInformation;
        $scope.addDefrostUserSchedule = addDefrostUserSchedule;
        $scope.increaseDayContinuously = increaseDayContinuously;
        $scope.decreaseDayContinuously = decreaseDayContinuously;
        $scope.filterNotificationClose = filterNotificationClose;
        $scope.increaseMinContinuously = increaseMinContinuously;
        $scope.decreaseMinContinuously = decreaseMinContinuously;
        $scope.increaseScheduleInterval = increaseScheduleInterval;
        $scope.decreaseScheduleInterval = decreaseScheduleInterval;
        $scope.handleEnergyMonitorErrors = handleEnergyMonitorErrors;
        $scope.increaseMonthContinuously = increaseMonthContinuously;
        $scope.decreaseMonthContinuously = decreaseMonthContinuously;
        $scope.deleteDefrostUserSchedule = deleteDefrostUserSchedule;
        $scope.selectIndexFromOptionMenu = selectIndexFromOptionMenu;
        $scope.showConnectionFailurePopup = showConnectionFailurePopup;
        $scope.closeCountrySelectionPopup = closeCountrySelectionPopup;
        $scope.onCountrySelectedOkClicked = onCountrySelectedOkClicked;
        $scope.closeConnectionFailurePopup = closeConnectionFailurePopup;
        $scope.onscheduleIntervalLostFocus = onscheduleIntervalLostFocus;
        $scope.startEnergyConsumptionTimer = startEnergyConsumptionTimer;
        $scope.showcustCountrySelectionPopup = showcustCountrySelectionPopup;
        $scope.increaseScheduleIntervalContinuously = increaseScheduleIntervalContinuously;
        $scope.decreaseScheduleIntervalContinuously = decreaseScheduleIntervalContinuously;
        

        function playVideo(ind){ 

            $scope.currentScreen = REFCONST.SCREENS_DETAILEDTUTORIAL;
            changeScreen();

            $scope.videoPopup = ind;
            document.getElementById("vid").src = "https://s3.amazonaws.com/smartthingsda/solutionmodule/tutorial/changewaterfilter2.mp4";
        }
        
        /**    	 
         * @function
         * @private
    	 * @name isPopupOpen        
         * @description Check if loading bar on or any popup open.         
    	 */
        function isPopupOpen () {
            if($scope.showConnFailurePopup || isElementVisible(strShowModuleInfoPopup) || $scope.loadingBar) {
                return true;
            }
            return false;
        }
        
	/**    	 
         * @function 
    	 * @name isElementVisible	 
    	 * @description Checks visibility of an element using display style.
         * @private 
	 * @param {String} elemId elemId Id of element to be checked.
	 * @return true if element display property has value 'block'. Otherwise false.
	 * @author Lokesh Sharma
    	 */
	function isElementVisible(elemId) {
            var elem = document.getElementById(elemId);
            return elem && elem.style.display === strBlock;
        }
	
        /**    	 
         * @function
         * @private
    	 * @name init
    	 * @description after getting the getDevice response, it is called from app.js
    	 */
        function init() {
            $scope.Device = {
                id: '',
                description: '',
                Alarms: [{
                        alarmType: '',
                        code: '', 
                        id: '',
                        triggeredTime: ''
                    }],
                ConfigurationLink: {
                    href: ''
                },
                Doors: [],
                Fridge: {
                    filterState: '',
                    iceMaker:'Off',
                    iceMakerEventTime: 0,
                    rapidFreezing: 'Off',
                    rapidFridge: 'Off',
                    delayDefrost: '',
                    DefrostReservations: [{id:'', endTime: '', startTime: '', period: '', endDate:'', startDate:''}]                              
                },
                InformationLink: {
                    href: ''
                },
                DRLC : {
                    drlcLevel : 0,
                    drlcStartTime : '',
                    duration : '',
                    override : ''  
                },
                Mode: {
                    modes: [],
                    supportedModes: [],
                    cvRoom: '',
                    iceStatus: '',
                    sodaGas: '',
                    waterFilter: '',
                    airFilter: '',
                    fconvert: '',
                    rvacation: '',
                    otn: '',
                    cvn: '',
                    blockStatus: '', 
                    chefMode: ''
                },
                energyMonitorLocation:'',
                cumulativeConsumption: 0,
                monthlyConsumption: 0,
                Diagnosis: {
                    diagnosisStart:''
                },
                connected: '',
                name: '',
                resources: [],
                type: '', 
                uuid: '',
                Temperatures: [
                    {
                        id: '0',
                        current: 0,
                        desired: 0,
                        unit: '', 
                        eventTime: 0
                    },
                    {
                        id: '1',
                        current: 0,
                        desired: 0,
                        unit: '',
                        eventTime: 0
                    }
                ]
            };
            
            $scope.isCoolSelect = false;
            $scope.powerFreeze = false;
            $scope.powerFridge = false;
            $scope.isPowerFridge = false;
            $scope.isPowerFreeze = false;
            $scope.isIceMaker = false;
            $scope.isFilterStatusShow = false;
//            $scope.powerFridgeClicked = false;    //Lokesh 20Jan16 -> Not needed.
            $scope.powerFreezeClicked = false;
            $scope.filterStatusArr = [];
            $scope.currentScreen = REFCONST.SCREENS_HOME;
            $scope.isDelayDefrost = false;
            $scope.isDrlcOverride = false;

            $scope.popupBackGround = false;

            $scope.isDRLC = false;

            $scope.startDate = {
                            month : "",
                            day : "",
                            hour : 0,
                            minutes : 0,
                            amPm : ""
            };
            $scope.endDate = {
                            month : "",
                            day : "",
                            hour : 0,
                            minutes : 0,
                            amPm : ""
            };
            
            changeScreen();
            startEnergyConsumptionTimer();
        }

        /**    	 
         * @function
         * @private
    	 * @name updateDeviceData
         * @param {String} data
         * data is passed to updateDeviceData which is in app.js
    	 * @description it gets called from nativeCallback function (app.js), it is for passing the data from native app to web app
    	 */
        function updateDeviceData(data) {
            SHPService.updateDeviceData($scope, data);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name manageRefModels
         * @param {String} description : description string of REF from get device
         * @description Based on description of REF model, set various settings.
    	*/
        function manageRefModels(description){
            switch(description) {
                case REFCONST.LCD15K :
                    $scope.isCoolSelect = true;
                case REFCONST.LCD16K :
                    $scope.isPowerFridge = true;
                    $scope.isPowerFreeze = true;                    
                    $scope.isFilterStatusShow = false;
                    break;
                case REFCONST.LED15K :
                    $scope.isPowerFridge = false;
                    $scope.isPowerFreeze = true;
                    $scope.isCoolSelect = false;
                    $scope.isFilterStatusShow = false;
                    break;
                default:
                    $scope.isPowerFridge = false;
                    $scope.isPowerFreeze = false;
                    $scope.isCoolSelect = false;
                    $scope.isFilterStatusShow = false;
            }
        }

        /**    	 
         * @function
         * @private
    	 * @name aboutCommand
         * @param {String} cmdstring : Type of about command to be called
         * @param {boolean} changedCode : Need to check, what is this meant for
         * @description Set payload for about command.
    	*/
        function aboutCommand(cmdstring, changedCode) {
            if(changedCode) {
                return '{"service" : "' + cmdstring + '","selectedCountry" : "' + $scope.selectedCountry + '_FR"}';
            } else {
                return '{"service" : "' + cmdstring + '","selectedCountry" : "' + $scope.selectedCountry + '"}';
            }
        }
        
        function showcustCountrySelectionPopup() {
            debugMessage("showcustCountrySelectionPopup :: "+countryName[0]);
            $scope.countrySelectionPopupVisible = true;
        }

        function onCountrySelected(countryIndex) {
            debugMessage("country selected is "+countryIndex);
            $scope.countrySelectedIndex = countryIndex;
        }

        function onCountrySelectedOkClicked(isCountrySelected) {
            if(isCountrySelected /*&& !$scope.isStatic*/) { //Removed static check as in demo mode also we need to run native commands
                if($scope.countrySelectedIndex === 0){
                    $timeout(function () {
                        nativeInterface.runOnNative("customerService", aboutCommand(reqParamCountrySelection));
                    }, REFCONST.LIST_ANIMATION_TIME);
                } else {
                    $timeout(function () {
                        nativeInterface.runOnNative("customerService", aboutCommand(reqParamCountrySelection, true));
                    }, REFCONST.LIST_ANIMATION_TIME);
                }
            }
            closeCountrySelectionPopup();
        }

        function closeCountrySelectionPopup() {
            $scope.countrySelectionPopupVisible = false;
        }
        
        /**    	 
         * @function
         * @private
    	 * @name onAboutAction
         * @param {String} reqParam : Type of about command to be sent to device
         * @description Set payload for about command.
    	*/
        function onAboutAction(reqParam) {
            
            if(reqParam === 'tutorial' || reqParam === 'manual' || reqParam === 'helpdesk') {
                if($scope.selectedCountry === "BE") {
                    $scope.countryToSelect = [];
                    $scope.countryToSelect.push({'Country': 'Belgium'});
                    $scope.countryToSelect.push({'Country': 'Belgium(French)'});
                    reqParamCountrySelection = reqParam;
                    showcustCountrySelectionPopup();
                } else if($scope.selectedCountry === "CA") {
                    $scope.countryToSelect = [];
                    $scope.countryToSelect.push({'Country': 'Canada'});
                    $scope.countryToSelect.push({'Country': 'Canada(French)'});
                    reqParamCountrySelection = reqParam;
                    showcustCountrySelectionPopup();
                } else if($scope.selectedCountry === "CH") {
                    $scope.countryToSelect = [];
                    $scope.countryToSelect.push({'Country': 'Switzerland'});
                    $scope.countryToSelect.push({'Country': 'Switzerland(French)'});
                    reqParamCountrySelection = reqParam;
                    showcustCountrySelectionPopup();
                } else {
//                    if($scope.isStatic) {
//                        $scope.selectedCountry = "BE";
//                        showcustCountrySelectionPopup();
//                        return;
//                    }
//                  //Removed static check above as in demo mode also we need to run native commands and we are getting selected country so 
                    //no need to set hard code country code 
                    $timeout(function () {
                        nativeInterface.runOnNative("customerService", aboutCommand(reqParam));
                    }, REFCONST.LIST_ANIMATION_TIME);
                }
            } else {
                $timeout(function () {
                    nativeInterface.runOnNative("customerService", aboutCommand(reqParam));
                }, REFCONST.LIST_ANIMATION_TIME);
            }
        }
        
        /**    	 
         * @function
         * @private
    	 * @name setFreezerTemp
         * @param {Object} temp : Temprature object
         * @param {String} eventTime : Notification event time
         * @description Set frreezer temperature related settings
    	*/
        function setFreezerTemp(temp, eventTime) {
            if($scope.Device.Temperatures[0].eventTime > eventTime) {
                return;
            }
            $scope.Device.Temperatures[0].id = temp.id;
            if (angular.isDefined(temp["current"])) {
                $scope.Device.Temperatures[0].current = temp.current;
            }

            if (angular.isDefined(temp["desired"])) {
                $scope.Device.Temperatures[0].desired = temp.desired;
            }

            if (angular.isDefined(temp["name"])) {
                $scope.Device.Temperatures[0].name = temp.name;
            }

            if (angular.isDefined(temp["unit"])) {
                $scope.Device.Temperatures[0].unit = temp.unit;
            }
            $scope.Device.Temperatures[0].eventTime = eventTime;
        }
        
        /**    	 
         * @function
         * @private
    	 * @name setFridgeTemp
         * @param {Object} temp : Temprature object
         * @param {String} eventTime : Notification event time
         * @description Set freezer temperature related settings
    	*/
        function setFridgeTemp(temp, eventTime) {
            if($scope.Device.Temperatures[1].eventTime > eventTime) {
                return;
            }
            $scope.Device.Temperatures[1].id = temp.id;
            if (angular.isDefined(temp["current"])) {
                $scope.Device.Temperatures[1].current = temp.current;
            }
            if (angular.isDefined(temp["desired"])) {
                $scope.Device.Temperatures[1].desired = temp.desired;
            }
            if (angular.isDefined(temp["name"])) {
                $scope.Device.Temperatures[1].name = temp.name;
            }
            if (angular.isDefined(temp["unit"])) {
                $scope.Device.Temperatures[1].unit = temp.unit;
            }
            $scope.Device.Temperatures[1].eventTime = eventTime;

        }
        
        /**    	 
         * @function
         * @private
    	 * @name setAlarms
         * @param {Object} deviceAlarms : Alarm object
         * @description Handle all errors and alarms.
    	*/
        function setAlarms(deviceAlarms) {

            for (var i = 0; i < deviceAlarms.length; i++) {
                if ($scope.Device.Alarms[i] === undefined) {
                    $scope.Device.Alarms[i] = {alarmType: '', code: '', id: '', triggeredTime: ''};
                }
                if (angular.isDefined(deviceAlarms[i].alarmType)) {
                    $scope.Device.Alarms[i].alarmType = deviceAlarms[i].alarmType;
                }
                if (angular.isDefined(deviceAlarms[i].code)) {
                    $scope.Device.Alarms[i].code = deviceAlarms[i].code;
                }
                if (angular.isDefined(deviceAlarms[i].id)) {
                    $scope.Device.Alarms[i].id = deviceAlarms[i].id;
                }
                if (angular.isDefined(deviceAlarms[i].triggeredTime)) {
                    $scope.Device.Alarms[i].triggeredTime = deviceAlarms[i].triggeredTime;
                }
            }
        }
        
         /**    	 
         * @function
         * @private
    	 * @name setModeOptions
         * @param {Array} modes
         * @description Mode parsing done in this method and based on modes values, 
         * we set different functions of Refrigerator. 
        */
        function setModeOptions(modes) {
            modes.forEach(function (value) {
                var modeOptionArr  = value.split("_");

                switch(modeOptionArr[0]) {
                    case "CV":
                        $scope.Device.Mode.cvRoom = value;
                        setCVRoomData();
                        break;
                    case "ICESTATUS":
                        $scope.Device.Mode.iceStatus = modeOptionArr[1];
                        break;
                    case "SODAGAS":
                        $scope.Device.Mode.sodaGas = modeOptionArr[1];
                        if($scope.Device.Mode.sodaGas === 'ENABLE') {
                            $scope.filterStatusArr.push('WEBMOB_device_ref_replace_sodagas_msg');
                            $scope.isFilterStatusShow = true;
                        }
                        break;
                    case "WATERFILTER":
                        $scope.Device.Mode.waterFilter = modeOptionArr[1];
                        if($scope.Device.Mode.waterFilter === 'ENABLE') {
                            $scope.filterStatusArr.push('REFM02_waterFilter_status3');
                            $scope.isFilterStatusShow = true;
                        }
                        break;
                    case "AIRFILTER":
                        $scope.Device.Mode.airFilter = modeOptionArr[1];
                        if($scope.Device.Mode.airFilter === 'ENABLE') {
                            $scope.filterStatusArr.push('WEBMOB_device_ref_replace_cleanfilter_msg');
                            $scope.isFilterStatusShow = true;
                        }
                        break;
                    case "OTN":
                        $scope.Device.Mode.otn = modeOptionArr[2];
                        break;
                    case "RVACATION":
                        $scope.Device.Mode.rvacation = modeOptionArr[1];
                        break;
                    case "FCONVERT":
                        var mode = '';
                        if (modeOptionArr[2] !== undefined && modeOptionArr[2] !== '') {
                            mode = modeOptionArr[1] + '_' + modeOptionArr[2];
                        } else {
                            mode = modeOptionArr[1];
                        }
                        $scope.Device.Mode.fconvert = mode;
                        break;
                    case "CVN":
                        $scope.Device.Mode.cvn = value.substring(4);
                        break;
                    case "DEFROST":                	
                            if(value == "DEFROST_BLOCK_OFF")                	
                                    $scope.Device.Mode.blockStatus = false;
                            if(value == "DEFROST_BLOCK_ON")                	
                                    $scope.Device.Mode.blockStatus = true;
                            console.log(" mode : " +$scope.Device.Mode.blockStatus + "    "+value);
                        break;
                    case "CHEFMODE"://CHEFMODE feature for BMF model
                        $scope.Device.Mode.chefMode = modeOptionArr[1];
                        break;
                    default:
                        break;                        
                }

                if($scope.Device.Fridge.iceMaker === 'On' && $scope.Device.Mode.iceStatus === 'RUN'){
                    $scope.ice_maker = $scope.SERVERURL+"KREF/assets/img/main_rf_list_ic_icemaker_animation.gif";
                }else{
                    $scope.ice_maker = $scope.SERVERURL+"KREF/assets/img/main_rf_list_ic_icemaker.png";
                }

                if($scope.Device.Mode.rvacation === 'ON') {
                    $scope.isPowerFridge = false;
                } else if($scope.Device.Mode.rvacation !== 'ON' && $scope.Device.Mode.rvacation !== '') {
                    $scope.isPowerFridge = true;
                }

                if($scope.Device.Mode.fconvert === 'FREEZER' || $scope.Device.Mode.fconvert === '') {
                    $scope.isPowerFreeze = true;
                } else {
                   $scope.isPowerFreeze = false; 
                }
            });

            setCVRoomName($scope.Device.Mode.cvn);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name setCVRoomName
         * @param {String} cvn
         * @description CV Room name setting, 
        */
        function setCVRoomName(cvn) {
            switch(cvn) {
                case "COOL_SELECT+":
                    $scope.coolSelectName = 'WEBMOB_cvn_cool_select_plus';
                    break;
                case "CONVERTIBLE_ZONE":
                    $scope.coolSelectName = 'WEBMOB_cvn_convertible_zone';
                    break;
                case "CHAMMAT_FREEZER":
                    $scope.coolSelectName = 'WEBMOB_cvn_chammat_freezer';
                    break;
                case "3STEP_CHAMMAT":
                    $scope.coolSelectName = 'WEBMOB_cvn_3step_chammat';
                    break;
                case "COOL_SELECT_ZONE":
                    $scope.coolSelectName = 'WEBMOB_cool_select_zone';
                    break;
                case "FHUB_CHAMMAT":
                    $scope.coolSelectName = 'WEBMOB_cvn_chammat_freezer_2';
                    break;    
                default:
                    if($scope.isStatic && LANG.toUpperCase() === "ZH-CN") {
                            $scope.coolSelectName = 'WEBMOB_cvn_cool_select_plus';
                    } else {
                            $scope.coolSelectName = 'REFM02_cool_select';
                    }
                break;
           } 
        }
      
        /**    	 
         * @function
         * @private
    	 * @name setCVRoomData
         * @description CV Room type and icon setting. 
        */
        function setCVRoomData() {
            $scope.cvRoomIconViewBox = "0 0 356 356";        
            switch ($scope.Device.Mode.cvRoom) {
                case "CV_FHUB_FREEZER":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_41;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_freezer.svg#Layer_1';
                    break;
                case "CV_FHUB_MEAT_FISH":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_43;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_meat_fish.svg#Layer_1';
                    break;
                case "CV_FHUB_BEER":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_45;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_144_beer_wine.svg#Layer_1';
                    $scope.cvRoomIconViewBox = "0 0 144 144";
                    break;
                case "CV_CHEESE":
                    $scope.cvRoomName = 'CHEESE';
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_cheese.svg#Layer_1';
                    break;
                case "CV_FHUB_VEGETABLE_CHEESE":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_44;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_144_cheese_vegetable.svg#Layer_1';
                    $scope.cvRoomIconViewBox = "0 0 144 144";
                    break;
                case "CV_VEGETABLE":
                    $scope.cvRoomName = 'VEGETABLE';
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_vegetable.svg#Layer_1';
                    break;
                case "CV_TTYPE_COOL":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_4;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_144_cool.svg#Layer_1';
                    $scope.cvRoomIconViewBox = "0 0 144 144";
                    break;
                case "CV_SSEC_COOL":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_31;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_144_cool.svg#Layer_1';
                    $scope.cvRoomIconViewBox = "0 0 144 144";
                    break;
                case "CV_TTYPE_CHILL":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_3;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_144_chill.svg#Layer_1';
                     $scope.cvRoomIconViewBox = "0 0 144 144";
                    break;
                case "CV_SSEC_CHILL":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_32;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_144_chill.svg#Layer_1';
                     $scope.cvRoomIconViewBox = "0 0 144 144";
                    break;
                case "CV_TTYPE_SOFT_FREEZER":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_2;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_softfreezer.svg#Layer_1';
                    break;
                case "CV_SOFT_FREEZER"://FALLTHROUGH    
                case "CV_TMF_SOFT_FREEZER":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_22;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_softfreezer.svg#Layer_1';
                    break;
                case "CV_SSEC_SOFT_FREEZER":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_33;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_softfreezer.svg#Layer_1';
                    break;
                case "CV_FHUB_SOFT_FREEZER":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_42;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_softfreezer.svg#Layer_1';
                    break;
                case "CV_FREEZER"://FALLTHROUGH    
                case "CV_TMF_FREEZER":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_23;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_freezer.svg#Layer_1';
                    break;
                case "CV_TTYPE_FREEZER":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_1;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_freezer.svg#Layer_1';
                    break;
                case "CV_SSEC_FREEZER":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_34;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_freezer.svg#Layer_1';
                    break;
                case "CV_SBS_FRIDGE":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_11;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_fridge.svg#Layer_1';
                    break;
                case "CV_FRIDGE"://FALLTHROUGH     
                case "CV_TMF_FRIDGE":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_21;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_fridge.svg#Layer_1';
                    break;
                case "CV_FRESH":
                    $scope.cvRoomName = $scope.translation.REFM02_cool_select_fresh;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_freezer.svg#Layer_1';
                    break;
                case "CV_DRY":
                    $scope.cvRoomName = $scope.translation.REFM02_cool_select_dry;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_freezer.svg#Layer_1';
                    break;
                case "CV_SBS_DEFROST_5HR":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_13;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_5hr_thaw.svg#Layer_1';
                    break;
                case "CV_SBS_DEFROST_10HR":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_14;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_10_hrthaw.svg#Layer_1';
                    break;
                case "CV_TTYPE_FERMENT":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_5;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_144_kimchmaturation.svg#Layer_1';
                    $scope.cvRoomIconViewBox = "0 0 144 144";
                    break;
                case "CV_TTYPE_KIMCHI_WARM":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_8;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_kimchi_storage.svg#Layer_1';
                    break;
                case "CV_TTYPE_KIMCHI_NORMAL":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_7;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_kimchi_storage.svg#Layer_1';
                    break;
                case "CV_TTYPE_KIMCHI_COLD":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_6;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_kimchi_storage.svg#Layer_1';
                    break;
                case "CV_SBS_RAPID_FRIDGE":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_12;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_fastcool.svg#Layer_1';
                    break;
                case "CV_MINUS_1_ZONE"://FALLTHROUGH    
                case "CV_TMF_MINUS_1_ZONE":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_24;
                    $scope.cvRoomIcon = "";//$scope.SERVERURL + 'KREF/assets/img/main_rf_ic_fastcool.svg#Layer_1';
                    break;
                case "CV_FAST_COOLING"://FALLTHROUGH    
                case "CV_TMF_FAST_COOLING":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_25;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_356_fast_cool.svg#Layer_1';
                    break;
                case "CV_OFF"://FALLTHROUGH    
                case "CV_TMF_OFF":
                    $scope.cvRoomName = $scope.translation.WEBMOB_device_ref_cool_select_26;
                    $scope.cvRoomIcon = $scope.SERVERURL + 'KREF/assets/img/main_rf_ic_356_fridge_off.svg#Layer_1';
                    break;
                default:
                    $scope.cvRoomName = '';
                    $scope.cvRoomIcon = '';
                    break;
            }

            var svg = document.getElementById('coolSelectSVG');
            svg && svg.setAttribute('viewBox', $scope.cvRoomIconViewBox);
            SHPService.loadSVG($scope.cvRoomIcon, svg);
        }
        
        /**    	 
         * @function
         * @private
         * @name checkCVRoom
         * @param {Object} supportedModes : Supported Modes object from get device response
         * @description Check whether REF support cool select zone. 
        */
        function checkCVRoom(supportedModes) {
            supportedModes.forEach(function (value) {
                var modeOptionArr  = value.split("_");
                if(modeOptionArr[0] === "CV") {
                    $scope.isCoolSelect = true;
                }
            });
        }    
        
        /**    	 
         * @function
         * @private
    	 * @name setDefrostUserSchedule
         * @param {Object} dfrostSchedule : 
         * @description 
        */
        function setDefrostUserSchedule(dfrostSchedule) {
            for (var i = 0; i < dfrostSchedule.length; i++) {
                if ($scope.Device.Fridge.DefrostReservations[i] === undefined) {
                    $scope.Device.Fridge.DefrostReservations[i] = {id:'', endTime: '', startTime: '', period : '', endDate : '', startDate : ''};
                }          

                $scope.Device.Fridge.DefrostReservations[i].scheduleId = dfrostSchedule[i].id;
                $scope.Device.Fridge.DefrostReservations[i].endTime = setTimeInGMT(dfrostSchedule[i].endTime);
                $scope.Device.Fridge.DefrostReservations[i].startTime = setTimeInGMT(dfrostSchedule[i].startTime);
                $scope.Device.Fridge.DefrostReservations[i].period = dfrostSchedule[i].period;                           
            }

            if(dfrostSchedule.length > 2){ //0,1=default 2,3=>user       	
                if(dfrostSchedule.length === 3){  
                        if($scope.Device.Fridge.DefrostReservations[2].scheduleId === '2') {
                            $scope.defrostDelay.userScheduleDisplay_2 = true;
                        } else {
                            $scope.defrostDelay.userScheduleDisplay_3= true;
                        }
                } else if(dfrostSchedule.length === 4){
                    $scope.defrostDelay.userScheduleDisplay_2 = true;
                    $scope.defrostDelay.userScheduleDisplay_3 = true;
                } else {
                    ;//NULL
                }
            }        

            var CurrDate = new Date();
            var CurrMonth = CurrDate.getMonth() + 1;
            var defrostFirstSchedule;
            if(CurrMonth > 4 && CurrMonth < 11) {
                defrostFirstSchedule = $scope.Device.Fridge.DefrostReservations[0];
            } else {
                defrostFirstSchedule = $scope.Device.Fridge.DefrostReservations[1];       	
            }
            $scope.defrostSchedule = defrostFirstSchedule.startTime[0] + " ~ " + defrostFirstSchedule.endTime[0];
            //addDefrostUserSchedule('start');         
        }

        /**    	 
         * @function
         * @private
    	 * @name parseDeviceData     
         * @param {Object} device
         *       getDevice response object
    	 * @description To parse getDevice response, it contains complete details of present device state
    	 */
        function parseDeviceData(device) {
            if ($scope.isStatic === false && angular.isDefined(device["uuid"]) && device["uuid"] !== $scope.deviceUuid) {
               return;
            }
            if(!angular.isDefined($scope.Device)) {
                return;
            }
            $scope.powerFridgeClicked = false;
//            $scope.powerFreezeClicked = false;        //Lokesh 20Jan16 -> Not needed.

            if (angular.isDefined(device["DefrostReservations"])) {
                if( device["event"] === 'Created') {    
                    var scheduleData = device["DefrostReservations"][0];
                    var id = Number(device["DefrostReservations"][0].id);
                    if($scope.Device.Fridge.DefrostReservations[id] === undefined) {
                        $scope.Device.Fridge.DefrostReservations[id] = {id:'', endTime: '', startTime: '', period : '', endDate : '', startDate : ''};
                    } 
                    $scope.Device.Fridge.DefrostReservations[id].scheduleId = scheduleData.id;
                    $scope.Device.Fridge.DefrostReservations[id].endTime = setTimeInGMT(scheduleData.endTime);
                    $scope.Device.Fridge.DefrostReservations[id].startTime = setTimeInGMT(scheduleData.startTime);
                    $scope.Device.Fridge.DefrostReservations[id].period = scheduleData.period;                                
                    $scope.defrostDelay.addScheduleId = device["DefrostReservations"][0].id;

                    addDefrostUserSchedule('update');               			
                }    		

                //defrost delete schedule noti
                if( device["event"] === 'Deleted') {  
                    $scope.defrostDelay.deleteScheduleId = device["DefrostReservations"][0].id;

                    if($scope.defrostDelay.deleteScheduleId === '2'){    		
                            $scope.defrostDelay.userScheduleDisplay_2 = false;
                    }
                    else if ($scope.defrostDelay.deleteScheduleId === '3'){
                            $scope.defrostDelay.userScheduleDisplay_3 = false;
                    }        	     
                }
            }

            if (angular.isDefined(device["Temperatures"])) {
                device["Temperatures"].forEach(function (temp) {
                    var eventTime;
                    if (!angular.isDefined($scope.Device.Temperatures)) {
                        $scope.Device.Temperatures = [{'id': '0', 'current': 0, 'desired': 0, 'unit': '', 'eventTime':0}, {'id': '1', 'current': 0, 'desired': 0, 'unit': ''}];
                    }

                    if(angular.isDefined(device.eventTime)) {
                        eventTime = +new Date(device.eventTime);
                    } else {
                        eventTime = 0;
                    }

                    if (temp.id === "0") {  //Freezer
                        setFreezerTemp(temp, eventTime);
                    }

                    if (temp.id === "1") { //Fridge
                        setFridgeTemp(temp, eventTime); 
                    }
                });

            }
            if (angular.isDefined(device["type"])) {
                $scope.deviceType = device["type"];
            }
            if (angular.isDefined(device["Alarms"])) {
                setAlarms(device["Alarms"]);
            }
            if (angular.isDefined(device["Diagnosis"]) && angular.isDefined(device["Diagnosis"].diagnosisStart)) {
                $scope.Device.Diagnosis.diagnosisStart = device["Diagnosis"].diagnosisStart;
                if($scope.smartCare.diagnosisStarted === true && device["Diagnosis"].diagnosisStart === "Ready") {
                    dismissLoadingBar(REFCONST.DIAGNOSIS);
                    progressSmartCare();  
                    $scope.smartCare.diagnosisStarted = false;
                }
            }

            if (angular.isDefined(device["connected"])) {
                $scope.Device.connected = device.connected;
            }

            if (angular.isDefined(device["description"])) {
                $scope.Device.description = device["description"];
                manageRefModels($scope.Device.description);
            }

            if (angular.isDefined(device["id"])) {
                $scope.Device.id = device["id"];
            }

            if (angular.isDefined(device["name"])) {
                $scope.Device.name = device["name"];
            }

            if (angular.isDefined(device["resources"])) {
                $scope.Device.resources = device["resources"];
            }

            if (angular.isDefined(device["type"])) {
                $scope.Device.type = device["type"];
            }

            if (angular.isDefined(device["uuid"])) {
                $scope.Device.uuid = device["uuid"];
            }

            if (angular.isDefined(device["ConfigurationLink"]) && angular.isDefined(device["ConfigurationLink"].href)) {
                $scope.Device.ConfigurationLink.href = device["ConfigurationLink"].href;
            }
            if (angular.isDefined(device["InformationLink"]) && angular.isDefined(device["InformationLink"].href)) {
                $scope.Device.InformationLink.href = device["InformationLink"].href;
            }
            if (angular.isDefined(device["Fridge"])) {

                if (angular.isDefined(device["Fridge"].filterState)) {
                    $scope.Device.Fridge.filterState = device["Fridge"].filterState;
                    if ($scope.Device.Fridge.filterState === '2') {
                        $scope.isFilterStatusShow = true;
                        $scope.filterStatusArr.push('REFM02_waterFilter_status2');
                    } else if ($scope.Device.Fridge.filterState === '3') {
                        $scope.isFilterStatusShow = true;
                        $scope.filterStatusArr.push('REFM02_waterFilter_status3');
                    }else {
                        $scope.isFilterStatusShow = false;
                    }
                }

                if (angular.isDefined(device["Fridge"].rapidFreezing)) {
                    $scope.Device.Fridge.rapidFreezing = device["Fridge"].rapidFreezing;
                    
                    if($scope.Device.Mode.fconvert === 'FREEZER' || $scope.Device.Mode.fconvert === '') {
                        $scope.isPowerFreeze = true;
                    } else {
                       $scope.isPowerFreeze = false; 
                    }
                }

                if (angular.isDefined(device["Fridge"].rapidFridge)) {
                    $scope.Device.Fridge.rapidFridge = device["Fridge"].rapidFridge;
                    if($scope.Device.Mode.rvacation !== 'ON') {
                        $scope.isPowerFridge = true;
                    }
                }

                if (angular.isDefined(device["Fridge"].iceMaker)) {
                    var newIceMakerEventTime;
                    if(angular.isDefined(device.eventTime)) {
                        newIceMakerEventTime = +new Date(device.eventTime);
                    } else {
                        newIceMakerEventTime = 0;
                    }
                    if($scope.Device.Fridge.iceMakerEventTime <= newIceMakerEventTime) {
                        $scope.Device.Fridge.iceMaker = device["Fridge"].iceMaker;


                        if($scope.Device.Fridge.iceMaker === 'On' && $scope.Device.Mode.iceStatus === 'RUN'){
                            $scope.ice_maker = "KREF/assets/img/main_rf_list_ic_icemaker_animation.gif";
                        }else{
                            $scope.ice_maker = "KREF/assets/img/main_rf_list_ic_icemaker.png";
                        }
                    }
                    $scope.isIceMaker = true;
                }

                if ($scope.Device.Fridge.rapidFridge === 'On') {
                    $scope.powerFridge = true;
                } else {
                    $scope.powerFridge = false;
                }

                if ($scope.Device.Fridge.rapidFreezing === 'On') {
                    $scope.powerFreeze = true;
                } else {
                    $scope.powerFreeze = false;
                }

                if (angular.isDefined(device["Fridge"].delayDefrost)) {
                    $scope.Device.Fridge.delayDefrost = device["Fridge"].delayDefrost;          	
                    $scope.isDelayDefrost = true;
                } 
                else {
                    $scope.isDelayDefrost = false;
                }

                if (angular.isDefined(device["Fridge"].DefrostReservations)) {
                    setDefrostUserSchedule(device["Fridge"].DefrostReservations);
                }           

                dismissLoadingBar(REFCONST.POWER_FREEZE);
                dismissLoadingBar(REFCONST.POWER_COOL);
                dismissLoadingBar(REFCONST.ICE_MAKER);
                dismissLoadingBar(REFCONST.DELAY_DEFROST);
            }

            if (angular.isDefined(device["Mode"])) {
                if(device["Mode"].modes.length > 0) {
                    setModeOptions(device["Mode"].modes);
                }
                if(device["Mode"].supportedModes) {
                    checkCVRoom(device["Mode"].supportedModes);
                }            

                $scope.Device.Mode.modes = device["Mode"].modes;
                $scope.Device.Mode.supportedModes = device["Mode"].supportedModes;

            }
            // Energy monitor
            if (angular.isDefined(device["EnergyConsumption"]) && angular.isDefined(device["EnergyConsumption"].saveLocation)) {
                $scope.Device.energyMonitorLocation = device["EnergyConsumption"].saveLocation;
                if (!$scope.isStatic) {
                    nativeInterface.getResourceUrl("/" + $scope.peerId + $scope.Device.energyMonitorLocation);
                    if (angular.isDefined(device["EnergyConsumption"].monthlyConsumption) || angular.isDefined(device["EnergyConsumption"].cumulativeConsumption)) {
                    	parsePowerConsumption(device["EnergyConsumption"]);
                    }
                }
            }

            // DRLC
            if( angular.isDefined(device["DRLC"]) && !$scope.isStatic ){
                $scope.isDRLC = true;             	
                if( angular.isDefined(device["event"])) {        		
                    if($scope.checkResponse.indexOf(REFCONST.DRLC) !== -1) {
                        $scope.Device['DRLC'].override = device["DRLC"].override;
                        dismissLoadingBar(REFCONST.DRLC);            			
                    } else {
                        // to process notification 
                        if(angular.isDefined(device["DRLC"].override)){
                            $scope.Device['DRLC'].override = device["DRLC"].override;        				
                        }

                        if(angular.isDefined(device["DRLC"].drlcLevel)) {
                            $scope.Device['DRLC'].drlcLevel = device["DRLC"].drlcLevel;
                        }
                    }
                }
                else {        		
                    $scope.Device['DRLC'] = device["DRLC"];
                }

                $scope.Device['DRLC'].drlcLevel < 1 ? $scope.statusDR = false : $scope.statusDR = true;
                $scope.demandResponseStatus = $scope.demandResponseStatusNames[$scope.Device['DRLC'].drlcLevel];

                if($scope.Device['DRLC'].drlcLevel > 0 && $scope.Device['DRLC'].override === 'On'){
                    $scope.demandResponseStatus = $scope.demandResponseStatusNames[0];
                    $scope.statusDR = false;
                } else if($scope.Device['DRLC'].drlcLevel >= 0 && $scope.Device['DRLC'].override === 'Off') {
                    $scope.demandResponseStatus = $scope.demandResponseStatusNames[$scope.Device['DRLC'].drlcLevel];
                    $scope.Device['DRLC'].drlcLevel < 1 ? $scope.statusDR = false : $scope.statusDR = true;
                }
            }

            if (angular.isDefined(device["AddDefrostSchedule"])) {
                dismissLoadingBar(REFCONST.DEFROST_ADD_SCHEDULE);
                onBackPressed(true);        	

            }
            if (angular.isDefined(device["deleteDefrostSchedule"])) {
                dismissLoadingBar(REFCONST.DELAY_DEFROST_DELETDSCHEDULE);       	 
            }

            if ($scope.checkResponse.indexOf(REFCONST.GET_DEVICE) !== -1) {
                $scope.mainLoadingScreen = true;
                $scope.containSubCl = "containSub"; //initially the class should be present.
                $scope.homePage = true;
            }

            dismissLoadingBar(REFCONST.GET_DEVICE);          
        }
        
        /**    	 
         * @function
         * @private
    	 * @name parseStaticData       
    	 * @description To parse static data.
    	 * To run the application in static mode in browser, add ng-init="parseStaticData()" to body tag in html
         */
        function parseStaticData() {
            $scope.init();
            var device = JSON.parse(SHPService.getSHPData('Refrigerator'));
            $scope.checkResponse.push(REFCONST.GET_DEVICE);
            $scope.isStatic = true;
            $scope.parseDeviceData(device.Device);
            $scope.isStatic = true;
            $scope.isIceMaker = true; 
        }

        /**    	 
         * @function
         * @private
    	 * @name powerButtonPressed        
         * @param {String} buttonType identifier to button Type.
         * @description this method set button pressed state on/off.
    	 */
        function powerButtonPressed(buttonType) {
            if(buttonType === "FRIDGE") {
//                $scope.powerFridgeClicked = true; //Lokesh 20Jan16 -> Not needed.
            } else {
                $scope.powerFreezeClicked = true;
            }
        }
        
        /**    	 
         * @function
         * @private
    	 * @name powerFridgeOpr        
         * @param {boolean} status
         * @description this method send command to device for power cool on/off.
    	 */
        function powerFridgeOpr(status) {
//            $scope.powerFridgeClicked = true; //Lokesh 20Jan16 -> Not needed.
            $timeout(function () {
//                $scope.powerFridgeClicked = false;//Lokesh 20Jan16 -> Not needed.
                    var payload;
                    if (status) {
                        payload = '{"Fridge": {"rapidFridge": "Off"}}';
                    } else {
                        payload = '{"Fridge": {"rapidFridge": "On"}}';
                    }
                    console.log("payload = " + payload);
                    if ($scope.isStatic) {
                        $scope.powerFridge = !$scope.powerFridge;
//                        $scope.powerFridgeClicked = false;    //Lokesh 20Jan16 -> Not needed.
                    } else {
                        $scope.checkResponse.push(REFCONST.POWER_COOL);
                        SHPService.sendSHPCommand(REFCONST.PUT, payload, "/" + $scope.peerId + "/devices/0/fridge");
//                        $scope.powerFridgeClicked = false;    //Lokesh 20Jan16 -> Not needed.
                    }
            }, 100);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name powerFreezeOpr        
         * @param {boolean} status
         * @description this method send command to device for power freeze on/off.
    	 */
        function powerFreezeOpr(status) {
            $scope.powerFreezeClicked = true;
            $timeout(function () {
                $scope.powerFreezeClicked = false;
                var payload;
                if (status) {
                     payload = '{"Fridge": {"rapidFreezing": "Off"}}';

                } else {
                     payload = '{"Fridge": {"rapidFreezing": "On"}}';
                }

                if ($scope.isStatic) {
                    $scope.powerFreeze = !$scope.powerFreeze;
                    $scope.powerFreezeClicked = false;

                } else {
                    $scope.checkResponse.push(REFCONST.POWER_FREEZE); 
                    SHPService.sendSHPCommand(REFCONST.PUT, payload, "/" + $scope.peerId + "/devices/0/fridge");
                    $scope.powerFreezeClicked = false;
                }
            }, 200);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name iceMakerOpr        
         * @param 
         * @description this method send command to device for icemaker on/off.
    	 */
        function iceMakerOpr() {
            var status = $scope.Device['Fridge'].iceMaker === 'On'? true : false;
            var payload;
            if (status) {
                payload = '{"Fridge": {"iceMaker": "Off"}}';

            } else {
                payload = '{"Fridge": {"iceMaker": "On"}}';
            }

            if ($scope.isStatic) {
                $scope.Device['Fridge'].iceMaker = $scope.Device['Fridge'].iceMaker === 'On'? 'Off' : 'On';
             
            } else {
                $scope.checkResponse.push(REFCONST.ICE_MAKER); 
                SHPService.sendSHPCommand(REFCONST.PUT, payload, "/" + $scope.peerId + "/devices/0/fridge");
                //$scope.powerFreezeClicked = false;
            }
        }

        function demandResponseOpr () {
            var status = $scope.Device['DRLC'].override === 'On' ? true : false;
            var payload;
            if (status) {
                 payload = '{"DRLC": {"override": "Off"}}';
            } 
            else {
                 payload = '{"DRLC": {"override": "On"}}';
            }

            if ($scope.isStatic) {
                    $scope.Device['DRLC'].override = $scope.Device['DRLC'].override === 'On'? 'Off' : 'On';
            } 
            else {
                $scope.checkResponse.push(REFCONST.DRLC); 
                SHPService.sendSHPCommand(REFCONST.PUT, payload, "/" + $scope.peerId + "/devices/0/drlc");
            }
        }

        function defrostResponseOpr() {

            if($scope.Device.Mode.blockStatus){
                return;
            }  		

            var status = $scope.Device['Fridge'].delayDefrost === 'On'? true : false;    	
            var payload;
            if (status) {
                payload = '{"Fridge": {"delayDefrost" : "Off"}}';
            } 
            else {
                payload = '{"Fridge": {"delayDefrost" : "On"}}';
            }

            if ($scope.isStatic) {  
                $scope.Device['Fridge'].delayDefrost = $scope.Device['Fridge'].delayDefrost === 'On'? 'Off' : 'On'; 
                $scope.isDelayDefrost = false;
            } 
            else { 
                $scope.checkResponse.push(REFCONST.DELAY_DEFROST);
                SHPService.sendSHPCommand(REFCONST.PUT, payload, "/" + $scope.peerId + "/devices/0/fridge");
                $scope.isDelayDefrost = false;
            }    	
        }

        function setEnergyValue(){
            debugMessage("in setEnergy");
            if ($scope.Device.cumulativeConsumption === 0 && $scope.Device.monthlyConsumption === 0) {
                debugMessage("calculation in EnergyDB");
                $scope.energyManager.thisMonthEnergy = EnergyService.currMonthEnergyData;
                $scope.energyManager.lastMonthEnergy = EnergyService.prevMonthEnergyData;
            } else {
                debugMessage("support EnergyReporting(cumulativeConsumption,monthlyConsumption)");
                $scope.energyManager.thisMonthEnergy = $scope.Device.cumulativeConsumption.toFixed(1);
                $scope.energyManager.lastMonthEnergy = $scope.Device.monthlyConsumption.toFixed(1);
            }
        }

        function addDefrostUserSchedule(mode) {      	
            var scheduleLen = $scope.Device.Fridge.DefrostReservations.length;
            var secheduleData = $scope.Device.Fridge.DefrostReservations;   
            var startDate = '';    
            var endDate = '';
            var startHours = 0;
            var startMinutes = 0;
            var startAmPm = '';
            var period = '';    	    	
            var endHours = 0;    	
            var endAmPm = '';    	

            if(mode === "start"){//menu start
                for(var i=0; i<scheduleLen; i++) {	
                    $scope.defrostDelay.timeData = {start:'', startAmPm:'', end:'', endAmPm:''};									
                    startDate = secheduleData[i].startTime[0];
                    endDate = secheduleData[i].endTime[0];
                    startHours = secheduleData[i].startTime[1];	
                    startMinutes = secheduleData[i].startTime[2];
                    startAmPm = secheduleData[i].startTime[3];
                    period = String(secheduleData[i].period).substr(0,2);
                    endHours = Number(startHours) + Number(period);

                    if(startHours === '00'){
                        startHours = 12;
                    }
                    if(endHours < 12 ){
                        endAmPm = startAmPm;
                    } else {
                        endHours = endHours - 12;	
                        endAmPm = startAmPm === 'AM' ? 'PM' : 'AM';	
                    }
                    endHours = endHours < 10 ? '0' + endHours : endHours;			

                    $scope.defrostDelay.timeData.start = startHours + ":" + startMinutes;
                    $scope.defrostDelay.timeData.startAmPm = startAmPm;
                    $scope.defrostDelay.timeData.end = endHours + ":" + startMinutes;			
                    $scope.defrostDelay.timeData.endAmPm = endAmPm;
                    $scope.defrostDelay.defrostScheduleTime.push($scope.defrostDelay.timeData);	
                    $scope.defrostDelay.defrostScheduleDate.push(startDate+ " ~ " +endDate);				

                    if(scheduleLen === 3 && $scope.Device.Fridge.DefrostReservations[i].scheduleId === '3'){							
                        startDate = secheduleData[2].startTime[0];
                        endDate = secheduleData[2].endTime[0];
                        $scope.defrostDelay.defrostScheduleDate[2] = null;
                        $scope.defrostDelay.defrostScheduleTime[2] = null;
                        $scope.defrostDelay.defrostScheduleDate[3] = (startDate+ " ~ " +endDate);				
                        $scope.defrostDelay.defrostScheduleTime[3] = $scope.defrostDelay.timeData;								
                    }
                } 			
            } else {//add schedule update      		
                $scope.defrostDelay.timeData = {start:'', startAmPm:'', end:'', endAmPm:''};				   	
                var id = Number($scope.defrostDelay.addScheduleId);						
                startDate = secheduleData[id].startTime[0];
                endDate = secheduleData[id].endTime[0];
                startHours = secheduleData[id].startTime[1];	
                startMinutes = secheduleData[id].startTime[2];
                startAmPm = secheduleData[id].startTime[3];
                period = String(secheduleData[id].period).substr(0,2);
                endHours = Number(startHours) + Number(period);

                if(startHours === '00') {
                    startHours = 12;
                }
                if(endHours < 12 ) {
                    endAmPm = startAmPm;
                } else {
                    endHours = endHours - 12;	
                    endAmPm = startAmPm == 'AM' ? 'PM' : 'AM';	
                }
                endHours = endHours < 10 ? '0' + endHours : endHours;			

                $scope.defrostDelay.timeData.start = startHours + ":" + startMinutes;
                $scope.defrostDelay.timeData.startAmPm = startAmPm;
                $scope.defrostDelay.timeData.end = endHours + ":" + startMinutes;			
                $scope.defrostDelay.timeData.endAmPm = endAmPm;
                $scope.defrostDelay.defrostScheduleTime.push($scope.defrostDelay.timeData);	
                $scope.defrostDelay.defrostScheduleDate.push(startDate+ " ~ " +endDate);			

                $scope.defrostDelay.defrostScheduleDate[id] = startDate+ " ~ " +endDate;        		
                $scope.defrostDelay.defrostScheduleTime[id] = $scope.defrostDelay.timeData;	  

                if(id === 2) {	
                    $scope.defrostDelay.userScheduleDisplay_2 = true;
                }
                if(id === 3) {
                    $scope.defrostDelay.userScheduleDisplay_3 = true;    
                }
            }
        }   

        function deleteDefrostUserSchedule(index) {
            var id = index;	      
            $scope.checkResponse.push(REFCONST.DELAY_DEFROST_DELETDSCHEDULE);    
            SHPService.sendSHPCommand(REFCONST.DELETE, "/" + $scope.peerId + "/devices/0/fridge/defrostreservations/"+id);		    
        }  

        function initAddScheduleDate(){
            $scope.today = new Date();

            var hours = $scope.today.getHours();
            var minutes = $scope.today.getMinutes();
            var amPm = hours >= 12 ? "PM" : "AM";
            hours = hours > 21 ? hours - 12 : hours > 12 ? appendZero(hours-12) : hours > 9? hours : appendZero(hours);
            minutes = appendZero(minutes);

            $scope.startDate.month = $scope.today.getMonth(); 
            $scope.startDate.day = $scope.today.getDate();

            $scope.startDate.hours = hours;
            $scope.startDate.minutes = minutes;
            $scope.startDate.amPm = amPm;

            $scope.displayStartDate = $scope.monthNames[$scope.startDate.month] + " "+ $scope.startDate.day;
            $scope.displayStartTime = hours +":"+minutes + " "+amPm;
            $scope.displayEndDate = "-";
            $scope.displayEndTime = "-";

            $scope.selectedSchedule.hours = hours;
            $scope.selectedSchedule.minutes = minutes;
            $scope.selectedSchedule.amPm = amPm;

            $scope.selectedSchedule.month = $scope.monthNames[$scope.startDate.month];
            $scope.selectedSchedule.day =  $scope.startDate.day;

            $scope.pickerStatus = "";

           $scope.btnRemoveDim = true;
        }

        function terminatePopup() {
            $scope.isTimeSelecter = false;
            $scope.popupBackGround = false;
        }

        function setDate() {
            var month = textMonthToNumber($scope);

            if($scope.pickerStatus === "startDate") {
                $scope.startDate.month = month;
                $scope.startDate.day = $scope.selectedSchedule.day;

                $scope.displayStartDate = $scope.selectedSchedule.month + " "+ $scope.selectedSchedule.day;
            } else if($scope.pickerStatus === "endDate") {
                $scope.endDate.month = month;
                $scope.endDate.day = $scope.selectedSchedule.day;

                $scope.displayEndDate = $scope.selectedSchedule.month + " "+ $scope.selectedSchedule.day;
            }
            if($scope.displayEndTime !== "-" && $scope.displayEndDate !== "-") {
                $scope.btnRemoveDim = false;
            }
            onBackPressed(true);
        }
    
//        $scope.popTimeSelecter = function () {
//            $scope.popupBackGround = true;
//            $scope.isTimeSelecter = true;
//        }

        function periodSelect(addHours) {
            $scope.selectedPeriodHours = addHours;

            var tempTime = Number($scope.selectedSchedule.hours)+Number(addHours);
            var tempAmPm ="";

            if(tempTime > 11 && (tempTime -addHours) != 12) {
                $scope.selectedSchedule.amPm == "AM" ? tempAmPm = "PM" : tempAmPm = "AM";
            }

            if(tempTime > 12) {
                tempTime = tempTime - 12;
            }
            tempTime = appendZero(tempTime);

            $scope.endDate.hours = tempTime;
            $scope.endDate.minutes = $scope.selectedSchedule.minutes;
            $scope.endDate.amPm = tempAmPm;
            $scope.displayEndTime = tempTime+":"+appendZero(parseInt($scope.selectedSchedule.minutes,10))+" "+((tempAmPm =="") ? $scope.selectedSchedule.amPm : tempAmPm);

            $scope.isTimeSelecter = false;
            $scope.popupBackGround = false;
            if ($scope.displayEndTime !== "-" && $scope.displayEndDate !== "-") {
                $scope.btnRemoveDim = false;
            }
        }

        function saveSchedule() {
            if($scope.pickerStatus === "startTime" ){
                if($scope.selectedSchedule.hours.length === 0){
                    var hours = $scope.today.getHours();
                    hours = hours > 21 ? hours - 12 : hours > 12 ? hours-12 : hours > 9? hours : hours;
                    $scope.startDate.hours = appendZero(hours);
                    $scope.selectedSchedule.hours = $scope.startDate.hours;
                } else {
                    if (Number($scope.selectedSchedule.hours) === 0) {
                        $scope.selectedSchedule.hours = 12; 
                        $scope.startDate.hours = 12;
                    } else {
                        $scope.startDate.hours = appendZero(Number($scope.selectedSchedule.hours));
                    }
                }

                if($scope.selectedSchedule.minutes.length === 0) {
                    $scope.startDate.minutes = appendZero($scope.today.getMinutes());
                    $scope.selectedSchedule.minutes = $scope.startDate.minutes;
                }
                else {
                    $scope.startDate.minutes = appendZero(Number($scope.selectedSchedule.minutes));
                }
                $scope.startDate.amPm = $scope.selectedSchedule.amPm;

                var tempTime = appendZero(Number($scope.selectedSchedule.hours)) +":"+appendZero(Number($scope.selectedSchedule.minutes))+" "+$scope.selectedSchedule.amPm;
                $scope.displayStartTime = tempTime;
                //if($scope.displayEndTime != "-"){
                //         periodSelect($scope.selectedPeriodHours);
                //}
            } else if($scope.pickerStatus === "periodTime" ){
                periodSelect($scope.schedulePeriod);
            } else {
                ;//NULL
            }

            onBackPressed(true);
        }

        function saveWind(){    	
            if(!$scope.btnRemoveDim) {
                postDefrostReservation();     	
            } 	
        }

        function postDefrostReservation() {
            if($scope.defrostDelay.userScheduleDisplay_2 && $scope.defrostDelay.userScheduleDisplay_3) {
                $timeout(function () {
                        toaster.success({title: "", body: 'Schedule is already full'});
                }, 200);
                return;
            }
            var time = "";    	
            var startHours = change12To24((Number($scope.startDate.hours)), $scope.startDate.amPm);
            /*var startUTCDate = new Date();
            startUTCDate.setMonth($scope.startDate.month);
            startUTCDate.setDate($scope.startDate.day);
            startUTCDate.setHours(startHours);
            startUTCDate.setMinutes($scope.startDate.minutes);*/
            //var month 	= appendZero(startUTCDate.getUTCMonth()+1);
            //var day		= appendZero(startUTCDate.getUTCDate());
            var month = appendZero(Number($scope.startDate.month)+1);
            var day = appendZero($scope.startDate.day);

            //time = appendZero(startUTCDate.getUTCHours())+":"+ appendZero(startUTCDate.getUTCMinutes());
            time = startHours+":"+ appendZero($scope.startDate.minutes);
            var startDateStr = "0000-"+month+"-"+day+"T"+time+":00";

            var endHours = change12To24((Number($scope.endDate.hours)), $scope.endDate.amPm);
            /*var endUTCDate = new Date();
            endUTCDate.setMonth($scope.endDate.month);
            endUTCDate.setDate($scope.endDate.day);
            endUTCDate.setHours(endHours);
            endUTCDate.setMinutes($scope.endDate.minutes);

            month 	= appendZero(endUTCDate.getUTCMonth()+1);
            day		= appendZero(endUTCDate.getUTCDate());*/
            month 	= appendZero(Number($scope.endDate.month)+1);
            day		= appendZero($scope.endDate.day);

            //time = appendZero(endUTCDate.getUTCHours())+":"+ appendZero(endUTCDate.getUTCMinutes());
            time = endHours+":"+ appendZero($scope.endDate.minutes);
            var endDateStr = "0000-"+month+"-"+day+"T"+time+":00";

            var hours = appendZero($scope.selectedPeriodHours);
            var period = hours+":00:00";
            var description ="";    	
            var payload = '{"DefrostReservation" : {"startTime" : "'+startDateStr+'", "endTime" : "'+endDateStr+'", "period" : "'+period+'", "description" : "'+description+'"}}';

            $scope.checkResponse.push(REFCONST.DEFROST_ADD_SCHEDULE);
            SHPService.sendSHPCommand(REFCONST.POST, payload, "/" + $scope.peerId + "/devices/0/fridge/defrostreservations");
        }

        function appendZero(value) {
            value = parseInt(value, 10);
            if (value < 10) {
                return '0' + value;
            }
            return value;
        }

        function change12To24(hours, amPm){
            if((hours+12) == 24) {
                if(amPm == "AM") {
                    return "00";
                } else {
                    return "12";
                }
            }

            if(amPm == "AM") {
                return appendZero(hours);
            } else {
                return (hours+12);
            }
        }

        function increaseMonths() {
            ScheduleService.increaseMonths($scope);
        }

        function decreaseMonths() {
            ScheduleService.decreaseMonths($scope);
        }

        function increaseDays() {
            ScheduleService.increaseDays($scope);
        }

        function decreaseDays() {
            ScheduleService.decreaseDays($scope);
        }

        function stopLongPressEvent() {
            if (!!promise) {
                //check if promise exist. Then call cancel.
                $interval.cancel(promise);
                promise = undefined;
            }
            if ($scope.longPress) {
                $scope.longPress = false;
                return;
            }
        }

        function increaseMonthContinuously() {
            if (!!!promise) {
                promise = $interval(function () {
                    increaseMonths();
                }, 200);
            }
        }

        function decreaseMonthContinuously() {
             if (!!!promise) {
                 promise = $interval(function () {
                    decreaseMonths();
                 }, 200);
             }
        }

        function increaseDayContinuously() {
            if (!!!promise) {
                promise = $interval(function () {
                    increaseDays();
                }, 200);
            }
        }
    
        function decreaseDayContinuously() {
            if (!!!promise) {
                promise = $interval(function () {
                    decreaseDays();
                }, 200);
            }
        }

        function increaseHours() {
            ScheduleService.increaseHours($scope);
        }
        
        function decreaseHours() {
            ScheduleService.decreaseHours($scope);
        }

        function increaseMinutes() {
            ScheduleService.increaseMinutes($scope);
        }

        function decreaseMinutes() {
            ScheduleService.decreaseMinutes($scope);
        }
        
        function changeMeridian() {
            ScheduleService.changeMeridian($scope);
        }

        function increaseHrContinuously() {
            if (!!!promise) {
                promise = $interval(function () {
                    increaseHours();
                }, 200);
            }
        }

        function decreaseHrContinuously() {
            if (!!!promise) {
                promise = $interval(function () {
                    decreaseHours();
                }, 200);
            }
        }

        function increaseMinContinuously() {
            if (!!!promise) {
                promise = $interval(function () {
                    increaseMinutes();
                }, 200);
            }
        }
    
        function decreaseMinContinuously() {
            if (!!!promise) {
                promise = $interval(function () {
                    decreaseMinutes();
                }, 200);
            }
        }
        
     // Control hours value in add schedule screen
        $scope.$watch(function () {
            return $scope.selectedSchedule.hours;
        }, function (newVal, oldVal) {
            if (newVal > 12) {
                $scope.selectedSchedule.hours = oldVal;
            }
            if (newVal === undefined || newVal === 0) {
                $scope.selectedSchedule.hours = oldVal;
            }
        });

        // Control minutes value in add schedule screen
        $scope.$watch(function () {
            return $scope.selectedSchedule.minutes;
        }, function (newVal, oldVal) {
            if (newVal > 59) {
                $scope.selectedSchedule.minutes = oldVal;
            }
            if (newVal === undefined) {
                $scope.selectedSchedule.minutes = oldVal;
            }
        });  
    
        // Go to Settings screen
        function goToSettings() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = REFCONST.SCREENS_SETTINGS;
                changeScreen();
            }, REFCONST.LIST_ANIMATION_TIME);
        }
        
        // Go to About screen
        function goToAbout() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = REFCONST.SCREENS_ABOUT;
                changeScreen();
            }, REFCONST.LIST_ANIMATION_TIME);
        }
        
        // Go to About screen
        function goToErrorPage() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = REFCONST.SCREENS_ERRORPAGE;
                changeScreen();
            }, REFCONST.LIST_ANIMATION_TIME);
        }

        // Go to Settings screen
        function goToSmartCare() {
            $scope.smartCare.started = true;
            $scope.smartCare.viewErros = false;
            $scope.smartCare.detected = false;
            $scope.smartCare.diagnosisStarted = false;
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = REFCONST.SCREENS_SMART_CARE;
                changeScreen();
            }, REFCONST.LIST_ANIMATION_TIME);
        }
        
        //To show scroll
        function scrollStart() {
            document.getElementById("headerDiv").style.overflowY = "auto";
        }
        
        //To hide scroll
        function scrollEnd() {
            document.getElementById("headerDiv").style.overflowY = "scroll";
            
        }
    
        // Change screen
        function changeScreen() {
            $scope.homePage = false;
            $scope.aboutPage = false;
            $scope.settingsPage = false;
            $scope.energyMonitorPage = false;
            $scope.energyManagerPage = false;
            $scope.demandResponsePage = false;
            $scope.defrostPage = false;
            $scope.addSchedulePage = false;
            $scope.addDateSchedulePage = false;
            $scope.addTimeSchedulePage = false;
            $scope.smartCarePage = false;
            $scope.tutorialPage = false; 
            $scope.detailedTutorial = false;
            $scope.errorPage = false;

            $(".scrollable_area").scrollTop(0);
            //if(!$scope.addSchedulePage)
                    //$(".scrollable_area").css("overflow-y", "overlay");

            switch ($scope.currentScreen) {
                case REFCONST.SCREENS_HOME:
                    $scope.homePage = true;
                    break;
                case REFCONST.SCREENS_ABOUT:
                    $scope.aboutPage = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_about_device;
                    break;
                case REFCONST.SCREENS_ERRORPAGE:
                    $scope.errorPage = true;
                    $scope.subHeaderTitle = 'Error';
                    break;
                case REFCONST.SCREENS_SETTINGS:
                    $scope.settingsPage = true;
                    if ($scope.isStatic === true) {                    
                        $scope.isDBDownloaded = true;
                    }
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_settings;
                    break;
                case REFCONST.SCREENS_ENERGY_MONITOR:
                    $scope.energyMonitorPage = true;               
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_energy_monitor_title;
                    break;
                case REFCONST.SCREENS_ENERGY_MANAGER:
                    $scope.energyManagerPage = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ref_energymanager_title;
                    break;
                case REFCONST.SCREENS_DEMAND_RESPONSE:
                    $scope.demandResponsePage = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ref_energymanager_demand_respons_title;
                    break;
                case REFCONST.SCREENS_DEFROST:
                    $scope.defrostPage = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ref_energymanager_defrost_delay_title;
                    break;
                case REFCONST.SCREENS_ADDSCHEDULE:
                    $scope.addSchedulePage = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ref_energymanager_addschedule_title;
                    break;
                case REFCONST.SCREENS_SMART_CARE:
                    $scope.smartCarePage = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ref_smartcare_smartcare;
                    break;
                case REFCONST.SCREENS_ADD_DATE_SCHEDULE:
                    $scope.addDateSchedulePage = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ref_energymanager_addschedule_title;
                    break;
                case REFCONST.SCREENS_ADD_TIME_SCHEDULE:
                    $scope.addTimeSchedulePage = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ref_energymanager_addschedule_title;
                    break;
                case REFCONST.SCREENS_TUTORIAL:
                    $scope.tutorialPage = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_tutorial;
                    document.getElementById("vid").src = "";
                    break;
                case REFCONST.SCREENS_DETAILEDTUTORIAL:
                    $scope.detailedTutorial = true;
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_tutorial;
                    document.getElementById("vid").src = "https://s3.amazonaws.com/smartthingsda/solutionmodule/tutorial/changewaterfilter2.mp4";
                    break;
                default:
                    break;
            }
        }

        /**    	 
         * @function
         * @private
    	 * @name onBackPressed
         * @param {boolean} ignoreAnim ignores adding press Effect for 
         *                  Animation if true. Otherwise, add animation effect.      
    	 * @description Manages mobile hard back key press action.
    	 */
        function onBackPressed (ignoreAnim) { 
            $timeout(function () {
                
                if($scope.showNotificationPopUp || $scope.errorPage) {
                    closeApp();
                }
                
                // uncomment for testing notification popup
                //$scope.showNotificationPopUp = !$scope.showNotificationPopUp;
                //toggleBottomPopUpClass($scope.showNotificationPopUp);
                
                // uncomment for testing Error page
                //goToErrorPage();
                
                if($scope.countrySelectionPopupVisible) {
                    $scope.countrySelectionPopupVisible = false;
                    return;
                }

                if ($scope.connectionPopup) {
                    closeApp();
                    return;
                }
                if ($scope.dialogPopup.show) {
                    $scope.dialogPopup.show = false;
                    return;
                }
                if ($scope.showModuleInfoPopup) {
                    $scope.showModuleInfoPopup = false;
                    return;
                }
                if ($scope.homePage) {
                    closeApp();
                    return;
                } else {
                    switch ($scope.currentScreen) {
                        case REFCONST.SCREENS_ABOUT:
                        case REFCONST.SCREENS_SETTINGS:
                            {
                                toggleAnimClass();
                                $timeout(function () {
                                    toggleAnimClass();
                                    $scope.currentScreen = REFCONST.SCREENS_HOME;
                                    changeScreen();
                                }, REFCONST.LIST_ANIMATION_TIME);
                            }
                            break;
                        case REFCONST.SCREENS_ENERGY_MONITOR:
                        case REFCONST.SCREENS_ENERGY_MANAGER:
                            {
                                toggleAnimClass();
                                $timeout(function () {
                                    toggleAnimClass();
                                    $scope.currentScreen = REFCONST.SCREENS_SETTINGS;
                                    changeScreen();
                                }, REFCONST.LIST_ANIMATION_TIME);
                            }
                            break;
                        case REFCONST.SCREENS_DEMAND_RESPONSE:
                        case REFCONST.SCREENS_DEFROST:
                            {
                                toggleAnimClass();
                                $timeout(function () {
                                    toggleAnimClass();
                                    $scope.currentScreen = REFCONST.SCREENS_ENERGY_MANAGER;
                                    changeScreen();
                                }, REFCONST.LIST_ANIMATION_TIME);
                            }
                            break;
                        case REFCONST.SCREENS_ADDSCHEDULE:
                            {
                                toggleAnimClass();
                                if( $scope.isTimeSelecter){
                                        $scope.isTimeSelecter = false;
                                        $scope.popupBackGround = false;
                                        break;
                                }

                                $timeout(function () {
                                    toggleAnimClass();
                                    $scope.currentScreen = REFCONST.SCREENS_DEFROST;
                                    changeScreen();
                                }, REFCONST.LIST_ANIMATION_TIME);
                            }
                            break;
                        case REFCONST.SCREENS_ADD_DATE_SCHEDULE:
                        case REFCONST.SCREENS_ADD_TIME_SCHEDULE:
                            {
                                    toggleAnimClass();
                                    $timeout(function () {
                                        toggleAnimClass();
                                        $scope.currentScreen = REFCONST.SCREENS_ADDSCHEDULE;
                                        changeScreen();
                                    }, REFCONST.LIST_ANIMATION_TIME);
                            }
                            break;
                        case REFCONST.SCREENS_TUTORIAL:
                        case REFCONST.SCREENS_SMART_CARE:
                            {
                                toggleAnimClass();
                                $timeout(function () {
                                    toggleAnimClass();
                                    $scope.currentScreen = REFCONST.SCREENS_ABOUT;
                                    changeScreen();
                                }, REFCONST.LIST_ANIMATION_TIME);
                            }
                            break;
                        case REFCONST.SCREENS_DETAILEDTUTORIAL:
                            {
                                toggleAnimClass();
                                $timeout(function () {
                                    toggleAnimClass();
                                    $scope.currentScreen = REFCONST.SCREENS_TUTORIAL;
                                    changeScreen();
                                }, REFCONST.LIST_ANIMATION_TIME);
                            }
                            break;
                        case REFCONST.SCREENS_HOME:
                            closeApp();
                            break;
                        default:
                            break;
                    }
                }
            }, REFCONST.BACK_ANIMTION_TIMEOUT);
        }

        /**    	 
         * @function
         * @private
    	 * @name filterNotificationClose        
    	 * @description Manages display of multiple notifications.
    	 */
        function filterNotificationClose() {
            if($scope.filterStatusArr.length > 0) {
                $scope.filterStatusArr.splice(0, 1);
            } 
            if($scope.filterStatusArr.length === 0) {
                $scope.isFilterStatusShow = false;
            }
        }


        $scope.isEnergyDataAvailable = function(isNext) {
            var avail = EnergyService.isEnergyDataAvailable(isNext);
            if(!avail) {
                return false;
            } else {
                return true;
            }
        }
    
        function goToEnergyMonitor() {
            if(isDBDownloadedJS || $scope.isStatic) {
               toggleAnimClass();
               EnergyService.goToEnergyMonitor($scope);
               $timeout(function () {
                   toggleAnimClass();
                   $scope.currentScreen = REFCONST.SCREENS_ENERGY_MONITOR;
                   changeScreen();
               }, REFCONST.LIST_ANIMATION_TIME);
           }
        }

        function createOffset(date) {
            var sign = (date.getTimezoneOffset() > 0) ? "-" : "+";
            var offset = Math.abs(date.getTimezoneOffset());
            var hours = appendZero(Math.floor(offset / 60));
            var minutes = appendZero(offset % 60);
            return sign + hours + ":" + minutes;
        }
        
        function goToEnergyManager() {
            //if(!$scope.isStatic) {

            //}
            if(isDBDownloadedJS && !$scope.isStatic) {
            	//[START]Calculate EnergyConsumption For Dongle 
                var d = new Date();
                var currentTime =  d.getFullYear() + '-' + appendZero(d.getMonth() + 1) + '-' +appendZero(d.getDate()) + 'T' + appendZero(d.getHours()) + ':' + appendZero(d.getMinutes()) + ':' + appendZero(d.getSeconds());

                var _currentYear = d.getUTCFullYear();
                var _month = appendZero(d.getUTCMonth() + 1);
                var _date = appendZero(d.getUTCDate());
                var _hours = appendZero(d.getUTCHours());
                var _minutes = appendZero(d.getUTCMinutes());
                var _seconds = appendZero(d.getUTCSeconds());
                var utcTime = _currentYear + '-' + _month + '-' + _date + 'T' + _hours + ':' + _minutes + ':' + _seconds;
                var tz = d.getTimezoneOffset();
                var tzOffset = createOffset(d);

                // DST Start
                var firstOfMarch = new Date(_currentYear, 2, 1);
                var daysUntilFirstSundayInMarch = (7 - firstOfMarch.getDay()) % 7;
                var secondSundayInMarch = firstOfMarch.getDate() + daysUntilFirstSundayInMarch + 7;
                var dstStartDate = new Date(_currentYear, 2, secondSundayInMarch);
                var dstStartTime = _currentYear + '-03'+ '-' + appendZero(dstStartDate.getUTCDate()) + 'T' + appendZero(dstStartDate.getUTCHours()) + ':' + appendZero(dstStartDate.getUTCMinutes()) + ':00';

                // DST End
                var firstOfNovember = new Date(_currentYear, 10, 1);
                var daysUntilFirstSundayInNov = (7 - firstOfNovember.getDay()) % 7;
                var firstSundayInNovember = firstOfNovember.getDate() + daysUntilFirstSundayInNov;
                var dstEndDate = new Date(_currentYear, 10, firstSundayInNovember);
                var dstEndTime = _currentYear + '-11'+ '-' + appendZero(dstEndDate.getUTCDate()) + 'T' + appendZero(dstEndDate.getUTCHours()) + ':' + appendZero(dstEndDate.getUTCMinutes()) + ':00';

                //var timezone=String(new Date());
                var tzId;
                var tz = jstz.determine();
                if (typeof (tz) === 'undefined') {
                    tzId = 'No timezone found';
                } else {
                    tzId = tz.name();
                }
                console.log("tzId = " + tzId);

                var ds = dstStartDate.getTimezoneOffset();

                var payload = '{"Time": {"currentTime": "'+currentTime+'","tzOffset":"'+tzOffset+'","utcTime":"'+utcTime+'","tzID":"'+tzId+'"}}';
                SHPService.sendSHPCommand(REFCONST.PUT, payload, "/" + $scope.peerId + "/devices/0/configuration/time");

                EnergyService.showMonthlyData($scope);	//energy data get
            	//[END]Calculate EnergyConsumption For Dongle
                
            	SHPService.sendSHPCommand(REFCONST.GET, "/" + $scope.peerId + "/devices/0/energyconsumption", null, false, 300000);
                toggleAnimClass();
                $timeout(function () {
                    toggleAnimClass();
                    $scope.currentScreen = REFCONST.SCREENS_ENERGY_MANAGER;
                    changeScreen();	            
                    setEnergyValue();
                }, REFCONST.LIST_ANIMATION_TIME);
            }
        }

        function goToDemandResponse() {
            //EnergyService.goToEnergyMonitor($scope);
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = REFCONST.SCREENS_DEMAND_RESPONSE;
                changeScreen();
            }, REFCONST.LIST_ANIMATION_TIME);
        }

        function goToDefrost() {
            //EnergyService.goToEnergyMonitor($scope);
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = REFCONST.SCREENS_DEFROST;
                changeScreen();
                addDefrostUserSchedule('start');             
            }, REFCONST.LIST_ANIMATION_TIME);
        }

        function goToAddSchedule() {
            //user schedule full state
            if($scope.defrostDelay.userScheduleDisplay_2 === true && $scope.defrostDelay.userScheduleDisplay_3 === true) {
                    return;
                }

            initAddScheduleDate();

            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = REFCONST.SCREENS_ADDSCHEDULE;
                changeScreen();
            }, REFCONST.LIST_ANIMATION_TIME);
        }

        function goToAddDateSchedule(status) {
            $scope.pickerStatus = status;

            if($scope.pickerStatus === "startDate"){
                    var temp = $scope.displayStartDate.split(" ");
                    $scope.selectedSchedule.month = temp[0];
                    $scope.selectedSchedule.day = temp[1];
            } else if($scope.pickerStatus === "endDate"){
                    var temp = $scope.displayEndDate.split(" ");
                    if(temp[0] === "-") {
                        temp = $scope.displayStartDate.split(" ");
                    }
                    $scope.selectedSchedule.month = temp[0];
                    $scope.selectedSchedule.day = temp[1];
            } else {
                ;//NULL
            }

            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = REFCONST.SCREENS_ADD_DATE_SCHEDULE;
                changeScreen();
            }, REFCONST.LIST_ANIMATION_TIME);
        }

        function goToAddTimeSchedule(status) {
            $scope.pickerStatus = status;

            if ($scope.pickerStatus === "startDate") {
                var temp = $scope.displayStartTime.split(" ");
                var tempTime = temp[0].split(":");

                $scope.selectedSchedule.hours = tempTime[0];
                $scope.selectedSchedule.minutes = tempTime[1];
                $scope.selectedSchedule.amPm = temp[1];
            }

            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = REFCONST.SCREENS_ADD_TIME_SCHEDULE;
                changeScreen();
            }, REFCONST.LIST_ANIMATION_TIME);
        }

        function showDailyData() {
            EnergyService.showDailyData($scope);
        }

        function showWeeklyData() {
            EnergyService.showWeeklyData($scope);
        }

        function showMonthlyData() {
            EnergyService.showMonthlyData($scope);
        }

        function showNextDaysData() {
            EnergyService.showNextDaysData($scope);
        }

        function showPreviousDaysData() {
            EnergyService.showPreviousDaysData($scope);
        }

        function fetchDBfromDevice() {
            if (!$scope.isStatic) {
                nativeInterface.getResourceUrl("/" + $scope.peerId + $scope.Device.energyMonitorLocation);
            }
        }

        function parseEnergyUsageData (energyResponse) {
            EnergyService.parseEnergyUsageData($scope, energyResponse);
        }

        function handleEnergyMonitorErrors() {
            //toaster.success({title: "", body: 'No data available'});
        }

        /* Smart Care*/
        var onSwipe = function() { 
            $timeout(function () {
                $scope.smartCareErrorIndex = mySwipe.getPos();
            }, 0);
        };
        
        function startSmartCare() { 
            $scope.smartCare.started = false;
            $scope.smartCare.detected = false;
            $scope.smartCare.viewErros = false;
            $scope.smartCare.diagnosisStarted = true;
            $scope.smartCare.errors = [];
            if(!$scope.smartCare.detected) {
                $scope.smartCare.totalErrors = 0;
            }

            if (!$scope.isStatic) {
              var payload = '{"Diagnosis" : {"diagnosisStart" : "Run"}}'; 
              $scope.Device.Alarms = [];
              //$scope.checkResponse.push(REFCONST.DIAGNOSIS);
              //SHPService.sendSHPCommand(REFCONST.PUT, payload, "/" + $scope.peerId + "/devices/0/diagnosis", true, 300000);
               nativeInterface.putDataOnResourceUrl(payload, "/" + $scope.peerId + "/devices/0/diagnosis");
            } else { 
                progressSmartCare();
            }
        }

        function viewSmartCareErrors() {
            if($scope.smartCare.totalErrors === 0) {
               return; 
            }
            $timeout(initSwipe, 300);
            $scope.smartCare.started = false;
            $scope.smartCare.detected = false;
            $scope.smartCare.viewErros = true;
            $scope.smartCare.diagnosisStarted = false;
            $scope.smartCareErrorIndex = 0;
        }

        function progressSmartCare() {
            if(!$scope.smartCare.diagnosisStarted) {
                return;
            }
            $scope.smartCare.errors = [];
            if (!$scope.isStatic) { 
                $scope.smartCare.totalErrors = $scope.Device.Alarms.length;

                $scope.Device.Alarms.forEach(function(value) {

                    switch(value.code) {
                        case 'SENSOR_F':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_f","description":"WEBMOB_device_ref_smartcare_sensor_f_msg", isVisible:true});
                            break;
                        case 'SENSOR_R':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r","description":"WEBMOB_device_ref_smartcare_sensor_r_msg", isVisible:true});
                            break;
                        case 'SENSOR_F_DF':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_f_df","description":"WEBMOB_device_ref_smartcare_sensor_f_df_msg", isVisible:true});
                            break;
                        case 'SENSOR_R_DF':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r_df","description":"WEBMOB_device_ref_smartcare_sensor_r_df_msg", isVisible:true});
                            break;
                        case 'SENSOR_TEMP':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_temp","description":"WEBMOB_device_ref_smartcare_sensor_temp_msg", isVisible:true});
                            break;
                        case 'SENSOR_CV':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_cv","description":"WEBMOB_device_ref_smartcare_sensor_cv_msg", isVisible:true});
                            break;
                        case 'SENSOR_CV_DF':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_cv_df","description":"WEBMOB_device_ref_smartcare_sensor_cv_df_msg", isVisible:true});
                            break;
                        case 'PANEL_MAIN':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_panel_main","description":"WEBMOB_device_ref_smartcare_panel_main_msg", isVisible:true});
                            break;
                        case 'BLINKING_F_AT':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_bl_f_at","description":"WEBMOB_device_ref_smartcare_bl_f_at_msg", isVisible:true});
                            break;
                        case 'BLINKING_R_AT':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_bl_r_at","description":"WEBMOB_device_ref_smartcare_bl_r_at_msg", isVisible:true});
                            break;
                        case 'SENSOR_HD':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_hd","description":"WEBMOB_device_ref_smartcare_sensor_hd_msg", isVisible:true});
                            break;
                        case 'SENSOR_R_HD':
                            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r_hd","description":"WEBMOB_device_ref_smartcare_sensor_r_hd_msg", isVisible:true});
                            break;
                        case 'ERROR_14':
                            if($scope.Device.Temperatures[0].unit === 'Celsius') {
                                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_fri_ice_maker","description":"WEBMOB_device_ref_smartcare_error_ice_maker", isVisible:true});
                            }else {
                                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_fri_ice_maker","description":"WEBMOB_device_ref_smartcare_error_ice_maker_f", isVisible:true});
                            }
                            break;
                        case 'ERROR_15':
                            if($scope.Device.Temperatures[0].unit === 'Celsius') {
                                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_ice_room","description":"WEBMOB_device_ref_smartcare_error_ice_room", isVisible:true});
                            }else {
                                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_ice_room","description":"WEBMOB_device_ref_smartcare_error_ice_room_f", isVisible:true});
                            }
                            break;
                        default:
                            $scope.smartCare.totalErrors--;
                    }
                });


            } else {
                $scope.smartCare.totalErrors = 10;
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_f","description":"WEBMOB_device_ref_smartcare_sensor_f_msg", isVisible:true});
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r","description":"WEBMOB_device_ref_smartcare_sensor_r_msg", isVisible:true});
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_f_df","description":"WEBMOB_device_ref_smartcare_sensor_f_df_msg", isVisible:true});
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r_df","description":"WEBMOB_device_ref_smartcare_sensor_r_df_msg", isVisible:true});
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_temp","description":"WEBMOB_device_ref_smartcare_sensor_temp_msg", isVisible:true});
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_cv","description":"WEBMOB_device_ref_smartcare_sensor_cv_msg", isVisible:true});
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_cv_df","description":"WEBMOB_device_ref_smartcare_sensor_cv_df_msg", isVisible:true});
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_panel_main","description":"WEBMOB_device_ref_smartcare_panel_main_msg", isVisible:true});
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_bl_f_at","description":"WEBMOB_device_ref_smartcare_bl_f_at_msg", isVisible:true});
                $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_bl_r_at","description":"WEBMOB_device_ref_smartcare_bl_r_at_msg", isVisible:true});
            }

            $scope.errorString = 'WEBMOB_device_ref_smartcare_smartcarecount';
            if($scope.smartCare.totalErrors === 0) {
                $scope.errorString = 'WEBMOB_device_ref_smartcare_no_error_ref_msg';
            }

            if($scope.smartCare.totalErrors > 1) {
                viewSmartCareErrors();
            } else {
                $scope.smartCare.started = false;
                $scope.smartCare.detected = true;
                $scope.smartCare.viewErros = false;
                $scope.smartCare.diagnosisStarted = false;
            }
        }

        function doAlarmService() {
            serviceCenterNum.forEach(function (value) {
                if (angular.isDefined(value[$scope.selectedCountry])) {
                    $scope.serviceCallNumber = value[$scope.selectedCountry];
                }
            });
            $scope.dialogPopup.show = true;
            toggleBottomPopUpClass(true);
            $scope.dialogPopup.dialogPopupTitle = $scope.translation.WEBMOB_common_service_call;
            $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_common_connect_service_call+"<br>" + ($scope.serviceCenterNumber || "");
            $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_common_cancel;
            $scope.dialogPopup.button2Enabled = true;
            $scope.dialogPopup.button2Title = $scope.translation.WEBMOB_common_ok;
            $scope.dialogPopup.button2Handler = function() {
                $scope.dialogPopup.show = false;
                if($scope.supportCalling) {
                    var payload = '{"selectedCountry" : "' + $scope.selectedCountry + '"}';
                    nativeInterface.runOnNative("callDial", payload);
                }
            }; 
        }

    //    $scope.showConnectionFailurePopup = function() {
    //        $scope.dialogPopup.show = false;
    //        $scope.connectionPopup = true;
    //        dismissLoadingBar('FAIL');
    //        $scope.dialogPopup.show = true;
    //        toggleBottomPopUpClass(true);
    //        $scope.dialogPopup.dialogPopupTitle = $scope.translation.WEBMOB_common_connection_failure;
    //        $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_common_check_power_and_network;
    //        $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_common_ok;
    //        $scope.dialogPopup.button1Handler = function() {
    //            $scope.dialogPopup.show = false;
    //            closeApp();
    //        };
    //        $scope.dialogPopup.button2Enabled = false;
    //    };

        /**    	 
         * @function
         * @private
    	 * @name showConnectionFailurePopup       
         * @description To display connection failure popup, it is called from app.js
    	 */
        function showConnectionFailurePopup() {
            document.getElementById("connFailCancel").style.visibility = 'hidden';
            $scope.showConnFailurePopup = true;
            toggleBottomPopUpClass($scope.showConnFailurePopup);
        };       
        
        /**    	 
         * @function
         * @private
    	 * @name closeConnectionFailurePopup        
         * @description Close connection failure popup, it is called from app.js
    	 */
        function closeConnectionFailurePopup() {
           $scope.showConnFailurePopup = false;
           toggleBottomPopUpClass($scope.showConnFailurePopup);
        }

        /**    	 
         * @function
         * @private
    	 * @name requestFailHandler        
         * @param {String} response
         *       JSON response received in case of request failed
    	 * @description Handling of request failed cases
    	 */
        function requestFailHandler (response) {  	
            var msg;
            if (response["resourceURL"].indexOf("/defrostreservations") !== -1) {
                $timeout(function () {
                    toaster.success({title: "", body: $scope.translation.WEBMOB_common_schedule_save_failed});
                }, 500);

                goToDefrost();

            } else if (response["resourceURL"].indexOf("/actions/") !== -1) {           
                if (response.data.message === 'InvokeDelScheduleFunc') {
                    msg = $scope.translation.WEBMOB_common_schedule_delele_fail_reservation;
                } else if (response.data.message === 'InvokeSetScheduleFunc') {
                    msg = $scope.translation.WEBMOB_common_schedule_duplicate;
                }
                else{//Sonar Issue
                    ;//NULL            	
                }
            } else {

                if(response["status"] === REFCONST.STATUS_507){
                    $timeout(function () {        			
                        toaster.success({title: "", body: $scope.translation.WEBMOB_common_save_failed});
                        dismissLoadingBar(REFCONST.DEFROST_ADD_SCHEDULE);
                    }, 2000);
                    return;
                }
                else {
                    msg = $scope.translation.WEBMOB_common_connection_failure_confirm;
                    $scope.showConnectionFailurePopup();
                }
            }
            //$timeout(function () {
            //    toaster.success({title: "", body: msg});
            //}, 500);
        }
    
        /**    	 
         * @function
         * @private
    	 * @name FUNCTION NAME        
         * @description FUNCTION DESCRIPTION TO BE ADDED HERE.         
    	 */
        function closeApp () {          
            nativeInterface.runOnNative("closeWebApp", "");
        }

        function isEnergyReset() {
            return false;
        }

        /**    	 
         * @function
         * @private
    	 * @name dismissLoadingBar
         * @param {String} value dismissLoadingBar
    	 * @description To dismiss loading bar based on response
    	 */
        function dismissLoadingBar(value) {
            if ($scope.checkResponse.indexOf(value) !== -1) {
                $scope.checkResponse.splice($scope.checkResponse.indexOf(value), 1);
                if ($scope.checkResponse.length === 0) {
                    $scope.loadingBar = false;
                }
            } else {
                if(value === 'FAIL') {
                    $scope.checkResponse = [];
                    $scope.loadingBar = false;
                }
            }
        }

        function setTimeInGMT(utcString) {

            if(utcString !== undefined) {
                var firstSplit = utcString.split("T");
                var dayMinSplit = firstSplit[0].split("-");
                var hourMinSplit = firstSplit[1].split(":");
                var month = parseInt(Number(dayMinSplit[1])-1);
                var date = parseInt(dayMinSplit[2]);
                var hour = parseInt(hourMinSplit[0],10);
                var minute = parseInt(hourMinSplit[1],10);

                var start_date = new Date();
                start_date.setMonth(month, date);		        
                start_date.setHours(hour);
                start_date.setMinutes(minute);
    //	        var newDate = ConvertUTCTimeToLocalTime(start_date, false);

                var newDay = $scope.monthNames[start_date.getMonth()] + " " + start_date.getDate();		        
                var amPm = 'AM';
                var newHour = start_date.getHours();
                if (newHour > 12) {
                    newHour = newHour - 12;
                    amPm = 'PM';
                }
                var newMinutes = start_date.getMinutes();

                newHour = newHour >= 10 ? newHour : '0' + newHour;
                newMinutes = newMinutes >= 10 ? newMinutes : '0' + newMinutes;
                return [newDay, newHour, newMinutes, amPm];
            }
        }

        function ConvertUTCTimeToLocalTime(date, toUTC) {

            var localOffset = date.getTimezoneOffset() * 60000;
            var localTime = date.getTime();
            if (toUTC) {
                date = localTime + localOffset;
            } else {
                date = localTime - localOffset;
            }
            date = new Date(date);        
            return date;
        }


        /**    	 
         * @function
         * @private
    	 * @name FUNCTION NAME
         * @param {PARAM TYPE} popupName
         *       PARAM DESCRIPTION
    	 * @description FUNCTION DESCRIPTION TO BE ADDED HERE.
    	 */
        function toggleBottomPopUpClass(popupName){
            if (popupName) {
                $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivIn";
                $scope.BottomPopupDivCls = "BottomPopupBGIn";
            } else {
                $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivOut";
                $scope.BottomPopupDivCls = "BottomPopupBGOut";
            }
        }
    
        function initSwipe() {
            var elem = document.getElementById('mySwipe'),
                mySwipeWrap,
                cnt,
                len,
                counter = $scope.smartCare.errors.length;
            window.mySwipe = Swipe(elem, {speed:100, continuous:false, callback: onSwipe});
            mySwipeWrap = document.getElementById('mySwipe-wrap');
            len = mySwipeWrap.childNodes.length;
            for (cnt = 0; cnt < len; ++cnt) {
                if (!!mySwipeWrap.childNodes[cnt].className && mySwipeWrap.childNodes[cnt].className === "errorSummary") {
                    if (counter <= 0) {
                        mySwipeWrap.removeChild(mySwipeWrap.childNodes[cnt]);
                        --cnt;
                        --len;
                    } else {
                        --counter;
                    }
                }
            }
        
            mySwipeWrap.style.width = screen.width * $scope.smartCare.errors.length;
            var pageViewBulletsid=document.getElementById('pageViewBulletsid');    
            pageViewBulletsid.style.marginLeft='-' + Math.round(pageViewBulletsid.offsetWidth / 2) + 'px';
            pageViewBulletsid.style.visibility = "visible";
        }
    
        function versionTouchStart() {
            if (!promise) {
                promise = $timeout(function () {
                    if (!$scope.isStatic) {
                        $scope.checkResponse.push(REFCONST.PARSE_MODULEINFO);
                        SHPService.sendSHPCommand(REFCONST.GET, "/" + $scope.peerId + "/devices/0/information");
                    }
                }, 5000);
            }
        }

        function versionTouchEnd() {
            if (promise) {
                $timeout.cancel(promise);
                promise = null;
            }
        }

        function parseModuleInformation(moduleInfo) {
            if (angular.isDefined(moduleInfo.Versions) && moduleInfo.Versions.length > 0) {
                $scope.moduleInfoText = "Software Ver : " + moduleInfo.Versions[0].number + " <br> Firmware Ver : " + moduleInfo.Versions[1].number;
                dismissLoadingBar(REFCONST.PARSE_MODULEINFO);
                document.getElementById("moduleInfoCancelID").style.visibility = 'hidden';
                $scope.showModuleInfoPopup = true;
                toggleBottomPopUpClass($scope.showModuleInfoPopup);
            }
        }
        
        function moduleInfoOk() {
            $scope.showModuleInfoPopup = false;
            toggleBottomPopUpClass($scope.showModuleInfoPopup);
        }
        
        function increaseScheduleInterval() {
            //ScheduleService.increaseScheduleInterval($scope);
            $scope.schedulePeriod = Number($scope.schedulePeriod);
            // defined steps as 1
            var step = 1;
            if ($scope.schedulePeriod !== (5 - step)) {
                $scope.schedulePeriod += step;
            } else {
                $scope.schedulePeriod = 1;
            }
            if(document.getElementById("shedulePeriodID")) {
                document.getElementById("shedulePeriodID").value = $scope.schedulePeriod;
            }
        }

        function decreaseScheduleInterval() {
            //ScheduleService.decreaseScheduleInterval($scope);
            $scope.schedulePeriod = Number($scope.schedulePeriod);
            var step = 1;
            if ($scope.schedulePeriod !== 1) {
                $scope.schedulePeriod -= step;
            } else {
                $scope.schedulePeriod = 4;
            }
            if(document.getElementById("shedulePeriodID")) {
                document.getElementById("shedulePeriodID").value = $scope.schedulePeriod;
            }
        }

        function increaseScheduleIntervalContinuously() {
            if (!!!promise) {
                promise = $interval(function () {
                    increaseScheduleInterval();
                }, 200);
            }
        }
    
        function decreaseScheduleIntervalContinuously() {
            if (!!!promise) {
                promise = $interval(function () {
                    decreaseScheduleInterval();
                }, 200);
            }
        }
        
        function onscheduleIntervalLostFocus() {
            console.log("$scope.schedulePeriod :: "+ $scope.schedulePeriod);
            if ($scope.schedulePeriod === undefined || $scope.schedulePeriod === "" || $scope.schedulePeriod < 1) {
                $scope.schedulePeriod = 1;
            }
            
            if($scope.schedulePeriod > 4) {
                $scope.schedulePeriod = 4;
            }
            document.getElementById("shedulePeriodID").value = $scope.schedulePeriod;
        }
         
        function inputKeyDown(e) {
            debugMessage("e.keycode :: "+e.keyCode);
            if(e.keyCode !== 8 && e.keyCode !== 49 && e.keyCode !== 50 && e.keyCode !== 51 && e.keyCode !== 52) {
                e.preventDefault();
            }
        }
        
        function parsePowerConsumption(powerConsumption) {
            debugMessage("parsePowerConsumption");
            if (angular.isDefined(powerConsumption["cumulativeConsumption"])) {
            	$scope.Device.cumulativeConsumption = powerConsumption["cumulativeConsumption"]/10;
            }
            else {
            	$scope.Device.cumulativeConsumption = 0;
            }
            if (angular.isDefined(powerConsumption["monthlyConsumption"])) {
            	$scope.Device.monthlyConsumption = powerConsumption["monthlyConsumption"]/10;
            } else {
            	$scope.Device.monthlyConsumption = 0;
            }
            
            if (angular.isDefined(powerConsumption["saveLocation"])) {
            	$scope.Device.energyMonitorLocation = powerConsumption["saveLocation"];
            }
            
            $scope.energyManager.thisMonthEnergy = $scope.Device.cumulativeConsumption.toFixed(1);
            $scope.energyManager.lastMonthEnergy = $scope.Device.monthlyConsumption.toFixed(1);
	        
            console.log("cumulativeConsumption : " + $scope.energyManager.thisMonthEnergy);
            console.log("monthlyConsumption : " + $scope.energyManager.lastMonthEnergy + " " + angular.isString($scope.energyManager.lastMonthEnergy));
        }
        
        function startEnergyConsumptionTimer() {
            if (angular.isDefined(EnergyConsumptionInterval)) {
                //check if EnergyConsumptionInterval exist. Then call cancel.
                $interval.cancel(EnergyConsumptionInterval);
                EnergyConsumptionInterval = undefined;
            }
            if (!angular.isDefined(EnergyConsumptionInterval)) {
            	EnergyConsumptionInterval = $interval(function () {
                	console.log("~getEnergyConsumption~");
                	console.log("$scope.checkResponse: "+ $scope.checkResponse);
                	if($scope.checkResponse.length === 0){
	                	nativeInterface.getResourceUrl("/" + $scope.peerId + "/devices/0/energyconsumption");
	                	$scope.fetchDBfromDevice();
                	}
                }, 300000);
            }
        }
        
        function showOptionPopUp() {
            $scope.bOptionPopUp = true;
        }

        function closeOptionPopUp() {
            $scope.bOptionPopUp = false;
        }
        
        function selectIndexFromOptionMenu(optionMenuindex) {
            switch(optionMenuindex) {
                case 0:
                    goToSettings();
                    break;
                case 1:
                    goToAbout();
                    break;
                default:
                    break;
            }
            closeOptionPopUp();
        }
        
        function errorVisibilityToggle(index) {
            $scope.smartCare.errors[index].isVisible = !$scope.smartCare.errors[index].isVisible;
        }
    }
