// sample: http://codepen.io/ktknest/full/LDljw/

angular.module('pinch-zoom', [])
/**
 * @ngdoc directive
 * @name pinchZoom
 * @restrict A
 * @scope false
 *
 * @description
 * 要素のピンチアウト・インでの拡大・縮小、拡大時のスワイプ動作を提供
 *
 **/
.directive('pinchZoom', function() {
  var _directive =  {
    restrict : 'A',
    scope    : false,
    link     : _link
  };

  function _link(scope, element, attrs) {
    var elWidth = element[0].offsetWidth;
    var elHeight = element[0].offsetHeight;
    // モード（pinch or swipe）
    var mode = '';

    // ピンチ時の2点間の距離
    var distance = 0;
    var initialDistance = 0;

    // 拡大率
    var scale = 1;
    var relativeScale = 1;
    var initialScale = 1;
    var MAX_SCALE = 1.5;

    // 要素の左上端の座標
    var positionX = 0;
    var positionY = 0;

    var initialPositionX = 0;
    var initialPositionY = 0;

    // ピンチ時の中心座標
    var originX = 0;
    var originY = 0;

    // スワイプ時のスタート座標・移動量
    var startX = 0;
    var startY = 0;
    var moveX = 0;
    var moveY = 0;
    var pageX = 0;
    var pageY = 0;
    var scrollHomePage = document.getElementById('scrollable_homePage');
    var transition  = 'transform cubic-bezier(0,0,.5,1) 1s';
    
    element.css({
      '-webkit-transform-origin' : '0 0',
      'transform-origin'         : '0 0'
    });
    
    element.on('load', function () {
        elWidth = element[0].clientWidth;
        elHeight = element[0].clientHeight;
    });
    
    element.on('touchstart', function(evt) {
        if (evt.touches.length === 1 && scale > 1) {
            if (!(window.navigator.platform === 'iPhone' || window.navigator.platform === 'iPad')) {
                scrollHomePage.style.overflowY = "hidden"; 
            }
            evt.stopImmediatePropagation();
        }
      startX = evt.touches[0].pageX;
      startY = evt.touches[0].pageY;
      initialPositionX = positionX;
      initialPositionY = positionY;
      moveX = 0;
      moveY = 0;
      mode = '';
      if (evt.touches.length === 2) {
        initialScale = scale;
        pageX = evt.touches[0].pageX - evt.touches[1].pageX;
        pageY = evt.touches[0].pageY - evt.touches[1].pageY;
        initialDistance = getDistance(evt);
        originX = evt.touches[0].pageX -
                 parseInt((evt.touches[0].pageX - evt.touches[1].pageX) / 2, 10) -
                 element[0].offsetLeft - initialPositionX;
        originY = evt.touches[0].pageY -
                 parseInt((evt.touches[0].pageY - evt.touches[1].pageY) / 2, 10) -
                 element[0].offsetTop - initialPositionY;
      }
    });

    element.on('touchmove', function(evt) {
        
      if (mode === 'swipe' && scale > 1) {
        if (!(window.navigator.platform === 'iPhone' || window.navigator.platform === 'iPad')) {
                scrollHomePage.style.overflowY = "hidden";
        }
        
        
        // If tag is clicked to move
        var ele = evt.srcElement;
        
        if($(ele).hasClass("expiryDigitContainer") || $(ele).hasClass("tag_element") || $(ele).hasClass("tag_element_scale") || $(ele).hasClass("expiryDayText")){
            evt.preventDefault(); 
            evt.stopImmediatePropagation();
            
            /*
             * Move Glaze screen opposite to the tag drag direction - Commented till further confirmation
             **/
//            moveX = evt.touches[0].pageX - startX; //if moveX traveled is negative, it indicates left swipe
//            moveY = evt.touches[0].pageY - startY; //if moveY traveled is negative, it indicates up swipe
//
//            positionX = initialPositionX - moveX;
//            positionY = initialPositionY - moveY;
//
//            if (positionX > 0) {
//                positionX = 0;
//            } else if (positionX < elWidth * (1 - scale)) {
//                positionX = elWidth * (1 - scale);
//            }
//            if (positionY > 0) {
//                positionY = 0;
//            } else if (positionY < elHeight * (1 - scale)) {
//                positionY = elHeight * (1 - scale);
//            }
//            transformElement();
            return;
        }else if($(ele).hasClass("refrig_img1_reminder")|| $(ele).hasClass("refrig_img2_reminder") || $(ele).hasClass("refrig_img3_reminder")){
            evt.preventDefault(); 
            moveX = evt.touches[0].pageX - startX;
            moveY = evt.touches[0].pageY - startY;

            positionX = initialPositionX + moveX;
            positionY = initialPositionY + moveY;

            if (positionX > 0) {
              positionX = 0;
            } else if (positionX < elWidth * (1 - scale)) {
              positionX = elWidth * (1 - scale);
            }
            if (positionY > 0) {
              positionY = 0;
            } else if (positionY < elHeight * (1 - scale)) {
              positionY = elHeight * (1 - scale);
            }
            transformElement();
        }

      } else if (mode === 'pinch') {
        dragCheck = true;
        evt.preventDefault(); 
        distance = getDistance(evt);
        relativeScale = distance / initialDistance;
        scale = relativeScale * initialScale;
        
        if (scale <= 1) {

            scale = 1;
            scaleFactor =   scale;
            positionX = 0;
            positionY = 0;
            transformElement(1);
//            var x = 1;
//            $(".food_reminder_refrigerator_tag").find(".tag_element").css({'transform': 'scale('+x+')'});
//            $(".food_reminder_refrigerator_tag").find(".tag_element").css({'transition': transition});
//          
//            $(".food_reminder_refrigerator_tag").find(".tag_element_scale").css({'transform': 'scale('+x+')'});
//            $(".food_reminder_refrigerator_tag").find(".tag_element_scale").css({'transition': transition});

        }else{
            scaleFactor = (scale > MAX_SCALE)?MAX_SCALE:scale;
            positionX = originX * (1 - relativeScale) + initialPositionX + moveX;
            positionY = originY * (1 - relativeScale) + initialPositionY + moveY;
            transformElement(1);
            //var transition  = 'transform cubic-bezier(0,0,.5,1) 1s';
//            var x = 1/scaleFactor;
//            $(".food_reminder_refrigerator_tag").find(".tag_element").css({'transform': 'scale('+x+')'});
//            $(".food_reminder_refrigerator_tag").find(".tag_element").css({'transition': transition});
//            
//            $(".food_reminder_refrigerator_tag").find(".tag_element_scale").css({'transform': 'scale('+x+')'});
//            $(".food_reminder_refrigerator_tag").find(".tag_element_scale").css({'transition': transition});
            }
        } else {
            if (evt.touches.length === 1) {
              mode = 'swipe';
            } else if (evt.touches.length === 2) {
              mode = 'pinch';
            }
        }
      
      //transformElement();
    });

    element.on('touchend', function(evt) {
        if (!(window.navigator.platform === 'iPhone' || window.navigator.platform === 'iPad')) {
            scrollHomePage.style.overflowY = "overlay";
        }
          
        if (mode === 'pinch') {

        if (scale <= 1) {
          
          scale = 1;
          scaleFactor =   scale;
          positionX = 0;
          positionY = 0;

        } else if (scale > MAX_SCALE) {
            
          scale = MAX_SCALE;
          scaleFactor =   scale;
          relativeScale = scale / initialScale;
          positionX = originX * (1 - relativeScale) + initialPositionX + moveX;
          positionY = originY * (1 - relativeScale) + initialPositionY + moveY;

        }

      }

      if (scale > 1) {
        if (positionX > 0) {
          positionX = 0;
        } else if (positionX < elWidth * (1 - scale)) {
          positionX = elWidth * (1 - scale);
        }
        if (positionY > 0) {
          positionY = 0;
        } else if (positionY < elHeight * (1 - scale)) {
          positionY = elHeight * (1 - scale);
        }

      }
      transformElement(0.1);
      dragCheck = false;
    });

    function getDistance(evt) {
        var d = 0;
        if(evt.touches[0] !== undefined && evt.touches[1] !== undefined){
             pageX = evt.touches[0].pageX - evt.touches[1].pageX;
             pageY = evt.touches[0].pageY - evt.touches[1].pageY;
            d = Math.sqrt(Math.pow(pageX, 2) +
                Math.pow(pageY, 2));
        }else{
            d = Math.sqrt(Math.pow(pageX, 2) +
                Math.pow(pageY, 2));
        }
    return parseInt(d, 10);
    }

    function transformElement(duration) {
        if(scale > MAX_SCALE || scale < 1){
            return;
        }
        transition  = duration ? 'transform cubic-bezier(0,0,.5,1) ' + duration + 's' : '',
          matrixArray = [scale, 0, 0, scale, positionX, positionY],
          matrix      = 'matrix(' + matrixArray.join(',') + ')';

      element.css({
        '-webkit-transition' : transition,
        'transition'         : transition,
        '-webkit-transform'  : matrix + ' translate3d(0,0,0)',
        'transform'          : matrix
      });
    }
  }
  return _directive;
});