/*  
 * cooktop.js
 * 
 * Copyright ( c ) 2016 by Samsung Electronics, Inc.,
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung.
 *
 * Created on : Wed Apr 24 17:00:00 KST 2016
 * Owner      : SRI
 * Author     : namit.bansal@samsung.com
 */

var app = angular.module("SmartThings", ['toaster', 'ngSanitize', 'ui.router']);

app.config(['$stateProvider', function ($stateProvider) {
        var MainLoadingPage = {
            name: 'MainLoadingPage',
            url: '/',
            templateUrl: 'MainLoadingScreen.html'
        },
        BurnerHomePage = {
            name: 'BurnerHomePage',
            url: '/',
            templateUrl: 'BurnerHomeScreen.html'
//            templateUrl: 'views/HomePageScreen.html'
        },
        KitchenTimerPage = {
            name: 'KitchenTimerPage',
            url: '/',
            templateUrl: 'KitchenTimerScreen.html'
        },
        HoodTimerPage = {
            name: 'HoodTimerPage',
            url: '/',
            templateUrl: 'HoodTimerScreen.html'
        },
        SettingPage = {
            name: 'SettingPage',
            url: '/',
            templateUrl: 'SettingScreen.html'
        },
        AboutPage = {
            name: 'AboutPage',
            url: '/',
            templateUrl: 'AboutScreen.html'
        },
        SmartCarePage = {
            name: 'SmartCarePage',
            url: '/',
            templateUrl: 'SmartCareScreen.html'
        },
        SmartCareNoErrorPage = {
            name: 'SmartCareNoErrorPage',
            url: '/',
            templateUrl: 'SmartCareNoErrorScreen.html'
        },
        ErrorPage = {
            name: 'ErrorPage',
            url: '/',
            templateUrl: 'ErrorScreen.html'
        };

        $stateProvider.state(MainLoadingPage);
        $stateProvider.state(BurnerHomePage);
        $stateProvider.state(KitchenTimerPage);
        $stateProvider.state(HoodTimerPage);
        $stateProvider.state(SettingPage);
        $stateProvider.state(AboutPage);
        $stateProvider.state(SmartCarePage);
        $stateProvider.state(SmartCareNoErrorPage);
        $stateProvider.state(ErrorPage);

    }]);

app.run(['$state', function ($state) {
        $state.transitionTo('MainLoadingPage');
    }]);

app.controller("cooktopAppController", ['$scope', '$state', 'SHPService', 'toaster', '$timeout', '$interval', cooktopAppController]);

function cooktopAppController($scope, $state, SHPService, toaster, $timeout, $interval) {
    var langArr = Object.keys(LANGUAGE_FILE.language), Debug_tag = 'CT : ', justFirstTime = true, configTimer = null, Sec20Call = 20000, promiseAbout,
            bridgeSelected = 'off', errorRemoved = true, changeSmartCarePage = false, doNotChangeScreen = false, scroll_middle_area = document.getElementById("scrollable_area_others"), parseNotification = false,
            kitchenTimerDrag = $("#kitchenTimerDrag"), hoodTimerDrag = $("#hoodTimerDrag"), roundShape;

    var COOKTOP_SCREENS = {
        COOKTOP_MAIN_LOADING_PAGE: 1,
        COOKTOP_BURNER_HOME_PAGE: 2,
        COOKTOP_KITCHEN_TIMER_PAGE: 3,
        COOKTOP_HOOD_TIMER_PAGE: 4,
        COOKTOP_SETTING_PAGE: 5,
        COOKTOP_ABOUT_PAGE: 6,
        COOKTOP_ERROR_PAGE: 7,
        WALLOVEN_SMART_CARE_PAGE: 8,
        WALLOVEN_SMART_CARE_NO_ERROR_PAGE: 9,
    },
            COOKTOP_COMMANDS = {
                "hoodTimer": "hoodTimer",
                "hoodPower": "hoodPower",
                "hoodLight": "hoodLight",
                "hoodFan": "hoodFan",
                "getDevices": "getDevices",
                "runOnNative": "runOnNative",
                "kitchenTimer": "kitchenTimer",
                "showModuleVersion": "showModuleVersion",
                "smartCareDiagnosis": "smartCareDiagnosis"
            },
    COOKTOP_ALERT_STATE = {
        ALERT_NONE: 0,
        ALERT_START_COOKING: 1,
        ALERT_CANCEL_COOKING: 2,
        ALERT_CONNECTION_ERROR: 3,
        ALERT_DEVELOPER_INFO: 4,
        ALERT_TIMER_FINISH: 5,
        ALERT_CHILD_LOCK: 6,
        ALERT_HOOD_TIMER_FINISH: 7,
        ALERT_SERVICE_CALL: 8,
    };

    if (langArr.indexOf(LANG.toUpperCase()) !== -1) {
        $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase()];
    } else if (langArr.indexOf(LANG.toUpperCase().substring(0, 2)) !== -1) {
        $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase().substring(0, 2)];
    } else {
        $scope.translation = LANGUAGE_FILE["language"]["DEFAULT"];
    }
    $scope.isSpanishLanguage = LANG.toUpperCase().substring(0, 2) === "ES";
    $scope.isFrenchLanguage = LANG.toUpperCase().substring(0, 2) === "FR";
    $scope.switchON = 'I';
    $scope.switchOff = 'O';
    if (LANG.toUpperCase().indexOf('EN') !== -1) {
        $scope.switchON = 'ON';
        $scope.switchOff = 'OFF';
    }

    loadVariables();

    function isTextInput(node) {
        return ['INPUT', 'TEXTAREA'].indexOf(node.nodeName) !== -1;
    }

    document.addEventListener('touchend', function (e) {
        if (!isTextInput(e.target) && isTextInput(document.activeElement)) {
            document.activeElement.blur();
        }
    }, false);

    function loadVariables() {
        $scope.isStatic = false;
        $scope.isAppV2 = false;
        $scope.hoodOff = false;
        $scope.showHood = false;
        $scope.hoodLight = false;
        $scope.loadingBar = false;
        $scope.showSmartCare = false;
        $scope.isTempFocusOn = false;
        $scope.ch0FlexUpperOn = false;
        $scope.ch0FlexLowerOn = false;
        $scope.ch1FlexUpperOn = false;
        $scope.ch1FlexLowerOn = false;
        $scope.cTopTabSelected = true;
        $scope.appAtForeground = false;
        $scope.showKitchenTimer = false;
        $scope.headerBarVisible = false;
        $scope.TabsVisibleAtPage = false;
        $scope.mainLoadingScreen = false;
        $scope.moreOptionVisible = false;
        $scope.showFullScreenPopup = false;
        $scope.showMenuOptionPopup = false;
        $scope.remoteControlEnabled = false;
        $scope.cookTopBurnerPause = false;

        $scope.checkResponse = [];
        $scope.runningBurner = [];

        $scope.kitchenTimer = 0;
        $scope.hoodTimer = 0;
        $scope.selectedFanValue = 1;
        $scope.flexCoilSelectedValue = -1;

        $scope.cookTopType = '1';
        $scope.fanStatus = 'low';
        $scope.startStopState = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_start;
        $scope.selectedBurnerDetail = '';
        $scope.KitchenTimerState = 'Ready';
        $scope.hoodLightPosition = 'off';
        $scope.timerText = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step1;
        $scope.timerStopText = $scope.translation.WEBMOB_device_hood_set_time_guide4;
        $scope.cookTopBaseImage = 'images/cooktop/burner/ct_ect';
        $scope.cookTopBurnerBridge = 'images/cooktop/burner/ct_ect_burner_sync_off';
        $scope.currentScreen = COOKTOP_SCREENS.COOKTOP_MAIN_LOADING_PAGE;

        $scope.init = init;
        $scope.closeApp = closeApp;
        $scope.onBackPressed = onBackPressed;
        $scope.parseStaticData = parseStaticData;
        $scope.parseDeviceData = parseDeviceData;
        $scope.handleStatusCode = handleStatusCode;
        $scope.updateDeviceData = updateDeviceData;
        $scope.onBackPressedEvent = onBackPressedEvent;
        $scope.parseConfiguration = parseConfiguration;
        $scope.requestFailHandler = requestFailHandler;
        $scope.parseNotificationData = parseNotificationData;
        $scope.parseModuleInformation = parseModuleInformation;
        $scope.showConnectionFailurePopup = showConnectionFailurePopup;
        $scope.closeConnectionFailurePopup = closeConnectionFailurePopup;

        $scope.goToAbout = goToAbout;
        $scope.isModelAA34 = isModelAA34;
        $scope.cTopTabClick = cTopTabClick;
        $scope.goToSettings = goToSettings;
        $scope.selectBurner = selectBurner;
        $scope.inputKeyDown = inputKeyDown;
        $scope.addHoodTimer = addHoodTimer;
        $scope.onAboutAction = onAboutAction;
        $scope.hoodFanButton = hoodFanButton;
        $scope.gotoHoodTimer = gotoHoodTimer;
        $scope.hideDummyInbox = hideDummyInbox;
        $scope.showDummyInbox = showDummyInbox;
        $scope.hideMenuOption = hideMenuOption;
        $scope.showMenuOption = showMenuOption;
        $scope.resetHoodTimer = resetHoodTimer;
        $scope.startHoodTimer = startHoodTimer;
        $scope.hoodLightButton = hoodLightButton;
        $scope.addKitchenTimer = addKitchenTimer;
        $scope.versionTouchEnd = versionTouchEnd;
        $scope.hoodPowerButton = hoodPowerButton;
        $scope.gotoKitchenTimer = gotoKitchenTimer;
        $scope.updateTimerValue = updateTimerValue;
        $scope.resetKitchenTimer = resetKitchenTimer;
        $scope.startKitchenTimer = startKitchenTimer;
        $scope.versionTouchStart = versionTouchStart;
        $scope.onFlexCoilClicked = onFlexCoilClicked;
        $scope.onAlertButton1Click = onAlertButton1Click;
        $scope.onAlertButton2Click = onAlertButton2Click;
        $scope.smartcareerrorcheck = smartcareerrorcheck;
        $scope.doOnServiceCallPopup = doOnServiceCallPopup;
        $scope.updateHoodTimerValue = updateHoodTimerValue;
        $scope.doValidationForTimer = doValidationForTimer;
        $scope.doValidationForHoodTimer = doValidationForHoodTimer;
    }

    function debugMessage(msg) {
        if (false) {
            console.log(Debug_tag + ": " + msg + '  >> called from ' + arguments.callee.caller.name);
        }
    }

    function init() {
        debugMessage('init called ' + $scope.appAtForeground + justFirstTime);
        if (!$scope.appAtForeground && justFirstTime) {
            $scope.cookTopInfo = {
                Alarms: [{
                        id: "0",
                        alarmType: "Device",
                        code: "NoPanDetect0",
                        triggeredTime: "2015-11-18T11:03:25"
                    }, {
                        id: "1",
                        alarmType: "Device",
                        code: "NoPanDetect1",
                        triggeredTime: "2015-11-18T11:03:25"
                    }, {
                        id: "2",
                        alarmType: "Device",
                        code: "NoPanDetect2",
                        triggeredTime: "2015-11-18T11:03:25"
                    }, {
                        id: "3",
                        alarmType: "Device",
                        code: "NoPanDetect3",
                        triggeredTime: "2015-11-18T11:03:25"
                    }, {
                        id: "4",
                        alarmType: "Device",
                        code: "NoPanDetect4",
                        triggeredTime: "2015-11-18T11:03:25"
                    }, {
                        id: "5",
                        alarmType: "Device",
                        code: "NoPanDetect5",
                        triggeredTime: "2015-11-18T11:03:25"
                    }],
                ConfigurationLink: {
                    href: "/devices/0/configuration"
                },
                Diagnosis: {
                    diagnosisStart: "Ready"
                },
                InformationLink: {
                    href: "/devices/0/information"
                },
                Mode: {
                    options: [{
                            DeviceType: "NV9300K-/AA4",
                            OperationType: "0",
                            FanSpeed: 0,
                            Lamp: false,
                            MainTimerCurrent: 0,
                            MainTimerSet: 0,
                            MainTimerState: 'Ready',
                            Pause: "0",
                            SyncFlex: 'Off',
                            FlexCoilValue: 0,
                            PanSize0: "Not_Supported",
                            PanSize1: "Not_Supported",
                            PanSize2: "Not_Supported",
                            PanSize3: "Not_Supported",
                            PanSize4: "Not_Supported",
                            PanSize5: "Not_Supported",
                            OperationState0: "Ready",
                            OperationState1: "Ready",
                            OperationState2: "Ready",
                            OperationState3: "Ready",
                            OperationState4: "Ready",
                            OperationState5: "Ready",
                            HotSurface0: "0",
                            HotSurface1: "0",
                            HotSurface2: "0",
                            HotSurface3: "0",
                            HotSurface4: "0",
                            HotSurface5: "0",
                            PowerLevel0: "0",
                            PowerLevel1: "0",
                            PowerLevel2: "0",
                            PowerLevel3: "0",
                            PowerLevel4: "0",
                            PowerLevel5: "0",
                            TimerState0: "Ready",
                            TimerState1: "Ready",
                            TimerState2: "Ready",
                            TimerState3: "Ready",
                            TimerState4: "Ready",
                            TimerState5: "Ready",
                            DownCoutTimerCurrent0: "0",
                            DownCoutTimerCurrent1: "0",
                            DownCoutTimerCurrent2: "0",
                            DownCoutTimerCurrent3: "0",
                            DownCoutTimerCurrent4: "0",
                            DownCoutTimerCurrent5: "0",
                            DownCoutTimerSet0: "0",
                            DownCoutTimerSet1: "0",
                            DownCoutTimerSet2: "0",
                            DownCoutTimerSet3: "0",
                            DownCoutTimerSet4: "0",
                            DownCoutTimerSet5: "0"}]
                },
                Operation: {
                    kidsLock: "Ready",
                    power: "On"
                },
                connected: true,
                description: "TP6X_CT_16K",
                id: "0",
                name: "CookTop",
                resources: ["Alarms",
                    "Configuration",
                    "Diagnosis",
                    "Doors",
                    "Information",
                    "Mode",
                    "Operation",
                    "Oven",
                    "Temperatures"],
                type: "CookTop",
                uuid: "F8042EEB-AD39-0000-0000-000000000000"
            };
            $scope.HoodInfo = {Alarms: [{
                        id: "0",
                        alarmType: "",
                        code: "",
                        triggeredTime: ""
                    }],
                ConfigurationLink: {
                    href: "/devices/1/configuration"
                },
                Diagnosis: {
                    diagnosisStart: "Ready"
                },
                InformationLink: {
                    href: "/devices/1/information"
                },
                Mode: {
                    options: [{
                            DeviceType: "NV9300K-/AA4",
                            FanSpeed: 0,
                            Bluetooth: 'On',
                            Lamp: false,
                            BTState: 15,
                            AutoMode: 'On',
                            ShutOffTimerSet: 0,
                            ShutOffTimerCurrent: 0,
                            ShutOffTimerState: 'Ready'}]
                },
                Operation: {
                    power: "Off"
                },
                connected: false,
                description: "TP6X_CT_16K_HOOD",
                id: "1",
                name: "Hood",
                resources: ["Alarms",
                    "Configuration",
                    "Diagnosis",
                    "Doors",
                    "Information",
                    "Mode",
                    "Operation"],
                type: "CookTop",
                uuid: "F8042EEB-AD39-0000-0000-000000000001"
            };
            $scope.cookTopBurnerArray = [{
                    id: 0,
                    AlarmCode: 'NoPanDetect0',
                    PanSize: 'Not_Supported',
                    OperationState: 'Ready',
                    HotSurface: '',
                    PowerLevel: '0',
                    TimerState: 'Ready',
                    DownCoutTimerCurrent: '0',
                    DownCoutTimerSet: '0',
                    BurnerImageID: 'burner_circle_08',
                    subBurnerId: '',
                    SubBurnerSupport: false,
                    ImageOff: true,
                    ImageOn: false,
                    ImageSelected: false,
                    CenterText: '',
                    UpperText: '',
                    LowerText: ''
                }, {
                    id: 1,
                    AlarmCode: 'NoPanDetect1',
                    PanSize: 'Not_Supported',
                    OperationState: 'Ready',
                    HotSurface: '',
                    PowerLevel: '0',
                    TimerState: 'Ready',
                    DownCoutTimerCurrent: '0',
                    DownCoutTimerSet: '0',
                    BurnerImageID: 'burner_circle_08',
                    subBurnerId: '',
                    SubBurnerSupport: false,
                    ImageOff: true,
                    ImageOn: false,
                    ImageSelected: false,
                    CenterText: '',
                    UpperText: 'ON',
                    LowerText: 'OFF'
                }, {
                    id: 2,
                    AlarmCode: 'NoPanDetect2',
                    PanSize: 'Not_Supported',
                    OperationState: 'Ready',
                    HotSurface: '',
                    PowerLevel: '0',
                    TimerState: 'Ready',
                    DownCoutTimerCurrent: '0',
                    DownCoutTimerSet: '0',
                    BurnerImageID: 'burner_circle_10',
                    subBurnerId: '',
                    SubBurnerSupport: false,
                    ImageOff: true,
                    ImageOn: false,
                    ImageSelected: false,
                    CenterText: '',
                    UpperText: '',
                    LowerText: ''
                }, {
                    id: 3,
                    AlarmCode: 'NoPanDetect3',
                    PanSize: 'Not_Supported',
                    OperationState: 'Ready',
                    HotSurface: '',
                    PowerLevel: '0',
                    TimerState: 'Ready',
                    DownCoutTimerCurrent: '0',
                    DownCoutTimerSet: '0',
                    BurnerImageID: 'burner_circle_10',
                    subBurnerId: '',
                    SubBurnerSupport: false,
                    ImageOff: true,
                    ImageOn: false,
                    ImageSelected: false,
                    CenterText: '',
                    UpperText: '',
                    LowerText: ''
                }, {
                    id: 4,
                    AlarmCode: 'NoPanDetect4',
                    PanSize: 'Not_Supported',
                    OperationState: 'Ready',
                    HotSurface: '',
                    PowerLevel: '0',
                    TimerState: 'Ready',
                    DownCoutTimerCurrent: '0',
                    DownCoutTimerSet: '0',
                    BurnerImageID: 'burner_circle_10',
                    subBurnerId: '',
                    SubBurnerSupport: false,
                    ImageOff: true,
                    ImageOn: false,
                    ImageSelected: false,
                    CenterText: '',
                    UpperText: '',
                    LowerText: ''
                }, {
                    id: 5,
                    AlarmCode: 'NoPanDetect5',
                    PanSize: 'Not_Supported',
                    OperationState: 'Ready',
                    HotSurface: '',
                    PowerLevel: '0',
                    TimerState: 'Ready',
                    DownCoutTimerCurrent: '0',
                    DownCoutTimerSet: '0',
                    BurnerImageID: 'burner_circle_10',
                    subBurnerId: '',
                    SubBurnerSupport: false,
                    ImageOff: true,
                    ImageOn: false,
                    ImageSelected: false,
                    CenterText: '',
                    UpperText: '',
                    LowerText: ''
                }];
            $scope.flexCoilStatus = [
                {FlexOn: false, UpperText: '', CenterText: '', LowerText: ''},
                {FlexOn: false, UpperText: '', CenterText: '', LowerText: ''},
                {FlexOn: false, UpperText: '', CenterText: '', LowerText: ''},
                {FlexOn: false, UpperText: '', CenterText: '', LowerText: ''}
            ];
        }
    }

    function updateDeviceData(data) {
        //debugMessage("updateDeviceData " + data);
        SHPService.updateDeviceData($scope, data);
    }

    function configurationCall() {
        if (!$scope.isStatic) {
            debugMessage("configurationCall");
            if (!!!configTimer) {
                configTimer = $interval(function () {
                    debugMessage("configurationCall Interval call");
                    //nativeInterface.getResourceUrl("/" + $scope.peerId + "/devices/0/configuration");
                    $scope.appAtForeground = true;
                    SHPService.sendSHPCommand("GET", "/" + $scope.peerId + "/devices", "", false);
                }, Sec20Call);
            }
        }
    }

    function parseStaticData() {
        debugMessage("parseStaticData");
        init();
        initializeScreen();
        $scope.kitchenTimer = 35;
        $scope.hoodTimer = 10;
        $scope.showSmartCare = true;
        $scope.KitchenTimerState = 'Run';
        $scope.cookTopBurnerArray[3].OperationState = 'Run';
        $scope.HoodInfo.connected = true;
        $scope.HoodInfo.Mode.options[0].Bluetooth = 'On';
        $scope.HoodInfo.Operation.power = 'On';
        showCurrentScreen();
    }

    function parseDeviceData(devices) {
        debugMessage("deviceUuid from app.js is :" + $scope.deviceUuid);
        if (angular.isDefined(devices) && angular.isDefined(devices[0]) && angular.isDefined(devices[0]["uuid"])) {
            if ($scope.deviceUuid !== devices[0]["uuid"] && $scope.deviceUuid !== devices[1]["uuid"]) {
                debugMessage("return due to missMatch UUID");
                return;
            }
        }

        debugMessage(JSON.stringify(devices));

        if (angular.isDefined(devices[0]) && devices[0]["id"] === '0') {
            parseCooktopNotification(devices[0]);
        }
        if (angular.isDefined(devices[1]) && devices[1]["id"] === '1') {
            parseHoodNotification(devices[1]);
        }
        if (angular.isDefined(devices[0]) && devices[0]["id"] === '1') {
            parseHoodNotification(devices[0]);
        }
        $scope.appAtForeground = false;
        if (justFirstTime) {
            if (!$scope.isStatic) {
                nativeInterface.getResourceUrl("/" + $scope.peerId + "/devices/0/configuration");
            }
            configurationCall();
            return;
        }
        if (!justFirstTime) {
            dismissLoadingBar(COOKTOP_COMMANDS.getDevices);
            initializeScreen();
            showCurrentScreen();
        }
    }

    function parseConfiguration(configInfo) {
        debugMessage("parseNotificationData configInfo : " + JSON.stringify(configInfo));
        $scope.parseConfig = true;
        parseNotification = true;
        if (angular.isDefined(configInfo["remoteControlEnabled"])) {
            $scope.remoteControlEnabled = configInfo["remoteControlEnabled"];
        }
        if (angular.isDefined(configInfo["region"])) {
            $scope.region = configInfo["region"];
        }
        if (justFirstTime) {
            dismissLoadingBar(COOKTOP_COMMANDS.getDevices);
            initializeScreen();
            showCurrentScreen();
        }
    }

    function parseNotificationData(notification) {
        debugMessage("parseNotificationData : " + JSON.stringify(notification));
        if (angular.isDefined(notification["uuid"])) {
            if ($scope.cookTopInfo !== undefined && notification["uuid"] === $scope.cookTopInfo.uuid) {
                parseCooktopNotification(notification);
                if ($scope.parseNotiLength === 0) {
                    $scope.appAtForeground = false;
                    showCurrentScreen();
                }
            } else if ($scope.HoodInfo !== undefined && notification["uuid"] === $scope.HoodInfo.uuid) {
                parseHoodNotification(notification);
                if ($scope.parseNotiLength === 0) {
                    $scope.appAtForeground = false;
                    showCurrentScreen();
                }
            } else {
                debugMessage("parseNotificationData uuid not matched notification uuid " + notification["uuid"]);
                $scope.appAtForeground = true;
                SHPService.sendSHPCommand("GET", "/" + $scope.peerId + "/devices", "", false);
            }
        }
    }

    function parseModuleInformation(moduleInfo) {
        debugMessage("parseModuleInformation " + JSON.stringify(moduleInfo));
        if (angular.isDefined(moduleInfo.Versions) && moduleInfo.Versions.length > 0) {
            $scope.moduleInfoText = "Software Ver : " + moduleInfo.Versions[0].number + " <br> Firmware Ver : " + moduleInfo.Versions[1].number;
            dismissLoadingBar(COOKTOP_COMMANDS.showModuleVersion);
            debugMessage("Infotext : " + $scope.moduleInfoText);
            showcustDialog(COOKTOP_ALERT_STATE.ALERT_DEVELOPER_INFO, $scope.translation.WEBMOB_common_version_info, $scope.moduleInfoText, $scope.translation.WEBMOB_common_ok, '');
        }
    }

    function parseCooktopNotification(deviceData) {
        debugMessage("parseCooktopNotification");
        if (angular.isDefined(deviceData["Configuration"])) {
            parseConfiguration(deviceData["Configuration"]);
        }
        if (angular.isDefined(deviceData["description"])) {
            $scope.cookTopInfo.description = deviceData["description"];
        }
        if (angular.isDefined(deviceData["id"])) {
            $scope.cookTopInfo.id = deviceData["id"];
        }

        if (angular.isDefined(deviceData["ConfigurationLink"])) {
            $scope.cookTopInfo.ConfigurationLink = deviceData["ConfigurationLink"];
        }
        if (angular.isDefined(deviceData["Diagnosis"])) {
            if (angular.isDefined(deviceData["Diagnosis"]["diagnosisStart"])) {
                $scope.cookTopInfo.Diagnosis.diagnosisStart = deviceData["Diagnosis"]["diagnosisStart"];
            }
            if ($scope.cookTopInfo.Diagnosis.diagnosisStart === "Ready") {
                if (!$scope.appAtForeground && ($scope.currentScreen === COOKTOP_SCREENS.WALLOVEN_SMART_CARE_PAGE || $scope.currentScreen === COOKTOP_SCREENS.WALLOVEN_SMART_CARE_NO_ERROR_PAGE)) {
                    dismissLoadingBar(COOKTOP_COMMANDS.smartCareDiagnosis);
                    $scope.currentScreen = COOKTOP_SCREENS.WALLOVEN_SMART_CARE_NO_ERROR_PAGE;
                    validateCurrentScreen('SmartCareNoErrorPage');
                }
            } else if ($scope.cookTopInfo.Diagnosis.diagnosisStart === "Run") {
                $scope.currentScreen = COOKTOP_SCREENS.WALLOVEN_SMART_CARE_PAGE;
                validateCurrentScreen('SmartCarePage');
            }
        }
        if (angular.isDefined(deviceData["EnergyConsumption"])) {
            $scope.cookTopInfo.EnergyConsumption = deviceData["EnergyConsumption"];
        }
        if (angular.isDefined(deviceData["InformationLink"])) {
            $scope.cookTopInfo.InformationLink = deviceData["InformationLink"];
        }
        if (angular.isDefined(deviceData["Mode"])) {
            if (angular.isDefined(deviceData["Mode"]["options"])) {
                deviceData["Mode"]["options"].forEach(function (value) {
                    parseCookTopInfoModeOptions(value);
                });
            }
        }
        if (angular.isDefined(deviceData["Operation"])) {
            if (angular.isDefined(deviceData["Operation"]["power"])) {
                $scope.cookTopInfo.Operation.power = deviceData["Operation"]["power"];
            }
            if (angular.isDefined(deviceData["Operation"]["kidsLock"])) {
                $scope.cookTopInfo.Operation.kidsLock = deviceData["Operation"]["kidsLock"];
            }
        }
        if (angular.isDefined(deviceData["connected"])) {
            $scope.cookTopInfo.connected = deviceData["connected"];
        }
        if (angular.isDefined(deviceData["name"])) {
            $scope.cookTopInfo.name = deviceData["name"];
        }
        if (angular.isDefined(deviceData["resources"])) {
            $scope.cookTopInfo.resources = deviceData["resources"];
            deviceData["resources"].forEach(function (value) {
                if ($scope.cookTopInfo.Mode.options[0].DeviceType === 'NV9300K-/AA1' || $scope.cookTopInfo.Mode.options[0].DeviceType === 'NV9300K-/AA2'
                        || $scope.cookTopInfo.Mode.options[0].DeviceType === 'NV9300K-/AA3' || $scope.cookTopInfo.Mode.options[0].DeviceType === 'NV9300K-/AA4') {
                    if (value.indexOf("Diagnosis") !== -1) {
                        $scope.showSmartCare = true;
                    }
                }
            });
        }
        if (angular.isDefined(deviceData["type"])) {
            $scope.cookTopInfo.type = deviceData["type"];
        }
        if (angular.isDefined(deviceData["uuid"])) {
            $scope.cookTopInfo.uuid = deviceData["uuid"];
        }

        if (angular.isDefined(deviceData["Alarms"])) {
            deviceData["Alarms"].forEach(function (value) {
                parseCookTopInfoAlarm(value);
            });
        }
    }

    function parseHoodNotification(deviceData) {
        debugMessage("parseHoodNotification");
        if (angular.isDefined(deviceData["Configuration"])) {
            parseConfiguration(deviceData["Configuration"]);
        }
        if (angular.isDefined(deviceData["description"])) {
            $scope.HoodInfo.description = deviceData["description"];
        }
        if (angular.isDefined(deviceData["id"])) {
            $scope.HoodInfo.id = deviceData["id"];
        }
        if (angular.isDefined(deviceData["Alarms"])) {
            if (angular.isDefined(deviceData["Alarms"][0])) {
                if (angular.isDefined(deviceData["Alarms"][0]["id"])) {
                    $scope.HoodInfo.Alarms[0].id = deviceData["Alarms"][0]["id"];
                }
                if (angular.isDefined(deviceData["Alarms"][0]["code"])) {
                    $scope.HoodInfo.Alarms[0].code = deviceData["Alarms"][0]["code"];
                }
                if (angular.isDefined(deviceData["Alarms"][0]["alarmType"])) {
                    $scope.HoodInfo.Alarms[0].alarmType = deviceData["Alarms"][0]["alarmType"];
                }
                if (angular.isDefined(deviceData["Alarms"][0]["triggeredTime"])) {
                    $scope.HoodInfo.Alarms[0].triggeredTime = deviceData["Alarms"][0]["triggeredTime"];
                }
            }
        }
        if (angular.isDefined(deviceData["ConfigurationLink"])) {
            $scope.HoodInfo.ConfigurationLink = deviceData["ConfigurationLink"];
        }
        if (angular.isDefined(deviceData["Diagnosis"])) {
            if (angular.isDefined(deviceData["Diagnosis"]["diagnosisStart"])) {
                $scope.HoodInfo.Diagnosis.diagnosisStart = deviceData["Diagnosis"]["diagnosisStart"];
            }
            if ($scope.HoodInfo.Diagnosis.diagnosisStart === "Ready") {
                if (!$scope.appAtForeground && ($scope.currentScreen === COOKTOP_SCREENS.WALLOVEN_SMART_CARE_PAGE || $scope.currentScreen === COOKTOP_SCREENS.WALLOVENWALLOVEN_SMART_CARE_NO_ERROR_PAGESMART_CARE_PAGE)) {
                    dismissLoadingBar(COOKTOP_COMMANDS.smartCareDiagnosis);
                    $scope.currentScreen = COOKTOP_SCREENS.WALLOVENWALLOVEN_SMART_CARE_NO_ERROR_PAGESMART_CARE_PAGE;
                    validateCurrentScreen('SmartCareNoErrorPage');
                }
            } else if ($scope.HoodInfo.Diagnosis.diagnosisStart === "Run") {
                $scope.currentScreen = COOKTOP_SCREENS.WALLOVEN_SMART_CARE_PAGE;
                validateCurrentScreen('SmartCarePage');
            }
        }
        if (angular.isDefined(deviceData["EnergyConsumption"])) {
            $scope.HoodInfo.EnergyConsumption = deviceData["EnergyConsumption"];
        }
        if (angular.isDefined(deviceData["InformationLink"])) {
            $scope.HoodInfo.InformationLink = deviceData["InformationLink"];
        }
        if (angular.isDefined(deviceData["Mode"])) {
            if (angular.isDefined(deviceData["Mode"]["options"])) {
                deviceData["Mode"]["options"].forEach(function (value) {
                    parseHoodInfoModeOptions(value);
                });
            }
        }
        if (angular.isDefined(deviceData["Operation"])) {
            if (angular.isDefined(deviceData["Operation"]["power"])) {
                dismissLoadingBar(COOKTOP_COMMANDS.hoodPower);
                if (($scope.HoodInfo.Operation.power !== deviceData["Operation"]["power"]) && deviceData["Operation"]["power"] === 'Off') {
                    showToast($scope.translation.WEBMOB_device_hood_off);
                }
                $scope.HoodInfo.Operation.power = deviceData["Operation"]["power"];
                $scope.hoodOff = $scope.HoodInfo.Operation.power === 'Off' ? true : false;
                $scope.selectedFanValue = ($scope.HoodInfo.Operation.power === 'on' && !$scope.appAtForeground) ? 1 : $scope.selectedFanValue;
                setLightStatus();
            }
        }
        if (angular.isDefined(deviceData["connected"])) {
            $scope.HoodInfo.connected = deviceData["connected"];
            //$scope.cTopTabSelected = true;
        }
        if (angular.isDefined(deviceData["Device"])) {
            if (angular.isDefined(deviceData["Device"]["connected"])) {
                $scope.HoodInfo.connected = deviceData["Device"]["connected"];
                //$scope.cTopTabSelected = true;
            }
        }
        if (angular.isDefined(deviceData["name"])) {
            $scope.HoodInfo.name = deviceData["name"];
        }
        if (angular.isDefined(deviceData["resources"])) {
            $scope.HoodInfo.resources = deviceData["resources"];
            deviceData["resources"].forEach(function (value) {
                if ($scope.HoodInfo.Mode.options[0].DeviceType === 'NV9300K-/AA1' || $scope.HoodInfo.Mode.options[0].DeviceType === 'NV9300K-/AA2'
                        || $scope.HoodInfo.Mode.options[0].DeviceType === 'NV9300K-/AA3' || $scope.HoodInfo.Mode.options[0].DeviceType === 'NV9300K-/AA4') {
                    if (value.indexOf("Diagnosis") !== -1) {
                        $scope.showSmartCare = true;
                    }
                }
            });
        }
//        if (angular.isDefined(deviceData["resources"])) {
//            $scope.HoodInfo.resources = deviceData["resources"];
//        }
        if (angular.isDefined(deviceData["type"])) {
            $scope.HoodInfo.type = deviceData["type"];
        }
        if (angular.isDefined(deviceData["uuid"])) {
            $scope.HoodInfo.uuid = deviceData["uuid"];
        }

    }

    function parseCookTopInfoAlarm(Alarm) {
        if (angular.isDefined(Alarm)) {
            var id = Alarm["id"];
            if (angular.isDefined(id)) {
                if (angular.isDefined(Alarm["id"])) {
                    $scope.cookTopInfo.Alarms[id].id = Alarm["id"];
                }
                if (angular.isDefined(Alarm["code"])) {
                    $scope.cookTopInfo.Alarms[id].code = Alarm["code"];
                    $scope.cookTopBurnerArray[id].code = Alarm["code"];
                    if ($scope.cookTopInfo.Alarms[0].code.indexOf("CT_E") !== -1) {
                        $scope.errorCode = getErrorCode($scope.cookTopInfo.Alarms[0].code);
//                        if ($scope.errorCode !== '') {
//                            debugMessage('idOne errorRemoved false');
//                            errorRemoved = false;
//                        }
                    }
                    if ($scope.cookTopInfo.Alarms[id].code.indexOf("NoPanDetect") !== -1) {
                        var tmpBurnerId = Number($scope.cookTopInfo.Alarms[id].code.charAt($scope.cookTopInfo.Alarms[0].code.length - 1));
                        debugMessage("No pan alarm for burner " + tmpBurnerId);
                        /*
                         * No pan detected make operation state as ready for all burners except flux area
                         */
                        if (!isNaN(tmpBurnerId) && tmpBurnerId < 6) {
                            if ($scope.isModelAA34() && tmpBurnerId > 1) {
                                $scope.cookTopBurnerArray[tmpBurnerId].OperationState = 'Ready';
                            }
                        }
                        showToast($scope.translation.WEBMOB_device_cooktop_no_pan_detect_popup);
                    }
                }
                if (angular.isDefined(Alarm["alarmType"])) {
                    $scope.cookTopInfo.Alarms[id].alarmType = Alarm["alarmType"];
                }
                if (angular.isDefined(Alarm["triggeredTime"])) {
                    $scope.cookTopInfo.Alarms[id].triggeredTime = Alarm["triggeredTime"];
                }
            }
        }
    }

    function parseCookTopInfoModeOptions(modeOptionData) {
        if (modeOptionData.indexOf("DeviceType") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DeviceType = modeOptionData.split("_")[1];
        }
        if (modeOptionData.indexOf("OperationType") !== -1) {
            $scope.cookTopInfo.Mode.options[0].OperationType = modeOptionData.split("_")[1];
        }
        if (modeOptionData.indexOf("Bluetooth") !== -1) {
            $scope.HoodInfo.Mode.options[0].Bluetooth = modeOptionData.split("_")[1];
            $scope.cTopTabSelected = true;
        }
        if (modeOptionData.indexOf("FanSpeed") !== -1) {
            $scope.HoodInfo.Mode.options[0].FanSpeed = Number(modeOptionData.split("_")[1]);
            $scope.selectedFanValue = $scope.HoodInfo.Mode.options[0].FanSpeed;
            setFanStatus($scope.selectedFanValue);
        }
        if (modeOptionData.indexOf("Lamp") !== -1) {
            $scope.HoodInfo.Mode.options[0].Lamp = modeOptionData.split("_")[1];
            $scope.hoodLight = $scope.HoodInfo.Mode.options[0].Lamp === 'On' ? true : false;
            setLightStatus();
        }
        if (modeOptionData.indexOf("OperationType") !== -1) {
            $scope.cookTopInfo.Mode.options[0].OperationType = modeOptionData.split("_")[1];
        }
        if (modeOptionData.indexOf("MainTimerCurrent") !== -1) {
            var timerValue = Number(modeOptionData.split("_")[1]);
            $scope.kitchenTimer = $scope.appAtForeground || $scope.cookTopInfo.Mode.options[0].MainTimerCurrent <= 0 ? $scope.kitchenTimer : timerValue;
            $scope.cookTopInfo.Mode.options[0].MainTimerCurrent = timerValue;
            kitchenTimerDrag.roundSlider({value: $scope.kitchenTimer, sliderType: "min-range", tab: 1});
            updateTimerValue($scope.kitchenTimer);
            if ($scope.cookTopInfo.Mode.options[0].MainTimerState === 'Run') {
                dismissLoadingBar(COOKTOP_COMMANDS.kitchenTimer);
            }
            if ($scope.cookTopInfo.Mode.options[0].MainTimerCurrent === 0 && $scope.cookTopInfo.Mode.options[0].MainTimerState !== 'Ready') {
                $scope.focuseRemoved = true;
            }
        }
        if (modeOptionData.indexOf("MainTimerSet") !== -1) {
            $scope.cookTopInfo.Mode.options[0].MainTimerSet = Number(modeOptionData.split("_")[1]);
            $scope.kitchenTimer = $scope.appAtForeground || $scope.cookTopInfo.Mode.options[0].MainTimerSet <= 0 ? $scope.kitchenTimer : $scope.cookTopInfo.Mode.options[0].MainTimerSet;
            $scope.kitchenTimer = $scope.kitchenTimer <= 0 ? $scope.cookTopInfo.Mode.options[0].MainTimerCurrent : $scope.kitchenTimer;
            //$scope.cookTopInfo.Mode.options[0].MainTimerCurrent = $scope.kitchenTimer;
            kitchenTimerDrag.roundSlider({value: $scope.kitchenTimer, sliderType: "min-range", tab: 1});
            updateTimerValue($scope.kitchenTimer);
            if ($scope.cookTopInfo.Mode.options[0].MainTimerState === 'Run') {
                dismissLoadingBar(COOKTOP_COMMANDS.kitchenTimer);
            }
        }
        if (modeOptionData.indexOf("MainTimerState") !== -1) {
            if ($scope.cookTopInfo.Mode.options[0].MainTimerState !== modeOptionData.split("_")[1]) {
                blurInputBox('kitchenTimer');
            }
            $scope.cookTopInfo.Mode.options[0].MainTimerState = modeOptionData.split("_")[1];
            dismissLoadingBar(COOKTOP_COMMANDS.kitchenTimer);
            if ($scope.cookTopInfo.Mode.options[0].MainTimerState === 'Finish' && !justFirstTime && !$scope.appAtForeground) {
                showcustDialog(COOKTOP_ALERT_STATE.ALERT_TIMER_FINISH, '', $scope.translation.WEBMOB_device_cooktop_downcount_timer_popup, $scope.translation.WEBMOB_common_ok, '');
            }
            if (!$scope.appAtForeground) {
                if ($scope.cookTopInfo.Mode.options[0].MainTimerState === 'Run') {
                    $scope.KitchenTimerState = 'Run';
                    $scope.startStopState = $scope.translation.WEBMOB_device_hood_stop_button;
                } else if ($scope.cookTopInfo.Mode.options[0].MainTimerState === 'Pause') {
                    $scope.KitchenTimerState = 'Pause';
                    $scope.startStopState = $scope.translation.WEBMOB_common_cooktop_pause;//'PAUSE'
                } else {
                    $scope.KitchenTimerState = 'Ready';
                    $scope.startStopState = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_start;
                }
            }
        }
        if (modeOptionData.indexOf("Pause") !== -1) {
            $scope.cookTopInfo.Mode.options[0].Pause = modeOptionData.split("_")[1];
        }
        if (modeOptionData.indexOf("SyncFlex") !== -1) {
            $scope.cookTopInfo.Mode.options[0].SyncFlex = modeOptionData.split("_")[1];
        }
        if (modeOptionData.indexOf("FlexCoil") !== -1) {
            $scope.cookTopInfo.Mode.options[0].FlexCoilValue = modeOptionData.split("_")[1];
            parseFlexCoilOnOffInfo();
        }
        if (modeOptionData.indexOf("OperationState0") !== -1) {
            $scope.cookTopInfo.Mode.options[0].OperationState0 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[0].OperationState = parseOperationState($scope.cookTopInfo.Mode.options[0].OperationState0);
        }

        if (modeOptionData.indexOf("PanSize0") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PanSize0 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[0].PanSize = parsePanSize($scope.cookTopInfo.Mode.options[0].PanSize0);
        }
        if (modeOptionData.indexOf("PanSize1") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PanSize1 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[1].PanSize = parsePanSize($scope.cookTopInfo.Mode.options[0].PanSize1);
        }
        if (modeOptionData.indexOf("PanSize2") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PanSize2 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[2].PanSize = parsePanSize($scope.cookTopInfo.Mode.options[0].PanSize2);
        }
        if (modeOptionData.indexOf("PanSize3") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PanSize3 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[3].PanSize = parsePanSize($scope.cookTopInfo.Mode.options[0].PanSize3);
        }
        if (modeOptionData.indexOf("PanSize4") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PanSize4 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[4].PanSize = parsePanSize($scope.cookTopInfo.Mode.options[0].PanSize4);
        }
        if (modeOptionData.indexOf("PanSize5") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PanSize5 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[5].PanSize = parsePanSize($scope.cookTopInfo.Mode.options[0].PanSize5);
        }

        if (modeOptionData.indexOf("OperationState1") !== -1) {
            $scope.cookTopInfo.Mode.options[0].OperationState1 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[1].OperationState = parseOperationState($scope.cookTopInfo.Mode.options[0].OperationState1);
        }
        if (modeOptionData.indexOf("OperationState2") !== -1) {
            $scope.cookTopInfo.Mode.options[0].OperationState2 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[2].OperationState = parseOperationState($scope.cookTopInfo.Mode.options[0].OperationState2);
        }
        if (modeOptionData.indexOf("OperationState3") !== -1) {
            $scope.cookTopInfo.Mode.options[0].OperationState3 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[3].OperationState = parseOperationState($scope.cookTopInfo.Mode.options[0].OperationState3);
        }
        if (modeOptionData.indexOf("OperationState4") !== -1) {
            $scope.cookTopInfo.Mode.options[0].OperationState4 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[4].OperationState = parseOperationState($scope.cookTopInfo.Mode.options[0].OperationState4);
        }
        if (modeOptionData.indexOf("OperationState5") !== -1) {
            $scope.cookTopInfo.Mode.options[0].OperationState5 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[5].OperationState = parseOperationState($scope.cookTopInfo.Mode.options[0].OperationState5);
        }

        if (modeOptionData.indexOf("HotSurface0") !== -1) {
            $scope.cookTopInfo.Mode.options[0].HotSurface0 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[0].HotSurface = parseHotSurface($scope.cookTopInfo.Mode.options[0].HotSurface0);
        }
        if (modeOptionData.indexOf("HotSurface1") !== -1) {
            $scope.cookTopInfo.Mode.options[0].HotSurface1 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[1].HotSurface = parseHotSurface($scope.cookTopInfo.Mode.options[0].HotSurface1);
        }
        if (modeOptionData.indexOf("HotSurface2") !== -1) {
            $scope.cookTopInfo.Mode.options[0].HotSurface2 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[2].HotSurface = parseHotSurface($scope.cookTopInfo.Mode.options[0].HotSurface2);
        }
        if (modeOptionData.indexOf("HotSurface3") !== -1) {
            $scope.cookTopInfo.Mode.options[0].HotSurface3 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[3].HotSurface = parseHotSurface($scope.cookTopInfo.Mode.options[0].HotSurface3);
        }
        if (modeOptionData.indexOf("HotSurface4") !== -1) {
            $scope.cookTopInfo.Mode.options[0].HotSurface4 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[4].HotSurface = parseHotSurface($scope.cookTopInfo.Mode.options[0].HotSurface4);
        }
        if (modeOptionData.indexOf("HotSurface5") !== -1) {
            $scope.cookTopInfo.Mode.options[0].HotSurface5 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[5].HotSurface = parseHotSurface($scope.cookTopInfo.Mode.options[0].HotSurface5);
        }

        if (modeOptionData.indexOf("PowerLevel0") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PowerLevel0 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[0].PowerLevel = parsePowerLevel($scope.cookTopInfo.Mode.options[0].PowerLevel0);
        }
        if (modeOptionData.indexOf("PowerLevel1") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PowerLevel1 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[1].PowerLevel = parsePowerLevel($scope.cookTopInfo.Mode.options[0].PowerLevel1);
        }
        if (modeOptionData.indexOf("PowerLevel2") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PowerLevel2 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[2].PowerLevel = parsePowerLevel($scope.cookTopInfo.Mode.options[0].PowerLevel2);
        }
        if (modeOptionData.indexOf("PowerLevel3") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PowerLevel3 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[3].PowerLevel = parsePowerLevel($scope.cookTopInfo.Mode.options[0].PowerLevel3);
        }
        if (modeOptionData.indexOf("PowerLevel4") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PowerLevel4 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[4].PowerLevel = parsePowerLevel($scope.cookTopInfo.Mode.options[0].PowerLevel4);
        }
        if (modeOptionData.indexOf("PowerLevel5") !== -1) {
            $scope.cookTopInfo.Mode.options[0].PowerLevel5 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[5].PowerLevel = parsePowerLevel($scope.cookTopInfo.Mode.options[0].PowerLevel5);
        }

        if (modeOptionData.indexOf("TimerState0") !== -1) {
            $scope.cookTopInfo.Mode.options[0].TimerState0 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[0].TimerState = parseTimerState($scope.cookTopInfo.Mode.options[0].TimerState0);
        }
        if (modeOptionData.indexOf("TimerState1") !== -1) {
            $scope.cookTopInfo.Mode.options[0].TimerState1 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[1].TimerState = parseTimerState($scope.cookTopInfo.Mode.options[0].TimerState1);
        }
        if (modeOptionData.indexOf("TimerState2") !== -1) {
            $scope.cookTopInfo.Mode.options[0].TimerState2 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[2].TimerState = parseTimerState($scope.cookTopInfo.Mode.options[0].TimerState2);
        }
        if (modeOptionData.indexOf("TimerState3") !== -1) {
            $scope.cookTopInfo.Mode.options[0].TimerState3 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[3].TimerState = parseTimerState($scope.cookTopInfo.Mode.options[0].TimerState3);
        }
        if (modeOptionData.indexOf("TimerState4") !== -1) {
            $scope.cookTopInfo.Mode.options[0].TimerState4 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[4].TimerState = parseTimerState($scope.cookTopInfo.Mode.options[0].TimerState4);
        }
        if (modeOptionData.indexOf("TimerState5") !== -1) {
            $scope.cookTopInfo.Mode.options[0].TimerState5 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[5].TimerState = parseTimerState($scope.cookTopInfo.Mode.options[0].TimerState5);
        }

        if (modeOptionData.indexOf("DownCoutTimerCurrent0") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent0 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[0].DownCoutTimerCurrent = parseDownCoutTimerCurrent($scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent0);
        }
        if (modeOptionData.indexOf("DownCoutTimerCurrent1") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent1 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[1].DownCoutTimerCurrent = parseDownCoutTimerCurrent($scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent1);
        }
        if (modeOptionData.indexOf("DownCoutTimerCurrent2") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent2 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[2].DownCoutTimerCurrent = parseDownCoutTimerCurrent($scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent2);
        }
        if (modeOptionData.indexOf("DownCoutTimerCurrent3") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent3 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[3].DownCoutTimerCurrent = parseDownCoutTimerCurrent($scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent3);
        }
        if (modeOptionData.indexOf("DownCoutTimerCurrent4") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent4 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[4].DownCoutTimerCurrent = parseDownCoutTimerCurrent($scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent4);
        }
        if (modeOptionData.indexOf("DownCoutTimerCurrent5") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent5 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[5].DownCoutTimerCurrent = parseDownCoutTimerCurrent($scope.cookTopInfo.Mode.options[0].DownCoutTimerCurrent5);
        }

        if (modeOptionData.indexOf("DownCoutTimerSet0") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerSet0 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[0].DownCoutTimerSet = parseDownCoutTimerSet($scope.cookTopInfo.Mode.options[0].DownCoutTimerSet0);
        }
        if (modeOptionData.indexOf("DownCoutTimerSet1") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerSet1 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[1].DownCoutTimerSet = parseDownCoutTimerSet($scope.cookTopInfo.Mode.options[0].DownCoutTimerSet1);
        }
        if (modeOptionData.indexOf("DownCoutTimerSet2") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerSet2 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[2].DownCoutTimerSet = parseDownCoutTimerSet($scope.cookTopInfo.Mode.options[0].DownCoutTimerSet2);
        }
        if (modeOptionData.indexOf("DownCoutTimerSet3") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerSet3 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[3].DownCoutTimerSet = parseDownCoutTimerSet($scope.cookTopInfo.Mode.options[0].DownCoutTimerSet3);
        }
        if (modeOptionData.indexOf("DownCoutTimerSet4") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerSet4 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[4].DownCoutTimerSet = parseDownCoutTimerSet($scope.cookTopInfo.Mode.options[0].DownCoutTimerSet4);
        }
        if (modeOptionData.indexOf("DownCoutTimerSet5") !== -1) {
            $scope.cookTopInfo.Mode.options[0].DownCoutTimerSet5 = modeOptionData.split("_")[1];
            $scope.cookTopBurnerArray[5].DownCoutTimerSet = parseDownCoutTimerSet($scope.cookTopInfo.Mode.options[0].DownCoutTimerSet5);
        }
    }

    function parseFlexCoilOnOffInfo() {
        ResetFlexCoilOnOffInfo();
        var binaryVal = (Number($scope.cookTopInfo.Mode.options[0].FlexCoilValue) >>> 0).toString(2);
        binaryVal = '000' + binaryVal;
        binaryVal = binaryVal.substr(-4);
        $scope.flexCoilStatus[0].FlexOn = binaryVal[3] === '1' ? true : false;
        $scope.flexCoilStatus[1].FlexOn = binaryVal[2] === '1' ? true : false;
        $scope.flexCoilStatus[2].FlexOn = binaryVal[1] === '1' ? true : false;
        $scope.flexCoilStatus[3].FlexOn = binaryVal[0] === '1' ? true : false;
    }

    function ResetFlexCoilOnOffInfo() {
        $scope.Ch0UpperFlexCoilValue = false;
        $scope.Ch0LowerFlexCoilValue = false;
        $scope.Ch1UpperFlexCoilValue = false;
        $scope.Ch1LowerFlexCoilValue = false;
    }

    function parseHoodInfoModeOptions(modeOptionData) {
        if (modeOptionData.indexOf("DeviceType") !== -1) {
            $scope.HoodInfo.Mode.options[0].DeviceType = modeOptionData.split("_")[1];
        }
        if (modeOptionData.indexOf("Bluetooth") !== -1) {
            $scope.HoodInfo.Mode.options[0].Bluetooth = modeOptionData.split("_")[1];
            //$scope.cTopTabSelected = true;
        }
        if (modeOptionData.indexOf("BTState") !== -1) {
            $scope.HoodInfo.Mode.options[0].BTState = Number(modeOptionData.split("_")[1]);
        }
        if (modeOptionData.indexOf("AutoMode") !== -1) {
            $scope.HoodInfo.Mode.options[0].AutoMode = modeOptionData.split("_")[1];
        }
        if (modeOptionData.indexOf("FanSpeed") !== -1) {
            $scope.HoodInfo.Mode.options[0].FanSpeed = Number(modeOptionData.split("_")[1]);
            $scope.selectedFanValue = $scope.HoodInfo.Mode.options[0].FanSpeed;
            setFanStatus($scope.selectedFanValue);
            dismissLoadingBar(COOKTOP_COMMANDS.hoodFan);
        }
        if (modeOptionData.indexOf("Lamp") !== -1) {
            $scope.HoodInfo.Mode.options[0].Lamp = modeOptionData.split("_")[1];
            $scope.hoodLight = $scope.HoodInfo.Mode.options[0].Lamp === 'On' ? true : false;
            setLightStatus();
            dismissLoadingBar(COOKTOP_COMMANDS.hoodLight);
        }
        if (modeOptionData.indexOf("ShutOffTimerState") !== -1) {
            if ($scope.HoodInfo.Mode.options[0].ShutOffTimerState !== modeOptionData.split("_")[1]) {
                blurInputBox('hoodTimer');
            }
            $scope.HoodInfo.Mode.options[0].ShutOffTimerState = modeOptionData.split("_")[1];
            dismissLoadingBar(COOKTOP_COMMANDS.hoodTimer);
            if ($scope.HoodInfo.Mode.options[0].ShutOffTimerState === 'Finish' && !justFirstTime && !$scope.appAtForeground) {
                showcustDialog(COOKTOP_ALERT_STATE.ALERT_HOOD_TIMER_FINISH, '', $scope.translation.WEBMOB_device_hood_time_left_guide, $scope.translation.WEBMOB_common_ok, '');
            }
            if (!$scope.appAtForeground) {
                if ($scope.HoodInfo.Mode.options[0].ShutOffTimerState === 'Run') {
                    $scope.HoodTimerState = 'Run';
                    $scope.startStopHoodState = $scope.translation.WEBMOB_device_hood_stop_button;
                    $scope.timerStopTextHood = ($scope.hoodTimer < 1) ? $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step1 : $scope.translation.WEBMOB_device_hood_set_time_guide4;
                } else {
                    $scope.HoodTimerState = 'Ready';
                    $scope.startStopHoodState = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_start;
                }
            }
        }
        if (modeOptionData.indexOf("ShutOffTimerCurrent") !== -1) {
            $scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent = Number(modeOptionData.split("_")[1]);
            $scope.hoodTimer = $scope.appAtForeground || $scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent <= 0 ? $scope.hoodTimer : $scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent;
            hoodTimerDrag.roundSlider({value: $scope.hoodTimer, sliderType: "min-range", tab: 2});
            updateHoodTimerValue($scope.hoodTimer);
            if ($scope.HoodInfo.Mode.options[0].ShutOffTimerState === 'Run') {
                dismissLoadingBar(COOKTOP_COMMANDS.hoodTimer);
            }
            if ($scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent === 0 && $scope.HoodInfo.Mode.options[0].ShutOffTimerState !== 'Ready') {
                $scope.focuseRemoved = true;
            }
        }
        if (modeOptionData.indexOf("ShutOffTimerSet") !== -1) {
            var shutOffTimerVal = Number(modeOptionData.split("_")[1]);
            $scope.hoodTimer = $scope.appAtForeground || $scope.HoodInfo.Mode.options[0].ShutOffTimerSet <= 0 ? $scope.hoodTimer : shutOffTimerVal;
            $scope.HoodInfo.Mode.options[0].ShutOffTimerSet = shutOffTimerVal;
            //$scope.hoodTimer = $scope.hoodTimer <= 0 ? $scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent : $scope.hoodTimer;
            //$scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent = $scope.hoodTimer;
            hoodTimerDrag.roundSlider({value: $scope.hoodTimer, sliderType: "min-range", tab: 2});
            updateHoodTimerValue($scope.hoodTimer);
            if ($scope.HoodInfo.Mode.options[0].ShutOffTimerState === 'Run') {
                dismissLoadingBar(COOKTOP_COMMANDS.hoodTimer);
            }
        }
    }

    function parsePanSize(value) {
        return value;
    }

    function parseOperationState(value) {
        return value;
    }

    function parseHotSurface(value) {
        return value;
    }

    function parsePowerLevel(value) {
        return value;
    }

    function parseTimerState(value) {
        return value;
    }

    function parseDownCoutTimerCurrent(value) {
        return value;
    }

    function parseDownCoutTimerSet(value) {
        return value;
    }

    function handleStatusCode(status) {
        if (status === 204) {
            //loadingValues();
        }
    }

    function requestFailHandler(response) {
        debugMessage(JSON.stringify(response));
        $scope.loadingBar = false;
        $scope.checkResponse = [];
    }

    function showConnectionFailurePopup() {
        if (isTextInput(document.activeElement)) {
            document.activeElement.blur();
        }
        showcustDialog(COOKTOP_ALERT_STATE.ALERT_CONNECTION_ERROR, $scope.translation.WEBMOB_device_range_alarm_connection_failure_title, $scope.translation.WEBMOB_common_check_power_and_network, '', $scope.translation.WEBMOB_common_ok);
    }

    function closeConnectionFailurePopup() {
        $scope.showFullScreenPopup = false;
        scrollScreen();
    }

    function preConditionValidation() {
        $scope.mainLoadingScreen = true;
        $scope.moreOptionVisible = false;
        $scope.showKitchenTimer = false;
        $scope.showHoodTimer = false;
        $scope.settingOption = false;
        $scope.aboutOption = true;
        doNotChangeScreen = false;
        if ($scope.showFullScreenPopup || $scope.showMenuOptionPopup) {
            fixScreen();
        } else {
            scrollScreen();
        }
    }

    function validateCurrentScreen(screen) {
        preConditionValidation();
        switch (screen) {
            case 'MainLoadingPage':
                debugMessage("validate screen : " + screen);
                $scope.currentScreen = COOKTOP_SCREENS.COOKTOP_MAIN_LOADING_PAGE;
                $scope.deviceName = "";
                $scope.headerBarVisible = false;
                $scope.TabsVisibleAtPage = false;
                $scope.mainLoadingScreen = false;
                $state.transitionTo('MainLoadingPage');
                break;
            case 'BurnerHomePage':
                debugMessage("validate screen : " + screen + " show tab : " + $scope.showHood);
                $scope.currentScreen = COOKTOP_SCREENS.COOKTOP_BURNER_HOME_PAGE;
                $scope.moreOptionVisible = true;
                $scope.TabsVisibleAtPage = $scope.showHood;
                $scope.deviceName = $scope.translation.WEBMOB_common_device_cooktop;
                $scope.kitchenTimer = Number($scope.cookTopInfo.Mode.options[0].MainTimerCurrent);
                $scope.hoodTimer = $scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent;
                $scope.kitchenTimerLeftTime = $scope.translation.WEBMOB_device_cooktop_timer_left.replace(/#1#/, $scope.kitchenTimer);
                $scope.hoodTimerLeftTime = $scope.translation.WEBMOB_device_cooktop_timer_left.replace(/#1#/, $scope.hoodTimer);
                $scope.headerBarVisible = true;
                $state.transitionTo('BurnerHomePage');
                break;
            case 'KitchenTimerPage':
                debugMessage("validate screen : " + screen);
                $scope.headerBarVisible = true;
                $scope.currentScreen = COOKTOP_SCREENS.COOKTOP_KITCHEN_TIMER_PAGE;
                $scope.TabsVisibleAtPage = false;
                doNotChangeScreen = true;
                roundShape = kitchenTimerDrag;
                selectBurner(-1);
                $scope.deviceName = $scope.translation.WEBMOB_device_cooktop_kitchen_timer;
                $state.transitionTo('KitchenTimerPage');
                document.getElementById("kitchenTimerId").style.zIndex = 1;
                $scope.showKitchenTimer = true;
                break;
            case 'HoodTimerPage':
                $scope.headerBarVisible = true;
                debugMessage("validate screen : " + screen);
                $scope.currentScreen = COOKTOP_SCREENS.COOKTOP_HOOD_TIMER_PAGE;
                $scope.TabsVisibleAtPage = false;
                doNotChangeScreen = true;
                roundShape = hoodTimerDrag;
                selectBurner(-1);
                $scope.deviceName = $scope.translation.WEBMOB_device_hood_set_timer;
                $state.transitionTo('HoodTimerPage');
                document.getElementById("hoodTimerId").style.zIndex = 1;
                $scope.showHoodTimer = true;
                break;
            case 'ErrorPage':
                debugMessage("validate screen : " + screen);
                $scope.headerBarVisible = true;
                $scope.currentScreen = COOKTOP_SCREENS.COOKTOP_ERROR_PAGE;
                $scope.deviceName = $scope.translation.WEBMOB_device_range_comm_error;
                $scope.TabsVisibleAtPage = false;
                $scope.showMenuOptionPopup = false;
                $scope.showFullScreenPopup = false;
                doNotChangeScreen = true;
                $state.transitionTo('ErrorPage');
                break;
            case 'SettingPage':
                debugMessage("validate screen : " + screen);
                $scope.headerBarVisible = true;
                $scope.currentScreen = COOKTOP_SCREENS.COOKTOP_SETTING_PAGE;
                $scope.TabsVisibleAtPage = false;
                $scope.deviceName = $scope.translation.WEBMOB_common_settings;
                //settingPageOptionsValidation();
                selectBurner(-1);
                doNotChangeScreen = true;
                $state.transitionTo('SettingPage');
                break;
            case 'AboutPage':
                debugMessage("validate screen : " + screen);
                $scope.headerBarVisible = true;
                $scope.currentScreen = COOKTOP_SCREENS.COOKTOP_ABOUT_PAGE;
                $scope.deviceName = $scope.translation.WEBMOB_common_about_device;
                $scope.TabsVisibleAtPage = false;
                doNotChangeScreen = true;
                selectBurner(-1);
                $state.transitionTo('AboutPage');
                break;
            case 'SmartCarePage':
                debugMessage("validate screen : " + screen);
                $scope.currentScreen = COOKTOP_SCREENS.WALLOVEN_SMART_CARE_PAGE;
                $scope.deviceName = $scope.translation.WEBMOB_common_self_diagnosis;
                $scope.TabsVisibleAtPage = false;
                $scope.headerBarVisible = true;
                changeSmartCarePage = true;
                $state.transitionTo('SmartCarePage');
                break;
            case 'SmartCareNoErrorPage':
                debugMessage("validate screen : " + screen);
                $scope.currentScreen = COOKTOP_SCREENS.WALLOVEN_SMART_CARE_NO_ERROR_PAGE;
                $scope.deviceName = $scope.translation.WEBMOB_common_self_diagnosis;
                $scope.TabsVisibleAtPage = false;
                $scope.headerBarVisible = true;
                changeSmartCarePage = true;
                $state.transitionTo('SmartCareNoErrorPage');
                break;
            default :
                debugMessage("validate screen else part: " + screen);
                break;
        }
    }

    function fixScreen() {
        scroll_middle_area.style.overflowY = "hidden";
        scroll_middle_area.style.position = "fixed";
    }

    function scrollScreen() {
        //document.getElementById("bodyTag").style.pointerEvents = 'auto';
        scroll_middle_area.style.position = "absolute";
        scroll_middle_area.style.overflowY = "overlay";
    }

    function onBackPressedEvent(e) {
        //e.stopPropagation();
        onBackPressed();
    }

    function refreshValue() {
        debugMessage('All value reset');
    }

    function onBackPressed() {
        debugMessage("Back key pressed from : " + $scope.currentScreen);
        if ($scope.showFullScreenPopup || $scope.showMenuOptionPopup) {//any popup is open then close and return
            $scope.showFullScreenPopup && onAlertButton1Click();
            $scope.showMenuOptionPopup && hideMenuOption();
            return;
        }

        refreshValue();
        switch ($scope.currentScreen) {
            case COOKTOP_SCREENS.COOKTOP_BURNER_HOME_PAGE:
            case COOKTOP_SCREENS.COOKTOP_ERROR_PAGE:
            case COOKTOP_SCREENS.COOKTOP_MAIN_LOADING_PAGE:
                closeApp();
                break;
            case COOKTOP_SCREENS.COOKTOP_KITCHEN_TIMER_PAGE:
                doNotChangeScreen = false;
                $scope.showKitchenTimer = false;
                $scope.cTopTabSelected = true;
                validateCurrentScreen('BurnerHomePage');
                break;
            case COOKTOP_SCREENS.COOKTOP_HOOD_TIMER_PAGE:
                doNotChangeScreen = false;
                $scope.showHoodTimer = false;
                $scope.cTopTabSelected = false;
                validateCurrentScreen('BurnerHomePage');
                break;
            case COOKTOP_SCREENS.COOKTOP_SETTING_PAGE:
            case COOKTOP_SCREENS.COOKTOP_ABOUT_PAGE:
                doNotChangeScreen = false;
                showCurrentScreen();
                validateCurrentScreen('BurnerHomePage');
                break;
            case COOKTOP_SCREENS.WALLOVEN_SMART_CARE_PAGE://Fallthrough
            case COOKTOP_SCREENS.WALLOVEN_SMART_CARE_NO_ERROR_PAGE:
                validateCurrentScreen('AboutPage');
                break;
            default :
                debugMessage("Back key pressed from else " + $scope.currentScreen);
                //closeApp();
                break;
        }
    }

    function onAboutAction(reqParam) {
        if (reqParam === "smartcare") {
            $scope.currentScreen = COOKTOP_SCREENS.WALLOVEN_SMART_CARE_PAGE;
            validateCurrentScreen('SmartCarePage');
            return;
        }
        nativeInterface.runOnNative("customerService", aboutCommand(reqParam));
    }

    function cTopTabClick(id) {
        if ($scope.TabsVisibleAtPage) {
            if (id === 1) {
                $scope.cTopTabSelected = false;
            } else {
                $scope.cTopTabSelected = true;
            }
            showCurrentScreen();
        }
    }

    function hideMenuOption() {
        debugMessage('hideMenuOption');
        scrollScreen();
        $scope.showMenuOptionPopup = false;
    }

    function showMenuOption() {
        debugMessage('showMenuOption');
        fixScreen();
        $scope.showMenuOptionPopup = true;
        $timeout(function () {
            document.getElementById("showMenufullScreenPopupId").style.zIndex = 999;
            document.getElementById("showMenuPopupDivId").style.zIndex = 999;
        }, 0);
    }

    function versionTouchStart() {
        debugMessage("ABOUT command versionTouchStart ");
        if (!!!promiseAbout) {
            promiseAbout = $timeout(function () {
                if (!$scope.isStatic) {
                    $scope.checkResponse.push(COOKTOP_COMMANDS.showModuleVersion);
                    SHPService.sendSHPCommand(CONSTANTS.GET, "/" + $scope.peerId + "/devices/0/information");
                }
            }, 5000);
        }
    }

    function doValidationForTimer() {
        showDummyInbox();
        if ($scope.focuseRemoved) {
            $scope.focuseRemoved = false;
            //return;
        }
        $scope.kitchenTimer = Number(document.getElementById('kitchenTimer').value);
        if ($scope.kitchenTimer > 99) {
            $scope.kitchenTimer = 99;
            var text = $scope.translation.WEBMOB_device_range_alarm_maximum_temp;
            text = text.replace(/#1#/, 99);
            showToast(text);
        }
        if ($scope.kitchenTimer < 0) {
            $scope.kitchenTimer = 1;
            var text = $scope.translation.WEBMOB_device_range_alarm_minimum_temp;
            text = text.replace(/#1#/, '01');
            showToast(text);
        }

        if (!($scope.kitchenTimer)) {
            $scope.kitchenTimer = 1;
            var text = $scope.translation.WEBMOB_device_range_alarm_minimum_temp;
            text = text.replace(/#1#/, '01');
            showToast(text);
        }

        if (($scope.kitchenTimer) % 1 !== 0 && ($scope.kitchenTimer >= 00) && ($scope.kitchenTimer <= 99)) {
            $scope.kitchenTimer = Math.ceil(Number($scope.kitchenTimer) / 1) * 1;
        }
        kitchenTimerDrag.roundSlider({value: $scope.kitchenTimer, sliderType: "min-range", tab: 1});
        updateTimerValue($scope.kitchenTimer);
    }

    function doValidationForHoodTimer() {
        showDummyInbox();
        if ($scope.focuseRemoved) {
            $scope.focuseRemoved = false;
            //return;
        }
        $scope.hoodTimer = Number(document.getElementById('hoodTimer').value);
        if ($scope.hoodTimer > 99) {
            $scope.hoodTimer = 99;
            var text = $scope.translation.WEBMOB_device_range_alarm_maximum_temp;
            text = text.replace(/#1#/, 99);
            showToast(text);
        }
        if ($scope.hoodTimer < 0) {
            $scope.hoodTimer = 1;
            var text = $scope.translation.WEBMOB_device_range_alarm_minimum_temp;
            text = text.replace(/#1#/, '01');
            showToast(text);
        }

        if (!($scope.hoodTimer)) {
            $scope.hoodTimer = 1;
            var text = $scope.translation.WEBMOB_device_range_alarm_minimum_temp;
            text = text.replace(/#1#/, '01');
            showToast(text);
        }

        if (($scope.hoodTimer) % 1 !== 0 && ($scope.hoodTimer >= 00) && ($scope.hoodTimer <= 99)) {
            $scope.hoodTimer = Math.ceil(Number($scope.hoodTimer) / 1) * 1;
        }
        hoodTimerDrag.roundSlider({value: $scope.hoodTimer, sliderType: "min-range", tab: 2});
        updateHoodTimerValue($scope.hoodTimer);
    }

    function versionTouchEnd() {
        debugMessage("ABOUT command versionTouchEnd ");
        if (!!promiseAbout) {
            $timeout.cancel(promiseAbout);
            promiseAbout = null;
        }
    }

    function goToSettings() {
        debugMessage('goToSettings');
        validateCurrentScreen('SettingPage');
    }

    function gotoKitchenTimer() {
        debugMessage('gotoKitchenTimer');
        $scope.kitchenTimer = Number($scope.cookTopInfo.Mode.options[0].MainTimerCurrent);
        KitchenTimerPage();
    }

    function KitchenTimerPage() {
        debugMessage('KitchenTimerPage');
        //showDummyInbox();  // CQ MW0600025669 - In Kitchen timer screen, key pad closed after getdevice response and any notification. 
        kitchenTimerDrag.roundSlider({value: $scope.kitchenTimer, sliderType: "min-range", max: 99, step: 1, tab: 1});
        $scope.updateTimerValue($scope.kitchenTimer);
        validateCurrentScreen('KitchenTimerPage');
    }

    function gotoHoodTimer() {
        debugMessage('gotoHoodTimer');
        if ($scope.hoodOff) {
            showToast($scope.translation.WEBMOB_device_hood_off);
            return;
        }
        //showDummyInbox(); // CQ MW0600025669 - In Kitchen timer screen, key pad closed after getdevice response and any notification. 
        //$scope.hoodTimer = $scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent;
        hoodTimerDrag.roundSlider({value: $scope.hoodTimer, sliderType: "min-range", max: 99, step: 1, tab: 2});
        $scope.updateHoodTimerValue($scope.hoodTimer);
        validateCurrentScreen('HoodTimerPage');
    }

    function addKitchenTimer(time) {
        if ($scope.cookTopInfo.Operation.power !== 'On') {
            showToast($scope.translation.WEBMOB_device_cooktop_off);
            return;
        }
        if ($scope.cookTopInfo.Operation.kidsLock === 'Run') {
            debugMessage("return Kids Lock is ON");
            showcustDialog(COOKTOP_ALERT_STATE.ALERT_CHILD_LOCK, $scope.translation.WEBMOB_common_device_cooktop, $scope.translation.WEBMOB_device_cooktop_child_lock_popup, $scope.translation.WEBMOB_common_ok, '');
            return;
        }
        var value = 0;
        /**
         * Updated as hood timer is behaving the same way - discussed with Mr. Choi
         */
        value = $scope.kitchenTimer; //Number($scope.cookTopInfo.Mode.options[0].MainTimerCurrent);
        value = value + Number(time);
        if (value > 99) {
            var text = $scope.translation.WEBMOB_device_range_alarm_maximum_temp;
            text = text.replace(/#1#/, 99);
            showToast(text);
            return;
        }
        $scope.kitchenTimer = Number(value);
        updateTimerValue($scope.kitchenTimer);
        //sendKitchenTimer(value);
    }

    function addHoodTimer(time) {
        if ($scope.hoodOff) {
            showToast($scope.translation.WEBMOB_device_hood_off);
            return;
        }
        var value = 0;
        value = $scope.hoodTimer;
        value = value + Number(time);
        if (value > 99) {
            var text = $scope.translation.WEBMOB_device_range_alarm_maximum_temp;
            text = text.replace(/#1#/, 99);
            showToast(text);
            return;
        }
        $scope.hoodTimer = Number(value);
        $scope.updateHoodTimerValue($scope.hoodTimer);
        //sendHoodTimer(value);
    }

    function startKitchenTimer() {
        if ($scope.cookTopInfo.Operation.power !== 'On') {
            showToast($scope.translation.WEBMOB_device_cooktop_off);
            return;
        }
        if ($scope.cookTopInfo.Operation.kidsLock === 'Run') {
            debugMessage("return Kids Lock is ON");
            showcustDialog(COOKTOP_ALERT_STATE.ALERT_CHILD_LOCK, $scope.translation.WEBMOB_common_device_cooktop, $scope.translation.WEBMOB_device_cooktop_child_lock_popup, $scope.translation.WEBMOB_common_ok, '');
            return;
        }
        var value = 0;
        value = $scope.kitchenTimer;
        if (value > 99) {
            value = '99';
        } else if (value === 0) {
            value = '00';
        } else if (value > 0 && value < 10) {
            value = '0' + value;
        } else {
            value = '' + value;
        }
        if ($scope.startStopState === $scope.translation.WEBMOB_device_hood_stop_button) {
            value = '00';
        }
        if ($scope.KitchenTimerState === 'Ready' && value === '00') {
            showToast('Please set some Timer for cooking');//TODO STRING
        } else {
            sendKitchenTimer(value);
        }
    }

    function startHoodTimer() {
        if ($scope.hoodOff) {
            showToast($scope.translation.WEBMOB_device_hood_off);
            return;
        }
        var value = 0;
        value = $scope.hoodTimer;
        if (value > 99) {
            value = '99';
        } else if (value === 0) {
            value = '00';
        } else if (value > 0 && value < 10) {
            value = '0' + value;
        } else {
            value = '' + value;
        }
        if ($scope.startStopHoodState === $scope.translation.WEBMOB_device_hood_stop_button) {
            value = '00';
        }
        if ($scope.HoodTimerState === 'Ready' && value === '00') {
            showToast('Please set some Timer for cooking');//TODO STRING
        } else {
            sendHoodTimer(value);
        }
    }

    function resetKitchenTimer() {
        debugMessage('resetKitchenTimer');
        showDummyInbox();
        if ($scope.KitchenTimerState !== 'Run') {
            $scope.kitchenTimer = 0;
            kitchenTimerDrag.roundSlider({value: $scope.kitchenTimer, sliderType: "min-range", tab: 1});
        }
    }

    function resetHoodTimer() {
        debugMessage('resetKitchenTimer');
        showDummyInbox();
        if ($scope.HoodTimerState !== 'Run') {
            $scope.hoodTimer = 0;
            hoodTimerDrag.roundSlider({value: $scope.hoodTimer, sliderType: "min-range", tab: 2});
        }
    }

    function sendKitchenTimer(value) {
        var command;
        $scope.kitchenTimer = Number(value);
        if (value === '00') {
            command = '{"Mode":{"options":["' + 'MainTimerSet_' + value + '","MainTimerState_Ready"]}}';
        } else {
            command = '{"Mode":{"options":["' + 'MainTimerSet_' + value + '","MainTimerState_Run"]}}';
        }
        debugMessage("Setting command " + command);
        if (!$scope.isStatic) {
            $scope.checkResponse.push(COOKTOP_COMMANDS.kitchenTimer);
            SHPService.sendSHPCommand(CONSTANTS.PUT, command, "/" + $scope.peerId + "/devices/0/mode");
        }
    }

    function sendHoodTimer(value) {
        var command;
        $scope.hoodTimer = Number(value);
        if (value === '00') {
            command = '{"Mode":{"options":["' + 'ShutOffTimerSet_' + value + '","ShutOffTimerState_Ready"]}}';
        } else {
            command = '{"Mode":{"options":["' + 'ShutOffTimerSet_' + value + '","ShutOffTimerState_Run"]}}';
        }
        debugMessage("Setting command " + command);
        if (!$scope.isStatic) {
            $scope.checkResponse.push(COOKTOP_COMMANDS.hoodTimer);
            SHPService.sendSHPCommand(CONSTANTS.PUT, command, "/" + $scope.peerId + "/devices/1/mode");
        }
    }

    function hoodPowerButton() {
        var command;
        if (!$scope.hoodOff) {
            command = '{"Operation": {"power": "Off"}}';
        } else {
            command = '{"Operation": {"power": "On"}}';
        }
        //$scope.hoodOff = !$scope.hoodOff;
        debugMessage(command + "/" + $scope.peerId + "/devices/1/operation");
        if (!$scope.isStatic) {
            $scope.checkResponse.push(COOKTOP_COMMANDS.hoodPower);
            SHPService.sendSHPCommand(CONSTANTS.PUT, command, "/" + $scope.peerId + "/devices/1/operation");
        }
    }

    function setLightStatus() {
        if ($scope.hoodLight) {
            $scope.hoodLightPosition = $scope.hoodOff ? 'off_light_on' : 'on_light_on';
        } else {
            $scope.hoodLightPosition = 'off';
        }
    }

    function setFanStatus(value) {
        if (value === 1) {
            $scope.fanStatus = 'low';
        } else if (value === 2) {
            $scope.fanStatus = 'med';
        } else if (value === 3) {
            $scope.fanStatus = 'high';
        } else if (value === 4) {
            $scope.fanStatus = 'boost';
        }
    }

    function hoodFanButton(value) {
        if ($scope.selectedFanValue === value) {
            return;
        }
        var command;
        setFanStatus(value);
        $scope.selectedFanValue = value;
        command = '{"Mode":{"options":["FanSpeed_' + value + '"]}}';
        debugMessage(command + "/" + $scope.peerId + "/devices/1/mode");
        if (!$scope.isStatic) {
            $scope.checkResponse.push(COOKTOP_COMMANDS.hoodFan);
            SHPService.sendSHPCommand(CONSTANTS.PUT, command, "/" + $scope.peerId + "/devices/1/mode");
        }
    }

    function hoodLightButton() {
        var command, value = 'Off';
        $scope.hoodLight = !$scope.hoodLight;
        setLightStatus();
        value = $scope.hoodLight ? 'On' : 'Off';
        command = '{"Mode":{"options":["Lamp_' + value + '"]}}';
        debugMessage(command + "/" + $scope.peerId + "/devices/1/mode");
        if (!$scope.isStatic) {
            $scope.checkResponse.push(COOKTOP_COMMANDS.hoodLight);
            SHPService.sendSHPCommand(CONSTANTS.PUT, command, "/" + $scope.peerId + "/devices/1/mode");
        }

    }

    function goToAbout() {
        debugMessage('goToAbout');
        validateCurrentScreen('AboutPage');
    }

    function closeApp() {
        debugMessage('closeApp');
        nativeInterface.runOnNative("closeWebApp", "");
    }

    function dismissLoadingBar(value) {
        debugMessage("dismissLoadingBar " + $scope.checkResponse + " value " + value);
        if ($scope.appAtForeground) {
            return;
        }

        if ($scope.checkResponse.indexOf(value) !== -1) {
            $scope.checkResponse.splice($scope.checkResponse.indexOf(value), 1);
            dismissLoadingBar(value);
            if ($scope.checkResponse.length === 0) {
                $scope.loadingBar = false;
            }
        }
    }

    function initializeScreen() {
        debugMessage('initializeScreen  and all model check');
        if ($scope.cookTopInfo === undefined) {
            return;
        }
        $scope.DeviceModelName = $scope.cookTopInfo.Mode.options[0].DeviceType;
        debugMessage('DeviceModelName ' + $scope.DeviceModelName);
        $scope.mainLoadingScreen = true;
        $scope.showBrige = false;
        /*Note: CookTop type based on Model ID(DeviceType)
         case 0:
         NE9000K
         case 1:
         case default:
         NV9300K-/AA0 >> NZ9300K(Radiant-P, 30/36)(No Pan)
         NV9300K-/AA1 >> NZ9300K(Radiant-P+, 30)
         case 2:
         NV9300K-/AA2 >> NZ9300K(Radiant-P+, 36) 
         case 3:
         NV9300K-/AA3 >> NZ9500K(Induction-P+, 30)
         case 4:
         NV9300K-/AA4 >> NZ9500K(Induction-P+, 36)
         case 5:
         NA9300K-/AA0 >> NA9300K(Gas P) 
         NA9300K-/AA1 >> NA9300K(Gas P+)*/
        switch ($scope.DeviceModelName) {
            case "NE9000K":
                $scope.cookTopBaseImage = 'images/cooktop/burner/ct_ne9000k';
                $scope.cookTopType = '0';
                $scope.cookTopBurnerArray[0].BurnerImageID = 'burner_01';
                $scope.cookTopBurnerArray[1].BurnerImageID = 'burner_01';
                $scope.cookTopBurnerArray[2].BurnerImageID = 'burner_02';
                $scope.cookTopBurnerArray[3].BurnerImageID = 'burner_03';
                $scope.cookTopBurnerArray[4].BurnerImageID = 'burner_03';
                $scope.cookTopBurnerArray[5].BurnerImageID = 'burner_01';//not using
                break;
            case "NV9300K-/AA1":
                $scope.cookTopBaseImage = 'images/cooktop/burner/ct_ect';
                $scope.cookTopType = '1';
                $scope.showBrige = true;
                $scope.cookTopBurnerArray[0].BurnerImageID = 'burner_circle_01';
                $scope.cookTopBurnerArray[1].BurnerImageID = 'burner_circle_01';
                $scope.cookTopBurnerArray[3].BurnerImageID = 'burner_circle_05';
                $scope.cookTopBurnerArray[4].BurnerImageID = 'burner_circle_03';
                $scope.cookTopBurnerArray[5].BurnerImageID = 'burner_circle_06';
                $scope.cookTopBurnerArray[2].BurnerImageID = 'burner_circle_01';//not using
                $scope.cookTopBurnerArray[3].subBurnerId = '_02';//01-02
                $scope.cookTopBurnerArray[5].subBurnerId = '_02';//01-02
                $scope.cookTopBurnerArray[3].SubBurnerSupport = true;
                $scope.cookTopBurnerArray[5].SubBurnerSupport = true;
                break;
            case "NV9300K-/AA2":
                $scope.cookTopBaseImage = 'images/cooktop/burner/ct_ect';
                $scope.cookTopType = '2';
                $scope.showBrige = true;
                $scope.cookTopBurnerArray[0].BurnerImageID = 'burner_circle_01';
                $scope.cookTopBurnerArray[1].BurnerImageID = 'burner_circle_01';
                $scope.cookTopBurnerArray[3].BurnerImageID = 'burner_circle_02';
                $scope.cookTopBurnerArray[4].BurnerImageID = 'burner_circle_03';
                $scope.cookTopBurnerArray[5].BurnerImageID = 'burner_circle_04';
                $scope.cookTopBurnerArray[2].BurnerImageID = 'burner_circle_01';//not using
                $scope.cookTopBurnerArray[3].subBurnerId = '_02';//01-02-03
                $scope.cookTopBurnerArray[5].subBurnerId = '_02';//01-02
                $scope.cookTopBurnerArray[3].SubBurnerSupport = true;
                $scope.cookTopBurnerArray[5].SubBurnerSupport = true;
                break;
            case "NV9300K-/AA3":
                $scope.cookTopBaseImage = 'images/cooktop/burner/ct_ect';
                $scope.cookTopType = '3';
                $scope.cookTopBurnerArray[0].BurnerImageID = 'burner_flex';//TODO
                $scope.cookTopBurnerArray[1].BurnerImageID = 'burner_flex';
                $scope.cookTopBurnerArray[2].BurnerImageID = '';
                $scope.cookTopBurnerArray[3].BurnerImageID = 'burner_circle_07';
                $scope.cookTopBurnerArray[4].BurnerImageID = '';
                $scope.cookTopBurnerArray[5].BurnerImageID = 'burner_circle_08';//TODO
                $scope.cookTopBurnerArray[0].subBurnerId = '_02';//01-02
                $scope.cookTopBurnerArray[0].SubBurnerSupport = true;
                break;
            case "NV9300K-/AA4":
                $scope.cookTopBaseImage = 'images/cooktop/burner/ct_ect';
                $scope.cookTopType = '4';
                $scope.cookTopBurnerArray[0].BurnerImageID = 'burner_flex';//TODO
                $scope.cookTopBurnerArray[1].BurnerImageID = 'burner_flex';
                $scope.cookTopBurnerArray[2].BurnerImageID = '';
                $scope.cookTopBurnerArray[3].BurnerImageID = 'burner_circle_07';
                $scope.cookTopBurnerArray[4].BurnerImageID = 'burner_circle_08';
                $scope.cookTopBurnerArray[5].BurnerImageID = 'burner_circle_09';//TODO
                $scope.cookTopBurnerArray[0].subBurnerId = '_02';//01-02
                $scope.cookTopBurnerArray[0].SubBurnerSupport = true;
                break;
            case "NA9300K-/AA0":
            case "NA9300K-/AA1":
                $scope.cookTopBaseImage = 'images/cooktop/burner/ct_gct';
                $scope.cookTopType = '5';
                $scope.cookTopBurnerArray[0].BurnerImageID = 'burner_circle_01';
                $scope.cookTopBurnerArray[1].BurnerImageID = 'burner_circle_01';
                $scope.cookTopBurnerArray[2].BurnerImageID = 'burner_circle_01';//not using
                $scope.cookTopBurnerArray[3].BurnerImageID = 'burner_circle_02';
                $scope.cookTopBurnerArray[4].BurnerImageID = 'burner_circle_03';
                $scope.cookTopBurnerArray[5].BurnerImageID = 'burner_circle_04';
                break;
            case "NV9300K-/AA0":
            default:
                $scope.cookTopBaseImage = 'images/cooktop/burner/ct_ect';
                $scope.cookTopType = '1';
                $scope.cookTopBurnerArray[0].BurnerImageID = 'burner_circle_01';
                $scope.cookTopBurnerArray[1].BurnerImageID = 'burner_circle_01';
                $scope.cookTopBurnerArray[3].BurnerImageID = 'burner_circle_02';
                $scope.cookTopBurnerArray[4].BurnerImageID = 'burner_circle_03';
                $scope.cookTopBurnerArray[5].BurnerImageID = 'burner_circle_04';
                $scope.cookTopBurnerArray[2].BurnerImageID = 'burner_circle_01';//not using
                break;
        }
    }

    function showCurrentScreen() {
        debugMessage("showCurrentScreen called");
        if ($scope.cookTopInfo === undefined || $scope.HoodInfo === undefined) {
            debugMessage("cookTopInfo is undefined");
            return;
        }

        if (!parseNotification && !$scope.isStatic) {
            debugMessage("waiting for getDevice response");
            return;
        }

        if (getAlarmStatusCooktop()) {
            return;
        }

        if (changeSmartCarePage && ($scope.currentScreen === COOKTOP_SCREENS.WALLOVEN_SMART_CARE_NO_ERROR_PAGE || $scope.currentScreen === COOKTOP_SCREENS.WALLOVEN_SMART_CARE_PAGE)) {
            debugMessage('return due to Smart Care');
            return;
        }
        //loadingValues();

        $scope.showHood = $scope.HoodInfo.connected && $scope.HoodInfo.Mode.options[0].Bluetooth === 'On';
        if (!$scope.showHood) {
            $scope.cTopTabSelected = true;
            if ($scope.currentScreen === 4) {
                doNotChangeScreen = false;
                validateCurrentScreen('BurnerHomePage');
            }
        }

        if ($scope.hoodOff) {
            $scope.HoodTimerState = 'Ready';
            $scope.hoodTimer = 0;
            $scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent = 0;
            hoodTimerDrag.roundSlider({value: $scope.hoodTimer, sliderType: "min-range", tab: 2});
            if ($scope.currentScreen === 4) {
                doNotChangeScreen = false;
                validateCurrentScreen('BurnerHomePage');
            }
        }

        if (!$scope.cTopTabSelected) {
            showHoodScreen();
            return;
        }

        if ($scope.currentScreen === COOKTOP_SCREENS.COOKTOP_KITCHEN_TIMER_PAGE) {
            debugMessage('return due to COOKTOP_KITCHEN_TIMER_PAGE screen');
            KitchenTimerPage();
            return;
        }

        if ($scope.currentScreen === COOKTOP_SCREENS.COOKTOP_HOOD_TIMER_PAGE) {
            debugMessage('return due to COOKTOP_HOOD_TIMER_PAGE screen');
            gotoHoodTimer();
            return;
        }

        if (doNotChangeScreen) {
            debugMessage('return due to doNotChangeScreen');
            return;
        }
        updateCookTopBurnerValues();

        justFirstTime = false;
        showScreen();
        debugMessage("Screen Loading finish and checkResponse values " + $scope.checkResponse);
    }

    function showScreen() {
        debugMessage("showScreen called");
        $timeout(function () {
            validateCurrentScreen('BurnerHomePage');
        }, 0);
    }

    function showHoodScreen() {
        debugMessage("showHoodScreen");
        $scope.hoodOff = $scope.HoodInfo.Operation.power === 'Off' ? true : false;
        $scope.hoodLight = $scope.HoodInfo.Mode.options[0].Lamp === 'On' ? true : false;
        $scope.selectedFanValue = $scope.HoodInfo.Operation.power === 'Off' ? 1 : $scope.selectedFanValue;
        $scope.hoodTimerLeftTime = $scope.translation.WEBMOB_device_cooktop_timer_left.replace(/#1#/, $scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent);
        setLightStatus();
        setFanStatus($scope.selectedFanValue);
    }

    function loadingValues() {
        if ($scope.checkResponse.indexOf(COOKTOP_COMMANDS.kitchenTimer) !== -1) {
            $timeout(function () {
                dismissKitchenTimerCommand();
            }, 20000);
        }
    }

    function dismissKitchenTimerCommand() {
        if ($scope.checkResponse.indexOf(COOKTOP_COMMANDS.kitchenTimer) !== -1) {
            dismissLoadingBar(COOKTOP_COMMANDS.kitchenTimer);
        }
    }

    function updateCookTopBurnerValues() {
        debugMessage('updateCookTopValues');
        updateZeroBurner();
        updateFirstBurner();
        updateSecondBurner();
        updateThirdBurner();
        updateFourthBurner();
        updateFifthBurner();
        $scope.runningBurner = [];
        $scope.cookTopBurnerPause = checkPauseState(); //check for cooktop pause state

        for (var i = 0; i < $scope.cookTopBurnerArray.length; i++) {
            if ($scope.cookTopBurnerArray[i].OperationState === 'Run' || $scope.cookTopBurnerArray[i].OperationState === 'Pause') {
                $scope.runningBurner.push($scope.cookTopBurnerArray[i].id);
            }
        }
        var text = $scope.translation.WEBMOB_device_cooktop_status1;
        text = text.replace(/#1#/, $scope.runningBurner.length);
        $scope.runningBurnerText = text;
        updateBrige();
        if ($scope.flexCoilSelectedValue === -1 && !$scope.cookTopBurnerArray[0].ImageSelected &&
                !$scope.cookTopBurnerArray[1].ImageSelected &&
                !$scope.cookTopBurnerArray[2].ImageSelected &&
                !$scope.cookTopBurnerArray[3].ImageSelected &&
                !$scope.cookTopBurnerArray[4].ImageSelected &&
                !$scope.cookTopBurnerArray[5].ImageSelected &&
                (document.getElementById("burnerDetail") !== undefined && document.getElementById("burnerDetail") !== null)) {
            document.getElementById("burnerDetail").style.visibility = "hidden";
        }
    }

    function updateZeroBurner() {
        //$scope.cookTopBurnerArray[0].ImageOff = true;//$scope.cookTopBurnerArray[0].OperationState === 'Ready';
        if (!$scope.isModelAA34()) {
            $scope.cookTopBurnerArray[0].ImageOn = $scope.cookTopBurnerArray[0].OperationState === 'Run' || $scope.cookTopBurnerArray[0].OperationState === 'Pause';
            $scope.cookTopBurnerArray[0].UpperText = getUpperTextStatus(0);
            $scope.cookTopBurnerArray[0].LowerText = getLowerTextStatus(0);
            $scope.cookTopBurnerArray[0].CenterText = getCenerTextStatus(0);
        } else {
            // ch0 upper
            $scope.flexCoilStatus[0].UpperText = getFlexUpperTextStatus(0, 0);
            $scope.flexCoilStatus[0].LowerText = getLowerTextStatus(0, 0);
            $scope.flexCoilStatus[0].CenterText = getFlexCenerTextStatus(0, 0);
            // ch0 lower
            $scope.flexCoilStatus[1].UpperText = getFlexUpperTextStatus(0, 1);
            $scope.flexCoilStatus[1].LowerText = getLowerTextStatus(0, 1);
            $scope.flexCoilStatus[1].CenterText = getFlexCenerTextStatus(0, 1);
        }
    }

    function updateFirstBurner() {
        //$scope.cookTopBurnerArray[1].ImageOff = true;//$scope.cookTopBurnerArray[1].OperationState === 'Ready';
        if (!$scope.isModelAA34()) {
            $scope.cookTopBurnerArray[1].ImageOn = $scope.cookTopBurnerArray[1].OperationState === 'Run' || $scope.cookTopBurnerArray[1].OperationState === 'Pause';
            $scope.cookTopBurnerArray[1].UpperText = getUpperTextStatus(1);
            $scope.cookTopBurnerArray[1].LowerText = getLowerTextStatus(1);
            $scope.cookTopBurnerArray[1].CenterText = getCenerTextStatus(1);
        } else {
            // ch1 upper
            $scope.flexCoilStatus[2].UpperText = getFlexUpperTextStatus(1, 2);
            $scope.flexCoilStatus[2].LowerText = getLowerTextStatus(1, 2);
            $scope.flexCoilStatus[2].CenterText = getFlexCenerTextStatus(1, 2);
            // ch1 lower
            $scope.flexCoilStatus[3].UpperText = getFlexUpperTextStatus(1, 3);
            $scope.flexCoilStatus[3].LowerText = getLowerTextStatus(1, 3);
            $scope.flexCoilStatus[3].CenterText = getFlexCenerTextStatus(1, 3);
        }
    }

    function updateSecondBurner() {
        //scope.cookTopBurnerArray[2].ImageOff = true;//$scope.cookTopBurnerArray[2].OperationState === 'Ready';
        $scope.cookTopBurnerArray[2].ImageOn = $scope.cookTopBurnerArray[2].OperationState === 'Run' || $scope.cookTopBurnerArray[2].OperationState === 'Pause';
        $scope.cookTopBurnerArray[2].UpperText = getUpperTextStatus(2);
        $scope.cookTopBurnerArray[2].LowerText = getLowerTextStatus(2);
        $scope.cookTopBurnerArray[2].CenterText = getCenerTextStatus(2);
    }

    function updateThirdBurner() {
        //$scope.cookTopBurnerArray[3].ImageOff = true;//$scope.cookTopBurnerArray[3].OperationState === 'Ready';
        $scope.cookTopBurnerArray[3].ImageOn = $scope.cookTopBurnerArray[3].OperationState === 'Run' || $scope.cookTopBurnerArray[3].OperationState === 'Pause';
        $scope.cookTopBurnerArray[3].UpperText = getUpperTextStatus(3);
        $scope.cookTopBurnerArray[3].LowerText = getLowerTextStatus(3);
        $scope.cookTopBurnerArray[3].CenterText = getCenerTextStatus(3);
        if ($scope.cookTopBurnerArray[3].OperationState === 'Run' && $scope.cookTopBurnerArray[3].SubBurnerSupport) {
            if ($scope.cookTopBurnerArray[3].PanSize === 'Single') {
                $scope.cookTopBurnerArray[3].subBurnerId = '_01';
            } else if ($scope.cookTopBurnerArray[3].PanSize === 'Dual') {
                $scope.cookTopBurnerArray[3].subBurnerId = '_02';
            } else if ($scope.cookTopBurnerArray[3].PanSize === 'Triple') {
                $scope.cookTopBurnerArray[3].subBurnerId = '_03';
            }
        } else {
            $scope.cookTopBurnerArray[3].subBurnerId = '';
        }
        if ($scope.cookTopBurnerArray[3].subBurnerId === '' && $scope.cookTopBurnerArray[3].SubBurnerSupport) {
            $scope.cookTopBurnerArray[3].subBurnerId = '_02';
        }
    }

    function updateFourthBurner() {
        //$scope.cookTopBurnerArray[4].ImageOff = true;//$scope.cookTopBurnerArray[4].OperationState === 'Ready';
        $scope.cookTopBurnerArray[4].ImageOn = $scope.cookTopBurnerArray[4].OperationState === 'Run' || $scope.cookTopBurnerArray[4].OperationState === 'Pause';
        $scope.cookTopBurnerArray[4].UpperText = getUpperTextStatus(4);
        $scope.cookTopBurnerArray[4].LowerText = getLowerTextStatus(4);
        $scope.cookTopBurnerArray[4].CenterText = getCenerTextStatus(4);
        if ($scope.cookTopBurnerArray[4].OperationState === 'Run' && $scope.cookTopBurnerArray[4].SubBurnerSupport) {
            if ($scope.cookTopBurnerArray[4].PanSize === 'Single') {
                $scope.cookTopBurnerArray[4].subBurnerId = '_01';
            } else if ($scope.cookTopBurnerArray[5].PanSize === 'Dual') {
                $scope.cookTopBurnerArray[4].subBurnerId = '_02';
            } else if ($scope.cookTopBurnerArray[5].PanSize === 'Triple') {
                $scope.cookTopBurnerArray[4].subBurnerId = '_03';
            }
        } else {
            $scope.cookTopBurnerArray[4].subBurnerId = '';
        }
        if ($scope.cookTopBurnerArray[4].subBurnerId === '' && $scope.cookTopBurnerArray[4].SubBurnerSupport) {
            $scope.cookTopBurnerArray[4].subBurnerId = '_02';
        }
    }

    function updateFifthBurner() {
        //$scope.cookTopBurnerArray[5].ImageOff = true;//$scope.cookTopBurnerArray[5].OperationState === 'Ready';
        $scope.cookTopBurnerArray[5].ImageOn = $scope.cookTopBurnerArray[5].OperationState === 'Run' || $scope.cookTopBurnerArray[5].OperationState === 'Pause';
        $scope.cookTopBurnerArray[5].UpperText = getUpperTextStatus(5);
        $scope.cookTopBurnerArray[5].LowerText = getLowerTextStatus(5);
        $scope.cookTopBurnerArray[5].CenterText = getCenerTextStatus(5);
        if ($scope.cookTopBurnerArray[5].OperationState === 'Run' && $scope.cookTopBurnerArray[5].SubBurnerSupport) {
            if ($scope.cookTopBurnerArray[5].PanSize === 'Single') {
                $scope.cookTopBurnerArray[5].subBurnerId = '_01';
            } else if ($scope.cookTopBurnerArray[5].PanSize === 'Dual') {
                $scope.cookTopBurnerArray[5].subBurnerId = '_02';
            } else if ($scope.cookTopBurnerArray[5].PanSize === 'Triple') {
                $scope.cookTopBurnerArray[5].subBurnerId = '_03';
            }
        } else {
            $scope.cookTopBurnerArray[5].subBurnerId = '';
        }
        if ($scope.cookTopBurnerArray[5].subBurnerId === '' && $scope.cookTopBurnerArray[5].SubBurnerSupport) {
            $scope.cookTopBurnerArray[5].subBurnerId = '_02';
        }
    }

    function getCurrentStatus(id) {
        var value = 'off';
        if (id !== undefined) {
            if ($scope.cookTopBurnerArray[id].OperationState === 'Ready') {
                value = 'off';
            }
            if ($scope.cookTopBurnerArray[id].OperationState === 'Run') {
                value = 'on';
            }
            if ($scope.cookTopBurnerArray[id].Selected) {
                value = 'select';
            }
        }
        //debugMessage('current value of ' + id + ' burner is : ' + value + " && " + $scope.runningBurner);
        return value;
    }

    function getCenerTextStatus(id) {
        var value = '';
        if (id !== undefined) {
            if ($scope.cookTopBurnerArray[id].OperationState === 'Ready') {
                value = 'OFF';
            }
            if ($scope.cookTopBurnerArray[id].OperationState === 'Run' && ($scope.cookTopBurnerArray[id].UpperText === '' || $scope.cookTopBurnerArray[id].LowerText === '')) {
                value = 'ON';
                $scope.cookTopBurnerArray[id].UpperText = '';
                $scope.cookTopBurnerArray[id].LowerText = '';
            }
        }
        if ($scope.cookTopBurnerArray[id].ImageSelected) {
            if ($scope.cookTopBurnerArray[id].OperationState === 'Pause') {
                var text = $scope.translation.WEBMOB_device_cooktop_detailed_level_info;
                text = text.replace(/#1#/, 2);
                $scope.selectedBurnerDetail = text;
            } else {
                var detailText = "";
                detailText = $scope.cookTopBurnerArray[id].LowerText;
                if (detailText !== '') {
                    if (detailText.indexOf($scope.translation.WEBMOB_device_cooktop_melt) !== -1) {
                        $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_melt;
                    } else if (detailText.indexOf($scope.translation.WEBMOB_device_cooktop_simmer) !== -1) {
                        $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_simmer;
                    } else if (detailText.indexOf($scope.translation.WEBMOB_device_hood_boost) !== -1) {
                        $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_boost;
                    } else if (detailText.indexOf($scope.translation.WEBMOB_device_cooktop_hot) !== -1 || detailText.indexOf($scope.translation.WEBMOB_device_cooktop_veryhot) !== -1) {
                        $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_hot_alarm;
                    } else {
                        var text = $scope.translation.WEBMOB_device_cooktop_detailed_level_info;
                        var splitLevelInfo = detailText.split(' ');
                        var tmpVal = (splitLevelInfo instanceof Array && angular.isDefined(splitLevelInfo[1])) ? splitLevelInfo[1] : detailText;
                        text = text.replace(/#1#/, tmpVal);
                        $scope.selectedBurnerDetail = text;
                    }
                } else {
                    var text = $scope.translation.WEBMOB_device_cooktop_burner_state;
                    text = text.replace(/#1#/, value.toLowerCase());
                    $scope.selectedBurnerDetail = text;
                }
            }
        }
        //debugMessage('current value of ' + id + ' Center text is : ' + value);
        return value;
    }

    function getFlexCenerTextStatus(id, flexid) {
        var value = '';
        if (id !== undefined) {
            if ($scope.flexCoilStatus[flexid].UpperText === '' || $scope.flexCoilStatus[flexid].LowerText === '') {
                if ($scope.flexCoilStatus[flexid].FlexOn) {
                    value = 'ON';
                } else {
                    value = 'OFF';
                }
                $scope.flexCoilStatus[flexid].UpperText = '';
                $scope.flexCoilStatus[flexid].LowerText = '';
            }
        }
        //debugMessage('current value of ' + id + ' Center text is : ' + value);
        return value;
    }

    function getUpperTextStatus(id) {
        var value = '';
        if (id !== undefined) {
            if ($scope.cookTopBurnerArray[id].OperationState === 'Run' || $scope.cookTopBurnerArray[id].OperationState === 'Pause') {
                value = 'ON';
            }
        }
        //debugMessage('current value of ' + id + ' Upper text is : ' + value);
        return value;
    }

    function getFlexUpperTextStatus(id, flexId) {
        var value = '';
        if (id !== undefined) {
            if ($scope.flexCoilStatus[flexId].FlexOn) {
                value = 'ON';
            }
            if (!$scope.flexCoilStatus[flexId].FlexOn && ($scope.cookTopBurnerArray[id].HotSurface === 'SmallHot' || $scope.cookTopBurnerArray[id].HotSurface === 'Hot')) {
                value = 'OFF';
            }
        }
        //debugMessage('current value of ' + id + ' Upper text is : ' + value);
        return value;
    }

    function getLowerTextStatus(id, flexId) {
        var value = '';
        if (id !== undefined) {
            var upperText = (angular.isDefined(flexId) && angular.isDefined($scope.flexCoilStatus[flexId])) ? $scope.flexCoilStatus[flexId].UpperText : $scope.cookTopBurnerArray[id].UpperText;
            if ($scope.cookTopBurnerArray[id].OperationState === 'Pause' && upperText !== '' && upperText !== 'OFF') {
                var text = $scope.translation.WEBMOB_device_cooktop_level_info;
                text = text.replace(/#1#/, 2);
                value = text;
                if ($scope.cookTopBurnerArray[id].ImageSelected) {//todo chek flex status selected
                    var text = $scope.translation.WEBMOB_device_cooktop_detailed_level_info;
                    text = text.replace(/#1#/, 2);
                    $scope.selectedBurnerDetail = text;
                }
                return value;
            }
            if ($scope.cookTopBurnerArray[id].OperationState === 'Run' && upperText !== '') {
                var powerLevel = Number($scope.cookTopBurnerArray[id].PowerLevel);
                if (0 <= powerLevel && powerLevel <= 999) {
                    if (powerLevel !== 0) {
                        var text = $scope.translation.WEBMOB_device_cooktop_level_info;
                        text = text.replace(/#1#/, $scope.cookTopBurnerArray[id].PowerLevel);
                        value = text;
                    } else {
                        value = '';
                    }
                } else if ($scope.cookTopBurnerArray[id].PowerLevel !== 'Off') {
                    var status = $scope.cookTopBurnerArray[id].PowerLevel;
                    if (status.toLowerCase().indexOf("melt") !== -1) {
                        value = $scope.translation.WEBMOB_device_cooktop_melt;
                    } else if (status.toLowerCase().indexOf("simmer") !== -1) {
                        value = $scope.translation.WEBMOB_device_cooktop_simmer;
                    } else if (status.toLowerCase().indexOf("boost") !== -1) {
                        value = $scope.translation.WEBMOB_device_hood_boost;
                    } else {
                        value = $scope.cookTopBurnerArray[id].PowerLevel;
                    }
                }
            }

            if ($scope.cookTopBurnerArray[id].OperationState === 'Ready' || (angular.isDefined(flexId) && angular.isDefined($scope.flexCoilStatus[flexId]) && !$scope.flexCoilStatus[flexId].FlexOn)) {
                if ($scope.cookTopBurnerArray[id].HotSurface === 'SmallHot') {
                    var text = $scope.translation.WEBMOB_device_cooktop_hot;//'Hot';
                    text = text.replace(/#1#/, $scope.cookTopBurnerArray[id].HotSurface);
                    value = text;
                } else if ($scope.cookTopBurnerArray[id].HotSurface === 'Hot') {
                    var text = $scope.translation.WEBMOB_device_cooktop_veryhot;//'Very Hot';
                    text = text.replace(/#1#/, $scope.cookTopBurnerArray[id].HotSurface);
                    value = text;
                } else {
                    ;//NULL
                }
            }

            /*TODO -- keep this logic outside and call after updating all burner values
             * Can be called at the bottom of updateCookTopBurnerValues method
             * */
            if ($scope.cookTopBurnerArray[id].ImageSelected) {
                var detailText = "";
                if ((angular.isDefined(flexId) && angular.isDefined($scope.flexCoilStatus[flexId]) && $scope.flexCoilSelectedValue !== -1)) {
                    detailText = $scope.flexCoilStatus[$scope.flexCoilSelectedValue].LowerText;
                } else {
                    detailText = $scope.cookTopBurnerArray[id].LowerText;
                }

                if (detailText !== '') {
                    if (detailText.indexOf($scope.translation.WEBMOB_device_cooktop_melt) !== -1) {
                        $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_melt;
                    } else if (detailText.indexOf($scope.translation.WEBMOB_device_cooktop_simmer) !== -1) {
                        $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_simmer;
                    } else if (detailText.indexOf($scope.translation.WEBMOB_device_hood_boost) !== -1) {
                        $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_boost;
                    } else if (detailText.indexOf($scope.translation.WEBMOB_device_cooktop_hot) !== -1 || detailText.indexOf($scope.translation.WEBMOB_device_cooktop_veryhot) !== -1) {
                        $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_hot_alarm;
                    } else {
                        var text = $scope.translation.WEBMOB_device_cooktop_detailed_level_info;
                        var splitLevelInfo = detailText.split(' ');
                        var tmpVal = (splitLevelInfo instanceof Array && angular.isDefined(splitLevelInfo[1])) ? splitLevelInfo[1] : detailText;
                        text = text.replace(/#1#/, tmpVal);
                        $scope.selectedBurnerDetail = text;
                    }
                } else {
                    if ((angular.isDefined(flexId) && angular.isDefined($scope.flexCoilStatus[flexId]) && $scope.flexCoilSelectedValue !== -1)) {
                        detailText = $scope.flexCoilStatus[$scope.flexCoilSelectedValue].CenterText;
                    } else {
                        detailText = $scope.cookTopBurnerArray[id].CenterText;
                    }

                    var text = $scope.translation.WEBMOB_device_cooktop_burner_state;
                    text = text.replace(/#1#/, detailText.toLowerCase());
                    $scope.selectedBurnerDetail = text;
                }
            }
        }
        //debugMessage('current value of ' + id + ' Lower text is : ' + value);
        return value;
    }

    function selectBurner(burnerId) {
        debugMessage('selected Burner ' + burnerId);
        bridgeSelected = 'off';
        $scope.flexCoilSelectedValue = -1;
        $scope.cookTopBurnerArray[0].ImageSelected = false;
        $scope.cookTopBurnerArray[1].ImageSelected = false;
        $scope.cookTopBurnerArray[2].ImageSelected = false;
        $scope.cookTopBurnerArray[3].ImageSelected = false;
        $scope.cookTopBurnerArray[4].ImageSelected = false;
        $scope.cookTopBurnerArray[5].ImageSelected = false;
        if (document.getElementById('burnerDetail') !== null) {
            document.getElementById("burnerDetail").style.visibility = "hidden";
        }
        if (burnerId === -2) {
            if ($scope.cookTopInfo.Mode.options[0].SyncFlex === 'On') {
                $scope.cookTopBurnerArray[0].ImageSelected = true;
                $scope.cookTopBurnerArray[1].ImageSelected = true;
            }
        } else if (burnerId === -1) {
            ;//NULL
        } else {
            $scope.cookTopBurnerArray[burnerId].ImageSelected = true;
            if (document.getElementById('burnerDetail') !== null) {
                document.getElementById("burnerDetail").style.visibility = "visible";
            }
            if ($scope.cookTopBurnerArray[burnerId].PanSize === 'Single') {
                $scope.cookTopBurnerArray[burnerId].subBurnerId = '_01';
            } else if ($scope.cookTopBurnerArray[burnerId].PanSize === 'Dual') {
                $scope.cookTopBurnerArray[burnerId].subBurnerId = '_02';
            } else if ($scope.cookTopBurnerArray[burnerId].PanSize === 'Triple') {
                $scope.cookTopBurnerArray[burnerId].subBurnerId = '_03';
            } else {
                $scope.cookTopBurnerArray[burnerId].subBurnerId = '';
            }
            var value = $scope.cookTopBurnerArray[burnerId].LowerText;
            if (value.indexOf($scope.translation.WEBMOB_device_cooktop_melt) !== -1) {
                $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_melt;
            } else if (value.indexOf($scope.translation.WEBMOB_device_cooktop_simmer) !== -1) {
                $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_simmer;
            } else if (value.indexOf($scope.translation.WEBMOB_device_hood_boost) !== -1) {
                $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_boost;
            } else if (value.indexOf($scope.translation.WEBMOB_device_cooktop_hot) !== -1 || value.indexOf($scope.translation.WEBMOB_device_cooktop_veryhot) !== -1) {
                $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_hot_alarm;
            } else {
                var text = $scope.translation.WEBMOB_device_cooktop_detailed_level_info;
                var splitLevelInfo = $scope.cookTopBurnerArray[burnerId].PowerLevel.split(' ');
                var val = (splitLevelInfo instanceof Array && angular.isDefined(splitLevelInfo[1])) ? splitLevelInfo[1] : $scope.cookTopBurnerArray[burnerId].PowerLevel;
                text = text.replace(/#1#/, val);
                $scope.selectedBurnerDetail = text;
            }

            if (value === '') {
                value = $scope.cookTopBurnerArray[burnerId].CenterText.toLowerCase();
                var text = $scope.translation.WEBMOB_device_cooktop_burner_state;
                text = text.replace(/#1#/, value);
                $scope.selectedBurnerDetail = text;
            }
        }

        updateBrige();
        updateCookTopBurnerValues();
    }

    function updateBrige() {
        if ($scope.showBrige) {
            if ($scope.cookTopInfo.Mode.options[0].SyncFlex === 'On') {
                if ($scope.cookTopBurnerArray[0].ImageSelected || $scope.cookTopBurnerArray[1].ImageSelected) {
                    bridgeSelected = 'select';
                    $scope.cookTopBurnerArray[0].ImageSelected = true;
                    $scope.cookTopBurnerArray[1].ImageSelected = true;
                    var text = $scope.translation.WEBMOB_device_cooktop_detailed_level_info;
                    var splitLevelInfo = $scope.cookTopBurnerArray[0].LowerText.split(' ');
                    var val = (splitLevelInfo instanceof Array && angular.isDefined(splitLevelInfo[1])) ? splitLevelInfo[1] : $scope.cookTopBurnerArray[0].LowerText;
                    text = text.replace(/#1#/, val);
                    $scope.selectedBurnerDetail = text;
                } else {
                    if (Number($scope.cookTopBurnerArray[0].PowerLevel) > 0) {
                        bridgeSelected = 'on';
                    } else {
                        bridgeSelected = 'off';
                    }
                    $scope.cookTopBurnerArray[0].ImageSelected = false;
                    $scope.cookTopBurnerArray[1].ImageSelected = false;
                }
            } else {
                bridgeSelected = 'off';
                if ($scope.cookTopBurnerArray[0].ImageSelected && $scope.cookTopBurnerArray[1].ImageSelected) {
                    $scope.cookTopBurnerArray[0].ImageSelected = false;
                    $scope.cookTopBurnerArray[1].ImageSelected = false;
                }
            }
            $scope.cookTopBurnerBridge = $scope.cookTopBaseImage + '_burner_sync_' + bridgeSelected;
        }
    }

    function updateTimerValue(value) {
        debugMessage('updateTimerValue according to ' + value);
        value = Number(value);
        document.getElementById('kitchenTimer').value = value;
        if (value < 10) {
            document.getElementById('kitchenTimer').value = '0' + value;
        }
        if (value >= 1) {
            $scope.timerText = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step2;
        } else {
            $scope.timerText = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step1;
        }
        if ($scope.KitchenTimerState === 'Run' && (value === $scope.cookTopInfo.Mode.options[0].MainTimerCurrent || value < 1)) {
            $scope.startStopState = $scope.translation.WEBMOB_device_hood_stop_button;
            /*
             * In run state if user swipes the round progress bar to value 0, change the stop message 
             */
            $scope.timerStopText = (value < 1) ? $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step1 : $scope.translation.WEBMOB_device_hood_set_time_guide4;
        } else if ($scope.KitchenTimerState === 'Pause') {
            $scope.startStopState = $scope.translation.WEBMOB_common_cooktop_pause;//'PAUSE'
            $scope.timerStopText = $scope.translation.WEBMOB_device_hood_set_time_guide4;
        } else {
            $scope.startStopState = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_start;
            $scope.timerStopText = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step2;
        }
    }

    function updateHoodTimerValue(value) {
        debugMessage('updateHoodTimerValue according to ' + value);
        value = Number(value);
        document.getElementById('hoodTimer').value = value;
        if (value < 10) {
            document.getElementById('hoodTimer').value = '0' + value;
        }
        if (value >= 1) {
            $scope.timerTextHood = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step2;
        } else {
            $scope.timerTextHood = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step1;
        }

        if ($scope.HoodTimerState === 'Run' && (value === $scope.HoodInfo.Mode.options[0].ShutOffTimerCurrent || value < 1)) {
            $scope.startStopHoodState = $scope.translation.WEBMOB_device_hood_stop_button;
            /*
             * In run state if user swipes the round progress bar to value 0, change the stop message 
             */
            $scope.timerStopTextHood = (value < 1) ? $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step1 : $scope.translation.WEBMOB_device_hood_set_time_guide4;
        } else {
            $scope.startStopHoodState = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_start;
            $scope.timerStopTextHood = $scope.translation.WEBMOB_device_cooktop_kitchen_timer_step2;
        }
    }

    function inputKeyDown(e) {
        if (e === undefined) {
            return;
        }
        var id = e.currentTarget.id;
        document.getElementById(id).onkeyup = function (e) {
            if (this.value === '' || !angular.isDefined(Number(this.value))) {
                e.preventDefault();
                roundShape.roundSlider({value: 0, sliderType: "min-range", tab: roundShape === hoodTimerDrag ? 2 : 1});
                document.getElementById(id).value = '';
                return;
            }
            checkForKey(e, this.value, id);
        };
    }

    function checkForKey(e, str, id) {
        var key = e.keyCode || e.which;
        if (key === 0 || key === 229) {
            key = str.charCodeAt(str.length - 1);
        }
        if ((key >= 96 && key <= 105 || key >= 48 && key <= 57) || key !== 8 || key !== 46) {
            var textValue = document.getElementById(id).value;
            roundShape.roundSlider({value: Number(textValue), sliderType: "min-range", tab: roundShape === hoodTimerDrag ? 2 : 1});
            document.getElementById(id).value = textValue;
        }

        if (key === 9 && (e.currentTarget.id === "kitchenTimer")) {
            showDummyInbox();
        }

        if (!(key >= 96 && key <= 105 || key >= 48 && key <= 57) && key !== 9 && key !== 8 && key !== 46) {
            e.preventDefault();
            return;
        }
    }

    function showDummyInbox() {
        debugMessage("showDummyInbox");
        debugMessage("BLUR CALLED>>>>>>>>>>>> showDummyInbox", arguments.callee.caller.name);
        if ((window.navigator.platform !== 'iPhone' || window.navigator.platform !== 'iPad')) {
            $scope.isTempFocusOn = true;
            document.activeElement.blur();
        }
    }

    function hideDummyInbox() {
        debugMessage("hideDummyInbox");
        $scope.isTempFocusOn = false;
    }

    function disableDummyInbox() {
        debugMessage("disableDummyInbox");
        if (document.getElementById("dummyInputBox") !== null) {
            document.getElementById("dummyInputBox").disabled = true;
        }
    }

    function showToast(txt) {
        debugMessage("showToast with txt " + txt);
        $timeout(function () {
            toaster.success({title: "", body: txt, bodyOutputType: 'trustedHtml'});
        }, 500);
    }

    function blurInputBox(id) {
        debugMessage("BLUR CALLED>>>>>>>>>>>> blurInputBox");
        if (document.getElementById(id) !== null && document.getElementById(id) !== undefined) {
            document.getElementById(id).blur();
            if (isTextInput(document.activeElement)) {
                document.activeElement.blur();
            }
        }
    }

    function showcustDialog(myAlertState, msgHeader, popuptext, txtButton1, txtButton2) {
        debugMessage("showcustDialog " + myAlertState + " " + msgHeader + " " + popuptext + " " + txtButton1 + " " + txtButton2);
        $scope.alertState = myAlertState;
        $scope.custPopupTitle = true;
        $scope.custPopupOKBtton = true;
        $scope.showFullScreenPopup = true;
        $scope.custPopupCancelButton = true;
        $scope.custPopupTitleText = msgHeader;
        $scope.custPopupMessageText = popuptext;
        $scope.custPopupCancelOkText = txtButton1;
        $scope.custPopupCancelButtonText = txtButton2;

        $scope.showMenuOptionPopup = false;

        fixScreen();
        $timeout(function () {
            document.getElementById("fullScreenPopupId").style.zIndex = 999;
            document.getElementById("custPopupDivId").style.zIndex = 999;
        }, 0);
        /*$timeout(function () {
         document.getElementById("bodyTag").style.pointerEvents = 'none';
         }, 200);*/
        $timeout(function () {
            var cHeight = 0, cWidth = 0, popupHeight = 0, base = 0, popupTop = 12;
            cHeight = window.screen.availHeight;
            cWidth = window.screen.availWidth;
            if (cHeight === 0 || cHeight === undefined) {
                cHeight = document.body.clientHeight;
            }
            if (cWidth === 0 || cWidth === undefined) {
                cWidth = document.body.clientWidth;
            }
            base = 2 * ((80 * cWidth) / 1440);
            popupHeight = document.getElementById("custPopupDivId").clientHeight;
            if (popupHeight !== 0 && popupHeight !== undefined && popupHeight !== '' && base > 0) {
                popupTop = (cHeight - popupHeight) / base;
            }
            document.getElementById("custPopupDivId").style.top = popupTop + 'rem';
        }, 10);


        if (msgHeader === undefined || msgHeader === '') {
            $scope.custPopupTitle = false;
        }
        if (txtButton1 === undefined || txtButton1 === '') {
            $scope.custPopupOKBtton = false;
        }
        if (txtButton2 === undefined || txtButton2 === '') {
            $scope.custPopupCancelButton = false;
        }

    }

    function onAlertButton1Click() {
        debugMessage("onAlertButton1Click " + $scope.alertState);
        $scope.showFullScreenPopup = false;
        scrollScreen();
        $timeout(function () {
            switch ($scope.alertState) {
                case COOKTOP_ALERT_STATE.ALERT_NONE:
                    break;
                case COOKTOP_ALERT_STATE.ALERT_CONNECTION_ERROR:
                    closeApp();
                    break;
                case COOKTOP_ALERT_STATE.ALERT_TIMER_FINISH:
                    debugMessage("Kitchen Timer pop up");
                    break;
                case COOKTOP_ALERT_STATE.ALERT_HOOD_TIMER_FINISH:
                    debugMessage("Hood Timer pop up");
                    break;
                case COOKTOP_ALERT_STATE.ALERT_DEVELOPER_INFO:
                    debugMessage("Developer info pop up");
                    break;
                case COOKTOP_ALERT_STATE.ALERT_CHILD_LOCK:
                    debugMessage("Child lock pop up");
                    break;
                case COOKTOP_ALERT_STATE.ALERT_SERVICE_CALL:
                    callServieCenter();
                    break;
                default:
                    break;
            }
        }, 200);

    }

    function onAlertButton2Click() {
        debugMessage("onAlertButton2Click " + $scope.alertState);
        $scope.showFullScreenPopup = false;
        scrollScreen();
        $timeout(function () {
            switch ($scope.alertState) {
                case COOKTOP_ALERT_STATE.ALERT_START_COOKING:
                    break;
                case COOKTOP_ALERT_STATE.ALERT_CONNECTION_ERROR:
                    debugMessage("cancel connection pop up");
                    break;
                case COOKTOP_ALERT_STATE.ALERT_TIMER_FINISH:
                    debugMessage("ok Kitchen Timer pop up");
                    break;
                case COOKTOP_ALERT_STATE.ALERT_HOOD_TIMER_FINISH:
                    debugMessage("Hood Timer pop up");
                    $scope.cTopTabSelected = false;
                    showHoodScreen();
                    validateCurrentScreen('BurnerHomePage');
                    break;
                case COOKTOP_ALERT_STATE.ALERT_DEVELOPER_INFO:
                    debugMessage("ok Developer info pop up");
                    break;
                case COOKTOP_ALERT_STATE.ALERT_CHILD_LOCK:
                    debugMessage("ok Child lock pop up");
                    break;
                case COOKTOP_ALERT_STATE.ALERT_SERVICE_CALL:
                    debugMessage("cancel service center call");
                    break;
                default:
                    break;
            }
            debugMessage("onAlertButton2Click");
        }, 200);
    }

    function smartcareerrorcheck() {
        if (!$scope.isStatic) {
            $scope.checkResponse.push(COOKTOP_COMMANDS.smartCareDiagnosis);
            debugMessage("Send command for smartCareDiagnosis " + getDiagnosisCommand());
            SHPService.sendSHPCommand(CONSTANTS.PUT, getDiagnosisCommand(), "/" + $scope.peerId + "/devices/0/diagnosis");
        } else {
            $scope.cookTopInfo.Alarms[0].code = "CT_E_F0";
            showCurrentScreen();
            //var temp = '[{"Alarms":[{"id":"0","code":"CT_E_d0","triggeredTime":"2016-07-08T10:59:28"}],"event":"Created","eventTime":"2016-07-08T10:59:28","resourceURI":"/devices/0/alarms","uuid":"C0972715-55BF-0000-0000-000000000000"},{"Operation":{"power":"Off"},"event":"Notified","eventTime":"2016-07-08T10:59:28","resourceURI":"/devices/0/operation","uuid":"C0972715-55BF-0000-0000-000000000000"},{"Mode":{"options":["PowerLevel1_Off"]},"event":"Notified","eventTime":"2016-07-08T10:59:28","resourceURI":"/devices/0/mode","uuid":"C0972715-55BF-0000-0000-000000000000"}]';
            //parseNotificationData(JSON.parse(temp)[0]);
        }
    }

    function getDiagnosisCommand() {
        return '{"Diagnosis": {"diagnosisStart": "Run"}}';
    }

    function getErrorCode(errorEnum) {
        switch (errorEnum) {
            case 'CT_E_d0':
                return "C-d0";
            case 'CT_E_F0':
                return "C-F0";
            case 'CT_E_F1':
                return "C-F1";
            case 'CT_E_F2':
                return "C-F2";
            case 'CT_E_30':
                return "C-30";
            case 'CT_E_31':
                return "C-31";
            case 'CT_E_A2':
                return "C-A2";
            case 'CT_E_C0':
                return "C-C0";
            case 'CT_E_C1':
                return "C-C1";
            case 'CT_E_C2':
                return "C-C2";
            case 'undefined':
            default:
                return "";
        }
    }

    function getAlarmStatusCooktop() {
        //$scope.cookTopInfo.Alarms[0].code = "CT_E_F0";
        if ($scope.cookTopInfo.Alarms[0] !== undefined && $scope.cookTopInfo.Alarms[0] !== null && $scope.cookTopInfo.Alarms[0].code !== '-1') {
            debugMessage('Check for alarm status 0 :' + $scope.cookTopInfo.Alarms[0].code);

            if ($scope.cookTopInfo.Alarms[0].code === 'CT_E') {
                $scope.cookTopInfo.Alarms[0].code = '-1';
                errorRemoved = true;
                return false;
            }

            if (!errorRemoved) {
                debugMessage('Check error status');
                return;
            }

            if ($scope.cookTopInfo.Alarms[0].code.indexOf("CT_E") !== -1) {
                $scope.errorCode = getErrorCode($scope.cookTopInfo.Alarms[0].code);
                if ($scope.errorCode !== '') {
                    errorRemoved = false;
                    validateCurrentScreen('ErrorPage');
                    return true;
                } else {
                    return false;
                }
            }
        }
        return false;
    }

    function doOnServiceCallPopup() {
        showcustDialog(COOKTOP_ALERT_STATE.ALERT_SERVICE_CALL, $scope.translation.WEBMOB_common_service_call, ($scope.translation.WEBMOB_common_connect_service_call) + "<br>" + $scope.serviceCenterNumber, $scope.translation.WEBMOB_common_cancel_capital, $scope.translation.WEBMOB_common_ok);
    }

    function callServieCenter() {
        debugMessage("call service center");
        nativeInterface.runOnNative("callDial", '{"selectedCountry" : "' + $scope.selectedCountry + '"}');
    }

    function aboutCommand(cmdstring) {
        return '{"service" : "' + cmdstring + '","selectedCountry" : "' + $scope.selectedCountry + '"}';
    }

    function isModelAA34() {
        return ($scope.DeviceModelName === 'NV9300K-/AA3' || $scope.DeviceModelName === 'NV9300K-/AA4');
    }

    function onFlexCoilClicked(val) {
        debugMessage('onFlexCoilClicked :' + val);
        $scope.flexCoilSelectedValue = val;
        $scope.cookTopBurnerArray[0].ImageSelected = ($scope.flexCoilSelectedValue > 1) ? false : true;
        $scope.cookTopBurnerArray[1].ImageSelected = ($scope.flexCoilSelectedValue < 2) ? false : true;
        $scope.cookTopBurnerArray[2].ImageSelected = false;
        $scope.cookTopBurnerArray[3].ImageSelected = false;
        $scope.cookTopBurnerArray[4].ImageSelected = false;
        $scope.cookTopBurnerArray[5].ImageSelected = false;

        if (document.getElementById('burnerDetail') !== null) {
            document.getElementById("burnerDetail").style.visibility = "visible";
        }

        var value = $scope.flexCoilStatus[val].LowerText;
        if (value.indexOf($scope.translation.WEBMOB_device_cooktop_melt) !== -1) {
            $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_melt;
        } else if (value.indexOf($scope.translation.WEBMOB_device_cooktop_simmer) !== -1) {
            $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_simmer;
        } else if (value.indexOf($scope.translation.WEBMOB_device_hood_boost) !== -1) {
            $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_level_info_boost;
        } else if (value.indexOf($scope.translation.WEBMOB_device_cooktop_hot) !== -1 || value.indexOf($scope.translation.WEBMOB_device_cooktop_veryhot) !== -1) {
            $scope.selectedBurnerDetail = $scope.translation.WEBMOB_device_cooktop_hot_alarm;
        } else {
            var text = $scope.translation.WEBMOB_device_cooktop_detailed_level_info;
            var splitLevelInfo = value.split(' ');
            var tmpVal = (splitLevelInfo instanceof Array && angular.isDefined(splitLevelInfo[1])) ? splitLevelInfo[1] : value;
            text = text.replace(/#1#/, tmpVal);
            $scope.selectedBurnerDetail = text;
        }

        if (value === '') {
            value = $scope.flexCoilStatus[val].CenterText.toLowerCase();
            var text = $scope.translation.WEBMOB_device_cooktop_burner_state;
            text = text.replace(/#1#/, value);
            $scope.selectedBurnerDetail = text;
        }
    }

    function checkPauseState() {
        var isPause = false;
        if ($scope.cookTopBurnerArray[0].OperationState === 'Pause'
                || $scope.cookTopBurnerArray[1].OperationState === 'Pause'
                || $scope.cookTopBurnerArray[2].OperationState === 'Pause'
                || $scope.cookTopBurnerArray[3].OperationState === 'Pause'
                || $scope.cookTopBurnerArray[4].OperationState === 'Pause'
                || $scope.cookTopBurnerArray[5].OperationState === 'Pause') {
            isPause = true;
        }
        return isPause;
    }

    if ($scope.isStatic) {
        parseStaticData();
    }
}

app.directive('stPressEffect', function (UtilityService) {
    return {
        restrict: 'A',
        link: function ($scope, $elm, $attrs) {
            var mouseEvts = {
                'touchstart': touchStartFun,
                'touchend': touchEndFun,
                'touchleave': touchEndFun,
                'touchcancel': touchEndFun,
                'touchmove': touchMoveFun
            },
            evnt;
            for (evnt in mouseEvts) {
                if (mouseEvts.hasOwnProperty(evnt) &&
                        UtilityService.isEventSupported(evnt)) {
                    $elm.bind(evnt, mouseEvts[evnt]);
                }
            }
            function touchStartFun() {
                $scope.$parent[$attrs.stPressEffect] = true;
                $scope.$apply();
            }
            function touchMoveFun(evt) {
                if (evt.touches !== undefined && evt.touches.length !== undefined && evt.touches[evt.touches.length - 1] !== undefined && evt.currentTarget !== undefined && evt.currentTarget.parentElement !== undefined) {
                    if (evt.touches[evt.touches.length - 1].clientY <= evt.currentTarget.parentElement.clientHeight || evt.touches[evt.touches.length - 1].clientX <= evt.currentTarget.parentElement.clientWidth) {
                        touchStartFun();
                    }
                } else {
                    touchEndFun();
                }
            }
            function touchEndFun() {
                $scope.$parent[$attrs.stPressEffect] = false;
                $scope.$apply();
            }
        }
    };
});

function updateKitchenTimerValue(value, tab) {
    var scope = angular.element($("#bodyTag")).scope();
    if (tab === 2) {
        scope.hoodTimer = Number(value);
        scope.updateHoodTimerValue(scope.hoodTimer);
    } else {
        scope.kitchenTimer = Number(value);
        scope.updateTimerValue(scope.kitchenTimer);
    }
    if (scope.$root.$$phase !== '$apply' && scope.$root.$$phase !== '$digest') {
        scope.$apply();
    }
}