(function() {
  var app;

  app = angular.module("ngQuickDate", []);

  app.provider("ngQuickDateDefaults", function() {
    return {
      options: {
        dateFormat: 'M/d/yyyy',
        timeFormat: 'h:mm a',
        labelFormat: null,
        placeholder: 'Click to Set Date',
        hoverText: null,
        buttonIconHtml: null,
        closeButtonHtml: '&times;',
        nextLinkHtml: 'Next &rarr;',
        prevLinkHtml: '&larr; Prev',
        disableTimepicker: false,
        disableClearButton: false,
        defaultTime: null,
        dayAbbreviations: ["monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"],
        dateFilter: null,
        parseDateFunction: function(str) {
          var seconds;
          seconds = Date.parse(str);
          if (isNaN(seconds)) {
            return null;
          } else {
            return new Date(seconds);
          }
        }
      },
      $get: function() {
        return this.options;
      },
      set: function(keyOrHash, value) {
        var k, v, _results;
        if (typeof keyOrHash === 'object') {
          _results = [];
          for (k in keyOrHash) {
            v = keyOrHash[k];
            _results.push(this.options[k] = v);
          }
          return _results;
        } else {
          return this.options[keyOrHash] = value;
        }
      }
    };
  });

  app.directive("quickDatepicker", [
    'ngQuickDateDefaults', '$filter', '$sce', '$timeout', 'toaster', function(ngQuickDateDefaults, $filter, $sce, $timeout, toaster) {
      return {
        restrict: "E",
        require: "?ngModel",
//        scope: {
//          dateFilter: '=?',
//          onChange: "&",
//          onCancel : "&",
//          required: '@'
//        },
        scope : false,
        replace: true,
        link: function(scope, element, attrs, ngModelCtrl) {
          var dateToString, datepickerClicked, datesAreEqual, datesAreEqualToMinute, debounce, getDaysInMonth, initialize, parseDateString, refreshView, setCalendarDate, setConfigOptions, setInputFieldValues, setupCalendarView, stringToDate;
          initialize = function() {
            var YEAR_COUNT = 22;
            var templateDate;
            setConfigOptions();
//            scope.toggleCalendar(false);
            scope.weeks = [];
            scope.inputDate = null;
            scope.inputTime = null;
            scope.invalid = true;
            scope.selectedYear = null;
//            scope.selectedDate = null;
            scope.selectedDate = -1;
            scope.prevDisable = false;
            scope.nextDisable = false;
            scope.years = [];
            scope.showYearList = false;
            if (typeof attrs.initValue === 'string') {
              ngModelCtrl.$setViewValue(attrs.initValue);
            }else{//if no default date is set
                ngModelCtrl.$setViewValue(new Date().toString());
            }
            if (!scope.defaultTime) {
              templateDate = new Date(2013, 0, 1, 12, 0);
              scope.datePlaceholder = $filter('date')(templateDate, scope.dateFormat);
              scope.timePlaceholder = $filter('date')(templateDate, scope.timeFormat);
            }
            setCalendarDate();            
            scope.selectedYear = scope.calendarDate.getFullYear().toString();
            ngModelCtrl.$modelValue = scope.calendarDate;
            ngModelCtrl.$modelValue.setDate(scope.selectedDate);
            for (var i = 0; i < YEAR_COUNT; i++) {
                scope.years.push((scope.calendarDate.getFullYear() + i).toString());
            }
            return refreshView();
          };
          
          scope.reinit = function(){
            var curDate = new Date();
            $timeout(function(){
                scope.selectedYear = curDate.getFullYear();
            }, 0);
            
            ngModelCtrl.$setViewValue(curDate);
          };
          setConfigOptions = function() {
            var key, value;
            for (key in ngQuickDateDefaults) {
              value = ngQuickDateDefaults[key];
              if (key.match(/[Hh]tml/)) {
                scope[key] = $sce.trustAsHtml(ngQuickDateDefaults[key] || "");
              } else if (!scope[key] && attrs[key]) {
                scope[key] = attrs[key];
              } else if (!scope[key]) {
                scope[key] = ngQuickDateDefaults[key];
              }
            }

            scope.labelFormat = (LANG.toUpperCase() === "KO-KR" || LANG.toUpperCase() === "ZH-CN")?'MMMM d yyyy':'MMMM d. yyyy';  

            if (attrs.iconClass && attrs.iconClass.length) {
              return scope.buttonIconHtml = $sce.trustAsHtml("<i ng-show='iconClass' class='" + attrs.iconClass + "'></i>");
            }
          };

          refreshView = function() {
            var date;
            date = ngModelCtrl.$modelValue ? parseDateString(ngModelCtrl.$modelValue) : new Date();
            setupCalendarView();
            var dateSplit = $filter('date')(date, scope.labelFormat).toString().split(" ");
            if(LANG.toUpperCase() === "KO-KR" || LANG.toUpperCase() === "ZH-CN"){
                scope.mainButtonStr = dateSplit[2] + scope.translation.WEBMOB_device_ref_food_reminder_year + " " + scope.translation['WEBMOB_device_ref_food_reminder_calendar_' + dateSplit[0].toLowerCase()] + " " + dateSplit[1] + scope.translation.WEBMOB_device_ref_food_reminder_date_day;
            }else{
                dateSplit[0] = scope.translation['WEBMOB_device_ref_food_reminder_calendar_' + dateSplit[0].toLowerCase()];//Applying localization for Month text
                scope.mainButtonStr = dateSplit.join(" ");
            }    
            return scope.invalid = ngModelCtrl.$invalid;
          };
          setInputFieldValues = function(val) {
            if (val != null) {
              scope.inputDate = $filter('date')(val, scope.dateFormat);
              return scope.inputTime = $filter('date')(val, scope.timeFormat);
            } else {
              scope.inputDate = null;
              return scope.inputTime = null;
            }
          };
          setCalendarDate = function(val) {
            var d;
            if (val == null) {
              val = null;
            }
            d = val != null ? new Date(val) : new Date();
            if (d.toString() === "Invalid Date") {
              d = new Date();
            }
            d.setDate(1);
            return scope.calendarDate = new Date(d);
          };    
          setupCalendarView = function() {
            var curDate, d, daysInMonth, numRows, offset, selected, time, today, weeks, i, j,startDay;
            
            startDay = scope.calendarDate.getDay();
            
//            offset = startDay > 1 ? startDay - 1 : 6-startDay; // Starting from Monday
            offset = startDay > 0 ? startDay - 1 : 6-startDay; // Starting from Monday
            daysInMonth = getDaysInMonth(scope.calendarDate.getFullYear(), scope.calendarDate.getMonth());
            numRows = Math.ceil((offset + daysInMonth) / 7);
            
            weeks = [];
            curDate = new Date(scope.calendarDate);
            curDate.setDate(curDate.getDate() + (offset * -1));
            for (i = 0; i < numRows; i++) {
                weeks.push([]);
                for (j = 0; j <= 6; j++) {
                    d = new Date(curDate);
                    if (scope.defaultTime) {
                        time = scope.defaultTime.split(':');
                        d.setHours(time[0] || 0);
                        d.setMinutes(time[1] || 0);
                        d.setSeconds(time[2] || 0);
                    }
                    
                    selected = ngModelCtrl.$modelValue && d && datesAreEqual(d, ngModelCtrl.$modelValue);
                    today = datesAreEqual(d, new Date());
                   
                   if((selected === undefined && today) || selected) {
                       scope.selectedDate = d.getDate();
                        
                   } 
                   
                    if(selected === undefined) {
                        selected = today;
                    }
                                    
                    weeks[i].push({
                        date: d,
                        selected: selected,
                        disabled: typeof scope.dateFilter === 'function' ? !scope.dateFilter(d) : false,
                        other: d.getMonth() !== scope.calendarDate.getMonth(),
                        today: today
                    });
                     curDate.setDate(curDate.getDate() + 1);
                }
            }
            return scope.weeks = weeks;
          };
          ngModelCtrl.$parsers.push(function(viewVal) {
            if (scope.required && (viewVal == null)) {
              ngModelCtrl.$setValidity('required', false);
              return null;
            } else if (angular.isDate(viewVal)) {
              ngModelCtrl.$setValidity('required', true);
              return viewVal;
            } else if (angular.isString(viewVal)) {
              ngModelCtrl.$setValidity('required', true);
              return scope.parseDateFunction(viewVal);
            } else {
              return null;
            }
          });
          ngModelCtrl.$formatters.push(function(modelVal) {
            if (angular.isDate(modelVal)) {
              return modelVal;
            } else if (angular.isString(modelVal)) {
              return scope.parseDateFunction(modelVal);
            } else {
              return void 0;
            }
          });
          dateToString = function(date, format) {
            return $filter('date')(date, format);
          };
          stringToDate = function(date) {
            if (typeof date === 'string') {
              return parseDateString(date);
            } else {
              return date;
            }
          };
          parseDateString = ngQuickDateDefaults.parseDateFunction;
          datesAreEqual = function(d1, d2, compareTimes) {
            if (compareTimes == null) {
              compareTimes = false;
            }
            if (compareTimes) {
              return (d1 - d2) === 0;
            } else {
              d1 = stringToDate(d1);
              d2 = stringToDate(d2);
              return d1 && d2 && (d1.getYear() === d2.getYear()) && (d1.getMonth() === d2.getMonth()) && (d1.getDate() === d2.getDate());
            }
          };
          datesAreEqualToMinute = function(d1, d2) {
            if (!(d1 && d2)) {
              return false;
            }
            return parseInt(d1.getTime() / 60000) === parseInt(d2.getTime() / 60000);
          };
          getDaysInMonth = function(year, month) {
            return [31, ((year % 4 === 0 && year % 100 !== 0) || year % 400 === 0 ? 29 : 28), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][month];
          };
          debounce = function(func, wait) {
            var args, context, later, result, timeout, timestamp;
            timeout = args = context = timestamp = result = null;
            later = function() {
              var last;
              last = +new Date() - timestamp;
              if (last < wait && last > 0) {
                return timeout = setTimeout(later, wait - last);
              } else {
                return timeout = null;
              }
            };
            return function() {
              context = this;
              args = arguments;
              timestamp = +new Date();
              if (!timeout) {
                timeout = setTimeout(later, wait);
                result = func.apply(context, args);
                context = args = null;
              }
              return result;
            };
          };
          ngModelCtrl.$render = function() {
            setCalendarDate(ngModelCtrl.$viewValue);
            return refreshView();
          };
          ngModelCtrl.$viewChangeListeners.unshift(function() {
            setCalendarDate(ngModelCtrl.$viewValue);
            refreshView();
            if (scope.onChange) {
              return scope.onChange();
            }
          });
          scope.$watch('calendarShown', function(newVal, oldVal) {
            var dateInput;
            if (newVal) {
              dateInput = angular.element(element[0].querySelector(".quickdate-date-input"))[0];
              return dateInput.select();
            }
          });
          scope.toggleCalendar = debounce(function(show) {
//            if (isFinite(show)) {
//              return scope.calendarShown = show;
//            } else {
//              return scope.calendarShown = !scope.calendarShown;
//            }
          }, 150);
          scope.selectDate = function(date, closeCalendar) {
            var changed;
            if (closeCalendar === null) {
              closeCalendar = true;
            }
            //if clicked date is not from same month( blank date), return 
            if(date.getMonth() !== scope.calendarDate.getMonth())
                return true;
            
             //if slected date is less than current date, return
            if(datesAreEqual(date, new Date())) {
                toaster.success({title: "", body: scope.translation.WEBMOB_device_ref_food_reminder_error_calrendar_today});
                return;
            }
            if(date - new Date() < 0){
                toaster.success({title: "", body: scope.translation.WEBMOB_device_ref_food_reminder_error_calrendar_earlier});
                return false;
            }
            
            changed = (!ngModelCtrl.$viewValue && date) || (ngModelCtrl.$viewValue && !date) || ((date && ngModelCtrl.$viewValue) && (date.getTime() !== ngModelCtrl.$viewValue.getTime()));
              if (typeof scope.dateFilter === 'function' && !scope.dateFilter(date)) {
              return false;
            }
            ngModelCtrl.$setViewValue(date);
            ngModelCtrl.$modelValue = date;
            if (closeCalendar) {
              scope.toggleCalendar(false);
            }
            return true;
          };
          scope.selectDateFromInput = function(closeCalendar) {
            var err, tmpDate, tmpDateAndTime, tmpTime;
            if (closeCalendar === null) {
              closeCalendar = false;
            }
            try {
              tmpDate = parseDateString(scope.inputDate);
              if (!tmpDate) {
                throw 'Invalid Date';
              }
              if (!scope.disableTimepicker && scope.inputTime && scope.inputTime.length && tmpDate) {
                tmpTime = scope.disableTimepicker ? '00:00:00' : scope.inputTime;
                tmpDateAndTime = parseDateString("" + scope.inputDate + " " + tmpTime);
                if (!tmpDateAndTime) {
                  throw 'Invalid Time';
                }
                tmpDate = tmpDateAndTime;
              }
              if (!datesAreEqualToMinute(ngModelCtrl.$viewValue, tmpDate)) {
                if (!scope.selectDate(tmpDate, false)) {
                  throw 'Invalid Date';
                }
              }
              if (closeCalendar) {
                scope.toggleCalendar(false);
              }
              scope.inputDateErr = false;
              return scope.inputTimeErr = false;
            } catch (_error) {
              err = _error;
              if (err === 'Invalid Date') {
                return scope.inputDateErr = true;
              } else if (err === 'Invalid Time') {
                return scope.inputTimeErr = true;
              }
            }
          };
          scope.onDateInputTab = function() {
            if (scope.disableTimepicker) {
              scope.toggleCalendar(false);
            }
            return true;
          };
          scope.onTimeInputTab = function() {
            scope.toggleCalendar(false);
            return true;
          };
          scope.nextMonth = function() {
            var d = new Date(new Date(scope.calendarDate).setMonth(scope.calendarDate.getMonth() + 1));
            d.setDate(scope.selectedDate);
            if(d.getFullYear() > scope.years[scope.years.length-1]) {
                var str = scope.translation.WEBMOB_device_ref_food_reminder_error_calrendar_after;
                var message = str.replace("%1", scope.years[scope.years.length-1]);
                toaster.success({title: "", body: message});
                nextDisable = true;
                return;
            } 
            scope.selectedYear =  d.getFullYear();
            ngModelCtrl.$modelValue = d;
            setCalendarDate(d);
            return refreshView();
          };
          scope.prevMonth = function() {
            var d = new Date(new Date(scope.calendarDate).setMonth(scope.calendarDate.getMonth() - 1)); 
            d.setDate(scope.selectedDate);
            if(!datesAreEqual(d, new Date()) && d - new Date() < 0   ) {
                toaster.success({title: "", body: scope.translation.WEBMOB_device_ref_food_reminder_error_calrendar_earlier});
                prevDisable = true;
                return;
            }    
            scope.selectedYear =  d.getFullYear();
            ngModelCtrl.$modelValue = d;
            setCalendarDate(d);
            return refreshView();
          };
           scope.openYearList = function() {
              scope.showYearList = !scope.showYearList;
          };
          scope.setSelectedYear = function(year) {
              scope.selectedYear = year;
              scope.showYearList = false;
              var d = new Date(new Date(ngModelCtrl.$modelValue).setFullYear(parseInt(scope.selectedYear)));
              d.setDate(scope.selectedDate);
              ngModelCtrl.$modelValue = d;
              setCalendarDate(d);
              return refreshView();
          };
          scope.clear = function() {
            return scope.selectDate(null, true);
          };
          scope.close = function() {
            scope.cancelDate();
          };
          scope.setPickedDate = function() {
            var d = new Date(scope.calendarDate);
            d.setDate(scope.selectedDate);
            if(datesAreEqual(d, new Date())) {
                toaster.success({title: "", body: scope.translation.WEBMOB_device_ref_food_reminder_error_calrendar_today});
                return;
            }
            scope.datepick = d;
            scope.setDate();
          };

          return initialize();
        },
        template: "<div class='quickdate' ng-click='$event.preventDefault()'>\n \n\
                    <div class='quickdate-popup open'>\
                    <div class='quickdate-calendar-header'>\
                    <div class='selected-date-div'> <span class='selected-date-text quickdate-month-heading'>{{mainButtonStr}}</span>\</div>\
                    <div class='quickdate-month-div'>\
                    <div class='select-container'>\
                    <div class='select-div' ng-click='openYearList()'><span class='select-year-div'>{{selectedYear}}</span> <img class='select-down-arrow' src='images/common/ws_timepicker_spinner.svg'></img></div>\
                    <div class='option'><ul class='select-year-list' ng-if='showYearList'><li class='select-year-list-item' ng-repeat='year in years' ng-click='setSelectedYear(year)'>{{year}}</li></ul></div>\
                    </div>\
                    <div class='month-div'>\
                    <img class='quickdate-prev-month' src='images/common/ws_datepicker_btn_previous.svg' ng-click='prevMonth()' ng-class='{disablePrev : prevDisable}'></img>\
                    <span class='quickdate-month'>{{translation['WEBMOB_device_ref_food_reminder_calendar_' + (calendarDate | date:'MMMM' | lowercase)]}}</span>\
                    <img class='quickdate-next-month' src='images/common/ws_datepicker_btn_next.svg' ng-click='nextMonth()' ng-class='{disableNext : nextDisable}'></img>\
                    </div>\
                    </div>\
                    <table class='quickdate-calendar' cellpadding='360'>\
                    <thead><tr><th ng-repeat='day in dayAbbreviations' ng-class='{sunday : $index==6 , weekText : $index!== 6}'>{{translation['WEBMOB_device_ref_food_reminder_calendar_' + day]}}</th></tr></thead> \
                    <tbody><tr ng-repeat='week in weeks'>\
                    <td ng-mousedown='selectDate(day.date, true, true)' ng-click='$event.preventDefault()'  ng-repeat='day in week'><div class='normal' ng-class='{selected : day.selected }'>{{day.other ? '' : day.date | date:'d'}}</div></td></tr>\
                    </tbody></table>\
                    </div>\
                    <div class='bottom-bar'>\
                    <span ng-click = 'close()' class='bottom-bar-text bottom-cancel-button'>{{translation.WEBMOB_device_ref_food_reminder_popup_cancel_capital}}</span>\
                    <span ng-click = 'setPickedDate()' class='bottom-bar-text'>{{translation.WEBMOB_device_ref_food_reminder_calendar_set_capital}}</span>\
                    </div>\
                   </div>"
      };
    }
  ]);

  app.directive('ngEnter', function() {
    return function(scope, element, attr) {
      return element.bind('keydown keypress', function(e) {
        if (e.which === 13) {
          scope.$apply(attr.ngEnter);
          return e.preventDefault();
        }
      });
    };
  });

  app.directive('onTab', function() {
    return {
      restrict: 'A',
      link: function(scope, element, attr) {
        return element.bind('keydown keypress', function(e) {
          if ((e.which === 9) && !e.shiftKey) {
            return scope.$apply(attr.onTab);
          }
        });
      }
    };
  });

}).call(this);
