/*
 * JNICIAgent.cpp
 *
 *  Created on: Feb 19, 2016
 *      Author: ranjith.bk@samsung.com
 */

#include "JNIGlobal.h"
#include "Configuration.h"
#include "SHPCIAgent.h"
#include "SHPCommon.h"
#include "Log.h"
#include "Connector/HTTP/HTTPMessage.h"
#include "SHPHTTPSServerConnector.h"
#include "SHPHTTPServerConnector.h"
#include "JNIHttpClient.h"
#include "JNIHttpServer.h"
#include "com_sec_shp_sdk_ra_ci_CIAgent.h"
#include "SHPError.h"
#include "MyDeviceFinderListener.h"

#define CLASS_NAME "JNICIAgent"
#define MODULE_ID	SHP_JNI


namespace Sec {
namespace Shp {
namespace Jni {

class JNICIAgent
{

public:
	Sec::Shp::RA::CI::ICIAgent				*pCIAgent;
	Sec::Shp::Connector::SSLConfiguration	mSSLConfiguration ;

	std::string mServerAddress;
	std::string mAppId;
	std::string mUserID;
	std::string mAccessToken;
	std::string mDeviceId;
	std::string mDeviceName;
	std::string mDeviceCatogory;
	std::string mClientVersion;

	bool isCIChannelConnected ;
	Sec::Shp::Configuration mConfiguration; // dummy not used any where
	Sec::Shp::Jni::JNIHTTPClient *pJNIHttpClient ;
	Sec::Shp::Jni::JNIHttpServer *pJNIHttpServer ;

	JNICIAgent()
	{
		pCIAgent 					= NULL;
		pJNIHttpClient				= NULL;
		pJNIHttpServer				= NULL;
		isCIChannelConnected 		= false;
	}

	~JNICIAgent()
	{
	}

	bool isLoggedIn()
	{
		if (pCIAgent != NULL)
		{
			return pCIAgent->isInitialized();
		}
		else
		{
			return false;
		}
	}

	bool init( 	std::string mServerAddress, std::string  mAppId, std::string mUserID,
				std::string mAccessToken,
				std::string mDeviceId,
				std::string mDeviceName,
				std::string mDeviceCatogory,
				std::string mClientVersion )
	{
		this->mServerAddress	= mServerAddress ;
		this->mAppId			= mAppId;
		this->mUserID			= mUserID;
		this->mAccessToken		= mAccessToken;
		this->mDeviceId			= mDeviceId;
		this->mDeviceName		= mDeviceName;
		this->mDeviceCatogory	= mDeviceCatogory;
		this->mClientVersion	= mClientVersion;

		if (NULL == pCIAgent)
		{
			this->pCIAgent	= Sec::Shp::RA::CI::SHPCIAgent::createInstance();
		}

		if (NULL == pCIAgent)
		{
			return false;
		}

		Sec::Shp::Connector::Client::IClientConnector *pClientConnector = pCIAgent->getClientConnector() ;
		TryReturn( pClientConnector != NULL, false,"Failed to instantiate Client Connector" ) ;

		pJNIHttpClient = new Sec::Shp::Jni::JNIHTTPClient(pClientConnector, true, false);
		TryReturn( pJNIHttpClient != NULL, false, "Failed to instantiate JNIHTTPClient" ) ;

		Sec::Shp::Connector::Server::IServerConnector *pServerConnector = pCIAgent->getServerConnector();
		TryReturn( pServerConnector != NULL, false,"Failed to instantiate Server Connector" ) ;

		pJNIHttpServer = new Sec::Shp::Jni::JNIHttpServer(0, pServerConnector, NULL, true, false, false);
		TryReturn( pJNIHttpServer != NULL, false, "Failed to instantiate JNIHttpServer" ) ;

		return true;
	}

	bool start()
	{
		SHP_LOG_ERROR_A(SHP_CI, "%s", "Start IN");
		TryReturn( isCIChannelConnected == false, false,"Already Started" ) ;
		TryReturn( pCIAgent != NULL, false,"init not called before.. " ) ;

		Sec::Shp::RA::CI::CIAgentConfig ciConfiguration ;
		ciConfiguration.m_accessToken		= mAccessToken;
		ciConfiguration.m_ciServerAddress	= mServerAddress;
		ciConfiguration.m_deviceID			= mDeviceId;
		ciConfiguration.m_deviceCatogoty	= mDeviceCatogory;
		ciConfiguration.m_userID			= mUserID;
		ciConfiguration.m_appID				= mAppId;
		ciConfiguration.m_clientVersion		= mClientVersion;
		ciConfiguration.m_deviceName		= mDeviceName;

		SHP_LOG_ERROR_A(SHP_CI, "%s", "Invoke Initialize");
		Sec::Shp::SHPError error = pCIAgent->initialize(&ciConfiguration, &(mSSLConfiguration));
		if (error.getErrorCode() == SHP_SUCCESS)
		{
			isCIChannelConnected = true;
		}
		else
		{
			isCIChannelConnected = false;
		}

		SHP_LOG_ERROR_A(SHP_CI, "%s", "Start Done");
		return isCIChannelConnected ;
	}

	bool stop()
	{
		SHP_LOG_ERROR_A(SHP_CI, "%s", "Stop invoked");
		isCIChannelConnected = false ;
		if (NULL == pCIAgent)
		{
			return false;
		}	

		SHP_LOG_ERROR_A(SHP_CI, "%s", "Deleting JNI Client");
		if (pJNIHttpClient != NULL)
		{
			pJNIHttpClient->invalidateConnector();
			Sec::Shp::Jni::JNIHTTPClient::decreaseReference(pJNIHttpClient);
			pJNIHttpClient = NULL ;
		}

		SHP_LOG_ERROR_A(SHP_CI, "%s", "Deleting JNI Server");
		if (pJNIHttpServer != NULL)
		{
			pJNIHttpServer->invalidateConnector();
			Sec::Shp::Jni::JNIHttpServer::decreaseReference(pJNIHttpServer);
			pJNIHttpServer = NULL ;
		}

		SHP_LOG_ERROR_A(SHP_CI, "%s", "Invoke Deinitialize");
		// TODO: Notify Connectors
		pCIAgent->deInitialize();
		isCIChannelConnected = false ;
		//delete pCIAgent;
		//pCIAgent = NULL;
		SHP_LOG_ERROR_A(SHP_CI, "%s", "Invoke Deinitialize Done");
		return true;
	}

	Sec::Shp::Jni::JNIHTTPClient* getClient()
	{
		if ((NULL == pCIAgent) || (NULL == pJNIHttpClient))
		{
			return NULL;
		}

		Sec::Shp::Jni::JNIHTTPClient::increaseReference(pJNIHttpClient);
		return pJNIHttpClient ;
	}

	Sec::Shp::Jni::JNIHttpServer* getServer( jobject requestListener )
	{
		if ((NULL == pCIAgent) || (NULL == pJNIHttpServer))
		{
			return NULL;
		}

		Sec::Shp::Jni::JNIHttpServer::increaseReference(pJNIHttpServer);
		pJNIHttpServer->setRequestListener(requestListener);

		return pJNIHttpServer ;
	}
};

}
}
}

/*
 * Class:     com_sec_shp_sdk_ra_ci_CIAgent
 * Method:    createInstance
 * Signature: (Lcom/sec/shp/sdk/http/server/IHttpServerListener;)J
 */
JNIEXPORT jlong JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_createInstance
	(JNIEnv *pEnv, jobject thisObject )
{
	SHP_LOG_ERROR_A(SHP_CI, "%s", "IN");
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = new Sec::Shp::Jni::JNICIAgent();
	TryReturn( jniCIAgent != NULL, 0, "Failed to instantiate jniCIAgent" ) ;

	jlong bRet = (jlong) jniCIAgent ;
	SHP_LOG_ERROR_A(SHP_CI, "%s", "OUT");
	return bRet ;
}

/*
 * Class:     com_sec_shp_sdk_ra_ci_CIAgent
 * Method:    destroyInstance
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_destroyInstance
  (JNIEnv *pEnv, jobject thisObject, jlong nativeInstance)
{
	SHP_LOG_ERROR_A(SHP_CI, "%s", "IN");
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	if (jniCIAgent != NULL)
	{
		delete jniCIAgent;
	}
	SHP_LOG_ERROR_A(SHP_CI, "%s", "OUT");
}

/*
 * Class:     com_sec_shp_sdk_ra_ci_CIAgent
 * Method:    getSSLConfiguration
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_getSSLConfiguration
  (JNIEnv *pEnv, jobject thisObject, jlong nativeInstance)
{
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturn(jniCIAgent != NULL, 0, "Invalid native instance");

	return (jlong)(&(jniCIAgent->mSSLConfiguration));
}

/*
 * Class:     com_sec_shp_sdk_ra_ci_CIAgent
 * Method:    start
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_start
  (JNIEnv *pEnv, jobject thisObject, jlong nativeInstance)
{
	SHP_LOG_ERROR_A(SHP_CI, "%s", "IN");
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturn(jniCIAgent != NULL, 0, "Invalid native instance");

	jboolean bRet = (jniCIAgent->start()) ? JNI_TRUE : JNI_FALSE ;
	SHP_LOG_ERROR_A(SHP_CI, "%s", "OUT");
	return bRet ;
}

/*
 * Class:     com_sec_shp_sdk_ra_ci_CIAgent
 * Method:    stop
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_stop
  (JNIEnv *pEnv, jobject thisObject, jlong nativeInstance)
{
	SHP_LOG_ERROR_A(SHP_CI, "%s", "IN");
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturn(jniCIAgent != NULL, 0, "Invalid native instance");

	jboolean bRet = (jniCIAgent->stop()) ? JNI_TRUE : JNI_FALSE ;
	SHP_LOG_ERROR_A(SHP_CI, "%s", "OUT");
	return bRet ;
}

JNIEXPORT jboolean JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_isLoggedIn
  (JNIEnv *pEnv, jobject thisObject, jlong nativeInstance)
{
	SHP_LOG_ERROR_A(SHP_CI, "%s", "IN");
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturn(jniCIAgent != NULL, 0, "Invalid native instance");

	jboolean bRet = (jniCIAgent->isLoggedIn()) ? JNI_TRUE : JNI_FALSE ;
	SHP_LOG_ERROR_A(SHP_CI, "%s", "OUT");
	return bRet ;
}


/*
 * Class:     com_sec_shp_sdk_ra_ci_CIAgent
 * Method:    createServerInstance
 * Signature: (JLcom/sec/shp/sdk/http/server/IHttpServerListener;)J
 */
JNIEXPORT jlong JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_createServerInstance
  (JNIEnv *pEnv, jobject thisObject, jlong nativeInstance, jobject requestListener)
{
	SHP_LOG_ERROR_A(SHP_CI, "%s", "IN");
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturn(jniCIAgent != NULL, 0, "Invalid native instance");

	jlong bRet = (jlong)(jniCIAgent->getServer(requestListener));
	SHP_LOG_ERROR_A(SHP_CI, "%s", "OUT");
	return bRet ;
}

/*
 * Class:     com_sec_shp_sdk_ra_ci_CIAgent
 * Method:    createClientInstance
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_createClientInstance
  (JNIEnv *pEnv, jobject thisObject, jlong nativeInstance)
{
	SHP_LOG_ERROR_A(SHP_CI, "%s", "IN");
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturn(jniCIAgent != NULL, 0, "Invalid native instance");

	jlong bRet = (jlong)(jniCIAgent->getClient());
	SHP_LOG_ERROR_A(SHP_CI, "%s", "OUT");
	return bRet ;
}

/*
 * Class:     com_sec_shp_sdk_ra_ci_CIAgent
 * Method:    init
 * Signature: (JLjava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_init
(JNIEnv *pEnv, jobject thisObject, jlong nativeInstance, jstring jniServerAddress, jstring jniAppId, jstring jniUserID, jstring JniAccessToken,
		jstring jniDeviceId, jstring jniDeviceName, jstring jniDeviceCatogory,
		jstring jniClientVersion)
{
	SHP_LOG_ERROR_A(SHP_CI, "%s", "IN");
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturn(jniCIAgent != NULL, JNI_FALSE, "Invalid native instance");

	std::string mServerAddress;
	std::string mAppId;
	std::string mUserID;
	std::string mAccessToken;
	std::string mDeviceId;
	std::string mDeviceName;
	std::string mDeviceCatogory;
	std::string mClientVersion;

	JNI_TryReturn(JNIGlobal::convertToStdString(jniServerAddress, mServerAddress), JNI_FALSE, "Failed to convert mServerAddress");
	JNI_TryReturn(JNIGlobal::convertToStdString(jniAppId, mAppId), JNI_FALSE, "Failed to convert mAppId");
	JNI_TryReturn(JNIGlobal::convertToStdString(jniUserID, mUserID), JNI_FALSE, "Failed to convert mUserID");
	JNI_TryReturn(JNIGlobal::convertToStdString(JniAccessToken, mAccessToken), JNI_FALSE, "Failed to convert mAccessToken");
	JNI_TryReturn(JNIGlobal::convertToStdString(jniDeviceId, mDeviceId), JNI_FALSE, "Failed to convert mDeviceId");
	JNI_TryReturn(JNIGlobal::convertToStdString(jniDeviceName, mDeviceName), JNI_FALSE, "Failed to convert mDeviceName");
	JNI_TryReturn(JNIGlobal::convertToStdString(jniDeviceCatogory, mDeviceCatogory), JNI_FALSE, "Failed to convert mDeviceCatogory");
	JNI_TryReturn(JNIGlobal::convertToStdString(jniClientVersion, mClientVersion), JNI_FALSE, "Failed to convert mClientVersion");

	jboolean jRet = ( jniCIAgent->init(mServerAddress, mAppId, mUserID, mAccessToken, mDeviceId, mDeviceName, mDeviceCatogory, mClientVersion)) ? JNI_TRUE : JNI_FALSE ;
	SHP_LOG_ERROR_A(SHP_CI, "%s", "OUT");
	return jRet ;
}


JNIEXPORT jboolean JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_addSubscription
(JNIEnv *pEnv, jobject thisObject, jlong nativeInstance, jstring jniDeviceUUId, jstring jniUrl)
{
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturn(jniCIAgent != NULL, JNI_FALSE, "Invalid native instance");

	std::string mDeviceUUID;
	std::string mUrl;
	uint64_t token ;

	JNI_TryReturn(JNIGlobal::convertToStdString(jniDeviceUUId, mDeviceUUID), JNI_FALSE, "Failed to convert mDeviceUUID");
	JNI_TryReturn(JNIGlobal::convertToStdString(jniUrl, mUrl), JNI_FALSE, "Failed to convert mAppId");

	if ( jniCIAgent->pCIAgent != NULL )
	{
		Sec::Shp::SHPError error = jniCIAgent->pCIAgent->subscribeResource(mDeviceUUID, mUrl, token);
		return ( error.getErrorCode() == Sec::Shp::SHP_SUCCESS) ? JNI_TRUE : JNI_FALSE ;
	}
	else
	{
		return JNI_FALSE;
	}
}

JNIEXPORT void JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_setDevicePresenceListener( JNIEnv *env, jobject thisObj, 
																					 jlong nativeInstance, jobject listener )
{
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturnVoid(jniCIAgent!=NULL, "Invalid native instance");
	JNI_TryReturnVoid((jniCIAgent->pCIAgent)!=NULL, "Invalid native instance - core deinitialized");

	if (listener == NULL)
	{
		jniCIAgent->pCIAgent->setDeviceFinderListener( NULL );
	}
	else
	{
		MyDeviceFinderListener* devFindListener = new MyDeviceFinderListener( listener, true );
		JNI_TryReturnVoid(devFindListener != NULL, "Failed to allocate memory");

		jniCIAgent->pCIAgent->setDeviceFinderListener( devFindListener );
	}	
}


JNIEXPORT void JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_subscribePresence( JNIEnv *env, jobject thisObj, 
																			jlong nativeInstance, jstring juuid, jboolean isSubsribe )
{
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturnVoid(jniCIAgent!=NULL, "Invalid native instance");
	JNI_TryReturnVoid((jniCIAgent->pCIAgent)!=NULL, "Invalid native instance - core deinitialized");

	std::string mUUID ;
	JNI_TryReturnVoid(JNIGlobal::convertToStdString(juuid, mUUID), "Failed to convert mUUID");

	jniCIAgent->pCIAgent->subscribePresence( mUUID, (isSubsribe==JNI_TRUE)?true : false) ;
}

JNIEXPORT void JNICALL Java_com_sec_shp_sdk_ra_ci_CIAgent_setIPAddress( JNIEnv *env, jobject thisObj,
																			jlong nativeInstance, jstring ipAddress )
{
	Sec::Shp::Jni::JNICIAgent *jniCIAgent = (Sec::Shp::Jni::JNICIAgent *)(nativeInstance) ;
	JNI_TryReturnVoid(jniCIAgent!=NULL, "Invalid native instance");
	JNI_TryReturnVoid((jniCIAgent->pCIAgent)!=NULL, "Invalid native instance - core deinitialized");

	std::string mIPAddress ;
	JNI_TryReturnVoid(JNIGlobal::convertToStdString(ipAddress, mIPAddress), "Failed to convert IPAddress");

	jniCIAgent->pCIAgent->setIPAddress(mIPAddress) ;
}


