/*
 * JNIConfiguration.cpp
 *
 *  Created on: Jul 5, 2013
 *      Author: kirtigupta
 */

#include <jni.h>
#include "Configuration.h"
#include "DeviceFinder/DeviceFactory.h"
#include "Server/ResourceHandler/ResourceHandlerFactory.h"
#include "Sec_Shp_Configuration.h"
#include "SHPDeviceFactory.h"
#include "SHPResourceHandlerFactory.h"
#include "string"
#include "SHPSerializableDataFactory.h"
#include "stdlib.h"
#include "SHPConfiguration.h"
#include <list>
#include "SHPClientConnector.h"
#include "SHPServerConnector.h"

#ifdef REMOTE_ACCESS_SUPPORT
#include "RemoteAccessConfig.h"
#endif /* REMOTE_ACCESS_SUPPORT */


JNIEXPORT jlong JNICALL Java_Sec_Shp_Configuration_constructNative
(JNIEnv *env, jobject thisObj)
{
	Sec::Shp::Configuration* cObject = new SHPConfiguration(thisObj);

	if (NULL == cObject)
	{
		JNIGlobal::throwException("Failed to allocate SHPConfiguration : OutOfMemory");
	}

	return (jlong) cObject ;
}

 JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_deleteNative
   (JNIEnv *env, jobject thisObj, jlong config)
 {
	 Sec::Shp::Configuration* cObject =  ( SHPConfiguration *) config;

	 if (NULL != cObject)
	 {
		 delete cObject;
	 }
 }

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_reset
(JNIEnv *env, jobject thisObj, jlong config)
 {
	 Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;

	 if ( NULL == con)
	 {
		 JNIGlobal::throwException("Native Configuration object is NULL") ;
		 return ;
	 }

	 //Sec::Shp::Server::ResourceHandler::ResourceHandlerFactory* pResourceHandlerFactory = con->getResourceHandlerFactory() ;
	 //Sec::Shp::DeviceFinder::DeviceFactory* pDeviceFactory = con->getDeviceFactory();
	 //Sec::Shp::Serialization::SerializableDataFactory* pSerializableFactory = con->getSerializableDataFactory();

	 //std::list<Sec::Shp::Connector::Server::IServerConnector*> serverConnectors = con->getServerConnectors();
	 //std::list<Sec::Shp::Connector::Client::IClientConnector*> clientConnectors = con->getClientConnectors();

	con->reset();

//	for ( std::list<Sec::Shp::Connector::Server::IServerConnector*>::iterator itr = serverConnectors.begin(); itr != serverConnectors.end(); itr++)
//	{
//		 Sec::Shp::Connector::Server::IServerConnector* pConnector = *(itr);
//
//		if ( NULL != pConnector )
//		{
//			delete pConnector;
//		}
//	}
//
//	for ( std::list<Sec::Shp::Connector::Client::IClientConnector*>::iterator itr = clientConnectors.begin(); itr != clientConnectors.end(); itr++)
//	{
//		 Sec::Shp::Connector::Client::IClientConnector* pConnector = *(itr);
//
//		if ( NULL != pConnector )
//		{
//			delete pConnector;
//		}
//	}
//
//	 if ( NULL != pResourceHandlerFactory )
//	 {
//		 delete pResourceHandlerFactory;
//	 }
//
//	 if ( NULL != pDeviceFactory )
//	 {
//		 delete pDeviceFactory;
//	 }
//
//	 if ( NULL != pSerializableFactory )
//	 {
//		 delete pSerializableFactory;
//	 }
 }

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setDeviceFactory
(JNIEnv *env, jobject thisObj, jlong config, jlong deviceFactory)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return ;
	}

	SHPDeviceFactory* devFactory = ( SHPDeviceFactory* )(deviceFactory);
	if ( NULL == devFactory )
	{
		JNIGlobal::throwException("Native DeviceFactory object is NULL") ;
		return ;
	}

	con->setDeviceFactory(devFactory);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setResourceHandlerFactory
  (JNIEnv *env, jobject thisObj, jlong config, jlong resourceHandler)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return ;
	}

	SHPResourceHandlerFactory* resHandlerFactory = (SHPResourceHandlerFactory* )resourceHandler ;
	if ( NULL == resHandlerFactory )
	{
		JNIGlobal::throwException("Native resHandlerFactory object is NULL") ;
		return ;
	}

	con->setResourceHandlerFactory(resHandlerFactory);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setSerializableDataFactory
  (JNIEnv *env,jobject thisObj,jlong config,jlong serFactory)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return ;
	}

	SHPSerializableDataFactory* serDataFactory = ( SHPSerializableDataFactory*) (serFactory) ;
	if ( NULL == serDataFactory )
	{
		JNIGlobal::throwException("Native serialization DataFactory object is NULL") ;
		return ;
	}

	con->setSerializableDataFactory(serDataFactory);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_addDataFormat
  (JNIEnv *env, jobject thisObj, jlong config, jstring contentType, jobject serializer, jobject deserializer)
{
//	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
//
//	const char* conType = env->GetStringUTFChars(contentType,0);
//
//	Sec::Shp::Serialization::IDeserializer& ideserializer = new Sec::Shp::Serialization::IDeserializer&;
//	ideserializer = deserializer;
//
//	if ( NULL == con)
//	{
//		jclass mapClass = env->FindClass("java/lang/Exception");
//		(env)->ThrowNew(mapClass, "Configuration object is NULL") ;
//	}
//	con->addDataFormat((char*)conType,serDataFac,ideserializer);

}

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setServerConnector
  (JNIEnv *env, jobject thisObj, jlong config, jlong serverConnector)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return ;
	}

	Sec::Shp::Connector::Server::IServerConnector* serConnector = (Sec::Shp::Connector::Server::IServerConnector*) serverConnector;
	if ( NULL == serConnector )
	{
		JNIGlobal::throwException("Native server Connector object is NULL") ;
		return ;
	}

	con->setServerConnector(*serConnector);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setClientConnector
  (JNIEnv *env, jobject thisObj, jlong config, jlong clientConnector)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;

	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	Sec::Shp::Connector::Client::IClientConnector* cConnector = (Sec::Shp::Connector::Client::IClientConnector*) clientConnector;
	if ( NULL == cConnector )
	{
		JNIGlobal::throwException("Native client Connector object is NULL") ;
		return ;
	}

	con->setClientConnector(*cConnector);
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Configuration_getClientConnectors
  (JNIEnv *env, jobject thisObj, jlong config)
{
	/*Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;

	if ( NULL == con)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Configuration object is NULL") ;
	}

	return (jobject)con->getClientConnectors();*/
	return NULL ;
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Configuration_getDeSerializers
  (JNIEnv *env, jobject thisObj, jlong config)
{
	/*Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;

	if ( NULL == con)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Configuration object is NULL") ;
	}

	return (jobject)con->getDeSerializers();*/
	return NULL ;
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Configuration_getDeviceFactory
  (JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	return (jobject)con->getDeviceFactory();
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Configuration_getResourceHandlerFactory
  (JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	return (jobject)con->getResourceHandlerFactory();
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Configuration_getSerializableDataFactory
  (JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	return (jobject)con->getSerializableDataFactory();
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Configuration_getSerializers
  (JNIEnv *env, jobject thisObj, jlong config)
{
	/*Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;

	if ( NULL == con)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Configuration object is NULL") ;
	}

	return (jobject)con->getSerializers();*/
	return NULL ;
}


JNIEXPORT jobject JNICALL Java_Sec_Shp_Configuration_getServerConnectors
  (JNIEnv *env, jobject thisObj, jlong config)
{
	/*Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;

	if ( NULL == con)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Configuration object is NULL") ;
	}

	return (jobject)con->getServerConnectors();*/
	return NULL ;
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Configuration_getAppType
  (JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}
	jobject apptype =  JNIGlobal::getEnum("Sec/Shp/ApplicationType","getAppType","(I)LSec/Shp/ApplicationType;",(int)con->getAppType()) ;
	return apptype ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setAppType
  (JNIEnv *env, jobject thisObj, jlong config, jint applicationType)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}
	Sec::Shp::ApplicationType appType = (Sec::Shp::ApplicationType)applicationType ;
	con->setAppType(appType);
}


JNIEXPORT jstring JNICALL Java_Sec_Shp_Configuration_getSubscriptionDbPath
  (JNIEnv *env, jobject thisObj, jlong config)
{
	/*Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;

	if ( NULL == con)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Configuration object is NULL") ;
	}

	return (jstring)(con->getSubscriptionDbPath());*/
	return NULL ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_setSubscriptionDbPath
  (JNIEnv *env, jobject thisObj, jlong config, jstring subsPath)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return JNI_FALSE;
	}

	std::string subPath ;
	if (false == JNIGlobal::convertToStdString(subsPath, subPath))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (subscripton db path)") ;
		return JNI_FALSE;
	}

	if (false == con->setSubscriptionDbPath(subPath.c_str())) { return JNI_FALSE; }
	return JNI_TRUE;
}


JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setRAConfigPath__JLjava_lang_String_2
  (JNIEnv *env, jobject thisObj, jlong config, jstring conPath)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturnVoid( NULL != con, "Native Configuration is invalid");

	std::string raPath ;
	bool bret = JNIGlobal::convertToStdString(conPath, raPath);
	JNI_TryReturnVoid( bret != false, "Failed to convert jstring to std::string (Remote Access Configuration Path)");

	con->setRAConfigPath(raPath.c_str());
#endif /* REMOTE_ACCESS_SUPPORT */
}

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setRAConfigPath__JLjava_lang_String_2Ljava_lang_String_2
  (JNIEnv *env, jobject thisObj, jlong config, jstring conPath, jstring jKey)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturnVoid( NULL != con, "Native Configuration is invalid");

	std::string raPath ;
	bool bret = JNIGlobal::convertToStdString(conPath, raPath);
	JNI_TryReturnVoid( bret != false, "Failed to convert jstring to std::string (Remote Access Configuration Path)");

	std::string privateKey ;
	bret = JNIGlobal::convertToStdString(jKey, privateKey);
	JNI_TryReturnVoid( bret != false, "Failed to convert jstring to std::string (Private Key)");

	con->setRAConfigPath(raPath.c_str(), privateKey.c_str());
#endif /* REMOTE_ACCESS_SUPPORT */
}


JNIEXPORT jstring JNICALL Java_Sec_Shp_Configuration_getRAConfigPath
(JNIEnv *env, jobject thisObj, jlong config )
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, NULL, "Native Configuration is invalid");

	return JNIGlobal::convertToJString(con->getRAConfigPath()) ;
#endif /* REMOTE_ACCESS_SUPPORT */
	return NULL;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Configuration_getRAPrivatetKey
(JNIEnv *env, jobject thisObj, jlong config )
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, NULL, "Native Configuration is invalid");

	return JNIGlobal::convertToJString(con->getRAPrivateKey()) ;
#endif /* REMOTE_ACCESS_SUPPORT */
	return NULL;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setRemoteAccessEnable
  (JNIEnv *env, jobject thisObj, jlong config, jboolean enable)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturnVoid( NULL != con, "Native Configuration is invalid");

	con->setRemoteAccessEnable(enable == JNI_TRUE);
#endif /* REMOTE_ACCESS_SUPPORT */
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_Configuration_getRemoteAccessConfig
(JNIEnv *env, jobject thisObj, jlong config)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return (jlong)0;
	}

	Sec::Shp::RemoteAccessConfig *rConfig = &(con->getRemoteAccessConfig());

	return (jlong)rConfig;
#endif /* REMOTE_ACCESS_SUPPORT */
	return (jlong)0;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setWifiDetails
  (JNIEnv *env, jobject thisObj, jlong config, jlong serializable)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config ;
	JNI_TryReturnVoid( NULL != con, "Native Configuration is invalid");

	Sec::Shp::Serialization::ISerializable  *iserializable = (Sec::Shp::Serialization::ISerializable*)serializable;

	con->setWifiDetails(iserializable);
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_setLogFilePath
  (JNIEnv *env, jobject thisObj, jlong config, jstring jDirectoryPath, jstring jFileName)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	std::string directoryPath ;
	bool bret = JNIGlobal::convertToStdString(jDirectoryPath, directoryPath);
	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (Log Directory Path)");

	std::string fileName ;
	bret = JNIGlobal::convertToStdString(jFileName, fileName);
	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (File Name)");

	return con->setLogFilePath(directoryPath.c_str(), fileName.c_str());
}


/*
 * Class:     Sec_Shp_Configuration
 * Method:    addFileSharing
 * Signature: (JLjava/lang/String;Ljava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_addFileSharing__JLjava_lang_String_2Ljava_lang_String_2
  (JNIEnv *env, jobject thisObj, jlong config,  jstring juri, jstring jfPath)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	std::string uri ;
	bool bret = JNIGlobal::convertToStdString(juri, uri);
	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (Uri)");

	std::string fPath ;
	bret = JNIGlobal::convertToStdString(jfPath, fPath);
	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (File Path)");

	if(con->addFileSharing(uri, fPath) == false) { return JNI_FALSE; }
	return JNI_TRUE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    removeFileSharing
 * Signature: (JLjava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_removeFileSharing__JLjava_lang_String_2
  (JNIEnv *env, jobject thisObj, jlong config,  jstring jfPath)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	std::string fPath ;
	bool bret = JNIGlobal::convertToStdString(jfPath, fPath);
	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (File Path)");

	if(con->removeFileSharing(fPath) == false) { return JNI_FALSE; }
	return JNI_TRUE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    addFileSharing
 * Signature: (J[Ljava/lang/String;[Ljava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_addFileSharing__J_3Ljava_lang_String_2_3Ljava_lang_String_2
  (JNIEnv *env, jobject thisObj, jlong config,  jobjectArray juriList, jobjectArray jfPathList)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	if (NULL == juriList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "URI Path List is NULL") ;
	}

	if (NULL == jfPathList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "File Path List is NULL") ;
	}

	if (NULL != juriList && NULL != jfPathList) {
		int uriCount = env->GetArrayLength(juriList);
		int fPathCount = env->GetArrayLength(jfPathList);

		if(uriCount!=fPathCount) {
			jclass mapClass = env->FindClass("java/lang/Exception");
			(env)->ThrowNew(mapClass, "URI-List and File Path List are not of equal size!") ;
		}

		std::list<std::string> uriList;
		std::list<std::string> fPathList;

		for (int i=0; i< uriCount; i++) {
	        jstring jUri = (jstring)(env->GetObjectArrayElement(juriList, i));

	        std::string uri ;
	    	bool bret = JNIGlobal::convertToStdString(jUri, uri);
	    	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (URI - String Array member)");

	    	jstring jfPath = (jstring)(env->GetObjectArrayElement(jfPathList, i));

	        std::string fPath ;
	    	bret = JNIGlobal::convertToStdString(jfPath, fPath);
	    	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (FilePath - String Array member)");

	    	uriList.push_back(uri);
	    	fPathList.push_back(fPath);
		}

		if(true == con->addFileSharing(uriList, fPathList)) {
			return JNI_TRUE;
		}
	}

	return JNI_FALSE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    removeFileSharing
 * Signature: (J[Ljava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_removeFileSharing__J_3Ljava_lang_String_2
  (JNIEnv *env, jobject thisObj, jlong config,  jobjectArray jfPathList)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	if (NULL == jfPathList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "File Path List is NULL") ;
	}

	if (NULL != jfPathList) {
		int fPathCount = env->GetArrayLength(jfPathList);

		if(0>fPathCount) {
			jclass mapClass = env->FindClass("java/lang/Exception");
			(env)->ThrowNew(mapClass, "Empty File Path List!") ;
		}

		std::list<std::string> fPathList;

		for (int i=0; i< fPathCount; i++) {
	    	jstring jfPath = (jstring)(env->GetObjectArrayElement(jfPathList, i));

	        std::string fPath ;
	    	bool bret = JNIGlobal::convertToStdString(jfPath, fPath);
	    	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (FilePath - String Array member)");

	    	fPathList.push_back(fPath);
		}

		if(true == con->removeFileSharing(fPathList)) {
			return JNI_TRUE;
		}
	}

	return JNI_FALSE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    getFileSharingList
 * Signature: (JLjava/util/List;Ljava/util/List;)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_getFileSharingList
  (JNIEnv *env, jobject thisObj, jlong config, jobject juriList, jobject jfPathList)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	printf("\n\n### Inside getFileSharingList");
	if (NULL == juriList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "URI Path List is NULL") ;
	}

	if (NULL == jfPathList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "File Path List is NULL") ;
	}

	if (NULL != juriList && NULL != jfPathList) {
		std::list<std::string> uriList;
		std::list<std::string> fPathList;

		printf("\n\n### Before calling C++");
		if(false == con->getFileSharingList(&uriList, &fPathList)) {
			return JNI_FALSE;
		}

		/////////////////////////////////////////////////////
#if UT
		jclass jListClass = env->FindClass( "java/util/List" );

		if(NULL == jListClass) {
			jclass mapClass = env->FindClass("java/lang/Exception");
			(env)->ThrowNew(mapClass, "Failed to find class of java/util/List") ;
		}

		jmethodID getAddMethodID = env->GetMethodID(jListClass, "add", "(Ljava/lang/String;)Z");
		//jmethodID getAddMethodID = env->GetMethodID(jListClass, "add", "(Ljava/lang/Object;)Z" );
		if(NULL == getAddMethodID) {
			jclass mapClass = env->FindClass("java/lang/Exception");
			(env)->ThrowNew(mapClass, "Failed to get method of for java/util/List:boolean add(String)") ;
		}
#endif

		std::list<std::string>::iterator uItr = uriList.begin();
		std::list<std::string>::iterator fItr = fPathList.begin();
		std::string uri = "";
		std::string fPath = "";

		printf("\n\n### Before getFileSharingList, uList[%d] and fList[%d]", uriList.size(), fPathList.size());

		for(; uItr!=uriList.end() && fItr!=fPathList.end(); ++uItr, ++fItr) {
			uri=(*uItr);
			printf("\n\n### Before converting juri");
			jstring juri = JNIGlobal::convertToJString(uri);

			printf("\n\n### Before converting executeBoolMethod juri");
			if(false == JNIGlobal::executeBoolMethod("java/util/List", "add", "(Ljava/lang/Object;)Z", juriList, juri)) {
				jclass mapClass = env->FindClass("java/lang/Exception");
				(env)->ThrowNew(mapClass, "Failed to add URI to uriList java/util/List:boolean add(String)") ;
			}

#if UT
			bool retVal = (bool)(callBooleanMethod(env, juriList, getAddMethodID, juri));
			if(false == retVal) {
				jclass mapClass = env->FindClass("java/lang/Exception");
				(env)->ThrowNew(mapClass, "Failed to add URI to uriList java/util/List:boolean add(String)") ;
			}
#endif

			fPath=(*fItr);
			printf("\n\n### Before converting jfPath");
			jstring jfPath = JNIGlobal::convertToJString(fPath);

			printf("\n\n### Before converting executeBoolMethod jfPath");
			if(false == JNIGlobal::executeBoolMethod("java/util/List", "add", "(Ljava/lang/Object;)Z", jfPathList, jfPath)) {
				jclass mapClass = env->FindClass("java/lang/Exception");
				(env)->ThrowNew(mapClass, "Failed to add FilePath to fPathList java/util/List:boolean add(String)") ;
			}

#if UT
			retVal = (bool)(env->callBooleanMethod(jfPathList, getAddMethodID, jfPath));
			if(false == retVal) {
				jclass mapClass = env->FindClass("java/lang/Exception");
				(env)->ThrowNew(mapClass, "Failed to add FilePath to fPathList java/util/List:boolean add(String)") ;
			}
#endif
		}

		//jobject addObj = env->NewObject(jListClass, getAddMethodID, string);
		//jobject identParamsObject = env->CallObjectMethod( listObject, getMethodID, i - 1 );

		//////////////////////////////////////////////////
		return JNI_TRUE;
	}

	return JNI_FALSE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    addDirectorySharing
 * Signature: (JLjava/lang/String;Ljava/lang/String;Z)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_addDirectorySharing__JLjava_lang_String_2Ljava_lang_String_2Z
  (JNIEnv *env, jobject thisObj, jlong config,  jstring jdUri, jstring jdPath, jboolean jincSubDir)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	std::string dUri ;
	bool bret = JNIGlobal::convertToStdString(jdUri, dUri);
	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (Dir Uri)");

	std::string dPath ;
	bret = JNIGlobal::convertToStdString(jdPath, dPath);
	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (Dir Path)");

	if(con->addDirectorySharing(dUri, dPath) == false) { return JNI_FALSE; }
	return JNI_TRUE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    removeDirectorySharing
 * Signature: (JLjava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_removeDirectorySharing__JLjava_lang_String_2
  (JNIEnv *env, jobject thisObj, jlong config,  jstring jdPath)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");


	std::string dPath ;
	bool bret = JNIGlobal::convertToStdString(jdPath, dPath);
	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (Dir Path)");

	if(con->removeDirectorySharing(dPath) == false) { return JNI_FALSE; }
	return JNI_TRUE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    addDirectorySharing
 * Signature: (J[Ljava/lang/String;[Ljava/lang/String;Z)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_addDirectorySharing__J_3Ljava_lang_String_2_3Ljava_lang_String_2Z
  (JNIEnv *env, jobject thisObj, jlong config, jobjectArray juriList, jobjectArray jdPathList, jboolean jincSubDirs)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	if (NULL == juriList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "URI Path List is NULL") ;
	}

	if (NULL == jdPathList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Directory Path List is NULL") ;
	}

	if (NULL != juriList && NULL != jdPathList) {
		int uriCount = env->GetArrayLength(juriList);
		int dPathCount = env->GetArrayLength(jdPathList);

		if(uriCount!=dPathCount) {
			jclass mapClass = env->FindClass("java/lang/Exception");
			(env)->ThrowNew(mapClass, "URI-List and Directory Path List are not of equal size!") ;
		}

		std::list<std::string> uriList;
		std::list<std::string> dPathList;

		for (int i=0; i< uriCount; i++) {
	        jstring jUri = (jstring)(env->GetObjectArrayElement(juriList, i));

	        std::string uri ;
	    	bool bret = JNIGlobal::convertToStdString(jUri, uri);
	    	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (URI - String Array member)");

	    	jstring jdPath = (jstring)(env->GetObjectArrayElement(jdPathList, i));

	        std::string dPath ;
	    	bret = JNIGlobal::convertToStdString(jdPath, dPath);
	    	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (DirecotryPath - String Array member)");

	    	uriList.push_back(uri);
	    	dPathList.push_back(dPath);
		}

		bool incSubDirs = (jincSubDirs==JNI_TRUE) ? (true) : (false);

		if(true == con->addDirectorySharing(uriList, dPathList, incSubDirs)) {
			return JNI_TRUE;
		}
	}

	return JNI_FALSE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    removeDirectorySharing
 * Signature: (J[Ljava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_removeDirectorySharing__J_3Ljava_lang_String_2
  (JNIEnv *env, jobject thisObj, jlong config, jobjectArray jdPathList)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	if (NULL == jdPathList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Directory Path List is NULL") ;
	}

	if (NULL != jdPathList) {
		int dPathCount = env->GetArrayLength(jdPathList);

		if(0>dPathCount) {
			jclass mapClass = env->FindClass("java/lang/Exception");
			(env)->ThrowNew(mapClass, "Empty Directory Path List!") ;
		}

		std::list<std::string> dPathList;

		for (int i=0; i< dPathCount; i++) {
	    	jstring jdPath = (jstring)(env->GetObjectArrayElement(jdPathList, i));

	        std::string dPath ;
	    	bool bret = JNIGlobal::convertToStdString(jdPath, dPath);
	    	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (DirecotryPath - String Array member)");

	    	dPathList.push_back(dPath);
		}

		if(true == con->removeDirectorySharing(dPathList)) {
			return JNI_TRUE;
		}
	}

	return JNI_FALSE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    getDirectorySharingList
 * Signature: (JLjava/util/List;Ljava/util/List;)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_getDirectorySharingList
  (JNIEnv *env, jobject thisObj, jlong config, jobject juriList, jobject jdPathList)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	if (NULL == juriList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "URI Path List is NULL") ;
	}

	if (NULL == jdPathList) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Directory Path List is NULL") ;
	}

	if (NULL != juriList && NULL != jdPathList) {
		std::list<std::string> uriList;
		std::list<std::string> dPathList;

		if(false == con->getDirectorySharingList(&uriList, &dPathList)) {
			return false;
		}

		/////////////////////////////////////////////////////
#if UT
		jclass jListClass = env->FindClass( "java/util/List" );

		if(NULL == jListClass) {
			jclass mapClass = env->FindClass("java/lang/Exception");
			(env)->ThrowNew(mapClass, "Failed to find class of java/util/List") ;
		}

		jmethodID getAddMethodID = env->GetMethodID(jListClass, "add", "(Ljava/lang/String;)Z");
		//jmethodID getAddMethodID = env->GetMethodID(jListClass, "add", "(Ljava/lang/Object;)Z" );
		if(NULL == getAddMethodID) {
			jclass mapClass = env->FindClass("java/lang/Exception");
			(env)->ThrowNew(mapClass, "Failed to get method of for java/util/List:boolean add(String)") ;
		}
#endif
		std::list<std::string>::iterator uItr = uriList.begin();
		std::list<std::string>::iterator fItr = dPathList.begin();
		std::string uri = "";
		std::string dPath = "";

		for(; uItr!=uriList.end() && fItr!=dPathList.end(); ++uItr, ++fItr) {
			uri=(*uItr);
			jstring juri = JNIGlobal::convertToJString(uri);

			if(false == JNIGlobal::executeBoolMethod("java/util/List", "add", "(Ljava/lang/Object;)Z", juriList, juri)) {
				jclass mapClass = env->FindClass("java/lang/Exception");
				(env)->ThrowNew(mapClass, "Failed to add URI to uriList java/util/List:boolean add(String)") ;
			}
#if UT
			bool retVal = (bool)(env->callBooleanMethod(juriList, getAddMethodID, juri));

			if(false == retVal) {
				jclass mapClass = env->FindClass("java/lang/Exception");
				(env)->ThrowNew(mapClass, "Failed to add URI to uriList java/util/List:boolean add(String)") ;
			}
#endif
			dPath=(*fItr);
			jstring jdPath = JNIGlobal::convertToJString(dPath);

			if(false == JNIGlobal::executeBoolMethod("java/util/List", "add", "(Ljava/lang/Object;)Z", jdPathList, jdPath)) {
				jclass mapClass = env->FindClass("java/lang/Exception");
				(env)->ThrowNew(mapClass, "Failed to add DirectoryPath to dPathList java/util/List:boolean add(String)") ;
			}

#if UT
			retVal = (bool)(env->callBooleanMethod(jdPathList, getAddMethodID, jdPath));

			if(false == retVal) {
				jclass mapClass = env->FindClass("java/lang/Exception");
				(env)->ThrowNew(mapClass, "Failed to add DirectoryPath to dPathList java/util/List:boolean add(String)") ;
			}
#endif
		}

		//jobject addObj = env->NewObject(jListClass, getAddMethodID, string);
		//jobject identParamsObject = env->CallObjectMethod( listObject, getMethodID, i - 1 );

		//////////////////////////////////////////////////
		return JNI_TRUE;
	}

	return JNI_FALSE;
}

/*
 * Class:     Sec_Shp_Configuration
 * Method:    setFileExtToContentTypeConfigFilePath
 * Signature: (JLjava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Configuration_setFileExtToContentTypeConfigFilePath
  (JNIEnv *env, jobject thisObj, jlong config,  jstring jfConfPath)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, JNI_FALSE, "Native Configuration is invalid");

	std::string fConfPath ;
	bool bret = JNIGlobal::convertToStdString(jfConfPath, fConfPath);
	JNI_TryReturn( bret != false, JNI_FALSE, "Failed to convert jstring to std::string (File Ext to Con Type File Path)");


	if(false == con->setFileExtToContentTypeConfigFilePath(fConfPath.c_str())) {
		return JNI_FALSE;
	}

	return JNI_TRUE;
}

/*
 * Class:     Sec_Shp_Configuration	return JNI_TRUE;
 * Method:    getFileExtToContentTypeConfigFilePath
 * Signature: (J)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_Sec_Shp_Configuration_getFileExtToContentTypeConfigFilePath
  (JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturn( NULL != con, NULL, "Native Configuration is invalid");

	return (JNIGlobal::convertToJString(con->getFileExtToContentTypeConfigFilePath())) ;
}


JNIEXPORT void JNICALL Java_Sec_Shp_Configuration_setDeviceTokenHashingEnable
  (JNIEnv *env, jobject thisObj, jlong config, jboolean enable)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	JNI_TryReturnVoid( NULL != con, "Native Configuration is invalid");

	con->setDeviceTokenHashingEnable(enable == JNI_TRUE);
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_Configuration_isDeviceTokenHashingEnabled
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Configuration* con = (Sec::Shp::Configuration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return (jlong)0;
	}

	return ( con->isDeviceTokenHashingEnabled() ) ? JNI_TRUE : JNI_FALSE ;
}
