/*
 * JNIDevice.cpp
 *
 *  Created on: Jul 7, 2013
 *      Author: kirtigupta
 */

#include "Sec_Shp_Device.h"
#include "DeviceType.h"
#include "DeviceDomain.h"
#include "Serializable.h"
#include "Device.h"
#include "SHPSerialization.h"
#include "JNISHPDev.h"
#include "Serializable.h"

#include "DeviceToken.h"

JNISHPDevice::JNISHPDevice( jobject jdevice, jint type, jint domain)
:Sec::Shp::Device((Sec::Shp::DeviceType)type, (Sec::Shp::DeviceDomain)domain)
{
	m_device = JNIGlobal::getGlobleRef(jdevice) ;
}

JNISHPDevice::~JNISHPDevice()
{
	JNIGlobal::executeVoidMethod("Sec/Shp/Device", "onNativeDelete", "()V", m_device);
	JNIGlobal::releaseGlobleRef(m_device) ;
}

bool
JNISHPDevice::loadDeviceDescription(Sec::Shp::Serialization::ISerializable& serializable)
{
	Serializable* data = dynamic_cast<Serializable*> (&serializable) ;
	if (data != NULL) {
		return JNIGlobal::executeBoolMethod("Sec/Shp/Device", "loadDeviceDescription", "(LSec/Shp/Serialization/ISerializable;)Z", m_device, data->ser_handler) ;
	}

	return false;
}

bool
JNISHPDevice::getDeviceDescription(Sec::Shp::Serialization::ISerializable** ppSerializable)
{
	jobject obj = JNIGlobal::executeObjectMethod ("Sec/Shp/Device", "getDeviceDescription", "()LSec/Shp/Serialization/ISerializable;", m_device) ;
	if (NULL == obj)
	{
		return false ;
	}

	*ppSerializable = new Serializable(obj) ;
	JNIGlobal::releaseLocalRef(obj);

	return ( NULL == *ppSerializable ) ? false : true ;
}

std::string
JNISHPDevice::getSubscriptionsLink()
{
	jstring strLink = (jstring)JNIGlobal::executeObjectMethod("Sec/Shp/Device", "getSubscriptionsLink", "()Ljava/lang/String;", m_device) ;
	if (NULL == strLink)
	{
		return ""; // TODO: Check return value
	}

	std::string subsLink ;
	if(false == JNIGlobal::convertToStdString(strLink, subsLink))
		return "";

	JNIGlobal::releaseLocalRef(strLink);

	return subsLink;
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_Device_constructNative
(JNIEnv *env, jobject thisObj, jint devType, jint devDomain)
{
	Sec::Shp::Device* deviceObj = new JNISHPDevice(thisObj, devType, devDomain) ;
	JNI_TryReturn( NULL != deviceObj, 0, "Memory Allocation failed : Out of memory")

	return (jlong)(deviceObj);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_deleteNative
(JNIEnv *, jobject, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev;

	if (NULL != device)
	{
		delete device;
	}
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_Device_getContext
  (JNIEnv *, jobject, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev;
	if (NULL != device)
	{
		return (jlong) device->getContext() ;
	}

	return 0 ;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getAddress
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getAddress()) ; 
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getProtocol
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getProtocol()) ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setAddress
(JNIEnv *env, jobject thisObj, jlong dev,jstring address )
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string dAddress ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(address, dAddress)), "device address conversion failed") ;

	device->setAddress(dAddress.c_str());
}


JNIEXPORT void JNICALL Java_Sec_Shp_Device_setProtocol
(JNIEnv *env, jobject thisObj, jlong dev,jstring address )
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string dAddress ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(address, dAddress)), "device address conversion failed") ;

	device->setProtocol(dAddress.c_str());
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setContext
(JNIEnv *env, jobject thisObj, jlong dev,jlong context )
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	JNI_TryReturnVoid( ( 0 != context), "context is invalid") ;

	device->setContext((Sec::Shp::IContext *)context) ;
}



JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getDeviceDescriptionUri
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getDeviceDescriptionUri()) ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setDeviceDescriptionUri
(JNIEnv *env, jobject thisObj, jlong dev, jstring devUri)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string dAddress ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(devUri, dAddress)), "devUri conversion failed") ;

	device->setDeviceDescriptionUri(dAddress.c_str());
}


JNIEXPORT jobject JNICALL Java_Sec_Shp_Device_getDeviceType
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	jobject jdeviceType = JNIGlobal::getEnum("Sec/Shp/DeviceType", "getDeviceType", "(I)LSec/Shp/DeviceType;" , (int)device->getDeviceType());
	JNI_TryReturn( NULL != jdeviceType, NULL, "Failed to get device type")

	return (jobject)jdeviceType;
}


JNIEXPORT void JNICALL Java_Sec_Shp_Device_setDeviceType
(JNIEnv *env, jobject thisObj, jlong dev, jint devType)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	device->setDeviceType((Sec::Shp::DeviceType)devType);
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Device_getDomain
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	jobject jdomain = JNIGlobal::getEnum("Sec/Shp/DeviceDomain", "getDeviceDomain", "(I)LSec/Shp/DeviceDomain;" , (int)device->getDomain());
	JNI_TryReturn( NULL != jdomain, NULL, "Failed to get domain")

	return (jobject)jdomain;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setDomain
(JNIEnv *env, jobject thisObj, jlong dev, jint domain)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	device->setDomain((Sec::Shp::DeviceDomain)domain);
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getUUID
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getUUID()) ;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getDeviceName
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getDeviceName()) ;
}
JNIEXPORT void JNICALL Java_Sec_Shp_Device_setDeviceName
(JNIEnv *env, jobject thisObj, jlong dev, jstring deviceName)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")
	std::string dName ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(deviceName ,dName)),"device name conversion failed") ;

	device->setDeviceName(dName.c_str());
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setUUID
(JNIEnv *env, jobject thisObj, jlong dev, jstring uuid)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string dAddress ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(uuid, dAddress)), "device uuid conversion failed") ;

	device->setUUID(dAddress.c_str());
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Device_isDeviceDescriptionLoaded
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, JNI_FALSE, "Native device is invalid")

	return device->isDeviceDescriptionLoaded() ? JNI_TRUE: JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Device_setDeviceDescriptionLoaded
  (JNIEnv *env, jobject thisObj, jlong dev, jboolean flag)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, JNI_FALSE, "Native device is invalid")

	if (device)
	{
		device->setDeviceDescriptionLoaded(flag == JNI_TRUE) ;
	}

	return JNI_TRUE ;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getDescription
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getDescription()) ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setDescription
(JNIEnv *env, jobject thisObj, jlong dev, jstring description)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string desc ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(description, desc)), "device description conversion failed") ;

	device->setDescription(desc.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getManufacturer
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getManufacturer()) ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setManufacturer
(JNIEnv *env, jobject thisObj, jlong dev, jstring manufacturer)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string manf ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(manufacturer, manf)), "device manufacturer conversion failed") ;

	device->setManufacturer(manf.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getDeviceSubType
(JNIEnv *env, jobject thisObj, jlong dev)
{
    Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
    JNI_TryReturn(NULL != device, NULL, "Native device is invalid")

    return JNIGlobal::convertToJString(device->getDeviceSubType());
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setDeviceSubType
(JNIEnv *env, jobject thisObj, jlong dev, jstring deviceSubType)
{
    Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
    JNI_TryReturnVoid(NULL != device, "Native device is invalid")

    std::string devSubTyp;
    JNI_TryReturnVoid((true == JNIGlobal::convertToStdString(deviceSubType, devSubTyp)), "device deviceSubType conversion failed");

    device->setDeviceSubType(devSubTyp.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getModelID
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getModelID()) ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setModelID
(JNIEnv *env, jobject thisObj, jlong dev, jstring id)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string mID ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(id, mID)), "device ModelID conversion failed") ;

	device->setModelID(mID.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getSerialNumber
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getSerialNumber()) ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setSerialNumber
(JNIEnv *env, jobject thisObj, jlong dev, jstring num)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string sNum ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(num, sNum)), "device SerialNumber conversion failed") ;

	device->setSerialNumber(sNum.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getSalesLocation
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(device->getSalesLocation()) ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setSalesLocation
(JNIEnv *env, jobject thisObj, jlong dev, jstring jLoc)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string location ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(jLoc, location)), "device SalesLocation conversion failed") ;

	device->setSalesLocation(location.c_str());
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Device_getSupportedResources
  (JNIEnv *, jobject, jlong)
{
	// Needs to implement
	return NULL;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setSupportedResource
(JNIEnv *env, jobject thisObj, jlong dev, jstring jresource)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturnVoid( NULL != device, "Native device is invalid")

	std::string resource ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(jresource, resource)), "ResourceName conversion failed") ;

	device->setSupportedResource(resource.c_str());
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Device_deRegisterFromSHS
(JNIEnv *env, jobject thisObj, jlong dev)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, JNI_FALSE, "Native device is invalid")
	return device->deRegisterFromSHS() ? JNI_TRUE: JNI_FALSE;
#else /* REMOTE_ACCESS_SUPPORT */
	return JNI_TRUE;
#endif
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Device_getDeviceOnlineStatus
(JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*) dev ;
	JNI_TryReturn( NULL != device, JNI_FALSE, "Native device is invalid")

	return device->getDeviceOnlineStatus() ? JNI_TRUE: JNI_FALSE;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setTokenGrantStatus
(JNIEnv *env, jobject thisObj, jlong dev, jboolean status)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
	JNI_TryReturnVoid(NULL != device, "Native device is invalid");

	device->setTokenGrantStatus(status == JNI_TRUE);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_updateDeviceTokenList
  (JNIEnv *env, jobject thisObj, jlong dev, jlongArray jdevtokenlist)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
	JNI_TryReturnVoid(NULL != device, "Native device is invalid");

	if (NULL == jdevtokenlist) {
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource Path array is NULL") ;
	}

	jboolean var = false;

	if (NULL != jdevtokenlist) {
		int count = env->GetArrayLength(jdevtokenlist);
		jlong* nativeList = env->GetLongArrayElements(jdevtokenlist, &var);
		for (int i=0; i< count; i++) {
			Sec::Shp::DeviceToken* devToken = (Sec::Shp::DeviceToken*) nativeList[i];
			if (devToken != NULL) {
				device->setPeerTokenIntoUUIDMap(devToken->getUUID().c_str(), devToken->getDeviceToken().c_str());
			}
		}
	}
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Device_initiateTokenRequest
(JNIEnv *env, jobject thisObj, jlong dev, jstring jIP, jstring jPort)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
	JNI_TryReturn( NULL != device, JNI_FALSE, "Native device is invalid")

	std::string mIP ;
	JNI_TryReturn((true == JNIGlobal::convertToStdString(jIP, mIP)), JNI_FALSE, "device IP conversion failed") ;
	return device->initiateTokenRequestUsingUUID(mIP.c_str(), false);
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Device_initiateTokenRequestUsingUUID
(JNIEnv *env, jobject thisObj, jlong dev, jstring jUUID, jboolean fromSimulator)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
	JNI_TryReturn( NULL != device, JNI_FALSE, "Native device is invalid")

	std::string mUUID;
	JNI_TryReturn((true == JNIGlobal::convertToStdString(jUUID, mUUID)), JNI_FALSE, "device UUID conversion failed") ;

	return device->initiateTokenRequestUsingUUID(mUUID.c_str(), fromSimulator == JNI_TRUE);
}


JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getMyDeviceToken
  (JNIEnv *env, jobject thisObj, jlong dev)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
	JNI_TryReturn(NULL != device, NULL, "Native device is invalid");

	return JNIGlobal::convertToJString(device->getMyDeviceToken());
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setMyDeviceToken
  (JNIEnv *env, jobject thisObj, jlong dev, jstring jToken)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
	JNI_TryReturnVoid(NULL != device, "Native device is invalid");

	std::string mToken ;
	JNI_TryReturnVoid((true == JNIGlobal::convertToStdString(jToken, mToken)), "device Token conversion failed") ;

	device->setMyDeviceToken(mToken.c_str()) ;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Device_getPeerTokenFromUUIDMap
  (JNIEnv *env, jobject thisObj, jlong dev, jstring jUUID)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
	JNI_TryReturn(NULL != device, NULL, "Native device is invalid");

	std::string mUUID ;
	JNI_TryReturn((true == JNIGlobal::convertToStdString(jUUID, mUUID)), NULL, "device UUID conversion failed") ;

	return JNIGlobal::convertToJString(device->getPeerTokenFromUUIDMap(mUUID.c_str())) ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Device_setPeerTokenIntoUUIDMap
  (JNIEnv *env, jobject thisObj, jlong dev, jstring jUUID, jstring jToken)
{
	Sec::Shp::Device* device = (Sec::Shp::Device*)dev;
	JNI_TryReturnVoid(NULL != device, "Native device is invalid");

	std::string mUUID ;
	JNI_TryReturnVoid((true == JNIGlobal::convertToStdString(jUUID, mUUID)), "device UUID conversion failed") ;

	std::string mToken ;
	JNI_TryReturnVoid((true == JNIGlobal::convertToStdString(jToken, mToken)), "device Token conversion failed") ;

	device->setPeerTokenIntoUUIDMap(mUUID.c_str(), mToken.c_str());
}
