/*
 * JNIDeviceFinder.cpp
 *
 *  Created on: Jul 4, 2013
 *      Author: kirtigupta
 */

#include <jni.h>
#include "Sec_Shp_DeviceFinder_IDeviceFinder.h"
#include "SHPDeviceFinder.h"
#include "MyDeviceFinderListener.h"
#include "MyResourceResponseListener.h"
#include "list"
#include "JNISHPDev.h"

JNIEXPORT jboolean JNICALL Java_Sec_Shp_DeviceFinder_IDeviceFinder_refresh( JNIEnv *env, jobject thisObj, jlong devFinder )
{
	Sec::Shp::DeviceFinder::IDeviceFinder* devFindObj = (Sec::Shp::DeviceFinder::IDeviceFinder* ) devFinder ;
	JNI_TryReturn( NULL != devFindObj, JNI_FALSE,  "Native instance is invalid")

	return devFindObj->refresh() ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_DeviceFinder_IDeviceFinder_setDeviceFinderListener__JLSec_Shp_DeviceFinder_IDeviceFinderListener_2( JNIEnv *env,jobject thisObj,jlong devFinder,jobject listener )
{
	Sec::Shp::DeviceFinder::IDeviceFinder* devFindObj = (Sec::Shp::DeviceFinder::IDeviceFinder* ) devFinder ;
	JNI_TryReturn( NULL != devFindObj, (jlong)0,  "Native instance is invalid")

	MyDeviceFinderListener* devFindListener = new MyDeviceFinderListener( listener );
	JNI_TryReturn( NULL != devFindListener, (jlong)0,  "Failed to allocate memory")

	devFindObj->setDeviceFinderListener( *devFindListener );
	return (jlong)devFindListener ;
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_DeviceFinder_IDeviceFinder_setDeviceFinderListener__JLSec_Shp_DeviceType_2LSec_Shp_DeviceFinder_IDeviceFinderListener_2
( JNIEnv *env,jobject thisObj,jlong devFinder,jint devType,jobject listener )
{
	Sec::Shp::DeviceFinder::IDeviceFinder* devFindObj = (Sec::Shp::DeviceFinder::IDeviceFinder* ) devFinder ;
	JNI_TryReturn( NULL != devFindObj, (jlong)0,  "Native instance is invalid")

	MyDeviceFinderListener* devFindListener = new MyDeviceFinderListener( listener );
	JNI_TryReturn( NULL != devFindListener, (jlong)0,  "Failed to allocate memory")

	devFindObj->setDeviceFinderListener((Sec::Shp::DeviceType) devType, *devFindListener);
	return (jlong)devFindListener ;
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_DeviceFinder_IDeviceFinder_setDeviceFinderListener__JLSec_Shp_DeviceDomain_2LSec_Shp_DeviceType_2LSec_Shp_DeviceFinder_IDeviceFinderListener_2
( JNIEnv *env,jobject thisObj,jlong devFinder,jint domain,jint devType,jobject listener )
{
	Sec::Shp::DeviceFinder::IDeviceFinder* devFindObj = (Sec::Shp::DeviceFinder::IDeviceFinder* ) devFinder ;
	JNI_TryReturn( NULL != devFindObj, (jlong)0,  "Native instance is invalid")

	MyDeviceFinderListener* devFindListener = new MyDeviceFinderListener( listener );
	JNI_TryReturn( NULL != devFindListener, (jlong)0,  "Failed to allocate memory")

	devFindObj->setDeviceFinderListener((Sec::Shp::DeviceDomain)domain, (Sec::Shp::DeviceType)devType, *devFindListener);
	return (jlong)devFindListener ;
}

JNIEXPORT jobjectArray JNICALL Java_Sec_Shp_DeviceFinder_IDeviceFinder_getDevices__J
( JNIEnv *env,jobject thisObj,jlong devFinder )
{
	jobjectArray resultList = NULL ;

	Sec::Shp::DeviceFinder::IDeviceFinder* devFindObj = (Sec::Shp::DeviceFinder::IDeviceFinder* ) devFinder ;
	JNI_TryReturn( NULL != devFindObj, NULL,  "Native instance is invalid")

	std::list<Sec::Shp::Device*> deviceList = devFindObj->getDevices();
	if (deviceList.size() != 0)
	{
		jsize size = devFindObj->getDevices().size();
		int index = 0 ;

		for ( std::list<Sec::Shp::Device*>::const_iterator itr = deviceList.begin(); itr != deviceList.end(); itr++, index++ )
		{
			JNISHPDevice *jniDevice = (JNISHPDevice*) (*itr) ;
			jobject javaDeviceObject = jniDevice->m_device ;

			if (NULL == resultList)
			{
				jclass javaDeviceClass = env->GetObjectClass(javaDeviceObject) ;
				resultList = env->NewObjectArray(size, javaDeviceClass, NULL) ;
			}

			env->SetObjectArrayElement(resultList, index ,javaDeviceObject) ;
		}
	}

	return resultList;
}

JNIEXPORT jobjectArray JNICALL Java_Sec_Shp_DeviceFinder_IDeviceFinder_getDevices__JI
( JNIEnv *env,jobject thisObj,jlong devFinder,jint deviceType )
{
	jobjectArray resultList = NULL ;

	Sec::Shp::DeviceFinder::IDeviceFinder* devFindObj = (Sec::Shp::DeviceFinder::IDeviceFinder* ) devFinder ;
	JNI_TryReturn( NULL != devFindObj, NULL,  "Native instance is invalid")

	Sec::Shp::DeviceType dType = (Sec::Shp::DeviceType)deviceType;
	std::list<Sec::Shp::Device*> deviceList = devFindObj->getDevices(dType);
	if (deviceList.size() != 0)
	{
		jsize size = devFindObj->getDevices().size();
		int index = 0 ;

		for ( std::list<Sec::Shp::Device*>::const_iterator itr = deviceList.begin(); itr != deviceList.end(); itr++, index++ )
		{
			JNISHPDevice *jniDevice = (JNISHPDevice*) (*itr) ;
			jobject javaDeviceObject = jniDevice->m_device ;

			if (NULL == resultList)
			{
				jclass javaDeviceClass = env->GetObjectClass(javaDeviceObject) ;
				resultList = env->NewObjectArray(size, javaDeviceClass, NULL) ;
			}

			env->SetObjectArrayElement(resultList, index ,javaDeviceObject) ;
		}
	}

	return resultList;
}

JNIEXPORT jobjectArray JNICALL Java_Sec_Shp_DeviceFinder_IDeviceFinder_getDevices__JII
( JNIEnv *env,jobject thisObj,jlong devFinder,jint devDomain,jint deviceType )
{
	jobjectArray resultList = NULL ;

	Sec::Shp::DeviceFinder::IDeviceFinder* devFindObj = (Sec::Shp::DeviceFinder::IDeviceFinder* ) devFinder ;
	JNI_TryReturn( NULL != devFindObj, NULL,  "Native instance is invalid")

	Sec::Shp::DeviceDomain dom = (Sec::Shp::DeviceDomain)devDomain;
	Sec::Shp::DeviceType dType = (Sec::Shp::DeviceType)deviceType;
	std::list<Sec::Shp::Device*> deviceList = devFindObj->getDevices(dom,dType);
	if (deviceList.size() != 0)
	{
		jsize size = devFindObj->getDevices().size();
		int index = 0 ;

		for ( std::list<Sec::Shp::Device*>::const_iterator itr = deviceList.begin(); itr != deviceList.end(); itr++, index++ )
		{
			JNISHPDevice *jniDevice = (JNISHPDevice*) (*itr) ;
			jobject javaDeviceObject = jniDevice->m_device ;

			if (NULL == resultList)
			{
				jclass javaDeviceClass = env->GetObjectClass(javaDeviceObject) ;
				resultList = env->NewObjectArray(size, javaDeviceClass, NULL) ;
			}

			env->SetObjectArrayElement(resultList, index ,javaDeviceObject) ;
		}
	}

	return resultList;
}

JNIEXPORT void JNICALL Java_Sec_Shp_DeviceFinder_IDeviceFinder_removeDeviceFinderListener( JNIEnv *env,jobject thisObj,jlong devFinder, jlong listener )
{
	Sec::Shp::DeviceFinder::IDeviceFinder* devFindObj = (Sec::Shp::DeviceFinder::IDeviceFinder* ) devFinder ;
	JNI_TryReturnVoid( NULL != devFindObj,  "Native instance is invalid")

	MyDeviceFinderListener* devFindListener = (MyDeviceFinderListener*) listener ;
	JNI_TryReturnVoid( NULL != devFindListener,  "Failed to allocate memory")

	devFindObj->removeDeviceFinderListener(*devFindListener) ;
	delete devFindListener ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_DeviceFinder_IDeviceFinder_addDevice( JNIEnv *env,jobject thisObj, jlong devFinder, jint type, jstring juuid, jstring jAddress, jobject  listener)
{
	Sec::Shp::DeviceFinder::IDeviceFinder* devFindObj = (Sec::Shp::DeviceFinder::IDeviceFinder* ) devFinder ;
	JNI_TryReturn( NULL != devFindObj, JNI_FALSE, "Native instance is invalid")

	MyResourceResponseListener* responseListener = new MyResourceResponseListener( listener );
	if ( NULL == responseListener)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource Listener object is NULL") ;
		return JNI_FALSE;
	}

	responseListener->forceDeleteOnError = true ;
	responseListener->setRequestCount(2) ;

	std::string dAddress ;
//	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(jAddress, dAddress)), JNI_FALSE, "device address conversion failed") ;
	if ( true != JNIGlobal::convertToStdString(jAddress, dAddress)) {
		if (responseListener != NULL) {
			delete responseListener; responseListener = NULL;
		}
		return JNI_FALSE;
	}

	std::string dUUID ;
//	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(juuid, dUUID)), JNI_FALSE,"uuid conversion failed") ;
	if ( true != JNIGlobal::convertToStdString(juuid, dUUID)) {
		if (responseListener != NULL) {
			delete responseListener; responseListener = NULL;
		}
		return JNI_FALSE;
	}

	devFindObj->addDevice((Sec::Shp::DeviceType)type, dUUID, dAddress, *responseListener) ;

	return JNI_TRUE;
}


