/*
 * JNIGlobal.h
 *
 *  Created on: Jul 5, 2013
 *      Author: kirtigupta
 */

#ifndef JNIGLOBAL_H_
#define JNIGLOBAL_H_

#include <jni.h>
#include "string"
#include "map"
#include "Thread/Mutex.h"

using namespace std ;

#ifdef __ANDROID__

#define SHP_JNI_VERSION JNI_VERSION_1_2

#else

#define SHP_JNI_VERSION JNI_VERSION_1_1

#endif

class JNIEXPORT JNIGlobal {

public:

	JNIGlobal();

	virtual ~JNIGlobal();

	static jobject getGlobleRef(jobject obj) ;

	static void releaseGlobleRef(jobject obj) ;

	static void releaseLocalRef(jobject obj) ;

	static void throwException(const char* errorMessage) ;

	static bool convertToStdString(jstring jinput, std::string& out);

	static jstring convertToJString( std::string value ) ;

	static jobject getEnum(const char* className, const char* method, const char* signature, int enumValue) ;

	static jobject constructObject( const char* className, const char* methodSignature, ... );

	static bool executeBoolMethod( const char* className, const char* methodName, const char* methodSignature, jobject object, ... );

	static jlong executeLongMethod( const char* className, const char* methodName, const char* methodSignature, jobject object, ... );

	static jint executeIntMethod( const char* className, const char* methodName, const char* methodSignature, jobject object, ... );

	static void executeVoidMethod( const char* className, const char* methodName, const char* methodSignature, jobject object, ... );

	static jobject executeObjectMethod( const char* className, const char* methodName, const char* methodSignature,jobject object, ... );

	static JNIEnv* getEnv(bool attachIfRequired, bool& isAttached) ;

	static jint attachCurrentThread( JNIEnv **p_env ) ;

	static jint detachCurrentThread() ;

	static jmethodID getMethodId( JNIEnv* pEnv, const char* className, jclass classObj,
							const char* methodName, const char* methodSignature, bool isStatic,
							jobject object, bool determineClassName = false ) ;

	static jclass getClass( JNIEnv* pEnv, const char* className, jobject obj, bool& isAllocated ) ;

	static bool JNIConvertToMap( JNIEnv *env, jobjectArray javaMapEntries,
								 std::map< std::string, std::string >&cppMapEntries ) ;

	static jbyteArray createByteArray( std::string &payload, long size);

	static JavaVM *currentVM ;

	static jobject classLoader ;

	static std::map<std::string, jmethodID> jniMethodCache ;

	static std::map<std::string, jclass> jniClassCache ;



private:

	static jmethodID getCachedMethod(std::string key) ;

	static void cacheMethod(std::string key, jmethodID methodId) ;

	static Sec::Shp::Platform::Thread::Mutex m_metex ;
};


#define JNI_TryReturn(condition, returnValue, message) \
	if (!(condition)) { \
		JNIGlobal::throwException(message) ; \
		return returnValue; \
	} else {;}

#define JNI_TryReturnVoid(condition, message) \
	if (!(condition)) { \
		JNIGlobal::throwException(message) ; \
		return ; \
	} else {;}

#define JNI_TryReturnEx(condition, returnValue, expressions, message) \
	if (!(condition)) { \
		JNIGlobal::throwException(message) ; \
		expressions; \
		return returnValue; \
	} else {;}

#define JNI_TryReturnVoidEx(condition, expressions, message) \
	if (!(condition)) { \
		JNIGlobal::throwException(message) ; \
		expressions; \
		return ; \
	} else {;}

#endif /* JNIGLOBAL_H_ */
