/*
 * JNIHttpClient.cpp
 *
 *  Created on: Oct 28, 2015
 *      Author: bhupathi.r
 */

#include <com_sec_shp_sdk_http_client_HttpClient.h>
#include <Client/Client.h>

#include "Log.h"
#include "SHPCommon.h"
#include <string>
#include "SHPErrorCode.h"
#include "Utils.h"
#include "Connector/HTTP/HTTPMessage.h"
#include "Connector/Client/ClientSessionData.h"
#include "SHPConfiguration.h"
#include "JNIGlobal.h"
#include "SHPHTTPClientConnector.h"
#include "SHPHTTPSClientConnector.h"
#include "JNIHTTPClient.h"

#define CLASS_NAME "HttpClient"
#define MODULE_ID	SHP_JNI

struct jni_http_client_req_data
{
	jobject listener;
	jobject requestData;
};

namespace Sec {
namespace Shp {
namespace Jni {

JNIHTTPClient::JNIHTTPClient( Sec::Shp::Connector::Client::IClientConnector *pClientConnector, bool bUseSSL,  bool isInternal )
{
	//pClientConnector 	= NULL ;
	//pClient			 	= NULL ;
	this->bUseSSL		= bUseSSL ;
	bisInternal		 	= false;
	isValid 			= true	;

	this->pClientConnector 	= pClientConnector ;
	this->bisInternal		= isInternal;

	this->pClient = new Sec::Shp::Core::Client::Client();
	if (this->pClient)
	{
		this->pClient->addClientListener(*this);
	}
}


void
JNIHTTPClient::invalidateConnector()
{
	if (isValid)
	{
		if ( NULL != pClient )
		{
			pClient->stop();
			delete pClient ;

			pClient = NULL;
		}

		if(bisInternal)
		{
			if ( NULL != pClientConnector )
			{
				delete pClientConnector ;
				pClientConnector = NULL;
			}
		}
	}

	isValid = false;
}

JNIHTTPClient::~JNIHTTPClient()
{
	invalidateConnector();
}

bool
JNIHTTPClient::init()
{
	bool bRet = pClient->init(1, *(pClientConnector), mConfiguration);
	TryReturn( bRet == true, JNI_FALSE ,"Failed to initialize Client" ) ;

	bRet = pClient->start();
	TryReturn( bRet == true, JNI_FALSE,"Failed to start Client" ) ;

	return true;
}

/**
 *
 * listener method to receive response from a client connector. SHP framework will register a
 * IClientListener with client connector. client connector is supposed to call this method on receiving
 * the response of a request
 *
 *
 * @param[in] client_sessioin	client session object
 * @since   1.0.0*/

void
JNIHTTPClient::onResponseReceived( Sec::Shp::Connector::Client::ClientSession& client_sessioin )
{
	std::map<std::string, std::string, caseInsensitiveCompareLess> m_responseHeaders ;
	std::string payload = client_sessioin.getResponse().m_payload;
	long responsePayloadSize = client_sessioin.getResponse().m_payloadSize;
	int statuscode = client_sessioin.getResponse().m_response_code ;

	struct jni_http_client_req_data *pUserData = (struct jni_http_client_req_data *) (client_sessioin.getSessionData().m_pUserData) ;
	if ( NULL != pUserData )
	{
		jobject		jheaderList = NULL  ;
		jheaderList = JNIGlobal::constructObject("Sec/Shp/Connector/HeadersList","(J)V",(jlong)(&(client_sessioin.getResponse().m_headers))) ;

		jbyteArray jpayloadArray = NULL ;
		if (responsePayloadSize > 0)
		{
			jpayloadArray = JNIGlobal::createByteArray(payload, responsePayloadSize);
		}

		JNIGlobal::executeVoidMethod("com/sec/shp/sdk/http/client/IHttpClientResponseListener", "onResponseReceived",
									 "(Ljava/lang/Object;ILSec/Shp/Connector/HeadersList;[BZ)V",pUserData->listener,
									 pUserData->requestData, statuscode, jheaderList, jpayloadArray, client_sessioin.isLastChunkReceived ? JNI_TRUE : JNI_FALSE );

		JNIGlobal::releaseLocalRef(jheaderList);
		JNIGlobal::releaseLocalRef(jpayloadArray);
	}
}


/**
 *
 * listener method to receive response from a client connector. SHP framework will register a
 * IClientListener with client connector. client connector is supposed to call this method on receiving
 * error response for a request
 *
 * @param[in] error				error code
 * @param[in] client_sessioin	client session object
 *
 * @since   1.0.0*/
void
JNIHTTPClient::onErrorRecieved( Sec::Shp::ErrorCode error,
					  Sec::Shp::Connector::Client::ClientSession& client_sessioin)
{
	struct jni_http_client_req_data *pUserData = (struct jni_http_client_req_data *) (client_sessioin.getSessionData().m_pUserData) ;
	if ( NULL != pUserData )
	{
		jstring errMsg = JNIGlobal::convertToJString("Network/HTTP Related error") ;

		JNIGlobal::executeVoidMethod("com/sec/shp/sdk/http/client/IHttpClientResponseListener", "onErrorReceived",
									 "(Ljava/lang/Object;ILjava/lang/String;)V",pUserData->listener,
									 pUserData->requestData, error, errMsg );

		JNIGlobal::releaseLocalRef(errMsg);
	}
}

/**
 *
 * listener method to notify a specific client session is completed and application can free the resources related to
 * this session (including session)
 *
 * @param[in] client_sessioin	client session object
 *
 * @since   1.0.0*/
void
JNIHTTPClient::onClientSessionCompleted( Sec::Shp::Connector::Client::ClientSession& client_sessioin)
{
	struct jni_http_client_req_data *pUserData = (struct jni_http_client_req_data *) (client_sessioin.getSessionData().m_pUserData) ;
	if ( NULL != pUserData )
	{
		client_sessioin.getSessionData().m_pUserData = NULL ;

		JNIGlobal::releaseGlobleRef(pUserData->listener) ;
		JNIGlobal::releaseGlobleRef(pUserData->requestData) ;

		free (pUserData) ;
	}
}



}  // namespace Jni
}  // namespace Shp
}  // namespace Sec

Sec::Shp::SHPError sendHTTPRequest(Sec::Shp::Jni::JNIHTTPClient* jniHttpClient, std::string &uri, std::string &method, std::map< std::string, std::string >& requiredHeaders,
				std::string &requestData, jint lengthOfArray, struct jni_http_client_req_data *pUserData)
{
	SHP_LOG_INFO_A(SHP_JNI, "%s", "Start");
	Sec::Shp::SHPError error ;

	std::string host = "", port = "", path = "", hostheader = "", protocol = "";
	if (false == parseUrl(uri, protocol, host, port, path, hostheader)) {
		SHP_LOG_ERROR_A(SHP_JNI, "%s", "failed to parse uri");
		return error.setErrorCode(Sec::Shp::SHP_INVALID_INPUT).setErrorMessage("Invalid Protocol") ;
	}

	SHP_LOG_DEBUG_A(SHP_JNI, "Protocol [%s] DA [%s], Path [%s], Method [%s]", protocol.c_str(), hostheader.c_str(), path.c_str(), method.c_str());

	if (protocol.empty() == true)
	{
		SHP_LOG_ERROR_A(SHP_JNI, "%s", "Invalid Protocol");
		return error.setErrorCode(Sec::Shp::SHP_INVALID_INPUT).setErrorMessage("Invalid Protocol") ;
	}

	if ((path.empty() == true) || (method.empty() == true))
	{
		SHP_LOG_ERROR_A(SHP_JNI, "%s", "Resource Path or Request Method are invalid");
		return error.setErrorCode(Sec::Shp::SHP_INVALID_INPUT).setErrorMessage("Resource Path or Request Method are invalid") ;
	}

	Sec::Shp::Core::Client::Client* pClient = jniHttpClient->pClient ;
	if (NULL == pClient)
	{
		SHP_LOG_ERROR_A(SHP_JNI, "%s", "Client with specified protocol not found");
		return error.setErrorCode(Sec::Shp::SHP_INVALID_INPUT).setErrorMessage("Client with specified protocol not found") ;
	}

	Sec::Shp::Connector::Client::ClientSession* pClientSession = pClient->createNewSession( hostheader );
	if (pClientSession == NULL)
	{
		SHP_LOG_ERROR_A(SHP_JNI, "%s", "Failed to create client session");
		return error.setErrorCode(Sec::Shp::SHP_MEMORY_ALLOCATION_FAILED).setErrorMessage("Failed to create client session") ;
	}

	pClientSession->isDownloadingSession = true;
	pClientSession->isLastChunkReceived = false;
	pClientSession->getSessionData().m_pUserData = (void *)pUserData ;
	Sec::Shp::Connector::SHPRequest& shpRequest = pClientSession->getRequest() ;

	if (requestData.empty() != true) {
		shpRequest.m_payload = requestData;
		shpRequest.m_payloadSize = requestData.size();
		shpRequest.m_payload_pretty_print = requestData;
	}
	else
	{
		shpRequest.m_payloadSize = 0 ;
	}

	shpRequest.m_resourceIdentifier.setResourcePath( path );
	shpRequest.m_methodIdentifier.setMethod( method );

	std::map<std::string, std::string>::iterator itr = requiredHeaders.begin();
	for (; itr != requiredHeaders.end(); itr++) {
		std::string name = itr->first;
		std::string value = itr->second;

		if ( 0 == shp_strcasecmp(name.c_str(), HTTP_HEADER_CONTENT_TYPE))
		{
			shpRequest.m_payloadType = value;
		}
		else if ( 0 == shp_strcasecmp(name.c_str(), HTTP_HEADER_ACCEPT))
		{
			// Accept Header Parsing
			SHP_STR_SIZE index_start = 0, index_end = 0;
			std::string val = value ;

			while (index_end != std::string::npos)
			{
				std::string token = get_next_token(val, index_start, index_end, ',', true);
				index_start = index_end + 1 ;

				if ( token.empty() )
				{
					continue ;
				}

				SHP_STR_SIZE colonStartIndex = 0, colonEndIndex = 0;
				while ( colonEndIndex != std::string::npos )
				{
					std::string subToken = get_next_token( token, colonStartIndex, colonEndIndex, ';', true );
					colonStartIndex = colonEndIndex + 1 ;

					if ( subToken.empty() == false )
					{
						shpRequest.m_accept_header_values.push_back(subToken) ;

						// TODO: HTTP Quality Values are not handled
						break ;
					}
				}
			}
		}

		shpRequest.m_headers.setHeader(name, value);
	}

	if ( 0 != pClient->sendRequest( *pClientSession ) )
	{
		SHP_LOG_ERROR_A(SHP_JNI, "%s", "Failed to Send Request");
		delete pClientSession ;
		return error.setErrorCode(Sec::Shp::SHP_FAILURE).setErrorMessage("Failed to Send Request") ;
	}

	SHP_LOG_INFO_A(SHP_JNI, "%s", "Leave");
	return error.setErrorCode(Sec::Shp::SHP_SUCCESS).setErrorMessage("") ;
}


/*
 * Class:     com_sec_shp_sdk_http_client_HttpClient
 * Method:    createClient
 * Signature: (Z)J
 */
JNIEXPORT jlong JNICALL Java_com_sec_shp_sdk_http_client_HttpClient_createClient
  (JNIEnv *pEnv, jobject thisObj, jboolean useSSL )
{
	bool bRet = false;
	bool bUseSSL = (useSSL == JNI_TRUE);
	Sec::Shp::Connector::Client::IClientConnector *pClientConnector = NULL ;
	if (bUseSSL)
	{
		pClientConnector = Sec::Shp::Connector::Client::SHPHTTPSClientConnector::createInstance();
	}
	else
	{
		pClientConnector = Sec::Shp::Connector::Client::SHPHTTPClientConnector::createInstance();
	}
	TryReturn( pClientConnector != NULL, 0,"Failed to instantiate Client Connector" ) ;

	Sec::Shp::Jni::JNIHTTPClient *jniHttpClient = new Sec::Shp::Jni::JNIHTTPClient(pClientConnector, bUseSSL, true);
	TryReturn( jniHttpClient != NULL, 0, "Failed to instantiate JNIHTTPClient" ) ;

	//Sec::Shp::Jni::JNIHTTPClient::increaseReference(jniHttpClient);
	return (jlong) jniHttpClient ;
}

/*
 * Class:     com_sec_shp_sdk_http_client_HttpClient
 * Method:    destroyClient
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_com_sec_shp_sdk_http_client_HttpClient_destroyClient
  (JNIEnv *pEnv, jobject thisObj, jlong nativeClient )
{
	Sec::Shp::Jni::JNIHTTPClient *jniHttpClient = (Sec::Shp::Jni::JNIHTTPClient *)(nativeClient) ;
	if (jniHttpClient != NULL)
	{
		Sec::Shp::Jni::JNIHTTPClient::decreaseReference(jniHttpClient);
	}
}

/*
 * Class:     com_sec_shp_sdk_http_client_HttpClient
 * Method:    getSSLConfiguration
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_com_sec_shp_sdk_http_client_HttpClient_getSSLConfiguration
  (JNIEnv *pEnv, jobject thisObj, jlong nativeClient)
{
	Sec::Shp::Jni::JNIHTTPClient *jniHttpClient = (Sec::Shp::Jni::JNIHTTPClient *)(nativeClient) ;
	JNI_TryReturn(jniHttpClient != NULL, 0, "Invalid native instance");
	JNI_TryReturn(jniHttpClient->isValid == true, 0, "parent object is already freed.. can not use this object");

	if ((jniHttpClient != NULL) && (jniHttpClient->bUseSSL))
	{
		return (jlong)(jniHttpClient->pClientConnector->getConnectorConfiguration());
	}
	else
	{
		return 0;
	}
}

JNIEXPORT jboolean JNICALL Java_com_sec_shp_sdk_http_client_HttpClient_initialize
(JNIEnv *pEnv, jobject thisObj, jlong nativeClient)
{
	Sec::Shp::Jni::JNIHTTPClient *jniHttpClient = (Sec::Shp::Jni::JNIHTTPClient *)(nativeClient) ;
	JNI_TryReturn(jniHttpClient != NULL, 0, "Invalid native instance");
	JNI_TryReturn(jniHttpClient->isValid == true, 0, "parent object is already freed.. can not use this object");

	if ( NULL == jniHttpClient)
	{
		return JNI_FALSE ;
	}

	return (jniHttpClient->init() == true)? JNI_TRUE: JNI_FALSE;
}


/*
 * Class:     com_sec_shp_sdk_http_client_HttpClient
 * Method:    sendRequest
 * Signature: (JLjava/lang/String;Ljava/lang/String;[LSec/Shp/Connector/Headers;[BLjava/lang/Object;Lcom/sec/shp/sdk/http/client/IHttpClientResponseListener;)Z
 */
JNIEXPORT jboolean JNICALL Java_com_sec_shp_sdk_http_client_HttpClient_sendRequest
	(JNIEnv *pEnv, jobject thisObj, jlong nativeClient,
     jstring jUri , jstring jMethod, jobjectArray jHeadersMap, jbyteArray jpayload, jobject jrequestData, jobject jListener)
{
	Sec::Shp::Jni::JNIHTTPClient *jniHttpClient = (Sec::Shp::Jni::JNIHTTPClient *)(nativeClient) ;
	JNI_TryReturn(jniHttpClient != NULL, 0, "Invalid native instance");
	JNI_TryReturn(jniHttpClient->isValid == true, 0, "parent object is already freed.. can not use this object");

	if (jniHttpClient != NULL)
	{
		std::string uri;
		JNIGlobal::convertToStdString(jUri, uri);

		std::string method;
		JNIGlobal::convertToStdString(jMethod, method);

		std::string payload;
		jsize lengthOfArray = 0;
		if (NULL != jpayload)
		{
			jsize lengthOfArray = pEnv->GetArrayLength(jpayload);
			if (lengthOfArray > 0)
			{
				jbyte* bufferPtr = pEnv->GetByteArrayElements(jpayload, NULL);
				if (bufferPtr)
				{
					payload.assign((char*)bufferPtr, lengthOfArray);
					pEnv->ReleaseByteArrayElements(jpayload, bufferPtr, 0);
				}
			}
		}

		std::map< std::string, std::string > headerMap;
		if (jHeadersMap != NULL)
		{
			if (false == JNIGlobal::JNIConvertToMap( pEnv, jHeadersMap, headerMap))
			{
				return JNI_FALSE;
			}
		}

		struct jni_http_client_req_data *pUserData = (struct jni_http_client_req_data *) calloc(1, sizeof(struct jni_http_client_req_data));
		if (pUserData == NULL)
		{
			return JNI_FALSE;
		}

		pUserData->listener = JNIGlobal::getGlobleRef(jListener) ;
		pUserData->requestData = JNIGlobal::getGlobleRef(jrequestData) ;

		Sec::Shp::SHPError error = sendHTTPRequest( jniHttpClient, uri, method, headerMap, payload, lengthOfArray, pUserData);
		if (error.getErrorCode() == Sec::Shp::SHP_SUCCESS)
		{
			return JNI_TRUE;
		}
	}

	return JNI_FALSE;
}



