/*
 * JNIHttpServer.h
 *
 *  Created on: Feb 19, 2016
 *      Author: bhupathi.r
 */

#ifndef SRC_JNIHTTPSERVER_H_
#define SRC_JNIHTTPSERVER_H_

#include "Server/Server.h"
#include "IReferenced.h"

namespace Sec {
namespace Shp {
namespace Jni {

#define SERVER_STARTED 	1
#define SERVER_STOPPED 	2
#define SERVER_STARTING 3
#define SERVER_STOPPING 4
#define SERVER_ERROR 	5

class JNIHttpServer : public Sec::Shp::Connector::Server::IServerListener,
					  public IReferenced
{

public:

	Sec::Shp::Connector::Server::IServerConnector *pServerConnector ;
	Sec::Shp::Core::Server::Server *pServer;
	jobject requestListener ;
	Sec::Shp::Configuration mConfiguration; // dummy not used any where
	bool 	bUseSSL ;
	bool 	bIsInternal ;
	int 	port ;
	int		state ;
	bool 	isStartStopRequires ;
	bool	isValid ;
	Sec::Shp::Platform::Thread::Mutex mStateMutex ;
	Sec::Shp::Platform::Thread::Mutex mServerMutex ;

	JNIHttpServer( int port, Sec::Shp::Connector::Server::IServerConnector *pServerConnector,
				   jobject requestListener, bool bUseSSL, bool isInternal, bool isStartStopRequires = true) ;

	void setRequestListener(jobject requestListener);

	bool initialize() ;

	void setState(int s) ;

	int getState() ;

	void invalidateConnector();

	/**
	 * Default destructor of this class
	 * @since   1.0.0*/
	virtual ~JNIHttpServer() ;

	bool startServer() ;

	bool stopServer() ;


	/**
	 * listener method that will be invoked when server starts
	 *
	 * @param[in] serverId	id of the server
	 * @param[in] address	address of the server which is started
	 * @since   1.0.0*/
	virtual void onServerStarted( int serverId, std::string address ) ;

	/**
	 * listener method that will be invoked when server stops
	 *
	 * @param[in] serverId	id of the server
	 * @since   1.0.0*/
	virtual void onServerStopped(int serverId) ;

	/**
	 * listener method that will be invoked when server encounters an error
	 *
	 * @param[in] serverId	Represents server identifier
	 * @param[in] errorCode Represents error code
	 * @since   1.0.0*/
	virtual void onServerError(int serverId, Sec::Shp::ErrorCode errorCode) ;

	/**
	 * listener method that will be invoked when server receives a request
	 *
	 * @param[in] session	server session
	 *
	 * @return	returns @c true if request is processed successfully @n
	 * 					@c false in case of any error
	 * @since   1.0.0*/
	virtual bool handleRequest( Sec::Shp::Connector::Server::ServerSession& session ) ;

	/**
	 * listener method that will be invoked when server starts a session
	 *
	 * @param[in] serverId	Represents server identifier
	 * @param[in] session 	Represents session instance
	 * @since   1.0.0*/
	virtual void onServerSessionStarted(int serverId, Sec::Shp::Connector::Server::ServerSession& session);


	/**
	 * listener method that will be invoked when a server session completes
	 *
	 * @param[in] serverId	Represents server identifier
	 * @param[in] session 	Represents session instance
	 * @since   1.0.0*/
	virtual void onServerSessionCompleted(int serverId, Sec::Shp::Connector::Server::ServerSession& session);
};

}  // namespace Jni
}  // namespace Shp
}  // namespace Sec




#endif /* SRC_JNIHTTPSERVER_H_ */
