/**
*
* @Copyright Sentence:
* This program is subject to copyright protection in accordance with the
* applicable law. It must not, except where allowed by law, by any means or
* in any form be reproduced, distributed or lent. Moreover, no part of the
* program may be used, viewed, printed, disassembled or otherwise interfered
* with in any form, except where allowed by law, without the express written
* consent of the copyright holder.
*
* Copyright (C) 2014 Samsung Electronics Co., Ltd.
* All Rights Reserved.
*
*
* @Project/Module Name 	: SHP Framework
* @filename             : JNINetworkConnMonitor.cpp
* @brief                :
* @author               : Seetaram Rathod (r.seetaram@samsung.com)
* @date                 : 11/21/2014
* @version              :
*
*
*/

/******************** START OF SYSTEM DEFINED FILE INCLUSION **********************/

#include <jni.h>

/******************* END OF SYSTEM DEFINED FILE INCLUSION ************************/

/******************** START OF USER DEFINED FILE INCLUSION **********************/

#include "Sec_Shp_NetworkConnMonitor.h"
#include <NetworkConnMonitor.h>
#include <INetworkConnMonitorListener.h>
#include <MyNetworkConnMonitorListener.h>
#include <JNIGlobal.h>

/******************* END OF USER DEFINED FILE INCLUSION ************************/

JNIEXPORT jlong JNICALL Java_Sec_Shp_NetworkConnMonitor_constructNative
  (JNIEnv *pEnv, jobject cl)
{
  Sec::Shp::NetworkConnMonitor* networkConnMonitorObj ;

  networkConnMonitorObj = new Sec::Shp::NetworkConnMonitor() ;
  jlong native = (jlong)(networkConnMonitorObj);

  return native;
}

JNIEXPORT void JNICALL Java_Sec_Shp_NetworkConnMonitor_deleteNative
  (JNIEnv *pEnv, jobject cl, jlong instance)
{
  Sec::Shp::NetworkConnMonitor* networkConnMonitorObj = (Sec::Shp::NetworkConnMonitor*) instance ;
  if (NULL != networkConnMonitorObj)
  {
    delete networkConnMonitorObj;
    networkConnMonitorObj = NULL;
  }
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_NetworkConnMonitor_start
  (JNIEnv *pEnv, jobject cl, jlong instance, jstring hostName)
{
  Sec::Shp::NetworkConnMonitor* networkConnMonitorObj = (Sec::Shp::NetworkConnMonitor*) instance ;
  if(NULL == networkConnMonitorObj)
  {
    jclass mapClass = pEnv->FindClass("java/lang/Exception");
	(pEnv)->ThrowNew(mapClass, "NetworkConnMonitor object is NULL.") ;
    return JNI_FALSE;
  }

  std::string dhostName ;
  if(true != JNIGlobal::convertToStdString(hostName, dhostName))
  {
	  return JNI_FALSE;
  }

  return networkConnMonitorObj->start(dhostName.c_str());
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_NetworkConnMonitor_stop
  (JNIEnv *pEnv, jobject cl, jlong instance)
{
  Sec::Shp::NetworkConnMonitor* networkConnMonitorObj = (Sec::Shp::NetworkConnMonitor*) instance ;

  if(NULL == networkConnMonitorObj)
  {
    jclass mapClass = pEnv->FindClass("java/lang/Exception");
	(pEnv)->ThrowNew(mapClass, "NetworkConnMonitor object is NULL.") ;
	return JNI_FALSE;
  }

  return networkConnMonitorObj->stop();
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_NetworkConnMonitor_setNetworkConnListener
  (JNIEnv *pEnv, jobject cl, jlong instance, jobject listener)
{
  Sec::Shp::NetworkConnMonitor* networkConnMonitorObj = (Sec::Shp::NetworkConnMonitor*) instance ;
  if(NULL == networkConnMonitorObj)
  {
    jclass mapClass = pEnv->FindClass("java/lang/Exception");
	(pEnv)->ThrowNew(mapClass, "NetworkConnMonitor object is NULL.") ;
	return 0;
  }

  MyNetworkConnMonitorListener * myListener = new MyNetworkConnMonitorListener(listener);

  if ( NULL == myListener)
  {
    jclass mapClass = pEnv->FindClass("java/lang/Exception");
    (pEnv)->ThrowNew(mapClass, "MyNetworkConnMonitorListener memory allocation failed.") ;
     return 0;
  }

  networkConnMonitorObj->setNetworkConnListener(myListener);
  return (jlong) myListener;
}

JNIEXPORT void JNICALL Java_Sec_Shp_NetworkConnMonitor_removeNetworkConnListener
  (JNIEnv *pEnv, jobject cl, jlong instance, jlong listener)
{
  Sec::Shp::NetworkConnMonitor* networkConnMonitorObj = (Sec::Shp::NetworkConnMonitor*) instance ;
  if(NULL == networkConnMonitorObj)
  {
    jclass mapClass = pEnv->FindClass("java/lang/Exception");
	(pEnv)->ThrowNew(mapClass, "NetworkConnMonitor object is NULL.") ;
	return ;
  }

  MyNetworkConnMonitorListener* myListener = (MyNetworkConnMonitorListener *)listener ;
  if ( NULL == myListener)
  {
    jclass mapClass = pEnv->FindClass("java/lang/Exception");
    (pEnv)->ThrowNew(mapClass, "MyNetworkConnMonitorListener is NULL.") ;
    return;
  }

  networkConnMonitorObj->removeNetworkConnListener();

  delete myListener ;
}
