/*
 * JNINotificationResponse.cpp
 *
 *  Created on: 03-Apr-2014
 *      Author: sanjay
 */

#include "Sec_Shp_Notification_NotificationResponse.h"
#include "JNINotificationResponse.h"
#include "JNIGlobal.h"
#include "Serializable.h"

JNINotificationResponse::JNINotificationResponse( jobject jnotificationresponse,  std::string resourcePath ,std::string notificationType ,std::string uuid, Sec::Shp::Serialization::ISerializable* notificationData )
{
	this->setNotificationData(notificationData);
	this->setResourcePath(resourcePath);
	this->setNotificationType(notificationType);
	this->setUuid(uuid);
	m_notificationResponse = JNIGlobal::getGlobleRef(jnotificationresponse) ;
}

JNINotificationResponse::~JNINotificationResponse()
{
	JNIGlobal::executeVoidMethod("Sec/Shp/Notification/NotificationResponse", "onNativeDelete", "()V", m_notificationResponse);
	JNIGlobal::releaseGlobleRef(m_notificationResponse) ;
}


JNIEXPORT jlong JNICALL Java_Sec_Shp_Notification_NotificationResponse_constructNative
  (JNIEnv *env, jobject thisObj, jstring jresourcePath, jstring jnotificationType, jstring juuid, jlong jnotificationData)
{
	std::string resourcePath ;
	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(jresourcePath, resourcePath)), (jlong)0, "Resource Path conversion failed") ;

	std::string notificationType ;
	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(jnotificationType, notificationType)), (jlong)0, "Notification Type conversion failed") ;

	std::string uuid ;
	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(juuid, uuid)), (jlong)0, "UUID conversion failed") ;

	Sec::Shp::Notification::NotificationResponse* notificationresponseObj = new JNINotificationResponse(thisObj, resourcePath, notificationType,uuid,(Sec::Shp::Serialization::ISerializable*)jnotificationData ) ;
	JNI_TryReturn( NULL != notificationresponseObj, (jlong)0, "Memory Allocation failed : Out of memory")

	return (jlong)(notificationresponseObj);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Notification_NotificationResponse_deleteNative
  (JNIEnv *env, jobject thisObj, jlong notificationResponse)
{
	Sec::Shp::Notification::NotificationResponse* notification = (Sec::Shp::Notification::NotificationResponse*) notificationResponse;

	if (NULL != notification)
	{
		delete notification;
	}
}

JNIEXPORT void JNICALL Java_Sec_Shp_Notification_NotificationResponse_setResourcePath
  (JNIEnv *env, jobject thisObj, jlong notificationResponse, jstring jresPath)
{
	Sec::Shp::Notification::NotificationResponse* notification = (Sec::Shp::Notification::NotificationResponse*) notificationResponse;
	JNI_TryReturnVoid( NULL != notification, "Native device is invalid")

	std::string resPath ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(jresPath, resPath)), "device uuid conversion failed") ;

	notification->setResourcePath(resPath);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Notification_NotificationResponse_setNotificationType
(JNIEnv *env, jobject thisObj, jlong notificationResponse, jstring jnotifType)
{
	Sec::Shp::Notification::NotificationResponse* notification = (Sec::Shp::Notification::NotificationResponse*) notificationResponse;
	JNI_TryReturnVoid( NULL != notification, "Native device is invalid")

	std::string notifType ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(jnotifType, notifType)), "device uuid conversion failed") ;

	notification->setNotificationType(notifType);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Notification_NotificationResponse_setNotificationData
(JNIEnv *env, jobject thisObj, jlong notificationResponse, jlong jnotifData)
{
	Sec::Shp::Notification::NotificationResponse* notification = (Sec::Shp::Notification::NotificationResponse*) notificationResponse;
	JNI_TryReturnVoid( NULL != notification, "Native device is invalid")

	notification->setNotificationData((Sec::Shp::Serialization::ISerializable*)jnotifData);
}

JNIEXPORT void JNICALL Java_Sec_Shp_Notification_NotificationResponse_setUUID
(JNIEnv *env, jobject thisObj, jlong notificationResponse, jstring jUUid)
{
	Sec::Shp::Notification::NotificationResponse* notification = (Sec::Shp::Notification::NotificationResponse*) notificationResponse;
	JNI_TryReturnVoid( NULL != notification, "Native device is invalid")

	std::string uuid ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(jUUid, uuid)), "device uuid conversion failed") ;

	notification->setUuid(uuid);
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Notification_NotificationResponse_getResourcePath
  (JNIEnv *env, jobject thisObj, jlong notificationResponse)
{
	Sec::Shp::Notification::NotificationResponse* notification = (Sec::Shp::Notification::NotificationResponse*) notificationResponse;
	JNI_TryReturn( NULL != notification,NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(notification->getResourcePath()) ;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Notification_NotificationResponse_getUUID
  (JNIEnv *env, jobject thisObj, jlong notificationResponse)
{
	Sec::Shp::Notification::NotificationResponse* notification = (Sec::Shp::Notification::NotificationResponse*) notificationResponse;
	JNI_TryReturn( NULL != notification,NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(notification->getUuid()) ;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Notification_NotificationResponse_getNotificationType
(JNIEnv *env, jobject thisObj, jlong notificationResponse)
{
	Sec::Shp::Notification::NotificationResponse* notification = (Sec::Shp::Notification::NotificationResponse*) notificationResponse;
	JNI_TryReturn( NULL != notification,NULL, "Native device is invalid")

	return JNIGlobal::convertToJString(notification->getNotificationType()) ;
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_Notification_NotificationResponse_getNotificationData
(JNIEnv *env, jobject thisObj, jlong notificationResponse)
{
	Sec::Shp::Notification::NotificationResponse* notification = (Sec::Shp::Notification::NotificationResponse*) notificationResponse;
	JNI_TryReturn( NULL != notification,NULL, "Native device is invalid")

	Serializable* data = (Serializable *)( notification->getNotificationData() ) ;
	if ( data == NULL )
	{
		return NULL;
	}
	else
	{
		return data->ser_handler;
	}
}

