/*
 * JNIRemoteAccessConfig.cpp
 *
 *  Created on: Nov 22, 2013
 *      Author: Sravana Kumar K
 */

#include "Sec_Shp_RemoteAccessConfig.h"
#include "RemoteAccessConfig.h"
#include "JNIGlobal.h"

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setAuthorizationType
(JNIEnv *env, jobject thisObj, jlong config, jint type )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	Sec::Shp::AuthorizationType authType = (Sec::Shp::AuthorizationType)type ;
	raConfig->setAuthorizationType(authType);
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_RemoteAccessConfig_getAuthorizationType
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, 0, "Native RemoteAccessConfig is invalid")

	jobject authtype = JNIGlobal::getEnum("Sec/Shp/AuthorizationType","getAuthorizationType","(I)LSec/Shp/AuthorizationType;",(int)raConfig->getAuthorizationType()) ;
	return authtype ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setNetworkType
(JNIEnv *env, jobject thisObj, jlong config, jint type )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	raConfig->setNetworkType((Sec::Shp::NetworkType)type);
}

JNIEXPORT jint JNICALL Java_Sec_Shp_RemoteAccessConfig_getNetworkType
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, 0, "Native RemoteAccessConfig is invalid")

	return raConfig->getNetworkType();
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setEmail
(JNIEnv *env, jobject thisObj, jlong config, jstring id )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string email ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(id, email)), "Email conversion failed") ;

	raConfig->setEmail(email.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getEmail
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string email = raConfig->getEmail();
	jstring jEmail = JNIGlobal::convertToJString(email);

	return jEmail;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setPassword
(JNIEnv *env, jobject thisObj, jlong config, jstring pwd )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string password ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(pwd, password)), "Password conversion failed") ;

	raConfig->setPassword(password.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getPassword
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string password = raConfig->getPassword();
	jstring jpassword = JNIGlobal::convertToJString(password);

	return jpassword;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setCountryCode
(JNIEnv *env, jobject thisObj, jlong config, jstring countryCode)
{
    Sec::Shp::RemoteAccessConfig *raConfig = (Sec::Shp::RemoteAccessConfig*)config;
    JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

    std::string cppCountryCode;
    JNI_TryReturnVoid(true == JNIGlobal::convertToStdString(countryCode, cppCountryCode), "CountryCode conversion failed");

    raConfig->setCountryCode(cppCountryCode.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getCountryCode
(JNIEnv *env, jobject thisObj, jlong config)
{
    Sec::Shp::RemoteAccessConfig *raConfig = (Sec::Shp::RemoteAccessConfig*)config;
    JNI_TryReturn(NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

    std::string countryCode = raConfig->getCountryCode();
    jstring jCountryCode = JNIGlobal::convertToJString(countryCode);

    return jCountryCode;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setAuthAccountServerAddress
(JNIEnv *env, jobject thisObj, jlong config, jstring add )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string address ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(add, address)), "Address conversion failed") ;

	raConfig->setAuthAccountServerAddress(address.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getAuthAccountServerAddress
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string address = raConfig->getAuthAccountServerAddress();
	jstring jaddress = JNIGlobal::convertToJString(address);

	return jaddress;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setApiAccountServerAddress
(JNIEnv *env, jobject thisObj, jlong config, jstring add )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string address ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(add, address)), "Address conversion failed") ;

	raConfig->setApiAccountServerAddress(address.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getApiAccountServerAddress
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string address = raConfig->getApiAccountServerAddress();
	jstring jaddress = JNIGlobal::convertToJString(address);

	return jaddress;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setAccountServerPort
(JNIEnv *env, jobject thisObj, jlong config, jstring port )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string serPort ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(port, serPort)), "Port conversion failed") ;

	raConfig->setAccountServerPort(serPort.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getAccountServerPort
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string port = raConfig->getAccountServerPort();
	jstring jport = JNIGlobal::convertToJString(port);

	return jport;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setServiceServerAddress
(JNIEnv *env, jobject thisObj, jlong config, jstring add )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string address ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(add, address)), "Address conversion failed") ;

	raConfig->setServiceServerAddress(address.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getServiceServerAddress
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string address = raConfig->getServiceServerAddress();
	jstring jaddress = JNIGlobal::convertToJString(address);

	return jaddress;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setServiceServerPort
(JNIEnv *env, jobject thisObj, jlong config, jstring port )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string serPort ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(port, serPort)), "Port conversion failed") ;

	raConfig->setServiceServerPort(serPort.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getServiceServerPort
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string port = raConfig->getServiceServerPort();
	jstring jport = JNIGlobal::convertToJString(port);

	return jport;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setRemoteServicePort
(JNIEnv *env, jobject thisObj, jlong config, jstring port )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string serPort ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(port, serPort)), "Port conversion failed") ;

	raConfig->setRemoteServicePort(serPort.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getRemoteServicePort
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string port = raConfig->getRemoteServicePort();
	jstring jport = JNIGlobal::convertToJString(port);

	return jport;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setInstanceId
(JNIEnv *env, jobject thisObj, jlong config, jstring id )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string instId ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(id, instId)), "Instance ID conversion failed") ;

	raConfig->setInstanceId(instId.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getInstanceId
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string id = raConfig->getInstanceId();
	jstring jid = JNIGlobal::convertToJString(id);

	return jid;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setAppId
(JNIEnv *env, jobject thisObj, jlong config, jstring id )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string appId ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(id, appId)), "Application ID conversion failed") ;

	raConfig->setAppId(appId.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getAppId
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string id = raConfig->getAppId();
	jstring jid = JNIGlobal::convertToJString(id);

	return jid;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setAppSecret
(JNIEnv *env, jobject thisObj, jlong config, jstring id )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string secret ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(id, secret)), "App Secret conversion failed") ;

	raConfig->setAppSecret(secret.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getAppSecret
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string secret = raConfig->getAppSecret();
	jstring jsecret = JNIGlobal::convertToJString(secret);

	return jsecret;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setAuthCode
(JNIEnv *env, jobject thisObj, jlong config, jstring code )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string authCode ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(code, authCode)), "Auth Code conversion failed") ;

	raConfig->setAuthCode(authCode.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getAuthCode
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string code = raConfig->getAuthCode();
	jstring jcode = JNIGlobal::convertToJString(code);

	return jcode;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setAccessToken
(JNIEnv *env, jobject thisObj, jlong config, jstring jtoken )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string token ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(jtoken, token)), "Auth Code conversion failed") ;

	raConfig->setAccessToken(token.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getAccessToken
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string token = raConfig->getAccessToken();
	jstring jtoken = JNIGlobal::convertToJString(token);

	return jtoken;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setRefreshToken
(JNIEnv *env, jobject thisObj, jlong config, jstring jtoken )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string token ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(jtoken, token)), "Auth Code conversion failed") ;

	raConfig->setRefreshToken(token.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getRefreshToken
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string token = raConfig->getRefreshToken();
	jstring jtoken = JNIGlobal::convertToJString(token);

	return jtoken;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setUserID
(JNIEnv *env, jobject thisObj, jlong config, jstring jid )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string id ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(jid, id)), "UserID conversion failed") ;

	raConfig->setUserID(id.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getUserID
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string id = raConfig->getUserID();
	jstring jid = JNIGlobal::convertToJString(id);

	return jid;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setRemoteConfigPath
(JNIEnv *env, jobject thisObj, jlong config, jstring path )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string remotePath ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(path, remotePath)), "RemoteFilePath conversion failed") ;

	raConfig->setRemoteConfigPath(remotePath.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getRemoteConfigPath
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string path = raConfig->getRemoteConfigPath();
	jstring jpath = JNIGlobal::convertToJString(path);

	return jpath;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setSCSLogFilePath
(JNIEnv *env, jobject thisObj, jlong config, jstring path )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string scsPath ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(path, scsPath)), "SCSFilePath conversion failed") ;

	raConfig->setSCSLogFilePath(scsPath.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getSCSLogFilePath
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string path = raConfig->getSCSLogFilePath();
	jstring jpath = JNIGlobal::convertToJString(path);

	return jpath;
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setSCSLogLevel
(JNIEnv *env, jobject thisObj, jlong config, jstring jlevel )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string loglevel ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(jlevel, loglevel)), "SCS Log Level conversion failed") ;

	raConfig->setSCSLogLevel(loglevel.c_str());
}

JNIEXPORT jint JNICALL Java_Sec_Shp_RemoteAccessConfig_getSCSLogLevel
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, 0, "Native RemoteAccessConfig is invalid")

	return raConfig->getSCSLogLevel();
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setUserAuthTokenMethod
  (JNIEnv *env, jobject thisObj, jlong config, jboolean enable)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	raConfig->setUserAuthTokenMethod(enable == JNI_TRUE);
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setUseServerURLsFromController
  (JNIEnv *env, jobject thisObj, jlong config, jboolean enable)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	raConfig->setUseServerURLsFromController(enable == JNI_TRUE);
}

JNIEXPORT void JNICALL Java_Sec_Shp_RemoteAccessConfig_setCIServerAddress
(JNIEnv *env, jobject thisObj, jlong config, jstring add )
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturnVoid( NULL != raConfig, "Native RemoteAccessConfig is invalid")

	std::string address ;
	JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(add, address)), "Address conversion failed") ;

	raConfig->setCIServerAddress(address.c_str());
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_RemoteAccessConfig_getCIServerAddress
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	std::string address = raConfig->getCIServerAddress();
	jstring jaddress = JNIGlobal::convertToJString(address);

	return jaddress;
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_RemoteAccessConfig_getCISSLConfiguration
  (JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::RemoteAccessConfig* raConfig = (Sec::Shp::RemoteAccessConfig*) config ;
	JNI_TryReturn( NULL != raConfig, NULL, "Native RemoteAccessConfig is invalid")

	return (jlong)(raConfig->getCISSLConfiguration()) ;
}


