/*
 * JNIResource.cpp
 *
 *  Created on: Jul 3, 2013
 *      Author: kirtigupta
 */

#include <jni.h>
#include "Client/Resource/Resource.h"
#include "Sec_Shp_Client_Resource_Resource.h"
#include "MyResourceResponseListener.h"
#include "IContext.h"

JNIEXPORT jlong JNICALL Java_Sec_Shp_Client_Resource_Resource_constructNativeClass
  (JNIEnv *env, jobject thisObj, jlong context, jstring protocol, jstring deviceAddress, jstring resourcePath)
{
	const char* devAdd = env->GetStringUTFChars(deviceAddress,0);
	const char* resPath = env->GetStringUTFChars(resourcePath,0);
	const char* cproto = env->GetStringUTFChars(protocol,0);

	Sec::Shp::IContext* pContext =  (Sec::Shp::IContext*)context ;

	std::string dAdd(devAdd);

	std::string rPath(resPath);

	std::string proto(cproto);

	Sec::Shp::Client::Resource::Resource* resourceObj = NULL ;
	
	if ( rPath.empty() )
	{	
		resourceObj = new Sec::Shp::Client::Resource::Resource(pContext, proto, dAdd ) ;
	}
	else
	{
		resourceObj = new Sec::Shp::Client::Resource::Resource(pContext, proto, dAdd, rPath ) ;
	}

	resourceObj->forceApplicationSerializables = true ;

	env->ReleaseStringUTFChars(deviceAddress,devAdd);

	env->ReleaseStringUTFChars(resourcePath,resPath);

	env->ReleaseStringUTFChars(resourcePath, cproto);

	jlong nativeResource = (jlong)(resourceObj);

	return nativeResource;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Client_Resource_Resource_deleteNativeResource
  (JNIEnv *env, jobject thisObj, jlong resource)
{
	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;
	delete resourceObj;

}
JNIEXPORT jint JNICALL Java_Sec_Shp_Client_Resource_Resource_get
  (JNIEnv *env, jobject thisObj, jlong resource, jobjectArray map, jobject listener)
{
	int requestId;
	Sec::Shp::SHPError error ;

	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;
	if ( NULL == resourceObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource object is NULL") ;
		return -1 ;
	}

	MyResourceResponseListener* responseListener = new MyResourceResponseListener( listener );
	if ( NULL == responseListener)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource Listener object is NULL") ;
		return -1 ;
	}

	std::map< std::string, std::string > queryMap ;
	JNIGlobal::JNIConvertToMap( env, map, queryMap );
	if (queryMap.size() > 0)
	{
		error = resourceObj->get(requestId, &queryMap, *responseListener);
	}
	else
	{
		error = resourceObj->get(requestId, NULL, *responseListener);
	}
	if (error.getErrorCode() != Sec::Shp::SHP_SUCCESS )
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, error.getErrorMessage().c_str()) ;
		return -1 ;
	}
	return requestId;
}

JNIEXPORT jint JNICALL Java_Sec_Shp_Client_Resource_Resource_put
  (JNIEnv *env,jobject thisObj,jlong resource, jobjectArray map,jlong serializable, jobject listener)

{
	int requestId;
	Sec::Shp::SHPError error ;

	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;

	if ( NULL == resourceObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource object is NULL") ;
		return -1 ;
	}

	MyResourceResponseListener* responseListener = new MyResourceResponseListener( listener );
	if ( NULL == responseListener)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource Listener object is NULL") ;
		return -1 ;
	}

	std::map< std::string, std::string > queryMap ;
	JNIGlobal::JNIConvertToMap( env, map, queryMap );
	if (queryMap.size() > 0)
	{
		error = resourceObj->put(requestId, &queryMap, (Sec::Shp::Serialization::ISerializable*)serializable, *responseListener) ;
	}
	else
	{
		error = resourceObj->put(requestId, NULL, (Sec::Shp::Serialization::ISerializable*)serializable, *responseListener) ;
	}
	if (error.getErrorCode() != Sec::Shp::SHP_SUCCESS )
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, error.getErrorMessage().c_str()) ;
		return -1 ;
	}

	return requestId ;
}

JNIEXPORT jint JNICALL Java_Sec_Shp_Client_Resource_Resource_post
  (JNIEnv *env,jobject thisObj, jlong resource, jobjectArray map,jlong serializable, jobject listener)
{
	int requestId;
	Sec::Shp::SHPError error ;

	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;
	if ( NULL == resourceObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource object is NULL") ;
		return -1 ;
	}

	MyResourceResponseListener* responseListener = new MyResourceResponseListener( listener );
	if ( NULL == responseListener)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource Listener object is NULL") ;
		return -1 ;
	}

	std::map< std::string, std::string > queryMap ;
	JNIGlobal::JNIConvertToMap( env, map, queryMap );
	if (queryMap.size() > 0)
	{
		error = resourceObj->post(requestId, &queryMap,(Sec::Shp::Serialization::ISerializable*)serializable,*responseListener);
	}
	else
	{
		error = resourceObj->post(requestId, NULL,(Sec::Shp::Serialization::ISerializable*)serializable,*responseListener);
	}
	if (error.getErrorCode() != Sec::Shp::SHP_SUCCESS )
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, error.getErrorMessage().c_str()) ;
		return -1 ;
	}
	return requestId ;
}

JNIEXPORT jint JNICALL Java_Sec_Shp_Client_Resource_Resource_delete
  (JNIEnv *env,  jobject thisObj,jlong resource, jobjectArray map,jobject listener)
{
	int requestId;
	Sec::Shp::SHPError error ;

	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;

	if ( NULL == resourceObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource object is NULL") ;
		return -1 ;
	}

	MyResourceResponseListener* responseListener = new MyResourceResponseListener( listener );
	if ( NULL == responseListener)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource Listener object is NULL") ;
		return -1 ;
	}

	std::map< std::string, std::string > queryMap ;
	JNIGlobal::JNIConvertToMap( env, map, queryMap );
	if (queryMap.size() > 0)
	{
		error = resourceObj->deleteResource(requestId, &queryMap, *responseListener) ;
	}
	else
	{
		error = resourceObj->deleteResource(requestId ,NULL, *responseListener) ;
	}
	if (error.getErrorCode() != Sec::Shp::SHP_SUCCESS )
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, error.getErrorMessage().c_str()) ;
		return -1 ;
	}
	return requestId ;
}

JNIEXPORT jint JNICALL Java_Sec_Shp_Client_Resource_Resource_subscribe
  (JNIEnv *env, jobject thisObj,jlong resource, jstring jDUuid, jstring jSUuid, jobject listener, jstring jdescription )
{
	int requestId;

	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;

	if ( NULL == resourceObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource object is NULL") ;
		return -1 ;
	}

	MyResourceResponseListener* responseListener = new MyResourceResponseListener( listener );

	if ( NULL == responseListener)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource Listener object is NULL") ;
		return -1 ;
	}

	std::string dUuid ;
	if( false == JNIGlobal::convertToStdString(jDUuid, dUuid) ) {
		if (responseListener != NULL) {
			delete responseListener; responseListener = NULL;
		}
		return -1;
	}

	std::string sUuid ;
	if( false == JNIGlobal::convertToStdString(jSUuid, sUuid) ) {
		if (responseListener != NULL) {
			delete responseListener; responseListener = NULL;
		}
		return -1;
	}

	std::string sDescription ;
	if( false == JNIGlobal::convertToStdString(jdescription, sDescription) ) {
		if (responseListener != NULL) {
			delete responseListener; responseListener = NULL;
		}
		return -1;
	}

	resourceObj->subscribe(requestId, dUuid,sUuid, *responseListener,sDescription) ;

	return requestId ;

}

JNIEXPORT jint JNICALL Java_Sec_Shp_Client_Resource_Resource_unSubscribe
  (JNIEnv *env, jobject thisObj, jlong resource, jstring juuid, jobject listener)
{
	int requestId;

	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;

	if ( NULL == resourceObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource object is NULL") ;
		return -1 ;
	}
	MyResourceResponseListener* responseListener = new MyResourceResponseListener( listener );

	if ( NULL == responseListener)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource Listener object is NULL") ;
		return -1 ;
	}

	std::string uuid ;
	if (false== JNIGlobal::convertToStdString(juuid, uuid)) {
		if (responseListener != NULL) {
			delete responseListener; responseListener = NULL;
		}
		return -1;
	}

	resourceObj->unSubscribe(requestId, uuid, *responseListener) ;

	return requestId ;

}

JNIEXPORT void JNICALL Java_Sec_Shp_Client_Resource_Resource_setFilter
  (JNIEnv *env, jobject thisObj, jlong resource, jstring filter)
{
	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;

	if ( NULL != resourceObj)
	{
		std::string cfilter ;
		JNI_TryReturnVoid( ( true == JNIGlobal::convertToStdString(filter, cfilter)), "string conversion failed") ;
		resourceObj->setFilter(cfilter);
	}
}

JNIEXPORT void JNICALL Java_Sec_Shp_Client_Resource_Resource_setRecursive
  (JNIEnv *env, jobject thisObj, jlong resource, jboolean recursive)
{
	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;

	if ( NULL != resourceObj)
	{
		if (recursive == JNI_TRUE)
		{
			resourceObj->setRecursive(true);
		}
		else
		{
			resourceObj->setRecursive(false);
		}
	}
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Client_Resource_Resource_setResourcePath
  (JNIEnv *env, jobject thisObj, jlong resource, jstring filter)
{
	Sec::Shp::Client::Resource::Resource* resourceObj = (Sec::Shp::Client::Resource::Resource* ) resource ;

	if ( NULL != resourceObj)
	{
		std::string cfilter ;

		if (false == JNIGlobal::convertToStdString(filter, cfilter))
		{
			return JNI_FALSE ;
		}

		resourceObj->setResourcePath(cfilter) ;
		return JNI_TRUE;
	}

	return JNI_FALSE ;
}
