/*
 * JNISHP.cpp
 *
 *  Created on: Jul 7, 2013
 *      Author: kirtigupta
 */

#include <jni.h>
#include "Sec_Shp_SHP.h"
#include "SHP.h"
#include "Device.h"
#include "MySHPListener.h"
#include "iostream"
#include "SubscriptionManager.h"
#include "SHPResourceHandlerFactory.h"
#include "TimerType.h"
#include "SHPLogType.h"

#include "SHPModes.h"

JNIEXPORT jlong JNICALL Java_Sec_Shp_SHP_constructNative
(JNIEnv *pEnv, jclass cl)
{
	Sec::Shp::SHP* shpObj ;

	shpObj = new Sec::Shp::SHP() ;

	jlong native = (jlong)(shpObj);

	return native;
}

JNIEXPORT void JNICALL Java_Sec_Shp_SHP_deleteNative
  (JNIEnv *pEnv, jclass cl, jlong instance)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) instance ;
	if (NULL != shpObj)
	{
		Sec::Shp::Notification::ISubscriptionManager* pSubMgr = shpObj->getSubscriptionManager();

		delete shpObj; shpObj = NULL;

		if (NULL != pSubMgr)
		{
			delete pSubMgr;
		}
	}
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_start
(JNIEnv *env, jobject thisObj, jlong shp, jlong device)
{
	std::cout<< "in Java_Sec_Shp_SHP_start " <<std::endl ;
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	Sec::Shp::Device* myDevice = (Sec::Shp::Device*)device;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}

	std::cout<< "calling native start " <<std::endl ;
	return shpObj->start(*myDevice);
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_stop
(JNIEnv *env, jobject thisObj, jlong shp, jboolean isInternalStop)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}

	return shpObj->stop(isInternalStop == JNI_TRUE);
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_isStarted
(JNIEnv *env, jobject thisObj, jlong shp)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}

	return shpObj->isStarted();
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_isSCSConnected
(JNIEnv *env, jobject thisObj, jlong shp)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}
#ifdef REMOTE_ACCESS_USE_CI
	return JNI_FALSE;
#endif
#ifdef REMOTE_ACCESS_USE_SCS
	return shpObj->isSCSConnected();
#endif
#else
	return JNI_FALSE;
#endif /* REMOTE_ACCESS_SUPPORT */
}


JNIEXPORT jobject JNICALL Java_Sec_Shp_SHP_getSHPState
  (JNIEnv *env, jobject thisObj, jlong shp)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return NULL;
	}

	jobject shpState =  JNIGlobal::getEnum("Sec/Shp/SHPStates","getSHPState","(I)LSec/Shp/SHPStates;",(int)shpObj->getSHPState()) ;
	return shpState;
}

/*
 * Class:     Sec_Shp_SHP
 * Method:    isRequiresAuthModeForDT
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_isRequiresAuthModeForDT
  (JNIEnv *env, jobject thisObj, jlong shp)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}

	return (shpObj->isRequiresAuthModeForDT()) ? JNI_TRUE : JNI_FALSE ;
}

/*
 * Class:     Sec_Shp_SHP
 * Method:    setAuthMode
 * Signature: (JZ)V
 */
JNIEXPORT void JNICALL Java_Sec_Shp_SHP_setAuthMode
  (JNIEnv *env, jobject thisObj, jlong shp, jboolean jFlag)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return;
	}

	shpObj->setAuthMode(jFlag == JNI_TRUE);
}

/*
 * Class:     Sec_Shp_SHP
 * Method:    setRequiresAuthModeForDT
 * Signature: (JZ)V
 */
JNIEXPORT void JNICALL Java_Sec_Shp_SHP_setRequiresAuthModeForDT
  (JNIEnv *env, jobject thisObj, jlong shp, jboolean jFlag)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return;
	}

	shpObj->setRequiresAuthModeForDT(jFlag == JNI_TRUE);
}

JNIEXPORT void JNICALL Java_Sec_Shp_SHP_setEnableRAUsingCI
  ( JNIEnv *env, jobject thisObj, jlong shp, jboolean jFlag )
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return;
	}

	#ifdef REMOTE_ACCESS_USE_CI
		shpObj->setEnableRAUsingCI(jFlag == JNI_TRUE);
	#endif
}

JNIEXPORT void JNICALL Java_Sec_Shp_SHP_setEnableLocalDiscovery
  ( JNIEnv *env, jobject thisObj, jlong shp, jboolean jFlag )
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return;
	}

	shpObj->setEnableLocalDiscovery(jFlag == JNI_TRUE);
}

JNIEXPORT void JNICALL Java_Sec_Shp_SHP_setEnableRAUsingSCS
  ( JNIEnv *env, jobject thisObj, jlong shp, jboolean jFlag )
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return;
	}

	#ifdef REMOTE_ACCESS_USE_SCS
		shpObj->setEnableRAUsingSCS(jFlag == JNI_TRUE);
	#endif
}


JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_isAuthModeEnabled
  (JNIEnv *env, jobject thisObj, jlong shp)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}

	return (shpObj->isAuthModeEnabled()) ? JNI_TRUE : JNI_FALSE ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_getSimulatorFlag
  (JNIEnv *env, jobject thisObj, jlong shp)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}

	return shpObj->getSimulatorFlag();
}

JNIEXPORT void JNICALL Java_Sec_Shp_SHP_setSimulatorFlag
  (JNIEnv *env, jobject thisObj, jlong shp, jboolean jFlag)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return;
	}

	shpObj->setSimulatorFlag(jFlag == JNI_TRUE);
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_setConfiguration
(JNIEnv *env, jobject thisObj, jlong shp, jlong config)
{

	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	Sec::Shp::Configuration* p_config = (Sec::Shp::Configuration*)config;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}

	SHPResourceHandlerFactory* factory =  (SHPResourceHandlerFactory*)p_config->getResourceHandlerFactory();
	if (NULL != factory)
	{
		factory->setContext((jlong) shpObj->getContext());
	}

	return shpObj->setConfiguration(p_config);
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_SHP_setSHPListener
(JNIEnv *env, jobject thisObj, jlong shp, jobject listener)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return jlong(0);
	}

	MySHPListener* myshpListener = new MySHPListener(listener);

	if ( NULL == myshpListener)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp listener is NULL") ;
		return (jlong)0;
	}

	shpObj->setSHPListener(*myshpListener);

	return (jlong) myshpListener ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_SHP_removeSHPListener
(JNIEnv *env, jobject thisObj, jlong shp,jlong listener)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return;
	}

	MySHPListener* myshpListener = (MySHPListener*)listener ;
	if ( NULL == myshpListener)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp listener is NULL") ;
		return;
	}

	shpObj->removeSHPListener(*myshpListener);

	delete myshpListener ;
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_SHP_getDeviceFinder
(JNIEnv *env, jobject thisObj, jlong shp)
{

	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return (jlong)0;
	}

	return (jlong)shpObj->getDeviceFinder();

}

JNIEXPORT jlong JNICALL Java_Sec_Shp_SHP_getConfiguration
(JNIEnv *env, jobject thisObj, jlong shp)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return (jlong)0;
	}

	return (jlong)(shpObj->getConfiguration());
}



JNIEXPORT jint JNICALL Java_Sec_Shp_SHP_getTimer
(JNIEnv *env, jobject thisObj, jlong shp, jint timerType)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return -1;
	}

	Sec::Shp::TimerType tType = (Sec::Shp::TimerType)timerType;

	return (jint)shpObj->getTimer(tType);
}

/*
 * Class:     Sec_Shp_SHP
 * Method:    setSHPLogLevel
 * Signature: (J)LSec/Shp/SHPLogType;
 */
JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_setSHPLogLevel
(JNIEnv *env, jclass thisClass, jint shpLogLevel)
{
	Sec::Shp::SHPLogType logType = (Sec::Shp::SHPLogType)shpLogLevel;
	return (Sec::Shp::SHP::setSHPLogLevel(logType)) ? JNI_TRUE : JNI_FALSE;
}

JNIEXPORT void JNICALL Java_Sec_Shp_SHP_setTimer
  (JNIEnv *env, jobject thisObj, jlong shp, jint timerType, jint value)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return;
	}

	Sec::Shp::TimerType tType = (Sec::Shp::TimerType)timerType;

	shpObj->setTimer(tType, value);
}



JNIEXPORT jlong JNICALL Java_Sec_Shp_SHP_getSubscriptionManager
(JNIEnv *env, jobject thisObj, jlong shp)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return (jlong)0;
	}

	return (jlong)shpObj->getSubscriptionManager();
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_validateConfiguration
(JNIEnv *env, jobject thisObj, jlong shp, jobject config )
{
	return JNI_TRUE ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_validateMyDevice
(JNIEnv *env, jobject thisObj, jlong shp, jobject device)
{
	return JNI_TRUE ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_sendCIServerNotification
(JNIEnv *env, jobject thisObj, jlong shp, jlong device)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return (jlong)JNI_FALSE;
	}

	Sec::Shp::Serialization::ISerializable *pData = (Sec::Shp::Serialization::ISerializable *)device ;
	if ( NULL == pData)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return (jlong)JNI_FALSE;
	}

	return ( (shpObj->sendCIServerNotification(pData, true) == true) ? JNI_TRUE : JNI_FALSE) ;
}

JNIEXPORT void JNICALL Java_Sec_Shp_SHP_configureClassLoader
(JNIEnv *env, jclass thisObject, jobject classloader)
{
	if (NULL == JNIGlobal::classLoader)
	{
		JNIGlobal::classLoader = JNIGlobal::getGlobleRef(classloader) ;
	}
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_SHP_getContext
  (JNIEnv *env, jobject thisObject, jlong shp)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	return (jlong)shpObj->getContext() ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_setSHPMode
  (JNIEnv *env, jobject thisObj, jlong shp, jint mode, jstring newIP, jboolean isEasySetupThroughDeviceDiscovery, jlong device)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}

    Sec::Shp::Device *peerDevice = (Sec::Shp::Device*)device;

	Sec::Shp::SHPModes shpMode = (Sec::Shp::SHPModes)mode ;
	std::string changedIP ;
	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(newIP, changedIP)), JNI_FALSE, "device address conversion failed") ;
	return shpObj->setSHPMode(shpMode, changedIP, isEasySetupThroughDeviceDiscovery == JNI_TRUE, peerDevice);
}

JNIEXPORT jobject JNICALL Java_Sec_Shp_SHP_getSHPMode
  (JNIEnv *env, jobject thisObj, jlong shp)
{
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;
	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return NULL;
	}

	jobject shpMode =  JNIGlobal::getEnum("Sec/Shp/SHPModes","getSHPMode","(I)LSec/Shp/SHPModes;",(int)shpObj->getSHPMode()) ;
	return shpMode;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_SHP_deviceUpdateToSHS
(JNIEnv *env, jobject thisObj, jlong shp, jlong device)
{
#ifdef REMOTE_ACCESS_SUPPORT
	std::cout<< "in Java_Sec_Shp_SHP_start " <<std::endl ;
	Sec::Shp::SHP* shpObj = (Sec::Shp::SHP*) shp;

	Sec::Shp::Device* myDevice = (Sec::Shp::Device*)device;

	if ( NULL == shpObj)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Shp object is NULL") ;
		return JNI_FALSE;
	}

	std::cout<< "calling native start " <<std::endl ;
	return shpObj->deviceUpdateToSHS(myDevice);
#else
	return JNI_FALSE;
#endif /* REMOTE_ACCESS_SUPPORT */
}
