/*
 * JNISHPClientConnector.cpp
 *
 *  Created on: Feb 4, 2014
 *      Author: bhupathi.r
 */

#include "JNISHPClientConnector.h"
#include "JNIGlobal.h"

JNISHPClientConnector::JNISHPClientConnector(jobject serConnector) {
	m_clientConnector = JNIGlobal::getGlobleRef(serConnector) ;
}

JNISHPClientConnector::~JNISHPClientConnector() {
	JNIGlobal::executeVoidMethod("Sec/Shp/Connector/Client/IClientConnector", "onNativeDelete", "()V", m_clientConnector);
	JNIGlobal::releaseGlobleRef(m_clientConnector) ;
}

void JNISHPClientConnector::addClientListener(Sec::Shp::Connector::Client::IClientListener& listener)
{
	jobject listenerObject = JNIGlobal::constructObject("Sec/Shp/Connector/Client/IClientListener", "(J)V", &listener);

	if (listenerObject != NULL)
	{
		JNIGlobal::executeVoidMethod("Sec/Shp/Connector/Client/IClientConnector", "addClientListener", "(LSec/Shp/Connector/Client/Client;)V", m_clientConnector, listenerObject);
		JNIGlobal::releaseLocalRef(listenerObject);
	}
}

bool
JNISHPClientConnector::init(int serverId, Sec::Shp::Configuration& configuration) {

	return JNIGlobal::executeBoolMethod("Sec/Shp/Connector/Client/IClientConnector", "init", "(I)Z", m_clientConnector, serverId) ;
}

bool
JNISHPClientConnector::deinit(){
	return JNIGlobal::executeBoolMethod("Sec/Shp/Connector/Client/IClientConnector", "deinit", "()Z", m_clientConnector) ;
}

bool
JNISHPClientConnector::start(){
	return JNIGlobal::executeBoolMethod("Sec/Shp/Connector/Client/IClientConnector", "start", "()Z", m_clientConnector) ; 
}

bool
JNISHPClientConnector::stop() {
	return JNIGlobal::executeBoolMethod("Sec/Shp/Connector/Client/IClientConnector", "stop", "()Z", m_clientConnector) ;
}

std::string JNISHPClientConnector::getProtocol()
{
	std::string retValue = "";
	jstring jprotocol 		= (jstring)JNIGlobal::executeObjectMethod("Sec/Shp/Connector/Client/IClientConnector", "getProtocol", "()Ljava/lang/String;", m_clientConnector) ;

	if (jprotocol != NULL)
	{
		if(false == JNIGlobal::convertToStdString(jprotocol, retValue)) {
			return retValue;
		}
		JNIGlobal::releaseLocalRef(jprotocol);
	}

	return retValue ;
}

int
JNISHPClientConnector::sendRequest( Sec::Shp::Connector::Client::ClientSession& client_session )
{
	int retValue = 0;
	jobject sessionObject = JNIGlobal::constructObject("Sec/Shp/Connector/Client/ClientSession", "(J)V", &client_session);

	retValue =  (int)JNIGlobal::executeIntMethod("Sec/Shp/Connector/Client/IClientConnector", "sendRequest", "(LSec/Shp/Connector/Client/ClientSession;)I", m_clientConnector, sessionObject);

	JNIGlobal::releaseLocalRef(sessionObject);
	return retValue;
}

Sec::Shp::Connector::Client::ClientSession*
JNISHPClientConnector::createNewSession(std::string& deviceAddress)
{
	Sec::Shp::Connector::Client::ClientSession* retValue = NULL ;

	jstring jaddress = JNIGlobal::convertToJString(deviceAddress);
	jobject sessionObj = JNIGlobal::executeObjectMethod("Sec/Shp/Connector/Client/IClientConnector", "createNewSession", "(Ljava/lang/String;)LSec/Shp/Connector/Client/ClientSession;", m_clientConnector, jaddress) ;

	if (sessionObj != NULL)
	{
		retValue = (Sec::Shp::Connector::Client::ClientSession*) (JNIGlobal::executeLongMethod("Sec/Shp/Connector/Client/ClientSession", "getNativeHandle", "()J", sessionObj)) ;
	}

	JNIGlobal::releaseLocalRef(jaddress);
	JNIGlobal::releaseLocalRef(sessionObj);

	return retValue ;
}

Sec::Shp::Connector::IConnectorConfiguration*
JNISHPClientConnector::getConnectorConfiguration()
{
	//	TODO: Invoke JAVA CALL
	// Currently Not implementing. AS This function wont be invoked from native

	return NULL;
}


