/*
 * SHPServerConnector.cpp
 *
 *  Created on: Sep 30, 2013
 *      Author: kirtigupta
 */

#include "JNISHPServerConnector.h"
#include "JNIGlobal.h"

JNISHPServerConnector::JNISHPServerConnector(jobject serConnector) {
	m_serverConnector = JNIGlobal::getGlobleRef(serConnector) ;
}

JNISHPServerConnector::~JNISHPServerConnector() {
	JNIGlobal::executeVoidMethod("Sec/Shp/Connector/Server/IServerConnector", "onNativeDelete", "()V", m_serverConnector);
	JNIGlobal::releaseGlobleRef(m_serverConnector) ;
}

void 
JNISHPServerConnector::addServerlistener(Sec::Shp::Connector::Server::IServerListener& listener)
{
	jobject listenerObject = JNIGlobal::constructObject("Sec/Shp/Connector/Server/IServerListener", "(J)V", &listener);

	if (listenerObject != NULL)
	{
		JNIGlobal::executeVoidMethod("Sec/Shp/Connector/Server/IServerConnector", "addServerlistener", "(LSec/Shp/Connector/Server/IServerListener;)V", m_serverConnector, listenerObject);
		JNIGlobal::releaseLocalRef(listenerObject);
	}
}

bool
JNISHPServerConnector::init(int serverId, Sec::Shp::Configuration& configuration) {

	return JNIGlobal::executeBoolMethod("Sec/Shp/Connector/Server/IServerConnector", "init", "(I)Z", m_serverConnector, serverId) ;
}

bool
JNISHPServerConnector::deinit(){
	return JNIGlobal::executeBoolMethod("Sec/Shp/Connector/Server/IServerConnector", "deinit", "()Z", m_serverConnector) ;
}

bool 
JNISHPServerConnector::start(){
	return JNIGlobal::executeBoolMethod("Sec/Shp/Connector/Server/IServerConnector", "start", "()Z", m_serverConnector) ;
}

bool 
JNISHPServerConnector::stop() {
	return JNIGlobal::executeBoolMethod("Sec/Shp/Connector/Server/IServerConnector", "stop", "()Z", m_serverConnector) ;
}

bool
JNISHPServerConnector::sendResponse(	Sec::Shp::Connector::Server::ServerSession& session )
{
	jobject jserverSesison = JNIGlobal::constructObject("Sec/Shp/Connector/Server/ServerSession","(JJ)V",(jlong)(&session), (jlong)NULL) ;
	bool bRet = false ;

	if (jserverSesison != NULL)
	{
		bRet = JNIGlobal::executeBoolMethod("Sec/Shp/Connector/Server/IServerConnector", "sendResponse", "(LSec/Shp/Connector/Server/ServerSession;)Z", m_serverConnector, jserverSesison) ;
		JNIGlobal::releaseLocalRef(jserverSesison);
	}

	return bRet ;
}

void 
JNISHPServerConnector::clearSession( Sec::Shp::Connector::Server::ServerSession& session) {
	jobject jserverSesison = JNIGlobal::constructObject("Sec/Shp/Connector/Server/ServerSession","(JJ)V",(jlong)(&session), (jlong)NULL) ;

	if (jserverSesison != NULL)
	{
		JNIGlobal::executeVoidMethod("Sec/Shp/Connector/Server/IServerConnector", "clearSession", "(LSec/Shp/Connector/Server/ServerSession;)Z", m_serverConnector, jserverSesison) ;
		JNIGlobal::releaseLocalRef(jserverSesison);
	}
}

std::string 
JNISHPServerConnector::getProtocol(){
	std::string retValue ;
	jstring jprotocol 	= (jstring) JNIGlobal::executeObjectMethod("Sec/Shp/Connector/Server/IServerConnector", "getProtocol", "()Ljava/lang/String;", m_serverConnector) ;

	if (jprotocol != NULL)
	{
		JNI_TryReturn( ( true == JNIGlobal::convertToStdString(jprotocol, retValue)), "", "string conversion failed") ;
		JNIGlobal::releaseLocalRef(jprotocol);
	}

	return retValue ;
}

Sec::Shp::Connector::IConnectorConfiguration* 
JNISHPServerConnector::getConnectorConfiguration()
{
	//TODO: Invoke JAVA CALL
	// Currently Not implementing. AS This function wont be invoked from native

	return NULL;
}
