/*
 * JNISSLConfiguration.cpp
 *
 *  Created on: Feb 4, 2014
 *      Author: bhupathi.r
 */

#include "Sec_Shp_Connector_SSLConfiguration.h"
#include "JNIGlobal.h"
#include "SSLConfiguration.h"

JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_addCACertificate
  (JNIEnv *env, jobject thisObj, jlong config, jstring CACertPath)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	std::string CAPath ;
	bool bret = JNIGlobal::convertToStdString(CACertPath, CAPath);
	if(!bret)
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (CA Certificate Path)") ;
		return;
	}

	con->addCACertificate(CAPath.c_str());
} 
JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_setCipherSuites
  (JNIEnv *env, jobject thisObj, jlong config, jstring cipherSuite)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}
	std::string cSuite ;
	bool bret = JNIGlobal::convertToStdString(cipherSuite, cSuite);
	if(!bret)
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (CA Certificate Path)") ;
		return;
	}
	con->setCipherSuites(cSuite.c_str());
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Connector_SSLConfiguration_loadFromFile
  (JNIEnv *env, jobject thisObj, jlong config, jstring sslConfigPath)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return JNI_FALSE;
	}

	std::string sslPath ;
	bool bret = JNIGlobal::convertToStdString(sslConfigPath, sslPath);
	if(!bret)
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (SSL Configuration Path)") ;
		return JNI_FALSE;
	}

	return (con->loadFromFile(sslPath.c_str()))? JNI_TRUE: JNI_FALSE;
}

JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_setEnableRejectMobileConnections
	(JNIEnv *env, jobject thisObj, jlong config, jboolean bEnable)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	con->setEnableRejectMobileConnections( bEnable == JNI_TRUE );
}

JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_setServerSessionID
  (JNIEnv *env, jobject thisObj, jlong config, jstring sessionID)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	std::string sslPath ;
	bool bret = JNIGlobal::convertToStdString(sessionID, sslPath);
	if(!bret)
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (sessionID)") ;
		return;
	}

	con->setServerSessionID(sslPath);
}


JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_enableDefaultCACertificates
  (JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	con->enableDefaultCACertificates();
}


JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_setSelfCertificate
  (JNIEnv *env, jobject thisObj, jlong config, jstring certificatePath, jstring privateKeyFilePath, jstring password )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	std::string CertPath ;
	if (false == JNIGlobal::convertToStdString(certificatePath, CertPath))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (Certificate Path)") ;
		return;
	}
	std::string PKeyPath ;
	if (false == JNIGlobal::convertToStdString(privateKeyFilePath, PKeyPath))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (Private Key File Path)") ;
		return;
	}
	std::string Pass ;
	if (false == JNIGlobal::convertToStdString(password, Pass))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (Password)") ;
		return;
	}

	con->setSelfCertificate(CertPath.c_str(), PKeyPath.c_str(), Pass.c_str());
}


JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_setSelfCertificateWithRSAPrivateKey
  (JNIEnv *env, jobject thisObj, jlong config, jstring certificatePath, jstring privateKeyFilePath, jstring password )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	std::string CertPath ;
	if (false == JNIGlobal::convertToStdString(certificatePath, CertPath))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (Certificate Path)") ;
		return;
	}
	std::string RSAPKeyPath ;
	if (false == JNIGlobal::convertToStdString(privateKeyFilePath, RSAPKeyPath))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string ( RSA Private Key File Path)") ;
		return;
	}
	std::string Pass ;
	if (false == JNIGlobal::convertToStdString(password, Pass))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (Password)") ;
		return;
	}

	con->setSelfCertificateWithRSAPrivateKey(CertPath.c_str(), RSAPKeyPath.c_str(), Pass.c_str());
}

JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_setTempDHParamFilePath
  (JNIEnv *env, jobject thisObj, jlong config, jstring tmpDHFile)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	std::string TempDHPath ;
	if (false == JNIGlobal::convertToStdString(tmpDHFile, TempDHPath))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (CA Certificate Path)") ;
		return;
	}

	con->setTempDHParamFilePath(TempDHPath.c_str());
}


JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_enablePeerVerification
  (JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	con->enablePeerVerification();
}


JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_setProxyServer
(JNIEnv *env, jobject thisObj, jlong config,jstring Address, jstring Port )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	std::string TempAddress ;
	if (false == JNIGlobal::convertToStdString(Address, TempAddress))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (Proxy Server Address)") ;
		return;
	}

	std::string TempPort ;
	if (false == JNIGlobal::convertToStdString(Port, TempPort))
	{
		JNIGlobal::throwException("Failed to convert jstring to std::string (Proxy Server Port)") ;
		return;
	}

	con->setProxyAddress(TempAddress.c_str(), TempPort.c_str());
}


JNIEXPORT jobjectArray JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getCACertificatePath
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	jobjectArray resultList = NULL ;

	std::list<std::string> certList = con->getCACertificatePaths() ;
	if (certList.size() != 0)
	{
		jsize size = certList.size();
		int index = 0 ;

		for ( std::list<std::string>::const_iterator itr = certList.begin(); itr != certList.end(); itr++, index++ )
		{
			jstring javacertString = JNIGlobal::convertToJString(*itr);

			if (NULL == resultList)
			{
				jclass javaDeviceClass = env->GetObjectClass(javacertString) ;
				resultList = env->NewObjectArray(size, javaDeviceClass, NULL) ;
			}

			env->SetObjectArrayElement(resultList, index ,javacertString) ;

			JNIGlobal::releaseLocalRef(javacertString);
		}
	}

	return resultList;
}


JNIEXPORT jstring JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getSelfCertificatePath
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	return JNIGlobal::convertToJString(con->getSelfCertificatePath()) ;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getPrivateKeyFilePath
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	return JNIGlobal::convertToJString(con->getPrivateKeyFilePath()) ;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getPrivateKeyPassword
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	return JNIGlobal::convertToJString(con->getPrivateKeyFilePassword()) ;
}

JNIEXPORT jstring JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getTemporaryDHParametersFile
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	return JNIGlobal::convertToJString(con->getTemporaryDHParametersFile()) ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getEnableDefaultCACertificates
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return JNI_FALSE;
	}

	return (con->getEnableDefaultCACertificates()) ? JNI_TRUE : JNI_FALSE ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getEnablePeerVerification
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return JNI_FALSE;
	}

	return (con->getEnablePeerVerification()) ? JNI_TRUE : JNI_FALSE ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Connector_SSLConfiguration_IsRSAformatPrivateKey
(JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return JNI_FALSE;
	}

	return (con->isRSAformatPrivateKey()) ? JNI_TRUE : JNI_FALSE ;
}


JNIEXPORT jstring JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getProxyAddress
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	return JNIGlobal::convertToJString(con->getProxyAddress()) ;
}


JNIEXPORT jstring JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getProxyPort
(JNIEnv *env, jobject thisObj, jlong config )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	return JNIGlobal::convertToJString(con->getProxyPort()) ;
}

JNIEXPORT jobjectArray JNICALL Java_Sec_Shp_Connector_SSLConfiguration_getProxyUsageList
  (JNIEnv *env, jobject thisObj, jlong config)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return NULL;
	}

	jobjectArray resultList = NULL ;

	std::list<std::string> proxyList = con->getProxyUsageList() ;
	if (proxyList.size() != 0)
	{
		jsize size = proxyList.size();
		int index = 0 ;

		for ( std::list<std::string>::const_iterator itr = proxyList.begin(); itr != proxyList.end(); itr++, index++ )
		{
			jstring javaProxyString = JNIGlobal::convertToJString(*itr);

			if (NULL == resultList)
			{
				jclass javaProxyClass = env->GetObjectClass(javaProxyString) ;
				resultList = env->NewObjectArray(size, javaProxyClass, NULL) ;
			}

			env->SetObjectArrayElement(resultList, index ,javaProxyString) ;

			JNIGlobal::releaseLocalRef(javaProxyString);
		}
	}

	return resultList;
}


JNIEXPORT void JNICALL Java_Sec_Shp_Connector_SSLConfiguration_setProxyUsageList
  (JNIEnv *env, jobject thisObj, jlong config, jobjectArray jproxyUsageList)
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return;
	}

	if (NULL == jproxyUsageList)
	{
		jclass mapClass = env->FindClass("java/lang/Exception");
		(env)->ThrowNew(mapClass, "Resource Path array is NULL") ;
		return;
	}

	std::list<std::string> proxyUsageList ;

	if (NULL != jproxyUsageList)
	{
		int count = env->GetArrayLength(jproxyUsageList);

		for (int i=0; i< count; i++)
		{
			jstring jaddress = (jstring)env->GetObjectArrayElement(jproxyUsageList, i);
			std::string addressString;

			if( true == JNIGlobal::convertToStdString(jaddress, addressString))
			{
				proxyUsageList.push_back(addressString);
			}
		}

		con->setProxyUsageList(proxyUsageList);
	}

}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Connector_SSLConfiguration_setTimeOut
(JNIEnv *env, jobject thisObj, jlong config, jint timeout )
{
	Sec::Shp::Connector::SSLConfiguration* con = (Sec::Shp::Connector::SSLConfiguration*) config;
	if ( NULL == con)
	{
		JNIGlobal::throwException("Native Configuration object is NULL") ;
		return JNI_FALSE;
	}

	return (con->setTimeout(timeout) ? JNI_TRUE :JNI_FALSE) ;
}


