/*
 * JNIStreamSocket.cpp
 *
 *  Created on: 17-Mar-2014
 *      Author: sravana.kk
 */

#include "Sec_Shp_StreamSocket.h"
#include "JNIGlobal.h"
#include "SCSStreamingSocket.h"

JNIEXPORT jlong JNICALL Java_Sec_Shp_StreamSocket_constructNative__Ljava_lang_String_2Ljava_lang_String_2
  (JNIEnv *env, jobject thisObj, jstring jid, jstring jname)
{
#ifdef REMOTE_ACCESS_SUPPORT
	std::string streamName;
	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(jname, streamName)),0, "Streaming Name conversion failed") ;

	std::string peerID;
	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(jid, peerID)),0, "PeerID conversion failed") ;

	Sec::Shp::SCSStreamingSocket* socketObj = new Sec::Shp::SCSStreamingSocket(peerID, streamName) ;
	JNI_TryReturn( NULL != socketObj, 0, "Memory Allocation failed : Out of memory")

	return (jlong)(socketObj);
#else
	return (jlong)0;
#endif /* REMOTE_ACCESS_SUPPORT */
}

JNIEXPORT jlong JNICALL Java_Sec_Shp_StreamSocket_constructNative__ILjava_lang_String_2
  (JNIEnv *env, jobject thisObj, jint jid, jstring jname)
{
#ifdef REMOTE_ACCESS_SUPPORT
	std::string streamName;
	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(jname, streamName)),0, "Streaming Name conversion failed") ;

	int connID = (int)jid;

	Sec::Shp::SCSStreamingSocket* socketObj = new Sec::Shp::SCSStreamingSocket(connID, streamName) ;
	JNI_TryReturn( NULL != socketObj, 0, "Memory Allocation failed : Out of memory")

	return (jlong)(socketObj);
#else
	return (jlong)0;
#endif /* REMOTE_ACCESS_SUPPORT */
}

JNIEXPORT void JNICALL Java_Sec_Shp_DeviceToken_deleteNative
  (JNIEnv *, jobject, jlong dev)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::SCSStreamingSocket* socketObj = (Sec::Shp::SCSStreamingSocket*) dev;

	if (NULL != socketObj) {
		delete socketObj; socketObj = NULL;
	}
#endif /* REMOTE_ACCESS_SUPPORT */
}

JNIEXPORT jint JNICALL Java_Sec_Shp_StreamSocket_connectForStreaming
  (JNIEnv *, jobject, jlong jsocket)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::SCSStreamingSocket* socketObj = (Sec::Shp::SCSStreamingSocket*) jsocket;
	JNI_TryReturn( NULL != socketObj, (jint)-1, "Native Object is invalid");

	return (jint)socketObj->connectForStreaming();
#else
	return (jint)0;
#endif /* REMOTE_ACCESS_SUPPORT */
}

JNIEXPORT void JNICALL Java_Sec_Shp_StreamSocket_closeStreaming
  (JNIEnv *, jobject, jlong jsocket)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::SCSStreamingSocket* socketObj = (Sec::Shp::SCSStreamingSocket*) jsocket;
	JNI_TryReturnVoid( NULL != socketObj, "Native Object is invalid");

	socketObj->closeStreaming();
#endif /* REMOTE_ACCESS_SUPPORT */
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_StreamSocket_isConnected
  (JNIEnv *, jobject, jlong jsocket)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::SCSStreamingSocket* socketObj = (Sec::Shp::SCSStreamingSocket*) jsocket;
	JNI_TryReturn( NULL != socketObj, (jboolean)false, "Native Object is invalid");

	return socketObj->isConnected();
#else
	return JNI_TRUE;
#endif /* REMOTE_ACCESS_SUPPORT */
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_StreamSocket_sendStream
  (JNIEnv *env, jobject, jlong jsocket, jbyteArray jbytes)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::SCSStreamingSocket* socketObj = (Sec::Shp::SCSStreamingSocket*) jsocket;
	JNI_TryReturn( NULL != socketObj, (jboolean)false, "Native Object is invalid");

	jboolean isCopy = JNI_FALSE;
	jbyte* bytesData = NULL ;

	bytesData = (env->GetByteArrayElements(jbytes, &isCopy));
	unsigned int size = env->GetArrayLength(jbytes);

	char *data = (char*)(bytesData);
	jint retValue = socketObj->sendStream(data, size);
	return retValue;
#else
	return JNI_TRUE;
#endif /* REMOTE_ACCESS_SUPPORT */
}

JNIEXPORT jint JNICALL Java_Sec_Shp_StreamSocket_receiveStream
  (JNIEnv *env, jobject, jlong jsocket, jbyteArray jbytes)
{
#ifdef REMOTE_ACCESS_SUPPORT
	Sec::Shp::SCSStreamingSocket* socketObj = (Sec::Shp::SCSStreamingSocket*) jsocket;
	JNI_TryReturn( NULL != socketObj, (jint)-1, "Native Object is invalid");

	jboolean isCopy = JNI_FALSE;
	jbyte* bytesData = NULL ;

	bytesData = (env->GetByteArrayElements(jbytes, &isCopy));
	unsigned int size = env->GetArrayLength(jbytes);

	char *data = (char*)(bytesData);
	jint retValue = socketObj->receiveStream(&data, size);

	env->ReleaseByteArrayElements(jbytes, bytesData, 0);
	return retValue;
#else
	return (jint)0;
#endif /* REMOTE_ACCESS_SUPPORT */
}
