/*
 * MyNotificationListener.cpp
 *
 *  Created on: Jul 4, 2013
 *      Author: kirtigupta
 */

#include "Serializable.h"
#include "MyNotificationListener.h"
#include "JNISHPDev.h"

using namespace std;

MyNotificationListener::MyNotificationListener(jobject myListener)
{
    listener = JNIGlobal::getGlobleRef(myListener);

}

MyNotificationListener::~MyNotificationListener()
{
    JNIGlobal::releaseGlobleRef(listener);
}

void
MyNotificationListener::onNotificationReceived(std::string& uuid,
                                               std::string& resourceURI,
                                               std::string& eventType,
                                               Sec::Shp::Serialization::ISerializable* changedResource,
                                               std::string& subscriptionURI,
                                               Sec::Shp::Serialization::Xsd::DateTimeType* eventTime)
{
    bool isAttached = false;

    JNIEnv *pEnv = JNIGlobal::getEnv(true, isAttached);

    if (pEnv == NULL)
        return;

    jobject javaSerObject = NULL;
    if (changedResource != NULL)
    {
        Serializable *jniSer = (Serializable*)(changedResource);
        javaSerObject = jniSer->ser_handler;
    }

    jstring javaEventTime = NULL;
    if (eventTime != NULL)
    {
        javaEventTime = JNIGlobal::convertToJString(eventTime->value);
    }

    jstring javaResourceURI = JNIGlobal::convertToJString(resourceURI);
    jstring javaEvent = JNIGlobal::convertToJString(eventType);
    jstring javaUuid = JNIGlobal::convertToJString(uuid);
    jstring javaSubscriptionURI = JNIGlobal::convertToJString(subscriptionURI);

    JNIGlobal::executeVoidMethod("Sec/Shp/Notification/INotificationListener", "onNotificationReceived", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;LSec/Shp/Serialization/ISerializable;Ljava/lang/String;Ljava/lang/String;)V", listener,
            javaUuid, javaResourceURI, javaEvent, javaSerObject, javaSubscriptionURI, javaEventTime);

    if (isAttached)
        JNIGlobal::detachCurrentThread();

    JNIGlobal::releaseLocalRef(javaResourceURI);
    JNIGlobal::releaseLocalRef(javaEvent);
    JNIGlobal::releaseLocalRef(javaUuid);
    JNIGlobal::releaseLocalRef(javaSubscriptionURI);
    JNIGlobal::releaseLocalRef(javaEventTime);
}

void 
MyNotificationListener::onNotificationReceived( Sec::Shp::Serialization::ISerializable* notication )
{
	jobject javaSerObject = NULL;
    if (notication != NULL)
    {
        Serializable *jniSer = (Serializable*)(notication);
        javaSerObject = jniSer->ser_handler;
    }

	JNIGlobal::executeVoidMethod("Sec/Shp/Notification/INotificationListener", "onNotificationReceived",
								 "(LSec/Shp/Serialization/ISerializable;)V",
								 listener, javaSerObject );
}

void
MyNotificationListener::onMulticastedNotifcationReceived( const Sec::Shp::Device& device, const std::string& elementType,
                                                                   const Sec::Shp::Serialization::ISerializable* notification )
{
    bool isAttached = false ;

    JNIEnv *pEnv = JNIGlobal::getEnv(true, isAttached) ;
    if (NULL == pEnv)
    {
        return ;
    }

    JNISHPDevice *jniDevice = (JNISHPDevice*) (&device) ;
    jobject javaDeviceObject = jniDevice->m_device ;

    Serializable *jniSer = (Serializable*) (&notification) ;
    jobject javaSerObject = jniSer->ser_handler ;

    const char* cAddress = elementType.c_str();
    jstring javaElementStr = pEnv->NewStringUTF(cAddress);
    if (NULL == NULL)
    {
        return ;
    }

    JNIGlobal::executeVoidMethod("Sec/Shp/Notification/INotificationListener","onMulticastedNotifcationReceived","(LSec/Shp/Device;Ljava/lang/String;LSec/Shp/Serialization/ISerializable;)V",listener, javaDeviceObject, javaElementStr, javaSerObject);
    if ( isAttached )
    {
        JNIGlobal::detachCurrentThread() ;
    }
}
