/*
 * MySHPListener.cpp
 *
 *  Created on: Jul 5, 2013
 *      Author: kirtigupta
 */

#include "MySHPListener.h"

using namespace std ;

MySHPListener::MySHPListener(jobject myListener)
{
	listener = JNIGlobal::getGlobleRef(myListener) ;
}

MySHPListener::~MySHPListener()
{
	JNIGlobal::releaseGlobleRef(listener) ;
}

void
MySHPListener::onStarted(std::string address)
{
	bool isAttached = false ;

	JNIEnv *pEnv = JNIGlobal::getEnv(true, isAttached) ;
	if (NULL == pEnv)
	{
		return;
	}

	const char* cAddress = address.c_str();
	jstring javaAddress = pEnv->NewStringUTF(cAddress);

	JNIGlobal::executeVoidMethod("Sec/Shp/ISHPListener","onStarted","(Ljava/lang/String;)V",listener, javaAddress) ;

	if ( isAttached )
	{
		JNIGlobal::detachCurrentThread() ;
	}
}

void
MySHPListener::onStopped()
{
	JNIGlobal::executeVoidMethod("Sec/Shp/ISHPListener","onStopped","()V",listener);
}

void
MySHPListener::onError(Sec::Shp::SHPError error)
{
	jobject errorObj = JNIGlobal::getEnum("Sec/Shp/SHPErrorCode", "getErrorCode", "(I)LSec/Shp/SHPErrorCode;",(int)error.getErrorCode());
	jstring errMsg 	 = JNIGlobal::convertToJString(error.getErrorMessage()) ;

	JNIGlobal::executeVoidMethod("Sec/Shp/ISHPListener","onError","(LSec/Shp/SHPErrorCode;Ljava/lang/String;)V",listener,errorObj,errMsg);

	JNIGlobal::releaseLocalRef(errorObj);
	JNIGlobal::releaseLocalRef(errMsg);
}

std::string
MySHPListener::getMyDeviceToken(void)
{
	jstring jToken = (jstring)JNIGlobal::executeObjectMethod("Sec/Shp/ISHPListener", "getMyDeviceToken", "()Ljava/lang/String;", listener) ;
	if (NULL == jToken)
	{
		return ""; // TODO: Check return value
	}

	std::string token ;
	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(jToken, token)), "", "string conversion failed") ;

	JNIGlobal::releaseLocalRef(jToken);

	return token;
}

void
MySHPListener::onDeviceTokenCancelled(std::string uuid, std::string ip)
{
	bool isAttached = false ;

	JNIEnv *pEnv = JNIGlobal::getEnv(true, isAttached) ;
	if (NULL == pEnv)
	{
		return;
	}

	jstring jUUID = pEnv->NewStringUTF(uuid.c_str());
	jstring jToken = pEnv->NewStringUTF(ip.c_str());

	JNIGlobal::executeVoidMethod("Sec/Shp/ISHPListener", "onDeviceTokenCancelled", "(Ljava/lang/String;Ljava/lang/String;)V", listener, jUUID, jToken ) ;

	if ( isAttached ) {
		JNIGlobal::detachCurrentThread() ;
	}

	JNIGlobal::releaseLocalRef(jUUID);
	JNIGlobal::releaseLocalRef(jToken);
}

void
MySHPListener::updateUUIDAndTokenMap(std::string uuid, std::string token)
{
	bool isAttached = false ;

	JNIEnv *pEnv = JNIGlobal::getEnv(true, isAttached) ;
	if (NULL == pEnv)
	{
		return;
	}

	jstring jUUID = pEnv->NewStringUTF(uuid.c_str());
	jstring jToken = pEnv->NewStringUTF(token.c_str());

	jobject devToken = JNIGlobal::constructObject("Sec/Shp/DeviceToken", "()V");
	JNIGlobal::executeVoidMethod("Sec/Shp/DeviceToken", "setUUID", "(Ljava/lang/String;)V", devToken, jUUID) ;
	JNIGlobal::executeVoidMethod("Sec/Shp/DeviceToken", "setDeviceToken", "(Ljava/lang/String;)V", devToken, jToken) ;

	JNIGlobal::executeVoidMethod("Sec/Shp/ISHPListener", "updateUUIDAndTokenMap", "(LSec/Shp/DeviceToken;)V", listener, devToken ) ;

	if ( isAttached ) {
		JNIGlobal::detachCurrentThread() ;
	}

	JNIGlobal::releaseLocalRef(devToken);
}

void
MySHPListener::tokenRequestNotification(std::string address)
{
	bool isAttached = false;

	JNIEnv *pEnv = JNIGlobal::getEnv(true, isAttached);
	if (NULL == pEnv) {
		return;
	}

	jstring jAddress = pEnv->NewStringUTF(address.c_str());

	JNIGlobal::executeVoidMethod("Sec/Shp/ISHPListener", "tokenRequestNotification", "(Ljava/lang/String;)V", listener, jAddress);

	if (isAttached) {
		JNIGlobal::detachCurrentThread();
	}
}

void
MySHPListener::easySetupNotification(int status)
{
	jobject jStatus = JNIGlobal::getEnum("Sec/Shp/EasySetupNotification", "getEasySetupNotification", "(I)LSec/Shp/EasySetupNotification;" , (int)status);

	JNIGlobal::executeVoidMethod("Sec/Shp/ISHPListener","easySetupNotification","(LSec/Shp/EasySetupNotification;)V", listener, jStatus);

	JNIGlobal::releaseLocalRef(jStatus);
}

std::string
MySHPListener::getCurrentTimeStamp()
{
	std::string refreshToken ;
	jstring jtoken = (jstring) JNIGlobal::executeObjectMethod("Sec/Shp/ISHPListener", "getCurrentTimeStamp", "()Ljava/lang/String;", listener);

	JNI_TryReturn(( true == JNIGlobal::convertToStdString(jtoken, refreshToken)), "", "Time Stamp conversion failed") ;

	JNIGlobal::releaseLocalRef(jtoken);

	return refreshToken ;
}

#ifdef REMOTE_ACCESS_SUPPORT
void
MySHPListener::scsErrorNotification(int errorCode)
{
	jint jErrorCode = (jint)errorCode;
	JNIGlobal::executeVoidMethod("Sec/Shp/ISHPListener","scsErrorNotification","(I)V", listener, jErrorCode) ;
}


void
MySHPListener::onChannelCreated(std::string channelName, int connID)
{
	jstring jChannelName = JNIGlobal::convertToJString(channelName);
	jint jConnID = (jint)connID;

	JNIGlobal::executeVoidMethod("Sec/Shp/ISHPListener","onChannelCreated","(Ljava/lang/String;I)V", listener, jChannelName, jConnID) ;

	JNIGlobal::releaseLocalRef(jChannelName);
}

std::string
MySHPListener::getAuthCode()
{
	std::string authCode ;
	jstring jCode = (jstring) JNIGlobal::executeObjectMethod("Sec/Shp/ISHPListener", "getAuthCode", "()Ljava/lang/String;", listener);

	JNI_TryReturn(( true == JNIGlobal::convertToStdString(jCode, authCode)), "", "Auth Code conversion failed") ;

	JNIGlobal::releaseLocalRef(jCode);

	return authCode ;
}

std::string
MySHPListener::getAccessToken()
{
	std::string accessToken ;
	jstring jtoken = (jstring) JNIGlobal::executeObjectMethod("Sec/Shp/ISHPListener", "getAccessToken", "()Ljava/lang/String;", listener);

	JNI_TryReturn(( true == JNIGlobal::convertToStdString(jtoken, accessToken)), "", "Access Token conversion failed") ;

	JNIGlobal::releaseLocalRef(jtoken);

	return accessToken ;
}

std::string
MySHPListener::getRefreshToken()
{
	std::string refreshToken ;
	jstring jtoken = (jstring) JNIGlobal::executeObjectMethod("Sec/Shp/ISHPListener", "getRefreshToken", "()Ljava/lang/String;", listener);

	JNI_TryReturn(( true == JNIGlobal::convertToStdString(jtoken, refreshToken)), "", "Refresh Token conversion failed") ;

	JNIGlobal::releaseLocalRef(jtoken);

	return refreshToken ;
}

std::string
MySHPListener::getUserID()
{
	std::string userID ;
	jstring jid = (jstring) JNIGlobal::executeObjectMethod("Sec/Shp/ISHPListener", "getUserID", "()Ljava/lang/String;", listener);

	JNI_TryReturn(( true == JNIGlobal::convertToStdString(jid, userID)), "", "UserID conversion failed") ;

	JNIGlobal::releaseLocalRef(jid);

	return userID ;
}

#endif /* REMOTE_ACCESS_SUPPORT */
