/*
 * SHPConfiguration.cpp
 *
 *  Created on: Aug 16, 2013
 *      Author: bhupathi.r
 */

#include "SHPConfiguration.h"
#include "JNIGlobal.h"
#include "SHPClientConnector.h"
#include "SHPServerConnector.h"
#include "SHPDeviceFactory.h"
#include "SHPResourceHandlerFactory.h"
#include "SHPSerializableDataFactory.h"

SHPConfiguration::SHPConfiguration(jobject config) {

	m_config = JNIGlobal::getGlobleRef(config) ;
}

SHPConfiguration::~SHPConfiguration()
{
	 Sec::Shp::Server::ResourceHandler::ResourceHandlerFactory* pResourceHandlerFactory = getResourceHandlerFactory() ;
	 Sec::Shp::DeviceFinder::DeviceFactory* pDeviceFactory = getDeviceFactory();
	 Sec::Shp::Serialization::SerializableDataFactory* pSerializableFactory = getSerializableDataFactory();

	 std::list<Sec::Shp::Connector::Server::IServerConnector*> serverConnectors = getServerConnectors();
	 std::list<Sec::Shp::Connector::Client::IClientConnector*> clientConnectors = getClientConnectors();

	Sec::Shp::Serialization::ISerializable *pWifiDetails = getWifiDetails();

	for ( std::list<Sec::Shp::Connector::Server::IServerConnector*>::iterator itr = serverConnectors.begin(); itr != serverConnectors.end(); itr++)
	{
		 Sec::Shp::Connector::Server::IServerConnector* pConnector = *(itr);

		if ( NULL != pConnector )
		{
			delete pConnector;
		}
	}

	for ( std::list<Sec::Shp::Connector::Client::IClientConnector*>::iterator itr = clientConnectors.begin(); itr != clientConnectors.end(); itr++)
	{
		 Sec::Shp::Connector::Client::IClientConnector* pConnector = *(itr);

		if ( NULL != pConnector )
		{
			delete pConnector;
		}
	}

	 if ( NULL != pResourceHandlerFactory )
	 {
		 delete pResourceHandlerFactory;
	 }

	 if ( NULL != pDeviceFactory )
	 {
		 delete pDeviceFactory;
	 }

	 if ( NULL != pSerializableFactory )
	 {
		 delete pSerializableFactory;
	 }

	 if ( NULL != pWifiDetails)
	 {
		 delete pWifiDetails;
	 }

	JNIGlobal::executeVoidMethod("Sec/Shp/Configuration", "onNativeDelete", "()V", m_config);
	JNIGlobal::releaseGlobleRef(m_config) ;
}
