/*
 * Serializable.cpp
 *
 *  Created on: Jul 4, 2013
 *      Author: kirtigupta
 */ 
 
#include "Serializable.h"
#include "SHPSerialization.h"
#include "Sec_Shp_Serialization_ISerializable.h"

#include "Serialization/Json/JsonDeserializer.h"
#include "SHPDataTypes.h"
#include "Sec_Shp_Serialization_SHPJsonValidator.h"
#include "Thread/Thread.h"

using namespace std ;

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Serialization_SHPJsonValidator_validatePayload
  (JNIEnv *pEnv, jclass classObj, jlong context, jstring jsonData, jlong serializable, jstring jmethod, jstring jFilter, jboolean jRecursive)
{
	Serializable *pObj = (Serializable *) serializable;
	if (NULL == pObj)
	{
		JNIGlobal::throwException("Invalid Serializable data");
		return JNI_FALSE;
	}

	std::string payload ;
	if (false == JNIGlobal::convertToStdString(jsonData, payload))
	{
		JNIGlobal::throwException("Failed to convert std string");
		return JNI_FALSE;
	}

	std::string filter ;
	if (false == JNIGlobal::convertToStdString(jFilter, filter))
	{
		JNIGlobal::throwException("Failed to convert std string");
		return JNI_FALSE;
	}

	std::string method ;
	if (false == JNIGlobal::convertToStdString(jmethod, method))
	{
		JNIGlobal::throwException("Failed to convert std string");
		return JNI_FALSE;
	}

	Sec::Shp::Serialization::IDeserializer *p_deserializer = NULL ;
	p_deserializer = new Sec::Shp::Core::Serialization::Json::JsonDeserializer((Sec::Shp::IContext*)context);
	if (p_deserializer == NULL)
	{
		return JNI_FALSE;
	}

	if (filter.size() > 0)
	{
		p_deserializer->setFilter(filter);
	}

	if (JNI_FALSE == jRecursive)
	{
		p_deserializer->setRecursive(false);
	}
	else
	{
		p_deserializer->setRecursive(true);
	}

	if (0==shp_strcasecmp(method.c_str(), "PUT"))
	{
		p_deserializer->setValidateMandatory(false) ;
	}
	else if (0==shp_strcasecmp(method.c_str(), "POST"))
	{
		p_deserializer->setIDExcemption(true) ;
	}

	if (false == p_deserializer->parse(payload, payload.size()))
	{
		JNIGlobal::throwException("Failed to parse json: invalid json content") ;
		delete p_deserializer;
		return JNI_FALSE;
	}

	if (false == pObj->deSerialize( *p_deserializer ))
	{
		JNIGlobal::throwException(p_deserializer->getErrorMessage().c_str()) ;
		delete p_deserializer;
		return JNI_FALSE;
	}

	delete p_deserializer ;
	return JNI_TRUE ;
}
 
JNIEXPORT jlong JNICALL Java_Sec_Shp_Serialization_ISerializable_constructNative
 (JNIEnv *pEnv, jobject obj)
{
	Serializable *pObj = new Serializable(obj) ;

	return (jlong)(pObj);
}


JNIEXPORT void JNICALL Java_Sec_Shp_Serialization_ISerializable_deleteNative
 (JNIEnv *pEnv, jobject curent, jlong obj)
{
	Serializable *pObj = (Serializable *) obj;
	if (NULL != pObj)
	{
		delete(pObj) ;
	}
}
 
JNIEXPORT jboolean JNICALL Java_Sec_Shp_Serialization_ISerializable_serialize
  (JNIEnv *pEnv, jobject curent, jlong obj, jlong nativeHandle)
{
	//jlong nativeHandle = JNIGlobal::executeLongMethod("Sec/Shp/Serialization/ISerializer","getNativeHandle","()J", serializer);

	Sec::Shp::Serialization::ISerializer* pSerial = (Sec::Shp::Serialization::ISerializer*)(nativeHandle) ;

	Sec::Shp::Serialization::ISerializable* pSerializable = (Sec::Shp::Serialization::ISerializable*)obj ;

	if (NULL == pSerializable) {
		JNIGlobal::throwException("Java_Sec_Shp_Serialization_ISerializable_serialize pSerializable is NULL");
		return JNI_FALSE;
	}

	jboolean serializable = (jboolean)pSerializable->serialize(*pSerial) ;

	return serializable ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Serialization_ISerializable_deSerialize
  (JNIEnv *pEnv, jobject curent, jlong obj, jlong nativeHandle)
{
//	jlong nativeHandle = JNIGlobal::executeLongMethod("Sec/Shp/Serialization/IDeserializer","getNativeHandle","()J", deserializer);

	Sec::Shp::Serialization::IDeserializer* pdSerial = (Sec::Shp::Serialization::IDeserializer*)(nativeHandle) ;

	Sec::Shp::Serialization::ISerializable* pSerializable = (Sec::Shp::Serialization::ISerializable*)obj ;

	if (NULL == pSerializable) {
		JNIGlobal::throwException("Java_Sec_Shp_Serialization_ISerializable_deSerialize pSerializable is NULL");
		return JNI_FALSE;
	}

	jboolean deser = (jboolean)pSerializable->deSerialize(*pdSerial) ;

	return deser;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Serialization_ISerializable_validateContent
  (JNIEnv *pEnv, jobject thisObject, jlong nativeHandle)
{
	Serializable *pObj = ( Serializable * ) nativeHandle ;
	if (NULL == pObj)
	{
		return JNI_FALSE ;
	}

	Sec::Shp::SHPError error = pObj->validateContent();
	if (error.getErrorCode() != Sec::Shp::SHP_SUCCESS)
	{
		JNIGlobal::throwException(error.getErrorMessage().c_str()) ;
	}

	return JNI_TRUE ;
}

JNIEXPORT jboolean JNICALL Java_Sec_Shp_Serialization_ISerializable_setInstanceName
  (JNIEnv *pEnv, jobject thisObject, jlong nativeHandle, jstring elementName)
{
	Serializable *pObj = ( Serializable * ) nativeHandle ;
	if (NULL == pObj)
	{
		return JNI_FALSE ;
	}

	std::string eName ;
	if ( false == JNIGlobal::convertToStdString( elementName, eName ) )
	{
		return JNI_FALSE ;
	}

	pObj->setInstanceName(eName) ;
	return JNI_TRUE ;
}


JNIEXPORT void JNICALL Java_Sec_Shp_Serialization_ISerializable_setList
  (JNIEnv *pEnv, jobject thisObject, jlong nativeHandle)
{
	Serializable *pObj = ( Serializable * ) nativeHandle ;
	if (NULL == pObj)
	{
		return ;
	}

	pObj->setList(true);
}


Serializable::Serializable(jobject handler)
{
	ser_handler = JNIGlobal::getGlobleRef(handler) ;
	m_pSchema_instance_data = NULL ;
}

Serializable::~Serializable()
{
	JNIGlobal::executeVoidMethod("Sec/Shp/Serialization/ISerializable", "onNativeDelete", "()V", ser_handler);
	JNIGlobal::releaseGlobleRef(ser_handler) ;
}

std::string
Serializable::getElementName()
{
	std::string retValue ;
	jstring elementName = (jstring) JNIGlobal::executeObjectMethod("Sec/Shp/Serialization/ISerializable", "getElementName", "()Ljava/lang/String;", ser_handler);

	JNI_TryReturn( ( true == JNIGlobal::convertToStdString(elementName, retValue)), "", "string conversion failed") ;

	JNIGlobal::releaseLocalRef(elementName);

	return retValue ;
}

bool
Serializable::serializeContents(Sec::Shp::Serialization::ISerializer&  serializer)
{
	jobject		jserializer = NULL  ;

	jserializer = JNIGlobal::constructObject("Sec/Shp/Serialization/ISerializer", "(J)V", (jlong)(&serializer)) ;

	if (NULL == jserializer)
	{
		return false ;
	}

	bool bRet = JNIGlobal::executeBoolMethod("Sec/Shp/Serialization/ISerializable","serializeContents","(LSec/Shp/Serialization/ISerializer;)Z", ser_handler,jserializer);

	JNIGlobal::releaseLocalRef(jserializer);

	return bRet ;
}

bool
Serializable::deSerializeContents(Sec::Shp::Serialization::IDeserializer&  deserializer)
{
	jobject		jdeserializer = NULL  ;

	jdeserializer = JNIGlobal::constructObject("Sec/Shp/Serialization/IDeserializer","(J)V",(jlong)(&deserializer)) ;

	if (NULL == jdeserializer)
	{
		return false ;
	}

	bool bRet = JNIGlobal::executeBoolMethod("Sec/Shp/Serialization/ISerializable","deSerializeContents","(LSec/Shp/Serialization/IDeserializer;)Z", ser_handler, jdeserializer);

	JNIGlobal::releaseLocalRef(jdeserializer);

	return bRet ;
}

