/*******************************************************************************
 * @author Ajay Singh (ajay1.s@samsung.com)
 * @file This file implements Air Purifier controller function  
 * Copyright 2015 by Samsung Electronics, Inc.,
 * @license This software is the confidential and proprietary information of Samsung
 * Electronics, Inc. ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Samsung.
 ******************************************************************************/
//(function () {
    
    //'use strict';
    
    var Debug_tag = 'AP : ';
    var app = angular.module("SmartThings", ['toaster', 'angularRangeSlider', 'ngSanitize', 'ngTouch', 'ui.router']);
    app.controller("airPurifierAppController", ["$scope", "SHPService", "ScheduleService", "toaster", "$timeout", "WeatherService", "AIRCON_SETTING_REGION", "AIRCON_SETTING_REGION_CHINA","$sce", "$interval", "homeMultiService", "$state", "STCONST", "UtilityService", airPurifierAppController]);
    
    app.config(['$stateProvider', function ($stateProvider) {
         var AboutPage = {
            name: 'AboutPage',
            url: '/AboutPage/:id',
            templateUrl: 'AP/views/about_page.html'
        };
         var AddSchedulePage = {
            name: 'AddSchedulePage',
            url: '/AddSchedulePage/:id',
            templateUrl: 'common/views/add_schedule.html'
        };
         var ScheduleListPage = {
            name: 'ScheduleListPage',
            url: '/ScheduleListPage/:id',
            templateUrl: 'AP/views/schedule_list_page.html'
        };
         var SettingPage = {
            name: 'SettingPage',
            url: '/SettingPage/:id',
            templateUrl: 'AP/views/setting_page.html'
        };
         var OptionPage = {
            name: 'OptionPage',
            url: '/OptionPage/:id',
            templateUrl: 'AP/views/option_page.html'
        };
         var ErrorPage = {
            name: 'ErrorPage',
            url: '/ErrorPage/:id',
            templateUrl: 'AP/views/error_page.html'
        };
         var HelpPage = {
            name: 'HelpPage',
            url: '/HelpPage/:id',
            templateUrl: 'AP/views/help_page.html'
        };
         var RecommendationPage = {
            name: 'RecommendationPage',
            url: '/RecommendationPage/:id',
            templateUrl: 'AP/views/recommendation.html'
        };
         var AirInfoPage = {
            name: 'AirInfoPage',
            url: '/AirInfoPage/:id',
            templateUrl: 'AP/views/air_info_page.html'
        };
         var AirFreashnessPage = {
            name: 'AirFreashnessPage',
            url: '/AirFreashnessPage/:id',
            templateUrl: 'AP/views/air_freshness_page.html'
        };
         var AirFreashnessStatusPage = {
            name: 'AirFreashnessStatusPage',
            url: '/AirFreashnessStatusPage/:id',
            templateUrl: 'AP/views/ap_air_freshness_status.html'
        };
         var FilterPage = {
            name: 'FilterPage',
            url: '/FilterPage/:id',
            templateUrl: 'AP/views/filter_page.html'
        };
         var StoreNCarePage = {
            name: 'StoreNCarePage',
            url: '/StoreNCarePage/:id',
            templateUrl: 'AP/views/supplyStore_smartcare_page.html'
        };
         var HomePage = {
            name: 'HomePage',
            url: '/HomePage/:id',
            templateUrl: 'AP/views/home_page.html'
        };
        var TrendInfoPage = {
            name: 'TrendInfoPage',
            url: '/TrendInfoPage/:id',
            templateUrl: 'AP/views/trend_info_page.html'
        };
        var VentilationRecommendPage = {
            name: 'VentilationRecommendPage',
            url: '/VentilationRecommendPage/:id',
            templateUrl: 'AP/views/ventilation_recommend_page.html'
        };
        var OutdoorPurityWarningPage = {
            name: 'OutdoorPurityWarningPage',
            url: '/OutdoorPurityWarningPage/:id',
            templateUrl: 'AP/views/outdoor_purity_warning_page.html'
        };
        var YellowDustWarningPage = {
            name: 'YellowDustWarningPage',
            url: '/YellowDustWarningPage/:id',
            templateUrl: 'AP/views/yellow_dust_warning_page.html'
        };
        
        $stateProvider.state(AboutPage);
        $stateProvider.state(AddSchedulePage);
        $stateProvider.state(ScheduleListPage);
        $stateProvider.state(SettingPage);
        $stateProvider.state(OptionPage);
        $stateProvider.state(ErrorPage);
        $stateProvider.state(HelpPage);
        $stateProvider.state(RecommendationPage);
        $stateProvider.state(AirInfoPage);
        $stateProvider.state(AirFreashnessPage);
        $stateProvider.state(AirFreashnessStatusPage); 
        $stateProvider.state(FilterPage);
        $stateProvider.state(StoreNCarePage);
        $stateProvider.state(HomePage);
        $stateProvider.state(TrendInfoPage);
        $stateProvider.state(VentilationRecommendPage);
        $stateProvider.state(OutdoorPurityWarningPage);
        $stateProvider.state(YellowDustWarningPage);
    }]);

    app.filter("contextMenuFilter", function(){
        return function(items, selectedCountry, scope){
            var resultArr = [];
            angular.forEach(items, function(item){
                if(item.id === 1){
                    if(scope.isLCDModel || (selectedCountry === "KR" || selectedCountry === "CN")) {
                        resultArr.push(item);
                    }
                }
                else{
                    resultArr.push(item);
                }
            });
            return resultArr;
        }
    });
    /**		 
    * @private
    * @function
    * @name airPurifierAppController
    * 
    * @description This method is a constructor function for Air Purifier module and is used to define 
    * properties and methods to module's scope (angular scope).
    */ 
    function airPurifierAppController($scope, SHPService, ScheduleService, toaster, $timeout, WeatherService, AIRCON_SETTING_REGION, AIRCON_SETTING_REGION_CHINA,$sce, $interval, homeMultiService, $state, STCONST, UtilityService) {
    
        var firstTimeLaunch,
        /**
        * @enum {String} APCONST
        * @readonly
        * @description  This is a global variable used to identify various constants of Air Purifier module
        * @private
        */        
        APCONST = {
            SCREEN_HOME: 1,
            SCREEN_FILTER: 2,
            SCREEN_AIRFRESHNESSINFOSTATUS: 3,
            SCREEN_WINDSTRENGTH: 4,
            SCREEN_SETTINGS: 5,
            SCREEN_SCHEDULER: 6,
            SCREEN_ADDSCHEDULE: 7,
            SCREEN_HELP: 8,
            SCREEN_RECOMMENDTINFO: 9,
            SCREEN_AIRINFO: 10,
            SCREEN_AIRFRESHNESSINFO: 11,
            SCREEN_ABOUT: 12,
            SCREEN_ERRORPAGE: 13,
            SCREEN_SUPPLY_STORE: 14,
            SCREENS_SMART_CARE: 15,
            SMARTCAREERROR: 16,
            ERRORPAGE: 17,
            SCREEN_USAGE_HELP: 18,
            SCREEN_TREND_INFO: 19,
            SCREEN_VENTILATION_RECOMMEND: 20,
            SCREEN_OUTDOOR_PURITY_WARNING: 21,
            SCREEN_YELLOW_DUST_WARNING: 22,
            TWO_VENT_MODEL: 2,
            THREE_VENT_MODEL: 3,
            VENTLEVEL_0: 0,
            VENTLEVEL_1: 1,
            VENTLEVEL_2: 2,
            VENTLEVEL_3: 3,
            VENTLEVEL_4: 4,
            VENTLEVEL_5: 5,
            VENTLEVEL_6: 6,
            VENTLEVEL_7: 7,
            WIND_SPEEDLEVEL_AUTO: 0,
            WIND_SPEEDLEVEL_SLEEP: 1,
            WIND_SPEEDLEVEL_LOW: 2,
            WIND_SPEEDLEVEL_MEDIUM: 3,
            WIND_SPEEDLEVEL_HIGH: 4,
            HUMIDIFI_AUTO: 0,
            HUMIDIFI_40: 40,
            HUMIDIFI_50: 50,
            HUMIDIFI_60: 60,
            HUMIDIFI_OFF: "Humidi_Off",
            FILTER_ALARM: 1,
            WATER_SUPPLEMENT: 2,
            WATER_REPLACING: 3,
            GET: "GET",
            PUT: "PUT",
            POST: "POST",
            DELETE: "DELETE",
            RUNONNATIVE: "RUNONNATIVE",
            SCHEDULEDATA: "SCHEDULEDATA",
            MODECHANGE: "MODECHANGE",
            POWERONOFF: "POWERONOFF",
            SETTING_COMMAND: "SETTING_COMMAND",
            WIND_STRENGTH_COMMAND: "WIND_STRENGTH_COMMAND",
            HUMIDITY_COMMAND: "HUMIDITY_COMMAND",
            VENT: "VENT",
            GET_DEVICES: "getDevices",
            PARSE_MODULEINFO: "PARSE_MODULEINFO",
            VOLUMECHANGE: "VOLUMECHANGE",
            DIAGNOSIS: "diagnosis",
            FILTER_RESET: "FILTER_RESET",
            INFORMATION: "INFORMATION",
            CONFIGURATION: "CONFIGURATION",
            STATUS_204: 204,
            STATUS_201: 201,
            LIST_ANIMATION_TIME: 150, // Animation time for ripple effect in list
            POWER_ANIMTION_TIMEOUT: 130,
            BACK_ANIMTION_TIMEOUT: 130,
            TOGGLE_TIME: 330,
            BTN_ANIMATION_TIME: 480,
            OPTION_CODE_SPEED_USE_BIT: 9,
            OPTION_CODE_LIGHT_SUPPORT_WHILE_STERILIZE_CLEAN_RUN_BIT: 10,
            OPTION_CODE_SOUND_LIGHT_DISTINGUISH_BIT: 13,
            OPTION_CODE_NEW_OPTION_STRUCTURE_USE_BIT: 14,    // applicable for 7, 8 bit,
            AIRPURIFIER_DEVICE: "70",
            LCD_DEVICE: "3",
            AP_PROVIDE_OUTDOOR: "1"
        },
        diagnosisStarted,
        isIndoorpopup = false,
        promise,
        mOrginialValues = [],
        debugMode = false,
        tempFilterStatusShow,
        langArr = Object.keys(LANGUAGE_FILE.language),
        alarmList = [],
        modelId,
        apSupportOutdoor,
        deviceRegion,
        deviceNation;
        
        
        if (langArr.indexOf(LANG.toUpperCase())!== -1){
            $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase()];
        } else if(langArr.indexOf(LANG.toUpperCase().substring(0,2))!== -1){
            $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase().substring(0,2)];
        } else {
            $scope.translation = LANGUAGE_FILE["language"]["DEFAULT"];
        }
       
        $scope.selectedLanguage = LANG;
        
        /**    	 
        * @function
        * @private
    	* @name FUNCTION NAME
        * @param {String} message
        * message to be printed on console
    	* @description to print messages on console
        */
        function debugMessage(message) {
            if (debugMode) {               
                console.log(message + "called from " + arguments.callee.caller.name);
            }
        }
        
        /*
        * Scope varibles and methods
        */
        $scope.isStatic = false;
        $scope.isAppV2 = false;
        $scope.SERVER_URL = "";

        // note : don't change these two variables' initial values.
        $scope.isAirPurifier = true;
        $scope.isHumidifierAirPurifier = false;
        $scope.isLCDModel = false;

        $scope.pageLoaded = false;
        $scope.subHeaderTitle = "";
        $scope.selectedCountry = 'KR';
        $scope.bAPHeader = true;
        $scope.bPowerButtonVisible = true;
        $scope.bBackKeyVisible = false;
        $scope.bMoreOptionsVisible = true;
        $scope.bHeaderCenterComponent = true;
        $scope.bAPScheduleScreenVisible = false;
        $scope.addSchedulePage = false;
        $scope.bAPAboutPage = false;
        $scope.bAPHomePageContainer = false;
        $scope.bOptionPopUp = false;
        $scope.bUsageHelpPage = false;
        $scope.bTurnOnOffPopup = false;
        $scope.currentScreen = APCONST.SCREEN_HOME;
        $scope.ventModel = APCONST.TWO_VENT_MODEL;
        $scope.ventUpper = false;
        $scope.ventMiddle = false;
        $scope.ventLower = false;
        $scope.powerState = false;
        $scope.isFilterEnable = true;
        $scope.countrySelectionPopupVisible = false;
        $scope.checkResponse = [];
        $scope.airTypeString = "";
        $scope.selectedWeatherLocation = getLocalStorageData('selectedRegionName#'+$scope.deviceUuid);
        $scope.airInfo = {desc1: '', desc2: '', desc3: '', desc4: '', isCAI: false, gradeA: '', gradeB: '', gradeC: '', gradeD: '', isAQI : false};
        $scope.ScheduleText = $scope.translation.WEBMOB_common_none;
        $scope.selectedSchedule = {id: '', power: 'On', dayOfWeek: [], disposable: true, hours: 0, minutes: 0, amPm: 0, keypadOn: false};
        $scope.helpPagePMIconColor = '';
        $scope.DeviceOptions = {
            spi: false,
            wide: false,
            three_d: false,
            turbo: false,
            comodeOff: false,
            smartSensor: false,
            currentVentLevel: 0,
            lighting: false,
            volume: 0,
            optionCode: '',
            deviceCountry: '',
            chinaAirSteps: 6,
            filterSteps: false,
            ventControl: false,
            pm10SupportOnly: false,
            pm25SupportOnly: false,
            pm10UltraFineSupportOnly: false,
            defaultPMSensor: '',
            isUsingNewOptionValue: false,
            isInfoButtonShow: false,
            isSpeedSupported: true,
            isDistinguishingMuteLightOption: false,
            isMuteSupported: false,
            isLightSupported: false,
            isLightEnabledWhileSterilizeCleanRunOn: false,
	    
            //HAP
            isHumidiOn: false,
            isSterilizeCleanOn: false,
            isSterilizeCleanRunOn: false,
            sterilizeProgress: 0,
            isSterilizeOn: false,
            isSterilizeRunOn: false,

            inDoorData: {
                pm10: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: '',
                    iconClass: 'purity-level-off'
                },
                pm25: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: '',
                    iconClass: 'purity-level-off'
                },
                pm10ultrafine: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                odor: {
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                freshness: {
                    purity: -1,
                    purityText: '',
                    colorClass: '',
                    iconClassBig: 'purity-level-off',
                    iconClass: 'purity-level-off'
                },
                tempFreshness: {
                    purity: -1,
                    purityText: '',
                    colorClass: '',
                    iconClassBig: 'purity-level-off',
                    iconClass: 'purity-level-off'
                }
            },
            outDoorData: {
                pm10: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                pm25: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                SO2: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                O3: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                CO: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                N2O2: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                KHAI: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                AQI: {
                    value: -1,
                    purity: -1,
                    purityText: '',
                    colorClass: ''
                },
                yellowDust: '',
                yellowLevel: ''
            },
            outdoorInfoText: '',
            outdoorRecommType: ''
        };
        $scope.dialogPopup = {
            show: false,
            dialogPopupTitle: '',
            dialogPopupText: '',
            button1Title: '',
            button1Handler: function () {
                $scope.dialogPopup.show = false;
                toggleBottomPopUpClass($scope.dialogPopup.show);
            },
            button2Enabled: false,
            button2Title: '',
            button2Handler: function () {
            }
        };
        $scope.containSubCl = "containSub"; //initially the class should be present.
        $scope.ModeSelectionPopupDivCls = '';// Initially no class should be applicable.
        $scope.headModeTextCls = 'header_mode_text_single_line';
        $scope.BottomPopupDivCls = '';// Initially no class should be applicable.
        $scope.BottomPopupDimBackgroundDiv = '';// Initially no class should be applicable.
        $scope.volumePopupBakgroundDivCls = '';// Initially no class should be applicable.
        $scope.volumePopupBGCls = '';// Initially no class should be applicable.
        $scope.toggleOnOff = "switch";
        $scope.tempVolume = 0;
        $scope.outDoorDataAvailable = false; // To check if all value in outdoor data available or not
        $scope.spiSupported = true; // To check if SPI or virus doctor supported for device or not
        $scope.turboModeSupported = false; // To check if to show turbo mode aur speed in option menu based on 15th bit of optionCode

        // HAP
        $scope.isHumidiSupport = false;
        $scope.isSterilizeCleanSupprot = false;
        $scope.isSterilizeSupport = false;
        $scope.selectedModeIcon = '';
        $scope.isWaterError = false;
        
        var reqParamCountrySelection;
        $scope.countrySelectedIndex = 0;
        $scope.countryToSelect = [];
        $scope.countryToSelect.push({'Country': 'Belgium'});
        $scope.countryToSelect.push({'Country': 'Belgium(F)'});
        $scope.inAppPanelPopupIcon = '';
        
        $scope.waterAlarmIcon = '';
        $scope.waterAlarmText = '';
        $scope.waterAlarmDesc = '';
       
        
        // Array items for all popups
        $scope.listOfModes = [
            {
                id: 0, 
                imageID: $scope.SERVER_URL+"AP/assets/img/main_drop_list_ic_auto.svg#Layer_1", 
                modeName: "ACMOB_VTL_main_dropdown_auto"
            },
            //{
            //    id: 5,  // not spec, it's a magic value to identify Speed mode for Humidifier device
            //    imageID: $scope.SERVER_URL+"AP/assets/img/main_list_ic_speed.svg#Layer_1",
            //    modeName: "ACMOB_VTL_options_speed"
            //},
            {
                id: 4, 
                imageID: $scope.SERVER_URL+"AP/assets/img/main_drop_list_ic_high.svg#Layer_1", 
                modeName: "ACMOB_VTL_main_dropdown_high"
            },
            {
                id: 3, 
                imageID: $scope.SERVER_URL+"AP/assets/img/main_drop_list_ic_medium.svg#Layer_1", 
                modeName: "ACMOB_VTL_main_dropdown_medium"
            },
            {
                id: 2, 
                imageID: $scope.SERVER_URL+"AP/assets/img/main_drop_list_ic_low.svg#Layer_1", 
                modeName: "ACMOB_VTL_main_dropdown_low"
            },
            {
                id: 1, 
                imageID: $scope.SERVER_URL+"AP/assets/img/main_drop_list_ic_sleep.svg#Layer_1", 
                modeName: "ACMOB_VTL_main_dropdown_sleep"
            }
        ];
        $scope.listOfHumidifiModes = [
             {
                id: 0,
                modeName: $scope.translation.ACMOB_VTL_main_dropdown_auto,
                value: -1  // -1 means auto mode
            },
            {
                id: 1,
                modeName: "40 %",
                value: 40
            },
            {
                id: 2, 
                modeName: "50 %",
                value: 50
            },
            {
                id: 3, 
                modeName: "60 %",
                value: 60
            },
            {
                id: 4, 
                modeName: $scope.translation.WEBMOB_VTL_device_off_post,
                value: 0
            }
        ];
        $scope.scheduleOption = [
            {
                id: 0, 
                modeName: "WEBMOB_common_schedule_turn_on",
                flag : true
            },
            {
                id: 1, 
                modeName: "WEBMOB_common_schedule_turn_off",
                flag : false
            },
        ];
         $scope.optionsMenu = [{
            id: 0,
            title: $scope.translation.WEBMOB_common_settings
        }, {
            id: 1,
            title: $scope.translation.WEBMOB_common_help
        }, {
            id: 2,
            title: $scope.translation.WEBMOB_common_about_device
        }];
        $('.ripplechkbox').materialChkBoxChecked();
        
        $scope.trendsTimeValues = [];
        $scope.trendsTimeValuesHelp = ["50" + $scope.translation.WEBMOB_device_ap_min_ago, "40" + $scope.translation.WEBMOB_device_ap_min_ago, "30" + $scope.translation.WEBMOB_device_ap_min_ago, "20" + $scope.translation.WEBMOB_device_ap_min_ago, "10" + $scope.translation.WEBMOB_device_ap_min_ago];
        var trendsImageKeyValues = {1: "good", 2: "normal", 3: "poor", 4: "very_poor"};
        $scope.trendGraphValues = [];
        var trendGraphValuesHelp = [3, 1, 2, 4, 3, 3].reverse();
        $scope.trendsGraphImgData = [];
        $scope.trendsGraphImgDataHelp = ["", "", "", "", ""];
        $scope.trendsHistoryImgData = ["very_poor", "poor", "normal", "good"];
        updateGraphHelp();

        //Function Declaration
        $scope.init = init;
        $scope.updateDeviceData = updateDeviceData;
        $scope.parseDeviceData = parseDeviceData;
        $scope.parseStaticData = parseStaticData;
        $scope.outDoorPuirtyData = outDoorPuirtyData;
        $scope.closeApp = closeApp;
        $scope.openAirInfoSetting = openAirInfoSetting;
        $scope.onBackPressed = onBackPressed;
        $scope.onBottonPopupCancel = onBottonPopupCancel;
        $scope.showConnectionFailurePopup = showConnectionFailurePopup;
        $scope.closeConnectionFailurePopup = closeConnectionFailurePopup;
        $scope.requestFailHandler = requestFailHandler;
        $scope.switchOpt = switchOpt;
        $scope.initSearchText = initSearchText;
        $scope.sendSensorRequest = homeMultiService.sendSensorRequest;;
        $scope.hideVentPopUp = hideVentPopUp;
        $scope.getVentImageClass = getVentImageClass;
        $scope.selectVent = selectVent;
        $scope.getVentText = getVentText;
        $scope.openPopoverVentSelection = openPopoverVentSelection;
        $scope.openAPAirInfo = openAPAirInfo;
        $scope.selectMode = selectMode;
        $scope.selectHumidityValue = selectHumidityValue;
        $scope.isModeSelected = isModeSelected;
        $scope.openPopoverMode = openPopoverMode;
        $scope.closePopoverMode = closePopoverMode;
        $scope.selectLocation = selectLocation;
        $scope.powerOnOff = powerOnOff;
        $scope.setHumidityMode = setHumidityMode;
        $scope.setSterilizationMode = setSterilizationMode;
        $scope.setAutoCleanMode = setAutoCleanMode;
        $scope.onWindStrengthUpdate = onWindStrengthUpdate;
        $scope.openIndoorPopup = openIndoorPopup;
        $scope.openPopoverForImage = openPopoverForImage;
        $scope.filterNotificationClose = filterNotificationClose;
        $scope.handleStatusCode = handleStatusCode;
        $scope.hightlightSearchText = hightlightSearchText;
        $scope.searchValueUpdated = searchValueUpdated;
        $scope.gotoErrorPage = gotoErrorPage;
        $scope.openAPSettings = openAPSettings;
        $scope.openAPHelp = openAPHelp;
        $scope.openAPAbout = openAPAbout;
        $scope.openAPFilterScreen = openAPFilterScreen;
        $scope.openAPAirFreshnessStatus = openAPAirFreshnessStatus;
        $scope.openAPAirFreshnessLevelStatus = openAPAirFreshnessLevelStatus;
        $scope.openAPRecommendInfo = openAPRecommendInfo;
        $scope.openSupplyStoreScreen = openSupplyStoreScreen;
        $scope.onPowerOffUpdate = onPowerOffUpdate;
        $scope.scheduleList = scheduleList;
        $scope.gotoAddSchedule = gotoAddSchedule;
        $scope.addSchedule = addSchedule;
        $scope.removeItemFromSchedule = removeItemFromSchedule;
        $scope.asTabSelected = asTabSelected;
        $scope.increaseHours = increaseHours;
        $scope.decreaseHours = decreaseHours;
        $scope.onHoursLostFocus = onHoursLostFocus;
        $scope.onMinsLostFocus = onMinsLostFocus;
        $scope.onAsRepeatWeakly = onAsRepeatWeakly;
        $scope.increaseMinutes = increaseMinutes;
        $scope.decreaseMinutes = decreaseMinutes;
        $scope.changeMeridian = changeMeridian;
        $scope.saveSchedule = saveSchedule;
        $scope.keypadClicked = keypadClicked;
        $scope.daySelected = daySelected;
        $scope.dayColorSelected = dayColorSelected;
        $scope.openPopoverVolumeSelection = openPopoverVolumeSelection;
        $scope.updateVolumeSlider = updateVolumeSlider;
        $scope.updateVolumeIconUpdate = updateVolumeIconUpdate;
        $scope.onVolumeSave = onVolumeSave;
        $scope.onVolumeCancel = onVolumeCancel;
        $scope.getVolumeLevelText = getVolumeLevelText;
        $scope.getVolumeLevelNum = getVolumeLevelNum;
        $scope.getVolumeLevelTextOnLoad = getVolumeLevelTextOnLoad;
        $scope.getVolumeLevelNumOnLoad = getVolumeLevelNumOnLoad;
        $scope.popUpOkApHome = popUpOkApHome;
        //$scope.doAlarmService = doAlarmService;
        $scope.showOptionPopUp = showOptionPopUp;
        $scope.closeOptionPopUp = closeOptionPopUp;
        $scope.selectIndexFromOptionMenu = selectIndexFromOptionMenu;
        $scope.showUsageHelpPage = showUsageHelpPage;
        $scope.closeUsageHelpPage = closeUsageHelpPage;
        $scope.changeVolume = changeVolume;
        $scope.openTurnOnOffPopup = openTurnOnOffPopup;
        $scope.closeTurnOnOffPopup = closeTurnOnOffPopup;
        $scope.getOptionText = getOptionText;
        $scope.isPopupOpen = isPopupOpen;
        $scope.isGlobal = isGlobal;
        $scope.getVisiblityOfHAPIndoor = getVisiblityOfHAPIndoor;
        $scope.isValidOutdoorData = isValidOutdoorData;
        $scope.updateFilterLayerStyle = updateFilterLayerStyle;
        $scope.getFilterStatusVisibility = getFilterStatusVisibility;
        $scope.getScrollableAreaClass = getScrollableAreaClass;
        $scope.getModeSelectionPopupClass = getModeSelectionPopupClass;
        $scope.openHumidifiSettingPopup = openHumidifiSettingPopup;
        $scope.closeHumidifiSettingPopup = closeHumidifiSettingPopup;
        $scope.isSterilizationDisabled = isSterilizationDisabled;
        $scope.isHumidificationDisabled = isHumidificationDisabled;
        $scope.isSpiDisabled = isSpiDisabled;
        $scope.isWindStrengthDisabled = isWindStrengthDisabled;
        $scope.getHumidItemTextClass = getHumidItemTextClass;
        $scope.getHumidListItemClass = getHumidListItemClass;
        $scope.setProgress = setProgress;
        $scope.isInfoButtonHide = isInfoButtonHide;
        $scope.isLightMenuShow = isLightMenuShow;
        $scope.isMuteMenuShow = isMuteMenuShow;
        $scope.isSoundMenuShow = isSoundMenuShow;
        $scope.callGetDevice = callGetDevice;
        $scope.parseRequestResponse = parseRequestResponse;
        $scope.parseNotification = parseNotification;
        $scope.parseRequestAccepted = parseRequestAccepted;
        $scope.parseActionData = parseActionData;
        $scope.isOptionDisabled = isOptionDisabled;
        $scope.isLightSettingDisabled = isLightSettingDisabled;
        $scope.isAutoCleanSettingDisabled = isAutoCleanSettingDisabled;
        $scope.getLowerHumidityLable = getLowerHumidityLable;
        $scope.getUpperHumidityLable = getUpperHumidityLable;
        $scope.getLowerHumidityLableClass = getLowerHumidityLableClass;
        $scope.getUpperHumidityLableClass = getUpperHumidityLableClass;
        $scope.isUpperMark = isUpperMark;
        $scope.isLowerMark = isLowerMark;
        $scope.onCancelProgress = onCancelProgress;
        $scope.openAPTrendInfo = openAPTrendInfo;
        $scope.openAPVentilationRecommend = openAPVentilationRecommend;
        $scope.openAPOutdoorPurityWarning = openAPOutdoorPurityWarning;
        $scope.openAPYellowDustWarning = openAPYellowDustWarning;
        $scope.getHistoryDotURL = getHistoryDotURL;	
	$scope.getHistoryDotVisibility = getHistoryDotVisibility;
        $scope.getHistoryDotVisibilityHelp = getHistoryDotVisibilityHelp;
        $scope.isValidDataKR = isValidDataKR;
        $scope.isValidGradeKR = isValidGradeKR;
        $scope.getOutdoorAttributeValue = getOutdoorAttributeValue;
        $scope.isOutdoorAirInfoUnavailable = isOutdoorAirInfoUnavailable;
        $scope.isCAIUnavailable = isCAIUnavailable;
        $scope.isAirInfoUnavailable = isAirInfoUnavailable;
        $scope.isOutdoorInfoButtonDisable = isOutdoorInfoButtonDisable;
        if (window.navigator.platform === 'iPhone' || window.navigator.platform === 'iPad') {
            $scope.isAppleDevice = true;
        } else {
            $scope.isAppleDevice = false;
        }
        
        /**    	 
        * @function
        * @private
    	* @name switchOpt        
        * @description
    	*/
        function switchOpt(m) {
            if (m === "lightingSwitch" && !$scope.powerState) {
                return;
            }
            _switchOpt(m);
        }

        /**    	 
        * @function
        * @private
    	* @name toggleAnimClass        
        * @description Toggle anim class on body.
    	*/
        function toggleAnimClass() {
            if ($scope.containSubCl === "containSub") {
                $scope.containSubCl = "";
            } else {
                $scope.containSubCl = "containSub";
            }
        }

        /**    	 
        * @function
        * @private
    	* @name toggleOnOffClass        
        * @description Toggle on/off switch class.
    	*/
        function toggleOnOffClass() {
            if (($scope.toggleOnOff === "switch") || ($scope.toggleOnOff === "switch switch-mm")) {
                $scope.toggleOnOff = "switch switch-m";
            } else {
                $scope.toggleOnOff = "switch switch-mm";
            }

        }

        //function init() {
        //    setHeaderModeTextCls('modeNameSelected');
        //}
        
        /**    	 
        * @function
        * @private
    	* @name setHeaderModeTextCls      
        * @param {String} elementName element Name  
        * @description Reduce font size of header mode text if it going in 2 lines.
    	*/
        function setHeaderModeTextCls(elementName) {
            var elem = document.getElementById(elementName)/*,
             textWidth = util.getTextWidth(elem.textContent)*/,
                    availSize = 0.6 * screen.width,
                    oldWidth;

            if (!!!elem) {
                //if element not found, recheck again after some time.
                $timeout(function() {setHeaderModeTextCls(elementName);}, 100);
                return;
            }

            //preserve old width.
            oldWidth = elem.style.width;
            elem.style.width = 'auto';

            //if text width is more than 60% of screen width
            if (elem.offsetWidth > (oldWidth || availSize)) {//for text only case
                //apply change in angular scope.
                $scope.$apply(function () {
                    $scope.headModeTextCls = 'header_mode_text_multi_line';
                });
                elem.style.width = (oldWidth || availSize);
            } else {
                $scope.$apply(function () {
                    $scope.headModeTextCls = 'header_mode_text_single_line';
                });

                elem.style.width = oldWidth || 'auto'; //set old width again.
            }
        }

        /**    	 
        * @function
        * @private
    	* @name changePopUpClass        
        * @description Change popup background class based on popup open or close.
    	*/
        function changePopUpClass() {
            if ($scope.showModeSelectionPopup) {
                $scope.ModeSelectionPopupDivCls = "ModeSelectionPopupDiv";
            } else {
                $scope.ModeSelectionPopupDivCls = "ModeSelectionPopOutDiv";
            }
        }

        /**    	 
        * @function
        * @private
    	* @name toggleBottomPopUpClass        
        * @param {Boolean} popupName
        * @description Change bottom popup background class based on popup open or close.
    	 */
        function toggleBottomPopUpClass(popupName) {
            if (popupName) {
                $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivIn";
                $scope.BottomPopupDivCls = "ventBgIn";
            } else {
                $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivOut";
                $scope.BottomPopupDivCls = "ventBgOut";
            }
        }
    
        /**    	 
        * @function
        * @private
    	* @name togglePurityPopUpClass        
        * @description Change purity popup background class based on popup open or close.
    	 */
        function togglePurityPopUpClass(popupName) {
            if (popupName) {
                $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivIn";
            } else {
                $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivOut";
            }
        }
    
        /**    	 
        * @function
        * @private
    	* @name toggleVolumePopUpClass        
        * @description Change volume popup background class based on popup open or close.
    	 */
        function toggleVolumePopUpClass() {
            var OkCanceldialogPopup = document.getElementById("volumeControlPopup");
            if ($scope.showVolumePopup) {
                $scope.volumePopupBakgroundDivCls = "volumePopupDivIn";
                OkCanceldialogPopup.style.display = "block";
                $scope.volumePopupBGCls = "volumePopupBGIn";

            } else {
                $scope.volumePopupBakgroundDivCls = "volumePopupDivOut";
                $scope.volumePopupBGCls = "volumePopupBGOut";
                setTimeout(function () {
                    OkCanceldialogPopup.style.display = "none";
                }, 0);
            }
        }
           
        
        /**    	 
        * @function
        * @private
    	* @name updateRecommondationText        
        * @description Display recommondation text if indoor data and outdoor data matches the conditions.
    	 */
        function updateRecommondationText() {
            var eFreshness = $scope.DeviceOptions.outDoorData.KHAI.purity;
            var eFine = $scope.DeviceOptions.outDoorData.pm10.purity;
            var eO3 = $scope.DeviceOptions.outDoorData.O3.purity;
            var eCO = $scope.DeviceOptions.outDoorData.CO.purity;
            var eNO2 = $scope.DeviceOptions.outDoorData.N2O2.purity;
            var eSO2 = $scope.DeviceOptions.outDoorData.SO2.purity;
            var eIndoor = $scope.DeviceOptions.inDoorData.freshness.purity;
            var eYellowLevel = $scope.DeviceOptions.outDoorData.yellowLevel;

            if ($scope.isLCDModel) {
                // Type 1 - Yellow dust warning
                // Type 2 - Outdoor purity warning
                // Type 3 - Ventilation recommend warning
                $scope.DeviceOptions.outdoorRecommType = "";
                if ($scope.selectedCountry === "KR" && $scope.DeviceOptions.deviceCountry === 'KR') {
                    if ($scope.DeviceOptions.outDoorData.yellowDust) {
                        $scope.DeviceOptions.outdoorRecommType = "1";
                        if (eYellowLevel == 0) {
                            $scope.recommText1 = $scope.translation.WEBMOB_device_ap_yellowdust_advisory;
                        } else if (eYellowLevel == 1) {
                            $scope.recommText1 = $scope.translation.WEBMOB_device_ap_yellowdust_warning;
                        }
                        $scope.recommText2 = $scope.translation.WEBMOB_device_ap_by_kma;
                    } else if (eFreshness == 2 || eFreshness == 3 || eFreshness == 4) {
                        $scope.DeviceOptions.outdoorRecommType = "2";
                        if (eFreshness == 2) {
                            $scope.recommText1 = $scope.translation.WEBMOB_device_ap_outdoor_warning_normal;
                        } else if (eFreshness == 3) {
                            $scope.recommText1 = $scope.translation.WEBMOB_device_ap_outdoor_warning_bad;
                        } else if (eFreshness == 4) {
                            $scope.recommText1 = $scope.translation.WEBMOB_device_ap_outdoor_warning_verybad;
                        }
                        $scope.recommText2 = $scope.translation.WEBMOB_VTL_air_by_kor;
                    } else if (eFreshness == 1 && eFine == 1 && eO3 == 1 && eCO == 1 && eNO2 == 1 && eSO2 == 1 && (eIndoor == 2 || eIndoor == 3 || eIndoor == 4)) {
                        $scope.DeviceOptions.outdoorRecommType = "3";
                        $scope.recommText1 = $scope.translation.WEBMOB_device_ap_ventilation_recommend;
                        $scope.recommText2 = $scope.translation.WEBMOB_VTL_air_by_kor;
                    }
                }
            } else {
                if (!$scope.DeviceOptions.ventControl || $scope.DeviceOptions.outDoorData.KHAI.purity === -1) {
                    $scope.DeviceOptions.outdoorInfoText = '';
                    return;
                }

                if ($scope.DeviceOptions.deviceCountry === 'KR' && eIndoor == 3) {
                    eIndoor = 4;
                }

                if (eFreshness == 1 && eFine == 1 && eO3 == 1 && eCO == 1 && eNO2 == 1 && eSO2 == 1 && (eIndoor == 4 || eIndoor == 5)) {
                    $scope.DeviceOptions.outdoorInfoText = $scope.translation.ACMOB_VTL_main_recommendation_good;
                } else if ((eFreshness == 3 || eFreshness == 4 || eFreshness == 5) && (eFine == 3 || eFine == 4 || eFine == 5)
                        && (eO3 == 3 || eO3 == 4 || eO3 == 5)
                        && (eCO == 3 || eCO == 4 || eCO == 5)
                        && (eNO2 == 3 || eNO2 == 4 || eNO2 == 5)
                        && (eSO2 == 3 || eSO2 == 4 || eSO2 == 5)
                        && (eIndoor == 4 || eIndoor == 5)) {
                    $scope.DeviceOptions.outdoorInfoText = $scope.translation.ACMOB_VTL_main_recommendation_bad;
                } else {
                    $scope.DeviceOptions.outdoorInfoText = '';
                }
            }
        }
    
        /**    	 
        * @function
        * @private
    	* @name outDoorPuirtyData        
        * @param {Object} data
        * @description Parse the outdoor json object and set data to display.
    	 */
        function outDoorPuirtyData(data) {
            var currentDataJson;
             $scope.outDoorDataAvailable = true;
            if ($scope.selectedCountry === 'CN') {
                level = -1;
                $scope.DeviceOptions.outDoorData.KHAI.purity = level;
                $scope.DeviceOptions.outDoorData.KHAI.purityText = getPurityText(level, 'CN', $scope.DeviceOptions.chinaAirSteps);
                $scope.DeviceOptions.outDoorData.KHAI.colorClass = getAirPurityColor(level, 'CN', $scope.DeviceOptions.chinaAirSteps);
                $scope.DeviceOptions.outDoorData.KHAI.iconClass = getAirPurityImage(level, 'CN', $scope.DeviceOptions.chinaAirSteps);

                currentDataJson = data["sinaWeatherData"];
                //only values as we dont have to show color
                $scope.DeviceOptions.outDoorData.AQI.purity = currentDataJson["aqi"];                
                $scope.DeviceOptions.outDoorData.pm10.value = currentDataJson["pm10"];
                $scope.DeviceOptions.outDoorData.pm25.value = currentDataJson["pm25"];
                $scope.DeviceOptions.outDoorData.N2O2.value = currentDataJson["no2"];
                $scope.DeviceOptions.outDoorData.SO2.value = currentDataJson["so2"];

                var level = '';
                if(currentDataJson["aqi"] !== '' && currentDataJson["pm10"] !== '' && currentDataJson["pm25"] !== '' && currentDataJson["no2"] !== '' && currentDataJson["so2"] !== '') {
                    level = currentDataJson["level"];
                   
                } else {
                    level = -1;
                }             
                $scope.DeviceOptions.outDoorData.KHAI.purity = level;
                $scope.DeviceOptions.outDoorData.KHAI.purityText = getPurityText(level, 'CN', $scope.DeviceOptions.chinaAirSteps);                
                $scope.DeviceOptions.outDoorData.KHAI.colorClass = getAirPurityColor(level, 'CN', $scope.DeviceOptions.chinaAirSteps);
                $scope.DeviceOptions.outDoorData.KHAI.iconClass = getAirPurityImage(level, 'CN', $scope.DeviceOptions.chinaAirSteps);   
     
            } else {
                khai_purity = -1;
                $scope.DeviceOptions.outDoorData.KHAI.purity = khai_purity;
                $scope.DeviceOptions.outDoorData.KHAI.purityText = getPurityText(khai_purity, 'KR', $scope.DeviceOptions.chinaAirSteps);
                $scope.DeviceOptions.outDoorData.KHAI.colorClass = getAirPurityColor(khai_purity, 'KR', $scope.DeviceOptions.chinaAirSteps);
                $scope.DeviceOptions.outDoorData.KHAI.iconClass = getAirPurityImage(khai_purity, 'KR', $scope.DeviceOptions.chinaAirSteps);

                var xmlString = decodeURIComponent(data["airData"]);
                var nxmlString = xmlString.replace(/[+]/g, ' ');
                var x2js = new X2JS();
                currentDataJson = x2js.xml_str2json(nxmlString);                
                
                // If we don't get proper response then return from here
                if (!angular.isDefined(currentDataJson["PM10"]["RECORD"])) {
                    $scope.outDoorDataAvailable = false;
                    return;
                }
                
                if (angular.isDefined(currentDataJson["PM10"]) && angular.isDefined(currentDataJson["PM10"]["RECORD"])) {
                    var aRecords = currentDataJson["PM10"]["RECORD"];
                    for (var a = 0; a < aRecords.length; a++) {
                        if (aRecords[a].region === $scope.selectedWeatherLocationCode) {
                            $scope.DeviceOptions.outDoorData.pm10.value = (parseInt(aRecords[a]["PM10"], 10) <= 8 && parseInt(aRecords[a]["PM10"], 10) >= 0) ? 9 : parseInt(aRecords[a]["PM10"], 10);
                            $scope.DeviceOptions.outDoorData.pm10.purity = getPM10GradeKR(aRecords[a]["PM10"]); //aRecords[a]["PM10_grade"];
                            $scope.DeviceOptions.outDoorData.pm10.purityText = getPurityText(getPM10GradeKR(aRecords[a]["PM10"]), 'KR', $scope.DeviceOptions.chinaAirStep);
                            $scope.DeviceOptions.outDoorData.pm10.colorClass = getAirPurityColor(getPM10GradeKR(aRecords[a]["PM10"]), $scope.selectedCountry);
                            
                            $scope.DeviceOptions.outDoorData.pm25.value = (parseInt(aRecords[a]["PM25"], 10) <= 8 && parseInt(aRecords[a]["PM25"], 10) >= 0) ? 9 : parseInt(aRecords[a]["PM25"], 10);
                            $scope.DeviceOptions.outDoorData.pm25.purity = getPM25GradeKR(aRecords[a]["PM25"]); //aRecords[a]["PM25_grade"];
                            $scope.DeviceOptions.outDoorData.pm25.purityText = getPurityText(getPM25GradeKR(aRecords[a]["PM25"]), 'KR', $scope.DeviceOptions.chinaAirStep);
                            $scope.DeviceOptions.outDoorData.pm25.colorClass = getAirPurityColor(getPM25GradeKR(aRecords[a]["PM25"]), $scope.selectedCountry);

                            $scope.DeviceOptions.outDoorData.SO2.value = Number(aRecords[a]["so2"]);
                            $scope.DeviceOptions.outDoorData.SO2.purity = aRecords[a]["so2_grade"];
                            $scope.DeviceOptions.outDoorData.SO2.purityText = getPurityText(aRecords[a]["so2_grade"], 'KR', $scope.DeviceOptions.chinaAirStep);
                            $scope.DeviceOptions.outDoorData.SO2.colorClass = getAirPurityColor(aRecords[a]["so2_grade"], 'KR', $scope.DeviceOptions.chinaAirSteps);

                            $scope.DeviceOptions.outDoorData.O3.value = Number(aRecords[a]["o3"]);
                            $scope.DeviceOptions.outDoorData.O3.purity = aRecords[a]["o3_grade"];
                            $scope.DeviceOptions.outDoorData.O3.purityText = getPurityText(aRecords[a]["o3_grade"], 'KR', $scope.DeviceOptions.chinaAirStep);
                            $scope.DeviceOptions.outDoorData.O3.colorClass = getAirPurityColor(aRecords[a]["o3_grade"], 'KR', $scope.DeviceOptions.chinaAirSteps);

                            $scope.DeviceOptions.outDoorData.CO.value = Number(aRecords[a]["co"]);
                            $scope.DeviceOptions.outDoorData.CO.purity = aRecords[a]["co_grade"];
                            $scope.DeviceOptions.outDoorData.CO.purityText = getPurityText(aRecords[a]["co_grade"], 'KR', $scope.DeviceOptions.chinaAirStep);
                            $scope.DeviceOptions.outDoorData.CO.colorClass = getAirPurityColor(aRecords[a]["co_grade"], 'KR', $scope.DeviceOptions.chinaAirSteps);

                            $scope.DeviceOptions.outDoorData.N2O2.value = Number(aRecords[a]["no2"]);
                            $scope.DeviceOptions.outDoorData.N2O2.purity = aRecords[a]["no2_grade"];
                            $scope.DeviceOptions.outDoorData.N2O2.purityText = getPurityText(aRecords[a]["no2_grade"], 'KR', $scope.DeviceOptions.chinaAirStep);
                            $scope.DeviceOptions.outDoorData.N2O2.colorClass = getAirPurityColor(aRecords[a]["no2_grade"], 'KR', $scope.DeviceOptions.chinaAirSteps);

                            $scope.DeviceOptions.outDoorData.KHAI.value = Number(aRecords[a]["khai"]);
                            $scope.DeviceOptions.outDoorData.KHAI.purity = getKHAIGradeKR(aRecords[a]["khai"]);
                            $scope.DeviceOptions.outDoorData.KHAI.purityText = getPurityText(getKHAIGradeKR(aRecords[a]["khai"]), 'KR', $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.outDoorData.KHAI.colorClass = getAirPurityColor(getKHAIGradeKR(aRecords[a]["khai"]), 'KR', $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.outDoorData.KHAI.iconClass = getAirPurityImage(getKHAIGradeKR(aRecords[a]["khai"]), 'KR', $scope.DeviceOptions.chinaAirSteps);
                            
                            break;
                        }
                    }
                }
                if(angular.isDefined(data["yellowDust"])) {
                    $scope.DeviceOptions.outDoorData.yellowDust = data["yellowDust"];
                }
                if(angular.isDefined(data["yellowLevel"])) {
                    $scope.DeviceOptions.outDoorData.yellowLevel = data["yellowLevel"];
                }
            }

            updateRecommondationText();

        }
        
        /**    	 
        * @function
        * @private
    	* @name getOutDoorAir        
        * @description Send request for outdoor data based on region selected.
    	 */
        function getOutDoorAir() {
            if ($scope.selectedCountry === "CN") {
                if ($scope.selectedWeatherLocation !== null && $scope.selectedWeatherLocation !== undefined) {
                    $scope.selectedWeatherLocationCode = WeatherService.setRegionCodeCN($scope.selectedWeatherLocation);
                    debugMessage('selectedWeatherLocationCode = ' + $scope.selectedWeatherLocationCode);
                    if ($scope.isStatic) {
                        var data = '{"cityCode":"CHXX0103","sinaWeatherData":{"aqi":"28","pollutant":"pm25","level":"1","description":"?","pm25":"20","pm10":"25","so2":"18","no2":"24","source":"??????","time":"2016-4-6 19:00:00"}}';
                        $scope.outDoorPuirtyData(JSON.parse(data));
                    } else {
                        nativeInterface.runOnNative("getSinaWeatherData", '{ "cityCode": "' + $scope.selectedWeatherLocationCode + '" }');
                    }
                }
            } else if($scope.selectedCountry === "KR") {

                if ($scope.selectedWeatherLocation !== null && $scope.selectedWeatherLocation !== undefined) {
                    $scope.selectedWeatherLocationCode = WeatherService.setRegionCode($scope.selectedWeatherLocation);
                    debugMessage('selectedWeatherLocationCode = ' + $scope.selectedWeatherLocationCode);

                    if ($scope.isStatic) {
                        var data = '{"airData": "%3C%3Fxml+version%3D%221.0%22+encoding%3D%22utf-8%22%3F%3E%0A%3CPM10%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E216%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E32.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E25.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02900%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E75.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E248%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E53.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E31.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.04200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01100%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E100.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E235%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E35.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E23.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.01900%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02900%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E73.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E155%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E33.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00200%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03400%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01200%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E72.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E261%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E49.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E32.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00500%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E156%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E37.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E22.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03300%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01600%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E88.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E159%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E95.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E3%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E54.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E3%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.01400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03400%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.02400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E192.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E3%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E277%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E47.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01300%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E74.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E192%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E42.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E27.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00200%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00700%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E80.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E133%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E24.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E24.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.20000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03100%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01200%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E73.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E271%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E44.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00800%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03400%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E77.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E258%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E22.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.70000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03300%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00600%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E61.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E273%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E22.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00200%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.02600%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E238%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E24.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E24.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.20000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03100%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01200%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E73.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E136%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E47.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01300%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E74.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E135%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E34.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E27.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03600%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E83.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E253%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E61.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01200%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E84.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E278%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E47.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01300%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E74.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E279%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E32.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03400%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01100%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E90.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E236%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E20.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E12.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03000%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E63.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E138%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E48.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E32.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00600%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E84.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E119%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E46.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E16.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00700%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E3.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E2%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.01500%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.03900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E2%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E74.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E257%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E77.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E58.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00000%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E0%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.00000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E0%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.00000%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E0%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00000%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E0%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E254%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E34.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.30000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03700%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00600%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E78.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E170%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E49.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E32.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00500%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E272%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E44.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00800%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03400%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E77.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E256%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E42.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E25.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E84.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E175%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E49.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E32.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00500%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E211%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E17.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E1%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00200%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E57.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E112%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E37.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00600%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02700%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01600%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E58.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E226%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E29.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00500%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.80000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02400%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E73.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E130%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E55.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.04000%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00800%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E75.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E289%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E37.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03100%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E288%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E75.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E3%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.04500%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E123.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E3%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E260%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E49.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E32.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00500%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E162%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E89.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E3%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03600%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00800%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E110.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E3%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E221%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E44.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E32.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.30000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02300%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01700%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E78.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E184%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E52.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E3%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E23.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E3%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.30000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.04300%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E179.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E3%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E185%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E25.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00100%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03500%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00300%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E281%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E52.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02600%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00800%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E93.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E283%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E49.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.00000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E0%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02900%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E78.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E189%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E52.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E3%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E23.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E3%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.30000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.04300%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E179.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E3%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E285%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E37.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03100%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E284%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E37.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03100%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E201%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E26.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E1%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E16.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00600%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.20000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03300%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E54.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E203%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E33.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00800%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02900%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.02300%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E79.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E202%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E34.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E28.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02500%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01600%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E70.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E143%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E31.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E24.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00100%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01200%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E78.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E140%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E28.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E23.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.80000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02900%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01000%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E152%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E83.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00800%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03400%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E99.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E245%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E39.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.80000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01100%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E77.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E262%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E22.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.70000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03300%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00600%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E61.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E247%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E53.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E31.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.04200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01100%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E100.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E244%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E53.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E31.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.04200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01100%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E100.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E168%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E27.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02500%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01200%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E66.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E266%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E29.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E23.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00500%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02900%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E75.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E243%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E39.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.80000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01100%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E77.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E127%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E22.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00200%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.02600%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E102%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E26.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E1%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E16.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00600%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.20000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03300%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E54.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E165%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E37.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00700%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03400%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01100%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E100%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E0.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E0%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00000%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E0%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.00000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E0%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.00000%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E0%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00000%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E0%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E101%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E36.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E22.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E1.00000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02600%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01800%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E106%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E32.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E19.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00200%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.30000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03500%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00600%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E67.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E188%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E52.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E3%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E23.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E3%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.30000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.04300%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E179.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E3%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E129%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E10.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E1%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E6.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.80000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01300%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E105%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E36.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E24.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03100%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01000%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E74.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E115%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E32.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E25.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02900%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E75.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E121%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E0.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E0%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00000%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E0%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.00000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E0%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.00000%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E0%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00000%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E0%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E108%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E32.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E27.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.40000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02700%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E68.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E232%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E226.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E3%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.03400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E2%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.20000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02700%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.02300%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E231.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E3%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E131%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E29.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00500%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.80000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02400%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E73.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E259%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E49.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E32.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00500%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E114%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E52.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E45.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E3%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00300%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02200%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01500%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E82.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E137%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E34.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E27.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.60000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03600%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01400%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E83.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E294%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E55.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E0%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E40.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00400%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.04600%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.01000%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E0.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E0%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E295%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E39.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E0.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E0%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00200%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.80000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.02700%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E1%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.02000%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E64.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%09%3CRECORD%3E%0A%09%09%3CNAME%3E169%3C%2FNAME%3E%0A%09%09%3CaplYmd%3E2016-12-09+14%3A00%3A00%3C%2FaplYmd%3E%0A%09%09%3CPM10%3E49.00000%3C%2FPM10%3E%0A%09%09%3CPM10_grade%3E2%3C%2FPM10_grade%3E%0A%09%09%3CPM25%3E32.00000%3C%2FPM25%3E%0A%09%09%3CPM25_grade%3E2%3C%2FPM25_grade%3E%0A%09%09%3Cso2%3E0.00500%3C%2Fso2%3E%0A%09%09%3Cso2_grade%3E1%3C%2Fso2_grade%3E%0A%09%09%3Cco%3E0.50000%3C%2Fco%3E%0A%09%09%3Cco_grade%3E1%3C%2Fco_grade%3E%0A%09%09%3Co3%3E0.03800%3C%2Fo3%3E%0A%09%09%3Co3_grade%3E2%3C%2Fo3_grade%3E%0A%09%09%3Cno2%3E0.00900%3C%2Fno2%3E%0A%09%09%3Cno2_grade%3E1%3C%2Fno2_grade%3E%0A%09%09%3Ckhai%3E81.00000%3C%2Fkhai%3E%0A%09%09%3Ckhai_grade%3E2%3C%2Fkhai_grade%3E%0A%09%3C%2FRECORD%3E%0A%3C%2FPM10%3E%0A","yellowDust": false,"yellowLevel": 0}';
                        $scope.outDoorPuirtyData(JSON.parse(data));
                    } else {
                        //nativeInterface.runOnNative("getAirData", '{ "regionCode": "' + $scope.selectedWeatherLocationCode + '" }');
                        nativeInterface.runOnNative("getDAairData", '{ "regionCode": "' + $scope.selectedWeatherLocationCode + '" }');
                    }
                }
            } else {
                return false;
            }            
        }

        
        /**    	 
         * @function
         * @private
    	 * @name init
    	 * @description after getting the getDevice response, it is called from app.js
    	 */
        function init() {
            $scope.Device = {
                Alarms: [{
                        alarmType: '',
                        code: '',
                        id: '',
                        triggeredTime: ''
                    }],
                ConfigurationLink: {
                    href: ''
                },
                InformationLink: {
                    href: ''
                },
                Consumables: [],
                Light: {
                    dimmingLevel: 0,
                    maxDimmingLevel: 0
                },
                Mode: {
                    options: []
                },
                Sensors: [{id: '', type: '', values: ''}],
                Operation: {
                    power: ''
                },
                Wind: {
                    direction: "Off",
                    speedLevel: 0
                },
                Diagnosis: {
                    diagnosisStart:''
                },//HAP
                Humidity:{
                    current: 0,
                    desired: 0,
                    purityText: '',
                    colorClass: '',
                    iconClass : ''
                },
                connected: '',
                description: '',
                id: '', name: '',
                resources: [],
                type: '',
                uuid: '',
                apcountryValue: '',
                apIndoor: {
                    apindoorFine: '',
                    apindoorFreshness: '',
                    apindoorSmell: '',
                    apindoorUltraFine: ''
                },
                apOutdoor: {
                    apoutdoorCOLevel: '',
                    apoutdoorCOValue: '',
                    apoutdoorFineLevel: '',
                    apoutdoorFineValue: '',
                    apoutdoorFreshness: '',
                    apoutdoorNO2Level: '',
                    apoutdoorNO2Value: '',
                    apoutdoorO3Level: '',
                    apoutdoorO3Value: '',
                    apoutdoorSO2Level: '',
                    apoutdoorSO2Value: ''
                }
            };
            $scope.Actions = [];

            $scope.supportedModes = [];
            $scope.smartCare = {
                started: true,
                diagnosisStarted: false,
                detected:false,
                viewErros:false,
                totalErrors:0,
                errors:[]
            };

            if (!$scope.isStatic) {
                $scope.checkResponse.push(APCONST.SCHEDULEDATA);
                SHPService.sendSHPCommand(APCONST.GET, "/" + $scope.peerId + "/actions");
            } 
        }
        
        /**    	 
        * @function
        * @private
    	* @name dismissAllPopups
    	* @description To dismiss all popups
    	*/
        function dismissAllPopups(modePopup) {
            if ($scope.showVentSelectionPopup) {
                //$scope.showVentSelectionPopup = false;
                hideVentPopUp();
                return;
            }
            if ($scope.dialogPopup.show) {
                $scope.dialogPopup.show = false;
                toggleBottomPopUpClass($scope.dialogPopup.show);
                return;
            }
            if ($scope.showPurityPopup) {
                popUpOkApHome();
                return;
            }
            if ($scope.showModeSelectionPopup && !modePopup) {
                $scope.showModeSelectionPopup = false;
                return;
            }
            if ($scope.showHumidfiSettingPopup) {
                $scope.showHumidfiSettingPopup = false;
                return;
            }

            if ($scope.showVolumePopup) {
                $scope.showVolumePopup = false;
                toggleVolumePopUpClass();
                return;
            }
        }
        
        /**    	 
        * @function
        * @private
    	* @name showConnectionFailurePopup       
        * @description To display connection failure popup, it is called from app.js
    	*/
        function showConnectionFailurePopup () {
            dismissAllPopups();
            document.getElementById("connFailCancel").style.visibility = 'hidden';
            $scope.showConnFailurePopup = true;
            $scope.loadingBar = false;
            toggleBottomPopUpClass($scope.showConnFailurePopup);
            $scope.appAlive = true;
        };

        /**    	 
         * @function
         * @private
    	 * @name closeConnectionFailurePopup        
         * @description Close connection failure popup, it is called from app.js
    	 */
        function closeConnectionFailurePopup() {
           $scope.showConnFailurePopup = false;
           toggleBottomPopUpClass($scope.showConnFailurePopup);
        }

        /**    	 
         * @function
         * @private
    	 * @name closeApp
         * @description To close application, called from app.js        
    	 */
        function closeApp () {
            nativeInterface.runOnNative("closeWebApp", "");
            
        };
    
    
        /* Volume slider popup start */
        /**    	 
        * @function
        * @private
    	* @name openPopoverVolumeSelection        
        * @description Open volume selection popup.
    	*/
        function openPopoverVolumeSelection() {
            $scope.showVolumePopup = true;
            toggleVolumePopUpClass();
            updateVolumeSlider();       
        }
        
        /**    	 
        * @function
        * @private
    	* @name updateVolumeSlider        
        * @description
    	*/
        function updateVolumeSlider() {
            if ($scope.showVolumePopup) {
                $scope.tempVolume = getVolumeLevelNum($scope.DeviceOptions.volume);
                updateVolumeIconUpdate($scope.tempVolume);
            }
        }
        
        /**    	 
        * @function
        * @private
    	* @name updateVolumeIconUpdate        
        * @description
    	*/
        function updateVolumeIconUpdate(val) {
            var temp;
            if (val == '0' || val == "Mute") {
                temp = $scope.SERVER_URL+"AP/assets/img/main_ac_volume_ic_off.svg#Layer_1";
            } else {
                temp = $scope.SERVER_URL+"AP/assets/img/main_ac_volume_ic_on.svg#Layer_1";
            }
            var svg = document.getElementById('volumePopupIcon');
            SHPService.loadSVG(temp, svg);
        };
        
        /**    	 
        * @function
        * @private
    	* @name onVolumeSave        
        * @description
    	*/
        function onVolumeSave(newVolume) {
            $scope.showVolumePopup = false;
            toggleVolumePopUpClass();
            if (!$scope.isStatic) {
                var vol = getVolumeLevelBig(newVolume);
                if ($scope.DeviceOptions.volume !== ''+ vol) {
                    var payload = '{"Device":{"Mode":{"options":["Volume_' + vol + '"]}}}';
                    $scope.checkResponse.push(APCONST.VOLUMECHANGE);
                    SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0");
                    
                }
            } else {
                $scope.DeviceOptions.volume = newVolume;
            }
        }
        
        /**    	 
        * @function
        * @private
    	* @name changeVolume        
        * @description
    	*/
        function changeVolume() {
           var vol = '';
            if($scope.DeviceOptions.volume === 'Mute') {
                vol = '3';
            } else {
                vol = '0';
            }
            onVolumeSave(vol);
        }
        
        /**    	 
        * @function
        * @private
    	* @name onVolumeCancel        
        * @description
    	*/
        function onVolumeCancel() {
            $scope.showVolumePopup = false;
            toggleVolumePopUpClass();
        }
        
        /**    	 
        * @function
        * @private
    	* @name getVolumeLevelText        
        * @description
    	*/
        function getVolumeLevelText(volume) {
            updateVolumeIconUpdate(volume);
            if (volume == '0' || volume == 'Mute') {
                return $scope.translation.ACMOB_VTL_settings_volume_mute;
            } else if (volume == '1' || volume == '33') {
                return $scope.translation.ACMOB_VTL_settings_volume_low;
            } else if (volume == '2' || volume == '66') {
                return $scope.translation.ACMOB_VTL_settings_volume_mid;
            } else if (volume == '3' || volume == '100') {
                return $scope.translation.ACMOB_VTL_settings_volume_high;
            }
            else {
                return "";
            }
        }
        
        /**    	 
        * @function
        * @private
    	* @name getVolumeLevelNum        
        * @description
    	*/
        function getVolumeLevelNum(volume) {
            if (volume === "Mute") {
                return 0;
            } else if (volume === '33') {
                return 1;
            } else if (volume === '66') {
                return 2;
            } else if (volume === '100') {
                return 3;
            } else {
                return volume;
            }
        }
        
        /**    	 
        * @function
        * @private
    	* @name getVolumeLevelTextOnLoad        
        * @description
    	*/
        function getVolumeLevelTextOnLoad(volume) {
            if (volume == '0' || volume == 'Mute') {
                return $scope.translation.ACMOB_VTL_settings_volume_mute;
            } else if (volume == '1' || volume == '33') {
                return $scope.translation.ACMOB_VTL_settings_volume_low;
            } else if (volume == '2' || volume == '66') {
                return $scope.translation.ACMOB_VTL_settings_volume_mid;
            } else if (volume == '3' || volume == '100') {
                return $scope.translation.ACMOB_VTL_settings_volume_high;
            }
            else {
                return "";
            }
        }
        
        /**    	 
        * @function
        * @private
    	* @name getVolumeLevelNumOnLoad        
        * @description
    	*/
        function getVolumeLevelNumOnLoad(volume) {
            if (volume === "Mute") {
                return 0;
            } else if (volume === '33') {
                return 1;
            } else if (volume === '66') {
                return 2;
            } else if (volume === '100') {
                return 3;
            } else {
                return volume;
            }
        }
        
        /**    	 
        * @function
        * @private
    	* @name getVolumeLevelBig        
        * @description
    	*/
        function getVolumeLevelBig(volume) {
            volume = '' + volume;
            if (volume === '0') {
                return "Mute";
            } else if (volume === '1') {
                return 33;
            } else if (volume === '2') {
                return 66;
            } else if (volume === '3') {
                return 100;
            } else {
                return volume;
            }
        };
        // Volume slider popup end

        /**    	 
         * @function
         * @private
    	 * @name openAirInfoSetting
         * @description To display set air infor popup if no region set.        
    	 */
        function openAirInfoSetting() {
            $scope.dialogPopup.show = true;
            $scope.dialogPopup.dialogPopupTitle = $scope.translation.WEBMOB_device_VTL_noregion_title;
            $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_device_VTL_noregion_description;
            $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_common_cancel;
            $scope.dialogPopup.button2Enabled = true;
            $scope.dialogPopup.button2Title = $scope.translation.WEBMOB_common_ok;
            $scope.dialogPopup.button2Handler = function () {
            $scope.dialogPopup.show = false;
            showRegionListScreen();

            };
        }
    
        /**    	 
        * @function
        * @private
    	* @name onBottonPopupCancel
    	* @description To close vent selection popup
    	*/
        function onBottonPopupCancel() {
            $scope.showVentSelectionPopup = false;
            toggleBottomPopUpClass($scope.showVentSelectionPopup);
        }

        /**    	 
         * @function
         * @private
    	 * @name selectVent        
         * @param {String} id
         *       vent item id
    	 * @description Send vent selection request
         */
        function selectVent (id) {
            $scope.tempVentLevel = getVentLevel(id);
            onSaveVentSelection();
        }

        /**    	 
         * @function
         * @private
    	 * @name getVentLevel       
         * @param {String} id
         *       Vent id
    	 * @description To get vent level based on vents selected
         */
        function getVentLevel(id) {
            var temp = "",
                    ventUpper = $scope.ventUpper,
                    ventMiddle = $scope.ventMiddle,
                    ventLower = $scope.ventLower;
            switch (id) {
                case "ventUpper":
                    ventUpper = !ventUpper;
                    break;
                case "ventMiddle":
                    ventMiddle = !ventMiddle;
                    break;
                case "ventLower":
                    ventLower = !ventLower;
                    break;
                default:
                    break;
            }
            if ($scope.ventModel === APCONST.THREE_VENT_MODEL) {
                temp = (ventLower === true ? 1 : 0) + "" + (ventMiddle === true ? 1 : 0) + "" + (ventUpper === true ? 1 : 0);
            } else if ($scope.ventModel === APCONST.TWO_VENT_MODEL) {
                temp = (ventLower === true ? 1 : 0) + "" + (ventUpper === true ? 1 : 0);
            }
            else {
                temp = 0;
            }
            return parseInt(temp, 2);
        };

        /**    	 
        * @function
        * @private
    	* @name getVentText    
        * @param {Number} ventLevel
        *       Vent level
    	* @description To get text based on vent level
        */
        function getVentText (ventLevel) {
                      
            switch (ventLevel) {
                case APCONST.VENTLEVEL_0:
                    return $scope.translation.WEBMOB_common_none;
                case APCONST.VENTLEVEL_1:
                    return $scope.translation.ACMOB_VTL_options_upper_vent;
                case APCONST.VENTLEVEL_2:
                    return $scope.translation.ACMOB_VTL_options_lower_vent;
                case APCONST.VENTLEVEL_3:
                    return  $scope.translation.ACMOB_VTL_options_upper_vent + ", " + $scope.translation.ACMOB_VTL_options_lower_vent;
                default:
                    return "";
            }
        }

        /**    	 
         * @function
         * @private
    	 * @name decodeVentLevel      
         * @param {Number} ventLevel
         *       Vent level
    	 * @description Update lower, middle and upper based on vent level
         */
        function decodeVentLevel (ventLevel) {
            var temp = "0" + ventLevel.toString(2);
            temp = temp.substr(-2);
            $scope.ventLower = temp.charAt(0) === '1' ? true : false;
            $scope.ventUpper = temp.charAt(1) === '1' ? true : false;
                        
            if ($scope.showVentSelectionPopup) {
                updateVentImage(ventLevel);
            }
        }

        /**    	 
         * @function
         * @private
    	 * @name onSaveVentSelection      
         * @description Send vent save request to device.
         */
        function onSaveVentSelection() {
            if ($scope.DeviceOptions.currentVentLevel !== $scope.tempVentLevel) {
                var secondReq = false, 
                        payload;
                $scope.dialogPopup.show = false;
                if (!$scope.isStatic) {
                    var curVentLevel = $scope.tempVentLevel * 2;
                    if ($scope.DeviceOptions.ventControl && $scope.DeviceOptions.turbo) {
                        if(curVentLevel === 0) {
                            payload = '{"Device": {"Mode": {"options": ["Blooming_2", "Comode_Off"]},  "Wind": {}}}';
                            secondReq = true;
                        } else {
                            payload = '{"Device": {"Mode": {"options": ["Blooming_' + curVentLevel + '", "Comode_Off"]},  "Wind": {}}}';
                        }
                    } else {
                        payload = '{"Device":{"Mode":{"options":["Blooming_' + curVentLevel + '"]}}}';
                    }
                    if(!$scope.powerState) {
                        payload = '{"Device":{"Operation": {"power": "On"}, "Mode":{"options":["Blooming_' + curVentLevel + '"]}}}';
                    }
                    $scope.checkResponse.push(APCONST.VENT);
                    SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0");
                    if(secondReq) {
                        $timeout(function () {
                            SHPService.sendSHPCommand(APCONST.PUT, '{"Device":{"Mode":{"options":["Blooming_0"]}}}', "/" + $scope.peerId + "/devices/0");
                        }, 3000);
                    }
                } else {
                    $scope.DeviceOptions.currentVentLevel = $scope.tempVentLevel;
                }
            }
        };

        /**    	 
         * @function
         * @private
    	 * @name hideVentPopUp      
         * @description Close the vent selection popup.
         */
        function hideVentPopUp() {
            $scope.showVentSelectionPopup = false;
            toggleBottomPopUpClass($scope.showVentSelectionPopup);
        }
            
        /**    	 
         * @function
         * @private
    	 * @name getVentImageClass
         * @param {Number} ventlevel      
         * @description Get vent image class based on vent level.
         */
        function getVentImageClass(ventlevel) {
            var ventClass;
            ventClass = getTwoVentImageClass(ventlevel);
            
            return ventClass;
        }
        
        /**    	 
         * @function
         * @private
    	 * @name getTwoVentImageClass
         * @param {Number} ventlevel      
         * @description Update vent image based on vent level.
         */
        function getTwoVentImageClass(ventlevel) {
            switch (ventlevel) {
            case APCONST.VENTLEVEL_0:
                return $scope.SERVER_URL + "AP/assets/img/main_list_ic_2_vent_none.svg#Layer_1";
            case APCONST.VENTLEVEL_1:
                return $scope.SERVER_URL + "AP/assets/img/main_list_ic_2_vent_upper.svg#Layer_1";
            case APCONST.VENTLEVEL_2:
                return $scope.SERVER_URL + "AP/assets/img/main_list_ic_2_vent_lower.svg#Layer_1";
            case APCONST.VENTLEVEL_3:
                return $scope.SERVER_URL + "AP/assets/img/main_list_ic_2_vent_upper_lower.svg#Layer_1";
            default:
                return "";
            }
        }

    
    
        /**    	 
         * @function
         * @private
    	 * @name updateTwoVentImage
         * @param {Number} ventlevel      
         * @description Update vent image based on vent level.
         */
        function updateTwoVentImage(ventlevel) {
           switch (ventlevel) {
            case APCONST.VENTLEVEL_0:
                document.getElementById("ventImage").src = $scope.SERVER_URL + "AP/assets/img/main_img_air_2_00.png";
                break;
            case APCONST.VENTLEVEL_1:
                document.getElementById("ventImage").src = $scope.SERVER_URL + "AP/assets/img/main_img_air_2_01.png";
                break;
            case APCONST.VENTLEVEL_2:
                document.getElementById("ventImage").src = $scope.SERVER_URL + "AP/assets/img/main_img_air_2_02.png";
                break;
            case APCONST.VENTLEVEL_3:
                document.getElementById("ventImage").src = $scope.SERVER_URL + "AP/assets/img/main_img_air_2_03.png";
                break;
            default:
                break;
            } 
        }

        /**    	 
        * @function
        * @private
    	* @name updateVentImage       
        * @param {Number} ventLevel
        *       Vent level
        * @description To display vent selected image in vent selection popup based on vent level
        */
        function updateVentImage (ventLevel) {
            updateTwoVentImage(ventLevel);
            
        }

        /**    	 
        * @function
        * @private
    	* @name changeAPScreenHeader       
        * @param 
        * @description To change screen header based on page to display
        */
        function changeAPScreenHeader() {
            switch ($scope.currentScreen) {
                case APCONST.SCREEN_HOME:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = true;
                    $scope.bBackKeyVisible = false;
                    $scope.bHeaderCenterComponent = true;

                    break;
                case APCONST.SCREEN_FILTER:
                    $scope.bAPHeader = false;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;

                    break;
                case APCONST.SCREEN_AIRFRESHNESSINFOSTATUS:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;
                    break;
                case APCONST.SCREEN_WINDSTRENGTH:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;

                    break;
                case APCONST.SCREEN_SETTINGS:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;

                    break;
                case APCONST.SCREEN_SCHEDULER:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;

                    break;
                case APCONST.SCREEN_ADDSCHEDULE:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;
                    //$scope.subHeaderTitle = $scope.translation.WEBMOB_common_schedule_add_schedule;
                    break;
                case APCONST.SCREEN_HELP:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;

                    break;
                case APCONST.SCREEN_RECOMMENDTINFO:
                case APCONST.SCREEN_TREND_INFO:
                case APCONST.SCREEN_VENTILATION_RECOMMEND:
                case APCONST.SCREEN_OUTDOOR_PURITY_WARNING:
                case APCONST.SCREEN_YELLOW_DUST_WARNING:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;

                    break;
                case APCONST.SCREEN_AIRINFO:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;

                    break;
                case APCONST.SCREEN_AIRFRESHNESSINFO:
                    $scope.bAPHeader = true;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;

                    break;
                case APCONST.SCREEN_ABOUT:
                    $scope.bAPHeader = false;
                    $scope.bPowerButtonVisible = false;
                    $scope.bBackKeyVisible = true;
                    $scope.bMoreOptionsVisible = false;
                    $scope.bHeaderCenterComponent = true;
                    break;
                case APCONST.SCREEN_SUPPLY_STORE:
                    $scope.bAPHeader = false;
                    $scope.supplyStoreScreenVisible = true;
                    $scope.bBackKeyVisible = true;

                    break;
                case APCONST.SCREENS_SMART_CARE:
                    $scope.bAPHeader = false;
                    $scope.smartCarePage = true;
                    $scope.bBackKeyVisible = true;

                    break;
                default:
                    break;
            }
        };

        /**    	 
        * @function
        * @private
    	* @name showScreens       
        * @param (Number) index
        * @description To show screen based on page to display
        */
        function showScreens(index) {
            
            switch (index) {
                case 0:
                    $scope.bAPHeader = true;
                    break;
                case APCONST.SCREEN_HOME:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_device_air_purifier;
                    $scope.bAPHomePageContainer = true;
                    $state.transitionTo('HomePage');
                    //checking if to show first timw usage help page
                    var hasLaunchedBefore = getLocalStorageData('firstTimeLaunch');
                    if (!hasLaunchedBefore) {
                        showUsageHelpPage();
                        setLocalStorageData('firstTimeLaunch', true);
                    }
                    break;
                case APCONST.SCREEN_FILTER:
                    $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_filter_header_title;
                     $state.transitionTo('FilterPage');
                    break;
                case APCONST.SCREEN_AIRFRESHNESSINFOSTATUS:
                     $state.transitionTo('AirFreashnessStatusPage');
                    break;
                case APCONST.SCREEN_WINDSTRENGTH:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_options;
                     $state.transitionTo('OptionPage');
                    break;
                case APCONST.SCREEN_SETTINGS:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_settings;
                    $state.transitionTo('SettingPage');
                    break;
                case APCONST.SCREEN_SCHEDULER:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_schedule_schedule;
                    $scope.bAPScheduleScreenVisible = true;
                    $state.transitionTo('ScheduleListPage');
                    break;
                case APCONST.SCREEN_ADDSCHEDULE:
                    $scope.addSchedulePage = true;
                    $state.transitionTo('AddSchedulePage');
                    break;
                case APCONST.SCREEN_HELP:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_help;
                    $scope.bAPHelpScreenVisible = true;
                     $state.transitionTo('HelpPage');
                    break;
                case  APCONST.SCREEN_RECOMMENDTINFO:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_VTL_help_recommend;
                    $state.transitionTo('RecommendationPage');
                    break;
                case  APCONST.SCREEN_AIRINFO:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ap_air_info;
                    $state.transitionTo('AirInfoPage');
                    break;
                case APCONST.SCREEN_AIRFRESHNESSINFO:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_VTL_help_airfreshness;
                     $state.transitionTo('AirFreashnessPage');
                    break;
                case APCONST.SCREEN_ABOUT:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_about_device;
                    $scope.bAPAboutPage = true;
                    $state.transitionTo('AboutPage');
                    break;
                case APCONST.SCREEN_SUPPLY_STORE:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_VTL_deviceinfo_supplystore;
                    $scope.supplyStoreScreenVisible = true;
                    $state.transitionTo('StoreNCarePage');
                    break;
                case APCONST.SCREENS_SMART_CARE:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_VTL_smartcare_smartcare;
                    $scope.smartCarePage = true;
                    $state.transitionTo('StoreNCarePage');
                    break;
                case APCONST.SMARTCAREERROR:
                    $scope.smartcareerror = true;
                    $state.transitionTo('StoreNCarePage');
                    break;
                case APCONST.SCREEN_USAGE_HELP:
                    $scope.bUsageHelpPage = true;
                    break;    
                case APCONST.ERRORPAGE:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_common_device_air_purifier;
                    $state.transitionTo('ErrorPage');
                    dismissAllPopups();
                    break;
                case APCONST.SCREEN_TREND_INFO:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ap_trend_info;
                    $state.transitionTo('TrendInfoPage');
                    break;
                case APCONST.SCREEN_VENTILATION_RECOMMEND:
                    $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_help_ventilation;
                    $state.transitionTo('VentilationRecommendPage');
                    break;
                case APCONST.SCREEN_OUTDOOR_PURITY_WARNING:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ap_purity_warning;
                    $state.transitionTo('OutdoorPurityWarningPage');
                    break;
                case APCONST.SCREEN_YELLOW_DUST_WARNING:
                    $scope.subHeaderTitle = $scope.translation.WEBMOB_device_ap_dust_warning;
                    $state.transitionTo('YellowDustWarningPage');
                    break;
                default:
                    break;
            }
        }

        /**    	 
         * @function
         * @private
    	 * @name changeScreen
         * @param 
         * @description Change screen logic - visibility and invisibilty of pages
    	 */
        function changeScreen() {

            changeAPScreenHeader();

            $scope.bAPHomePageContainer = false;
            $scope.bAPScheduleScreenVisible = false;
            $scope.addSchedulePage = false;
            $scope.bAPHelpScreenVisible = false;
            $scope.bAPAboutPage = false;
            $scope.supplyStoreScreenVisible = false;
            $scope.smartCarePage = false;
            $scope.smartcareerror = false;

            showScreens($scope.currentScreen);

        }

        /**    	 
         * @function
         * @private
    	 * @name updateDeviceData
         * @param {String} data
         * data is passed to updateDeviceData which is in app.js
    	 * @description it gets called from nativeCallback function (app.js), it is for passing the data from native app to web app
    	 */
        function updateDeviceData(data) {
            SHPService.updateDeviceData($scope, data);
        };

        /**    	 
        * @function
        * @private
    	* @name startApp
        * @param 
        * @description All neccessory operation perform to launch home screen of app.
    	**/
        function startApp() {
            if ($scope.isLCDModel && apSupportOutdoor && $scope.selectedCountry === "KR") {
                if (deviceNation === $scope.selectedCountry) {
                    var found = false;
                    for (var i = 0; i < $scope.searchValues.length; i++) {
                        if (getRegionCodeFromName($scope.searchValues[i], $scope.selectedCountry) === deviceRegion) {
                            $scope.selectedWeatherLocation = $scope.searchValues[i];
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        $scope.selectedWeatherLocation = null;
                    }
                }
            } else {
                var locStringKey = getLocalStorageData('selectedRegionName#'+$scope.deviceUuid);
                var idx = $scope.weatherLocations.indexOf(locStringKey);
                $scope.selectedWeatherLocation = mOrginialValues[idx];
            }            
            $scope.mainLoadingScreen = true;
            $scope.pageLoaded = true;

            var prevSelectedCountry = ($scope.isLCDModel && apSupportOutdoor) ? deviceNation :  getLocalStorageData('selectedCountry#'+$scope.deviceUuid);
            setLocalStorageData('selectedCountry#'+$scope.deviceUuid, $scope.selectedCountry); 
            
           if((prevSelectedCountry !== null && prevSelectedCountry !== undefined) && (prevSelectedCountry === "KR" || prevSelectedCountry === "CN") && prevSelectedCountry !== $scope.selectedCountry) {
                // delete local storage and save new selected country
                removeLocalStorageData('selectedRegionName#'+$scope.deviceUuid);
                $scope.selectedWeatherLocation = null;
                //if current country is korea or china , show the region list
                if ($scope.selectedCountry === "KR" || $scope.selectedCountry === "CN") {
                   showRegionListScreen();
                }
                firstTimeLaunch = true;
            } else if(($scope.selectedCountry === "KR" || $scope.selectedCountry === "CN") && ($scope.selectedWeatherLocation === null || $scope.selectedWeatherLocation === undefined)) {
                if ($scope.diagnosisStart !== "Run" && !$scope.showPurityPopup && $scope.appAlive !== true) {
                   showRegionListScreen();
                }
                firstTimeLaunch = true;
            } else {
                getOutDoorAir();
            }
            
            sendConfigToDevice($scope.selectedWeatherLocation);
            dismissLoadingBar(APCONST.GET_DEVICES);
            //if not showing air purity popup (if its the response of sensor request, dont change to Air Info Screen)
            if(!$scope.showPurityPopup) {
                changeScreen();
            }
        }
        
        /**    	 
         * @function
         * @private
    	 * @name showRegionListScreen     
         * @param 
    	 * @description To launch the air info region list screen
    	 */
        function showRegionListScreen() {
            $scope.currentScreen = APCONST.SCREEN_AIRINFO;
            changeScreen();
        }
        
        /**    	 
         * @function
         * @private
    	 * @name parseDeviceData     
         * @param {Object} device
         *       getDevice response object
    	 * @description To parse getDevice response, it contains complete details of present device state
    	 */
        function parseDeviceData (device) {
            console.log(" parseDeviceData -->" + JSON.stringify(device));
            // Checking Uuid
            if (!!!$scope.isStatic && angular.isDefined(device["uuid"]) && device["uuid"] !== $scope.deviceUuid) {
                return;
            }
            
            if(!!!$scope.Device) {
                return;
            }
//            console.log('ap:'+$scope.isAirPurifier + ', hap:' +$scope.isHumidifierAirPurifier);

            // Device ID
            if (angular.isDefined(device["id"])) {
                $scope.deviceID = parseInt(device["id"], 10);
            }
            
            if (!!device.Alarms && device.Alarms.length) {
                if(~JSON.stringify(device.Alarms).indexOf("ErrorCode_E154") || ~JSON.stringify(device.Alarms).indexOf("ErrorCode_E155") ||
                    ~JSON.stringify(device.Alarms).indexOf("ErrorCode_E176") || ~JSON.stringify(device.Alarms).indexOf("ErrorCode_E184") ) {
                    homeMultiService.handleAlarms(device.Alarms, "ErrorCode_", "ErrorCode_E", "WEBMOB_device_VTL_error_code_", "", APCONST.SMARTCAREERROR)
                }
                 
                if(!angular.isDefined(device.event)) {
                    angular.copy(device.Alarms, $scope.Device.Alarms);                  
                    addInAppAlarm($scope.Device.Alarms); 
                } else {
                     if(device.event === "Created") {
                       addInAppAlarm(device.Alarms); 
                     } else {
                       deleteInAppAlarm(device.Alarms);
                     }
                }
                
                showInAppPopup();
            }
        
            if (angular.isDefined(device["Diagnosis"]) && angular.isDefined(device["Diagnosis"].diagnosisStart)) {
   
                $scope.showSmartCare = true;
                $scope.diagnosisStart = device["Diagnosis"].diagnosisStart;
                if ($scope.diagnosisStart === "Ready") {
                    //Lokesh 17Nov15_2050 -> Should dismiss loading bar as soon as getting 'Ready' for diagnosis.
                    //As per Jihee Jang, after 'Run', we get ready only when diagnosis is completed.
                    dismissLoadingBar(APCONST.DIAGNOSIS);
                    diagnosisStarted = false;
                    if ($scope.currentScreen === APCONST.SCREENS_SMART_CARE) {
                        $scope.currentScreen = APCONST.SMARTCAREERROR;
                        changeScreen();
                    }
                } else {
                    $scope.currentScreen = APCONST.SCREENS_SMART_CARE;
                    changeScreen();
                }

            }
            
            // Information URL
            if (angular.isDefined(device["InformationLink"]) && angular.isDefined(device["InformationLink"].href)) {
                $scope.Device.InformationLink.href = device["InformationLink"].href;
                if (!$scope.isStatic) {
                    $scope.checkResponse.push(APCONST.INFORMATION);
                    SHPService.sendSHPCommand(APCONST.GET, "/" + $scope.peerId + $scope.Device.InformationLink.href);
                }
            }
            // Configuration URL
            if (angular.isDefined(device["ConfigurationLink"]) && angular.isDefined(device["ConfigurationLink"].href)) {
                $scope.Device.ConfigurationLink.href = device["ConfigurationLink"].href;
            }

            if (!!device.Consumables && device.Consumables.length) {
                    $scope.Device.Consumables = angular.copy(device.Consumables);
                    updateFilterLayerStyle();
                    dismissLoadingBar(APCONST.FILTER_RESET);
            }
            
            //HAP
            if(angular.isDefined(device["Humidity"])) {
            	if (angular.isDefined(device["Humidity"].current)) {
                    $scope.Device.Humidity.current = device["Humidity"].current;
                }
                if (angular.isDefined(device["Humidity"].desired)) {
                    dismissLoadingBar(APCONST.HUMIDITY_COMMAND);
                    $scope.Device.Humidity.desired = device["Humidity"].desired;
                    $scope.desiredHumidityValue = $scope.DeviceOptions.isHumidiOn ? getDesiredHumidityValueName(device["Humidity"].desired) : $scope.listOfHumidifiModes[4].modeName;
                }
            }
   
            if (angular.isDefined(device["Mode"]) && angular.isDefined(device["Mode"].options)) {
                $scope.Device.Mode.options = device["Mode"].options;

                $scope.Device.Mode.options.forEach(function (value) {
                    if (value.indexOf("Comode") !== -1) {
                        $scope.DeviceOptions.quiet = value.indexOf("Comode_Quiet") !== -1 ? true : false;
                        $scope.DeviceOptions.turbo = value.indexOf("Comode_Speed") !== -1 ? true : false;
                        $scope.DeviceOptions.comodeOff = value.indexOf("Comode_Off") !== -1 ? true : false;
                    }
                    if (value.indexOf("Spi") !== -1) {
                        $scope.DeviceOptions.spi = value.indexOf("Spi_On") !== -1 ? true : false;
                    }
                    if (value.indexOf("Blooming") !== -1) {
                        $scope.DeviceOptions.currentVentLevel = (parseInt(value.slice(-1), 10)) / 2;
                        $scope.tempVentLevel = $scope.DeviceOptions.currentVentLevel;
                        decodeVentLevel($scope.DeviceOptions.currentVentLevel);
                        dismissLoadingBar(APCONST.VENT);
                    }
                    if (value.indexOf("RestWind") !== -1) {
                        $scope.DeviceOptions.restWind = value.split("_")[1];
                    }
                    if (value.indexOf("Sleep_") !== -1) {
                        $scope.DeviceOptions.sleep = value.split("_")[1];
                    }
                    if (value.indexOf("Light_") !== -1) {
                        $scope.DeviceOptions.lighting = value.indexOf("Light_On") !== -1 ? true : false;
                    }
                    if (value.indexOf("Volume_") !== -1) {
                        // If we don't get Volume_NotSupported, then that means mute option is supported
                        $scope.DeviceOptions.volume = value.split("_")[1];
                        if($scope.DeviceOptions.volume === "NotSupported"){
                            $scope.DeviceOptions.isMuteSupported = false;
                        }else{
                            $scope.DeviceOptions.isMuteSupported = true;
                        }
                        updateVolumeSlider();
                        dismissLoadingBar(APCONST.VOLUMECHANGE);
                    }
                    if (value.indexOf("Autoclean_") !== -1) {
                        $scope.DeviceOptions.autoClean = value.indexOf("Autoclean_On") !== -1 ? true : false;
                    }
                    if (value.indexOf("OptionCode_") !== -1) {
                      
                        var ocode = parseInt(value.split("_")[1], 10).toString(2);
                        $scope.DeviceOptions.optionCode = ocode.split('').reverse().join('');
                        
                        //Checking for model (0 korea model, 1 global model)
                        if ($scope.DeviceOptions.optionCode.charAt(0) === "1") {
                            $scope.DeviceOptions.deviceCountry = 'CN';
                        } else if ($scope.DeviceOptions.optionCode.charAt(0) === "0") {
                            $scope.DeviceOptions.deviceCountry = 'KR';
                        }
                        
                        // if its global model, check for steps (1 means 6steps, 0 means 5steps)
                        if ($scope.DeviceOptions.deviceCountry === 'CN' && $scope.DeviceOptions.optionCode.charAt(1) === "1") {
                            $scope.DeviceOptions.chinaAirSteps = 6;
                        } else if ($scope.DeviceOptions.deviceCountry === 'CN' && $scope.DeviceOptions.optionCode.charAt(1) === "0") {
                            $scope.DeviceOptions.chinaAirSteps = 5;
                        }

                        if ($scope.DeviceOptions.optionCode.charAt(2) === "1") {
                            $scope.DeviceOptions.filterSteps = true;
                        } else {
                            $scope.DeviceOptions.filterSteps = false;
                        }

                        if ($scope.DeviceOptions.optionCode.charAt(3) === "1") {
                            $scope.DeviceOptions.ventControl = false;
                        } else {
                            $scope.DeviceOptions.ventControl = true;
                            $scope.listOfModes[1].modeName = 'ACMOB_VTL_main_dropdown_high_15K';
                            $scope.listOfModes[3].modeName = 'ACMOB_VTL_main_dropdown_low_15K';
                            if(ocode.charAt(0) === "1") {
                               $scope.DeviceOptions.deviceCountry = 'CN';
                               $scope.DeviceOptions.chinaAirSteps = 5;
                            } else {
                               $scope.DeviceOptions.deviceCountry = 'KR'; 

                            } 
                        }

                        if($scope.DeviceOptions.optionCode.charAt(4) === '1' || $scope.DeviceOptions.ventControl) {
                            $scope.DeviceOptions.pm10SupportOnly = true;
                        }
                        if($scope.DeviceOptions.optionCode.charAt(5) === '1' || $scope.DeviceOptions.ventControl) {
                            $scope.DeviceOptions.pm25SupportOnly = true;
                        }
                        if($scope.DeviceOptions.optionCode.charAt(11) === '1'){
                            $scope.DeviceOptions.pm10UltraFineSupportOnly = true;
                        }
                        // checking if spi (virus doctor) supported or not
                        if($scope.DeviceOptions.optionCode.charAt(6) !== '' && $scope.DeviceOptions.optionCode.charAt(6) === '1') {
                            $scope.spiSupported = false;
                        }

                        if($scope.DeviceOptions.optionCode.charAt(APCONST.OPTION_CODE_SPEED_USE_BIT) !== '' && $scope.DeviceOptions.optionCode.charAt(APCONST.OPTION_CODE_SPEED_USE_BIT) === '1') {
                            $scope.DeviceOptions.isSpeedSupported = false;
                        }

                        if($scope.DeviceOptions.optionCode.charAt(APCONST.OPTION_CODE_LIGHT_SUPPORT_WHILE_STERILIZE_CLEAN_RUN_BIT) !== '' && $scope.DeviceOptions.optionCode.charAt(APCONST.OPTION_CODE_LIGHT_SUPPORT_WHILE_STERILIZE_CLEAN_RUN_BIT) === '1') {
                            $scope.DeviceOptions.isLightEnabledWhileSterilizeCleanRunOn = true;
                            console.log("new hap device");
                        }

                        if($scope.DeviceOptions.optionCode.charAt(APCONST.OPTION_CODE_SOUND_LIGHT_DISTINGUISH_BIT) !== '' && $scope.DeviceOptions.optionCode.charAt(APCONST.OPTION_CODE_SOUND_LIGHT_DISTINGUISH_BIT) === '1') {
                            $scope.DeviceOptions.isDistinguishingMuteLightOption = true;
                        }

                        // checking whether using new Option Structure for 7/8 bit
                        if($scope.DeviceOptions.optionCode.charAt(APCONST.OPTION_CODE_NEW_OPTION_STRUCTURE_USE_BIT) !== '' && $scope.DeviceOptions.optionCode.charAt(APCONST.OPTION_CODE_NEW_OPTION_STRUCTURE_USE_BIT) === '1') {
                            $scope.DeviceOptions.isUsingNewOptionValue = true;
                            // checking whether information button shows on indoor information section.
                            if ($scope.DeviceOptions.optionCode.charAt(8) === '1') {
                                console.log("information button is necessary");

                                $scope.DeviceOptions.isInfoButtonShow = true;

                                if ($scope.DeviceOptions.optionCode.charAt(7) === '1') {
                                    $scope.DeviceOptions.defaultPMSensor = 'PM25';
                                    console.log("defaultPMSensor is PM25");
                                }
                                else {
                                    $scope.DeviceOptions.defaultPMSensor = 'PM10';
                                    console.log("defaultPMSensor is PM10");
                                }
                            }
                            else {
                                console.log("information button is not necessary");
                                $scope.DeviceOptions.isInfoButtonShow = false;
                            }
                        }
			    else {
                             $scope.DeviceOptions.isUsingNewOptionValue = false;
                        }

                        // checking whether turboMode or SpeedMode
                        if($scope.DeviceOptions.optionCode.charAt(15) !== '' && $scope.DeviceOptions.optionCode.charAt(15) === '1') {
                            $scope.turboModeSupported = true;
                        } 
                    }
                    
                    //HAP
                    if (value.indexOf("Humidi_") !== -1) {
                        if( value.indexOf("Humidi_NotSupported") !== -1 || value.indexOf("Humidi_Unknown") !== -1 ){
                            $scope.isHumidiSupport = false;
                        }
                        else{
                            $scope.isHumidiSupport = true;
                        }
                        $scope.DeviceOptions.isHumidiOn = value.indexOf("Humidi_On") !== -1 ? true : false;
                        if (value.indexOf(APCONST.HUMIDIFI_OFF) !== -1) {
                            $scope.desiredHumidityValue = $scope.listOfHumidifiModes[4].modeName;
                        } else {
                            $scope.desiredHumidityValue = getDesiredHumidityValueName($scope.Device.Humidity.desired);
                        }
                        dismissLoadingBar(APCONST.HUMIDITY_COMMAND);
                    }
                    if (value.indexOf("SterilizeClean_") !== -1) {
                        if( value.indexOf("SterilizeClean_NotSupported") !== -1 || value.indexOf("SterilizeClean_Unknown") !== -1 ){
                            $scope.isSterilizeCleanSupprot = false;
                        }
                        else{
                            $scope.isSterilizeCleanSupprot = true;
                        }
                        $scope.DeviceOptions.isSterilizeCleanOn = value.indexOf("SterilizeClean_On") !== -1 ? true : false;
                    }
                    if (value.indexOf("SterilizeCleanRun_") !== -1) {
                        $scope.DeviceOptions.isSterilizeCleanRunOn = value.indexOf("SterilizeCleanRun_On") !== -1 ? true : false;
                        setProgress($scope.DeviceOptions.sterilizeProgress);
                    }
                    if (value.indexOf("Sterilize_") !== -1) {
                        if( value.indexOf("Sterilize_NotSupported") !== -1 || value.indexOf("Sterilize_Unknown") !== -1 ){
                            $scope.isSterilizeSupport = false;
                        }
                        else{
                            $scope.isSterilizeSupport = true;
                        }
                        $scope.DeviceOptions.isSterilizeOn = value.indexOf("Sterilize_On") !== -1 ? true : false;
                    }
                    if (value.indexOf("SterilizeRun_") !== -1) {
                        $scope.DeviceOptions.isSterilizeRunOn = value.indexOf("SterilizeRun_On") !== -1 ? true : false;
                        setProgress($scope.DeviceOptions.sterilizeProgress);
                    }
                    
                    /** note : Please remove while testing and don't commit. Code for testing the sterilize/auto clean progress starts here */
                    // if(value.indexOf("SterilizeRun_Off") !== -1) {
                    //   $scope.DeviceOptions.isSterilizeRunOn = value.indexOf("SterilizeRun_Off") !== -1 ? true : false;
                    // }
                    //
                    // if(value.indexOf("SterilizeCleanRun_Off") !== -1) {
                    //     $scope.DeviceOptions.isSterilizeCleanRunOn = value.indexOf("SterilizeCleanRun_Off") !== -1 ? true : false;
                    // }
                    // if(value.indexOf("SterilizeRun_0") !== -1) {
                    //   $scope.DeviceOptions.isSterilizeRunOn = value.indexOf("SterilizeRun_0") !== -1 ? false : true;
                    // }
                    //
                    // if(value.indexOf("SterilizeCleanRun_0") !== -1) {
                    //     $scope.DeviceOptions.isSterilizeCleanRunOn = value.indexOf("SterilizeCleanRun_0") !== -1 ? false : true;
                    // }
                    /** note : Code for testing the sterilize/auto clean progress ends here */
                    
                    if (value.indexOf("SterilizeProgress_") !== -1) {
                        var progress = parseInt(value.split("_")[1], 0);
                        $scope.DeviceOptions.sterilizeProgress =  progress >= 100 ? 100 : progress;                        
                        setProgress($scope.DeviceOptions.sterilizeProgress);
                    }

                    console.log("humidi : " + $scope.DeviceOptions.isHumidiOn + ", sterilizeClean : " + $scope.DeviceOptions.isSterilizeCleanOn + ", sterilize : " + $scope.DeviceOptions.isSterilizeOn);
                    
                    if (value.indexOf("CleanHistory_") !== -1) {
                        $scope.trendGraphValues = value.split("_")[1].substr(0,6).split("").map(Number).filter(function(x){return x !== 0 && x < 5;}).reverse();
                        updateGraph();
                    }
                });
                dismissLoadingBar(APCONST.WIND_STRENGTH_COMMAND);
            }
	    
	        if (angular.isDefined(device["Light"])) {
                if($scope.DeviceOptions.isDistinguishingMuteLightOption){
                    $scope.DeviceOptions.isLightSupported = true;
                }

                if (angular.isDefined(device["Light"].dimmingLevel)) {
                    $scope.Device.Light.dimmingLevel = device["Light"].dimmingLevel;
                    $scope.DeviceOptions.lighting = $scope.Device.Light.dimmingLevel === 1 ? true : false;
                }
                if (angular.isDefined(device["Light"].lightPower)) {
                    $scope.Device.Light.lightPower = device["Light"].lightPower;
                }
                if (angular.isDefined(device["Light"].maxDimmingLevel)) {
                    $scope.Device.Light.maxDimmingLevel = device["Light"].maxDimmingLevel;
                }
                dismissLoadingBar(APCONST.SETTING_COMMAND);
            }

            if (angular.isDefined(device["Operation"]) && angular.isDefined(device["Operation"].power)) {
                $scope.Device.Operation.power = device["Operation"].power;
                if ($scope.Device.Operation.power === "On") {
                    $scope.powerState = true;
                    //if($scope.DeviceOptions.filterSteps) {
                        $scope.isFilterEnable = true;
                    //}

                } else {
                    $scope.powerState = false;
                    $scope.DeviceOptions.inDoorData = {
                        pm10: {value: -1, purity: -1, purityText: '', colorClass: ''},
                        pm25: {value: -1, purity: -1, purityText: '', colorClass: ''},
                        pm10ultrafine : {value: -1, purity: -1, purityText: '', colorClass: ''},
                        odor: {purity: -1, purityText: '', colorClass: ''},
                        freshness: {purity: -1, purityText: '', colorClass: '', iconClassBig: 'purity-level-off', iconClass: 'purity-level-off'},
                        tempFreshness: {purity: -1, purityText: '', colorClass: '', iconClassBig: 'purity-level-off', iconClass: 'purity-level-off'}
                    };
                    //if($scope.DeviceOptions.filterSteps) {
                        $scope.isFilterEnable = false;

                        if($scope.currentScreen === APCONST.SCREEN_FILTER) {
                            $scope.currentScreen = APCONST.SCREEN_HOME;
                            changeScreen();
                        }
                    //}
                    dismissAllPopups();

                }
                dismissLoadingBar(APCONST.POWERONOFF);
                //Lokesh 26Nov15_2328 -> As per Mr. Kim, since Air purifier not sending list of actions, we've to make separate request to get list of schedules silently.
                getSchedules(true);
            }

            if (device.Sensors && device.Sensors.length) {
                $scope.Device.Sensors = angular.copy(device.Sensors);
                for (var sens = 0; sens < device.Sensors.length; sens++) {
    //                var val0Parsed = parseInt($scope.Device["Sensors"][sens].values[0], 10),
    //                        val1Parsed = $scope.Device["Sensors"][sens].values[1],
                    switch (device.Sensors[sens].id) {
                        case '0':
                            $scope.DeviceOptions.inDoorData.pm10.value = parseInt($scope.Device["Sensors"][sens].values[0], 10);
                            $scope.DeviceOptions.inDoorData.pm10.purity = parseInt($scope.Device["Sensors"][sens].values[1], 10);
                            $scope.DeviceOptions.inDoorData.pm10.purityText = getPurityText($scope.Device["Sensors"][sens].values[1], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.inDoorData.pm10.colorClass = getAirPurityColor($scope.Device["Sensors"][sens].values[1], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.inDoorData.pm10.iconClass = getAirPurityImage($scope.Device["Sensors"][sens].values[1], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            break;
                        case '1':
                            $scope.DeviceOptions.inDoorData.pm25.value = parseInt($scope.Device["Sensors"][sens].values[0], 10);
                            $scope.DeviceOptions.inDoorData.pm25.purity = parseInt($scope.Device["Sensors"][sens].values[1], 10);
                            $scope.DeviceOptions.inDoorData.pm25.purityText = getPurityText($scope.Device["Sensors"][sens].values[1], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.inDoorData.pm25.colorClass = getAirPurityColor($scope.Device["Sensors"][sens].values[1], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.inDoorData.pm25.iconClass = getAirPurityImage($scope.Device["Sensors"][sens].values[1], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            break;
                        case '2':
                            $scope.DeviceOptions.inDoorData.odor.purity = parseInt($scope.Device["Sensors"][sens].values[0], 10);
                            $scope.DeviceOptions.inDoorData.odor.purityText = getPurityText($scope.Device["Sensors"][sens].values[0], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.inDoorData.odor.colorClass = getAirPurityColor($scope.Device["Sensors"][sens].values[0], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            break;
                        case '3':
                            $scope.DeviceOptions.inDoorData.freshness.purity = parseInt($scope.Device["Sensors"][sens].values[0], 10);
                            $scope.DeviceOptions.inDoorData.freshness.purityText = getPurityText($scope.Device["Sensors"][sens].values[0], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.inDoorData.freshness.colorClass = getAirPurityColor($scope.Device["Sensors"][sens].values[0], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.inDoorData.freshness.iconClass = getAirPurityImage($scope.Device["Sensors"][sens].values[0], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.inDoorData.tempFreshness.purity = $scope.DeviceOptions.inDoorData.freshness.purity;
                            $scope.DeviceOptions.inDoorData.tempFreshness.purityText = $scope.DeviceOptions.inDoorData.freshness.purityText;
                            $scope.DeviceOptions.inDoorData.tempFreshness.colorClass = $scope.DeviceOptions.inDoorData.freshness.colorClass;
                            $scope.DeviceOptions.inDoorData.tempFreshness.iconClass = $scope.DeviceOptions.inDoorData.freshness.iconClass;
                            //$scope.DeviceOptions.inDoorData.freshness.iconClassBig = getAirPurityImage($scope.Device["Sensors"][sens].values[0], true, $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            break;
                        case '4':
                            $scope.DeviceOptions.inDoorData.pm10ultrafine.value = parseInt($scope.Device["Sensors"][sens].values[0], 10);
                            $scope.DeviceOptions.inDoorData.pm10ultrafine.purity = parseInt($scope.Device["Sensors"][sens].values[1], 10);
                            $scope.DeviceOptions.inDoorData.pm10ultrafine.purityText = getPurityText($scope.Device["Sensors"][sens].values[1], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.DeviceOptions.inDoorData.pm10ultrafine.colorClass = getAirPurityColor($scope.Device["Sensors"][sens].values[1], $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            break;
                        default:
                            break;

                    }
                }

                if(!$scope.DeviceOptions.ventControl) {
                    if(($scope.DeviceOptions.pm10SupportOnly == true) && ($scope.DeviceOptions.pm25SupportOnly == true))
                    {
                        if($scope.DeviceOptions.deviceCountry === 'KR') {
                            $scope.DeviceOptions.inDoorData.freshness.purity = $scope.DeviceOptions.inDoorData.pm10.purity;
                            $scope.DeviceOptions.inDoorData.freshness.purityText = $scope.DeviceOptions.inDoorData.pm10.purityText;
                            $scope.DeviceOptions.inDoorData.freshness.colorClass = $scope.DeviceOptions.inDoorData.pm10.colorClass;
                            $scope.DeviceOptions.inDoorData.freshness.iconClass = getAirPurityImage($scope.DeviceOptions.inDoorData.pm10.purity, $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            
                            $scope.airTypeString = $scope.translation.ACMOB_VTL_main_PM10;
                            /*if(!!$scope.DeviceOptions.inDoorData.pm10.value) {
                                $scope.airTypeString = $scope.translation.ACMOB_VTL_main_PM10 + " : " + $scope.DeviceOptions.inDoorData.pm10.value + " ㎍/㎥";
                            } else {
                                $scope.airTypeString = $scope.translation.ACMOB_VTL_main_PM10;
                            } */
                        } else {
                            $scope.DeviceOptions.inDoorData.freshness.purity = $scope.DeviceOptions.inDoorData.pm25.purity;
                            $scope.DeviceOptions.inDoorData.freshness.purityText = $scope.DeviceOptions.inDoorData.pm25.purityText;
                            $scope.DeviceOptions.inDoorData.freshness.colorClass = $scope.DeviceOptions.inDoorData.pm25.colorClass;
                            $scope.DeviceOptions.inDoorData.freshness.iconClass = getAirPurityImage($scope.DeviceOptions.inDoorData.pm25.purity, $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            
                            $scope.airTypeString = $scope.translation.ACMOB_VTL_main_sensing_left;
                            /*if(!!$scope.DeviceOptions.inDoorData.pm25.value) {
                                $scope.airTypeString = $scope.translation.ACMOB_VTL_main_sensing_left + " : " + $scope.DeviceOptions.inDoorData.pm25.value + " ㎍/㎥";
                            } else {
                                $scope.airTypeString = $scope.translation.ACMOB_VTL_main_sensing_left;
                            } */
                        }
                    }
                    else if(($scope.DeviceOptions.pm10SupportOnly == false) && ($scope.DeviceOptions.pm25SupportOnly == true))
                    {
                    	console.log("china delux(pm2.5 only)");
                    	$scope.DeviceOptions.inDoorData.freshness.purity = $scope.DeviceOptions.inDoorData.pm25.purity;
                        $scope.DeviceOptions.inDoorData.freshness.purityText = $scope.DeviceOptions.inDoorData.pm25.purityText;
                        $scope.DeviceOptions.inDoorData.freshness.colorClass = $scope.DeviceOptions.inDoorData.pm25.colorClass;
                        $scope.DeviceOptions.inDoorData.freshness.iconClass = getAirPurityImage($scope.DeviceOptions.inDoorData.pm25.purity, $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                        
                        $scope.airTypeString = $scope.translation.ACMOB_VTL_main_sensing_left;
                    }
                }

                if($scope.DeviceOptions.isUsingNewOptionValue){
                    if($scope.DeviceOptions.isInfoButtonShow){
                        // defaultPMSensor var has values only for delux/premium global set
                        if($scope.DeviceOptions.defaultPMSensor === 'PM25'){
                            console.log("set indoorData PM25");
                            $scope.DeviceOptions.inDoorData.freshness.purity = $scope.DeviceOptions.inDoorData.pm25.purity;
                            $scope.DeviceOptions.inDoorData.freshness.purityText = $scope.DeviceOptions.inDoorData.pm25.purityText;
                            $scope.DeviceOptions.inDoorData.freshness.colorClass = $scope.DeviceOptions.inDoorData.pm25.colorClass;
                            $scope.DeviceOptions.inDoorData.freshness.iconClass = getAirPurityImage($scope.DeviceOptions.inDoorData.pm25.purity, $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.airTypeString = $scope.translation.ACMOB_VTL_main_sensing_left;
                        }
                        else if($scope.DeviceOptions.defaultPMSensor === 'PM10'){
                            console.log("set indoorData PM10");
                            $scope.DeviceOptions.inDoorData.freshness.purity = $scope.DeviceOptions.inDoorData.pm10.purity;
                            $scope.DeviceOptions.inDoorData.freshness.purityText = $scope.DeviceOptions.inDoorData.pm10.purityText;
                            $scope.DeviceOptions.inDoorData.freshness.colorClass = $scope.DeviceOptions.inDoorData.pm10.colorClass;
                            $scope.DeviceOptions.inDoorData.freshness.iconClass = getAirPurityImage($scope.DeviceOptions.inDoorData.pm10.purity, $scope.DeviceOptions.deviceCountry, $scope.DeviceOptions.chinaAirSteps);
                            $scope.airTypeString = $scope.translation.ACMOB_VTL_main_PM10;
                        }
                    }
                    else{
                        $scope.DeviceOptions.inDoorData.freshness.purity = $scope.DeviceOptions.inDoorData.tempFreshness.purity;
                        $scope.DeviceOptions.inDoorData.freshness.purityText = $scope.DeviceOptions.inDoorData.tempFreshness.purityText;
                        $scope.DeviceOptions.inDoorData.freshness.colorClass = $scope.DeviceOptions.inDoorData.tempFreshness.colorClass;
                        $scope.DeviceOptions.inDoorData.freshness.iconClass = $scope.DeviceOptions.inDoorData.tempFreshness.iconClass;
                        $scope.airTypeString = "";
                    }
                }
                /* test log */
                console.log('pm10:' + $scope.DeviceOptions.pm10SupportOnly + ', pm25:' + $scope.DeviceOptions.pm25SupportOnly + ', newOptions:' + $scope.DeviceOptions.isUsingNewOptionValue +
                            ', info button show:' + $scope.DeviceOptions.isInfoButtonShow + ', default PM : ' + $scope.DeviceOptions.defaultPMSensor +
                            ', Mute/Light Distinguish Model : ' + $scope.DeviceOptions.isDistinguishingMuteLightOption);
                /* test log */
                if(isIndoorpopup) {
                    isIndoorpopup = false;
                    $scope.openPopoverForImage('indoor');
                    dismissLoadingBar("SENSORS");
                }
                updateRecommondationText();
            }

            if (angular.isDefined(device["Wind"])) {
                if (angular.isDefined(device["Wind"].direction)) {
                    $scope.Device.Wind.direction = device["Wind"].direction;

                    if (device["Wind"].direction === 'Off') {
                        $scope.DeviceOptions.three_d = false;
                        $scope.DeviceOptions.wide = false;
                    }
                    if (device["Wind"].direction === 'Center') {
                        $scope.DeviceOptions.three_d = true;
                        $scope.DeviceOptions.wide = false;
                    }
                    if (device["Wind"].direction === 'Wide') {
                        $scope.DeviceOptions.three_d = false;
                        $scope.DeviceOptions.wide = true;
                    }
                    dismissLoadingBar(APCONST.WIND_STRENGTH_COMMAND);
                }
                if (angular.isDefined(device["Wind"].speedLevel)) {
                    $scope.Device.Wind.speedLevel = device["Wind"].speedLevel;

                    switch (device["Wind"].speedLevel) {
                        case APCONST.WIND_SPEEDLEVEL_AUTO:
                            $scope.selectedMode = $scope.listOfModes[0].modeName;
                            $scope.selectedModeIcon = $scope.listOfModes[0].imageID;
                            break;
                        case APCONST.WIND_SPEEDLEVEL_LOW:
                            $scope.selectedMode = $scope.listOfModes[3].modeName;
                            $scope.selectedModeIcon = $scope.listOfModes[3].imageID;
                            break;
                        case APCONST.WIND_SPEEDLEVEL_MEDIUM:
                            $scope.selectedMode = $scope.listOfModes[2].modeName;
                            $scope.selectedModeIcon = $scope.listOfModes[2].imageID;
                            break;
                        case APCONST.WIND_SPEEDLEVEL_HIGH:
                            $scope.selectedMode = $scope.listOfModes[1].modeName;
                            $scope.selectedModeIcon = $scope.listOfModes[1].imageID;
                            break;
                        case APCONST.WIND_SPEEDLEVEL_SLEEP:
                            $scope.selectedMode = $scope.listOfModes[4].modeName;
                            $scope.selectedModeIcon = $scope.listOfModes[4].imageID;
                            if($scope.showVentSelectionPopup) {
                                hideVentPopUp();
                            }
                            break;
                        // note : do not handle not defined variable. it might work in an expected way.
                        default:
                            /*$scope.selectedMode = $scope.listOfModes[0].modeName;
                            $scope.selectedModeIcon = $scope.listOfModes[0].imageID;*/
                            break;
                    }

                }
                dismissLoadingBar(APCONST.MODECHANGE);
            }

            if (angular.isDefined(device["description"])) {
                $scope.Device.description = device["description"];
            }
            if (angular.isDefined(device["connected"])) {
                $scope.Device.connected = device["connected"];
            }
            if (angular.isDefined(device["id"])) {
                $scope.Device.id = device["id"];
            }
            if (angular.isDefined(device["name"])) {
                $scope.Device.name = device["name"];
            }
            if (angular.isDefined(device["resources"])) {
                $scope.Device.resources = device["resources"];
                if($scope.Device.resources.indexOf("Humidity") !== -1) {
                    $scope.isHumidifierAirPurifier = true;
                    $scope.isAirPurifier = false;
                }
            }
            if (angular.isDefined(device["type"])) {
                $scope.Device.type = device["type"];
            }
            if (angular.isDefined(device["uuid"])) {
                $scope.Device.uuid = device["uuid"];
            }

            if (!$scope.powerState || $scope.Device.Wind.speedLevel === APCONST.WIND_SPEEDLEVEL_SLEEP) {
                $scope.DeviceOptions.inDoorData = {
                    pm10: {value: -1, purity: -1, purityText: '', colorClass: ''},
                    pm25: {value: -1, purity: -1, purityText: '', colorClass: ''},
                    pm10ultrafine : {value: -1, purity: -1, purityText: '', colorClass: ''},
                    odor: {purity: -1, purityText: '', colorClass: ''},
                    freshness: {purity: -1, purityText: '', colorClass: '', iconClassBig: 'purity-level-off', iconClass: 'purity-level-off'},
                    tempFreshness: {purity: -1, purityText: '', colorClass: '', iconClassBig: 'purity-level-off', iconClass: 'purity-level-off'}
                };
                //popUpOkApHome();
                /*if ($scope.Device.Wind.speedLevel === APCONST.WIND_SPEEDLEVEL_SLEEP) {
                    dismissAllPopups(true); 
                } else {
                   dismissAllPopups(false); 
                }*/
            }
            //$scope.mainLoadingScreen = true;     
            if ($scope.isStatic && $scope.checkResponse.indexOf(APCONST.GET_DEVICES) !== -1) { // For try it now
                getWeatherLocationList();
                startApp();
            }

            if(!diagnosisStarted) {
                $scope.loadingBar = false;
            }

        };
        
         /**    	 
         * @function
         * @private
    	 * @name addInAppAlarm       
    	 * @description To add the alarm data in alarm list to show InApp alarm.
         */
        function addInAppAlarm(alarms) {
            alarms.forEach(function (value) {
                if (value.code === "FilterAlarm" || value.code === "WaterSupplement" || value.code === "WaterReplacing") {
                    switch (value.code) {
                        case "FilterAlarm":
                            alarmList.push({'type': APCONST.FILTER_ALARM, 'text': $scope.translation.WEBMOB_device_ap_alarm_exchange_filter, 'icon' : "AP/assets/img/main_rf_ic_noti_filter.svg#Layer_1"});
                            break;
                        case "WaterSupplement":
                            $scope.isWaterError = true;
                            $scope.waterAlarmText = $scope.translation.WEBMOB_device_ap_alarm_fill_water;
                            $scope.waterAlarmIcon = "AP/assets/img/main_rf_ic_noti_fill_water.svg#Layer_1";
                            $scope.waterAlarmDesc = $scope.translation.WEBMOB_device_ap_unavailable_water_supplement

                            break;
                        case "WaterReplacing":
                            $scope.isWaterError = true;
                            $scope.waterAlarmText = $scope.translation.WEBMOB_device_ap_alarm_replace_water;
                            $scope.waterAlarmIcon = "AP/assets/img/main_ac_ic_water_replace.svg#Layer_1";
                            $scope.waterAlarmDesc = $scope.translation.WEBMOB_device_ap_unavailable_water_replacement;
                            break;
                        default:
                            break;
                    }
                } else if (value.code === "FilterAlarm_OFF") {
                    $scope.isFilterStatusShow = false;
                    $scope.inAppPanelAlarmCategory = 0;
                } else if(value.code === "WaterSupplement_OFF" && value.code === "WaterReplacing_OFF") {
                    $scope.isWaterError = false;
                }
            });
        }

         /**    	 
         * @function
         * @private
    	 * @name deleteInAppAlarm       
    	 * @description To delete alarm data from alarmList.
         */
        function deleteInAppAlarm(alarms) {
            console.log("deleteInAppAlarm", alarms);
            alarms.forEach(function (value) {
                var temp = [];
                if(value.code !== "FilterAlarm") {
                    $scope.isWaterError = false;
                    $scope.waterAlarmDesc = '';
                    $scope.waterAlarmIcon = '';
                    $scope.waterAlarmText = '';
                } else {
                    for (var i = 0; i < alarmList.length; i++) {
                        //Prepare latest alarm list with updated value
                        if (value.id != alarmList[i].type) {
                            temp.push(alarmList[i]);
                        } 
                    }
                    alarmList = temp;
                    showInAppPopup();
            }
            });
        }

        /**    	 
         * @function
         * @private
    	 * @name parseStaticData       
    	 * @description To Show InApp popup
    	 * To Show InApp popup with the data from alarmList
         **/
        function showInAppPopup() {
            //check if there exist atleast one error to show
            if (alarmList.length > 0) {
                //sorting to show water alarms first
                alarmList.sort(function (a, b) {
                    return(b.type - a.type);
                });
                $scope.isFilterStatusShow = true;
                var alarm = alarmList[0];
                $scope.inAppPanelAlarmCategory = alarm.type;
                $scope.inAppPanelPopupText = alarm.text;
                $scope.inAppPanelPopupIcon = alarm.icon;
                
                var svg = document.getElementById('inAppPopIcon');
                $timeout(function(){SHPService.loadSVG($scope.inAppPanelPopupIcon, svg)},100);
                
            } else {
                $scope.isFilterStatusShow = false;
                $scope.inAppPanelAlarmCategory = 0;
            }
        }


        /**    	 
         * @function
         * @private
    	 * @name parseStaticData       
    	 * @description To parse static data.
    	 * To run the application in static mode in browser, add ng-init="parseStaticData" to body tag in html
         */
        function parseStaticData() {
            $scope.isStatic = true;
            init();
            var response;
            if ($scope.isAirPurifier) {
                response = '{"Device":{"resources":["Alarms","Configuration","Diagnosis","Information","Light","Mode","Operation","Sensors","Wind"],"Operation":{"power":"On"},"Sensors":[{"type":"Dust","id":"0","values":[9,1]},{"type":"FineDust","id":"1","values":[9,1]},{"type":"Odor","id":"2","values":[1]},{"type":"CleanLevel","id":"3","values":[1]},{"id":"4","type":"UltraFineDust","values":[2,4]}],"connected":true,"ConfigurationLink":{"href":"/devices/0/configuration"},"type":"Air_Purifier","Mode":{"options":["Comode_Off","Blooming_6","Function_0","Volume_66","Spi_Off","OptionCode_16703","FilterCleanAlarm_0","CleanHistory_312433"]},"InformationLink":{"href":"/devices/0/information"},"Consumables":[{"state":"1","id":"0","name":"FilterProgress"}],"id":"0","Diagnosis":{"diagnosisStart":"Ready"},"description":"TP6X_VTL_16K","name":"AIRPURIFIER","Alarms":[{"alarmType":"Device","id":"0","triggeredTime":"2015-11-10T03:31:55","code":"ErrorCode_OFF"},{"alarmType":"Device","id":"1","triggeredTime":"2015-11-10T03:31:55","code":"FilterAlarm_OFF"}],"Light":{"dimmingLevel":1,"maxDimmingLevel":1},"uuid":"F8042EC5-A977-0000-0000-000000000000","Wind":{"speedLevel":2,"direction":"Center"}}}';
            } else {
                response = '{"Device":{"description":"TP6X_HVTL_17K","id":"0","Alarms":[{"id":"0","alarmType":"Device","code":"ErrorCode_OFF","triggeredTime":"2016-09-01T13:27:10"},{"id":"1","alarmType":"Device","code":"FilterAlarm_OFF","triggeredTime":"2016-09-01T13:27:10"},{"id":"2","alarmType":"Device","code":"WaterSupplement_OFF","triggeredTime":"2016-09-01T13:27:10"},{"id":"3","alarmType":"Device","code":"WaterReplacing_OFF","triggeredTime":"2016-09-01T13:27:10"}],"ConfigurationLink":{"href":"/devices/0/configuration"},"Consumables":[{"id":"0","name":"FilterProgress","state":"3"}],"Diagnosis":{"diagnosisStart":"Ready"},"Humidity":{"current":40,"desired":60,"unit":"Percentage"},"InformationLink":{"href":"/devices/0/information"},"Light":{"dimmingLevel":0,"maxDimmingLevel":1},"Mode":{"options":["Comode_Off","Blooming_4","Humidi_On","Sterilize_On","SterilizeClean_Off","Volume_100","Spi_Off","OptionCode_16703","FilterCleanAlarm_0","SterilizeProgress_25","SterilizeRun_Off","SterilizeCleanRun_Off"]},"Operation":{"power":"On"},"Sensors":[{"id":"0","type":"Dust","values":[9,1]},{"id":"1","type":"FineDust","values":[9,1]},{"id":"2","type":"Odor","values":[1]},{"id":"3","type":"CleanLevel","values":[1]}],"Wind":{"direction":"Off","speedLevel":0},"connected":true,"name":"AIRPURIFIER","resources":["Alarms","Configuration","Diagnosis","Humidity","Information","Light","Mode","Operation","Sensors","Wind"],"type":"Air_Purifier","uuid":"F8042EC5-A977-0000-0000-000000000000"}}';
            }
            var device = JSON.parse(response);
            $scope.checkResponse.push(APCONST.GET_DEVICES);
            $scope.parseDeviceData(device.Device);
            $scope.isStatic = true;
            $scope.inAppPanelPopupText = $scope.translation.WEBMOB_device_ap_alarm_exchange_filter;
            $scope.inAppPanelPopupIcon = "AP/assets/img/main_rf_ic_noti_filter.svg#Layer_1";
            $scope.isFilterStatusShow = true;            
        }

        if ($scope.isStatic === true && !$scope.deviceUuid) {
            $scope.parseStaticData();
        }
    
        /**    	 
         * @function
         * @private
    	 * @name onBackPressed        
    	 * @description On back pressed handling of any screen
    	 */
        function onBackPressed() {
            $timeout(function () {
                $scope.airInfo.isCAI = false;   //Lokesh 25Nov15 -> Used to identify long text in title. Should be disabled when coming out of that screen.
                if($scope.countrySelectionPopupVisible) {
                    $scope.countrySelectionPopupVisible = false;
                    return;
                }
                if ($scope.diagnosisStart === "Run") {
                    return;
                }
                if ($scope.showModeSelectionPopup) {
                    $scope.showModeSelectionPopup = false;
                    toggleBottomPopUpClass($scope.showModeSelectionPopup);
                    return;
                }
                if ($scope.showVentSelectionPopup) {
                    $scope.showVentSelectionPopup = false;
                    toggleBottomPopUpClass($scope.showVentSelectionPopup);
                    return;
                }
                if ($scope.showPurityPopup) {
                    $scope.showPurityPopup = false;
                    togglePurityPopUpClass($scope.showPurityPopup);
                    return;
                }
                if ($scope.showVolumePopup) {
                    $scope.showVolumePopup = false;
                    toggleVolumePopUpClass();
                    return;
                }
                if ($scope.dialogPopup.show) {
                    $scope.dialogPopup.show = false;
                    toggleBottomPopUpClass($scope.dialogPopup.show);
                    return;
                }
                if ($scope.bUsageHelpPage) {
                    closeUsageHelpPage();
                    return;
                }
                if ($scope.showHumidfiSettingPopup) {
                    closeHumidifiSettingPopup();
                    return;
                }
                if ($scope.bTurnOnOffPopup) {
                    closeTurnOnOffPopup();
                    return;
                }
                
                switch ($scope.currentScreen) {
                    case APCONST.SCREEN_HOME:
                        closeApp();
                        break;
                    case APCONST.SCREEN_FILTER:
                        $scope.currentScreen = APCONST.SCREEN_SETTINGS;
                        changeScreen();
                        break;
                    case APCONST.SCREEN_AIRFRESHNESSINFOSTATUS:
                        $scope.currentScreen = APCONST.SCREEN_AIRFRESHNESSINFO;
                        changeScreen();
                        break;
                    case APCONST.SCREEN_WINDSTRENGTH:
                        $scope.currentScreen = APCONST.SCREEN_HOME;
                        changeScreen();
                        break;
                    case APCONST.SCREEN_SETTINGS:
                        $scope.currentScreen = APCONST.SCREEN_HOME;
                        changeScreen();
                        break;
                    case APCONST.SCREEN_SCHEDULER:
                        $scope.currentScreen = APCONST.SCREEN_HOME;
                        changeScreen();
                        break;
                    case APCONST.SCREEN_ADDSCHEDULE:
                        if($scope.Actions.length !== 0) {
                            $scope.scheduleList();
                        } else {
                           $scope.currentScreen = APCONST.SCREEN_HOME;
                            changeScreen(); 
                        }
                        break;
                    case APCONST.SCREEN_HELP:
                        $scope.currentScreen = APCONST.SCREEN_HOME;
                        changeScreen();
                        break;
                    case APCONST.SCREEN_RECOMMENDTINFO:
                    case APCONST.SCREEN_TREND_INFO:
                        $scope.currentScreen = APCONST.SCREEN_HELP;
                        changeScreen();
                        break;
                    case APCONST.SCREEN_AIRINFO:
                        if (firstTimeLaunch) {
                            $scope.currentScreen = APCONST.SCREEN_HOME;
                            changeScreen();
                            $scope.locationToBeSearched = "";
                            $scope.searchValues = mOrginialValues;
                        } else {
                            $scope.currentScreen = APCONST.SCREEN_SETTINGS;
                            changeScreen();
                            $scope.locationToBeSearched = "";
                            $scope.searchValues = mOrginialValues;
                        }
                        break;
                    case APCONST.SCREEN_AIRFRESHNESSINFO:
                        $scope.currentScreen = APCONST.SCREEN_HELP;
                        changeScreen();
                        break;
                    case APCONST.SCREEN_SUPPLY_STORE:
                    case APCONST.SCREENS_SMART_CARE:
                        $scope.currentScreen = APCONST.SCREEN_ABOUT;
                        changeScreen();
                        break;
                    case APCONST.SMARTCAREERROR:
                        $scope.currentScreen = APCONST.SCREEN_HOME;
                        changeScreen();
                        break;                    
                    case APCONST.SCREEN_VENTILATION_RECOMMEND:
                    case APCONST.SCREEN_OUTDOOR_PURITY_WARNING:
                    case APCONST.SCREEN_YELLOW_DUST_WARNING:
                        $scope.currentScreen = APCONST.SCREEN_RECOMMENDTINFO;
                        changeScreen();
                        break;                    
                    default:
                        $scope.currentScreen = APCONST.SCREEN_HOME;
                        changeScreen();
                        break;
                }
            }, APCONST.POWER_ANIMTION_TIMEOUT);
        };

        /**    	 
         * @function
         * @private
    	 * @name openPopoverVentSelection     
         * @description To display vent selection popup
    	 */
        function openPopoverVentSelection() {
            $timeout(function () {
              $scope.showVentSelectionPopup = true;
                toggleBottomPopUpClass($scope.showVentSelectionPopup);
                $scope.tempVentLevel = $scope.DeviceOptions.currentVentLevel;
                decodeVentLevel($scope.DeviceOptions.currentVentLevel);
                updateVentImage($scope.DeviceOptions.currentVentLevel);
                
            }, APCONST.LIST_ANIMATION_TIME);
        };
        
        /**    	 
         * @function
         * @private
    	 * @name flipVentAnimation
         * @param {Boolean} popupName popup open or close     
         * @description To animate vent selection popup
    	 */
        function flipVentAnimation(popupName) {
            var OkCanceldialogPopup = document.getElementById("ventSelection");
            if (popupName) {
                OkCanceldialogPopup.style.display = "block";
                $scope.BottomPopupDivCls = "ventBgIn";

            } else {
                $scope.BottomPopupDivCls = "ventBgOut";
                setTimeout(function () {
                    OkCanceldialogPopup.style.display = "none";
                }, 300);
            }
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openWindStrengthAirPurifier     
         * @description To display wind strength screen
        */
        $scope.openWindStrengthAirPurifier = function () {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_WINDSTRENGTH;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        };
        
        /**    	 
         * @function
         * @private
    	 * @name openWindStrengthAirPurifier     
         * @description To display wind strength screen
        */
        function openAPSettings() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_SETTINGS;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openAPHelp     
         * @description To display help screen
        */
        function openAPHelp() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_HELP;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openAPAbout     
         * @description To display about screen
        */
        function openAPAbout() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_ABOUT;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openAPAirInfo     
         * @description To display air purity popup
         *              Reset location search input box value.
         *              Reset region list to original values.
    	 */
        function openAPAirInfo() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                showRegionListScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openAPFilterScreen     
         * @description To display about screen
        */
        function openAPFilterScreen() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_FILTER;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name updateFilterLayerStyle     
         * @description Update filter progress bar based on device type.
        */
        function updateFilterLayerStyle(element) {
            if($scope.DeviceOptions.filterSteps) {
                var valState = parseInt($scope.Device.Consumables[0].state,10);
                if(valState > 4) {
                    $scope.isFilterEnable = false;
                } else {
                    filterLayerStyleNew(element);
                }
            } else {
                filterLayerStyle(element);
            }
        }
            
        /**    	 
         * @function
         * @private
    	 * @name openAPAirFreshnessStatus     
         * @description To display ai fressness screen of help section
        */
        function openAPAirFreshnessStatus() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_AIRFRESHNESSINFO;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openAPAirFreshnessLevelStatus     
         * @description To display air pfressness level screen of help section
        */
        function openAPAirFreshnessLevelStatus(AirInfoType) {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                switch (AirInfoType) {
                    case 'CAI':
                        $scope.airInfo.desc1 = $scope.translation.ACMOB_VTL_help_cai_A;
                        $scope.airInfo.desc2 = $scope.translation.ACMOB_VTL_help_cai_B;
                        $scope.airInfo.desc3 = $scope.translation.ACMOB_VTL_help_cai_C;
                        $scope.airInfo.desc4 = $scope.translation.ACMOB_VTL_help_cai_DE;
                        $scope.airInfo.gradeA = $scope.translation.ACMOB_VTL_help_cai_grade_A;
                        $scope.airInfo.gradeB = $scope.translation.ACMOB_VTL_help_cai_grade_B;
                        $scope.airInfo.gradeC = $scope.translation.ACMOB_VTL_help_cai_grade_C;
                        $scope.airInfo.gradeD = $scope.translation.ACMOB_VTL_help_cai_grade_DE;
                        $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_outdoor_info;
                        $scope.airInfo.isCAI = true;
                        $scope.airInfo.isAQI = false;
                        break;
                    case 'PM25':
                        if($scope.selectedCountry === 'KR') {
                            $scope.airInfo.desc1 = $scope.translation.ACMOB_VTL_help_pm10_descripaion_1;
                            $scope.airInfo.desc2 = $scope.translation.ACMOB_VTL_help_pm10_descripaion_2;
                            $scope.airInfo.gradeA = $scope.translation.ACMOB_VTL_help_pm25_grade_A;
                            $scope.airInfo.gradeB = $scope.translation.ACMOB_VTL_help_pm25_grade_B;
                            $scope.airInfo.gradeC = $scope.translation.ACMOB_VTL_help_pm25_grade_C;
                            $scope.airInfo.gradeD = $scope.translation.ACMOB_VTL_help_pm25_grade_DE;
                        } else if($scope.selectedCountry === 'CN') {
                            $scope.airInfo.desc1 = $scope.translation.ACMOB_VTL_help_pm25_descripaion_1;
                            $scope.airInfo.desc2 = $scope.translation.ACMOB_VTL_help_pm25_descripaion_2;
                        }
                      
                        $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_main_sensing_left;                  
                        $scope.airInfo.isCAI = false;
                        $scope.airInfo.isAQI = false;
                        break;
                    case 'PM10':
                        $scope.airInfo.desc1 = $scope.translation.ACMOB_VTL_help_pm10_descripaion_1;
                        $scope.airInfo.desc2 = $scope.translation.ACMOB_VTL_help_pm10_descripaion_2;
                        $scope.airInfo.gradeA = $scope.translation.ACMOB_VTL_help_pm10_grade_A;
                        $scope.airInfo.gradeB = $scope.translation.ACMOB_VTL_help_pm10_grade_B;
                        $scope.airInfo.gradeC = $scope.translation.ACMOB_VTL_help_pm10_grade_C;
                        $scope.airInfo.gradeD = $scope.translation.ACMOB_VTL_help_pm10_grade_DE;
                        $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_ultrafine;
                        $scope.airInfo.isCAI = false;
                        $scope.airInfo.isAQI = false;
                        break;
                    case 'O3':
                        $scope.airInfo.desc1 = $scope.translation.ACMOB_VTL_help_O3_descripaion_1;
                        $scope.airInfo.desc2 = $scope.translation.ACMOB_VTL_help_O3_descripaion_2;
                        $scope.airInfo.gradeA = $scope.translation.ACMOB_VTL_help_O3_grade_A;
                        $scope.airInfo.gradeB = $scope.translation.ACMOB_VTL_help_O3_grade_B;
                        $scope.airInfo.gradeC = $scope.translation.ACMOB_VTL_help_O3_grade_C;
                        $scope.airInfo.gradeD = $scope.translation.ACMOB_VTL_help_O3_grade_DE;
                        $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_O3;
                        $scope.airInfo.isCAI = false;
                        $scope.airInfo.isAQI = false;
                        break;
                    case 'N2O2':
                        $scope.airInfo.desc1 = $scope.translation.ACMOB_VTL_help_N2O2_descripaion_1;
                        $scope.airInfo.desc2 = $scope.translation.ACMOB_VTL_help_N2O2_descripaion_2;
                        $scope.airInfo.gradeA = $scope.translation.ACMOB_VTL_help_N2O2_grade_A;
                        $scope.airInfo.gradeB = $scope.translation.ACMOB_VTL_help_N2O2_grade_B;
                        $scope.airInfo.gradeC = $scope.translation.ACMOB_VTL_help_N2O2_grade_C;
                        $scope.airInfo.gradeD = $scope.translation.ACMOB_VTL_help_N2O2_grade_DE;
                        $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_N2O;
                        $scope.airInfo.isCAI = false;
                        $scope.airInfo.isAQI = false;
                        break;
                    case 'CO':
                        $scope.airInfo.desc1 = $scope.translation.ACMOB_VTL_help_CO2_descripaion_1;
                        $scope.airInfo.desc2 = $scope.translation.ACMOB_VTL_help_CO2_descripaion_2;
                        $scope.airInfo.gradeA = $scope.translation.ACMOB_VTL_help_CO2_grade_A;
                        $scope.airInfo.gradeB = $scope.translation.ACMOB_VTL_help_CO2_grade_B;
                        $scope.airInfo.gradeC = $scope.translation.ACMOB_VTL_help_CO2_grade_C;
                        $scope.airInfo.gradeD = $scope.translation.ACMOB_VTL_help_CO2_grade_DE;
                        $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_CO;
                        $scope.airInfo.isCAI = false;
                        $scope.airInfo.isAQI = false;
                        break;
                    case 'SO2':
                        $scope.airInfo.desc1 = $scope.translation.ACMOB_VTL_help_SO2_descripaion_1;
                        $scope.airInfo.desc2 = $scope.translation.ACMOB_VTL_help_SO2_descripaion_2;
                        $scope.airInfo.gradeA = $scope.translation.ACMOB_VTL_help_SO2_grade_A;
                        $scope.airInfo.gradeB = $scope.translation.ACMOB_VTL_help_SO2_grade_B;
                        $scope.airInfo.gradeC = $scope.translation.ACMOB_VTL_help_SO2_grade_C;
                        $scope.airInfo.gradeD = $scope.translation.ACMOB_VTL_help_SO2_grade_DE;
                        $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_SO2;
                        $scope.airInfo.isCAI = false;
                        $scope.airInfo.isAQI = false;
                        break;
                    case 'AQI':
                        $scope.airInfo.desc1 = $scope.translation.ACMOB_VTL_help_SO2_descripaion_1;
                        $scope.airInfo.desc2 = $scope.translation.ACMOB_VTL_help_SO2_descripaion_2;
                        $scope.airInfo.gradeA = $scope.translation.ACMOB_VTL_help_SO2_grade_A;
                        $scope.airInfo.gradeB = $scope.translation.ACMOB_VTL_help_SO2_grade_B;
                        $scope.airInfo.gradeC = $scope.translation.ACMOB_VTL_help_SO2_grade_C;
                        $scope.airInfo.gradeD = $scope.translation.ACMOB_VTL_help_SO2_grade_DE;
                        $scope.subHeaderTitle = $scope.translation.ACMOB_VTL_outdoor_info;
                        $scope.airInfo.isCAI = false;
                        $scope.airInfo.isAQI = true;
                        break;
                    default:
                        break;
                }
                $scope.currentScreen = APCONST.SCREEN_AIRFRESHNESSINFOSTATUS;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openAPRecommendInfo     
         * @description To display air recommondation screen of help section
        */
        function openAPRecommendInfo() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_RECOMMENDTINFO;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        
        /**    	 
         * @function
         * @private
    	 * @name openSupplyStoreScreen     
         * @description To display air supply store screen of about section
        */
        function openSupplyStoreScreen() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_SUPPLY_STORE;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        };
        
        /**    	 
         * @function
         * @private
    	 * @name openAPTrendInfo     
         * @description To display trend info screen of help section
        */
        function openAPTrendInfo() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_TREND_INFO;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openAPVentilationRecommend     
         * @description To display ventilation recommend screen of recommendation section
        */
        function openAPVentilationRecommend() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_VENTILATION_RECOMMEND;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openAPOutdoorPurityWarning     
         * @description To display outdoor purity warning screen of recommendation section
        */
        function openAPOutdoorPurityWarning() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_OUTDOOR_PURITY_WARNING;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
         * @function
         * @private
    	 * @name openAPYellowDustWarning     
         * @description To display yellow dust warning screen of recommendation section
        */
        function openAPYellowDustWarning() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_YELLOW_DUST_WARNING;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }

        /**    	 
         * @function
         * @private
    	 * @name selectMode      
         * @param {Object} selectedItem
         *       Mode object
    	 * @description To send mode request
         */
        function selectMode(selectedItem) {
            $scope.showModeSelectionPopup = false;
            changePopUpClass();
            if ($scope.selectedMode !== selectedItem.modeName || ($scope.DeviceOptions.turbo && selectedItem.id === 0)) {                
                if (!$scope.isStatic) {
                    var payload = '{"Device":{"Wind":{"speedLevel":' + selectedItem.id + '},"Mode":{"options":["Comode_Off"]}}}';

                    // todo : define 5 for speed mode as wind strength for humidifier device only
                    //if(selectedItem.id === 5){   // speed mode for humidiifer device only
                    //    payload = '{"Device": {"Mode": {"options": ["Comode_Speed"]},  "Wind": {"direction": "Off"}}}';
                    //}
                    $scope.checkResponse.push(APCONST.MODECHANGE);
                    SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0");
                } else {
                    $scope.selectedMode = selectedItem.modeName;
                    $scope.selectedModeIcon = selectedItem.imageID;
                }
            }
        }
        
        /**    	 
         * @function
         * @private
    	 * @name isModeSelected       
         * @param {Object} selectedItem
         *       Mode object
    	 * @description To check passed mode item is selected or not
         */
        function isModeSelected(selectedItem) {
            if ($scope.selectedMode === selectedItem.modeName) {
                return true;
            }
            else {
                return false;
            }
        }

        /**    	 
         * @function
         * @private
    	 * @name openPopoverMode        
         * @description To display mode selection popup
    	 */
        function openPopoverMode () {
            if ($scope.powerState) {
                $scope.showModeSelectionPopup = true;
                changePopUpClass();
            }
        }

        /**    	 
         * @function
         * @private
    	 * @name closePopoverMode        
         * @description To close mode selection popup
    	*/
        function closePopoverMode() {
            $scope.showModeSelectionPopup = false;
            changePopUpClass();
        };
        
         /**    	 
         * @function
         * @private
    	 * @name selectHumidiValue      
         * @param {Object} selectedItem
         *       Mode object
    	 * @description To set humdification setting
         */
        function selectHumidityValue(selectedItem) {
            closeHumidifiSettingPopup();
            if ($scope.desiredHumidityValue !== selectedItem.modeName) {
                if (!$scope.isStatic) {
                    $scope.checkResponse.push(APCONST.HUMIDITY_COMMAND);
                     if(selectedItem.value === -1) { // for auto
                        console.log("High Selected");
                        var payload = '{"Device":{"Mode":{"options":["Humidi_On"]}}}';
//                        var payload = '{"Device":{"Humidity":{"desired":"Humidi_On"},"Mode":{"options":["Humidi_On"]}}}';
                        SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0");
                     }
                     else if (selectedItem.value !== 0) { // for 40/50/60%
                         var payload = '';
                         if($scope.DeviceOptions.isHumidiOn){
                             payload = '{"Device":{"Humidity":{"desired":' + selectedItem.value + '}}}';
                         }
                         else{
                             payload = '{"Device":{"Humidity":{"desired":' + selectedItem.value + '},"Mode":{"options":["Humidi_On"]}}}';
                         }

                         SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0");
                     }
                     else { // for Off
                        var payload = '{"Device":{"Mode":{"options":["Humidi_Off"]}}}';
                        SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0");
                    }
                } else {
                    $scope.desiredHumidityValue = selectedItem.modeName;
                }
            }
        }
                
        /**    	 
         * @function
         * @private
    	 * @name openHumidifiSettingPopup        
         * @description To display humidify mode selection popup
    	 */
        function openHumidifiSettingPopup () {
            $scope.showHumidfiSettingPopup = true;
            toggleBottomPopUpClass($scope.showHumidfiSettingPopup);
        }

        /**    	 
         * @function
         * @private
    	 * @name closeHumidifiSettingPopup        
         * @description To close humidify mode selection popup
    	*/
        function closeHumidifiSettingPopup() {
            $scope.showHumidfiSettingPopup = false;
            toggleBottomPopUpClass($scope.showHumidfiSettingPopup);
        };
        
        /**    	 
         * @function
         * @private
    	 * @name onPowerOffUpdate        
         * @description
    	*/
        function onPowerOffUpdate(powerOff) {
            $scope.dialogPopup.show = false;
            toggleBottomPopUpClass($scope.dialogPopup.show);
            if (powerOff) {

                powerOnOff();
            } else {
                $scope.showVentSelectionPopup = true;
                decodeVentLevel($scope.tempVentLevel);
                updateVentImage($scope.tempVentLevel);
                toggleBottomPopUpClass(true);
            }
        };
                
        /**    	 
         * @function
         * @private
    	 * @name scheduleList     
         * @description To display ai fressness screen of help section
        */
        function scheduleList(flag) {
            //if no schedule available, open add schedule screen
            if($scope.Actions.length === 0 && $scope.currentScreen === APCONST.SCREEN_HOME) {
                addSchedule(-1);
                return;
            }
            
            if(!flag) {
                toggleAnimClass();
                $timeout(function () {
                    toggleAnimClass();
                    getSchedules();
                    $scope.currentScreen = APCONST.SCREEN_SCHEDULER;
                    changeScreen();
                }, APCONST.LIST_ANIMATION_TIME);
            } else {
                getSchedules();
                $scope.currentScreen = APCONST.SCREEN_SCHEDULER;
                changeScreen();
            }
        }
        
        /**    	 
         * @function
         * @private
    	 * @name getSchedules     
         * @description
        */
        function getSchedules(noLoadingBar) {
           if ($scope.isStatic) {
                var actData = [{"id": "2", "Actuators": [{"Operation": {"power": "Off"}, "deviceIDs": ["54880E1"]}], "dayOfWeek": ["Thu", "Fri", "Sat"], "disposable": true, "enabled": true, "startTime": "1900-01-01T05:57:00"}, {"id": "0", "Actuators": [{"Operation": {"power": "On"}, "deviceIDs": ["54880E1"]}], "dayOfWeek": ["Tue", "Wed"], "disposable": true, "enabled": true, "startTime": "1900-01-01T02:46:00"}];
                $timeout(function(){
                    $scope.parseActionData(actData);    //parseActionData() not created in scope. Hence, a timout.
                }, 500);
            } else {
                if(!!!noLoadingBar) {
                    $scope.checkResponse.push(APCONST.SCHEDULEDATA);
                }
                SHPService.sendSHPCommand(APCONST.GET, "/" + $scope.peerId + "/actions");
            } 
        }

        /**    	 
        * @function
        * @private
    	* @name gotoErrorPage       
        * @param 
        * @description Go to error page
        */
        function gotoErrorPage() {
            $scope.currentScreen = APCONST.ERRORPAGE;
            changeScreen();
        }
        
        /**    	 
        * @function
        * @private
    	* @name gotoAddSchedule       
        * @param 
        * @description Go to add schedule screen
        */
        function gotoAddSchedule() {
            toggleAnimClass();
            $timeout(function () {
                toggleAnimClass();
                $scope.currentScreen = APCONST.SCREEN_ADDSCHEDULE;
                changeScreen();
            }, APCONST.LIST_ANIMATION_TIME);
        }
        
        /**    	 
        * @function
        * @private
    	* @name     
        * @param 
        * @description
        */
        function addSchedule(act) {
            ScheduleService.addSchedule($scope, act);
        }
        
        /**    	 
        * @function
        * @private
    	* @name removeItemFromSchedule     
        * @param 
        * @description
        */
        function removeItemFromSchedule(schedule) {
            ScheduleService.removeItemFromSchedule($scope, schedule, APCONST.SCHEDULEDATA);
        }
        
        /**    	 
        * @function
        * @private
    	* @name asTabSelected    
        * @param 
        * @description
        */
        function asTabSelected(bool) {
            ScheduleService.asTabSelected($scope, bool);
            closeTurnOnOffPopup();
        }
        
        /**    	 
        * @function
        * @private
    	* @name increaseHours    
        * @param 
        * @description
        */
        function increaseHours() {
            ScheduleService.increaseHours($scope);
        }
        
        /**    	 
        * @function
        * @private
    	* @name decreaseHours    
        * @param 
        * @description
        */
        function decreaseHours() {
            ScheduleService.decreaseHours($scope);
        };
        
        /**    	 
        * @function
        * @private
    	* @name onHoursLostFocus    
        * @param 
        * @description
        */
        function onHoursLostFocus() {
            ScheduleService.onHoursLostFocus($scope);
        };
        
        /**    	 
        * @function
        * @private
    	* @name onMinsLostFocus    
        * @param 
        * @description
        */
        function onMinsLostFocus() {
            ScheduleService.onMinsLostFocus($scope);
        };
        
        /**    	 
        * @function
        * @private
    	* @name onAsRepeatWeakly    
        * @param 
        * @description
        */
        function onAsRepeatWeakly() {
            ScheduleService.onAsRepeatWeakly($scope);
        };
        
        /**    	 
        * @function
        * @private
    	* @name increaseMinutes    
        * @param 
        * @description
        */
        function increaseMinutes() {
            ScheduleService.increaseMinutes($scope);
        };
        
        /**    	 
        * @function
        * @private
    	* @name decreaseMinutes     
        * @param 
        * @description
        */
        function decreaseMinutes() {
            ScheduleService.decreaseMinutes($scope);
        };
        
        /**    	 
        * @function
        * @private
    	* @name changeMeridian    
        * @param 
        * @description
        */
        function changeMeridian() {
            ScheduleService.changeMeridian($scope);
        };
        
        /**    	 
        * @function
        * @private
    	* @name saveSchedule    
        * @param 
        * @description
        */
        function saveSchedule() {
            ScheduleService.saveSchedule($scope);
        };
        
        /**    	 
        * @function
        * @private
    	* @name keypadClicked     
        * @param 
        * @description
        */
        function keypadClicked() {
            ScheduleService.keypadClicked();
        };
        
        /**    	 
        * @function
        * @private
    	* @name daySelected    
        * @param 
        * @description
        */
        function daySelected(dayValue) {
            ScheduleService.daySelected($scope, dayValue);
        };
        
        /**    	 
        * @function
        * @private
    	* @name dayColorSelected    
        * @param 
        * @description
        */
        function dayColorSelected(dayValue) {
            return ScheduleService.dayColorSelected($scope, dayValue);
        };

        $scope.getWeatherDayList = ScheduleService.getWeatherDayList;
        
        /**
         * @function
         * @returns {undefined}
         * @description To push data location list on the selected country 
         */
        function getWeatherLocationList() {
            $scope.weatherLocations = "";
            mOrginialValues = [];
            if ($scope.selectedCountry === "KR") {
                $scope.weatherLocations = AIRCON_SETTING_REGION;
            } else if ($scope.selectedCountry === "CN") {
                $scope.weatherLocations = AIRCON_SETTING_REGION_CHINA;
            } else {
                return;
            }
            $scope.locationToBeSearched = '';
            $scope.searchTxtWithSpaces = "       " + $scope.translation.WEBMOB_common_search;
            $scope.weatherLocations.forEach(function (value) {
                mOrginialValues.push($scope.translation[value]);
            });           
            $scope.searchValues = mOrginialValues;
        }
        
        /**    	 
         * @function
         * @private
    	 * @name selectLocation
         * @param {String} location
         *       Location name
    	 * @description To set location and request for weather, outdoor air purity data
    	 */
        function selectLocation(location) {
            if ($scope.isLCDModel && apSupportOutdoor && !$scope.isStatic) {
                sendConfigToDevice(location);
                if ($scope.selectedCountry !== "KR") {
                    setLocalStorageData('selectedRegionName#'+$scope.Device.uuid, $scope.weatherLocations[mOrginialValues.indexOf(location)]);
                }
            } else {
                setLocalStorageData('selectedRegionName#'+$scope.Device.uuid, $scope.weatherLocations[mOrginialValues.indexOf(location)]);
            }
            $scope.selectedWeatherLocation = location;
            if (firstTimeLaunch) {
                firstTimeLaunch = false;
                $scope.currentScreen = APCONST.SCREEN_HOME;
                changeScreen();
            }
            getOutDoorAir();
        };
        
        /**    	 
        * @function
        * @private
    	* @name powerOnOff
    	* @description To send power ON or OFF command
    	*/
        function powerOnOff(status) {
            $timeout(function () {
                var payload = powerOnOffPayload(status);

                if ($scope.isStatic) {
                    $scope.powerState = !$scope.powerState;

                } else {
                    $scope.checkResponse.push(APCONST.POWERONOFF);
                    SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0");
                }
            }, APCONST.POWER_ANIMTION_TIMEOUT);
        }

        $scope.onSettingsUpdate = function (id, status) {
            $timeout(function () {
                if (!$scope.powerState) {
                    return;
                }

                var payloadData = settingPayload(id, status);
                var payload = payloadData[0];
                var suffixURL = payloadData[1];

                if ($scope.isStatic) {
                    switch (id) {
                        case "lighting":
                            $scope.DeviceOptions.lighting = !$scope.DeviceOptions.lighting;
                            break;
                        default:
                            break;
                    }

                } else {
                    $scope.checkResponse.push(APCONST.SETTING_COMMAND);
                    SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0" + suffixURL);
                }
            }, APCONST.TOGGLE_TIME);
        };
        //HAP
        /**
         * @function
         * @private
         * @name setHumidityMode
         * @description To send humidity mode
         * @param id( humidi, sterilizeClean, sterilize ), status( On, Off )
         */
        function setHumidityMode(id, currentStatus){
            if ($scope.powerState == false) {
                return;
            }

            var payloadData = humidityModePayload(id, currentStatus);
            var payload = payloadData[0];

            /*if ($scope.isStatic) {
                staticWindStrengthUpdate(id);
            } else {*/
            $scope.checkResponse.push(APCONST.WIND_STRENGTH_COMMAND);
            SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0");
            //}
        }
        
        function setSterilizationMode(currentStatus) {
            if ($scope.isStatic) {
                $scope.DeviceOptions.isSterilizeOn = !$scope.DeviceOptions.isSterilizeOn;
            } else {
                setHumidityMode('sterilize', currentStatus);
            }
        }
        
        function setAutoCleanMode(currentStatus) {
            if ($scope.isStatic) {
                $scope.DeviceOptions.isSterilizeCleanOn = !$scope.DeviceOptions.isSterilizeCleanOn;
            } else {
                setHumidityMode('sterilizeClean', currentStatus);                
            }
        }
        

    $scope.rippleToggleThree_D = false;
    $scope.rippleToggleWide = false;
    $scope.rippleToggleSpi = false;
    $scope.rippleToggleSpeed = false;

    $scope.showSettingsLightingToggle = function (id, status) {
        toggleOnOffClass();
        switch (id) {
            case "3d":
                $scope.rippleToggleThree_D = true;
                break;
            case "wide":
                $scope.rippleToggleWide = true;
                break;
            case "spi":
                $scope.rippleToggleSpi = true;
                break;
            case "speed":
                $scope.rippleToggleSpeed = true;
                break;
            default:
                break;
        }
        $scope.onWindStrengthUpdate(id, status);
    };

        /**    	 
        * @function
        * @private
    	* @name staticWindStrengthUpdate
        * @param {String} id : type of wind strength
    	* @description To set wind strength options in demo mode
    	*/
        function staticWindStrengthUpdate(id) {
            switch (id) {
                case "3d":
                    $scope.DeviceOptions.three_d = !$scope.DeviceOptions.three_d;
                    break;
                case "wide":
                    $scope.DeviceOptions.wide = !$scope.DeviceOptions.wide;
                    break;
                case "spi":
                    $scope.DeviceOptions.spi = !$scope.DeviceOptions.spi;
                    break;
                case "speed":
                    $scope.DeviceOptions.turbo = !$scope.DeviceOptions.turbo;
                    break;
                default:
                    break;
            }
        }

        function manageWindRipple(id) {
            switch (id) {
                case "3d":
                    $scope.rippleToggleThree_D = false;
                    break;
                case "wide":
                    $scope.rippleToggleWide = false;
                    break;
                case "spi":
                    $scope.rippleToggleSpi = false;
                    break;
                case "speed":
                    $scope.rippleToggleSpeed = false;
                    break;
                default:
                    break;
            }
        }

        /**    	 
        * @function
        * @private
    	* @name onWindStrengthUpdate
        * @param id : type of wind strength
        *        status : On or Off
    	* @description To send power ON or OFF command
    	*/
        function onWindStrengthUpdate(id, status) {
            $timeout(function () {
                if (!$scope.powerState) {
                    return;
                }
                if(id === 'spi' && $scope.isSpiDisabled()) {
                   return; 
                }
                if ($scope.DeviceOptions.turbo && (id === '3d' || id === 'wide')) {
                    return;
                }

                var payloadData = WindStrengthPayload(id, status);
                var payload = payloadData[0];
                var suffixURL = payloadData[1];

                if ($scope.isStatic) {
                    staticWindStrengthUpdate(id);
                } else {
                    $scope.checkResponse.push(APCONST.WIND_STRENGTH_COMMAND);
                    SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0" + suffixURL);
                }
                manageWindRipple(id);

            }, APCONST.TOGGLE_TIME);
        }

         /**    	 
        * @function
        * @private
    	* @name openIndoorPopup
    	* @description To open indoor air purity popup.
    	*/
        function openIndoorPopup() {
            if (!$scope.powerState || $scope.DeviceOptions.inDoorData.freshness.purity === -1 || $scope.Device.Wind.speedLevel === APCONST.WIND_SPEEDLEVEL_SLEEP
                    || $scope.DeviceOptions.isSterilizeCleanRunOn || $scope.DeviceOptions.isSterilizeRunOn || ($scope.isWaterError && $scope.DeviceOptions.isHumidiOn)) {
                return;
            }

            if(isInfoButtonHide()){
                return;
            }

            $scope.openPopoverForImage('indoor');
        }
        
        /**    	 
        * @function
        * @private
    	* @name openPopoverForImage
        * @param {String} type indoor or outdoor
    	* @description To send power ON or OFF command
    	*/
        function openPopoverForImage(type) {
            if (type === 'indoor') {
                if (!$scope.powerState || $scope.DeviceOptions.inDoorData.freshness.purity < 1 || $scope.DeviceOptions.inDoorData.freshness.purity > 6 || $scope.Device.Wind.speedLevel === APCONST.WIND_SPEEDLEVEL_SLEEP) {
                    return;
                }

            }

            $scope.freshnessPopupType = type;
            // $timeout(function ($event) {
            $scope.showPurityPopup = true;
            togglePurityPopUpClass($scope.showPurityPopup);
            if (type === 'indoor') {
               $scope.sendSensorRequest("SENSORS"); 
            }
            //}, 200);
        };
        
        /**    	 
        * @function
        * @private
    	* @name popUpOkApHome
    	* @description To close purity popup.
    	*/
        function popUpOkApHome() {
            $scope.showPurityPopup = false;
            togglePurityPopUpClass($scope.showPurityPopup);
        };
        
        /**    	 
        * @function
        * @private
    	* @name filterNotificationClose
    	* @description To close filter notification popup.
    	*/
        function filterNotificationClose() {
             $scope.isFilterStatusShow = false;
        };

    $scope.versionTouchStart = function () {
        if($scope.DeviceOptions.ventControl) {
          return;  
        }
        if (!promise) {
            promise = $timeout(function () {
                if (!$scope.isStatic) {
                    $scope.checkResponse.push(APCONST.PARSE_MODULEINFO);
                    SHPService.sendSHPCommand(APCONST.GET, "/" + $scope.peerId + "/devices/0/information");
                }
            }, 5000);
        }
    };

    $scope.versionTouchEnd = function () {
        if (promise) {
            $timeout.cancel(promise);
            promise = null;
        }
    };

    $scope.parseModuleInformation = function (moduleInfo) {
        if (angular.isDefined(moduleInfo.Versions) && moduleInfo.Versions.length > 0) {
            $scope.moduleInfoText = "Software Ver : " + moduleInfo.Versions[0].number + " <br> Firmware Ver : " + moduleInfo.Versions[1].number;
            dismissLoadingBar(APCONST.PARSE_MODULEINFO);
            document.getElementById("moduleInfoCancelID").style.visibility = 'hidden';
            $scope.showModuleInfoPopup = true;
            toggleBottomPopUpClass($scope.showModuleInfoPopup);
        }
    };

    $scope.moduleInfoOk = function () {
        $scope.showModuleInfoPopup = false;
        toggleBottomPopUpClass($scope.showModuleInfoPopup);
    };

    // About page start
    $scope.onAboutAction = function (reqParam, isFilterPay) {
        var selectedCountry;
        selectedCountry = $scope.selectedCountry; 
        
        if(reqParam === 'tutorial' || reqParam === 'manual' || reqParam === 'helpdesk') {
            if($scope.selectedCountry === "BE") {
                $scope.countryToSelect = [];
                $scope.countryToSelect.push({'Country': 'Belgium'});
                $scope.countryToSelect.push({'Country': 'Belgium(French)'});
                reqParamCountrySelection = reqParam;
                $scope.showcustCountrySelectionPopup();
            } else if($scope.selectedCountry === "CA") {
                $scope.countryToSelect = [];
                $scope.countryToSelect.push({'Country': 'Canada'});
                $scope.countryToSelect.push({'Country': 'Canada(French)'});
                reqParamCountrySelection = reqParam;
                $scope.showcustCountrySelectionPopup();
            } else if($scope.selectedCountry === "CH") {
                $scope.countryToSelect = [];
                $scope.countryToSelect.push({'Country': 'Switzerland'});
                $scope.countryToSelect.push({'Country': 'Switzerland(French)'});
                reqParamCountrySelection = reqParam;
                $scope.showcustCountrySelectionPopup();
            } else {
//                if($scope.isStatic) {
//                    $scope.selectedCountry = "BE";
//                    $scope.showcustCountrySelectionPopup();
//                    return;
//                }
                //Removed static check above as in demo mode also we need to run native commands and we are getting selected country so 
                //no need to set hard code country code 
                nativeInterface.runOnNative("customerService", aboutCommand(reqParam));
            }
        } else {
            if (!$scope.isStatic) {
                $timeout(function () {
                    if(isFilterPay){
                        nativeInterface.runOnNative("customerService", '{"service" : "' + 'filterpay' + '","uuid" : "' + $scope.deviceUuid + '"}');
                    }
                    else{
                        nativeInterface.runOnNative("customerService", '{"service" : "' + reqParam + '","selectedCountry" : "' + selectedCountry + '"}');
                    }
                }, APCONST.LIST_ANIMATION_TIME);
            }
        }
    };
    
    $scope.showcustCountrySelectionPopup = function() {
        $scope.countrySelectionPopupVisible = true;
    };
    
    $scope.closeCountrySelectionPopup = function () {
        $scope.countrySelectionPopupVisible = false;
    };

    $scope.onCountrySelected = function (countryIndex) {
        $scope.countrySelectedIndex = countryIndex;
    };

    $scope.onCountrySelectedOkClicked = function (isCountrySelected) {
        if(isCountrySelected /*&& !$scope.isStatic*/) { //Removed static check as in demo mode also we need to run native commands
            if($scope.countrySelectedIndex === 0){
                nativeInterface.runOnNative("customerService", aboutCommand(reqParamCountrySelection));
            } else {
                nativeInterface.runOnNative("customerService", aboutCommand(reqParamCountrySelection, true));
            }
        }
        $scope.closeCountrySelectionPopup();
    };
    
    function aboutCommand(cmdstring, changedCode) {
        if(changedCode) {
            return '{"service" : "' + cmdstring + '","selectedCountry" : "' + $scope.selectedCountry + '_FR"}';
        } else {
            return '{"service" : "' + cmdstring + '","selectedCountry" : "' + $scope.selectedCountry + '"}';
        }
    }

        /**    	 
        * @function
        * @private
    	* @name requestFailHandler        
        * @param {String} response
        *       JSON response received in case of request failed
    	* @description Handling of request failed cases
    	*/
        function requestFailHandler(response) {
            // When requestFailed occurs, dismiss loading bar
            $scope.checkResponse = [];
            $scope.loadingBar = false;

            var msg;
            if (angular.isDefined(response["resourceURL"]) && response["resourceURL"].indexOf("/actions") !== -1 && response["resourceURL"].indexOf("/actions/") === -1) {
                $timeout(function () {
                    toaster.success({title: "", body: $scope.translation.WEBMOB_common_schedule_duplication});
                }, 500);

                $scope.scheduleList();

            } else if (angular.isDefined(response["resourceURL"]) && response["resourceURL"].indexOf("/actions/") !== -1) {
                if (response.data.message.indexOf('InvokeDelScheduleFunc') !== -1) {
                    msg = $scope.translation.WEBMOB_common_schedule_delele_fail_reservation;
                } else if (response.data.message.indexOf('InvokeSetScheduleFunc') !== -1) {
                    msg = $scope.translation.WEBMOB_common_schedule_duplicate;
                } else {
                    // Nothing
                }
                if (angular.isDefined(msg)) {
                    $timeout(function () {
                        toaster.success({title: "", body: msg});
                    }, 500);
                }
                $scope.scheduleList();
            } else {
                // Nothing
            }
        };
    
        function isActionIdxPresent(resp) {
            var resArr = !!resp.resourceURL && resp.resourceURL.split("/");
            return (typeof resArr === "object" && resArr[2] === "actions" && !!resArr[3])
        } 
    
        /**    	 
         * @function
         * @private
    	 * @name handleStatusCode       
         * @description dismiss loading bar if checkResponse is having some request data and it is called from app.js
    	 */
        function handleStatusCode(status, response) {
            if ($scope.currentScreen === APCONST.SCREEN_SCHEDULER && isActionIdxPresent(response) && status !== APCONST.STATUS_204) {  //Lokesh 26Nov15_2036 -> if current screen is schedule list and resourceUrl contains "actions" and has an id after that then it is response of schedule deleting.
                //schedule deleted successfully. Show toast.
                $timeout(function () {
                    toaster.success({title: "", body: $scope.translation.WEBMOB_common_schedule_delete_success, bodyOutputType: 'trustedHtml'});
                }, 500);
                dismissLoadingBar(APCONST.SCHEDULEDATA);
            } else if (($scope.currentScreen === APCONST.SCREEN_SCHEDULER || $scope.currentScreen === APCONST.SCREEN_ADDSCHEDULE) && status === APCONST.STATUS_201) {
                //if status = 201, current screen is schedule list then this is response of add schedule.
                $timeout(function () {
                    toaster.success({title: "", body: $scope.translation.WEBMOB_common_schedule_save_successful, bodyOutputType: 'trustedHtml'});
                }, 500);
                dismissLoadingBar(APCONST.SCHEDULEDATA);
            } else if(!!response.resourceURL && ~response.resourceURL.indexOf("diagnosis")) {
                //do nothing in case of dianosis
            }
            //dismiss loading bar anonymously for all success responses.
            else if ($scope.checkResponse.length > 0 && 
                    $scope.checkResponse[$scope.checkResponse.length - 1] != APCONST.DIAGNOSIS &&
                    $scope.checkResponse[$scope.checkResponse.length - 1] != APCONST.VENT && 
                    $scope.checkResponse[$scope.checkResponse.length - 1] != APCONST.GET_DEVICES
                    ) {   //on a positive side, check for fony success response.
                //dismissLoadingBar($scope.checkResponse[$scope.checkResponse.length - 1]);
            } else {
                //None.
            }
        }
    
       
        /**    	 
         * @function
         * @private
    	 * @name dismissLoadingBar
    	 * @description To dismiss loading bar based on response
    	 */
        function dismissLoadingBar(value) {
            if ($scope.checkResponse.indexOf(value) !== -1) {
                $scope.checkResponse.splice($scope.checkResponse.indexOf(value), 1);
                if ($scope.checkResponse.length === 0) {
                    $scope.loadingBar = false;
                }
            }
        }

        /**    	 
         * @function
         * @private
    	 * @name hightlightSearchText        
         * @param {String} text
         *       location names
         * @param {String} phrase
         *       location to be searched
    	 * @description While user typing something in search, appropriate results should get highlighted
    	 */
        function hightlightSearchText (text, phrase) {
            if ($scope.locationToBeSearched) {
                text = text.replace(new RegExp('(' + $scope.locationToBeSearched + ')', 'i'), '<span class="highlighted">$1</span>');
            }
            return $sce.trustAsHtml(text);
        };

        /**    	 
         * @function
         * @private
    	 * @name searchValueUpdated        
         * @description On search value is updated it gets called
    	 */
        function searchValueUpdated () {
            $scope.locationToBeSearched = document.getElementById("facSearchID").value;
            $scope.searchValues = performSearchOperation($scope.locationToBeSearched, mOrginialValues);
        };
    
    /***** Schedule Related methods  *********/
    $scope.$watch(function () {
        return $scope.selectedSchedule.hours;
    }, function (newVal, oldVal) {
        if (newVal > 12) {
            $scope.selectedSchedule.hours = oldVal;
        }
        if (newVal === undefined) {
            $scope.selectedSchedule.hours = oldVal;
        }
    });

    // Control minutes value in add schedule screen
    $scope.$watch(function () {
        return $scope.selectedSchedule.minutes;
    }, function (newVal, oldVal) {
        if (newVal > 59) {
            $scope.selectedSchedule.minutes = oldVal;
        }
        if (newVal === undefined) {
            $scope.selectedSchedule.minutes = oldVal;
        }
    });
    
    /***** Wind strength image change  *********/
    $scope.$watch(function () {
        return $scope.selectedModeIcon;
    }, function (newVal, oldVal) {
        if(newVal !== oldVal) {
            var svg = document.getElementById('windStrengthId');
            SHPService.loadSVG($scope.selectedModeIcon, svg);
        }
    });
    
    function parseActionData (actData) {
        $scope.Actions = [];
        if (actData.length === 0) {
            $scope.scheduleDataAvailable = false;
        } else {
            $scope.scheduleDataAvailable = true;
        }

        if (actData.length > 0) {
            var i = 0;
            actData.forEach(function (act) {
                var subUid = $scope.Device.uuid.substr(0, 7);
//                if ((act.Actuators[0].deviceIDs).indexOf(subUid) !== -1) {

                if ($scope.Actions[i] === undefined) {
                    $scope.Actions[i] = {id: '', 'Actuators': [{Operation: {power: ''}, deviceIDs: []}], dayOfWeek: [], disposable: false, enabled: true, startTime: ''};
                }
                if (angular.isDefined(act.id)) {
                    $scope.Actions[i].id = act.id;
                }

                if (angular.isDefined(act.Actuators)) {
                    var Actuators = act.Actuators;
                    if (Actuators.length > 0) {
                        for (var j = 0; j < Actuators.length; j++) {
                            if ($scope.Actions[i].Actuators[j] === undefined) {
                                $scope.Actions[i].Actuators[j] = {Operation: {power: ''}, deviceIDs: []};
                            }
                            if (angular.isDefined(Actuators[j].Operation && (angular.isDefined(Actuators[j].Operation.power)))) {
                                $scope.Actions[i].Actuators[j].Operation.power = Actuators[j].Operation.power;
                            }
                            if (angular.isDefined(Actuators[j].deviceIDs)) {
                                $scope.Actions[i].Actuators[j].deviceIDs = Actuators[j].deviceIDs;
                            }
                        }
                    }
                }
                if (!!act.dayOfWeek && !!act.startTime) {
                    $scope.Actions[i].dayOfWeek = ScheduleService.getAdjustedWeekDays(act.dayOfWeek, act.startTime);
                }
                if (angular.isDefined(act.disposable)) {
                    $scope.Actions[i].disposable = act.disposable;
                }
                if (angular.isDefined(act.enabled)) {
                    $scope.Actions[i].enabled = act.enabled;
                }
                
                if (angular.isDefined(act.startTime)) {
                    $scope.Actions[i].startTime = act.startTime;
                    var gmt = ScheduleService.setTimeInGMT(act.startTime);

                    $scope.Actions[i].newStartTime = gmt[0] + ':' + gmt[1];
                    $scope.Actions[i].timeConvention = gmt[2];

                }
                i = i + 1;
//                }
            });

        }
        for (var j = actData.length; j < $scope.Actions.length; j++) {
            $scope.Actions.splice(j, 1);
        }

        if ($scope.Actions.length === 0) {
            $scope.ScheduleText = $scope.translation.WEBMOB_common_none;
        } else {
            ScheduleService.calculateUpcomingSchedule($scope);
        }
        dismissLoadingBar(APCONST.SCHEDULEDATA);
    };

    $scope.onHoursLostFocus = function () {
        $scope.inputFocusHoursMin = false;
        if ($scope.selectedSchedule.hours === "0" || $scope.selectedSchedule.hours === "00") {
            $scope.selectedSchedule.hours = 12;
        }
        if ($scope.selectedSchedule.hours === "") {
            $scope.selectedSchedule.hours = $scope.oldValueHours;
        }
        $scope.selectedSchedule.hours = Number($scope.selectedSchedule.hours);
        $scope.selectedSchedule.hours = ($scope.selectedSchedule.hours < 10) ? ('0' + $scope.selectedSchedule.hours) : $scope.selectedSchedule.hours;
    };

    $scope.onMinsLostFocus = function () {
        $scope.inputFocusHoursMin = false;
        if ($scope.selectedSchedule.minutes === "") {
            $scope.selectedSchedule.minutes = $scope.oldValueMins;
        }
        $scope.selectedSchedule.minutes = Number($scope.selectedSchedule.minutes);
        $scope.selectedSchedule.minutes = ($scope.selectedSchedule.minutes < 10) ? ('0' + $scope.selectedSchedule.minutes) : $scope.selectedSchedule.minutes;
    };

    $scope.onHoursMinsFocusGain = function () {
        $scope.oldValueHours = $scope.selectedSchedule.hours;
        $scope.oldValueMins = $scope.selectedSchedule.minutes;
        if (!$scope.isAppleDevice) {
            $scope.inputFocusHoursMin = true;
            setTimeout(function() {
                document.activeElement.scrollIntoView();
            }, 10);
        }
    };

    function increaseHrTime() {
        $scope.selectedSchedule.hours = Number($scope.selectedSchedule.hours);
        if ($scope.selectedSchedule.hours !== 12) {
            $scope.selectedSchedule.hours += 1;
        } else {
            $scope.selectedSchedule.hours = 1;
        }
        $scope.selectedSchedule.hours = ($scope.selectedSchedule.hours < 10) ? ('0' + $scope.selectedSchedule.hours) : $scope.selectedSchedule.hours;

        document.getElementById("asHoursID").value = $scope.selectedSchedule.hours;
    }

    function decreaseHrTime() {
        $scope.selectedSchedule.hours = Number($scope.selectedSchedule.hours);
        if ($scope.selectedSchedule.hours > 1) {
            $scope.selectedSchedule.hours -= 1;
        } else {
            $scope.selectedSchedule.hours = 12;
        }
        $scope.selectedSchedule.hours = ($scope.selectedSchedule.hours < 10) ? ('0' + $scope.selectedSchedule.hours) : $scope.selectedSchedule.hours;

        document.getElementById("asHoursID").value = $scope.selectedSchedule.hours;
    }

    function increaseMinTime() {
        $scope.selectedSchedule.minutes = Number($scope.selectedSchedule.minutes);
        // defined steps as 1
        var step = 1;
        if ($scope.selectedSchedule.minutes !== (60 - step)) {
            $scope.selectedSchedule.minutes += step;
        } else {
            $scope.selectedSchedule.minutes = 0;
        }
        $scope.selectedSchedule.minutes = ($scope.selectedSchedule.minutes < 10) ? ('0' + $scope.selectedSchedule.minutes) : $scope.selectedSchedule.minutes;

        document.getElementById("asMinutesID").value = $scope.selectedSchedule.minutes;
    }

    function decreaseMinTime() {
        $scope.selectedSchedule.minutes = Number($scope.selectedSchedule.minutes);
        var step = 1;
        if ($scope.selectedSchedule.minutes !== 0) {
            $scope.selectedSchedule.minutes -= step;
        } else {
            $scope.selectedSchedule.minutes = 60 - step;
        }
        $scope.selectedSchedule.minutes = ($scope.selectedSchedule.minutes < 10) ? ('0' + $scope.selectedSchedule.minutes) : $scope.selectedSchedule.minutes;

        document.getElementById("asMinutesID").value = $scope.selectedSchedule.minutes;
    }

    $scope.increaseHrContinuously = function () {
        if (!!!promise) {
            //It has been observed that touchend event is not fired every time. 
            //Hence, limit the $interval to a number of executions. 
            //Here this number is 14 = intervals required to reach maximum from minimum value.
            promise = $interval(function () {
                increaseHrTime();
            }, 200);
        }
    };

    $scope.stopLongPressEvent = function () {
        if (!!promise) {
            //check if promise exist. Then call cancel.
            $interval.cancel(promise);
            promise = undefined;
        }
        if ($scope.longPress) {
            $scope.longPress = false;
            clearInterval($scope.timer);
            return;
        }
    };

    $scope.decreaseHrContinuously = function () {
        if (!!!promise) {
            //It has been observed that touchend event is not fired every time. 
            //Hence, limit the $interval to a number of executions. 
            //Here this number is 14 = intervals required to reach maximum from minimum value.
            promise = $interval(function () {
                decreaseHrTime();
            }, 200);
        }
    };

    $scope.increaseMinContinuously = function () {
        if (!!!promise) {
            //It has been observed that touchend event is not fired every time. 
            //Hence, limit the $interval to a number of executions. 
            //Here this number is 14 = intervals required to reach maximum from minimum value.
            promise = $interval(function () {
                increaseMinTime();
            }, 200);
        }
    };

    $scope.decreaseMinContinuously = function () {
        if (!!!promise) {
            //It has been observed that touchend event is not fired every time. 
            //Hence, limit the $interval to a number of executions. 
            //Here this number is 14 = intervals required to reach maximum from minimum value.
            promise = $interval(function () {
                decreaseMinTime();
            }, 200);
        }
    };
    
    /* Filter Starts */
    $scope.openPopoverFilterReset = function () {
        $scope.dialogPopup.show = true;
        toggleBottomPopUpClass($scope.dialogPopup.show);
        $scope.dialogPopup.dialogPopupTitle = $scope.translation.ACMOB_VTL_filter_btn_top;
        $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_common_energy_monitor_reset_confirm;
        $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_common_cancel;
        $scope.dialogPopup.button2Enabled = true;
        $scope.dialogPopup.button2Title = $scope.translation.WEBMOB_common_ok;
        $scope.dialogPopup.button2Handler = function () {

            filterResetOK();
        };
    };

    function filterResetOK() {
        $scope.dialogPopup.show = false;
        if (!$scope.isStatic) {
            var payload = '{"Device":{"Mode":{"options":["FilterCleanAlarm_Clear"]}}}';
            $scope.checkResponse.push(APCONST.FILTER_RESET);
            SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0");
        } else {
            //$scope.resetOptimusTimeHrs = "00";
            //$scope.resetOptimusTimeMns = "00";
            if ($scope.DeviceOptions.filterSteps) {
                filterLayerStyleNew();
            } else {
                filterLayerStyle();
            }
        }
    };
    
    $scope.filterResetCancel = function () {
        if (angular.isDefined($scope.popoverfilterreset)) {
            $scope.popoverfilterreset.hide();
        }
    };
    //Old
    function filterLayerStyle(element) {
        if ($scope.currentScreen === APCONST.SCREEN_FILTER) {
            var filterLayerElement = angular.isDefined(element) ? element : document.getElementById('filterlayerstyle');
            var areacovered = $scope.Device.Consumables[0].state;
            if (areacovered >= 100) {
                filterLayerElement.style.width = "100%";
                filterLayerElement.style.backgroundColor = "#ff7366";
                $scope.showFilterAlertLayerText = true;
            } else {
                areacovered = areacovered + '%';
                filterLayerElement.style.backgroundColor = "#89cfec";
                filterLayerElement.style.width = areacovered;
                $scope.showFilterAlertLayerText = false;
            }
            debugMessage("Area: " + areacovered);
        }
    };
    
    //New 
     function filterLayerStyleNew(element) {
         $scope.isFilterEnable = true;
         if ($scope.currentScreen === APCONST.SCREEN_FILTER) {
            var filterLayerElement = angular.isDefined(element) ? element : document.getElementById('filterlayerstyle');
            //var areacoveredNew = parseInt($scope.Device.Consumables[0].state)*25;
            var valState = parseInt($scope.Device.Consumables[0].state,10);
            var areacoveredNew = (valState && valState *25) || valState;
            if (areacoveredNew === 0) {
                filterLayerElement.style.width = "100%";
                filterLayerElement.style.left = "0%";
                filterLayerElement.style.backgroundColor = "#ff7366";
                $scope.showFilterAlertLayerText = true;
            } else {
                var leftCover = 100 - areacoveredNew;
                areacoveredNew = areacoveredNew + '%';
                var leftCoverArea = leftCover + '%';
                filterLayerElement.style.width = areacoveredNew;
                filterLayerElement.style.left = leftCoverArea;
                filterLayerElement.style.backgroundColor = "#89cfec";
                $scope.showFilterAlertLayerText = false;
            }
            debugMessage("Area new: " + areacoveredNew);
        }
    };

    $scope.filterMode = function(mode){
        if($scope.isAirPurifier){
            // todo : change magic number with meaningful const variable
            if(mode.id === 5)   // 5 : speed mode for humidifer device only ( no need to show ap device )
            {
                return false;
            }
            else
            {
                return true;
            }
        }
        else{   // note : defense code in case it's called with hap device.
            return true;
        }
    };

    /*Filter Ends */
    
    /* SMART CARE */
    $scope.goToSmartCare = function () {
        $scope.smartCare.started = true;
        $scope.smartCare.viewErros = false;
        $scope.smartCare.detected = false;
        toggleAnimClass();
        $timeout(function () {
            toggleAnimClass();
            $scope.currentScreen = APCONST.SCREENS_SMART_CARE;
            changeScreen();
        }, APCONST.LIST_ANIMATION_TIME);
    };
    
    $scope.startSmartCare = function() { 
        $scope.smartCare.viewErros = false;
        $scope.smartCare.diagnosisStarted = true;
        $scope.smartCare.errors = [];
        $scope.smartCare.totalErrors = 0;
        
        if (!$scope.isStatic) {
          var payload = '{"Diagnosis" : {"diagnosisStart" : "Run"}}'; 
          $scope.checkResponse.push(APCONST.DIAGNOSIS);
          $scope.Device.Alarms = [];
          diagnosisStarted = true;
          SHPService.sendSHPCommand(APCONST.PUT, payload, "/" + $scope.peerId + "/devices/0/diagnosis", true, 300000);      //Lokesh 16Nov15_1421 -> fall back time for smart care is 5 minutes (300000 ms).
        } else { 
            //$scope.loadingBar = true;
            progressSmartCare();
        }
    };
    
    $scope.viewSmartCareErrors = function() {
        if($scope.smartCare.totalErrors === 0) {
           return; 
        }
        $timeout(initSwipe, 300);
        $scope.smartCare.started = false;
        $scope.smartCare.detected = false;
        $scope.smartCare.viewErros = true;
        $scope.smartCareErrorIndex = 0;
        
    };

    $scope.getCheckOptionsContainer = function(){
        var optionsClass;
        if ($scope.isAirPurifier) {
            optionsClass = "help-check-options-container";
        } else {
            if ($scope.selectedCountry != 'KR' && $scope.selectedCountry != 'CN') {
                optionsClass = "help-check-options-container-HAP-no-outdoor";
            } else {
                optionsClass = "help-check-options-container-HAP";
            }
        }
        return optionsClass;        
    }
    
    function progressSmartCare() {
        if(!$scope.smartCare.diagnosisStarted) {
            return;
        }
        $scope.smartCare.errors = [];
        if (!$scope.isStatic) { 
            $scope.smartCare.totalErrors = $scope.Device.Alarms.length;
            $scope.Device.Alarms.forEach(function(value) {
    
                switch(value.code) {
                    case 'SENSOR_F':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_f","description":"WEBMOB_device_ref_smartcare_sensor_f_msg"});
                        break;
                    case 'SENSOR_R':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r","description":"WEBMOB_device_ref_smartcare_sensor_r_msg"});
                        break;
                    case 'SENSOR_F_DF':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_f_df","description":"WEBMOB_device_ref_smartcare_sensor_f_df_msg"});
                        break;
                    case 'SENSOR_R_DF':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r_df","description":"WEBMOB_device_ref_smartcare_sensor_r_df_msg"});
                        break;
                    case 'SENSOR_TEMP':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_temp","description":"WEBMOB_device_ref_smartcare_sensor_temp_msg"});
                        break;
                    case 'SENSOR_CV':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_cv","description":"WEBMOB_device_ref_smartcare_sensor_cv_msg"});
                        break;
                    case 'SENSOR_CV_DF':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_cv_df","description":"WEBMOB_device_ref_smartcare_sensor_cv_df_msg"});
                        break;
                    case 'PANEL_MAIN':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_panel_main","description":"WEBMOB_device_ref_smartcare_panel_main_msg"});
                        break;
                    case 'BLINKING_F_AT':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_bl_f_at","description":"WEBMOB_device_ref_smartcare_bl_f_at_msg"});
                        break;
                    case 'BLINKING_R_AT':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_bl_r_at","description":"WEBMOB_device_ref_smartcare_bl_r_at_msg"});
                        break;
                    case 'SENSOR_HD':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_hd","description":"WEBMOB_device_ref_smartcare_sensor_hd_msg"});
                        break;
                    case 'SENSOR_R_HD':
                        $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r_hd","description":"WEBMOB_device_ref_smartcare_sensor_r_hd_msg"});
                        break;
                    default:
                        $scope.smartCare.totalErrors--;
                }
            });
            
            
        } else {
            $scope.smartCare.totalErrors = 10;
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_f","description":"WEBMOB_device_ref_smartcare_sensor_f_msg"});
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r","description":"WEBMOB_device_ref_smartcare_sensor_r_msg"});
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_f_df","description":"WEBMOB_device_ref_smartcare_sensor_f_df_msg"});
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_r_df","description":"WEBMOB_device_ref_smartcare_sensor_r_df_msg"});
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_temp","description":"WEBMOB_device_ref_smartcare_sensor_temp_msg"});
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_cv","description":"WEBMOB_device_ref_smartcare_sensor_cv_msg"});
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_sensor_cv_df","description":"WEBMOB_device_ref_smartcare_sensor_cv_df_msg"});
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_panel_main","description":"WEBMOB_device_ref_smartcare_panel_main_msg"});
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_bl_f_at","description":"WEBMOB_device_ref_smartcare_bl_f_at_msg"});
            $scope.smartCare.errors.push({"title":"WEBMOB_device_ref_smartcare_bl_r_at","description":"WEBMOB_device_ref_smartcare_bl_r_at_msg"});
        }
        
        $scope.errorString = 'WEBMOB_device_VTL_smartcare_smartcarecount';
        if($scope.smartCare.totalErrors === 0) {
            $scope.errorString = 'WEBMOB_device_ref_smartcare_no_error_ref_msg';
        }
        
        if($scope.smartCare.totalErrors === 1) {
            $scope.viewSmartCareErrors();
        } else {
            $scope.currentScreen = APCONST.ERRORPAGE;
            changeScreen();
//            $scope.showErrorPage = true;
//            $scope.smartCare.started = false;
//            $scope.smartCare.viewErros = false;
//            $scope.smartCare.detected = true; 
        }
    }
    
    $scope.doAlarmService = function() {
//        serviceCenterNum.forEach(function (value) {
//                if (angular.isDefined(value[$scope.selectedCountry])) {
//                    $scope.serviceCallNumber = value[$scope.selectedCountry];
//                }
//            });
        $scope.dialogPopup.show = true;
        toggleBottomPopUpClass(true);
        $scope.dialogPopup.dialogPopupTitle = $scope.translation.WEBMOB_common_service_call;
        $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_common_connect_service_call+"<br>" + $scope.serviceCenterNumber;
        $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_common_cancel;
        $scope.dialogPopup.button2Enabled = true;
        $scope.dialogPopup.button2Title = $scope.translation.WEBMOB_common_ok;
        $scope.dialogPopup.button2Handler = function() {
            $scope.dialogPopup.show = false;
            if($scope.supportCalling) {
                var payload = '{"selectedCountry" : "' + $scope.selectedCountry + '"}';
                nativeInterface.runOnNative("callDial", payload);
            }
        }; 
    };
    
     function showOptionPopUp() {
            $scope.bOptionPopUp = true;
     }
     
     function closeOptionPopUp() {
            $scope.bOptionPopUp = false;
     }
     
     function openTurnOnOffPopup() {
            $scope.bTurnOnOffPopup = true;
            toggleBottomPopUpClass($scope.bTurnOnOffPopup);
     }
     
     function closeTurnOnOffPopup() {
            $scope.bTurnOnOffPopup = false;
            toggleBottomPopUpClass($scope.bTurnOnOffPopup);
     }
     
    function selectIndexFromOptionMenu(items, optionMenuindex) {
        switch(optionMenuindex) {
            case 0:
                $scope.currentScreen = APCONST.SCREEN_SETTINGS;
                break;
            case 1:
                $scope.currentScreen = APCONST.SCREEN_HELP;
                break;
            case 2:
                $scope.currentScreen = APCONST.SCREEN_ABOUT;
                break;
            default:
                break;
        }
        changeScreen();
        closeOptionPopUp();
    }

    function showUsageHelpPage() {
        setTimeout(function(){
            document.getElementById("scrollable_area_id").scrollTop = 0;
        }, 10);
        $scope.bUsageHelpPage = true;
        tempFilterStatusShow = $scope.isFilterStatusShow;
        $scope.isFilterStatusShow = false;
    }

    function closeUsageHelpPage() {
        $scope.isFilterStatusShow = tempFilterStatusShow;
        $scope.bUsageHelpPage = false;
    }
    
    function getOptionText(status) {
        var text;
        if(status === 'On') {
            text = $scope.translation.WEBMOB_common_schedule_turn_on;
        } else {
            text = $scope.translation.WEBMOB_common_schedule_turn_off;
        }
        return text;
    }
    
    function getSelectedModeIcon() {
        for (var i = 0; i < $scope.listOfModes.length; i++) {
            if ($scope.listOfModes[i].modeName === $scope.selectedMode) {
                return $scope.listOfModes[i].imageID;
            }
        }
        return '';
    }
   

        function initSwipe() {
            var elem = document.getElementById('mySwipe'),
                    mySwipeWrap,
                    cnt,
                    len,
                    counter = $scope.smartCare.errors.length;
            window.mySwipe = Swipe(elem, {speed:100, continuous:false, callback: onSwipe
            });
            mySwipeWrap = document.getElementById('mySwipe-wrap');
            len = mySwipeWrap.childNodes.length;
            for (cnt = 0; cnt < len; ++cnt) {
                if (!!mySwipeWrap.childNodes[cnt].className && mySwipeWrap.childNodes[cnt].className === "errorSummary") {
                    if (counter <= 0) {
                        mySwipeWrap.removeChild(mySwipeWrap.childNodes[cnt]);
                        --cnt;
                        --len;
                    } else {
                        --counter;
                    }
                }
            }

            mySwipeWrap.style.width = screen.width * $scope.smartCare.errors.length;
            var pageViewBulletsid=document.getElementById('pageViewBulletsid');    
            pageViewBulletsid.style.marginLeft='-' + Math.round(pageViewBulletsid.offsetWidth / 2) + 'px';
             pageViewBulletsid.style.visibility = "visible";

        }
    
        /**    	 
         * @function
         * @private
    	 * @name initSearchText        
         * @description reset search input value and search list with original values
    	 */
        function initSearchText() {
            document.getElementById("facSearchID").value = $scope.locationToBeSearched = '';
            $scope.searchValues = mOrginialValues;
        }
	
	function isInfoButtonHide(){
            if($scope.DeviceOptions.isUsingNewOptionValue){
                if(!$scope.DeviceOptions.isInfoButtonShow)
                    return true;
            }
            else{
                if($scope.DeviceOptions.pm10SupportOnly && !$scope.DeviceOptions.pm25SupportOnly)
                    return true;
            }
            return false;
        }

        function isLightMenuShow(){
            if($scope.DeviceOptions.isLightSupported){
                return true;
            }
            return false;
        }

        function isMuteMenuShow(){
            if($scope.DeviceOptions.isDistinguishingMuteLightOption && $scope.DeviceOptions.isMuteSupported && !$scope.isLCDModel){
                return true;
            }
            return false;
        }
        
        function isSoundMenuShow(){
            if($scope.DeviceOptions.isDistinguishingMuteLightOption && $scope.DeviceOptions.isMuteSupported && $scope.isLCDModel){
                return true;
            }
            return false;
        }
        
        function isPopupOpen() {
            if ($scope.showVentSelectionPopup || $scope.showModeSelectionPopup || $scope.showPurityPopup || $scope.bOptionPopUp || $scope.bUsageHelpPage
                    || $scope.bTurnOnOffPopup || $scope.dialogPopup.show || $scope.showVolumePopup || $scope.showConnFailurePopup || $scope.showModuleInfoPopup 
                    || $scope.loadingBar || $scope.showHumidfiSettingPopup) {
                return true;
            }
            return false;
        }
        
        function isGlobal() {
            if ($scope.selectedCountry !== 'KR' && $scope.selectedCountry !== 'CN') {
                return true;
            }
            return false;
        }
        
        function getVisiblityOfHAPIndoor() {
            var state;
            if (isGlobal()) {
                if (!$scope.powerState) {
                    state = false;
                } else {
                    state = true;
                }
            } else {
                state = true;
            }
            
            return state;
        }
        
        function isValidOutdoorData() {
            if ($scope.DeviceOptions.outDoorData.KHAI.iconClass !== "purity-level-off") {
                return true;
            }
              return false;
        }
        
        function getFilterStatusVisibility() {
            if ($scope.isFilterStatusShow && $scope.bAPHomePageContainer) {
                return true;
            }
            return false;
        }
        
        function getScrollableAreaClass() {
            var scrollClass = "";
            if (isPopupOpen()) {
                scrollClass += "noscroll";
            }
            if ($scope.addSchedulePage) {
                if ($scope.isAppV2) {
                    scrollClass += " scrollable_area_no_header scrollable_area_no_header_V2";
                } else {
                    scrollClass += " scrollable_area_no_header";
                }
            } else if ($scope.bAPHomePageContainer) {
                if ($scope.isAppV2) {
                    if(getFilterStatusVisibility()) {
                        scrollClass += " scrollable_area scrollable_area_in_app_noti_V2";
                    } else {
                        scrollClass += " scrollable_area scrollable_area_V2";
                    }
                } else {
                    if(getFilterStatusVisibility()) {
                        scrollClass += " scrollable_area scrollable_area_in_app_noti";
                    } else {
                        scrollClass += " scrollable_area";
                    }                    
                }
            } else if (!$scope.bAPHomePageContainer) {
                if ($scope.isAppV2) {
                    scrollClass += " scrollable_area_others scrollable_area_others_V2";
                } else {
                    scrollClass += " scrollable_area_others";
                }
            }
            return scrollClass;
        }
        
        function getModeSelectionPopupClass() {
            var modeClass = ""; 
            if(getFilterStatusVisibility()) {
                if ($scope.isAppV2) {
                    modeClass += " ModeSelectionPopupConInAppNoti_V2";
                } else {
                    modeClass += " ModeSelectionPopupConInAppNoti";
                }
            } else {
                if ($scope.isAppV2) {
                    modeClass += " ModeSelectionPopupCon_V2";
                }
            }
            return modeClass;
        }
        
        function getDesiredHumidityValueName(value) {
            var temp;
            switch (value) {
                case APCONST.HUMIDIFI_AUTO:
                    temp = $scope.listOfHumidifiModes[0].modeName;
                    break;
                case APCONST.HUMIDIFI_40:
                    temp = $scope.listOfHumidifiModes[1].modeName;
                    break;
                case APCONST.HUMIDIFI_50:
                    temp = $scope.listOfHumidifiModes[2].modeName;
                    break;
                case APCONST.HUMIDIFI_60: 
                    temp = $scope.listOfHumidifiModes[3].modeName;
                    break;
                default:
                    break;
            }
            return temp;
        }

        function isSterilizationDisabled() {
            if (!$scope.powerState  || $scope.isWaterError || $scope.DeviceOptions.isSterilizeCleanRunOn || !$scope.DeviceOptions.isHumidiOn) {
                return true;
            }

            if($scope.DeviceOptions.isLightEnabledWhileSterilizeCleanRunOn === false){
                if($scope.DeviceOptions.isSterilizeRunOn){
                    return true;
                }
            }

            return false;
        }
        
        function isHumidificationDisabled() {
            if (!$scope.powerState || $scope.DeviceOptions.isSterilizeCleanRunOn && $scope.DeviceOptions.isHumidiOn) {
                return true;
            }
            return false;
        }

        function isSpiDisabled() {
            if (!$scope.powerState || $scope.Device.Wind.speedLevel === APCONST.WIND_SPEEDLEVEL_SLEEP || $scope.DeviceOptions.isSterilizeCleanRunOn
                || ($scope.isWaterError && $scope.DeviceOptions.isHumidiOn) ) {
                return true;
            }
            return false;
        }
        
        function isWindStrengthDisabled() {
            if ($scope.DeviceOptions.isSterilizeCleanRunOn || $scope.isWaterError && $scope.DeviceOptions.isHumidiOn) {
                return true;
            }
            return false;
        }
        
    function getHumidItemTextClass(humidOption) {
        var humidClass = '';
        if ($scope.selectedMode !== $scope.listOfModes[0].modeName) { // if wind strength is not auto
            if (humidOption !== $scope.listOfHumidifiModes[0].modeName && humidOption !== $scope.listOfHumidifiModes[4].modeName) {
                humidClass = 'BottomPopupItemTextDisable';
            } else{
                humidClass =  $scope.desiredHumidityValue === humidOption ? humidClass + ' BottomPopupItemTextSelected ' : ' ';
            } 
        } else {  // wind strength is auto
            if (humidOption === $scope.listOfHumidifiModes[0].modeName) { // humidity option is auto
                humidClass = 'BottomPopupItemTextDisable';
            } else {
                humidClass =  $scope.desiredHumidityValue === humidOption ? humidOption + ' BottomPopupItemTextSelected ' : ' ';
            }
        }
        return humidClass;
    } 
    
    function getHumidListItemClass(humidOption) {
        var humidClass = '';
        if ($scope.selectedMode !== $scope.listOfModes[0].modeName) {
            if (humidOption !== $scope.listOfHumidifiModes[0].modeName && humidOption !== $scope.listOfHumidifiModes[4].modeName) {
                humidClass = 'no_click';
            }
        } else {
            if (humidOption === $scope.listOfHumidifiModes[0].modeName) {
                humidClass = 'no_click';
            }
        }
        return humidClass;
    }
    
    
    function setProgress(progress) {
        var progressBar = document.getElementById("auto_clean_progress_blue");         
        if (progressBar !== null) {
            if (progress === 100) {
                progressBar.style.borderRadius = ".15rem";// if 100% progress acheived, make the right end of bar circular
            }
            progressBar.style.width = progress + "%";
        } else {
            $timeout(function() {setProgress(progress);}, 100);
        }
    }
    
    function callGetDevice() {
        SHPService.getDevices($scope);
    }
    
    function parseRequestResponse(response) {
        console.log(" parseRequestResponse " + STCONST);
        if (angular.isDefined(response["status"]) && response["status"] === STCONST.STATUS_99000) {
            showConnectionFailurePopup();
        }

        if (angular.isDefined(response["data"])) {
            // For home multi specific code
            var device;
            if (angular.isDefined(response["data"]["Devices"])) {
                response["data"]["Devices"].forEach(function (value) {
                    if (value["uuid"] === $scope.deviceUuid) {
                        device = value;
                    }
                });
                if (angular.isDefined(device)) {
                    init();
                    parseDeviceData(device, response.resourceURL);
                } else {
                    showConnectionFailurePopup();
                }
                if (!!!$scope.loadingBar) { // No loading bar means request handled properly. So, clear array.
                    SHPService.clearStopVarArray();
                }
                isCallFromReset = 1;    //Lokesh 16Nov11_1840 -> For energy monitor reset not working first time.
                return;
            }
            if (angular.isDefined(response["data"]["Sensors"])) {
                parseDeviceData(response["data"]);
            }
            if (angular.isDefined(response["data"]["Configuration"])) {
                parseConfiguration(response["data"]["Configuration"]);
            }
            if (angular.isDefined(response["data"]["Actions"])) {
                parseActionData(response["data"]["Actions"]);
            }
            if (angular.isDefined(response["data"]["Information"])) {
                if ($scope.checkResponse.indexOf(APCONST.PARSE_MODULEINFO) !== -1) {
                    $scope.parseModuleInformation(response["data"]["Information"]);
                } else {
                    parseModelInfo(response["data"]["Information"]);
                }                
            }
            if (angular.isDefined(response["data"]["DAairData"])) {
                $scope.outDoorPuirtyData(response["data"]["DAairData"]);
            }
            //Outdoor Wheather data for China Model Airepurifier
            if (angular.isDefined(response["data"]["sinaWeatherData"])) {
                $scope.outDoorPuirtyData(response["data"]);
            }
        }
        if (angular.isDefined(response["resourceURL"])) {
            if (response["resourceURL"].indexOf("files") != -1) {
                $scope.isDBDownloaded = true;
                isDBDownloadedJS = true;
                if (isCallFromReset === 2) {
                    $scope.loadingBar = false;
                    isCallFromReset = 1;
                    goToEnergyMonitor();
                }
            }
            if (angular.isDefined(response["data"])) {
                if (!angular.isDefined(response["data"]["Actions"]) && (response["resourceURL"].indexOf("/actions/") !== -1 || response["resourceURL"].indexOf("/actions") !== -1)) {
                    scheduleList(true);
                }
            } else {
                if (response["resourceURL"].indexOf("/undefined/") !== -1 || response["resourceURL"].indexOf("/undefined") !== -1) {
                    return;
                }
                if (response["resourceURL"].indexOf("/actions/") !== -1 || response["resourceURL"].indexOf("/actions") !== -1) {
                    scheduleList(true);
                }
            }
        }
        if (response.status >= 200 && response.status < 300) { //<300 is succes only. Refer http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html for status code definitions.
            handleStatusCode(response.status, response);
        }
    }
    
    function parseNotification(response) {
        if (angular.isDefined(response["softKeypadVisibility"])) {
            onKeypadShown(false);
        }
        if (angular.isDefined(response["data"])) {
            if (response["data"] === "backPressed") {
                //if loding bar visible, don't handle back key
                if (!!!$scope.loadingBar) {
                    onBackPressed();
                }
                return;
            }
            if (angular.isDefined(response["data"]["mobileTimeChanged"])){
                return;
            }
            if (angular.isDefined(response["data"].isApplicationActive) && !$scope.isStatic) {
                handleAppAtForeground();
                $scope.appAlive = true;
                callGetDevice();
                return;
            }
            if (angular.isDefined(response["data"][0]) && angular.isDefined(response["data"][0]["Device"]) && angular.isDefined(response["data"][0]["Device"]["scsConnectionState"])) {
                if (!response["data"][0]["Device"]["scsConnectionState"]) {
                    showConnectionFailurePopup();
                } else if (response["data"][0]["Device"]["scsConnectionState"]) {
                    closeConnectionFailurePopup();                        
                    callGetDevice();
                }
                return;
            }
            if(response["data"] instanceof Array) {
                response["data"].forEach(function (value) {
                    parseDeviceData(value);
                });
            } else {
                parseDeviceData(response["data"]);
            }
        }
    }
    
    function parseRequestAccepted(response){
        if (angular.isDefined(response["data"]) && angular.isDefined(response["data"]["energyData"])) {
            $scope.parseEnergyUsageData(response["data"]);
        }
    }

    function onKeypadShown(boolean) {
        if (!boolean && !$scope.isAppleDevice && $scope.inputFocusHoursMin) {
            document.activeElement.blur();
            $scope.inputFocusHoursMin = false;
        }
    }
    
    function isOptionDisabled() {
        if($scope.powerState){
            if($scope.DeviceOptions.ventControl || $scope.DeviceOptions.isSpeedSupported || $scope.spiSupported || $scope.isSterilizeCleanSupprot){
                return false;
            }
        }
        return true;
    }

    function isLightSettingDisabled() {
        if (!$scope.powerState || ($scope.Device.Wind.speedLevel === APCONST.WIND_SPEEDLEVEL_SLEEP) || $scope.isWaterError) {
            return true;
        }

        if($scope.DeviceOptions.isLightEnabledWhileSterilizeCleanRunOn === false){
            if($scope.DeviceOptions.isSterilizeCleanRunOn){
                return true;
            }
        }

        return false;
    }
    
    function isAutoCleanSettingDisabled() {
        if (!$scope.powerState || $scope.isWaterError || !$scope.DeviceOptions.isHumidiOn
                || ($scope.Device.Wind.speedLevel === 1) || ($scope.selectedMode === "ACMOB_VTL_main_dropdown_sleep")) {
            return true;
        }

        if($scope.DeviceOptions.isLightEnabledWhileSterilizeCleanRunOn === false){
            if($scope.DeviceOptions.isSterilizeRunOn){
                return true;
            }
        }

        return false;
    }
    
     function getLowerHumidityLable() {
         var lable = '';
          var language = LANG.toUpperCase().substring(0,2);
            if(language === 'EN') {
                if($scope.Device.Humidity.current <= 70 && $scope.Device.Humidity.current >= 30) { 
                    lable = '';
                } else {
                     lable = getCurrentHumidValueLable();
                }
            } else {
                if ($scope.Device.Humidity.current > 70) {
                    lable = $scope.translation.WEBMOB_device_ap_above;
                } else if ($scope.Device.Humidity.current < 30) {
                    lable = $scope.translation.WEBMOB_device_ap_below;
                }  else {
                    lable = '';
                }  
            }
            return lable;
        }
        
        function getUpperHumidityLable() {
            var language = LANG.toUpperCase().substring(0,2);
            var lable = '';
            if(language === 'EN') {
               if($scope.Device.Humidity.current > 70)
                    lable = $scope.translation.WEBMOB_device_ap_above;
                else if($scope.Device.Humidity.current < 30)
                    lable = $scope.translation.WEBMOB_device_ap_below;
                else
                    lable = getCurrentHumidValueLable();
            } else {
                lable = getCurrentHumidValueLable();
            }
            return lable;
        }
        
        
        function getUpperHumidityLableClass() {
            var styleClass = '';
            var language = LANG.toUpperCase().substring(0,2);
            if($scope.Device.Humidity.current <= 70 && $scope.Device.Humidity.current >=30) {
                 styleClass = 'humid-lable-value';
            } else if(language === 'EN') {
                styleClass = 'humid-lable-text';
            } else {
                styleClass = 'humid-lable-value humid-lable-value-kr';
            }
            return styleClass;
        }
        
        function getLowerHumidityLableClass() {
            var styleClass = '';
            var language = LANG.toUpperCase().substring(0,2);
            if(language === 'EN') {
               styleClass = 'humid-lable-value';
            } else {
                 styleClass = 'humid-lable-text humid-lable-text-kr';
            }
            return styleClass;
        }
    
        function isUpperMark() {
            var language = LANG.toUpperCase().substring(0,2);
            if($scope.Device.Humidity.current <= 70 && $scope.Device.Humidity.current >=30) {
                return true;
            } else if(($scope.Device.Humidity.current > 70 || $scope.Device.Humidity.current < 30) && (language === 'EN')) {
                return false;
            } else {
                return true;
            }
        }
        
        function isLowerMark() {
            var language = LANG.toUpperCase().substring(0,2);
            if($scope.Device.Humidity.current <= 70 && $scope.Device.Humidity.current >=30) {
                return false;
            } else if(($scope.Device.Humidity.current > 70 || $scope.Device.Humidity.current < 30) && (language === 'EN')) {
                return true;
            } else {
                return false;
            }
        }
        
        function getCurrentHumidValueLable() {
            var lable = 0;
            if($scope.Device.Humidity.current > 70) {
                lable = 70;
            } else if($scope.Device.Humidity.current < 30) {
                lable = 30;
            } else {
                lable = $scope.Device.Humidity.current;
            }
            return lable;
        }
        
        function onCancelProgress() {
            if($scope.DeviceOptions.isSterilizeCleanRunOn) {                
                $scope.setAutoCleanMode($scope.DeviceOptions.isSterilizeCleanOn);
            } else {
                $scope.setSterilizationMode($scope.DeviceOptions.isSterilizeOn);
            }
        }
    
        //init();
	
	/* Trends Graph starts here */
        function updateGraph() {
            $scope.trendsGraphImgData = [];
            var i = 0;
            while (i < $scope.trendGraphValues.length - 1) {
                $scope.trendsGraphImgData[i] = trendsImageKeyValues[$scope.trendGraphValues[i]] + "_" + trendsImageKeyValues[$scope.trendGraphValues[i + 1]];
                i++;
            }
            updateTimeArray();
        }

        function updateGraphHelp() {
            var i = 0;
            while (i < trendGraphValuesHelp.length - 1) {
                $scope.trendsGraphImgDataHelp[i] = trendsImageKeyValues[trendGraphValuesHelp[i]] + "_" + trendsImageKeyValues[trendGraphValuesHelp[i + 1]];
                i++;
            }
        }

        function getHistoryDotVisibility(col, index) {
            index = Math.abs(index - 4);
            if (index === $scope.trendGraphValues[col]) {
                return true;
            }
            return false;
        }

        function getHistoryDotVisibilityHelp(col, index) {
            index = Math.abs(index - 4);
            if (index === trendGraphValuesHelp[col]) {
                return true;
            }
            return false;
        }

        function updateTimeArray() {
            $scope.trendsTimeValues = [];
            var temp = $scope.trendsTimeValuesHelp.slice().reverse();
            for (var i = 0; i < $scope.trendGraphValues.length - 1; i++) {
                $scope.trendsTimeValues[i] = temp[i];
            }
            $scope.trendsTimeValues.reverse();
        }
        
        function getHistoryDotURL(index, item) {
            var temp;
            if (index === $scope.trendGraphValues.length - 1) {
                temp = "AP/assets/img/lcd/ap_history_graph_recent_dot_" + item + ".png"
            } else {
                temp = "AP/assets/img/lcd/ap_history_graph_dot_" + item + ".png"
            }
            return temp;
        }
        /* Trends Graph ends here */

        function parseModelInfo(information) {
            dismissLoadingBar(APCONST.INFORMATION);
            if(angular.isDefined(information["modelID"])) {
                modelId = information["modelID"].split("|")[2];
                if(angular.isDefined(modelId)){
                    if (modelId.substr(0, 2) === APCONST.AIRPURIFIER_DEVICE && modelId.substr(13, 1) === APCONST.LCD_DEVICE) {
                        $scope.isLCDModel = true;
                    }
                    //if (UtilityService.pad(parseInt(modelId.substr(15, 1), 10).toString(2), 4).charAt(1) === APCONST.AP_PROVIDE_OUTDOOR) {
                    if (UtilityService.pad(parseInt(modelId.substr(21, 1), 10).toString(2), 4).charAt(3) === APCONST.AP_PROVIDE_OUTDOOR) {
                        apSupportOutdoor = true;
                    }
                }
                console.log("LCD:" + $scope.isLCDModel + ", apSupportOutdoor:" + apSupportOutdoor);
            }
            if ($scope.isLCDModel && apSupportOutdoor && $scope.selectedCountry === 'KR') {
                $scope.checkResponse.push(APCONST.CONFIGURATION);
                SHPService.sendSHPCommand(APCONST.GET, "/" + $scope.peerId + $scope.Device.ConfigurationLink.href);
            } else {
                if ($scope.checkResponse.indexOf(APCONST.GET_DEVICES) !== -1) {
                    getWeatherLocationList();
                    startApp();
                }
            }
        }

        function parseConfiguration(configuration) {
            dismissLoadingBar(APCONST.CONFIGURATION);
            if (angular.isDefined(configuration["region"])) {
                deviceRegion = configuration["region"];
            }
            if (angular.isDefined(configuration["nationCode"])) {
                deviceNation = configuration["nationCode"];
            }
            if ($scope.checkResponse.indexOf(APCONST.GET_DEVICES) !== -1) {
                getWeatherLocationList();
                startApp();
            }
        }
        
        /*
        * note :   wifi module library only support 3 length region value
        *          so, app sends 000 to wifi module for CN region Code.
        *          when LCD device comes for CN, wifi module would be available to accept more than 3 len value of region
        *          until then, mobile needs to keep this code.( remove when LCD device comes for CN )
        */
        function sendConfigToDevice(location) {
            if (!$scope.isStatic) {
                var payload;
                if ($scope.selectedCountry === 'KR' && location !== null && location !== undefined) {
                    payload = '{"Configuration" : {"region" : "' + getRegionCodeFromName(location, $scope.selectedCountry) + '", "nationCode":"' + $scope.selectedCountry + '"}}';
                } else {
                    payload = '{"Configuration" : {"region" : "' + '000' + '", "nationCode":"' + $scope.selectedCountry + '"}}';
                }
                console.log("send config:" + payload);
                nativeInterface.putDataOnResourceUrl(payload, "/" + $scope.peerId + $scope.Device.ConfigurationLink.href);
            }
        }

        function getRegionCodeFromName(regionName, country) {
            var temp = "";
            if (country === "KR") {
                temp = WeatherService.setRegionCode(regionName);            
            } else if (country === "CN") {
                temp = WeatherService.setRegionCodeCN(regionName);
            }
            return temp;
        }
        
        /* Outdoor related start here */
        function isValidDataKR(value) {
            if (value >= 0 && value <= 999) {
                return true;
            }
            return false;
        }

        function isValidGradeKR(value) {
            if (value >= 1 && value <= 4) {
                return true;
            }
            return false;
        }
        
        function getOutdoorAttributeValue(value, grade) {
            var temp = "";
            if (isValidDataKR(value) && isValidGradeKR(grade)) {
                temp = value + " ppm";
            } else if((isValidDataKR(value) && !isValidGradeKR(grade)) || (!isValidDataKR(value) && !isValidGradeKR(grade))) {
                temp = $scope.translation.WEBMOB_device_ap_unavailable;
            } else if (!isValidDataKR(value) && isValidGradeKR(grade)){
                temp = "";
            }
            return temp;
        }
        
        function isOutdoorAirInfoUnavailable() {
            if ($scope.selectedCountry === "KR") {
                if ($scope.selectedWeatherLocation === null || !$scope.outDoorDataAvailable || !isValidOutdoorData() 
                    || isCAIUnavailable() || isAirInfoUnavailable()) {
                    return true;
                }
                return false;
            } else {
                if ($scope.selectedWeatherLocation === null || !$scope.outDoorDataAvailable || !isValidOutdoorData()) {
                    return true;
                }
                return false;
            }
        }
        
        function isCAIUnavailable() {
            if((isValidDataKR($scope.DeviceOptions.outDoorData.pm10.value) || isValidDataKR($scope.DeviceOptions.outDoorData.pm25.value))
                && !isValidDataKR($scope.DeviceOptions.outDoorData.KHAI.value)) {
                return true;
            }
            return false;
        }
        
        function isAirInfoUnavailable() {
            if(!isValidDataKR($scope.DeviceOptions.outDoorData.pm10.value) && !isValidDataKR($scope.DeviceOptions.outDoorData.pm25.value)
                && !isValidDataKR($scope.DeviceOptions.outDoorData.KHAI.value)) {
                return true;
            }
            return false;
        }
        
        function isOutdoorInfoButtonDisable() {
            if($scope.selectedCountry === "KR") {
                if($scope.selectedWeatherLocation === null || !$scope.outDoorDataAvailable || (!isValidOutdoorData() && !isCAIUnavailable()) || isAirInfoUnavailable()) {
                    return true;
                }
                return false;
            } else {
                if($scope.selectedWeatherLocation === null || !$scope.outDoorDataAvailable || !isValidOutdoorData()) {
                    return true;
                }
                return false;
            }
        }
        /* Outdoor related ends here */
    }

app.directive('applyRipple', function () {
    return {
        restrict: 'A',
        link: function (scope, element) {
            //$(element).materialripple();
        }
    };
});
app.directive('autoSuggest', function () {
    return {
        restrict: 'A',
        link: function (scope, element, attr, ngModel) {
            element.on('input', function() {          
            scope.ngModel = element.val();
            scope.searchValueUpdated();
        });
        }
    };
});

app.directive('myFilter', function () {
    return {
        restrict: 'A',
        link: function (scope, element) {
            scope.updateFilterLayerStyle(element[0]);
        }
    };
});

app.directive('sterilizationProgress', function () {
    return {
        restrict: 'A',
        link: function (scope) {
            scope.setProgress(scope.DeviceOptions.sterilizeProgress);            
        }
    };
});

app.directive('weatherListItemLoaded', function ($timeout) {
    return {        
        restrict: 'A',
        link: function (scope, element) {      
            if (scope.OSPlatform === "IOS" && scope.OSVersion >= 11) {
                // Do nothing
            } else {
                $(element).hide();
                $timeout(function () {
                    $(element).show();
                    $(element).addClass('force-render-region-list');
                }, 400);
            }
        }
    };
});
